#include <wx/wx.h>
#include <wx/statline.h>

#include "globaldefs.h"
#include "FindReplDialog.h" //!Classes declared in a separate file are globally available
//_________________________________
#define FRMinX 176  /// width for find/replace combos
#define FRMinY 18   ///height for find/replace combos

//! Custom event for FindResults Dialog-----------------------------------------------
DEFINE_EVENT_TYPE( wxEVT_RESULTS_CLICKED ) 
DEFINE_EVENT_TYPE( wxEVT_RESULTS_CLOSING )
DEFINE_EVENT_TYPE( wxEVT_LOCALFINDALLTWEAK )

IMPLEMENT_DYNAMIC_CLASS(myFindResultsEvent, wxCommandEvent)

myFindResultsEvent::myFindResultsEvent(wxEventType commandType, int id)
    : wxCommandEvent(commandType, id)
{
    m_x = 0;  m_y = 0; m_z = 0;
}
//! Copy constructor - required!
myFindResultsEvent::myFindResultsEvent(const myFindResultsEvent& event)
    : wxCommandEvent(event)
{
    m_x =            event.m_x;
    m_y =            event.m_y;
    m_z =            event.m_z;
}
///----------------------------------------------------------------------------------------

/// FindRepl Events
DEFINE_EVENT_TYPE( wxEVT_FIND ) 
DEFINE_EVENT_TYPE( wxEVT_FIND_NEXT )
DEFINE_EVENT_TYPE( wxEVT_FIND_REPLACE ) 
DEFINE_EVENT_TYPE( wxEVT_FIND_REPLACE_ALL ) 
DEFINE_EVENT_TYPE( wxEVT_FIND_CLOSE ) 

IMPLEMENT_DYNAMIC_CLASS(FindReplEvent, wxCommandEvent)

FindReplEvent::FindReplEvent(wxEventType commandType, int id) //! Constructor
    : wxCommandEvent(commandType, id)
{
//    m_flags = 0;  m_findString = m_replString = wxEmptyString;
}
//! Copy constructor - required!
FindReplEvent::FindReplEvent(const FindReplEvent& event)
    : wxCommandEvent(event)
{
//    SetFlags(event.GetFlags());  SetFindString(event.GetFindString()); SetReplString(event.GetReplString());
}

///----------------------------------------------------------------------------------------
/*! Find Dialog */
FindDialog::FindDialog( wxWindow *parent, wxPoint pos, int parentWidth, FindReplData * frData, FindReplData * frData2, wxArrayString * rdirs,
        bool findAllMode, bool allFilesMode )
	: wxDialog( parent, wxID_ANY, _T("Find"), pos, wxDefaultSize, wxDEFAULT_DIALOG_STYLE ) //! wx Default Args required here
{
    m_frData = frData;
    m_frData2 = frData2;
    m_parent = parent;
    m_rtFind = false;

    m_findCombo = new wxComboBox( this, ZION_FIND_COMBO, m_frData->GetFindString(), /// iniitial value
            wxDefaultPosition, wxDefaultSize, m_frData->m_findStrs, wxCB_SORT );
    m_findCombo->SetMinSize( wxSize(168, FRMinY) ); //! Size must be requested BEFORE adding to the parent sizer
    m_text = new wxTextCtrl( this, ZION_FIND_TEXT, _T("") ); /// iniitial value
    m_text->SetMinSize( wxSize(168, FRMinY) ); //! Size must be requested BEFORE adding to the parent sizer
    m_list = new wxListBox( this, ZION_FIND_LIST, wxDefaultPosition, wxSize(-1, 120) );

    m_case = new wxCheckBox( this, ZION_FR_MATCHCASE, _T("Match ca&se") );
    m_case->SetValue( m_frData->GetFlags() & ZION_FR_FLAG_MATCHCASE );
    m_wholeWord = new wxCheckBox( this, ZION_FR_WHOLEWORD, _T("Match &whole word") ); //! Init according to frData!!!
    m_wholeWord->SetValue( m_frData->GetFlags() & ZION_FR_FLAG_WHOLEWORD );
    m_regex = new wxCheckBox( this, ZION_FR_REGEX, _T("RegEx") );  //! We need a unique ID if we want to handle events from a widget
    m_regex->SetValue( m_frData->GetFlags() & ZION_FR_FLAG_REGEX );
    
    m_all = new wxCheckBox( this, ZION_FINDALL, _T("List &All") );  //! We need a unique ID if we want to handle events from a widget
    m_all->SetValue(true); ///Default to list all mode instead of incremental search
    m_all->Enable( !findAllMode && !allFilesMode );
      
    m_topSizer = new wxBoxSizer( wxVERTICAL );
    m_topSizer ->Add( m_findCombo, 0, wxALIGN_CENTER_HORIZONTAL|wxGROW | wxALL, 4 );
    m_topSizer ->Add( m_text, 0, wxALIGN_CENTER_HORIZONTAL|wxGROW | wxALL, 4 );
    m_text->Show(false);
    m_topSizer ->Add( m_list, 0, wxALIGN_CENTER_HORIZONTAL|wxGROW| wxALL, 4 );
    m_list->Show(false); ///Initially hidden

    wxFlexGridSizer *flagsSizer = new wxFlexGridSizer( 1, 4, 2, 3 );  //! Rows, cols, vgap, hgap // FlexGridSizer//wxBoxSizer( wxHORIZONTAL );
    flagsSizer->Add( m_case, 0, wxALIGN_LEFT |wxALL, 2);
    flagsSizer->Add( m_wholeWord, 0, wxALIGN_LEFT |wxALL, 2);
    flagsSizer->Add( m_regex, 0, wxALIGN_LEFT |wxALL, 2);
    flagsSizer->Add( m_all, 0, wxALIGN_LEFT |wxALL, 2);
    m_topSizer->Add( flagsSizer,  0, wxALIGN_CENTER_HORIZONTAL|wxALL, 3 );
    
    ///Search in All Files
    m_allFiles = new wxCheckBox( this, ZION_FR_ALLFILES, _T("Search ALL &Files of known ext. (Settings | Ext. Map)") );
    m_allFiles->SetValue(allFilesMode);
    m_allFiles->Show(! findAllMode); ///Lock it out only in Find All mode.
    
    if( rdirs == NULL ) rdirs = new wxArrayString; ///blank for non-allFiles mode 
    m_dirName = new wxComboBox( this, wxID_ANY, wxGetCwd(), wxDefaultPosition, wxDefaultSize, * rdirs );
    m_dirName->Enable(allFilesMode);
    m_dirName->Show( ! findAllMode );
           
    m_dirBrowse = new wxButton( this, ZION_DIRBROWSE, _T("..."), wxDefaultPosition, wxSize(26,-1) );
    m_dirBrowse->Enable( allFilesMode );
    m_dirBrowse->Show( ! findAllMode );
    m_dirBrowse->SetToolTip(_T("Browse for Folder") );
    m_okButton = new wxButton( this, wxID_OK, _T("&Go") );
    m_okButton->SetDefault();
    
    wxStaticLine * statLine = new wxStaticLine(this);
    statLine->Show( ! findAllMode );
    m_topSizer->Add( statLine, 1, wxGROW | wxALL, 3 );
    m_topSizer->Add( m_allFiles, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 3 );
    wxBoxSizer *allFilesSizer = new wxBoxSizer( wxHORIZONTAL );
    wxStaticText * statText = new wxStaticText( this, wxID_ANY, _T("Start in") );
    statText->Show( ! findAllMode );
    allFilesSizer->Add( statText,  0, wxALIGN_CENTER_VERTICAL|wxALL, 3 );
    allFilesSizer->Add( m_dirName, 0, wxALIGN_CENTER_VERTICAL|wxALL, 3 );
    allFilesSizer->Add( m_dirBrowse, 0, wxALIGN_CENTER_VERTICAL|wxALL, 2 );
    m_topSizer->Add( allFilesSizer, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 1 );
    m_topSizer->Add( (new wxStaticLine(this) ), 1, wxGROW | wxALL, 3 );

    wxBoxSizer *buttonSizer = new wxBoxSizer( wxHORIZONTAL );
    buttonSizer->Add( m_okButton, 1 ); //! Proportion =1;
    buttonSizer->AddSpacer( 14 );
    m_doneBtn = new wxButton( this, wxID_CANCEL, _T("&Done") );
    buttonSizer->Add( m_doneBtn, 1 );
    m_topSizer->Add( buttonSizer, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5 );
    SetSizer(m_topSizer);  ///use the sizer for layout
    m_topSizer->SetSizeHints(this);  //! Coordinate minimal window size
    ///Positioning
    int dlgW = GetSize().GetWidth();
    int edgeOff = 20;  ///edge offset
    if( parentWidth - dlgW > dlgW )  edgeOff = ( parentWidth - dlgW ) >> 2;  ///More centered
    
    int x = pos.x + parentWidth - dlgW - edgeOff;
    int y = pos.y + 200;
    SetSize( x, y, wxDefaultCoord, wxDefaultCoord );
}

FindDialog::~FindDialog() {}

BEGIN_EVENT_TABLE(FindDialog, wxDialog)
    EVT_BUTTON( wxID_OK, FindDialog::OnOK )
    EVT_BUTTON( wxID_CANCEL, FindDialog::OnCancel )
    EVT_BUTTON( ZION_DIRBROWSE, FindDialog::OnChooseDir )
    EVT_CHECKBOX( ZION_FR_ALLFILES, FindDialog::OnAllFilesCheck )
    //EVT_COMBOBOX( ZION_FIND_COMBO, FindDialog::OnFindCombo )
    EVT_TEXT( ZION_FIND_COMBO, FindDialog::OnFindComboText )
    EVT_TEXT( ZION_FIND_TEXT, FindDialog::OnRTFind )
    EVT_LISTBOX( ZION_FIND_LIST, FindDialog::OnFindListClick )
    EVT_CHECKBOX( ZION_FR_MATCHCASE, FindDialog::OnRTFind )
    EVT_CHECKBOX( ZION_FR_WHOLEWORD, FindDialog::OnRTFind )
    EVT_CHECKBOX( ZION_FR_REGEX, FindDialog::OnRTFind )
END_EVENT_TABLE()

void FindDialog::OnFindComboText( wxCommandEvent & event )  ///Do a new search
{   ///This will only be called once when typing begins
    m_text->SetValue( m_findCombo->GetValue() );
    m_findCombo->Show( false );
    m_text->Show( true );
    m_text->SetInsertionPoint( m_findCombo->GetInsertionPoint() );
    m_text->SetFocus();
    m_topSizer->SetSizeHints(this);  /// We have to do this here since OnRTFind will not update till 3 chars entered
    m_topSizer->Layout();
    m_rtFind = true;  ///Start realtime find mode
    wxCommandEvent anEvent( wxEVT_COMMAND_TEXT_UPDATED, ZION_FIND_TEXT );  ///Hand off to OnRTFind
    AddPendingEvent(anEvent);
}

void FindDialog::OnRTFind( wxCommandEvent & event )  ///Do a new search
{
    if( m_rtFind ) {
        wxString str = m_text->GetValue();
        if( str.Len() > 2 && ! m_allFiles->IsChecked() ) {
            m_frData2->UpdateFindString( str ); //! Save the pattern used
            m_frData2->SetFlags( ( 0 | m_regex->GetValue() ? ZION_FR_FLAG_REGEX : 0 ) |
                    ( m_case->GetValue() ? ZION_FR_FLAG_MATCHCASE : 0 ) |
                    ( m_wholeWord->GetValue() ? ZION_FR_FLAG_WHOLEWORD : 0 ) );
            wxCommandEvent anEvent( wxEVT_COMMAND_TEXT_UPDATED, ZION_RTFIND );  ///TODO - change this to ZION_RT_FIND
            m_parent->ProcessEvent(anEvent);
            m_list->Set( m_frData2->m_results );
            m_list->Show(true);
            //RedrawFindCombo();
            SetTitle(wxString::Format(_T("Found:  %d"), m_frData2->m_results.GetCount() ) );
            m_topSizer->SetSizeHints(this);  //! Coordinate minimal window size
            m_topSizer->Layout();
            m_doneBtn->SetDefault();  ///Since we already did a search it is likely we want to quit
        }
    }
}
//
//void FindDialog::RedrawFindCombo() {
//    m_topSizer->Detach( (wxWindow *) m_findCombo );
//    m_findCombo->Destroy(); //m_findCombo = (wxComboBox *) NULL;
//    m_findCombo = new wxComboBox( this, ZION_FIND_COMBO, m_frData2->GetFindString(),
//            wxDefaultPosition, wxDefaultSize, m_frData2->m_findStrs, wxCB_SORT );
//    //m_findCombo->SetMinSize( wxSize(FRMinX, FRMinY) );
//    m_topSizer->Insert( 0, (wxWindow *) m_findCombo, 0, wxGROW|wxALL, 2 );
//}

void FindDialog::OnFindListClick( wxCommandEvent &event )
{
    wxString str( event.GetString() );
    DoSelection(str);
}

void FindDialog::DoSelection( wxString & str )
{
    wxString str2;
    long x=0, y=0, z=0;
    str = str.BeforeFirst(']'); //! Cut end
    str = str.AfterFirst('['); //! Cut start
    str2 = str.BeforeFirst(','); //! Isolate line value
    str2.ToLong(&y);  //! Put result in y
    str2 = str.AfterFirst(','); /// save positions
    /// Patch in here for max position
    str = str2.BeforeFirst('-'); /// get start
    str.ToLong(&x);
    str = str2.AfterFirst('-');  /// get end
    str.ToLong(&z);
    myFindResultsEvent evt( wxEVT_RESULTS_CLICKED, -1 ); /// Type, Id //evt.SetId(-1); evt.SetEventType(wxEVT_RESULTS_CLICKED);
    evt.SetEventObject(this);
    evt.SetX(x); evt.SetY(y); evt.SetZ(z); //! Store Linenum and positions in the event: [y:x-z]
    m_parent->ProcessEvent(evt); //AddPendingEvent(evt);
    //m_lastLine = (int) y; ///Last Line!!!
}

void FindDialog::OnOK( wxCommandEvent &event )
{   
    m_frData->UpdateFindString( m_rtFind ? m_text->GetValue() : m_findCombo->GetValue() ); //! Save the pattern used

    m_frData->SetFlags( (0 | m_regex->GetValue() ? ZION_FR_FLAG_REGEX : 0 ) |
            ( m_case->GetValue() ? ZION_FR_FLAG_MATCHCASE : 0 ) |
            ( m_wholeWord->GetValue() ? ZION_FR_FLAG_WHOLEWORD : 0 ) );
    m_frData2->m_findStrs.Empty();  ///These are temporary, clean up
    m_frData2->m_results.Empty();
    event.Skip(); //! Allow the default handler to do its thing i.e. pass the event on
}

void FindDialog::OnCancel( wxCommandEvent &event )
{   
    m_frData2->m_findStrs.Empty();  ///These are temporary, clean up
    m_frData2->m_results.Empty();
    event.Skip(); //! Allow the default handler to do its thing i.e. pass the event on
}

void FindDialog::OnAllFilesCheck( wxCommandEvent & event )
{
    m_dirName->Enable( m_allFiles->IsChecked() ); 
    m_dirBrowse->Enable( m_allFiles->IsChecked() ); 
}

void FindDialog::OnChooseDir( wxCommandEvent &event)
{
    wxDirDialog wd( this, _T("Choose a Folder"), m_dirName->GetValue() );
    if( wd.ShowModal() == wxID_OK ) m_dirName->SetValue( wd.GetPath() );
}
//----------------------------------------------------------------------

/*! FindResults Dialog */

FindResultsDialog::FindResultsDialog(wxWindow *parent, wxPoint pos, int parentWidth, FindReplData * frData)
	: wxDialog( parent, wxID_ANY, _T("Find Results"), pos, wxDefaultSize, wxDEFAULT_DIALOG_STYLE ) //! wx Default Args required !
{
    m_lastLine = 0;
    m_parent = parent;  //! save pointer to parent window (edit window)
    m_frData = frData;
    
    m_topSizer = new wxBoxSizer( wxVERTICAL );
    m_replStuff = new wxBoxSizer( wxVERTICAL );
    
    ///We will update m_topText on Flag Tweak (Re-Find)
    int c = frData->m_results.GetCount();    
    m_topText = new wxStaticText( this, wxID_ANY, wxString::Format(_T("Found %d occurrence%s of  " ),
            c, ( c<= 1 ) ? "" : "s" ) );
    m_findText = new wxTextCtrl( this, ZION_FR_FIND_TEXT, wxString::Format(_T("%s"), frData->GetFindString() ),
            wxDefaultPosition, wxDefaultSize, wxTE_PROCESS_ENTER );
    m_findText->SetMinSize(wxSize(160, FRMinY)); //! Size must be requested BEFORE adding to the parent sizer
    wxBoxSizer * topTextSizer = new wxBoxSizer( wxHORIZONTAL );
    topTextSizer->Add( m_topText, 0, wxALIGN_LEFT| wxALIGN_CENTER_VERTICAL |wxALL, 2 );
    topTextSizer->Add( m_findText, 0, wxALIGN_LEFT| wxALIGN_CENTER_VERTICAL |wxALL, 2 );
    m_topSizer->Add(topTextSizer,  0, wxALIGN_LEFT |wxALL, 3);
    
    m_listBox = new wxListBox( this, ZION_FRES_LISTBOX, wxDefaultPosition, wxDefaultSize,
            frData->m_results, wxLB_HSCROLL /* |wxLB_EXTENDED */ ); ///Single selection
    m_topSizer->Add(m_listBox,  0, wxALIGN_CENTER_HORIZONTAL |wxALL, 3);
    
    m_case = new wxCheckBox( this, ZION_FR_MATCHCASE, _T("Match Case") );
    m_case->SetValue( frData->m_flags & ZION_FR_FLAG_MATCHCASE );
    m_wholeWord = new wxCheckBox( this, ZION_FR_WHOLEWORD, _T("Match Whole Word") );
    m_wholeWord->SetValue( frData->m_flags & ZION_FR_FLAG_WHOLEWORD );
    m_regex = new wxCheckBox( this, ZION_FR_REGEX, _T("Regular Expression") );
    m_regex->SetValue( frData->m_flags & ZION_FR_FLAG_REGEX );
    m_preserve = new wxCheckBox( this, wxID_ANY, _T("Preserve Replaced Case") ); ///We are not handling events on this
    m_preserve->SetValue( frData->m_flags & ZION_FR_FLAG_PRESERVECASE );
    m_preserve->Enable(false); ///Disabled on startup

    wxFlexGridSizer *flagsSizer = new wxFlexGridSizer( 2, 2, 3, 3);  //! Rows, cols, vgap, hgap // FlexGridSizer//wxBoxSizer( wxHORIZONTAL );
    flagsSizer->Add( m_wholeWord, 0, wxALIGN_LEFT |wxALL, 2);
    flagsSizer->Add( m_regex, 0, wxALIGN_LEFT |wxALL, 2);
    flagsSizer->Add( m_case, 0, wxALIGN_LEFT |wxALL, 2);
    flagsSizer->Add( m_preserve, 0, wxALIGN_LEFT |wxALL, 2);
    m_topSizer->Add( flagsSizer, 0, wxALIGN_CENTER_HORIZONTAL |wxALL, 2 );
    
    m_searchAgain = new wxButton( this, ZION_FR_SEARCHAGAIN, _T("Search Again") );
    m_okButton = new wxButton( this, wxID_OK, _T("Done") );
    m_replMode = new wxCheckBox( this, ZION_FR_REPLMODE, _T("Replace Mode") );
    m_replMode->SetValue( false);
    wxBoxSizer *buttonSizer = new wxBoxSizer( wxHORIZONTAL );
    buttonSizer->Add( m_searchAgain,  0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
    buttonSizer->AddSpacer(10);
    buttonSizer->Add( m_replMode,  0, wxALIGN_CENTER_HORIZONTAL| wxALIGN_CENTER_VERTICAL|wxALL, 5); 
    buttonSizer->AddSpacer(10);
    buttonSizer->Add( m_okButton,  0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
    m_topSizer->Add(buttonSizer,  0, wxALIGN_CENTER_HORIZONTAL |wxALL, 2);
    
    ///REPLACE MODE STUFF
    m_statLine = new wxStaticLine(this);
    m_replStuff->Add( m_statLine, 1, wxGROW | wxALL, 3 );
    m_replStuff->Add( new wxStaticText( this, wxID_ANY, _T("Click an occurrence ABOVE to verify location, then\nDouble Click to replace") ),
            0, wxALIGN_CENTER_HORIZONTAL |wxALL, 2);
    m_replCombo = new wxComboBox( this, ZION_FR_REPL_COMBO, frData->GetReplString(), /// iniitial value
            wxDefaultPosition, wxDefaultSize, frData->m_replStrs, wxCB_SORT );
    m_replCombo->SetMinSize(wxSize(168, FRMinY)); //! Size must be requested BEFORE adding to the parent sizer
    m_replText = new wxTextCtrl( this, ZION_REPL_TEXT, _T("") );
    m_statText = new wxStaticText( this, wxID_ANY, _T("Replace with") );
    m_replAllButton = new wxButton( this, ZION_FR_REPLALL, _T("Replace All") );
    wxBoxSizer *replwSizer = new wxBoxSizer( wxHORIZONTAL );
    replwSizer->Add(m_statText, 0, wxALIGN_CENTER_VERTICAL| wxALL, 2 );
    replwSizer->Add( m_replCombo, 0, wxALIGN_CENTER_VERTICAL|wxALL, 2 );
    replwSizer->Add( m_replText, 0, wxALIGN_CENTER_VERTICAL|wxALL, 2 );
    m_replText->Show(false);
    replwSizer->AddSpacer(6);
    replwSizer->Add( m_replAllButton,  0, wxALIGN_CENTER_HORIZONTAL|wxALL, 4); 
    m_replStuff->Add( replwSizer,  0, wxALIGN_CENTER_HORIZONTAL |wxALL, 4);
    
    m_rlistBox = new wxListBox( this, ZION_RLISTBOX, wxDefaultPosition, wxDefaultSize, 0, ///wxString[] based
            NULL, wxLB_HSCROLL /* |wxLB_EXTENDED */ ); ///Single selection
    m_rlistBox->SetMinSize( wxSize(-1, 80) ); /// w, h 

    m_replStuff->Add(m_rlistBox,  0, wxALIGN_CENTER_HORIZONTAL |wxALL, 3);
    
    m_topSizer->Add( m_replStuff, 0, wxALIGN_CENTER_HORIZONTAL |wxALL, 2 );
    m_topSizer->Show( m_replStuff, false, true );  /// false = don't show, true = recursive
    
    SetSizer(m_topSizer);
    m_topSizer->Layout();
    m_topSizer->SetSizeHints(this);  //! Coordinate minimal window size
    m_okButton->SetDefault();
    ///Positioning
    int dlgW = GetSize().GetWidth();
    int edgeOff = 20;  ///edge offset
    if( parentWidth - dlgW > dlgW )  edgeOff = ( parentWidth - dlgW ) >> 3;
    
    int x = pos.x + parentWidth - dlgW - edgeOff;
    int y = pos.y + 48;
    SetSize( x, y, wxDefaultCoord, wxDefaultCoord );
}

FindResultsDialog::~FindResultsDialog() {}

BEGIN_EVENT_TABLE(FindResultsDialog, wxDialog)
    EVT_CHECKBOX( ZION_FR_MATCHCASE, FindResultsDialog::OnReloadList )  ///TODO: Rename to ReloadList
    EVT_CHECKBOX( ZION_FR_WHOLEWORD, FindResultsDialog::OnReloadList )
    EVT_CHECKBOX( ZION_FR_REGEX, FindResultsDialog::OnReloadList )
    //EVT_TEXT( ZION_FR_FIND_TEXT,  FindResultsDialog::OnReloadList )
    EVT_TEXT_ENTER( ZION_FR_FIND_TEXT,  FindResultsDialog::OnReloadList )
    EVT_BUTTON( ZION_FR_SEARCHAGAIN, FindResultsDialog::OnReloadList )
    EVT_BUTTON( ZION_FR_REPLALL, FindResultsDialog::OnReplAll )
    EVT_BUTTON( wxID_OK, FindResultsDialog::OnOK )
    EVT_CHECKBOX( ZION_FR_REPLMODE, FindResultsDialog::OnReplMode )
    EVT_TEXT( ZION_FR_REPL_COMBO,  FindResultsDialog::OnReplCombo )
    EVT_LISTBOX( ZION_FRES_LISTBOX, FindResultsDialog::OnLBClick )
    EVT_LISTBOX( ZION_RLISTBOX, FindResultsDialog::OnLBClick )
    EVT_LISTBOX_DCLICK( ZION_FRES_LISTBOX, FindResultsDialog::OnReplSels )
END_EVENT_TABLE()


void FindResultsDialog::OnOK( wxCommandEvent &event)
{
    myFindResultsEvent evt;
    evt.SetId(-1); evt.SetEventType(wxEVT_RESULTS_CLOSING);
    evt.SetEventObject(this);
    m_parent->ProcessEvent(evt); //! This should destroy the dialog
    //event.Skip();
}

///This helps us avoid the bug/feature of MS Combos trying to "help" you with autofilling and ignoring case
void FindResultsDialog::OnReplCombo( wxCommandEvent & event )
{   ///This will only be called once when typing begins (side effect - also on combo list selection
    m_replText->SetValue( m_replCombo->GetValue() );
    m_replText->SetInsertionPoint( m_replCombo->GetInsertionPoint() );
    m_replText->SetFocus();
    m_replText->Show(true); m_replCombo->Show(false); ///Get rid of combo
    m_topSizer->Layout();
    m_topSizer->SetSizeHints(this);  /// We have to do this here since we wont be doing a realtime find
}

void FindResultsDialog::OnReplMode( wxCommandEvent & event )  ///Initial switch to repl mode
{
    m_preserve->Enable( m_replMode->IsChecked() );
    m_topSizer->Show( m_replStuff, m_replMode->IsChecked(), true );  ///true = show, true = recursive
    m_replText->Show(false);  ///keep hidden
    m_topSizer->Layout();
    m_topSizer->SetSizeHints(this);  //! Coordinate minimal window size
}

void FindResultsDialog::OnReloadList( wxCommandEvent & event )  /// RELOAD LISTBOX
{
    m_frData->m_lastSearchPos = m_lastLine; ///Last Line!!!
    int diaFlags = 0;
    if( m_replText->IsShown() ) m_replCombo->SetValue( m_replText->GetValue() );
    if( m_replMode->IsChecked() && m_replCombo->GetValue() != wxEmptyString ) {
    ///Reload m_rlistBox
    diaFlags |= ( m_case->IsChecked() )? ZION_FR_FLAG_MATCHCASE : 0;
        m_frData->m_flags = diaFlags;
        m_frData->UpdateFindString(m_replCombo->GetValue());
        myFindResultsEvent evt( wxEVT_LOCALFINDALLTWEAK, -1 ); ///type, id
        evt.SetEventObject(this);  m_parent->ProcessEvent(evt);
        m_rlistBox->Set( m_frData->m_results );
        //m_rlistBox->InsertItems( m_frData->m_results, 0 );
        m_replText->Show(false); m_replCombo->Show(true); ///Restore to normal
    }
    ///Reload Find Listbox
    ///Prepare m_filesData with flags and search patt
    if( m_findText->GetValue() == _T("") )  return;  /// will not search for nothing
    diaFlags = 0;
    diaFlags |= ( m_case->IsChecked() )? ZION_FR_FLAG_MATCHCASE : 0;
    diaFlags |= ( m_wholeWord->IsChecked() )? ZION_FR_FLAG_WHOLEWORD : 0;
    diaFlags |= ( m_regex->IsChecked() )? ZION_FR_FLAG_REGEX : 0;
    diaFlags |= ( m_preserve->IsChecked() )? ZION_FR_FLAG_PRESERVECASE : 0; ///This f() is used in replace mode, add this flag

    m_frData->m_flags = diaFlags;
    m_frData->UpdateFindString(m_findText->GetValue());

    myFindResultsEvent evt( wxEVT_LOCALFINDALLTWEAK, -1 ); ///type, id
    evt.SetEventObject(this);  m_parent->ProcessEvent(evt);
    
    int c = m_frData->m_results.GetCount();
    m_topText->SetLabel( wxString::Format(_T("Found %d occurrence%s of  " ), c, (c<= 1) ? "" : "s" ) );
    /// Reload ListBox
    m_listBox->Clear();
    m_listBox->InsertItems( m_frData->m_results, 0 );
    m_topSizer->Layout();
    m_topSizer->SetSizeHints(this);  //! Coordinate minimal window size
}

void FindResultsDialog::OnReplAll( wxCommandEvent &event)
{
    wxCommandEvent anEvent;  OnReloadList( anEvent ); /// File could have changed ( NonModal Dialog)
    ///All Flags and Findtext should already be loaded
    m_frData->UpdateReplString( m_replText->IsShown() ? m_replText->GetValue() : m_replCombo->GetValue() );
    FindReplEvent evt( wxEVT_FIND_REPLACE_ALL, -1 ); ///type, id
    evt.SetEventObject(this);
    m_parent->ProcessEvent(evt); //! Send it
    anEvent;  OnReloadList( anEvent ); /// File could have changed ( NonModal Dialog)
}

void FindResultsDialog::OnReplSels( wxCommandEvent &event)
{   ///As a bonus, a double click will also preselect the found string for us because it generates a single-click event before a dbl click - NEED TO DOUBLE CHECK THIS ON OTHER WIN32 PLATs
    if( m_replMode->IsChecked() ) {
    
        m_frData->UpdateReplString( m_replText->IsShown() ? m_replText->GetValue() : m_replCombo->GetValue() ); ///Other Flags and Findtext are irrelevant at this point
        if( m_preserve->IsChecked() ) m_frData->m_flags |= ZION_FR_FLAG_PRESERVECASE;

        ///Send out a single REPLACE event
        FindReplEvent evt( wxEVT_FIND_REPLACE, -1 );
        evt.SetEventObject(this);
        m_parent->ProcessEvent(evt); //! Send it
        ///Refresh List
        wxCommandEvent anEvent;  OnReloadList( anEvent );
    }
}

void FindResultsDialog::OnLBClick(wxCommandEvent &event)
{
    wxString str(event.GetString());
    DoSelection( str );
}

void FindResultsDialog::DoSelection( wxString & str )
{
    wxString str2;
    long x=0, y=0, z=0;
    str = str.BeforeFirst(']'); //! Cut end
    str = str.AfterFirst('['); //! Cut start
    str2 = str.BeforeFirst(','); //! Isolate line value
    str2.ToLong(&y);  //! Put result in y
    str2 = str.AfterFirst(','); /// save positions
    /// Patch in here for max position
    str = str2.BeforeFirst('-'); /// get start
    str.ToLong(&x);
    str = str2.AfterFirst('-');  /// get end
    str.ToLong(&z);
    myFindResultsEvent evt( wxEVT_RESULTS_CLICKED, -1 ); /// Type, Id //evt.SetId(-1); evt.SetEventType(wxEVT_RESULTS_CLICKED);
    evt.SetEventObject(this);
    evt.SetX(x); evt.SetY(y); evt.SetZ(z); //! Store Linenum and positions in the event: [y:x-z]
    m_parent->ProcessEvent(evt); //AddPendingEvent(evt);
    m_lastLine = (int) y; ///Last Line!!!
}
//--------------------------------------------------------------------------------------------------------------------------------------------------------
/*! FAFReplace Dialog */
FAFReplaceDialog::FAFReplaceDialog( wxWindow *parent, wxPoint pos, FilesResultsData * filesData, FindReplData * frData, const wxString & file)
	: wxDialog( parent, wxID_ANY, _T("Global Replace"), pos, wxDefaultSize, wxDEFAULT_DIALOG_STYLE ) //! wx Default Args required here
{
    ///Create Widgets
    m_filesData = filesData;
    wxStaticText * topText = new wxStaticText( this, wxID_ANY, wxString::Format(_T("In   %s - %s"), file.AfterLast('\\'), file.BeforeLast('\\') ) );
    wxStaticText * upperText = new wxStaticText( this, wxID_ANY, _T("Replace ALL occurrences of ") );
    m_findText = new wxTextCtrl( this, wxID_ANY, wxString::Format(_T("%s"), filesData->m_patt) );
    m_findText->SetMinSize(wxSize(160, FRMinY)); //! Size must be requested BEFORE adding to the parent sizer

    m_replCombo = new wxComboBox( this, ZION_FR_REPL_COMBO, frData->GetReplString(), /// iniitial value
            wxDefaultPosition, wxDefaultSize, frData->m_replStrs, wxCB_SORT );
    m_replCombo->SetMinSize(wxSize(168, FRMinY)); //! Size must be requested BEFORE adding to the parent sizer
    
    m_findList = new wxListBox( this, wxID_ANY, wxDefaultPosition, wxDefaultSize, filesData->m_results, wxLB_HSCROLL|wxLB_NEEDED_SB  );
    
    m_case = new wxCheckBox( this, ZION_FR_MATCHCASE, _T("Match Case") );
    m_case->SetValue( filesData->m_flags & ZION_FR_FLAG_MATCHCASE );
    m_wholeWord = new wxCheckBox( this, ZION_FR_WHOLEWORD, _T("Match Whole Word") );
    m_wholeWord->SetValue( filesData->m_flags & ZION_FR_FLAG_WHOLEWORD );
    m_regex = new wxCheckBox( this, ZION_FR_REGEX, _T("Regular Expression") );
    m_regex->SetValue( filesData->m_flags & ZION_FR_FLAG_REGEX );
    m_preserve = new wxCheckBox( this, wxID_ANY, _T("Preserve Replaced Case") ); ///We are not handling events on this
    m_preserve->SetValue( filesData->m_flags & ZION_FR_FLAG_PRESERVECASE );

    m_yesButton = new wxButton( this, wxID_YES, "Yes" );
    m_noButton = new wxButton( this, wxID_NO, "No" );
    m_yesToAllButton = new wxButton( this, ZION_FAF_YESTOALL, "Yes To All" );
    
    ///LAYOUT
    wxBoxSizer *topSizer = new wxBoxSizer( wxVERTICAL );

    topSizer->Add( topText, 0, wxALIGN_CENTER_HORIZONTAL |wxALL, 5 );
    
    wxBoxSizer *upperSizer = new wxBoxSizer( wxHORIZONTAL );
    upperSizer->Add( upperText, 0, wxALIGN_CENTER_VERTICAL|wxALL, 2 ); 
    upperSizer->Add( m_findText, 0, wxALIGN_CENTER_VERTICAL|wxALL, 2 ); 
    topSizer->Add( upperSizer, 0, wxALIGN_CENTER_HORIZONTAL |wxALL, 3);

    topSizer->Add( m_findList, 0, wxALIGN_CENTER_HORIZONTAL |wxALL, 2);
    
    wxBoxSizer *midSizer = new wxBoxSizer( wxHORIZONTAL );
    midSizer->Add( new wxStaticText(this, wxID_ANY, _T("with")), 0, wxALIGN_CENTER_VERTICAL|wxALL, 5 ); 
    midSizer->Add( m_replCombo, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5 ); 
    topSizer->Add(midSizer, 0, wxALIGN_CENTER_HORIZONTAL |wxALL, 4);
    
    topSizer->Add(new wxStaticText( this, wxID_ANY, _T("Tweak Flags") ), 0, wxALIGN_CENTER_HORIZONTAL |wxALL, 2);

    wxFlexGridSizer *flagsSizer = new wxFlexGridSizer( 2, 2, 3, 3);  //! Rows, cols, vgap, hgap // FlexGridSizer//wxBoxSizer( wxHORIZONTAL );
    flagsSizer->Add( m_wholeWord, 0, wxALIGN_LEFT |wxALL, 2);
    flagsSizer->Add( m_regex, 0, wxALIGN_LEFT |wxALL, 2);
    flagsSizer->Add( m_case, 0, wxALIGN_LEFT |wxALL, 2);
    flagsSizer->Add( m_preserve, 0, wxALIGN_LEFT |wxALL, 2);
    topSizer->Add( flagsSizer, 0, wxALIGN_CENTER_HORIZONTAL |wxALL, 2 );

    wxBoxSizer *buttonSizer = new wxBoxSizer( wxHORIZONTAL );
	buttonSizer->Add( m_yesButton, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5); 
	buttonSizer->Add( m_yesToAllButton, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5); 
	buttonSizer->Add( m_noButton, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);  
	buttonSizer->Add( new wxButton(this, wxID_CANCEL, "Cancel"), 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);   //! wxID_CANCEL handled by the parent
    topSizer->Add(buttonSizer,  0, wxALIGN_CENTER_HORIZONTAL |wxALL, 2);
    
    SetSizer(topSizer);  //use the sizer for layout
    topSizer->SetSizeHints(this);  //! Coordinate minimal window size
    m_noButton->SetDefault();  
    ///Positioning  - Input is "DefaultPosition"
//    int dlgW = GetSize().GetWidth();
//    int parW = parent->GetSize().GetWidth();
//    int edgeOff = 20;  ///edge offset
//    if( parW - dlgW > dlgW )  edgeOff = ( parW - dlgW ) >> 2;
    int x = pos.x + 90;
    int y = pos.y + 46;
    SetSize( x, y, wxDefaultCoord, wxDefaultCoord );
}

FAFReplaceDialog::~FAFReplaceDialog() {}

BEGIN_EVENT_TABLE(FAFReplaceDialog, wxDialog)
    EVT_BUTTON( wxID_YES, FAFReplaceDialog::OnButton )
    EVT_BUTTON( wxID_NO, FAFReplaceDialog::OnButton )
    EVT_BUTTON( ZION_FAF_YESTOALL, FAFReplaceDialog::OnButton )
    EVT_CHECKBOX( ZION_FR_REGEX, FAFReplaceDialog::OnFlagChecked )
    EVT_CHECKBOX( ZION_FR_MATCHCASE, FAFReplaceDialog::OnFlagChecked )
    EVT_CHECKBOX( ZION_FR_WHOLEWORD, FAFReplaceDialog::OnFlagChecked )
END_EVENT_TABLE()

void FAFReplaceDialog::OnButton( wxCommandEvent &event)
{
    if( event.GetId() == ZION_FAF_YESTOALL ) {
            if( wxMessageBox( _T("Are You Absolutely Sure you want to perform a Replace\n\
in this and ALL remaining Files? Changes may not be reversible!"), _T("Confirm"),
            wxYES_NO|wxICON_EXCLAMATION  ) != wxYES )
                return;
    }
    EndModal( event.GetId() );
}

void FAFReplaceDialog::OnFlagChecked( wxCommandEvent & event )  /// RELOAD LISTBOX
{
    //Diag //wxMessageBox(_T("In OnFlagChecked Handler"), _T("In Handler") );
    ///Prepare m_filesData with flags and search patt
    if( m_findText->GetValue() == _T("") )  return;  /// will not search for nothing
    int diaFlags = 0;
    diaFlags |= ( m_case->IsChecked() )? ZION_FR_FLAG_MATCHCASE : 0;
    diaFlags |= ( m_wholeWord->IsChecked() )? ZION_FR_FLAG_WHOLEWORD : 0;
    diaFlags |= ( m_regex->IsChecked() )? ZION_FR_FLAG_REGEX : 0;

    m_filesData->m_flags = diaFlags;
    m_filesData->m_patt = m_findText->GetValue();

    FilesResultsEvent evt( wxEVT_REMOTEFINDALL, -1 ); ///type, id
    evt.SetEventObject(this);  GetGrandParent()->ProcessEvent(evt);
    /// Reload ListBox
    m_findList->Clear();
    m_findList->InsertItems( m_filesData->m_results, 0 );
}

//---------------------------------------------------------------------------------

/** FILES RESULTS DIALOG **/
FilesResultsDialog::FilesResultsDialog(wxWindow *parent, wxPoint pos, int parentWidth, wxArrayString * frStr,
        const wxString & pattern, const wxString & topDir, FilesResultsData * filesData, FindReplData * frData)
	: wxDialog( parent, wxID_ANY, _T("Find in All Files Results"), pos, wxDefaultSize, wxDEFAULT_DIALOG_STYLE ) //! wx Default Args required !
{
    m_results = frStr;  //! save the pointer
    m_filesData = filesData; m_filesData->m_lastPid = -1;
    m_frData = frData;
    m_parent = parent;  //! save pointer to parent window (edit window)
    
    wxBoxSizer *topSizer = new wxBoxSizer( wxVERTICAL );
    wxStaticText * st = new wxStaticText( this, wxID_ANY, wxString::Format(_T("Found %d file%s containing \"%s\" [ %s ]\n\
FIRST occurrence shown - Click to show ALL occurrences" ),
            m_results->GetCount(), (m_results->GetCount() <= 1) ? "" : "s", pattern,topDir ) );
    topSizer->Add(st,  0, wxALIGN_LEFT |wxALL, 3);
    
    m_listBox = new wxListBox( this, ZION_ALLFILES_LISTBOX, wxDefaultPosition, wxDefaultSize, *frStr );
    topSizer->Add(m_listBox,  0, wxALIGN_CENTER_HORIZONTAL |wxALL, 3);
    
    m_closeSw = new wxCheckBox( this, ZION_FAF_CLOSESW, _T("Close Previous Window on Select") );
    m_closeSw->SetValue(true); ///Default to close previous window
    m_replButton = new wxButton( this, ZION_FAF_REPLACE, _T("Global Replace...") );
    m_okButton = new wxButton( this, wxID_OK, _T("Done") );

    wxBoxSizer *replSizer = new wxBoxSizer( wxHORIZONTAL );
    replSizer->Add( m_closeSw, 0, wxALIGN_CENTER_VERTICAL|wxALL, 4 );
    replSizer->Add( m_replButton, 0, wxALIGN_CENTER_VERTICAL|wxALL, 4 );
    replSizer->AddSpacer(6);
    replSizer->Add( m_okButton, 0, wxALIGN_CENTER_VERTICAL|wxALL, 4 );
    topSizer->Add(replSizer,  0, wxALIGN_CENTER_HORIZONTAL |wxALL, 2);
    
    SetSizer(topSizer);  ///use the sizer for layout
    topSizer->SetSizeHints(this);  //! Coordinate minimal window size
    m_okButton->SetDefault();  
    int x = pos.x + 20;
    int y = pos.y + 68;
    SetSize( x, y, wxDefaultCoord, wxDefaultCoord );
}

FilesResultsDialog::~FilesResultsDialog() {}

BEGIN_EVENT_TABLE(FilesResultsDialog, wxDialog)
    EVT_BUTTON( wxID_OK, FilesResultsDialog::OnOK )
    EVT_BUTTON( ZION_FAF_REPLACE, FilesResultsDialog::OnReplace )
    EVT_LISTBOX( ZION_ALLFILES_LISTBOX, FilesResultsDialog::OnLBClick )
    EVT_CHECKBOX( ZION_FAF_CLOSESW, FilesResultsDialog::OnCloseSw )
END_EVENT_TABLE()

void FilesResultsDialog::OnReplace( wxCommandEvent &event )
{
    int exitVal= 0;
    ///Loop thru all found files
    for( m_filesData->m_idx=0; m_filesData->m_idx < m_filesData->m_files.GetCount(); m_filesData->m_idx++ ) {
        ///Get FindAll Results for the current file
        FilesResultsEvent evt( wxEVT_REMOTEFINDALL, -1 ); ///type, id
        evt.SetEventObject(this);  m_parent->ProcessEvent(evt);

        ///Replace Dialog
        FAFReplaceDialog replDialog( this, GetPosition(), m_filesData, m_frData, m_filesData->m_files[m_filesData->m_idx] );
        exitVal = replDialog.ShowModal();
        
        if( exitVal == wxID_YES ) {
            ///Prepare m_filesData with flags, search patt and replace patt; Index already set
            int diaFlags = 0;
            diaFlags |= ( replDialog.m_case->IsChecked() )? ZION_FR_FLAG_MATCHCASE : 0;
            diaFlags |= ( replDialog.m_wholeWord->IsChecked() )? ZION_FR_FLAG_WHOLEWORD : 0;
            diaFlags |= ( replDialog.m_regex->IsChecked() )? ZION_FR_FLAG_REGEX : 0;
            diaFlags |= ( replDialog.m_preserve->IsChecked() )? ZION_FR_FLAG_PRESERVECASE : 0;
            m_filesData->m_flags = diaFlags;
            m_filesData->m_patt = replDialog.m_findText->GetValue();
            m_filesData->m_rPatt = replDialog.m_replCombo->GetValue();
            m_frData->UpdateReplString( m_filesData->m_rPatt );
            ///Do Replace on this file
            FilesResultsEvent evt( wxEVT_FILESREPLACE, -1 ); ///type, id
            evt.SetEventObject(this);
            m_parent->ProcessEvent(evt);
        }
        else if( exitVal == wxID_NO ) {
            continue;  /// skip this file
        }
        else if( exitVal == ZION_FAF_YESTOALL ) { ///Replace Dialog will confirm
            ///Prepare m_filesData with flags, search patt and replace patt; Index already set
            int diaFlags = 0;
            diaFlags |= ( replDialog.m_case->IsChecked() )? ZION_FR_FLAG_MATCHCASE : 0;
            diaFlags |= ( replDialog.m_wholeWord->IsChecked() )? ZION_FR_FLAG_WHOLEWORD : 0;
            diaFlags |= ( replDialog.m_regex->IsChecked() )? ZION_FR_FLAG_REGEX : 0;
            diaFlags |= ( replDialog.m_preserve->IsChecked() )? ZION_FR_FLAG_PRESERVECASE : 0;
            m_filesData->m_flags = diaFlags;
            m_filesData->m_patt = replDialog.m_findText->GetValue();
            m_filesData->m_rPatt = replDialog.m_replCombo->GetValue();
            m_frData->UpdateReplString( m_filesData->m_rPatt );
            
            FilesResultsEvent evt( wxEVT_FILESREPLACEALL, -1 ); ///type, id
            evt.SetEventObject(this);
            m_parent->ProcessEvent(evt);
            break;
        }
        else break; /// Includes case of wxID_CANCEL
    }
}

void FilesResultsDialog::OnCloseSw( wxCommandEvent &event)
{
    m_filesData->m_closeOnSwitch = m_closeSw->IsChecked(); 
}

void FilesResultsDialog::OnOK( wxCommandEvent &event)
{
    FilesResultsEvent evt( wxEVT_FILESRESULTS_CLOSING, -1 ); ///type, id
    evt.SetEventObject(this);
    m_parent->ProcessEvent(evt); /// This should destroy the dialog
    //event.Skip();
}

void FilesResultsDialog::OnLBClick( wxCommandEvent &event)
{
    FilesResultsEvent evt( wxEVT_FILESRESULTS_CLICKED, -1 ); /// Type, Id //evt.SetId(-1); evt.SetEventType(wxEVT_FILESRESULTS_CLICKED);
    evt.SetEventObject(this);
    evt.SetIdx( event.GetInt() ); ///We will send back only an index which will index our FilesResultsData.m_files wxArrayString
    m_parent->ProcessEvent(evt);
 }

//----------------------------------------------------------------------------------------

//! Custom event for Find in All Files Dialog    ---------------------------------
DEFINE_EVENT_TYPE( wxEVT_FILESRESULTS_CLICKED ) 
DEFINE_EVENT_TYPE( wxEVT_FILESRESULTS_CLOSING )
DEFINE_EVENT_TYPE( wxEVT_REMOTEFINDALL )
DEFINE_EVENT_TYPE( wxEVT_FILESREPLACE )
DEFINE_EVENT_TYPE( wxEVT_FILESREPLACEALL )
DEFINE_EVENT_TYPE( wxEVT_FILESFINDALL )

IMPLEMENT_DYNAMIC_CLASS(FilesResultsEvent, wxCommandEvent)
///Constructor
FilesResultsEvent::FilesResultsEvent(wxEventType commandType, int id)
    : wxCommandEvent(commandType, id)
{
    m_idx = 0;
}
/// Copy constructor - required!
FilesResultsEvent::FilesResultsEvent(const FilesResultsEvent& event)
    : wxCommandEvent(event)
{
    m_idx = event.m_idx;
}
//! -----------------------------------------------------------------------------------------------------------------------------

/*! FindRepl Data */
FindReplData::FindReplData( int flags, const wxString& findstr, const wxString& replstr, int lastSearchPos )
{
    m_flags = flags;
    m_findStr = findstr;
    m_replStr = replstr;
    m_lastSearchPos = lastSearchPos;  ///Used with FindAll Results list to track last replace pos
}

void FindReplData::UpdateFindString( const wxString & findstr )
{
    //! Add the current findString to frData->m_findStrs (wxArrayString) if not already present. i.e. search, if not found add
    bool found = false;
    for( int i = 0; i < m_findStrs.GetCount(); i++ ) {
        if( m_findStrs[i].Len() == findstr.Len() && m_findStrs[i].Cmp(findstr) == 0 ) { found = true; break; }
    }
    if(! found) {
        m_findStrs.Insert( findstr, 0 ); //m_findStrs.Sort();
        if( m_findStrs.GetCount() > 200 ) {
            m_findStrs.RemoveAt( 200, m_findStrs.GetCount() - 200 );
        }
    }
        ///m_diagStr = wxString::Format(_T("Updated m_findStrs array with \"%s\""), findstr );
    SetFindString( findstr ); //! Store the current findString separately in the frData
}

void FindReplData::UpdateReplString(const wxString & replstr)
{
    bool found = false;
    for( int i = 0; i < m_replStrs.GetCount(); i++ ) {
        if( m_replStrs[i] == replstr ) { found = true; break; }
    }
    if(! found) { m_replStrs.Add(replstr); m_replStrs.Sort(); }
    
    SetReplString( replstr ); //! Store the current findString separately in the frData
}

void FindReplData::Synch()
{
    ///Update find with all of replace
    for( int i = 0; i < m_replStrs.GetCount(); i++ ) {
        UpdateFindString( m_replStrs[i] ); 
    }
    m_replStrs = m_findStrs;
}

//! ------------------------------------------------------------------
/*! FindRepl Dialog */
FindReplDialog::FindReplDialog(wxWindow *parent, wxPoint pos, int parentWidth, FindReplData * frData, FindReplData * frData2)
	: wxDialog( parent, wxID_ANY, _T("Find - Replace Dialog"), pos, wxDefaultSize, wxDEFAULT_DIALOG_STYLE ) //! wx Default Args required !
{
    m_frData = frData; //! Store pointer to the FindReplData structure, created and initialized in zionedit.cpp
    m_frData2 = frData2;
    m_parent = parent;  //! save pointer to parent window (edit window)
    //strings are auto-inited on construction //m_lastFindStr = wxEmptyString;
    m_rtFind = false;
    m_rtRepl = false;
    
    m_lastFlags = 0;
    m_popup = new wxMenu;
    m_popup->Append(ZION_FR_CLEAR_FIND, _T("Clear Find History"));
    m_popup->Append(ZION_FR_CLEAR_REPL, _T("Clear Replace History"));
    //strings are auto-inited on construction //m_diagStr = wxEmptyString; // _T("Diagnostics Output"); //! DIAG
    
    ///Create widgets
    m_findCombo = new wxComboBox( this, ZION_FR_FIND_COMBO, m_frData->GetFindString(),
            wxDefaultPosition, wxDefaultSize, m_frData->m_findStrs, wxCB_SORT );
    m_findText = new wxTextCtrl( this, ZION_FIND_TEXT );
    m_replCombo = new wxComboBox( this, ZION_FR_REPL_COMBO, m_frData->GetReplString(),
            wxDefaultPosition, wxDefaultSize, m_frData->m_replStrs, wxCB_SORT );
    m_replText = new wxTextCtrl( this, ZION_REPL_TEXT );
    wxSize asize(FRMinX, FRMinY);  //! Size must be requested BEFORE adding to the parent sizer
    m_findCombo->SetMinSize(asize);
    m_findText->SetMinSize(asize);  m_findText->Show(false);
    m_replCombo->SetMinSize(asize);
    m_replText->SetMinSize(asize);  m_replText->Show(false);
    m_case = new wxCheckBox( this, ZION_FR_MATCHCASE, _T("Match &case") );
    m_case->SetValue( m_frData->GetFlags() & ZION_FR_FLAG_MATCHCASE );
    m_wholeWord = new wxCheckBox( this, ZION_FR_WHOLEWORD, _T("Match &whole word") ); //! Init according to frData!!!
    m_wholeWord->SetValue( m_frData->GetFlags() & ZION_FR_FLAG_WHOLEWORD );
    m_regex = new wxCheckBox( this, ZION_FR_REGEX, _T("Regular &Exp. Find") ); 
    m_regex->SetValue( m_frData->GetFlags() & ZION_FR_FLAG_REGEX );
    m_preserve = new wxCheckBox( this, ZION_FR_PRESERVECASE, _T("&Preserve replaced case") );
    m_preserve->SetValue( m_frData->GetFlags() & ZION_FR_FLAG_PRESERVECASE );
    m_preserve->SetToolTip(_T("Try to maintain the case of the string to be replaced") );
    m_clearButton = new wxButton( this, ZION_FR_CLEAR, _T("C&lear >"), wxDefaultPosition, wxSize( 50, -1 ) );
    m_clearButton->SetToolTip(_T("Clear Find or Replace History") );
    m_synchButton = new wxButton( this, ZION_FR_SYNCH, _T("F <=> R"), wxDefaultPosition, wxSize( 58, -1) );
    m_synchButton->SetToolTip(_T("Synch Find and Replace history") );
    m_static = new wxStaticText( this, wxID_ANY, _T("") );  ///DIAG
    m_list = new wxListBox( this, ZION_FIND_LIST );
    m_list->Show(false);
            
    m_mainSizer = new wxBoxSizer( wxVERTICAL );
    wxBoxSizer *topSizer = new wxBoxSizer( wxHORIZONTAL );
    wxBoxSizer *rightSizer = new wxBoxSizer( wxVERTICAL );
    wxBoxSizer *leftSizer = new wxBoxSizer( wxVERTICAL );
    wxFlexGridSizer *flagsSizer = new wxFlexGridSizer( 2, 2, 3, 3);  //! Rows, cols, vgap, hgap // FlexGridSizer//wxBoxSizer( wxHORIZONTAL );
    
    m_findReplSizer = new wxFlexGridSizer( -1, 2, 3, 3);  //! Rows, cols, vgap, hgap // FlexGridSizer
    //! If we want to receive events from our widgets we must give them unique IDs
    m_findReplSizer->Add(new wxStaticText( this, wxID_ANY, _T("Find what:") ), 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL,2 );
    m_findReplSizer->Add( m_findCombo, 1, wxALIGN_RIGHT|wxGROW|wxALL, 2 );
    m_findReplSizer->Add(new wxStaticText( this, wxID_ANY, _T("Replace with:") ),   0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 2 );
    m_findReplSizer->Add( m_replCombo, 1, wxALIGN_RIGHT|wxGROW|wxALL, 2 );
    leftSizer->Add( m_findReplSizer, 0, wxALIGN_LEFT | wxALIGN_TOP| wxALL, 2);

    m_findReplSizer2 = new wxFlexGridSizer( -1, 2, 3, 3);  //! Rows, cols, vgap, hgap // FlexGridSizer
    //! If we want to receive events from our widgets we must give them unique IDs
    m_findReplSizer2->Add(new wxStaticText( this, wxID_ANY, _T("Find what:") ), 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL,2 );
    m_findReplSizer2->Add( m_findText, 1, wxALIGN_RIGHT|wxGROW|wxALL, 2 );
    m_findReplSizer2->Add(new wxStaticText( this, wxID_ANY, _T("Replace with:") ),   0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 2 );
    m_findReplSizer2->Add( m_replText, 1, wxALIGN_RIGHT|wxGROW|wxALL, 2 );
    leftSizer->Add( m_findReplSizer2, 0, wxALIGN_LEFT | wxALIGN_TOP| wxALL, 2);
    m_findReplSizer2->Show(false);

    flagsSizer->Add( m_case, 0, wxALIGN_LEFT |wxALL, 2);
    flagsSizer->Add( m_regex, 0, wxALIGN_LEFT |wxALL, 2);
    flagsSizer->Add( m_wholeWord, 0, wxALIGN_LEFT |wxALL, 2);
    flagsSizer->Add( m_preserve, 0, wxALIGN_LEFT |wxALL, 2);

    leftSizer->Add( flagsSizer, 0, wxALIGN_LEFT |wxALL, 2);
    
    wxBoxSizer * lowerSizer = new wxBoxSizer( wxHORIZONTAL );
    lowerSizer->Add( m_clearButton, 0, wxALIGN_LEFT |wxALL, 2);
    lowerSizer->Add( m_synchButton, 0, wxALIGN_LEFT |wxALL, 2);
    lowerSizer->Add( m_static, 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL |wxALL, 2);
    leftSizer->Add( lowerSizer, 0, wxALIGN_LEFT |wxALL, 1);
    
    m_findButton = new wxButton( this, ZION_FR_FIND, _T("&Find") );
    m_replButton = new wxButton( this, ZION_FR_REPLACE, _T("&Replace") );
    m_replAllButton = new wxButton( this, ZION_FR_REPLACE_ALL, _T("Replace &All") );
    m_doneButton = new wxButton( this, wxID_OK, _T("&Done") );
    rightSizer->Add( m_findButton, 0, wxALIGN_CENTER_HORIZONTAL |wxALL, 3 );
    rightSizer->Add( m_replButton, 0, wxALIGN_CENTER_HORIZONTAL |wxALL, 3 );
    rightSizer->Add( m_replAllButton, 0, wxALIGN_CENTER_HORIZONTAL |wxALL, 3 );
    rightSizer->Add( m_doneButton, 0, wxALIGN_CENTER_HORIZONTAL |wxALL, 3 );
    
    topSizer->Add(leftSizer, 0, wxALIGN_LEFT |wxALL, 2);
    topSizer->Add(rightSizer, 0, wxALIGN_TOP| wxALIGN_CENTER_HORIZONTAL |wxALL, 2);
    
    m_mainSizer->Add( topSizer, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 2 );
    m_mainSizer->Add( m_list, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 2 );
    
    SetSizer(m_mainSizer);  ///use the sizer for layout
    m_mainSizer->SetSizeHints(this);  //! Coordinate minimal window size
    m_findButton->SetDefault();
    ///Positioning
    int dlgW = GetSize().GetWidth();
    int edgeOff = 20;  ///edge offset
    if( parentWidth - dlgW > dlgW )  edgeOff = ( parentWidth - dlgW ) >> 3;
    
    int x = pos.x + parentWidth - dlgW - edgeOff;
    int y = pos.y + 200;
    SetSize( x, y, wxDefaultCoord, wxDefaultCoord );
}

FindReplDialog::~FindReplDialog() {}

BEGIN_EVENT_TABLE(FindReplDialog, wxDialog)
    EVT_BUTTON( wxID_OK, FindReplDialog::OnDone )
    EVT_BUTTON( ZION_FR_FIND, FindReplDialog::OnFind )
    EVT_BUTTON( ZION_FR_CLEAR, FindReplDialog::OnClear )
    EVT_BUTTON( ZION_FR_SYNCH, FindReplDialog::OnSynch )
    EVT_TEXT_ENTER( ZION_FR_FIND_COMBO, FindReplDialog::OnFind )
    
    EVT_TEXT( ZION_FR_FIND_COMBO, FindReplDialog::OnFindComboChange )
    EVT_TEXT( ZION_FR_REPL_COMBO, FindReplDialog::OnReplComboChange )
    EVT_TEXT( ZION_FIND_TEXT, FindReplDialog::OnRTFind )
    EVT_CHECKBOX( ZION_FR_WHOLEWORD, FindReplDialog::OnRTFind )
    EVT_CHECKBOX( ZION_FR_MATCHCASE, FindReplDialog::OnRTFind )
    EVT_CHECKBOX( ZION_FR_REGEX, FindReplDialog::OnRTFind )
    EVT_LISTBOX( ZION_FIND_LIST, FindReplDialog::OnFindListClick )
    
    EVT_BUTTON( ZION_FR_REPLACE, FindReplDialog::OnReplace )
    EVT_BUTTON( ZION_FR_REPLACE_ALL, FindReplDialog::OnReplace )
    //EVT_CHECKBOX( ZION_FR_PRESERVECASE, FindReplDialog::OnRTFind )
END_EVENT_TABLE()
/* Luci you got some 'splaining to do:
Both m_rtFind and m_rtRepl  remove the combos and replace them with text boxes.
The only difference is that the m_rtRepl doesn't really do a real time replace or find. It
is called that just for symmetrical purposes.
*/

void FindReplDialog::OnClear( wxCommandEvent &event )
{
//    wxPoint pt0 = m_parent->GetPosition();
//    wxPoint pt = this->GetPosition();  /// The dialog //relative to the parent window
    //ClientToScreen(&pt.x, &pt.y);
//    wxPoint pt2 = m_clearButton->GetPosition(); /// Relative to the dialog
        /// Do it
    m_parent->PopupMenu(m_popup); /// Use default position 
    // wxPoint(pt0.x+pt.x+pt2.x, pt0.y+pt.y+pt2.y) */ ); //! wxWindow->PopupMenu
}
void FindReplDialog::OnSynch( wxCommandEvent &event )
{
    m_frData->Synch();
    RedrawFRCombo( true ); ///Redraw find
    RedrawFRCombo( false ); ///Redraw Replace
}

void FindReplDialog::OnFindComboChange( wxCommandEvent & event )  ///Do a new search
{   ///This will only be called once when typing begins (side effect - also when listbox selected)
    ///TODO FIX BELOW
    m_findText->SetValue( m_findCombo->GetValue() );
    m_replText->SetValue( m_replCombo->GetValue() );
    //m_findCombo->Show( false );
    m_findReplSizer2->Show( true );
    m_findReplSizer->Show( false );
    m_findText->SetInsertionPoint( m_findCombo->GetInsertionPoint() );
    m_findText->SetFocus();
    //wxLogStatus(wxString::Format(_T("Insertion Point: %d"), m_findCombo->GetInsertionPoint() ) );
    m_mainSizer->Layout();
    m_mainSizer->SetSizeHints(this);  /// We have to do this here since OnRTFind will not update till 3 chars entered
    m_rtFind = true;  ///Start realtime find mode
    wxCommandEvent anEvent( wxEVT_COMMAND_TEXT_UPDATED, ZION_FIND_TEXT );  ///Hand off to OnRTFind
    AddPendingEvent(anEvent);
}

void FindReplDialog::OnReplComboChange( wxCommandEvent & event )
{   ///This will only be called once when typing begins (side effect - also on combo list selection
    ///TODO FIX BELOW
    m_findText->SetValue( m_findCombo->GetValue() );
    m_replText->SetValue( m_replCombo->GetValue() );
    //m_findCombo->Show( false );
    m_findReplSizer2->Show( true );
    m_findReplSizer->Show( false );
    m_replText->SetInsertionPoint( m_replCombo->GetInsertionPoint() );
    m_replText->SetFocus();
    m_mainSizer->Layout();
    m_mainSizer->SetSizeHints(this);  /// We have to do this here since we wont be doing a realtime find
    m_rtRepl = true;  ///Start realtime repl mode, just simply switches out the combos
}

void FindReplDialog::OnRTFind( wxCommandEvent & event )  ///Do a new search
{
    if( m_rtFind ) {
        wxString str = m_findText->GetValue();
        if( str.Len() > 2 ) {
            m_frData2->UpdateFindString( str ); //! Save the pattern used
            m_frData2->SetFlags( ( 0 | m_regex->GetValue() ? ZION_FR_FLAG_REGEX : 0 ) |
                    ( m_case->GetValue() ? ZION_FR_FLAG_MATCHCASE : 0 ) |
                    ( m_wholeWord->GetValue() ? ZION_FR_FLAG_WHOLEWORD : 0 ) );
            wxCommandEvent anEvent( wxEVT_COMMAND_TEXT_UPDATED, ZION_RTFIND );
            m_parent->ProcessEvent(anEvent);
            //RedrawFindCombo();
            //m_findButton->SetLabel(_T("&Find")); ///New search pattern //m_findButton->SetDefault();
//    SetTitle(wxString::Format(_T("Found:  %d"), m_frData2->m_results.GetCount() ) );
        int cnt = m_frData2->m_results.GetCount();
        SetTitle( wxString::Format(_T("Found:  %d"), cnt ) );
        m_findButton->SetLabel( (cnt > 0 ? _T("Find Next") : _T("Find")) ) ;
    m_list->Set( m_frData2->m_results );
    m_list->Show(true);
            m_mainSizer->Layout();
            m_mainSizer->SetSizeHints(this);  //! Coordinate minimal window size ///MUST BE PLACED AFTER Layout()
        }
    }
}

void FindReplDialog::RedrawFRCombo(bool find)
{
    ///TODO - ANALYSE THIS! Combo pointer is loose.
    wxComboBox * combo = find? m_findCombo : m_replCombo;
    m_findReplSizer->Detach( (wxWindow *) combo );
    combo->Destroy(); combo = (wxComboBox *) NULL;
    combo = new wxComboBox( this, (find? ZION_FR_FIND_COMBO : ZION_FR_REPL_COMBO),
            (find? m_frData->GetFindString() : m_frData->GetReplString()),
            wxDefaultPosition, wxDefaultSize,
            (find? m_frData->m_findStrs : m_frData->m_replStrs), wxCB_SORT );
    combo->SetMinSize(wxSize(FRMinX, FRMinY)); //! Size must be requested BEFORE adding to the parent sizer
    m_findReplSizer->Insert( (find?1:3), (wxWindow *) combo, 1, wxALIGN_RIGHT|wxGROW|wxALL, 2 );
    m_findReplSizer->Layout();
    if(find) m_findCombo = combo;
    else m_replCombo = combo;
}

void FindReplDialog::OnFind( wxCommandEvent &event )
{
    ///Restore Combos
    m_findReplSizer->Show( true );
    m_findReplSizer2->Show( false );
        //m_list->Show(false);  ///We won't hide the list at all
//    
//    m_frData2->m_findStrs.Empty();
//    m_frData2->m_results.Empty();
//    ///SHOW A FIND ALL LIST
//    wxString str = (m_rtFind || m_rtRepl) ? m_findText->GetValue() : m_findCombo->GetValue();
//    m_frData2->UpdateFindString( str ); //! Save the pattern to search for
//    m_frData2->SetFlags( ( 0 | m_regex->GetValue() ? ZION_FR_FLAG_REGEX : 0 ) |
//            ( m_case->GetValue() ? ZION_FR_FLAG_MATCHCASE : 0 ) |
//            ( m_wholeWord->GetValue() ? ZION_FR_FLAG_WHOLEWORD : 0 ) );
//    wxCommandEvent anEvent( wxEVT_COMMAND_TEXT_UPDATED, ZION_RTFIND );
//    m_parent->ProcessEvent(anEvent);  ///Edit module does the handling
    
    //m_findButton->SetLabel(_T("&Find")); ///New search pattern //m_findButton->SetDefault();
    
    wxString findstr = (m_rtFind || m_rtRepl) ? m_findText->GetValue() : m_findCombo->GetValue();
    if( findstr != wxEmptyString ) {
        //m_diagStr.Empty(); ///Init
        m_frData->UpdateFindString(findstr);
        int currentFlags = 0 | ( m_wholeWord->GetValue() ? ZION_FR_FLAG_WHOLEWORD : 0 ) |
                ( m_case->GetValue() ? ZION_FR_FLAG_MATCHCASE : 0 ) |
                ( m_preserve->GetValue() ? ZION_FR_FLAG_PRESERVECASE : 0 ) |
                ( m_regex->GetValue() ? ZION_FR_FLAG_REGEX : 0 );
        m_frData->SetFlags(currentFlags);
        
        //! Detach and Recreate our combo to ensure it includes our latest findstr
        RedrawFRCombo(true); /// true for find
        
        /// Replace Stuff */
        wxString  replstr = (m_rtFind || m_rtRepl) ? m_replText->GetValue() : m_replCombo->GetValue();  ///TODO - conditional for rtRepl
        if( replstr != wxEmptyString ) {
            m_frData->UpdateReplString(replstr);
            //! Detach and Recreate our combo to ensure it includes our latest replstr
            RedrawFRCombo(false);  /// false for replace
        }
        ///Reset to combo mode & clean up
        m_rtFind = false;
        m_rtRepl = false;
        
//        int savPos = GetCurrentPos();
        ///SHOW A FIND ALL LIST  - Initial Pos saved in the handler
        m_frData2->m_findStrs.Empty();
        m_frData2->m_results.Empty();
        wxString str = (m_rtFind || m_rtRepl) ? m_findText->GetValue() : m_findCombo->GetValue();
        m_frData2->UpdateFindString( str ); //! Save the pattern to search for
        m_frData2->SetFlags( ( 0 | m_regex->GetValue() ? ZION_FR_FLAG_REGEX : 0 ) |
                ( m_case->GetValue() ? ZION_FR_FLAG_MATCHCASE : 0 ) |
                ( m_wholeWord->GetValue() ? ZION_FR_FLAG_WHOLEWORD : 0 ) );
        wxCommandEvent anEvent( wxEVT_COMMAND_TEXT_UPDATED, ZION_RTFIND );
        m_parent->ProcessEvent(anEvent);  ///Edit module does the handling
        m_list->Set( m_frData2->m_results );
        m_list->Show(true);
        int cnt = m_frData2->m_results.GetCount();
        SetTitle( wxString::Format(_T("Found:  %d"), cnt ) );
        m_findButton->SetLabel( (cnt > 0 ? _T("Find Next") : _T("Find")) ) ;

        //! Prepare an event and send to parent window
        FindReplEvent evt; evt.SetId(-1);
        if( m_lastFindStr != findstr || currentFlags != m_lastFlags ) {
            m_lastFindStr = findstr; m_lastFlags = currentFlags;
            evt.SetEventType(wxEVT_FIND);
            //m_diagStr.Append(_T("  |  FIND Event"));
        }
        else {
            evt.SetEventType(wxEVT_FIND_NEXT); //! findstr didn't change so do a FIND_NEXT
            //m_diagStr.Append(_T("  |  FIND_NEXT Event"));
        }
        evt.SetEventObject(this);
        //m_static->SetLabel(m_diagStr); //! DIAG
        m_parent->ProcessEvent(evt); //! Send it
        
        //m_findButton->SetLabel(_T("Find Next")); //m_findButton->SetDefault();
        
        ///Let layout be the last action step
        m_mainSizer->Layout();
        m_mainSizer->SetSizeHints(this);  /// We have to do this here since OnRTFind will not update till 3 chars entered
    }
    else m_static->SetLabel(_T("No Find String Specified!"));
    
    m_findCombo->SetFocus(); ///CRITICAL - for default button bug
}

void FindReplDialog::OnReplace(wxCommandEvent &event)
{
    m_findReplSizer->Show( true );
    m_findReplSizer2->Show( false );

    /// Find Stuff
    wxString findstr = (m_rtFind || m_rtRepl) ? m_findText->GetValue() : m_findCombo->GetValue();
    if( findstr == wxEmptyString ) return;  //! No sense trying to replace nothing
    m_frData->UpdateFindString(findstr);
    //! Detach and Recreate our combo to ensure it includes our latest findstr
    RedrawFRCombo(true);

    /// Replace Stuff 
    wxString replstr = (m_rtFind || m_rtRepl) ? m_replText->GetValue() : m_replCombo->GetValue();
    m_frData->UpdateReplString(replstr);
    //! FLAGS
    m_frData->SetFlags( 0 | ( m_wholeWord->GetValue() ? ZION_FR_FLAG_WHOLEWORD : 0 ) |
        ( m_case->GetValue() ? ZION_FR_FLAG_MATCHCASE : 0 ) |
        ( m_preserve->GetValue() ? ZION_FR_FLAG_PRESERVECASE : 0 ) |
        ( m_regex->GetValue() ? ZION_FR_FLAG_REGEX : 0 ) );
    //! Detach and Recreate our combo to ensure it includes our latest replstr
    RedrawFRCombo(false);  /// false for replace
    
    ///Reset to combo mode & clean up
    m_rtFind = false;
    m_rtRepl = false;

    //! Prepare an event and send to parent window
    FindReplEvent evt;
    if(event.GetId() == ZION_FR_REPLACE) evt.SetEventType(wxEVT_FIND_REPLACE);
    else if(event.GetId() == ZION_FR_REPLACE_ALL) evt.SetEventType(wxEVT_FIND_REPLACE_ALL);
    evt.SetId(-1);
    evt.SetEventObject(this);
    m_parent->ProcessEvent(evt); //! Send it
    
    m_frData2->m_findStrs.Empty();
    m_frData2->m_results.Empty();
    ///SHOW A FIND ALL LIST
    wxString str = (m_rtFind || m_rtRepl) ? m_findText->GetValue() : m_findCombo->GetValue();
    m_frData2->UpdateFindString( str ); //! Save the pattern to search for
    m_frData2->SetFlags( ( 0 | m_regex->GetValue() ? ZION_FR_FLAG_REGEX : 0 ) |
            ( m_case->GetValue() ? ZION_FR_FLAG_MATCHCASE : 0 ) |
            ( m_wholeWord->GetValue() ? ZION_FR_FLAG_WHOLEWORD : 0 ) );
    wxCommandEvent anEvent( wxEVT_COMMAND_TEXT_UPDATED, ZION_RTFIND );
    m_parent->ProcessEvent(anEvent);  ///Edit module does the handling
    m_list->Set( m_frData2->m_results );
    m_list->Show(true);
    int cnt = m_frData2->m_results.GetCount();
    SetTitle( wxString::Format(_T("Found:  %d"), cnt ) );
    m_findButton->SetLabel( (cnt > 0 ? _T("Find Next") : _T("Find")) ) ;
    
    ///Let layout be the last action step
    m_mainSizer->Layout();
    m_mainSizer->SetSizeHints(this);  /// We have to do this here due to the possiibility of a previous RTFind
}

void FindReplDialog::OnFindListClick( wxCommandEvent &event )
{
    wxString str( event.GetString() );
    DoSelection(str);
}

void FindReplDialog::DoSelection( wxString & str )
{
    wxString str2;
    long x=0, y=0, z=0;
    str = str.BeforeFirst(']'); //! Cut end
    str = str.AfterFirst('['); //! Cut start
    str2 = str.BeforeFirst(','); //! Isolate line value
    str2.ToLong(&y);  //! Put result in y
    str2 = str.AfterFirst(','); /// save positions
    /// Patch in here for max position
    str = str2.BeforeFirst('-'); /// get start
    str.ToLong(&x);
    str = str2.AfterFirst('-');  /// get end
    str.ToLong(&z);
    myFindResultsEvent evt( wxEVT_RESULTS_CLICKED, -1 ); /// Type, Id //evt.SetId(-1); evt.SetEventType(wxEVT_RESULTS_CLICKED);
    evt.SetEventObject(this);
    evt.SetX(x); evt.SetY(y); evt.SetZ(z); //! Store Linenum and positions in the event: [y:x-z]
    m_parent->ProcessEvent(evt); //AddPendingEvent(evt);
    //m_lastLine = (int) y; ///Last Line!!!
}

void FindReplDialog::OnDone(wxCommandEvent &event)
{
    m_frData2->m_findStrs.Empty();  ///These are temporary, clean up
    m_frData2->m_results.Empty();
    FindReplEvent evt(wxEVT_FIND_CLOSE, -1); //done in one step //evt.SetId(-1); evt.SetEventType();
    evt.SetEventObject(this);
    m_parent->AddPendingEvent(evt); //! This should destroy the dialog ///process immediately
    //event.Skip();
}
//------------------------------------------------------------------------------------------------------------------------------------------------------------
