#ifndef FINDREPLDIALOG_H
#define FINDREPLDIALOG_H
///We didn't want to include all of stc.h here
#define ZION_FR_FLAG_WHOLEWORD 2 ///We use the same integers as wxStc
#define ZION_FR_FLAG_MATCHCASE 4
#define ZION_FR_FLAG_PRESERVECASE 0x01000000  /// Roh
#define ZION_FR_FLAG_REGEX 0x00200000

class FindReplData {
    public:
        FindReplData::FindReplData( int flags=0, const wxString& findstr=wxEmptyString,
                const wxString& replstr=wxEmptyString, int m_lastSearchPos=0 );
        void SetFindString( const wxString& findStr) { m_findStr = findStr; }  ///attrib_writer
        void SetReplString( const wxString& replStr) { m_replStr = replStr; }
        wxString GetFindString() { return m_findStr; }  ///attrib_reader
        wxString GetReplString() { return m_replStr; }
        void SetFlags( int flags ) { m_flags = flags; }
        int GetFlags() { return m_flags; }
        void UpdateFindString(const wxString& findstr);  ///Add a new string to the wxArrayString of Find Strings if necessary and store it as the current string/last string used
        void UpdateReplString(const wxString& replstr); ///Ditto for replace
        void Synch(); ///synchronize m_findStrs and m_replStrs
        wxArrayString m_findStrs;
        wxArrayString m_replStrs;
        wxArrayString m_results;  /// Local FindAll results
        int m_flags;  //! Search flag
        int m_lastSearchPos; ///Last search line ///Track find all results clicked (or replaced)
    private:
        wxString m_findStr, m_replStr; ///Current or last strings used
};

/** Find Dialog **/
class FindDialog: public wxDialog
{
  public:
    FindDialog( wxWindow *parent, wxPoint pos, int parentWidth, FindReplData * frData,
            FindReplData * frData2, wxArrayString * rdirs = NULL,
            bool findAllMode = false, bool allFilesMode = false );
    ~FindDialog();
    wxCheckBox * m_all, * m_regex, * m_wholeWord, * m_case, *m_allFiles;
    wxComboBox * m_dirName;
    wxListBox * m_list;
    
  private:
    void OnOK( wxCommandEvent &event );
    void OnCancel( wxCommandEvent &event );
    void OnFindListClick( wxCommandEvent & event );
    void DoSelection( wxString & str );
    void OnFindComboText( wxCommandEvent &event );
    void OnRTFind( wxCommandEvent &event );
    bool m_rtFind;  ///Are we in realtime find mode?
//    void RedrawFindCombo();
    void OnChooseDir( wxCommandEvent & );
    void OnAllFilesCheck( wxCommandEvent & );
    wxBoxSizer * m_topSizer;
    wxButton * m_okButton, * m_doneBtn, * m_dirBrowse;
    int m_flags;
    wxWindow * m_parent;
    wxComboBox * m_findCombo;
    wxTextCtrl * m_text;
    FindReplData * m_frData;
    FindReplData * m_frData2;
  
  DECLARE_EVENT_TABLE()
};

///--------------------------------------------------------------------------------------------------FILES RESULTS
class FilesResultsData ///Storage class for Find in All Files dialog
{
    public:
    //FilesResultsData::FilesResultsData( const wxString& patt, const int flags, const wxArrayString& files );
    FilesResultsData::FilesResultsData() { ///wxString, wxArrayString is always init to empty, so no need to add them here
        m_flags=0; m_idx=0; m_lastPid=-1; m_closeOnSwitch = true; }
    FilesResultsData::~FilesResultsData() { };
        wxString m_patt;
        wxString m_rPatt;
        int m_flags;
        // currentFile == m_files[m_idx] //wxString m_currentFile; ///For use in a replace iteration
        wxArrayString m_results;  ///From a findAll operation on currentFile m_files[m_idx]
        int m_idx;
        bool m_closeOnSwitch;  ///Close previous window on exit
        long m_lastPid;
        wxArrayString m_files; ///List of full filenames - absolute paths
};

class FAFReplaceDialog: public wxDialog  /// Replace from Find in All Files //will be used as modal
{
  public:
    FAFReplaceDialog( wxWindow *parent, wxPoint pos, FilesResultsData * filesData, FindReplData * frData, const wxString & file  ); /// frData Needed for replace history
    ~FAFReplaceDialog();
    wxCheckBox * m_case;
    wxCheckBox * m_wholeWord;
    wxCheckBox * m_regex;
    wxCheckBox * m_preserve;
    wxListBox * m_findList;
    wxComboBox * m_replCombo;
    wxTextCtrl * m_findText;
    
  private:
    void OnButton( wxCommandEvent &event );
    void OnFlagChecked( wxCommandEvent &event );
//    void OnNo( wxCommandEvent &event );
//    void OnYesToAll( wxCommandEvent &event );
    wxButton * m_yesButton;
    wxButton * m_noButton;
    wxButton * m_yesToAllButton;
//    wxButton * m_cancelButton;
    FilesResultsData * m_filesData;
    //FindReplData * m_frData;
  
  DECLARE_EVENT_TABLE()
};

class FilesResultsDialog: public wxDialog
{
  public:
    FilesResultsDialog( wxWindow *parent, wxPoint pos, int parentWidth, wxArrayString *, ///OPTIMIZE - we should have the "pattern" in "frData", so redundant
            const wxString& pattern, const wxString & topDir, FilesResultsData * filesData, FindReplData * frData ); ///TODO Names need changing - frData
    ~FilesResultsDialog();
    
  private:
    void OnOK( wxCommandEvent &event );
    void OnLBClick( wxCommandEvent &event );
    void OnCloseSw( wxCommandEvent &event );
    void OnReplace( wxCommandEvent & );
    void OnFlagChecked( wxCommandEvent &event );
    wxButton * m_okButton;
    wxCheckBox * m_closeSw;
    wxArrayString * m_results;
    wxListBox * m_listBox;
    wxWindow * m_parent;
    wxTextCtrl * m_replText;
    wxCheckBox * m_preserveCase;
    wxButton * m_replButton;
    int m_idx;  ///This will track our iteration in a global replacement loop,
            ///so that a "Yes To All" applies to all the rest of the files
    FilesResultsData * m_filesData;
    FindReplData * m_frData;
  
  DECLARE_EVENT_TABLE()
};

class FilesResultsEvent : public wxCommandEvent {
    public:
        FilesResultsEvent( wxEventType commandType = 0, int id=0 );
        FilesResultsEvent( const FilesResultsEvent& event ); //! Copy constructor
        
        void SetIdx( int val )    { m_idx = val; }
        int GetIdx() const        { return m_idx; }
        
        virtual wxEvent* Clone() const { return new FilesResultsEvent(*this); }
    private:
        DECLARE_DYNAMIC_CLASS(FilesResultsEvent)
        
        int m_idx; /// Index into the results wxArrayString
};

BEGIN_DECLARE_EVENT_TYPES()
    DECLARE_EXPORTED_EVENT_TYPE( , wxEVT_FILESRESULTS_CLICKED,             23261)
    DECLARE_EXPORTED_EVENT_TYPE( , wxEVT_FILESRESULTS_CLOSING,             23262)
    DECLARE_EXPORTED_EVENT_TYPE( , wxEVT_FILESREPLACE,             23263)
    DECLARE_EXPORTED_EVENT_TYPE( , wxEVT_FILESREPLACEALL,             23264)
    DECLARE_EXPORTED_EVENT_TYPE( , wxEVT_REMOTEFINDALL,             23265)
//    DECLARE_EXPORTED_EVENT_TYPE( , wxEVT_FILESFINDALL,             57265)
END_DECLARE_EVENT_TYPES()

typedef void (wxEvtHandler::*FilesResultsEventFunction)(FilesResultsEvent&);

#define EVT_FILESRESULTS_CLICKED(id, fn)  DECLARE_EVENT_TABLE_ENTRY( wxEVT_FILESRESULTS_CLICKED,  id, wxID_ANY, (wxObjectEventFunction) (wxEventFunction)  wxStaticCastEvent( FilesResultsEventFunction, & fn ), (wxObject *) NULL ),
#define EVT_FILESRESULTS_CLOSING(id, fn)  DECLARE_EVENT_TABLE_ENTRY( wxEVT_FILESRESULTS_CLOSING,  id, wxID_ANY, (wxObjectEventFunction) (wxEventFunction)  wxStaticCastEvent( FilesResultsEventFunction, & fn ), (wxObject *) NULL ),
#define EVT_REMOTEFINDALL(id, fn)  DECLARE_EVENT_TABLE_ENTRY( wxEVT_REMOTEFINDALL,  id, wxID_ANY, (wxObjectEventFunction) (wxEventFunction)  wxStaticCastEvent( FilesResultsEventFunction, & fn ), (wxObject *) NULL ),
#define EVT_FILESREPLACE(id, fn)  DECLARE_EVENT_TABLE_ENTRY( wxEVT_FILESREPLACE,  id, wxID_ANY, (wxObjectEventFunction) (wxEventFunction)  wxStaticCastEvent( FilesResultsEventFunction, & fn ), (wxObject *) NULL ),
//#define EVT_FILESFINDALL(id, fn)  DECLARE_EVENT_TABLE_ENTRY( wxEVT_FILESFINDALL,  id, wxID_ANY, (wxObjectEventFunction) (wxEventFunction)  wxStaticCastEvent( FilesResultsEventFunction, & fn ), (wxObject *) NULL ),
#define EVT_FILESREPLACEALL(id, fn)  DECLARE_EVENT_TABLE_ENTRY( wxEVT_FILESREPLACEALL,  id, wxID_ANY, (wxObjectEventFunction) (wxEventFunction)  wxStaticCastEvent( FilesResultsEventFunction, & fn ), (wxObject *) NULL ),

///--------------------------------------------------------------------------------------------------FIND RESULTS CLASSES
class FindResultsDialog: public wxDialog
{
  public:
    FindResultsDialog( wxWindow *parent, wxPoint pos, int parentWidth, FindReplData * frData );
    ~FindResultsDialog();
    
  private:
    void OnOK( wxCommandEvent &event );
    void OnLBClick( wxCommandEvent &event );
    void OnReplMode( wxCommandEvent & event );
    void OnReplCombo( wxCommandEvent & event );
    void OnReloadList( wxCommandEvent &event );
    //void ReloadRList( wxCommandEvent &event );
    void OnReplSels( wxCommandEvent &event );
    void OnReplAll( wxCommandEvent &event );
    void DoSelection( wxString & str );  /// str not const
    
    FindReplData * m_frData;
    int m_lastLine;
    wxStaticText * m_topText;
    wxTextCtrl * m_findText;
    wxListBox * m_listBox;
    wxListBox * m_rlistBox;
    wxCheckBox * m_replMode;
    wxButton * m_searchAgain;
    wxButton * m_okButton; ///Done Button
    wxStaticLine * m_statLine;
    //wxCheckBox * m_tweakFlags;
    wxCheckBox * m_case;
    wxCheckBox * m_wholeWord;
    wxCheckBox * m_regex;
    wxCheckBox * m_preserve;
    
    wxStaticText * m_statText;
    wxComboBox * m_replCombo;
    wxTextCtrl * m_replText;
    //wxButton * m_replSelButton;
    wxButton * m_replAllButton;
    wxWindow * m_parent;
    wxBoxSizer * m_topSizer;
    wxBoxSizer * m_replStuff;
  
  DECLARE_EVENT_TABLE()
};

/// Example of a Custom Event --------------------------------------------
class myFindResultsEvent : public wxCommandEvent {
    public:
        myFindResultsEvent( wxEventType commandType = 0, int id=0 );
        myFindResultsEvent( const myFindResultsEvent& event ); //! Copy constructor
        
        void SetX( int val )    { m_x = val; }
        void SetY( int val )    { m_y = val; }
        void SetZ( int val )    { m_z = val; }
        //void SetLength( int val )    { m_length = val; }
        //void SetCount( int val )    { m_count = val; }
        int GetX() const        { return m_x; }
        int GetY() const        { return m_y; }
        int GetZ() const        { return m_z; } 
        //int GetCount() const        { return m_count; } //! # of occurences
        
        virtual wxEvent* Clone() const { return new myFindResultsEvent(*this); }
    private:
        DECLARE_DYNAMIC_CLASS(myFindResultsEvent)
        
        int m_x, m_y, m_z; /// line, start, end
};

BEGIN_DECLARE_EVENT_TYPES()
    DECLARE_EXPORTED_EVENT_TYPE( , wxEVT_LOCALFINDALLTWEAK,             23258)
    DECLARE_EXPORTED_EVENT_TYPE( , wxEVT_RESULTS_CLICKED,             23259)
    DECLARE_EXPORTED_EVENT_TYPE( , wxEVT_RESULTS_CLOSING,             23260)
END_DECLARE_EVENT_TYPES()

typedef void (wxEvtHandler::*myFindResultsEventFunction)(myFindResultsEvent&);

#define EVT_LOCALFINDALLTWEAK(id, fn)  DECLARE_EVENT_TABLE_ENTRY( wxEVT_LOCALFINDALLTWEAK,  id, wxID_ANY, (wxObjectEventFunction) (wxEventFunction)  wxStaticCastEvent( myFindResultsEventFunction, & fn ), (wxObject *) NULL ),
#define EVT_RESULTS_CLICKED(id, fn)  DECLARE_EVENT_TABLE_ENTRY( wxEVT_RESULTS_CLICKED,  id, wxID_ANY, (wxObjectEventFunction) (wxEventFunction)  wxStaticCastEvent( myFindResultsEventFunction, & fn ), (wxObject *) NULL ),
#define EVT_RESULTS_CLOSING(id, fn)  DECLARE_EVENT_TABLE_ENTRY( wxEVT_RESULTS_CLOSING,  id, wxID_ANY, (wxObjectEventFunction) (wxEventFunction)  wxStaticCastEvent( myFindResultsEventFunction, & fn ), (wxObject *) NULL ),
//! ------------------------------------------------------------------------------------------------END FIND RESULTS


class FindReplDialog: public wxDialog
{
  public:
    FindReplDialog(wxWindow *parent, wxPoint pos, int parentWidth, FindReplData * frData, FindReplData * frData2);
    ~FindReplDialog();
    void RedrawFRCombo(bool find); /// true for find, false for replace
    
  private:
    wxWindow * m_parent;
    wxMenu * m_popup;
    wxButton * m_clearButton;
    wxButton * m_synchButton;
    //void OnFindChange( wxCommandEvent &event ); ///Notifies when there has been a change in the find conditions, so force a fresh find
    void OnSynch( wxCommandEvent &event );  //! Equiv of Cancel
    void OnDone( wxCommandEvent &event );  //! Equiv of Cancel
    void OnFind( wxCommandEvent &event );
    void OnReplace( wxCommandEvent &event );
    void OnReplaceAll( wxCommandEvent &event );
    void OnClear( wxCommandEvent &event );
    
    void OnFindComboChange( wxCommandEvent &event );
    void OnReplComboChange( wxCommandEvent &event );
    void OnRTFind( wxCommandEvent &event );
    void OnFindListClick( wxCommandEvent & event );
    void DoSelection( wxString & str );
    bool m_rtFind;  ///Are we in realtime find mode? Then do RT Search
    bool m_rtRepl;  ///Just tells us if we have a typed entry in the replace field
    wxButton * m_doneButton;
    wxButton * m_findButton;
    wxButton * m_replButton;
    wxButton * m_replAllButton;
    wxComboBox * m_findCombo;
    wxTextCtrl * m_findText;
    wxListBox * m_list;  ///RealTime find list
    wxComboBox * m_replCombo;
    wxTextCtrl * m_replText;
    wxCheckBox * m_wholeWord, * m_case, * m_regex, * m_preserve;
    wxFlexGridSizer * m_findReplSizer;
    wxFlexGridSizer * m_findReplSizer2;
    wxBoxSizer * m_mainSizer;  ///This is a renaming of topSizer
    wxStaticText * m_static;  //! DIAG
    wxString m_diagStr; //! DIAG
    wxString m_lastFindStr;
    FindReplData * m_frData;
    FindReplData * m_frData2;
    int m_lastFlags;
  DECLARE_EVENT_TABLE()
};

//! Example of a Custom Event -----------------------------------------------------------------
class FindReplEvent : public wxCommandEvent {
    public:
        FindReplEvent( wxEventType commandType = 0, int id=0 );
        FindReplEvent( const FindReplEvent& event ); //! Copy constructor //remove "const" from arg
        virtual wxEvent* Clone() const { return new FindReplEvent(*this); }
        //! Having compiler issues with "const". Opted to pass data via the FindReplData object of the edit class - turns out this was the better option
/*         void SetFindString( wxString val ) { m_findString = val; }
        void SetReplString( wxString val ) { m_replString = val; }
        void SetFlags(int val) { m_flags = val; }
        wxString GetFindString() { return m_findString; }
        wxString GetReplString() { return m_replString; }
        int GetFlags() { return m_flags;}
        int m_flags;
        wxString m_findString, m_replString; */
    private:
        DECLARE_DYNAMIC_CLASS(FindReplEvent)
};

BEGIN_DECLARE_EVENT_TYPES()  //! WARNING using same events names as standard FR Dialog ///Since we are not including that header file this should not be a problem
    DECLARE_EXPORTED_EVENT_TYPE( , wxEVT_FIND,             23270)
    DECLARE_EXPORTED_EVENT_TYPE( , wxEVT_FIND_NEXT,             23271)
    DECLARE_EXPORTED_EVENT_TYPE( , wxEVT_FIND_REPLACE,             23272)
    DECLARE_EXPORTED_EVENT_TYPE( , wxEVT_FIND_REPLACE_ALL,             23273)
    DECLARE_EXPORTED_EVENT_TYPE( , wxEVT_FIND_CLOSE,             23274)
END_DECLARE_EVENT_TYPES()

typedef void (wxEvtHandler::*FindReplEventFunction)(FindReplEvent&);
///The fun part ///Create the event type?
#define EVT_FIND(id, fn)  DECLARE_EVENT_TABLE_ENTRY( wxEVT_FIND,  id, wxID_ANY, (wxObjectEventFunction) (wxEventFunction)  wxStaticCastEvent( FindReplEventFunction, & fn ), (wxObject *) NULL ),
#define EVT_FIND_NEXT(id, fn)  DECLARE_EVENT_TABLE_ENTRY( wxEVT_FIND_NEXT,  id, wxID_ANY, (wxObjectEventFunction) (wxEventFunction)  wxStaticCastEvent( FindReplEventFunction, & fn ), (wxObject *) NULL ),
#define EVT_FIND_REPLACE(id, fn)  DECLARE_EVENT_TABLE_ENTRY( wxEVT_FIND_REPLACE,  id, wxID_ANY, (wxObjectEventFunction) (wxEventFunction)  wxStaticCastEvent( FindReplEventFunction, & fn ), (wxObject *) NULL ),
#define EVT_FIND_REPLACE_ALL(id, fn)  DECLARE_EVENT_TABLE_ENTRY( wxEVT_FIND_REPLACE_ALL,  id, wxID_ANY, (wxObjectEventFunction) (wxEventFunction)  wxStaticCastEvent( FindReplEventFunction, & fn ), (wxObject *) NULL ),
#define EVT_FIND_CLOSE(id, fn)  DECLARE_EVENT_TABLE_ENTRY( wxEVT_FIND_CLOSE,  id, wxID_ANY, (wxObjectEventFunction) (wxEventFunction)  wxStaticCastEvent( FindReplEventFunction, & fn ), (wxObject *) NULL ),
//! ---------------------------------------------------------------------------------------------------------

#endif // FINDREPLDIALOG_H