// AUTHOR: Rohan Allison
// COPYRIGHT: 2006
//  License: Artistic
// Filename: ModeOptsPanel.cpp

#include <wx/wx.h>
#include <wx/arrstr.h>
#include <wx/spinctrl.h>
#include <wx/statline.h>
#include <wx/colordlg.h>
#include <wx/notebook.h>
#include <wx/filepicker.h>
#include <wx/html/htmlwin.h>
// #include <wx/statline.h>

#include <wx/print.h>
// #include <wx/fdrepdlg.h>
#include <stc.h>

#include "globaldefs.h"
#include "syntax.h"
#include "FindReplDialog.h"
#include "ZionDialogs.h"
#include "zionedit.h"
#include "SettingsDialog.h"

#include "ModeOptsPanel.h"

TMO tempModeOpts;  ///Define hash map of pointers to TempModeOpts

TempModeOpts::TempModeOpts( smallSyntax * aSyntax )
{
    m_syntax = aSyntax;
}

TempModeOpts::~TempModeOpts() { }

void TempModeOpts::Apply()
{   
    m_syntax->option.line = m_line;
    m_syntax->option.fold = m_fold;
    m_syntax->option.wrap = m_wrap;
    m_syntax->option.indent = m_indent;
    m_syntax->option.usetabs = m_usetabs;
    m_syntax->option.viewws = m_viewws;
    m_syntax->option.horizSb = m_horizSb;
    m_syntax->option.vertSb = m_vertSb;
    
    long tab; m_tabWidth.ToLong(&tab); /// In TMOS, tabWidth is a string
    m_syntax->tabWidth = tab;
    m_syntax->pageColor = m_pageColor;
    m_syntax->lineForeColor = m_lineForeColor;
    m_syntax->lineBackColor = m_lineBackColor;
    m_syntax->selColor = m_selColor;
    m_syntax->foldColor = m_foldColor;
    m_syntax->indentColor = m_indentColor;
}

//------------------------------------------------------------------------------------------------MODE OPTIONS PANEL
ModeOptsPanel::ModeOptsPanel( wxWindow * parent, smallSyntax * aSyntax, wxPoint pos, int parentWidth ) ///Widgets are loaded later in MOPLoadWidgets()
    : wxPanel( parent, ZION_MODE_OPTS )
{
    m_syntax = aSyntax;  //! save ptr to syntax struct
    m_pos = pos;  ///Parent pos //Multi-monitor
    m_parentWidth = parentWidth;
    m_textMode =  (aSyntax == &defaultSyntax);
    //strcmp( aSyntax->name, defaultSyntax.name ) == 0; ///logic must be inverted for strcmp [0 is equal]
    wxString str( m_textMode? _T("Plain Text") : m_syntax->name );
    ///Init our page button pointers just in case (we are using a lot of conditional initializations below)
    m_pageColorBut = (wxButton*) NULL; m_textPageButton = (wxButton*) NULL;
    m_lineStyleButton = (wxButton*) NULL;
    //m_lineStyleButton = (wxButton*) NULL; m_lineBackButton = (wxButton*) NULL;
    m_foldButton = (wxButton*) NULL; m_selButton = (wxButton*) NULL;
    
     wxBoxSizer *topSizer = new wxBoxSizer( wxVERTICAL );
    ///Checked options
    topSizer->AddSpacer( 18 ); //new wxStaticText( this, wxID_ANY, _T("  ") ), 0, wxALIGN_CENTER_HORIZONTAL| wxALIGN_TOP |wxALL, 5 );
    wxBoxSizer *checkSizer = new wxBoxSizer( wxHORIZONTAL );
    wxBoxSizer *checkSizer2 = new wxBoxSizer( wxVERTICAL );
    wxBoxSizer *checkSizer3 = new wxBoxSizer( wxVERTICAL );
    m_line = new wxCheckBox(this, ZION_OPTS_LINE, _T("Show line numbers"));
    m_fold = new wxCheckBox(this, ZION_OPTS_FOLD, _T("Show fold margin"));
    m_wrap = new wxCheckBox(this, ZION_OPTS_WRAP, _T("Wrap lines"));
    m_indent = new wxCheckBox(this, ZION_OPTS_INDENT, _T("Show Indentation Guides"));
    m_usetabs = new wxCheckBox(this, ZION_OPTS_USETABS, _T("Use Real Tabs"));
    m_viewws = new wxCheckBox(this, ZION_OPTS_VIEWWS, _T("View Whitespace"));
    m_horizSb = new wxCheckBox(this, ZION_OPTS_HORIZSB, _T("Show Horiz. Scrollbar"));
    m_vertSb = new wxCheckBox(this, ZION_OPTS_VERTSB, _T("Show Vertical Scrollbar"));
    if( m_textMode ) {
        m_fold->Enable(false); ///We would never fold plain text
        m_textPageButton = new wxButton( this, ZION_STYLE_PAGE, _T("TEXT"), wxDefaultPosition,
                wxDefaultSize, wxNO_BORDER );
        m_textPageButton->SetBackgroundStyle( wxBG_STYLE_COLOUR );
    }
    else {
        m_pageColorBut = new wxButton( this, ZION_OPTS_PAGECOLOR, _T("") );
        m_pageColorBut->SetBackgroundStyle( wxBG_STYLE_COLOUR );
    }
    wxString widths[] = { _T("2"), _T("3"), _T("4"), _T("5"), _T("6"), _T("7"),
            _T("8"), _T("9"), _T("10"), _T("11"), _T("12"), _T("13"), _T("14") };
    m_tabWidth = new wxComboBox( this, ZION_TABWIDTH, _T(""), wxDefaultPosition, ///Will init later in MOPLoadWidgets
            wxDefaultSize, 13, widths, wxCB_READONLY );
    checkSizer2->Add( m_line, 0, wxALIGN_LEFT|wxALL, 2 );
    checkSizer2->Add( m_fold, 0, wxALIGN_LEFT|wxALL, 2 );
    checkSizer2->Add( m_indent, 0, wxALIGN_LEFT|wxALL, 2 );
    checkSizer2->Add( m_horizSb, 0, wxALIGN_LEFT|wxALL, 2 );
    checkSizer->Add( checkSizer2, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 4);
    checkSizer3->Add( m_wrap, 0, wxALIGN_LEFT|wxALL, 2 );
    checkSizer3->Add( m_usetabs, 0, wxALIGN_LEFT|wxALL, 2 );
    checkSizer3->Add( m_viewws, 0, wxALIGN_LEFT|wxALL, 2 );
    checkSizer3->Add( m_vertSb, 0, wxALIGN_LEFT|wxALL, 2 );
    checkSizer->Add( checkSizer3, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 4);
    topSizer->Add( checkSizer, 0, wxALIGN_CENTER_VERTICAL | wxALIGN_CENTER_HORIZONTAL| wxALL, 3);
    
    topSizer->Add( new wxStaticText( this, wxID_ANY, wxString(aSyntax->callTipFile).Prepend(_T("Top-level Call Tip file:  ") ) ), 0, wxALIGN_CENTER_VERTICAL | wxALIGN_CENTER_HORIZONTAL| wxALL, 2 );
    
    ///Line, Fold, Selection Colors
    wxBoxSizer * mdefL = new wxBoxSizer( wxVERTICAL ); ///mode default left
    wxBoxSizer * mdefR = new wxBoxSizer( wxVERTICAL );
    wxBoxSizer *pageSizer = new wxBoxSizer( wxHORIZONTAL );
    wxBoxSizer *lineForeSizer = new wxBoxSizer( wxHORIZONTAL );
    wxBoxSizer *tabSizer = new wxBoxSizer( wxHORIZONTAL );
    wxBoxSizer *foldSizer = new wxBoxSizer( wxHORIZONTAL );
    wxBoxSizer *selSizer = new wxBoxSizer( wxHORIZONTAL );
     wxBoxSizer * colors = new wxBoxSizer( wxHORIZONTAL );
   
    if( ! m_textMode ) { ///Text mode will use page style instead
        pageSizer->Add( new wxStaticText( this, wxID_ANY, _T("Page Color") ), 0, wxALIGN_CENTER_VERTICAL|wxALL, 2);
        pageSizer->Add( m_pageColorBut, 0, wxALIGN_CENTER_VERTICAL|wxALL, 2);
    }
    else {
        pageSizer->Add( new wxStaticText( this, wxID_ANY, _T("Page Style") ), 0, wxALIGN_CENTER_VERTICAL|wxALL, 2);
        pageSizer->Add( m_textPageButton, 0, wxALIGN_CENTER_VERTICAL|wxALL, 2);
    }
    mdefL->Add( pageSizer, 0,  wxALIGN_CENTER_VERTICAL |wxALIGN_RIGHT|wxALL, 3);
    
    m_lineStyleButton = new wxButton( this, ZION_LINEFORE, _T("2999") );
    m_lineStyleButton->SetBackgroundStyle( wxBG_STYLE_COLOUR );
    m_indentButton = new wxButton( this, ZION_STYLE_INDENT, _T("") );
    m_indentButton->SetBackgroundStyle( wxBG_STYLE_COLOUR );
    m_foldButton = new wxButton( this, ZION_FOLDCOLOR, _T("") );
    m_foldButton->SetBackgroundStyle( wxBG_STYLE_COLOUR );
    m_selButton = new wxButton( this, ZION_SELCOLOR, _T("") );
    m_selButton->SetBackgroundStyle( wxBG_STYLE_COLOUR );
    
    lineForeSizer->Add( new wxStaticText( this, wxID_ANY, _T("Line# Style") ), 0, wxALIGN_CENTER_VERTICAL|wxALL, 2);
    lineForeSizer->Add( m_lineStyleButton, 0, wxALIGN_CENTER_VERTICAL|wxALL, 2);
    mdefL->Add( lineForeSizer, 0,  wxALIGN_CENTER_VERTICAL |wxALIGN_RIGHT|wxALL, 3);

    foldSizer->Add( new wxStaticText( this, wxID_ANY, _T("Fold Margin") ), 0, wxALIGN_CENTER_VERTICAL|wxALL, 2);
    foldSizer->Add( m_foldButton, 0, wxALIGN_CENTER_VERTICAL|wxALL, 2);
    mdefL->Add( foldSizer, 0,  wxALIGN_CENTER_VERTICAL |wxALIGN_RIGHT|wxALL, 3);

    tabSizer->Add( new wxStaticText( this, wxID_ANY, _T("Tab Width") ), 0, wxALIGN_CENTER_VERTICAL|wxALL, 2);
    tabSizer->Add( m_tabWidth, 0, wxALIGN_CENTER_VERTICAL|wxALL, 2);
    mdefR->Add( tabSizer, 0,  wxALIGN_CENTER_VERTICAL |wxALIGN_RIGHT|wxALL, 3);
    
    wxBoxSizer *indentSizer = new wxBoxSizer( wxHORIZONTAL );
    indentSizer->Add( new wxStaticText( this, wxID_ANY, _T("Indent Guides") ), 0, wxALIGN_CENTER_VERTICAL|wxALL, 2);
    indentSizer->Add( m_indentButton, 0, wxALIGN_CENTER_VERTICAL|wxALL, 2);
    mdefR->Add( indentSizer, 0,  wxALIGN_CENTER_VERTICAL |wxALIGN_RIGHT|wxALL, 3);
    
    selSizer->Add( new wxStaticText( this, wxID_ANY, _T("Selection Color") ), 0, wxALIGN_CENTER_VERTICAL|wxALL, 2);
    selSizer->Add( m_selButton, 0, wxALIGN_CENTER_VERTICAL|wxALL, 2);
    mdefR->Add( selSizer, 0,  wxALIGN_CENTER_VERTICAL |wxALIGN_RIGHT|wxALL, 3);
    
    colors->Add( mdefL, 0, wxALIGN_CENTER_HORIZONTAL | wxALIGN_TOP | wxALL, 3 );
    colors->Add( mdefR, 0, wxALIGN_CENTER_HORIZONTAL | wxALIGN_TOP | wxALL, 3 );

    topSizer->Add( colors, 0,  wxALIGN_CENTER_VERTICAL |wxALIGN_CENTER_HORIZONTAL|wxALL, 3);
    
    /// Set All backgrounds
    /** Compiler  (Digital Mars up to version 8.49) exponentially increases it's compile time if the "if( ! m_textmode )" is applied to this whole logical block
     (As it should be if you are optimizing)
     Compiler seems to do better when similar items are created /operated on consecutively **/
    str.Printf(_T("Set Backgrounds for All %s Styles"), m_syntax->name );
    m_otherColorBut = new wxButton(this, ZION_OPTS_OTHERCOLOR, "Other Color");
    m_otherColorBut->SetBackgroundStyle( wxBG_STYLE_COLOUR );
    wxButton * setBacks = new wxButton(this, ZION_OPTS_SETBACK, _T("Set Backgrounds") );
    m_rb1 = new wxRadioButton( this, ZION_OPTS_RADIOPAGE, _T("Use Page Color"), wxDefaultPosition,
            wxDefaultSize, wxRB_GROUP);  ///Start a new RadioBox group
    m_rb2 = new wxRadioButton( this, ZION_OPTS_RADIOOTHER, _T("Use Other Color") );
    if( m_textMode ) {
        m_otherColorBut->Disable();
        setBacks->Disable();
        str = _T("");
        m_rb1 ->Disable();  m_rb2->Disable();
    }
    wxStaticBoxSizer * stbx = new wxStaticBoxSizer(wxVERTICAL, this, str);
    wxBoxSizer * rbx = new wxBoxSizer( wxHORIZONTAL );
 
    rbx->Add( m_rb1, 0, wxALIGN_CENTER_VERTICAL|wxALL, 4);
    rbx->Add( m_rb2, 0, wxALIGN_CENTER_VERTICAL|wxALL, 2);  ///will add its button beside it
    rbx->Add( m_otherColorBut, 0, wxALIGN_CENTER_VERTICAL|wxALL, 2);
    
    stbx->Add( rbx, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 4);
    stbx->Add( setBacks, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 4  );
            
    topSizer->Add( stbx, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 3);
    topSizer->AddSpacer( 18 ); //topSizer->Add( new wxStaticLine(this), 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 3);
    
    MOPLoadWidgets();

    SetSizer(topSizer);  //use the sizer for layout
    topSizer->SetSizeHints(this);  //! Coordinate minimal window size
}

ModeOptsPanel::~ModeOptsPanel() { }

BEGIN_EVENT_TABLE(ModeOptsPanel, wxPanel)
    EVT_CHECKBOX( ZION_OPTS_LINE, ModeOptsPanel::OnCheck )
    EVT_CHECKBOX( ZION_OPTS_FOLD, ModeOptsPanel::OnCheck )
    EVT_CHECKBOX( ZION_OPTS_WRAP, ModeOptsPanel::OnCheck )
    EVT_CHECKBOX( ZION_OPTS_INDENT, ModeOptsPanel::OnCheck )
    EVT_CHECKBOX( ZION_OPTS_USETABS, ModeOptsPanel::OnCheck )
    EVT_CHECKBOX( ZION_OPTS_VIEWWS, ModeOptsPanel::OnCheck )
    EVT_CHECKBOX( ZION_OPTS_HORIZSB, ModeOptsPanel::OnCheck )
    EVT_CHECKBOX( ZION_OPTS_VERTSB, ModeOptsPanel::OnCheck )
    EVT_COMBOBOX( ZION_TABWIDTH, ModeOptsPanel::OnCheck )
    EVT_BUTTON( ZION_STYLE_PAGE,  ModeOptsPanel::OnPageStyle) ///Text mode only
    EVT_BUTTON( ZION_OPTS_OTHERCOLOR,  ModeOptsPanel::OnOther)
    EVT_BUTTON( ZION_OPTS_PAGECOLOR,  ModeOptsPanel::OnPage)
    EVT_BUTTON( ZION_OPTS_SETBACK,  ModeOptsPanel::OnSetBack)
    EVT_BUTTON( ZION_LINEFORE,  ModeOptsPanel::OnLineStyle)
    EVT_BUTTON( ZION_STYLE_INDENT,  ModeOptsPanel::OnIndent)
    EVT_BUTTON( ZION_FOLDCOLOR,  ModeOptsPanel::OnFold)
    EVT_BUTTON( ZION_SELCOLOR,  ModeOptsPanel::OnSel)
END_EVENT_TABLE()

void ModeOptsPanel::OnPageStyle(wxCommandEvent & event)
{
    StyleDialog stydia( this, &defaultSyntax, 0, m_pos, m_parentWidth );
    if( (stydia.ShowModal()) == wxID_OK ) {
        m_textPageButton->SetBackgroundColour( stydia.m_stylePanel->m_backColor );
        m_textPageButton->SetForegroundColour( stydia.m_stylePanel->m_foreColor );
    }
}

void ModeOptsPanel::OnOther(wxCommandEvent & event)
{
    m_otherColor = ChooseColor(m_otherColor);  /// Save the chosen color in m_otherColor before applying to the button
    m_otherColorBut->SetBackgroundColour( m_otherColor );
    m_rb1->SetValue(false); m_rb2->SetValue(true);  ///User might want this after changing "Other Color"
}

void ModeOptsPanel::OnIndent(wxCommandEvent & event)
{
    m_indentButton->SetBackgroundColour( ChooseColor(m_indentButton->GetBackgroundColour()) );  /// Init with button color
    SaveState();
}

void ModeOptsPanel::OnPage(wxCommandEvent & event)
{
    m_pageColor = ChooseColor(m_pageColor);
    m_pageColorBut->SetBackgroundColour( m_pageColor );
    SaveState();  /// Update a TempModeOpts object
}

void ModeOptsPanel::OnLineStyle(wxCommandEvent & event)
{
    StyleDialog stydia( this, &defaultSyntax, 1, m_pos, m_parentWidth ); /// 1 = Index to line number style
    ///Override color inits of style dialog
    stydia.m_stylePanel->m_foreButton->SetBackgroundColour( m_lineStyleButton->GetForegroundColour() );
    stydia.m_stylePanel->m_foreColor = m_lineStyleButton->GetForegroundColour();
    stydia.m_stylePanel->m_backButton->SetBackgroundColour( m_lineStyleButton->GetBackgroundColour() );
    stydia.m_stylePanel->m_backColor = m_lineStyleButton->GetBackgroundColour();
    if( (stydia.ShowModal()) == wxID_OK ) {
        m_lineStyleButton->SetForegroundColour( stydia.m_stylePanel->m_foreColor );
        m_lineStyleButton->SetBackgroundColour( stydia.m_stylePanel->m_backColor );
        SaveState();
    }
}
//
//void ModeOptsPanel::OnLineFore(wxCommandEvent & event)
//{
//    m_lineStyleButton->SetBackgroundColour( ChooseColor(m_lineStyleButton->GetBackgroundColour() ) );
//    SaveState();  /// Update a TempModeOpts object
//}

void ModeOptsPanel::OnFold(wxCommandEvent & event)
{
    m_foldButton->SetBackgroundColour( ChooseColor(m_foldButton->GetBackgroundColour() ) );
    SaveState();  /// Update a TempModeOpts object
}

void ModeOptsPanel::OnSel(wxCommandEvent & event)
{
    m_selButton->SetBackgroundColour( ChooseColor(m_selButton->GetBackgroundColour() ) );
    SaveState();  /// Update a TempModeOpts object
}

wxColour ModeOptsPanel::ChooseColor( wxColour & aColor)
{
    wxColourData data; int i;
    data.SetChooseFull(true);
    for (i = 0; i < 16; i++)  data.SetCustomColour(i, customColors[i]); ///load from our array

    data.SetColour(aColor);

    wxColourDialog dialog(this, &data);
    if (dialog.ShowModal() == wxID_OK)
    {
        wxColourData retData = dialog.GetColourData();
        for( i=0; i < 16; i++)  customColors[i] = retData.GetCustomColour(i);
        return retData.GetColour();
    }
    return( wxColour(aColor) );
}
 
void ModeOptsPanel::OnSetBack(wxCommandEvent & event)
{
    wxString str2; str2.Printf(_T("Background changes apply immediately to all %s styles\n\
except for those already changed in this dialog.\nContinue?"),
            m_syntax->name );
    if( wxMessageBox( str2, _T("Warning"), wxICON_EXCLAMATION|wxYES_NO ) != wxYES ) return; //! Yes to continue, anything else aborts

///m_pageColorBut is already set to the proper color.
    //! Ascertain color to set backgrounds with and set m_pageColorBut accordingly
    wxColour useColor; 
    if(m_rb1->GetValue()) useColor = m_pageColor;
    else if(m_rb2->GetValue()) useColor = m_otherColor;
    else return;
    
    for( int i=0; i < m_syntax->stylesUsed; i++ )  {  //! Scan all styles
        str2.Printf("%02x%02x%02x", useColor.Red(), useColor.Green(), useColor.Blue() );
        strncpy( m_syntax->styles[i].back, str2.c_str(), 6 );
    }
}
void ModeOptsPanel::MOPLoadWidgets( )
/// Some colors are stored in the buttons themselves.
{
    TempModeOpts * tmos = NULL;
    //! See if it already exists
    TMO::iterator it;  bool alreadyExist = false;
    for( it = tempModeOpts.begin(); it != tempModeOpts.end(); ++it ) {
        if( it->first == m_syntax->name )  { alreadyExist = true; tmos = it->second;  break;  }
    }
    if(alreadyExist) { /// Init via our tempStyle
        m_line->SetValue(tmos->m_line);
        m_fold->SetValue(tmos->m_fold);
        m_indent->SetValue(tmos->m_indent);
        m_wrap->SetValue(tmos->m_wrap);
        m_usetabs->SetValue(tmos->m_usetabs);
        m_viewws->SetValue(tmos->m_viewws);
        m_horizSb->SetValue(tmos->m_horizSb);
        m_vertSb->SetValue(tmos->m_vertSb);
        m_tabWidth->SetValue(tmos->m_tabWidth);
        m_pageColor = tmos->m_pageColor;
        m_lineStyleButton->SetForegroundColour( tmos->m_lineForeColor );
        m_lineStyleButton->SetBackgroundColour( tmos->m_lineBackColor );
        m_foldButton->SetBackgroundColour( tmos->m_foldColor );
        m_selButton->SetBackgroundColour( tmos->m_selColor );
        m_indentButton->SetBackgroundColour( tmos->m_indentColor );
    }
    else {  /// Init via syntax struct   //! ...the first time a change is made
        m_line->SetValue( m_syntax->option.line );
        m_fold->SetValue( m_syntax->option.fold );
        m_wrap->SetValue( m_syntax->option.wrap );
        m_indent->SetValue( m_syntax->option.indent );
        m_usetabs->SetValue( m_syntax->option.usetabs );
        m_viewws->SetValue( m_syntax->option.viewws );
        m_horizSb->SetValue( m_syntax->option.horizSb );
        m_vertSb->SetValue( m_syntax->option.vertSb );
        wxString init; init.Printf(_T("%d"), m_syntax->tabWidth); ///cast to int // long doesn't work here--no match
        m_tabWidth->SetValue(init);
        m_pageColor = m_syntax->pageColor;
        m_lineStyleButton->SetForegroundColour( m_syntax->lineForeColor );
        m_lineStyleButton->SetBackgroundColour( m_syntax->lineBackColor );
        m_foldButton->SetBackgroundColour( m_syntax->foldColor );
        m_selButton->SetBackgroundColour( m_syntax->selColor );
        m_indentButton->SetBackgroundColour( m_syntax->indentColor );
    }
    ///If Wrap Lines Disable horiz scroll
    if( m_wrap->GetValue() ) m_horizSb->Enable(false);
    ///TextPageButton is treated separately. We will "store" the color in the button. i.e. we won't use a separate wxColour
    if( m_textMode ) {
        TempStyle * tsptr = (TempStyle *) NULL;
        if( (tsptr = CheckForTempStyle( &defaultSyntax, 0 )) != NULL) {
            m_textPageButton->SetBackgroundColour( StringToColor( tsptr->m_back ) );  //! StringToColor can take a wxString or a c_string
            m_textPageButton->SetForegroundColour( StringToColor( tsptr->m_fore ) );  //! StringToColor can take a wxString or a c_string
        }
        else {
            m_textPageButton->SetBackgroundColour( StringToColor(defaultSyntax.styles[0].back) );
            m_textPageButton->SetForegroundColour( StringToColor(defaultSyntax.styles[0].fore) );
        }
    }
    else m_pageColorBut->SetBackgroundColour(m_pageColor);  ///Must use an else here because m_pageColorBut is not initialized to a button in text mode
}

void ModeOptsPanel::SaveState()
{
    TempModeOpts * tmos;
    //! See if it already exists
    TMO::iterator it;  bool alreadyExist = false;
    for( it = tempModeOpts.begin(); it != tempModeOpts.end(); ++it ) {
        if( it->first == m_syntax->name )  { alreadyExist = true; tmos = it->second; break;  }
    }
    if(! alreadyExist)  {
        tmos = new TempModeOpts( m_syntax ); //! The new TempModeOpts will register itself with the hash map
        tempModeOpts[m_syntax->name] = tmos;  //! Register the pointer  into the hashmap
    }
    ///Init or update our tempStyle
    tmos->m_line = m_line->IsChecked();
    tmos->m_fold = m_fold->IsChecked();
    tmos->m_wrap = m_wrap->IsChecked();
    tmos->m_indent = m_indent->IsChecked();
    tmos->m_usetabs = m_usetabs->IsChecked();
    tmos->m_viewws = m_viewws->IsChecked();
    tmos->m_horizSb = m_horizSb->IsChecked();
    tmos->m_vertSb = m_vertSb->IsChecked();
    tmos->m_tabWidth = m_tabWidth->GetValue();
    tmos->m_pageColor = m_pageColor;
    tmos->m_lineForeColor = m_lineStyleButton->GetForegroundColour();
    tmos->m_lineBackColor = m_lineStyleButton->GetBackgroundColour();
    tmos->m_foldColor = m_foldButton->GetBackgroundColour();
    tmos->m_selColor = m_selButton->GetBackgroundColour();
    tmos->m_indentColor = m_indentButton->GetBackgroundColour();
}

void ModeOptsPanel::OnCheck(wxCommandEvent &event) 
{
    m_horizSb->Enable( ! m_wrap->GetValue() );
    SaveState();
}
