#ifndef SETTINGS_DIALOG_h
#define SETTINGS_DIALOG_h

extern wxColour StringToColor(const wxString & );

class TempStyle;
WX_DECLARE_STRING_HASH_MAP( TempStyle*, TempStyles );
class TempModeOpts;
WX_DECLARE_STRING_HASH_MAP( TempModeOpts *, TMO );

extern TMO tempModeOpts;
extern TempStyles tempStyles;

class TempStyle
{
    public:
        TempStyle( smallSyntax * aSyntax, int stylenum );
        ~TempStyle();
        void Apply();
        
        bool m_bold, m_italic;
        wxString m_fore, m_back, m_face, m_size;
        int m_stylenum;
        smallSyntax * m_syntax;
    private:
};

TempStyle * CheckForTempStyle( smallSyntax * aSyntax, int stylenum );

class ZionEdit;
class GeneralOptsPanel: public wxPanel
{
    friend class ZionEdit;
    public:
        GeneralOptsPanel(wxWindow * parent, ZionEdit * zedit); //wxStyledTextCtrl *
        ~GeneralOptsPanel();
        void Apply();  //! Since there will be only one general opts, we can apply here
        wxColour m_caretColor;  //m_braceHiliteColor, m_braceBadColor;
        wxComboBox * m_eolMode;
        wxComboBox * m_startLex;
        wxComboBox * m_pcolMode;
        wxComboBox * m_recentFiles;
        wxComboBox * m_recentDirs;
        wxComboBox * m_zoom;
        wxComboBox * m_numACChars;
        wxComboBox * m_lineNumWidth;
        wxCheckBox * m_acCheckBox;
        wxCheckBox * m_ctCheckBox;
        wxCheckBox * m_useCurrent;
        wxCheckBox * m_autoBook;
        wxCheckBox * m_autoZoom;
        
        ZionEdit * m_zedit;
        wxCheckBox * m_dontPrintNums;
        wxTextCtrl * m_pathToBrowser;
        wxButton * m_browse;
        wxTextCtrl * m_pathToApi;
        wxButton * m_apiUseDef;
        wxButton * m_browseApi;
        //wxDirPickerCtrl * m_apiDirPicker;
        wxRadioButton * m_rbNorm;
        wxRadioButton * m_rbMax;
        wxRadioButton * m_rbCustom;
        wxTextCtrl * m_top;
        wxTextCtrl * m_left;
        wxTextCtrl * m_width;
        wxTextCtrl * m_height;
    
    private:
        wxButton *m_caretButton, *m_braceHiliteButton, * m_braceBadButton;
        bool m_eolComboChanged;
        
        void OnChooseColor(wxCommandEvent & event);
        void OnBrowserBrowse( wxCommandEvent & );
        void OnApiBrowse( wxCommandEvent & );
        void OnApiUseDef( wxCommandEvent & );
        void OnUseCurrent( wxCommandEvent & );
        void OnBraceHilite(wxCommandEvent & event);
        void OnBraceBad(wxCommandEvent & event);
        void OnEOLCombo(wxCommandEvent & event);
    DECLARE_EVENT_TABLE()
};


class StylePanel: public wxPanel    //! One for each language element
{
    public:
        StylePanel( wxWindow * parent, smallSyntax * aSyntax, int element );
        ~StylePanel();
        void SPLoadWidgets();
        void SaveState();
        void Abort();
        //void Redisplay();
        void OnChooseColour(wxCommandEvent & event);
        void OnUsePage( wxCommandEvent & event );
        void OnWidgetChange( wxCommandEvent & event );
        wxStaticText * m_staticText;
        wxButton * m_foreButton;
        wxButton * m_backButton;
        wxCheckBox * m_bold;
        wxCheckBox * m_italic;
        wxCheckBox * m_usePage;
        wxComboBox* m_fontCombo; //! face
        //wxSpinCtrl * m_spin;    //! size
        wxComboBox * m_spin;
        wxColour m_foreColor, m_backColor;
        wxButton * m_applyButton;
        wxButton * m_cancelButton;
        int m_stylenum;
        wxString m_index;
    private:
        int m_size;
        smallSyntax * m_syntax;
    DECLARE_EVENT_TABLE()
};

class StyleDialog: public wxDialog
{
    public:
        StyleDialog( wxWindow * parent, smallSyntax * aSyntax, int element,
            wxPoint pos, int parentWidth );
        StylePanel * m_stylePanel;
    private:
        void OnOk( wxCommandEvent & );
        void OnCancel( wxCommandEvent & );
        //We will have stylepanel erase a tempStyle on cancel of StyleDialog
    DECLARE_EVENT_TABLE()
};

class ModeOptsPanel;
class ModePanel: public  wxPanel   //! One for each language mode
{
    public:
        ModePanel( wxWindow * parent,  smallSyntax * aSyntax );
        ~ModePanel();
        void OnLBSelect( wxCommandEvent &event);
        wxTextCtrl * m_words2;
        wxCheckBox * m_showWords;
        wxBoxSizer * m_leftSizer;
    private:
        //SettingsDialog * m_diaPtr;
        //bool wordsChanged;
        void OnShowWords( wxCommandEvent &event );
        smallSyntax * m_Syntax;
        StylePanel * m_stylePanel;
        ModeOptsPanel * m_modeOptsPanel;
        wxBoxSizer * m_modeSizer;
    DECLARE_EVENT_TABLE()
};

class ExtPanel: public  wxPanel   //! One for each language mode
{
    public:
        ExtPanel( wxWindow * parent );
        ~ExtPanel();
        void OnLBSelect( wxCommandEvent &event);
        void OnComboSelect( wxCommandEvent &event);
        stringToInt m_e2lcopy; ///working copy of ext to lexer hashmap
    private:
        wxListBox * m_list;
        wxComboBox * m_combo;
        wxArrayString m_exts;
        wxStaticText * m_msg;
    wxBoxSizer * m_extSizer; 
    DECLARE_EVENT_TABLE()
};

class SettingsDialog: public wxDialog   //! The Big Boy
{
    public:
        SettingsDialog( wxWindow *parent, int stylenum, int currentLex, wxPoint pos, int parentWidth );
        ~SettingsDialog();
        bool m_pageDirty;
        wxNotebook * m_notebook;
        GeneralOptsPanel * m_generalOptsPanel;
        ExtPanel * m_extPanel;
        int m_stylenum;
        wxString m_cppWords2;
        wxString m_javaWords2;
        wxString m_sqlWords2;
        wxString m_csWords2;
        bool m_cppWordsChanged;
        bool m_javaWordsChanged;
        bool m_sqlWordsChanged;
        bool m_csWordsChanged;
    private:
        //void OnPageChanged( wxNotebookEvent & event );
        void OnOK( wxCommandEvent &event);
        void OnCancel( wxCommandEvent &event);
        wxButton * m_okButton;
        wxButton * m_doneButton;
        ZionEdit * m_parent;
    DECLARE_EVENT_TABLE()
};

 #endif