#include <wx/wx.h>
#include <wx/arrstr.h>
#include <wx/combobox.h>
#include <wx/spinctrl.h>
#include <wx/colordlg.h>
#include <wx/html/htmlwin.h>
#include "globaldefs.h"
#include "ZionDialogs.h"
//!Classes declared in a separate file are globally available

ExecDialog::ExecDialog( wxWindow *parent, bool append, wxArrayString & execHistory,
        const wxString & lastExecStr, bool captureOut, bool runWithCmd, wxArrayString * rdirs, const wxString & lastWdir )
	: wxDialog( parent, wxID_ANY, "Capture Output of Command",
        wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE ) //! wx Default Args required here
{
    m_rbNew = new wxRadioButton( this, wxID_ANY, _T("Capture in New Window"), wxDefaultPosition, wxDefaultSize, wxRB_GROUP);  ///Start a new RadioBox group
    m_rbAppend = new wxRadioButton( this, wxID_ANY, _T("Append to this Window") );
    m_chkCaptureOut = new wxCheckBox( this, ZION_CAPTUREOUT, _T("Capture Output") );
    if( append ) m_rbAppend->SetValue(true);  ///Remember previous setting
    if( ! captureOut ) { 
        m_rbAppend->Enable( false );  m_rbNew->Enable( false );
    }
    else m_chkCaptureOut->SetValue( true ); ///Remember previous setting
 
    m_rbCmd = new wxRadioButton( this, wxID_ANY, _T("Run via Cmd.exe"), wxDefaultPosition, wxDefaultSize, wxRB_GROUP );  ///Start a new RadioBox group 
    m_rbPath = new wxRadioButton( this, wxID_ANY, _T("Specific Path and filename") );
    if( runWithCmd ) m_rbCmd->SetValue(true);
    else m_rbPath->SetValue(true);
 
    m_execCombo = new wxComboBox( this, ZION_EXEC_COMBO, lastExecStr, wxDefaultPosition,
            wxSize(232, -1), execHistory, wxCB_SORT );
    m_execText = new wxTextCtrl( this, wxID_ANY, _T(""), wxDefaultPosition, wxSize(232, -1) ); ///This is to alleviate the combo bug/feature of auto-guessing your input
    
    
 //shouldn't need this //if( rdirs == NULL ) rdirs = new wxArrayString; ///blank for non-allFiles mode 
    m_wdir = new wxComboBox( this, wxID_ANY, lastWdir /*wxGetCwd() */ , wxDefaultPosition, wxSize(232, -1), * rdirs );
//    m_dirCombo = new wxComboBox( this, wxID_ANY, lastExecStr,
//            wxDefaultPosition, wxSize(232, -1), execHistory, wxCB_SORT );
//    long insPt = m_execCombo->GetInsertionPoint();// test   wxString addStr(_T("My Text") ); //    m_execCombo->SetValue( addStr );
   
    m_wdirBrowse = new wxButton( this, ZION_EXEC_DIRBROWSE, _T("Browse Folders") );
    m_browse = new wxButton( this, ZION_INI_BROWSE, _T("Browse Files") );
    m_runButton = new wxButton( this, wxID_OK, _T("Run") );

    ///Layout
    m_mainSizer = new wxBoxSizer( wxVERTICAL );

    ///Show codes
    m_mainSizer->Add( new wxStaticText( this, wxID_ANY,
        _T("The command line accepts the following replaceable parameters\n(Environment variables must be via Cmd.exe)") ),
        0, wxALIGN_CENTER_HORIZONTAL |wxALL, 4 );
    wxFlexGridSizer *paramsSizer = new wxFlexGridSizer( 2, 2, 3);  //! Cols, vgap, hgap
    paramsSizer->Add( new wxStaticText( this, wxID_ANY, _T("#P  Full Path with filename") ), 0, wxALIGN_LEFT |wxALL, 2 );
    paramsSizer->Add( new wxStaticText( this, wxID_ANY, _T("#f   Filename with extension") ), 0, wxALIGN_LEFT |wxALL, 2 );
    paramsSizer->Add( new wxStaticText( this, wxID_ANY, _T("#p  Path without filename") ), 0, wxALIGN_LEFT |wxALL, 2 );
    paramsSizer->Add( new wxStaticText( this, wxID_ANY, _T("#b  Filename without extension (basename)") ), 0, wxALIGN_LEFT |wxALL, 2 );
    paramsSizer->Add( new wxStaticText( this, wxID_ANY, _T("#Z  Full Path to zion.exe") ), 0, wxALIGN_LEFT |wxALL, 2 );
    paramsSizer->Add( new wxStaticText( this, wxID_ANY, _T("#D  ISO Date") ), 0, wxALIGN_LEFT |wxALL, 2 );
    paramsSizer->Add( new wxStaticText( this, wxID_ANY, _T("#z  Just the Path to zion.exe") ), 0, wxALIGN_LEFT |wxALL, 2 );
    paramsSizer->Add( new wxStaticText( this, wxID_ANY, _T("#T  ISO Time") ), 0, wxALIGN_LEFT |wxALL, 2 );
    paramsSizer->Add( new wxStaticText( this, wxID_ANY, _T("%Var%  The Environment var. \"Var\"") ), 0, wxALIGN_LEFT |wxALL, 2 );
    paramsSizer->Add( new wxStaticText( this, wxID_ANY, _T("##  The # character") ), 0, wxALIGN_LEFT |wxALL, 2 );
    m_mainSizer->Add( paramsSizer, 0, wxALIGN_CENTER_HORIZONTAL |wxALL, 4 );

    wxStaticBoxSizer *captureSizer = new wxStaticBoxSizer( wxVERTICAL, this );
    captureSizer->Add( m_rbNew, 0, wxALIGN_LEFT|wxALL, 3);
    captureSizer->Add( m_rbAppend, 0, wxALIGN_LEFT|wxALL, 3);

    wxStaticBoxSizer *pathOptSizer = new wxStaticBoxSizer( wxVERTICAL, this );
    pathOptSizer->Add( m_rbCmd, 0, wxALIGN_LEFT|wxALL, 3);
    pathOptSizer->Add( m_rbPath, 0, wxALIGN_LEFT|wxALL, 3);
    
    wxBoxSizer * upperSizer = new wxBoxSizer( wxHORIZONTAL );
    upperSizer->Add( captureSizer, 0, wxALIGN_LEFT|wxALL, 3);
    upperSizer->Add( pathOptSizer, 0, wxALIGN_LEFT|wxALL, 3);
    m_mainSizer->Add( upperSizer, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 4);
    
    m_mainSizer->Add( new wxStaticText( this, wxID_ANY, _T("Example: echo Editor's working directory is #p  cmd.exe location - %comspec%") ), 0, wxALIGN_CENTER_HORIZONTAL |wxALL, 2 );

    wxBoxSizer *pathSizer = new wxBoxSizer( wxHORIZONTAL );
    pathSizer->Add( new wxStaticText( this, wxID_ANY, _T("Command Line: ") ), 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 3 );
    pathSizer->Add( m_execCombo,  0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 3); 
    pathSizer->Add( m_execText,  0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 3);
    m_execText->Show(false); 
    pathSizer->Add( m_browse,  0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 3); 
    m_mainSizer->Add( pathSizer,  0, wxALIGN_CENTER_HORIZONTAL |wxALL, 4 );

    wxBoxSizer *dirSizer = new wxBoxSizer( wxHORIZONTAL );
    dirSizer->Add( new wxStaticText( this, wxID_ANY, _T("Working Dir: ") ), 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 3 );
    dirSizer->Add( m_wdir,  0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 3); 
    dirSizer->Add( m_wdirBrowse,  0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 3); 
    m_mainSizer->Add( dirSizer,  0, wxALIGN_CENTER_HORIZONTAL |wxALL, 4 );

    wxBoxSizer *buttonSizer = new wxBoxSizer( wxHORIZONTAL );
    buttonSizer->Add( m_chkCaptureOut,  0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
    buttonSizer->AddSpacer(14);
    buttonSizer->Add( m_runButton,  0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
    buttonSizer->AddSpacer(24);
    buttonSizer->Add( new wxButton(this, wxID_CANCEL, "Cancel"), 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);   //! wxID_CANCEL handled by the parent
    m_mainSizer->Add(buttonSizer,  0, wxALIGN_CENTER_HORIZONTAL |wxALL, 5);

    SetSizer(m_mainSizer);  //use the sizer for layout
    m_mainSizer->SetSizeHints(this);  //! Coordinate minimal window size
    m_runButton->SetDefault();
    ///Positioning
    int dlgW = GetSize().GetWidth();
    int parW = parent->GetSize().GetWidth();
    int edgeOff = 20;
    if( parW - dlgW > dlgW )  edgeOff = ( parW - dlgW ) >> 3;
    
    int x = parent->GetPosition().x + parW - dlgW - edgeOff;
    int y = parent->GetPosition().y + 24;
    SetSize( x, y, wxDefaultCoord, wxDefaultCoord );
}

ExecDialog::~ExecDialog() {}

BEGIN_EVENT_TABLE(ExecDialog, wxDialog)
    EVT_BUTTON( wxID_OK, ExecDialog::OnOK )
    EVT_BUTTON( ZION_INI_BROWSE, ExecDialog::OnChooseFile )
    EVT_BUTTON( ZION_EXEC_DIRBROWSE, ExecDialog::OnChooseDir )
    EVT_TEXT( ZION_EXEC_COMBO, ExecDialog::OnExecCombo )
    EVT_CHECKBOX( ZION_CAPTUREOUT, ExecDialog::OnCaptureOut )
END_EVENT_TABLE()

void ExecDialog::OnExecCombo( wxCommandEvent &event )
{
    m_execText->SetValue( m_execCombo->GetValue() );
    m_execCombo->Show(false);  m_execText->Show(true);
    m_execText->SetFocus();
    m_execText->SetInsertionPointEnd();
    m_mainSizer->Layout();
    m_mainSizer->SetSizeHints(this);  /// We have to do this here due to the possiibility of a previous RTFind
}

void ExecDialog::OnCaptureOut( wxCommandEvent &event ) 
{   ///If we are not capturing, these need to be disabled
    m_rbAppend->Enable( m_chkCaptureOut->GetValue() );
    m_rbNew->Enable( m_chkCaptureOut->GetValue() );
}

void ExecDialog::OnOK( wxCommandEvent &event )
{ 
    if( m_execText->IsShown() )  m_execCombo->SetValue( m_execText->GetValue() );  ///Combo will be used externally
    event.Skip();
}
	
void ExecDialog::OnChooseFile( wxCommandEvent & event )
{
    wxFileDialog * ofDialog = new wxFileDialog( this, "Open File", wxGetCwd(),
            "", _T("*.*"), wxOPEN|wxFILE_MUST_EXIST /*, wxDefaultPosition*/ );	//no default file
    if( ofDialog->ShowModal() == wxID_OK) {
        m_execText->IsShown() ?
            m_execText->SetValue( ofDialog->GetPath() ) :
            m_execCombo->SetValue( ofDialog->GetPath() ) ;  ///get full filespec from dialog
    }
}

void ExecDialog::OnChooseDir(wxCommandEvent & event)
{
	wxDirDialog d( this, "Choose a directory", m_wdir->GetValue(),	wxDD_NEW_DIR_BUTTON );
	if( d.ShowModal()== wxID_OK)	m_wdir->SetValue(d.GetPath()); //!Get chosen dir 
}

// ---------------------------------------------------------------------------------------------------------------------------------------
ZionIniDialog::ZionIniDialog( wxWindow *parent, const wxString& exeDir )
	: wxDialog( parent, wxID_ANY, "Locate zion.ini",
    wxDefaultPosition, wxSize(300, 200), wxDEFAULT_DIALOG_STYLE ) //! wx Default Args required here
{
    m_text = new wxTextCtrl(this, wxID_ANY, exeDir, wxDefaultPosition, wxSize(230, 22) );
    m_browse = new wxButton( this, ZION_INI_BROWSE, "Browse Folders" );

    wxBoxSizer *topSizer = new wxBoxSizer( wxVERTICAL );

    topSizer->Add( new wxStaticText(this, wxID_ANY, "Could not find \"zion.ini\" in\nZionEdit's startup folder"),
            0, wxALIGN_CENTER_HORIZONTAL|wxALL, 8);
    topSizer->Add( new wxStaticText(this, wxID_ANY, "  Look for it in:"),
            0, wxALIGN_LEFT|wxALL, 5);

    wxBoxSizer *pathSizer = new wxBoxSizer( wxHORIZONTAL );
    pathSizer->Add( m_text, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 3);
    pathSizer->Add( m_browse, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 3);
    topSizer->Add( pathSizer, 0, wxALIGN_LEFT|wxALL, 2);
    
    m_createIt = new wxCheckBox(this, wxID_ANY, "If not found, create it");
    m_createIt->SetValue(true);
    topSizer->Add( m_createIt, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 4);
    
    topSizer->Add( new wxStaticText(this, wxID_ANY, "To Avoid seeing this dialog again\nensure zion.ini is in the same\nfolder as zion.exe"),
            0, wxALIGN_CENTER_HORIZONTAL|wxALL, 8);

    wxBoxSizer *buttonSizer = new wxBoxSizer( wxHORIZONTAL );
    m_okButton = new wxButton( this, wxID_OK, "OK" );
    buttonSizer->Add( m_okButton,  0, wxALIGN_CENTER_VERTICAL|wxALL, 5); 
    buttonSizer->Add( new wxButton(this, wxID_CANCEL, "Cancel"), 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);   //! wxID_CANCEL handled by the parent

    topSizer->Add(buttonSizer,  0, wxALIGN_CENTER_HORIZONTAL |wxALL, 5);
    SetSizer(topSizer);  //use the sizer for layout
    topSizer->SetSizeHints(this);  //! Coordinate minimal window size
    m_okButton->SetDefault();  
    ///Positioning
    int dlgW = GetSize().GetWidth();
    int parW = parent->GetSize().GetWidth();
    int edgeOff = 50;
    if( parW - dlgW > dlgW )  edgeOff = ( parW - dlgW ) >> 1;
    
    int x = parent->GetPosition().x + parW - dlgW - edgeOff;
    int y = parent->GetPosition().y + 100;
    SetSize( x, y, wxDefaultCoord, wxDefaultCoord );
}

ZionIniDialog::~ZionIniDialog() {}

BEGIN_EVENT_TABLE(ZionIniDialog, wxDialog)
    EVT_BUTTON( wxID_OK, ZionIniDialog::OnOK )
    EVT_BUTTON( ZION_INI_BROWSE, ZionIniDialog::OnChooseDir )
END_EVENT_TABLE()

void ZionIniDialog::OnOK( wxCommandEvent &event)
{   event.Skip();
}

void ZionIniDialog::OnChooseDir(wxCommandEvent & event)
{
	wxDirDialog d( this, "Choose a directory", m_text->GetValue(),	wxDD_NEW_DIR_BUTTON );
	if( d.ShowModal()== wxID_OK)	m_text->SetValue(d.GetPath()); //!Get chosen dir 
}
//_________________________________

StartFolderDlg::StartFolderDlg( wxWindow *parent, wxPoint pos, int parentWidth, wxArrayString & rdirs, const wxString & caption )
	: wxDialog( parent, wxID_ANY, caption, wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE )
{
    wxArrayString localRdirs = rdirs;
    localRdirs.Insert( wxString::Format(_T("CWD->%s"), wxGetCwd() ), 0 );  ///Curr directory at top
    m_list = new wxListBox( this, ZION_RLISTBOX );
    m_list->Set( localRdirs );
    m_list->SetSelection(0);
    m_okButton = new wxButton( this, wxID_OK, _T("OK") );

    wxBoxSizer *topSizer = new wxBoxSizer( wxVERTICAL );
    topSizer->Add( m_list, 0, wxALIGN_CENTER_HORIZONTAL| wxALL, 4 );

    wxBoxSizer * btnSizer = new wxBoxSizer( wxHORIZONTAL );
    btnSizer->Add( m_okButton, 0, wxALL, 2 );
    btnSizer->AddSpacer(10);
    btnSizer->Add( new wxButton( this, wxID_CANCEL, _T("Cancel") ), 0, wxALL, 2 );
    topSizer->Add( btnSizer, 0, wxALIGN_CENTER_HORIZONTAL| wxALL, 2 );

    SetSizer(topSizer);
    topSizer->SetSizeHints(this);  //! Coordinate minimal window size
    m_okButton->SetDefault();
    ///Positioning - supplied by edit module
    int dlgW = GetSize().GetWidth();
    int edgeOff = 50;
    if( parentWidth - dlgW > dlgW )  edgeOff = ( parentWidth - dlgW ) >> 1;
    
    int x = pos.x + parentWidth - dlgW - edgeOff;
    int y = pos.y + 118;
//    ///Positioning
//    int dlgW = GetSize().GetWidth();
//    int parW = parent->GetSize().GetWidth();
//    int edgeOff = 40;
//    if( parW - dlgW > dlgW )  edgeOff = ( parW - dlgW ) >> 2;
//    
//    int x = parent->GetPosition().x + parW - dlgW - edgeOff;
//    int y = parent->GetPosition().y + 118;
    SetSize( x, y, wxDefaultCoord, wxDefaultCoord );
}

StartFolderDlg::~StartFolderDlg() {}

BEGIN_EVENT_TABLE(StartFolderDlg, wxDialog)
    EVT_BUTTON( wxID_OK, StartFolderDlg::OnOK )
    EVT_LISTBOX( ZION_RLISTBOX, StartFolderDlg::OnLBSel )
END_EVENT_TABLE()

void StartFolderDlg::OnLBSel( wxCommandEvent &event ) {
    wxCommandEvent anEvent( wxEVT_COMMAND_BUTTON_CLICKED, wxID_OK ); ///type, id
    this->ProcessEvent(anEvent);
}

void StartFolderDlg::OnOK( wxCommandEvent &event )  { event.Skip(); }

//_________________________________

CallTipNoteDlg::CallTipNoteDlg( wxWindow *parent, wxPoint pos, int parentWidth, const wxString & fullDef, const wxString & descript )
	: wxDialog( parent, wxID_ANY, _T("NOTE"), wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE )
{
    m_htmlWin = new wxHtmlWindow( this, wxID_ANY, wxDefaultPosition, wxSize(320, 160) );
    wxString str = _T("<html><body BGCOLOR=\"#F2E494\"><p><b>") +
        fullDef + _T("</b></p><p>") + descript + _T("</p></body></html>");
    m_htmlWin->SetPage(str);  //    m_htmlWin->LoadPage("simple.html");
    //m_okButton = new wxButton( this, wxID_OK, "OK" );

    wxBoxSizer *topSizer = new wxBoxSizer( wxVERTICAL );
    topSizer->Add( m_htmlWin, 0, wxALIGN_CENTER_HORIZONTAL |wxALL, 1);
    //topSizer->Add( m_okButton, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 3);

    SetSizer(topSizer);  //use the sizer for layout
    topSizer->SetSizeHints(this);  //! Coordinate minimal window size
    //m_okButton->SetDefault();
    ///Positioning - supplied by edit module
    int dlgW = GetSize().GetWidth();
    int edgeOff = 50;
    if( parentWidth - dlgW > dlgW )  edgeOff = ( parentWidth - dlgW ) >> 2;
    
    int x = pos.x + parentWidth - dlgW - edgeOff;
    int y = pos.y + 118;
    SetSize( x, y, wxDefaultCoord, wxDefaultCoord );
}

CallTipNoteDlg::~CallTipNoteDlg() {}

DescriptDlg::DescriptDlg( wxWindow *parent, wxPoint pos, int parW, const wxString & heading, const wxString & descript )
	: wxDialog( parent, wxID_ANY, _T(""), pos, wxDefaultSize, wxNO_3D )
{
    m_htmlWin = new wxHtmlWindow( this, wxID_ANY, wxDefaultPosition, wxSize(parW -20, 47) );
    m_htmlWin->SetBorders(1);
    wxString str = _T("<html><body BGCOLOR=\"#F2E494\"><font color=\"#784520\"><b>") +
        heading + _T("</b></font> - ") + descript + _T("</body></html>");
    m_htmlWin->SetPage(str);

    wxBoxSizer *topSizer = new wxBoxSizer( wxVERTICAL );
    topSizer->Add( m_htmlWin, 0, wxALIGN_CENTER_HORIZONTAL |wxALL, 1);
    SetSizer(topSizer);
    topSizer->SetSizeHints(this);  //! Coordinate minimal window size

    ///Positioning - supplied by edit module
    int x = pos.x + 8;
    int y = pos.y + 1;
    SetSize( x, y, wxDefaultCoord, wxDefaultCoord );
}

DescriptDlg::~DescriptDlg() {}

WinLocDlg::WinLocDlg( wxWindow *parent )
	: wxDialog( parent, wxID_ANY, "Choose Window Location",
    wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE )
{
    m_thisWin = new wxCheckBox( this, ZION_THISWINCHECK, _T("Open in THIS Window") );
    m_thisWin->SetValue(true);
    m_newWin = new wxCheckBox( this, ZION_NEWWINCHECK, _T("Open in NEW Window") );
    m_newWin->SetValue(false);
    
    wxString choices[] = { _T("Left"), _T("Top"), _T("Right"), _T("Top Left"), _T("Center"), 
            _T("Top Right"), _T("Bottom Left"), _T("Bottom"), _T("Bottom Right") };
    m_radio = new wxRadioBox( this, ZION_NEWWINSELECTED, _T(""), wxDefaultPosition, wxDefaultSize, 9, choices, 3, wxRA_SPECIFY_COLS );
    ///Startup Values
    m_thisWin->SetValue(true);
    m_newWin->SetValue(false);
    m_radio->Enable( false );
    
    m_okButton = new wxButton( this, wxID_OK, _T("OK"), wxDefaultPosition, wxSize(-1, 20 ) );

    wxBoxSizer *topSizer = new wxBoxSizer( wxVERTICAL );
    wxBoxSizer *topHSizer = new wxBoxSizer( wxHORIZONTAL );

    wxBoxSizer *topLSizer = new wxBoxSizer( wxVERTICAL );
    wxBoxSizer *topRSizer = new wxBoxSizer( wxVERTICAL );
    
    topLSizer->Add( m_thisWin, 0, wxALL, 1 );
    topLSizer->AddSpacer(8);
    topLSizer->Add( m_newWin, 0, wxALL, 1 );

    topRSizer->Add( m_okButton, 0, wxALL, 1 );
    topRSizer->Add( new wxButton( this, wxID_CANCEL, _T("Cancel"), wxDefaultPosition, wxSize(-1, 20) ), 0, wxALL, 1 );


    topHSizer->Add( topLSizer, 0, wxALIGN_LEFT| wxALIGN_BOTTOM | wxALL, 0 );
    topHSizer->AddSpacer(10);
    topHSizer->Add( topRSizer, 0, wxALIGN_RIGHT|wxALL, 0 );

    topSizer->Add( topHSizer, 0, wxALIGN_CENTER_HORIZONTAL| wxALIGN_BOTTOM | wxALL, 0 );
    topSizer->Add( m_radio, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 1);

    SetSizer(topSizer);
    topSizer->SetSizeHints(this);  //! Coordinate minimal window size
    m_okButton->SetDefault();
    
    ///Positioning
    int dlgW = GetSize().GetWidth();
    int parW = parent->GetSize().GetWidth();
    int edgeOff = 50;
    if( parW - dlgW > dlgW )  edgeOff = ( parW - dlgW ) >> 1;
    
    int x = parent->GetPosition().x + parW - dlgW - edgeOff;
    int y = parent->GetPosition().y + 200;
    SetSize( x, y, wxDefaultCoord, wxDefaultCoord );
}

WinLocDlg::~WinLocDlg() {}

BEGIN_EVENT_TABLE(WinLocDlg, wxDialog)
    EVT_BUTTON( wxID_OK, WinLocDlg::OnOK )
    EVT_RADIOBOX( ZION_NEWWINSELECTED, WinLocDlg::OnNewWinSel )
    EVT_CHECKBOX( ZION_THISWINCHECK, WinLocDlg::OnThisWinCheck )
    EVT_CHECKBOX( ZION_NEWWINCHECK, WinLocDlg::OnNewWinCheck )
END_EVENT_TABLE()

void WinLocDlg::OnNewWinSel( wxCommandEvent &event ) {
    wxCommandEvent anEvent( wxEVT_COMMAND_BUTTON_CLICKED, wxID_OK ); ///type, id
    this->ProcessEvent(anEvent);
}

void WinLocDlg::OnThisWinCheck( wxCommandEvent &event) {
    m_newWin->SetValue( false );
    m_radio->Enable( false );
}

void WinLocDlg::OnNewWinCheck( wxCommandEvent &event) {
    m_thisWin->SetValue( false );
    m_radio->Enable( true );
}

void WinLocDlg::OnOK( wxCommandEvent &event)  { event.Skip(); }
//_________________________________

AddTagsDialog::AddTagsDialog( wxWindow *parent, wxPoint pos, int parentWidth )
	: wxDialog( parent, wxID_ANY, "Quick Addition of a Pair of Tags",
    wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE )
{
    m_text = new wxTextCtrl(this, wxID_ANY, _T(""), wxDefaultPosition, wxSize(60, 22) );
    m_okButton = new wxButton( this, wxID_OK, "OK" );

    wxBoxSizer *topSizer = new wxBoxSizer( wxVERTICAL );
    wxBoxSizer * resultSizer = new wxBoxSizer( wxHORIZONTAL );
    resultSizer->Add( new wxStaticText(this, wxID_ANY, " Type tag text only:  "), 0,
            wxALIGN_LEFT| wxALIGN_CENTER_VERTICAL |wxALL, 1);
    resultSizer->Add(m_text, 0, wxALIGN_LEFT| wxALIGN_CENTER_VERTICAL |wxALL, 1);
    resultSizer->Add(m_okButton,  0, wxALIGN_LEFT| wxALIGN_CENTER_VERTICAL  |wxALL, 3);
    
    topSizer->Add( resultSizer, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 3);

    SetSizer(topSizer);  //use the sizer for layout
    topSizer->SetSizeHints(this);  //! Coordinate minimal window size
    m_okButton->SetDefault();
    ///Positioning - supplied by edit module
    int dlgW = GetSize().GetWidth();
    int edgeOff = 50;
    if( parentWidth - dlgW > dlgW )  edgeOff = ( parentWidth - dlgW ) >> 2;
    
    int x = pos.x + parentWidth - dlgW - edgeOff;
    int y = pos.y + 200;
    SetSize( x, y, wxDefaultCoord, wxDefaultCoord );
}

AddTagsDialog::~AddTagsDialog() {}

BEGIN_EVENT_TABLE(AddTagsDialog, wxDialog)
    EVT_BUTTON( wxID_OK, AddTagsDialog::OnOK )
END_EVENT_TABLE()

void AddTagsDialog::OnOK( wxCommandEvent &event)  { event.Skip(); }
//_________________________________

ColorResultDialog::ColorResultDialog( wxWindow *parent, const wxString& hexResult, const wxString& decResult )
	: wxDialog( parent, wxID_ANY, "Color Result",
     wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE )
{
    wxBoxSizer *topSizer = new wxBoxSizer( wxVERTICAL );
    topSizer->Add( new wxStaticText(this, wxID_ANY, "\nThe chosen color is:"),
            0, wxALIGN_CENTER_HORIZONTAL|wxALL, 4);
    m_text = new wxTextCtrl(this, wxID_ANY, hexResult, wxDefaultPosition, wxSize(59, 22) ); ///TODO - we don't really need a member for this
    wxTextCtrl * text2 = new wxTextCtrl(this, wxID_ANY, decResult, wxDefaultPosition, wxSize(88, 22) );
    
    wxBoxSizer * resultSizer = new wxBoxSizer( wxHORIZONTAL );
    resultSizer->Add(new wxStaticText(this, wxID_ANY, "0x"), 0, wxALIGN_LEFT| wxALIGN_CENTER_VERTICAL |wxALL, 1);
    resultSizer->Add(m_text, 0, wxALIGN_LEFT|wxALL, 1);
    topSizer->Add( resultSizer, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
    wxBoxSizer * resultSizer2 = new wxBoxSizer( wxHORIZONTAL );
    resultSizer2->Add(new wxStaticText(this, wxID_ANY, "Dec (RGB)"), 0, wxALIGN_LEFT| wxALIGN_CENTER_VERTICAL |wxALL, 1);
    resultSizer2->Add(text2, 0, wxALIGN_LEFT|wxALL, 1);
    topSizer->Add( resultSizer2, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

//    wxBoxSizer *buttonSizer = new wxBoxSizer( wxHORIZONTAL );
    m_okButton = new wxButton( this, wxID_OK, "OK" ); ///this needs to be a member for event handling
    topSizer->Add(m_okButton,  0, wxALIGN_CENTER_HORIZONTAL |wxALL, 5);
    SetSizer(topSizer);  //use the sizer for layout
    topSizer->SetSizeHints(this);  //! Coordinate minimal window size
    m_okButton->SetDefault();  
    
    ///Positioning
    int dlgW = GetSize().GetWidth();
    int parW = parent->GetSize().GetWidth();
    int edgeOff = 50;
    if( parW - dlgW > dlgW )  edgeOff = ( parW - dlgW ) >> 2;
    
    int x = parent->GetPosition().x + parW - dlgW - edgeOff;
    int y = parent->GetPosition().y + 40;
    SetSize( x, y, wxDefaultCoord, wxDefaultCoord );
}

ColorResultDialog::~ColorResultDialog() {}

BEGIN_EVENT_TABLE(ColorResultDialog, wxDialog)
    EVT_BUTTON( wxID_OK, ColorResultDialog::OnOK )
END_EVENT_TABLE()

void ColorResultDialog::OnOK( wxCommandEvent &event)  { event.Skip(); }
//_________________________________

/*! Line Dialog */  ///Goto
LineDialog::LineDialog( wxWindow *parent, wxPoint pos, int parentWidth, int currentLine, int maxLines )
	: wxDialog( parent, wxID_ANY, _T("Go To Line"), wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE ) //! wx Default Args required here
{
    wxString str;   str.Printf("%d", currentLine);
    m_spin = new wxSpinCtrl(this, wxID_ANY, str, wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS,
                1 , maxLines );
    m_spin->SetSelection( -1, -1 );

    wxBoxSizer *topSizer = new wxBoxSizer( wxVERTICAL );
    topSizer->Add( m_spin, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5 );

    wxBoxSizer *buttonSizer = new wxBoxSizer( wxHORIZONTAL );
    m_okButton = new wxButton( this, wxID_OK, "OK" );
    buttonSizer->Add( m_okButton,  0, wxALIGN_CENTER_VERTICAL|wxALL, 5 );
    buttonSizer->AddSpacer(8);
    buttonSizer->Add( new wxButton(this, wxID_CANCEL, "Cancel"), 0, wxALIGN_CENTER_VERTICAL|wxALL, 5 );   //! wxID_CANCEL handled by the parent

    topSizer->Add(buttonSizer,  0, wxALIGN_CENTER_HORIZONTAL |wxALL, 2);
    
    SetSizer(topSizer);  //use the sizer for layout
    topSizer->SetSizeHints(this);  //! Coordinate minimal window size
    m_okButton->SetDefault(); 
    ///Stole from Visual Studio - giving credit where due!
    SetTitle( wxString::Format(_T("Go To Line - Max: %d"), maxLines) );
    
    ///Positioning - supplied by edit module
    int dlgW = GetSize().GetWidth();
    int edgeOff = 50;
    if( parentWidth - dlgW > dlgW )  edgeOff = ( parentWidth - dlgW ) >> 1;
    int x = pos.x + parentWidth - dlgW - edgeOff;
    int y = pos.y + 118;
    SetSize( x, y, wxDefaultCoord, wxDefaultCoord );
}

LineDialog::~LineDialog() {}

BEGIN_EVENT_TABLE(LineDialog, wxDialog)
    EVT_BUTTON( wxID_OK, LineDialog::OnOK )
END_EVENT_TABLE()

void LineDialog::OnOK( wxCommandEvent &event )
{   event.Skip();
}
