#ifndef ZION_DIALOGS_H
#define ZION_DIALOGS_H

class ExecDialog: public wxDialog
{
  public:
    ExecDialog( wxWindow *parent, bool append, wxArrayString & execHistory,
            const wxString & lastExecStr, bool captureOut, bool runWithCmd,
            wxArrayString * rdirs, const wxString & lastWdir );
    
    ~ExecDialog();
    wxRadioButton * m_rbNew;
    wxRadioButton * m_rbAppend;
    wxCheckBox * m_chkCaptureOut;
    wxRadioButton * m_rbPath;
    wxRadioButton * m_rbCmd;
    wxComboBox * m_execCombo;
    wxTextCtrl * m_execText;
    wxComboBox * m_wdir;
  private:
    void OnOK( wxCommandEvent &event );
    void OnCaptureOut( wxCommandEvent &event );
    void OnExecCombo( wxCommandEvent &event );
    void OnChooseFile( wxCommandEvent &event );
    void OnChooseDir( wxCommandEvent &event );
    wxBoxSizer * m_mainSizer;
    wxButton * m_runButton;
    wxButton * m_browse;
    wxButton * m_wdirBrowse;
  
  DECLARE_EVENT_TABLE()
};

class ZionIniDialog: public wxDialog
{
  public:
    ZionIniDialog( wxWindow *parent, const wxString& exeDir );
    
    ~ZionIniDialog();
    wxTextCtrl * m_text;    //! size
    wxCheckBox * m_createIt;
  private:
    void OnOK( wxCommandEvent &event );
    void OnChooseDir( wxCommandEvent &event );
    wxButton * m_okButton;
    wxButton * m_browse;
  
  DECLARE_EVENT_TABLE()
};

class CallTipNoteDlg: public wxDialog  ///Called from the editor so needs m_ourFrame reference
{
  public:
    CallTipNoteDlg( wxWindow *parent, wxPoint pos, int parentWidth,
            const wxString & fullDef, const wxString & descript );
    wxHtmlWindow * m_htmlWin;
    ~CallTipNoteDlg();
};

class DescriptDlg: public wxDialog  ///Called from the editor so needs m_ourFrame reference
{
  public:
    DescriptDlg( wxWindow *parent, wxPoint pos, int parentWidth, const wxString & heading, const wxString & descript );
    wxHtmlWindow * m_htmlWin;
    ~DescriptDlg();
};

class StartFolderDlg: public wxDialog
{
  public:
    StartFolderDlg( wxWindow *parent, wxPoint pos, int parentWidth, wxArrayString & rdirs, const wxString & caption = _T("Start \"Save As\" From Folder") );
    ~StartFolderDlg();
    wxListBox * m_list; ///Needs to be publicly accessible for later query
  private:
    void OnOK( wxCommandEvent &event );
    void OnLBSel( wxCommandEvent & event );
    wxButton * m_okButton;
  DECLARE_EVENT_TABLE()
};

class WinLocDlg: public wxDialog
{
  public:
    WinLocDlg( wxWindow *parent );
    ~WinLocDlg();
    wxRadioBox * m_radio; ///Needs to be publicly accessible for later query
    wxCheckBox * m_thisWin;
    wxCheckBox * m_newWin;
  private:
    void OnOK( wxCommandEvent &event );
    void OnNewWinCheck( wxCommandEvent & event );
    void OnNewWinSel( wxCommandEvent & event );
    void OnThisWinCheck( wxCommandEvent & event );
    wxButton * m_okButton;
  DECLARE_EVENT_TABLE()
};

class AddTagsDialog: public wxDialog
{
  public:
    AddTagsDialog( wxWindow *parent, wxPoint pos, int parentWidth );
    ~AddTagsDialog();
    wxTextCtrl * m_text; ///Needs to publicly accessible for later query
  private:
    void OnOK( wxCommandEvent &event );
    wxButton * m_okButton;
  DECLARE_EVENT_TABLE()
};

class ColorResultDialog: public wxDialog
{
  public:
    ColorResultDialog( wxWindow *parent, const wxString & hexResult, const wxString& decResult );
    ~ColorResultDialog();
  private:
    void OnOK( wxCommandEvent &event );
    wxTextCtrl * m_text;    //! No need to query this control after showModal();
    wxButton * m_okButton;
  DECLARE_EVENT_TABLE()
};

class LineDialog: public wxDialog
{
  public:
    LineDialog( wxWindow *parent, wxPoint pos, int parentWidth, int currentLine, int maxLines );
    ~LineDialog();
    wxSpinCtrl * m_spin;    //! size
    //bool TransferDataFromWindow();  //! Override default f()
    
  private:
    void OnOK( wxCommandEvent &event );
    wxButton * m_okButton;
  
  DECLARE_EVENT_TABLE()
};

#endif
