// File:  syntax.cpp
// Author: Rohan Allison

// Many keywords shamelessly borrowed from the SciTE project.
// www.scite.org

/*! Syntax structures */
#include <wx/wx.h>
#include <wx/hashmap.h>
#include <stc.h>
#include "syntax.h"

wxColour g_caretColor = wxColour(0xf0,0x00,0x00);
wxColour g_selColor = wxColour(0xee,0x78,0x75);
wxColour g_foldColor = wxColour(0x68,0x76,0x76);

wxColour customColors[16] = { wxColour(0x00,0x00,0x00), wxColour(0x00,0x00,0x00),
        wxColour(0x00,0x00,0x00), wxColour(0x00,0x00,0x00), wxColour(0x00,0x00,0x00),
        wxColour(0x00,0x00,0x00), wxColour(0x00,0x00,0x00), wxColour(0x00,0x00,0x00),
        wxColour(0x00,0x00,0x00), wxColour(0x00,0x00,0x00), wxColour(0x00,0x00,0x00),
        wxColour(0x00,0x00,0x00), wxColour(0xDA,0xDA,0xC2), wxColour(0xC1,0xD3,0xa9),
        wxColour(0x00,0x00,0x00), wxColour(0x00,0x00,0x00) };

smallSyntax defaultSyntax  = //!Just a wrapper for some global styles
{
    "DEFAULT", //! Must be all caps
    wxSTC_LEX_NULL,
    _T("text.api"), true, true, ///CallTipFile, batchLike, cLike
    wxColour( 0xDA, 0xDA, 0xC2 ), ///Page Color
    wxColour( 0x20, 0x20, 0x20 ),  ///Line Fore
    wxColour( 0xC0, 0xC0, 0xA0 ),  ///Line Back
    wxColour( 0xEE, 0x78, 0x75 ),  ///Selection
    wxColour( 0x68, 0x76, 0x76 ),  ///Fold
    wxColour( 0x9f, 0x69, 0x34 ),  ///Indent
    4, ///TabWidth
    _T(" "), _T(" "), _T(" "), _T(" "), _T(" "), _T(" "), 
    { true, false, false, true, false, false, true, true }, 6,   //! line, fold, wrap, indent, usetabs(good for makefiles), viewwhitespc, horizSb, vertSb
    0,
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    {
        {"Plain Text", wxSTC_STYLE_DEFAULT, "","","108010","DADAC2","wxNORMAL","9"},
        {"LineNumber", wxSTC_STYLE_LINENUMBER, "","","202020","C0C0A0","Tahoma","8"},
        {"BraceLight", wxSTC_STYLE_BRACELIGHT, "bold","","00A000","F4C005","Tahoma","9"},
        {"BraceBad", wxSTC_STYLE_BRACEBAD, "bold","","FF0000","DADAC2","Tahoma","9"},
        {"ControlChar", wxSTC_STYLE_CONTROLCHAR, "","","A000C0","DAD9C0","wxNORMAL","9"},
        { "IndentGuide", wxSTC_STYLE_INDENTGUIDE, "","","703002","DAD9C0","wxNORMAL","9"}
    }
};

smallSyntax vbscriptSyntax  =
{
    "VB",
    wxSTC_LEX_VBSCRIPT,
    _T("vb.api"), true, true, 
    wxColour( 0xBA, 0xDA, 0xC2 ), ///Page
    wxColour( 0x20, 0x20, 0x0 ),  ///Line Fore
    wxColour( 0xA0, 0xA0, 0xC0 ),  ///Line Back
    wxColour( 0xEE, 0x78, 0x75 ),  ///Selection
    wxColour( 0x68, 0x76, 0x76 ),  ///Fold
    wxColour( 0x9f, 0x69, 0x34 ),  ///Indent
    4,
    _T("abs and array asc ascb ascw atn call case cbool \
cbyte ccur cdate cdbl chr chrb chrw cint csng cstr date dateadd datediff \
datepart dateserial datevalue day description dim do each else elseif empty end eqv \
erase err escape eval execute executeglobal exit exp explicit false \
filter firstindex fix for formatcurrency formatdatetime formatnumber \
formatpercent function get getlocale getobject getref global helpcontext \
helpfile hex hour if ignorecase imp inputbox instr instrb instrrev int is \
isarray isdate isempty isnull isnumeric isobject join lbound lcase left \
leftb len lenb length let loadpicture log loop ltrim match matches mid \
midb minute mod month monthname msgbox next not nothing now null number \
object oct on option or pattern private property public raise randomize \
redim regexp rem replace rgb right rightb rnd round rtrim scriptengine \
scriptenginebuildversion scriptenginemajorversion scriptengineminorversion \
second select set setlocale sgn sin source space split sqr strcomp  string \
strreverse sub tan test then time timer timeserial timevalue to trim tristate \
true typename ubound ucase unescape until value vartype \
weekday weekdayname wend while with xor year vbabort vbabortretryignore \
vbapplicationmodal vbarray vbbinarycompare vbblack vbblue vbboolean vbbyte \
vbcancel vbcr vbcritical vbcrlf vbcurrency vbcyan vbdatabasecompare \
vbdataobject vbdate vbdecimal vbdefaultbutton1 vbdefaultbutton2 \
vbdefaultbutton3 vbdefaultbutton4 vbdouble vbempty vberror vbexclamation \
vbfalse vbfirstfourdays vbfirstfullweek vbfirstjan1 vbformfeed vbfriday \
vbgeneraldate vbgreen vbignore vbinformation vbinteger vblf vblong \
vblongdate vblongtime vbmagenta vbmonday vbmsgboxhelpbutton vbmsgboxright \
vbmsgboxrtlreading vbmsgboxsetforeground vbnewline vbno vbnull vbnullchar \
vbnullstring vbobject vbobjecterror vbok vbokcancel vbokonly vbquestion \
vbred vbretry vbretrycancel vbsaturday vbshortdate vbshorttime \
vbsingle vbstring vbsunday vbsystemmodal vbtab vbtextcompare vbthursday \
vbtrue vbtuesday vbusedefault vbusesystem vbusesystemdayofweek vbvariant \
vbverticaltab vbwednesday vbwhite vbyellow vbyes vbyesno vbyesnocancel \
"),
    _T(" "),_T(" "), _T(" "), _T(" "), _T(" "),
    { true, false, false, true, false, false, true, true }, 14,
    3,
    { 
        wxSTC_B_DEFAULT, wxSTC_B_IDENTIFIER, wxSTC_B_KEYWORD, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
    },
    {  {  "Default Style",wxSTC_B_DEFAULT,              "bold", "","","DADAC2","Courier New",""  },
        {  "Comment",wxSTC_B_COMMENT,     "", "italic","7D7D7D","DADAC2","Arial",""   },  
        {  "Number",wxSTC_B_NUMBER,               "bold", "","CE1C17","DADAC2","Tahoma",""   },  
        {  "Keyword",wxSTC_B_KEYWORD,                "bold", "","4EC415","DADAC2","Comic Sans MS",""  },  
        {  "String",wxSTC_B_STRING,                "bold", "","4EC415","DADAC2","Comic Sans MS",""  },  
        {  "Preprocessor",wxSTC_B_PREPROCESSOR,        "bold", "","4EC010","DADAC2","Tahoma","" },  
        {  "Operator",wxSTC_B_OPERATOR,           "bold", "","0000A0","DADAC2","Tahoma",""  },  
        {  "Identifier",wxSTC_B_IDENTIFIER,     "bold", "","826040","DADAC2","Tahoma",""  },  
        {  "Date",wxSTC_B_DATE,                "bold", "","010101","889847","Tahoma",""  },  
        {  "String To EOL",wxSTC_B_STRINGEOL,     "bold", "","3ED209","F0EA04","Comic Sans MS",""  },
        {  "Keyword2",wxSTC_B_KEYWORD2,                 "bold", "","2791AF","DADAC2","Verdana",""  },  
        {  "Keyword3",wxSTC_B_KEYWORD3,                 "bold", "","2791AF","DADAC2","Verdana",""  },  
        {  "Keyword4",wxSTC_B_KEYWORD4,                 "bold", "","2791AF","DADAC2","Verdana",""  },  
        {  "Constant",wxSTC_B_CONSTANT,     "bold", "","010101","889847","Tahoma",""  },  
        {  "ASM",wxSTC_B_ASM,         "bold", "","9B009B","DADAC2","Tahoma",""  },  
        {  "Label",wxSTC_B_LABEL,             "bold", "","D84080","DADAC2","Tahoma",""  },  
        {  "Error",wxSTC_B_ERROR,     "", "italic","505050","DADAC2","Tahoma",""  },  
        {  "Hex Number",wxSTC_B_HEXNUMBER,     "bold", "","3ED2A9","F0EA04","Comic Sans MS",""  },
        {  "Bin Number",wxSTC_B_BINNUMBER,     "bold", "","3EEF09","F0EA04","Comic Sans MS",""  }
    }  //! End array of structures init
};

smallSyntax pythonSyntax  =
{
    "PYTHON",
    wxSTC_LEX_PYTHON,
    _T("python.api"), true, true,
    wxColour( 0xBA, 0xDA, 0xC2 ), ///Page
    wxColour( 0x20, 0x20, 0x0 ),  ///Line Fore
    wxColour( 0xA0, 0xA0, 0xC0 ),  ///Line Back
    wxColour( 0xEE, 0x78, 0x75 ),  ///Selection
    wxColour( 0x68, 0x76, 0x76 ),  ///Fold
    wxColour( 0x9f, 0x69, 0x34 ),  ///Indent
    4,
    _T("and assert break class continue def del elif else except exec \
       finally for from global if import in is lambda None not or pass \
       print raise return self try while yield"),
    _T("ACCELERATORS ALT AUTO3STATE AUTOCHECKBOX AUTORADIOBUTTON BEGIN \
       BITMAP BLOCK BUTTON CAPTION CHARACTERISTICS CHECKBOX CLASS \
       COMBOBOX CONTROL CTEXT CURSOR DEFPUSHBUTTON DIALOG DIALOGEX \
       DISCARDABLE EDITTEXT END EXSTYLE FONT GROUPBOX ICON LANGUAGE \
       LISTBOX LTEXT MENU MENUEX MENUITEM MESSAGETABLE POPUP PUSHBUTTON \
       RADIOBUTTON RCDATA RTEXT SCROLLBAR SEPARATOR SHIFT STATE3 \
       STRINGTABLE STYLE TEXTINCLUDE VALUE VERSION VERSIONINFO VIRTKEY"),_T(" "), _T(" "), _T(" "), _T(" "),
    { true, false, false, true, false, false, true, true }, 14,
    3,
    { wxSTC_P_DEFAULT, wxSTC_P_IDENTIFIER, wxSTC_B_KEYWORD, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
    },
    {  {  "Default Style",wxSTC_P_DEFAULT,              "", "","","DADAC2","Courier New",""  },
        {  "Line Comment",wxSTC_P_COMMENTLINE,     "", "italic","7D7D7D","DADAC2","Arial",""   },  
        {  "Number",wxSTC_P_NUMBER,               "bold", "","CE1C17","DADAC2","Tahoma",""   },  
        {  "String",wxSTC_P_STRING,                "bold", "","4EC415","DADAC2","Comic Sans MS",""  },  
        {  "Character",wxSTC_P_CHARACTER,        "bold", "","4EC010","DADAC2","Tahoma","" },  
        {  "Word",wxSTC_P_WORD,                 "bold", "","2791AF","DADAC2","Verdana",""  },  
        {  "Triple",wxSTC_P_TRIPLE,                "bold", "","010101","889847","Tahoma",""  },  
        {  "Triple Double",wxSTC_P_TRIPLEDOUBLE,     "bold", "","010101","889847","Tahoma",""  },  
        {  "Class Name",wxSTC_P_CLASSNAME,         "bold", "","9B009B","DADAC2","Tahoma",""  },  
        {  "Definition Name",wxSTC_P_DEFNAME,             "bold", "","884080","DADAC2","Tahoma",""  },  
        {  "Operator",wxSTC_P_OPERATOR,           "bold", "","0000A0","DADAC2","Tahoma",""  },  
        {  "Identifier",wxSTC_P_IDENTIFIER,     "bold", "","826040","DADAC2","Tahoma",""  },  
        {  "CommentBlock",wxSTC_P_COMMENTBLOCK,     "", "italic","505050","DADAC2","Tahoma",""  },  
        {  "String To EOL",wxSTC_P_STRINGEOL,     "bold", "","3ED209","F0EA04","Comic Sans MS",""  }
    }  //! End array of structures init
};

const wxChar * F1Words =
    _T("access action advance allocatable allocate \
apostrophe assign assignment associate asynchronous backspace \
bind blank block blockdata call case character class close common \
complex contains continue cycle data deallocate decimal delim \
default dimension direct do dowhile double doubleprecision elemental else \
elseif elsewhere encoding end endassociate endblockdata enddo \
endfile endforall endfunction endif endinterface endmodule endprogram \
endselect endsubroutine endtype endwhere entry eor equivalence \
err errmsg exist exit external file flush fmt forall form format \
formatted function go goto id if implicit in include inout \
integer inquire intent interface intrinsic iomsg iolength \
iostat logical module name named namelist nextrec nml \
none nullify number only open opened operator optional out pad \
parameter pass pause pending pointer pos position \
print private procedure program protected public quote read readwrite \
real rec recl recursive result return rewind save select \
selectcase selecttype sequence sequential stat status stop stream \
subroutine target then to type unformatted unit use value \
volatile wait where while write");

const wxChar * F2Words =
    _T("abs achar acos acosd adjustl adjustr \
aimag aimax0 aimin0 aint ajmax0 ajmin0 akmax0 akmin0 all allocated alog \
alog10 amax0 amax1 amin0 amin1 amod anint any asin asind associated \
atan atan2 atan2d atand bitest bitl bitlr bitrl bjtest bit_size bktest break \
btest cabs ccos cdabs cdcos cdexp cdlog cdsin cdsqrt ceiling cexp char \
clog cmplx conjg cos cosd cosh count cpu_time cshift csin csqrt dabs \
dacos dacosd dasin dasind datan datan2 datan2d datand date \
date_and_time dble dcmplx dconjg dcos dcosd dcosh dcotan ddim dexp \
dfloat dflotk dfloti dflotj digits dim dimag dint dlog dlog10 dmax1 dmin1 \
dmod dnint dot_product dprod dreal dsign dsin dsind dsinh dsqrt dtan dtand \
dtanh eoshift epsilon errsns exp exponent float floati floatj floatk floor fraction \
free huge iabs iachar iand ibclr ibits ibset ichar idate idim idint idnint ieor ifix \
iiabs iiand iibclr iibits iibset iidim iidint iidnnt iieor iifix iint iior iiqint iiqnnt iishft \
iishftc iisign ilen imax0 imax1 imin0 imin1 imod index inint inot int int1 int2 int4 \
int8 iqint iqnint ior ishft ishftc isign isnan izext jiand jibclr jibits jibset jidim jidint \
jidnnt jieor jifix jint jior jiqint jiqnnt jishft jishftc jisign jmax0 jmax1 jmin0 jmin1 \
jmod jnint jnot jzext kiabs kiand kibclr kibits kibset kidim kidint kidnnt kieor kifix \
kind kint kior kishft kishftc kisign kmax0 kmax1 kmin0 kmin1 kmod knint knot kzext \
lbound leadz len len_trim lenlge lge lgt lle llt log log10 logical lshift malloc matmul \
max max0 max1 maxexponent maxloc maxval merge min min0 min1 minexponent minloc \
minval mod modulo mvbits nearest nint not nworkers number_of_processors pack popcnt \
poppar precision present product radix random random_number random_seed range real \
repeat reshape rrspacing rshift scale scan secnds selected_int_kind \
selected_real_kind set_exponent shape sign sin sind sinh size sizeof sngl snglq spacing \
spread sqrt sum system_clock tan tand tanh tiny transfer transpose trim ubound unpack verify");

const wxChar * F3Words =
_T("cdabs cdcos cdexp cdlog cdsin cdsqrt cotan cotand \
dcmplx dconjg dcotan dcotand decode dimag dll_export dll_import doublecomplex dreal \
dvchk encode find flen flush getarg getcharqq getcl getdat getenv gettim hfix ibchng \
identifier imag int1 int2 int4 intc intrup invalop iostat_msg isha ishc ishl jfix \
lacfar locking locnear map nargs nbreak ndperr ndpexc offset ovefl peekcharqq precfill \
prompt qabs qacos qacosd qasin qasind qatan qatand qatan2 qcmplx qconjg qcos qcosd \
qcosh qdim qexp qext qextd qfloat qimag qlog qlog10 qmax1 qmin1 qmod qreal qsign qsin \
qsind qsinh qsqrt qtan qtand qtanh ran rand randu rewrite segment setdat settim system \
timer undfl unlock union val virtual volatile zabs zcos zexp zlog zsin zsqrt");

const wxChar * CSSWords =
_T("azimuth background-attachment background-color background-image background-position \
background-repeat background border-collaspe border-color border-spacing border-style \
border-top border-right border-bottom border-left border-top-color border-right-color border-bottom-color \
border-left-color border-top-style border-right-style border-bottom-style border-left-style \
border-top-width border-right-width border-bottom-width border-left-width border-width border \
bottom caption-side clear clip color content counter-increment counter-reset cue-after cue-before \
cue cursor direction display elevation empty-cells float font-family font-size font-style font-variant \
font-weight font height left letter-spacing line-height list-style-image list-style-position list-style-type \
list-style margin-right margin-left margin-top margin-bottom margin max-height max-width min-height \
min-width orphans outline-color outline-style outline-width outline overflow padding-top padding-right \
padding-bottom padding-left padding page-break-after page-break-before page-break-inside \
pause-after pause-before pause pitch-range pitch play-during position quotes richness right speak-header \
speak-numeral speak-punctuation speak speech-rate stress table-layout text-align text-decoration \
text-indent text-transform top unicode-bidi vertical-align visibility voice-family volume \
white-space widows width word-spacing z-index \
");

smallSyntax fortranSyntax  =
{
    "FORTRAN9X",
    wxSTC_LEX_FORTRAN,
    _T("fortran.api"), false, true,
    wxColour( 0xDA, 0xDA, 0xC2 ), ///Page
    wxColour( 0x20, 0x20, 0x20 ),  ///Line Fore
    wxColour( 0xC0, 0xC0, 0xA0 ),  ///Line Back
    wxColour( 0xEE, 0x78, 0x75 ),  ///Selection
    wxColour( 0x68, 0x76, 0x76 ),  ///Fold
    wxColour( 0x9f, 0x69, 0x34 ),  ///Indent
    4,
    _T(""),
    _T(""),
    _T(""),
 _T(" "), _T(" "), _T(" "),
    { true, false, false, true, false, false, true, true }, 15,
    5,
    { 
        wxSTC_F_DEFAULT, wxSTC_F_IDENTIFIER, wxSTC_F_WORD, wxSTC_F_WORD2, wxSTC_F_WORD3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
    },
    {  {  "Default Style",wxSTC_F_DEFAULT,              "", "","","DADAC2","Courier New",""  },
        {  "Comment",wxSTC_F_COMMENT,     "", "italic","7D7D7D","DADAC2","Arial",""   },  
        {  "Number",wxSTC_F_NUMBER,               "bold", "","CE1C17","DADAC2","Tahoma",""   },  
        {  "String",wxSTC_F_STRING1,                "bold", "","4EC415","DADAC2","Tahoma",""  },  
        {  "String2",wxSTC_F_STRING2,        "bold", "","4EC010","DADAC2","Tahoma","" },  
        {  "String To EOL",wxSTC_F_STRINGEOL,     "bold", "","3ED209","F0EA04","Tahoma",""  },
        {  "Operator",wxSTC_F_OPERATOR,           "bold", "","0000A0","DADAC2","Tahoma",""  },  
        {  "Identifier",wxSTC_F_IDENTIFIER,     "bold", "","826040","DADAC2","Tahoma",""  },  
        {  "Keywords + I/O Specifiers", wxSTC_F_WORD,                 "bold", "","2791AF","DADAC2","Verdana",""  },  
        {  "Intrinsic + Ext. functions", wxSTC_F_WORD2,                 "bold", "","2791AF","DADAC2","Verdana",""  },  
        {  "Non-Std+ Ext'd + User def. f()s", wxSTC_F_WORD3,                 "bold", "","2791AF","DADAC2","Verdana",""  },  
        {  "Preprocessor",wxSTC_F_PREPROCESSOR,     "bold", "","010101","889847","Tahoma",""  },  
        {  "Operator2",wxSTC_F_OPERATOR2,           "bold", "","0000A0","DADAC2","Tahoma",""  },  
        {  "Label",wxSTC_F_LABEL,         "bold", "","9B009B","DADAC2","Tahoma",""  },  
        {  "Continuation",wxSTC_F_CONTINUATION,             "bold", "","884080","DADAC2","Tahoma",""  }
    }  //! End array of structures init
};


smallSyntax f77Syntax  =
{
    "FORTRAN77",
    wxSTC_LEX_F77,
    _T("fortran77.api"), false, true,
    wxColour( 0xDA, 0xDA, 0xC2 ),  ///Page
    wxColour( 0x20, 0x20, 0x20 ),  ///Line Fore
    wxColour( 0xC0, 0xC0, 0xA0 ),  ///Line Back
    wxColour( 0xEE, 0x78, 0x75 ),  ///Selection
    wxColour( 0x68, 0x76, 0x76 ),  ///Fold
    wxColour( 0x9f, 0x69, 0x34 ),  ///Indent
    3,
    _T(""),
    _T(""),
    _T(""),
 _T(" "), _T(" "), _T(" "),
    { true, false, false, true, false, false, true, true }, 15,
    2,
    { 
        wxSTC_F_DEFAULT, wxSTC_F_IDENTIFIER, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
    },
    {  {  "Default Style", wxSTC_F_DEFAULT,              "", "","","DADAC2","Courier New",""  },
        {  "Comment", wxSTC_F_COMMENT,     "", "italic","7D7D7D","DADAC2","Arial",""   },  
        {  "Number", wxSTC_F_NUMBER,               "bold", "","CE1C17","DADAC2","Tahoma",""   },  
        {  "String", wxSTC_F_STRING1,                "bold", "","4EC415","DADAC2","Tahoma",""  },  
        {  "String2", wxSTC_F_STRING2,        "bold", "","4EC010","DADAC2","Tahoma","" },  
        {  "String To EOL", wxSTC_F_STRINGEOL,     "bold", "","3ED209","F0EA04","Tahoma",""  },
        {  "Operator", wxSTC_F_OPERATOR,           "bold", "","0000A0","DADAC2","Tahoma",""  },  
        {  "Identifier", wxSTC_F_IDENTIFIER,     "bold", "","826040","DADAC2","Tahoma",""  },  
        {  "Keywords + I/O Specifiers", wxSTC_F_WORD,                 "bold", "","2791AF","DADAC2","Verdana",""  },  
        {  "Intrinsic + Ext. functions", wxSTC_F_WORD2,                 "bold", "","2791AF","DADAC2","Verdana",""  },  
        {  "Non-Std+ Ext'd + User def. f()s", wxSTC_F_WORD3,                 "bold", "","2791AF","DADAC2","Verdana",""  },  
        {  "Preprocessor", wxSTC_F_PREPROCESSOR,     "bold", "","010101","889847","Tahoma",""  },  
        {  "Operator2", wxSTC_F_OPERATOR2,           "bold", "","0000A0","DADAC2","Tahoma",""  },  
        {  "Label", wxSTC_F_LABEL,         "bold", "","9B009B","DADAC2","Tahoma",""  },  
        {  "Continuation", wxSTC_F_CONTINUATION,             "bold", "","884080","DADAC2","Tahoma",""  }
    } //! End array of structures init
};

smallSyntax sqlSyntax  =
{
    "SQL",
    wxSTC_LEX_SQL,
    _T("sql.api"), true, true,
    wxColour( 0xDA, 0xDA, 0xC2 ),  ///Page
    wxColour( 0x20, 0x20, 0x20 ),  ///Line Fore
    wxColour( 0xC0, 0xC0, 0xA0 ),  ///Line Back
    wxColour( 0xEE, 0x78, 0x75 ),  ///Selection
    wxColour( 0x68, 0x76, 0x76 ),  ///Fold
    wxColour( 0x9f, 0x69, 0x34 ),  ///Indent
    4,
    _T("absolute action add admin after aggregate \
alias all allocate alter analyse and any are array as asc \
asensitive assertion at authorization \
before begin between bigint binary bit blob boolean both breadth by \
call cascade cascaded case cast catalog change char character \
check class clob close collate collation column commit \
completion condition connect connection constraint constraints \
constructor continue convert corresponding create cross cube current \
current_date current_path current_role current_time current_timestamp \
current_user cursor cycle \
data database databases date day day_hour day_microsecond day_minute day_second \
deallocate dec decimal declare default \
deferrable deferred delayed delete depth deref desc describe descriptor \
destroy destructor deterministic dictionary diagnostics disconnect \
distinct distinctrow div domain double drop dual dynamic \
each else elseif enclosed end end-exec equals escape escaped every except \
exception exec execute exists exit explain external \
false fetch first float float4 float8 for force foreign found from free full \
fulltext function general get global go goto grant group grouping \
having high_priority host hour hour_microsecond hour_minute hour_second\
identity if ignore immediate in index indicator infile initialize initially \
inner inout input insensitive insert int int1 int2 int3 int4 int8 integer intersect interval \
into is isolation iterate join key keys kill \
language large last lateral leading leave left less level like \
limit lines load local localtime localtimestamp locator lock long longblob longtext loop\
low_priority map match mediumblob mediumint mediumtext middleint minute minute_microsecond \
minute_second mod modifies modify module month \
names national natural nchar nclob new next no no_write_to_binlog none \
not null numeric object of off old on only open operation optimize option optionally \
or order ordinality out outer outfile output \
pad parameter parameters partial path postfix precision prefix \
preorder prepare preserve primary prior privileges procedure public purge\
raid0 read reads real recursive ref references referencing regexp relative \
release rename repeat replace require restrict result return returns revoke right \
rlike role rollback rollup routine row rows \
savepoint schema schemas scroll scope search second second_microsecond section select \
sequence sensitive separator session session_user set sets show size smallint some soname space \
spatial specific specifictype sql sqlexception sqlstate sqlwarning sql_big_result \
sql_calc_found_rows sql_small_result ssl start starting\
state statement static straight_join structure system_user \
table temporary terminate terminated than then time timestamp \
timezone_hour timezone_minute tinyblob tinyint tinytext to trailing transaction translation \
treat trigger true under undo union unique unknown unlock \
unnest unsigned update usage use user using utc_date utc_time utc_timestamp\
value values varbinary varchar varcharacter variable varying view \
when whenever where while with without work write x509 xor \
year year_month zerofill zone"), 
    _T(""),_T(""), _T(""), _T(""), _T(""),
    { true, false, false, true, false, false, true, true }, 22,
    2,
    { 
        wxSTC_SQL_DEFAULT, wxSTC_SQL_IDENTIFIER, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
    },
    {  {  "Default Style",wxSTC_SQL_DEFAULT,  "", "","","DADAC2","Tahoma",""  },
        {  "Comment",wxSTC_SQL_COMMENT,     "", "italic","8D8D7D","CADAC2","Arial",""   },  
        {  "Line Comment",wxSTC_SQL_COMMENTLINE,     "", "italic","7D7D7D","DADAC2","Arial",""   },  
        {  "Comment Doc",wxSTC_SQL_COMMENTDOC,     "", "italic","7D7D7D","DADAC2","Arial",""   },  
        {  "Number",wxSTC_SQL_NUMBER,               "bold", "","CE1817","DADAC2","Tahoma",""   },  
        {  "Word",wxSTC_SQL_WORD,                 "bold", "","8091AF","DADAC2","Verdana",""  },  
        {  "String",wxSTC_SQL_STRING,                "bold", "","4EC415","DADAC2","Tahoma",""  },  
        {  "Character",wxSTC_SQL_CHARACTER,             "bold", "","88A090","DADAC2","Tahoma",""  },  
        {  "SqlPlus",wxSTC_SQL_SQLPLUS,        "bold", "","4EC010","DADAC2","Tahoma","" },  
        {  "SqlPlus Prompt",wxSTC_SQL_SQLPLUS_PROMPT,         "bold", "","9B009B","DADAC2","Tahoma",""  },  
        {  "Operator",wxSTC_SQL_OPERATOR,           "bold", "","A0A000","DADAC2","Tahoma",""  },  
        {  "Identifier",wxSTC_SQL_IDENTIFIER,     "bold", "","D0D4E9","DADAC2","Tahoma",""  },  
        {  "Sql Plus Comment",wxSTC_SQL_SQLPLUS_COMMENT,     "bold", "","909090","DADAC2","Tahoma",""  },  
        {  "Comment Line Doc",wxSTC_SQL_COMMENTLINEDOC,     "bold", "","806074","DADAC2","Tahoma",""  },  
        {  "Word2",wxSTC_SQL_WORD2,     "bold", "","B2A0B5","DADAC2","Tahoma",""  },  
        {  "Comment Doc Keyword",wxSTC_SQL_COMMENTDOCKEYWORD,     "bold", "","707070","DADAC2","Tahoma",""  },  
        {  "Comment Doc Keyword Error",wxSTC_SQL_COMMENTDOCKEYWORDERROR,     "bold", "","A26040","DADAC2","Tahoma",""  },  
        {  "User Words1",wxSTC_SQL_USER1,     "bold", "","826040","DADAC2","Tahoma",""  },  
        {  "User Words2",wxSTC_SQL_USER2,     "bold", "","A088B6","DADAC2","Tahoma",""  },  
        {  "User Words3",wxSTC_SQL_USER3,     "bold", "","826040","DADAC2","Tahoma",""  },  
        {  "User Words4",wxSTC_SQL_USER4,     "bold", "","826040","DADAC2","Tahoma",""  },  
        {  "Quoted Identifier",wxSTC_SQL_QUOTEDIDENTIFIER,     "bold", "","A0B4B2","DADAC2","Tahoma",""  }
    }  //! End array of structures init
};

smallSyntax rubySyntax  =
{
    "RUBY",
    wxSTC_LEX_RUBY,
    _T("ruby.api"), false, true,
    wxColour( 0xDA, 0xDA, 0xC2 ),  ///Page
    wxColour( 0x20, 0x20, 0x20 ),  ///Line Fore
    wxColour( 0xC0, 0xC0, 0xA0 ),  ///Line Back
    wxColour( 0xEE, 0x78, 0x75 ),  ///Selection
    wxColour( 0x68, 0x76, 0x76 ),  ///Fold
    wxColour( 0x9f, 0x69, 0x34 ),  ///Indent
    4,
    _T("__FILE__ and def print end in or self unless __LINE__ begin defined? \
    ensure module redo super until BEGIN break do false next rescue then when \
    END case else for nil retry true while alias class elsif if not return undef yield"),
    _T(""),_T(" "), _T(" "), _T(" "), _T(" "),
    { true, false, false, true, false, false, true, true }, 34,
    2,
    { 
        wxSTC_RB_DEFAULT, wxSTC_RB_IDENTIFIER, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
    },
    {  {  "Default Style",wxSTC_RB_DEFAULT,  "", "","","DADAC2","Courier New",""  },
        {  "Error", wxSTC_RB_ERROR,              "", "","","DADAC2","Courier New",""  },
        {  "Line Comment",wxSTC_RB_COMMENTLINE,     "", "italic","7D7D7D","DADAC2","Arial",""   },  
        {  "Pod",wxSTC_RB_POD,               "bold", "","CE1C17","DADAC2","Tahoma",""   },  
        {  "Number",wxSTC_RB_NUMBER,               "bold", "","CE1C17","DADAC2","Tahoma",""   },  
        {  "Word",wxSTC_RB_WORD,                 "bold", "","2791AF","DADAC2","Verdana",""  },  
        {  "String",wxSTC_RB_STRING,                "bold", "","4EC415","DADAC2","Comic Sans MS",""  },  
        {  "Character",wxSTC_RB_CHARACTER,        "bold", "","4EC010","DADAC2","Tahoma","" },  
        {  "Class Name",wxSTC_RB_CLASSNAME,         "bold", "","9B009B","DADAC2","Tahoma",""  },  
        {  "Definition Name",wxSTC_RB_DEFNAME,             "bold", "","884080","DADAC2","Tahoma",""  },  
        {  "Operator",wxSTC_RB_OPERATOR,           "bold", "","0000A0","DADAC2","Tahoma",""  },  
        {  "Identifier",wxSTC_RB_IDENTIFIER,     "bold", "","826040","DADAC2","Tahoma",""  },  
        {  _T("Regex"), wxSTC_RB_REGEX,                "bold", "","010101","889847","Tahoma",""  },  
        {  _T("Global"), wxSTC_RB_GLOBAL,                "bold", "","010101","889847","Tahoma",""  },  
        {  _T("Symbol"), wxSTC_RB_SYMBOL,                "bold", "","010101","889847","Tahoma",""  },  
        {  _T("Module Name"), wxSTC_RB_MODULE_NAME,                "bold", "","010101","889847","Tahoma",""  },  
        {  _T("Instance Var"), wxSTC_RB_INSTANCE_VAR,                "bold", "","010101","889847","Tahoma",""  },  
        {  _T("Class Var"), wxSTC_RB_CLASS_VAR,                "bold", "","010101","889847","Tahoma",""  },  
        {  _T("BackTicks"), wxSTC_RB_BACKTICKS,                "bold", "","010101","889847","Tahoma",""  },  
        {  _T("Data Section"), wxSTC_RB_DATASECTION,                "bold", "","010101","889847","Tahoma",""  },  
        {  _T("Here Delim"), wxSTC_RB_HERE_DELIM,                "bold", "","010101","889847","Tahoma",""  },  
        {  _T("Here Q"), wxSTC_RB_HERE_Q,                "bold", "","010101","889847","Tahoma",""  },  
        {  _T("Here QQ"), wxSTC_RB_HERE_QQ,                "bold", "","010101","889847","Tahoma",""  },  
        {  _T("Here QX"), wxSTC_RB_HERE_QX,                "bold", "","010101","889847","Tahoma",""  },  
        {  _T("String Q"), wxSTC_RB_STRING_Q,                "bold", "","010101","889847","Tahoma",""  },  
        {  _T("String QQ"), wxSTC_RB_STRING_QQ,                "bold", "","010101","889847","Tahoma",""  },  
        {  _T("String QX"), wxSTC_RB_STRING_QX,                "bold", "","010101","889847","Tahoma",""  },  
        {  _T("String QR"), wxSTC_RB_STRING_QR,                "bold", "","010101","889847","Tahoma",""  },  
        {  _T("Sring QW"), wxSTC_RB_STRING_QW,                "bold", "","010101","889847","Tahoma",""  },  
        {  _T("Word Demoted"), wxSTC_RB_WORD_DEMOTED,                "bold", "","010101","889847","Tahoma",""  },  
        {  _T("StdIn"), wxSTC_RB_STDIN,                "bold", "","010101","889847","Tahoma",""  },  
        {  _T("StdOut"), wxSTC_RB_STDOUT,                "bold", "","010101","889847","Tahoma",""  },  
        {  _T("StdErr"), wxSTC_RB_STDERR,                "bold", "","010101","889847","Tahoma",""  },  
        {  "Upper Bound", wxSTC_RB_UPPER_BOUND,     "bold", "","3ED209","F0EA04","Tahoma",""  }
    }  //! End array of structures init
};

smallSyntax lispSyntax =
{
    "LISP",
    wxSTC_LEX_LISP,
    _T("lisp.api"), false, true,
    wxColour( 0xDA, 0xDA, 0xC2 ),  ///Page
    wxColour( 0x20, 0x20, 0x20 ),  ///Line Fore
    wxColour( 0xC0, 0xC0, 0xA0 ),  ///Line Back
    wxColour( 0xEE, 0x78, 0x75 ),  ///Selection
    wxColour( 0x68, 0x76, 0x76 ),  ///Fold
    wxColour( 0x9f, 0x69, 0x34 ),  ///Indent
    4,
    _T("CAR CDR SETQ QUOTE EVAL APPEND LIST CONS ATOM LISTP NULL MEMBERP NIL T DEFUN ABS \
        EXPT SQRT MAX MIN COND LOAD :LD not defun + - * / = < > <= >= princ \
eval apply funcall quote identity function complement backquote lambda set setq setf \
defun defmacro gensym make symbol intern symbol name symbol value symbol plist get \
getf putprop remprop hash make array aref car cdr caar cadr cdar cddr caaar caadr cadar \
caddr cdaar cdadr cddar cdddr caaaar caaadr caadar caaddr cadaar cadadr caddar cadddr \
cdaaar cdaadr cdadar cdaddr cddaar cddadr cdddar cddddr cons list append reverse last nth \
nthcdr member assoc subst sublis nsubst  nsublis remove length list length \
mapc mapcar mapl maplist mapcan mapcon rplaca rplacd nconc delete atom symbolp numberp \
boundp null listp consp minusp zerop plusp evenp oddp eq eql equal cond case and or let l if prog \
prog1 prog2 progn go return do dolist dotimes catch throw error cerror break \
continue errset baktrace evalhook truncate float rem min max abs sin cos tan expt exp sqrt \
random logand logior logxor lognot bignums logeqv lognand lognor \
logorc2 logtest logbitp logcount integer length nil"),
    _T(""),_T(" "), _T(" "), _T(" "), _T(" "),
    { true, false, false, true, false, false, true, true }, 8,
    2,
    { 
        wxSTC_LISP_DEFAULT, wxSTC_LISP_IDENTIFIER, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
    },
    {  { "Default Style", wxSTC_LISP_DEFAULT, "", "", "000000", "DADAC2", "Tahoma",""  },
        { "Comment", wxSTC_LISP_COMMENT,  "", "","5D5D5D","DADAC2","Times New Roman","10"  },
        { "Number", wxSTC_LISP_NUMBER, "bold", "","CE1C17","DADAC2","Tahoma",""  },
        { "Keyword", wxSTC_LISP_KEYWORD,  "bold", "","8f010c","DADAC2","Tahoma",""  },
        { "String", wxSTC_LISP_STRING,  "bold", "","109006","DADAC2","Verdana","8"  },
        { "String to EOL", wxSTC_LISP_STRINGEOL, "bold", "","000000","F0EA04","Verdana",""  },
        { "Identifier", wxSTC_LISP_IDENTIFIER,  "bold", "","100030","DADAC2","Tahoma",""  },
        { "Operator", wxSTC_LISP_OPERATOR,  "bold", "","825002","DADAC2","Tahoma",""  }
    }  //! End array of structures init
};

smallSyntax batchSyntax =
{
    "BATCH",
    wxSTC_LEX_BATCH,
    _T("batch.api"), true, false,
    wxColour( 0xDA, 0xDA, 0xC2 ),  ///Page
    wxColour( 0x20, 0x20, 0x20 ),  ///Line Fore
    wxColour( 0xC0, 0xC0, 0xA0 ),  ///Line Back
    wxColour( 0xEE, 0x78, 0x75 ),  ///Selection
    wxColour( 0x68, 0x76, 0x76 ),  ///Fold
    wxColour( 0x9f, 0x69, 0x34 ),  ///Indent
    4,
    _T("IF ERRORLEVEL REM NOT NUL ECHO ECHO OFF FOR LABEL GOTO PAUSE CHOICE EXIST CALL\
        COMMAND SET SHIFT SGN CON PRN LPT COM ASSIGN ATTRIB CD CHDIR CLS COMP COPY DEL \
        DELTREE DIR ERASE EXIT FC FIND MD MKDIR MORE MOVE PATH REN RENAME RD RMDIR SORT \
        TIME TYPE XCOPY LH DEVICE lh device if errorlevel rem not nul echo echo off for label goto pause choice exist call\
        command set shift sgn con prn lpt com assign attrib cd chdir cls comp copy del \
        deltree dir erase exit fc find md mkdir more move path ren rename rd rmdir sort \
        time type xcopy"),
    _T(""),_T(" "), _T(" "), _T(" "), _T(" "),
    { true, false, false, true, false, false, true, true }, 8,
    4,
    { 
        wxSTC_BAT_DEFAULT, wxSTC_BAT_IDENTIFIER, wxSTC_BAT_WORD, wxSTC_BAT_COMMAND, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
    },
    {  { "Default Style", wxSTC_BAT_DEFAULT, "", "", "000000", "DADAC2", "Tahoma",""  },
        { "Comment", wxSTC_BAT_COMMENT,  "", "","5D5D5D","DADAC2","Times New Roman","10"  },
        { "Keyword", wxSTC_BAT_WORD,  "bold", "","8f010c","DADAC2","Tahoma",""  },
        { "Number", wxSTC_BAT_LABEL, "bold", "","CE1C17","DADAC2","Tahoma",""  },
        { "Hidden", wxSTC_BAT_HIDE,  "", "","109006","DADAC2","Verdana","8"  },
        { "Command", wxSTC_BAT_COMMAND,  "bold", "","9B009B","DADAC2","Verdana",""  },
        { "Identifier", wxSTC_BAT_IDENTIFIER,  "bold", "","100030","DADAC2","Tahoma",""  },
        { "Operator", wxSTC_BAT_OPERATOR,  "bold", "","825002","DADAC2","Tahoma",""  }
    }  //! End array of structures init
};

smallSyntax cppSyntax =
{
    "C++",
    wxSTC_LEX_CPP,
    _T("cpp.api"), false, true,
    wxColour( 0xDA, 0xDA, 0xC2 ),  ///Page
    wxColour( 0x20, 0x20, 0x20 ),  ///Line Fore
    wxColour( 0xC0, 0xC0, 0xA0 ),  ///Line Back
    wxColour( 0xEE, 0x78, 0x75 ),  ///Selection
    wxColour( 0x68, 0x76, 0x76 ),  ///Fold
    wxColour( 0x9f, 0x69, 0x34 ),  ///Indent
    4,
    _T("asm auto bool break case catch char class const const_cast \
       continue default delete do double dynamic_cast else enum explicit \
       export extern false float for friend goto if inline int long \
       mutable namespace new operator private protected public register \
       reinterpret_cast return short signed sizeof static static_cast \
       struct switch template this throw true try typedef typeid \
       typename union unsigned using virtual void volatile wchar_t \
       while"),
    _T(""), ///There might be a memory issue here//If the pointer is reassigned, what happens to this literal?  //main printf Printf wxString wx
/*    _T("a addindex addtogroup anchor arg attention \
author b brief bug c class code date def defgroup deprecated dontinclude \
e em endcode endhtmlonly endif endlatexonly endlink endverbatim enum example exception \
f$ f[ f] file fn hideinitializer htmlinclude htmlonly \
if image include ingroup internal invariant interface latexonly li line link \
mainpage name namespace nosubgrouping note overload \
p page par param post pre ref relates remarks return retval \
sa section see showinitializer since skip skipline struct subsection \
test throw todo typedef union until \
var verbatim verbinclude version warning weakgroup $ @ \\ & < > # { }"),*/
    _T(" "),
    _T(" "), _T(" "), _T(" "), 
    { true, false, false, true, false, false, true, true }, 19,
    4,
    {
        wxSTC_C_DEFAULT, wxSTC_C_IDENTIFIER, wxSTC_C_WORD, wxSTC_C_WORD2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
    },
                                    //bold,italic,fore,back,face,size(pts)
    {  {  "Default Style",wxSTC_C_DEFAULT, "", "","000000","DADAC2","Tahoma",""  },
        {  "Multi-line Comment",wxSTC_C_COMMENT,     "", "","6D6D6D","DADAC2","Arial",""  },  
        {  "Line Comment",wxSTC_C_COMMENTLINE,     "", "","5D5D5D","DADAC2","Times New Roman","10"  },  
        {  "Doxygen Comment",wxSTC_C_COMMENTDOC,     "bold", "","000000","CACAB2","Times New Roman",""  },  
        {  "Number",wxSTC_C_NUMBER,               "bold", "","CE1C17","DADAC2","Tahoma",""  },  
        {  "Word",wxSTC_C_WORD,                 "bold", "","8f010c","DADAC2","Tahoma",""  },  
        {  "String",wxSTC_C_STRING,                "bold", "","109006","DADAC2","Verdana","8"  },  
        {  "Char",wxSTC_C_CHARACTER,        "bold", "","4EC010","DADAC2","Tahoma",""  },  
        {  "UUID",wxSTC_C_UUID,                "bold", "","30E050","DADAC2","Tahoma",""  },  
        {  "Preprocessor",wxSTC_C_PREPROCESSOR,     "bold", "","e1952d","DADAC2","Tahoma",""  },  
        {  "Operator",wxSTC_C_OPERATOR,           "bold", "","825002","DADAC2","Tahoma",""  },  
        {  "Identifier", wxSTC_C_IDENTIFIER,     "bold", "","100030","DADAC2","Tahoma",""  },  
        {  "String To EOL",wxSTC_C_STRINGEOL,     "bold", "","000000","F0EA04","Verdana",""  },
        {  "Verbatim",wxSTC_C_VERBATIM,     "", "italic","505050","DADAC2","Tahoma",""  },  
        {  "Regular Exp",wxSTC_C_REGEX,         "bold", "","80F0F0","DADAC2","Tahoma",""  },  
        {  "Doxygen Line Comment", wxSTC_C_COMMENTLINEDOC,             "", "italic","60462A","DADAC2","Times New Roman","10"  },  
        {  "Word List 2",wxSTC_C_WORD2,     "bold", "","9B009B","DADAC2","Verdana",""  },  
        {  "Doxygen Keyword",wxSTC_C_COMMENTDOCKEYWORD,     "bold", "italic","204050","DADAC2","Arial",""  },  
        {  "Doxygen Keyword Error",wxSTC_C_COMMENTDOCKEYWORDERROR,     "bold", "italic","E05050","CACAB2","Arial",""  },  
    }  //! End array of structures init
};

smallSyntax cssSyntax =
{
    "CSS",
    wxSTC_LEX_CSS,
    _T("css.api"), false, true,
    wxColour( 0x36, 0x36, 0x36 ),  ///Page
    wxColour( 0x00, 0x00, 0x00 ),  ///Line Fore
    wxColour( 0x44, 0x53, 0x59 ),  ///Line Back
    wxColour( 0xEE, 0x78, 0x75 ),  ///Selection
    wxColour( 0x68, 0x76, 0x76 ),  ///Fold
    wxColour( 0x9f, 0x69, 0x34 ),  ///Indent
    4,
    _T(""),
    _T("first-child active focus lang first-line first-letter link visited hover after before"), ///There might be a memory issue here//If the pointer is reassigned, what happens to this literal?  //main printf Printf wxString wx
    _T(""),    _T(""), _T(""), _T(""), 
    { true, false, false, true, false, false, true, true }, 17,
                                    //bold,italic,fore,back,face,size(pts)
    3,
    { 
        wxSTC_CSS_DEFAULT, wxSTC_CSS_IDENTIFIER, wxSTC_CSS_ATTRIBUTE, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
    },
    {  {  "Default Style",wxSTC_CSS_DEFAULT, "bold", "","A0A0A0","000000","Tahoma",""  },
        {  "Tag",wxSTC_CSS_TAG,     "bold", "","886aac","000000","Tahoma","9"  },  
        {  "Class",wxSTC_CSS_CLASS,     "bold", "","35998c","000000","Tahoma","9"  },  
        {  "Pseudo Class",wxSTC_CSS_PSEUDOCLASS,     "bold", "","A4B2A9","000000","Tahoma",""  },  
        {  "Unknown Pseudo Class",wxSTC_CSS_UNKNOWN_PSEUDOCLASS,               "bold", "","CE1C17","000000","Tahoma",""  },  
        {  "Operator",wxSTC_CSS_OPERATOR,                 "bold", "","b05f37","000000","Tahoma",""  },  
        {  "Identifier",wxSTC_CSS_IDENTIFIER,                "bold", "","109006","000000","Tahoma","9"  },  
        {  "Unknown Identifier",wxSTC_CSS_UNKNOWN_IDENTIFIER,        "bold", "","4EC010","000000","Tahoma",""  },  
        {  "Value",wxSTC_CSS_VALUE,                "bold", "","c1736a","000000","Tahoma",""  },  
        {  "Comment",wxSTC_CSS_COMMENT,     "", "italic","938e79","000000","Times New Roman",""  },  
        {  "ID",wxSTC_CSS_ID,           "bold", "","d16d23","000000","Tahoma",""  },  
        {  "Important", wxSTC_CSS_IMPORTANT,     "bold", "","A0E430","000000","Tahoma",""  },  
        {  "Directive",wxSTC_CSS_DIRECTIVE,     "bold", "","000000","dcd218","Verdana",""  },
        {  "Double String",wxSTC_CSS_DOUBLESTRING,     "bold", "","47b03c","000000","Tahoma",""  },  
        {  "Single String",wxSTC_CSS_SINGLESTRING,         "bold", "","80F0F0","000000","Tahoma",""  },  
        {  "Identifier 2", wxSTC_CSS_IDENTIFIER2,             "bold", "","2ca36a","000000","Tahoma","9"  },  
        {  "Attribute",wxSTC_CSS_ATTRIBUTE,     "bold", "","9B009B","000000","Verdana",""  }
    }  //! End array of structures init
};

smallSyntax csSyntax =
{
    "C#",
    wxSTC_LEX_CS,
    _T("cs.api"), true, true,
    wxColour( 0xDA, 0xDA, 0xC2 ),  ///Page
    wxColour( 0x20, 0x20, 0x20 ),  ///Line Fore
    wxColour( 0xC0, 0xC0, 0xA0 ),  ///Line Back
    wxColour( 0xEE, 0x78, 0x75 ),  ///Selection
    wxColour( 0x68, 0x76, 0x76 ),  ///Fold
    wxColour( 0x9f, 0x69, 0x34 ),  ///Indent
    4,
    _T("abstract as base bool break byte case catch char checked class \
const continue decimal default delegate do double else enum \
event explicit extern false finally fixed float for foreach goto if \
implicit in int interface internal is lock long namespace new null \
object operator out override params private protected public \
readonly ref return sbyte sealed short sizeof stackalloc static \
string struct switch this throw true try typeof uint ulong \
unchecked unsafe ushort using virtual void while"),
    _T(""),_T(" "), _T(" "), _T(" "), _T(" "), 
    { true, false, false, true, false, false, true, true }, 19,
    4,
    { 
        wxSTC_C_DEFAULT, wxSTC_C_IDENTIFIER, wxSTC_C_WORD, wxSTC_C_WORD2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
    },
                                    //bold,italic,fore,back,face,size(pts)
    {  {  "Default Style",wxSTC_C_DEFAULT, "", "","000000","DADAC2","Tahoma",""  },
        {  "Multi-line Comment",wxSTC_C_COMMENT,     "", "","6D6D6D","DADAC2","Arial",""  },  
        {  "Line Comment",wxSTC_C_COMMENTLINE,     "", "","5D5D5D","DADAC2","Times New Roman","10"  },  
        {  "Doxygen Comment",wxSTC_C_COMMENTDOC,     "bold", "","000000","CACAB2","Times New Roman",""  },  
        {  "Number",wxSTC_C_NUMBER,               "bold", "","CE1C17","DADAC2","Tahoma",""  },  
        {  "Word",wxSTC_C_WORD,                 "bold", "","8f010c","DADAC2","Tahoma",""  },  
        {  "String",wxSTC_C_STRING,                "bold", "","109006","DADAC2","Verdana","8"  },  
        {  "Char",wxSTC_C_CHARACTER,        "bold", "","4EC010","DADAC2","Tahoma",""  },  
        {  "UUID",wxSTC_C_UUID,                "bold", "","30E050","DADAC2","Tahoma",""  },  
        {  "Preprocessor",wxSTC_C_PREPROCESSOR,     "bold", "","e1952d","DADAC2","Tahoma",""  },  
        {  "Operator",wxSTC_C_OPERATOR,           "bold", "","825002","DADAC2","Tahoma",""  },  
        {  "Identifier", wxSTC_C_IDENTIFIER,     "bold", "","100030","DADAC2","Tahoma",""  },  
        {  "String To EOL",wxSTC_C_STRINGEOL,     "bold", "","000000","F0EA04","Verdana",""  },
        {  "Verbatim",wxSTC_C_VERBATIM,     "", "italic","505050","DADAC2","Tahoma",""  },  
        {  "Regular Exp",wxSTC_C_REGEX,         "bold", "","80F0F0","DADAC2","Tahoma",""  },  
        {  "Doxygen Line Comment", wxSTC_C_COMMENTLINEDOC,             "", "italic","60462A","DADAC2","Times New Roman","10"  },  
        {  "Word List 2",wxSTC_C_WORD2,     "bold", "","9B009B","DADAC2","Verdana",""  },  
        {  "Doxygen Keyword",wxSTC_C_COMMENTDOCKEYWORD,     "bold", "italic","204050","DADAC2","Arial",""  },  
        {  "Doxygen Keyword Error",wxSTC_C_COMMENTDOCKEYWORDERROR,     "bold", "italic","E05050","CACAB2","Arial",""  },  
    }  //! End array of structures init
};

smallSyntax javaSyntax =
{
    "JAVA",
    wxSTC_LEX_JAVA,
    _T("java.api"), false, true,
    wxColour( 0xDA, 0xDA, 0xC2 ),  ///Page
    wxColour( 0x20, 0x20, 0x20 ),  ///Line Fore
    wxColour( 0xC0, 0xC0, 0xA0 ),  ///Line Back
    wxColour( 0xEE, 0x78, 0x75 ),  ///Selection
    wxColour( 0x68, 0x76, 0x76 ),  ///Fold
    wxColour( 0x9f, 0x69, 0x34 ),  ///Indent
    4,
    _T("abstract assert boolean break byte case catch char class \
const continue default do double else extends final finally float for future \
generic goto if implements import inner instanceof int interface long \
native new null outer package private protected public rest \
return short static super switch synchronized this throw throws \
transient try var void volatile while"),
    _T("system"),_T(" "), _T(" "), _T(" "), _T(" "), 
    { true, false, false, true, false, false, true, true }, 19,
    4,
    { 
        wxSTC_C_DEFAULT, wxSTC_C_IDENTIFIER, wxSTC_C_WORD, wxSTC_C_WORD2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
    },
                                    //bold,italic,fore,back,face,size(pts)
    {  {  "Default Style",wxSTC_C_DEFAULT, "", "","000000","DADAC2","Tahoma",""  },
        {  "Multi-line Comment",wxSTC_C_COMMENT,     "", "","6D6D6D","DADAC2","Arial",""  },  
        {  "Line Comment",wxSTC_C_COMMENTLINE,     "", "","5D5D5D","DADAC2","Times New Roman","10"  },  
        {  "Doxygen Comment",wxSTC_C_COMMENTDOC,     "bold", "","000000","CACAB2","Times New Roman",""  },  
        {  "Number",wxSTC_C_NUMBER,               "bold", "","CE1C17","DADAC2","Tahoma",""  },  
        {  "Word",wxSTC_C_WORD,                 "bold", "","8f010c","DADAC2","Tahoma",""  },  
        {  "String",wxSTC_C_STRING,                "bold", "","109006","DADAC2","Verdana","8"  },  
        {  "Char",wxSTC_C_CHARACTER,        "bold", "","4EC010","DADAC2","Tahoma",""  },  
        {  "UUID",wxSTC_C_UUID,                "bold", "","30E050","DADAC2","Tahoma",""  },  
        {  "Preprocessor",wxSTC_C_PREPROCESSOR,     "bold", "","e1952d","DADAC2","Tahoma",""  },  
        {  "Operator",wxSTC_C_OPERATOR,           "bold", "","825002","DADAC2","Tahoma",""  },  
        {  "Identifier", wxSTC_C_IDENTIFIER,     "bold", "","100030","DADAC2","Tahoma",""  },  
        {  "String To EOL",wxSTC_C_STRINGEOL,     "bold", "","000000","F0EA04","Verdana",""  },
        {  "Verbatim",wxSTC_C_VERBATIM,     "", "italic","505050","DADAC2","Tahoma",""  },  
        {  "Regular Exp",wxSTC_C_REGEX,         "bold", "","80F0F0","DADAC2","Tahoma",""  },  
        {  "Doxygen Line Comment", wxSTC_C_COMMENTLINEDOC,             "", "italic","60462A","DADAC2","Times New Roman","10"  },  
        {  "Word List 2",wxSTC_C_WORD2,     "bold", "","9B009B","DADAC2","Verdana",""  },  
        {  "Doxygen Keyword",wxSTC_C_COMMENTDOCKEYWORD,     "bold", "italic","204050","DADAC2","Arial",""  },  
        {  "Doxygen Keyword Error",wxSTC_C_COMMENTDOCKEYWORDERROR,     "bold", "italic","E05050","CACAB2","Arial",""  },  
    }  //! End array of structures init
};

smallSyntax perlSyntax =
{
    "PERL",
    wxSTC_LEX_PERL,
    _T("perl.api"), false, true,
    wxColour( 0xDA, 0xDA, 0xC2 ),  ///Page
    wxColour( 0x20, 0x20, 0x20 ),  ///Line Fore
    wxColour( 0xC0, 0xC0, 0xA0 ),  ///Line Back
    wxColour( 0xEE, 0x78, 0x75 ),  ///Selection
    wxColour( 0x68, 0x76, 0x76 ),  ///Fold
    wxColour( 0x9f, 0x69, 0x34 ),  ///Indent
    4,
    _T("NULL __FILE__ __LINE__ __PACKAGE__ __DATA__ __END__ AUTOLOAD \
BEGIN CORE DESTROY END EQ GE GT INIT LE LT NE CHECK abs accept \
alarm and atan2 bind binmode bless caller chdir chmod chomp chop \
chown chr chroot close closedir cmp connect continue cos crypt \
dbmclose dbmopen defined delete die do dump each else elsif endgrent \
endhostent endnetent endprotoent endpwent endservent eof eq eval \
exec exists exit exp fcntl fileno flock for foreach fork format \
formline ge getc getgrent getgrgid getgrnam gethostbyaddr gethostbyname \
gethostent getlogin getnetbyaddr getnetbyname getnetent getpeername \
getpgrp getppid getpriority getprotobyname getprotobynumber getprotoent \
getpwent getpwnam getpwuid getservbyname getservbyport getservent \
getsockname getsockopt glob gmtime goto grep gt hex if index \
int ioctl join keys kill last lc lcfirst le length link listen \
local localtime lock log lstat lt map mkdir msgctl msgget msgrcv \
msgsnd my ne next no not oct open opendir or ord our pack package \
pipe pop pos print printf prototype push quotemeta qu \
rand read readdir readline readlink readpipe recv redo \
ref rename require reset return reverse rewinddir rindex rmdir \
scalar seek seekdir select semctl semget semop send setgrent \
sethostent setnetent setpgrp setpriority setprotoent setpwent \
setservent setsockopt shift shmctl shmget shmread shmwrite shutdown \
sin sleep socket socketpair sort splice split sprintf sqrt srand \
stat study sub substr symlink syscall sysopen sysread sysseek \
system syswrite tell telldir tie tied time times truncate \
uc ucfirst umask undef unless unlink unpack unshift untie until \
use utime values vec wait waitpid wantarray warn while write \
xor"),
    /* NULL __FILE__ __LINE__ __PACKAGE__ __DATA__ __END__ AUTOLOAD \
BEGIN CORE DESTROY END EQ GE GT INIT LE LT NE CHECK \
if else elsif for or and accept not eq ne lt gt split undef foreach while next last continue close open\
        print chomp chop chr crypt hex index lc lcfirst length oct ord pack reverse \
        rindex sprintf substr tr uc ucfirst pos quotemeta split study qr abs atan2 cos exp\
        hex int log oct rand sin sqrt srand pop push shift splice unshift grep join map qw\
        reverse sort unpack delete each exists keys values each exists keys values binmode\
        closedir dbmclose dbmopen die eof fileno flock format getc print printf read readdir\
        rewinddir seek seekdir select system syscall sub sysread sysseek syswrite sysopen tell telldir truncate warn write\
        pack read syscall sysread syswrite unpack vec chdir opendir closedir mkdir chmod chown chroot fcntl glob ioctl link lstat\
        mkdir open opendir readlink rename rmdir stat symlink sysopen umask unlink utime caller continue\
        do dump eval exit goto last next redo return sub caller import local my our package use\
        defined  formline local my our reset scalar undef wantarray bless package\
        ref tie tied untie use qx readline readpipe ref tie tied uc ucfirst untie use */
    _T(""), _T(" "), _T(" "), _T(" "), _T(" "), 
    { true, false, false, true, false, false, true, true }, 30,    //! Style num are not in array order!
                                    //bold,italic,fore,back,face,size(pts)
    3,
    { 
        wxSTC_PL_DEFAULT, wxSTC_PL_IDENTIFIER, wxSTC_PL_WORD, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
    },
    {  {  "Default Style",wxSTC_PL_DEFAULT, "", "","000000","DADAC2","Tahoma","9"  },
        {  "Error",wxSTC_PL_ERROR,     "", "","A01D1D","DADAC2","Arial","9"  },  
        {  "Line Comment",wxSTC_PL_COMMENTLINE,     "", "","5D5D5D","DADAC2","Times New Roman","10"  },  
        {  "Pod",wxSTC_PL_POD,     "bold", "","000000","CACAB2","Times New Roman","9"  },  
        {  "Number",wxSTC_PL_NUMBER,               "bold", "","CE1C17","DADAC2","Tahoma","9"  },  
        {  "Word",wxSTC_PL_WORD,                 "bold", "","8f010c","DADAC2","Tahoma","9"  },  
        {  "String",wxSTC_PL_STRING,                "bold", "","109006","DADAC2","Verdana","8"  },  
        {  "Char",wxSTC_PL_CHARACTER,        "bold", "","4EC010","DADAC2","Tahoma","9"  },  
        {  "Punctuation",wxSTC_PL_PUNCTUATION,                "bold", "","30E050","DADAC2","Tahoma","9"  },  
        {  "Preprocessor",wxSTC_PL_PREPROCESSOR,     "bold", "","e1952d","DADAC2","Tahoma","9"  },  
        {  "Operator",wxSTC_PL_OPERATOR,           "bold", "","825002","DADAC2","Tahoma","9"  },  
        {  "Identifier", wxSTC_PL_IDENTIFIER,     "bold", "","100030","DADAC2","Tahoma","9"  },  
        {  "Scalar", wxSTC_PL_SCALAR,     "bold", "","265e42","DADAC2","Verdana","9"  },
        {  "Array",wxSTC_PL_ARRAY,     "", "italic","505050","DADAC2","Tahoma","9"  },  
        {  "Hash",wxSTC_PL_HASH,         "bold", "","80F0F0","DADAC2","Tahoma","9"  },  
        {  "Symbol Table", wxSTC_PL_SYMBOLTABLE,             "", "italic","60462A","DADAC2","Times New Roman","10"  },  
        {  "Regex",wxSTC_PL_REGEX,     "bold", "","9B009B","DADAC2","Verdana","9"  },  
        {  "Reg Subst",wxSTC_PL_REGSUBST,     "bold", "italic","204050","DADAC2","Arial","9"  },  
        {  "Long Quote",wxSTC_PL_LONGQUOTE,     "bold", "italic","E05050","CACAB2","Arial","9"  },  
        {  "Back Ticks",wxSTC_PL_BACKTICKS,                "bold", "","109006","DADAC2","Tahoma","9"  },  
        {  "Data Section",wxSTC_PL_DATASECTION,                "bold", "","109006","DADAC2","Tahoma","9"  },  
        {  "Here Delimit",wxSTC_PL_HERE_DELIM,                "bold", "","109006","DADAC2","Tahoma","9"  },  
        {  "Here Q",wxSTC_PL_HERE_Q,                "bold", "","109006","DADAC2","Tahoma","9"  },  
        {  "Here QQ",wxSTC_PL_HERE_QQ,                "bold", "","109006","DADAC2","Tahoma","9"  },  
        {  "Here QX",wxSTC_PL_HERE_QX,                "bold", "","109006","DADAC2","Tahoma","9"  },  
        {  "String Q",wxSTC_PL_STRING_Q,                "bold", "","109006","DADAC2","Tahoma","9"  },  
        {  "String QQ",wxSTC_PL_STRING_QQ,                "bold", "","109006","DADAC2","Tahoma","9"  },  
        {  "String QX",wxSTC_PL_STRING_QX,                "bold", "","109006","DADAC2","Tahoma","9"  },  
        {  "String QR",wxSTC_PL_STRING_QR,                "bold", "","109006","DADAC2","Tahoma","9"  },  
        {  "String QW",wxSTC_PL_STRING_QW,                "bold", "","109006","DADAC2","Tahoma","9"  }
    }  //! End array of structures init
};

smallSyntax makeSyntax =
{
    "MAKEFILE",
    wxSTC_LEX_MAKEFILE,
    _T("make.api"), true, false,
    wxColour( 0xDA, 0xDA, 0xC2 ), ///Page
    wxColour( 0x20, 0x20, 0x20 ),  ///Line Fore
    wxColour( 0xC0, 0xC0, 0xA0 ),  ///Line Back
    wxColour( 0xEE, 0x78, 0x75 ),  ///Selection
    wxColour( 0x68, 0x76, 0x76 ),  ///Fold
    wxColour( 0x9f, 0x69, 0x34 ),  ///Indent
    4,
    _T(""),
    _T(""),_T(" "), _T(" "), _T(" "), _T(" "), 
    { false, false, true, false, true, true, true, true }, 7,  //! line, fold, wrap, indent, usetabs(good for makefiles), viewwhitespc
    2,
    { 
        wxSTC_MAKE_DEFAULT, wxSTC_MAKE_IDENTIFIER, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
    },
                                    //bold,italic,fore,back,face,size(pts)
    {  {  "Default Style",wxSTC_MAKE_DEFAULT, "", "","000000","DADAC2","Tahoma",""  },
        {  "Comment",wxSTC_MAKE_COMMENT,     "", "","6D6D6D","DADAC2","Arial",""  },  
        {  "Preprocessor",wxSTC_MAKE_PREPROCESSOR,     "bold", "","e1952d","DADAC2","Tahoma",""  },
        {  "Identifier", wxSTC_MAKE_IDENTIFIER,     "bold", "","100030","DADAC2","Tahoma",""  },  
        {  "Operator",wxSTC_MAKE_OPERATOR,           "bold", "","825002","DADAC2","Tahoma",""  },  
        {  "Target",wxSTC_MAKE_TARGET,     "bold", "","9B009B","DADAC2","Verdana",""  },  
        {  "ID To EOL",wxSTC_MAKE_IDEOL,     "bold", "","000000","F0EA04","Verdana",""  }
    }  //! End array of structures init
};

smallSyntax asmSyntax =
{
    "ASM",
    wxSTC_LEX_ASM,
    _T("asm.api"), true, false,
    wxColour( 0xDA, 0xDA, 0xC2 ), ///Page
    wxColour( 0x20, 0x20, 0x20 ),  ///Line Fore
    wxColour( 0xC0, 0xC0, 0xA0 ),  ///Line Back
    wxColour( 0xEE, 0x78, 0x75 ),  ///Selection
    wxColour( 0x68, 0x76, 0x76 ),  ///Fold
    wxColour( 0x9f, 0x69, 0x34 ),  ///Indent
    3,
    _T("aaa aad aam aas adc add and call cbw \
	clc cld cli cmc cmp cmps cmpsb cmpsw cwd daa das dec div esc hlt \
	idiv imul in inc int into iret ja jae jb jbe jc jcxz je jg jge jl \
	jle jmp jna jnae jnb jnbe jnc jne jng jnge jnl jnle jno jnp jns \
	jnz jo jp jpe jpo js jz lahf lds lea les lods lodsb lodsw loop \
	loope loopew loopne loopnew loopnz loopnzw loopw loopz loopzw \
	mov movs movsb movsw mul neg nop not or out pop popf push pushf \
	rcl rcr ret retf retn rol ror sahf sal sar sbb scas scasb scasw \
	shl shr stc std sti stos stosb stosw sub test wait xchg xlat \
	xlatb xor \
	bound enter ins insb insw leave outs outsb outsw popa pusha pushw \
	arpl lar lsl sgdt sidt sldt smsw str verr verw clts lgdt lidt lldt lmsw ltr \
	bsf bsr bt btc btr bts cdq cmpsd cwde insd iretd iretdf  iretf \
	jecxz lfs lgs lodsd loopd  looped  loopned  loopnzd  loopzd  lss \
	movsd movsx movzx outsd popad popfd pushad pushd  pushfd scasd seta \
	setae setb setbe setc sete setg setge setl setle setna setnae setnb \
	setnbe setnc setne setng setnge setnl setnle setno setnp setns \
	setnz seto setp setpe setpo sets setz shld shrd stosd \
	bswap cmpxchg invd  invlpg  wbinvd  xadd \
	lock rep repe repne repnz repz\
    cflush cpuid emms femms \
	cmovo cmovno cmovb cmovc cmovnae cmovae cmovnb cmovnc \
	cmove cmovz cmovne cmovnz cmovbe cmovna cmova cmovnbe \
	cmovs cmovns cmovp cmovpe cmovnp cmovpo cmovl cmovnge \
	cmovge cmovnl cmovle cmovng cmovg cmovnle \
	cmpxchg486 cmpxchg8b  \
	loadall loadall286 ibts icebp int1 int3 int01 int03 iretw \
	popaw popfw pushaw pushfw rdmsr rdpmc rdshr rdtsc \
	rsdc rsldt rsm rsts salc smi smint smintold svdc svldt svts \
	syscall sysenter sysexit sysret ud0 ud1 ud2 umov xbts wrmsr wrshr"),
    _T("f2xm1 fabs fadd faddp fbld fbstp fchs fclex fcom fcomp fcompp fdecstp \
	fdisi fdiv fdivp fdivr fdivrp feni ffree fiadd ficom ficomp fidiv \
	fidivr fild fimul fincstp finit fist fistp fisub fisubr fld fld1 \
	fldcw fldenv fldenvw fldl2e fldl2t fldlg2 fldln2 fldpi fldz fmul \
	fmulp fnclex fndisi fneni fninit fnop fnsave fnsavew fnstcw fnstenv \
	fnstenvw fnstsw fpatan fprem fptan frndint frstor frstorw fsave \
	fsavew fscale fsqrt fst fstcw fstenv fstenvw fstp fstsw fsub fsubp \
	fsubr fsubrp ftst fwait fxam fxch fxtract fyl2x fyl2xp1 \
	fsetpm fcos fldenvd fnsaved fnstenvd fprem1 frstord fsaved fsin fsincos \
	fstenvd fucom fucomp fucompp fcomi fcomip ffreep \
	fcmovb fcmove fcmovbe fcmovu fcmovnb fcmovne fcmovnbe fcmovnu"),
    _T("ah al ax bh bl bp bx ch cl cr0 cr2 cr3 cr4 cs \
	cx dh di dl dr0 dr1 dr2 dr3 dr6 dr7 ds dx eax ebp ebx ecx edi edx \
	es esi esp fs gs si sp ss st tr3 tr4 tr5 tr6 tr7 \
	st0 st1 st2 st3 st4 st5 st6 st7 mm0 mm1 mm2 mm3 mm4 mm5 mm6 mm7 \
	xmm0 xmm1 xmm2 xmm3 xmm4 xmm5 xmm6 xmm7"),
    _T(".186 .286 .286c .286p .287 .386 .386c .386p .387 .486 .486p \
	.8086 .8087 .alpha .break .code .const .continue .cref .data .data?  \
	.dosseg .else .elseif .endif .endw .err .err1 .err2 .errb \
	.errdef .errdif .errdifi .erre .erridn .erridni .errnb .errndef \
	.errnz .exit .fardata .fardata? .if .lall .lfcond .list .listall \
	.listif .listmacro .listmacroall  .model .no87 .nocref .nolist \
	.nolistif .nolistmacro .radix .repeat .sall .seq .sfcond .stack \
	.startup .tfcond .type .until .untilcxz .while .xall .xcref \
	.xlist alias align assume catstr comm comment db dd df dosseg dq \
	dt dup dw echo else elseif elseif1 elseif2 elseifb elseifdef elseifdif \
	elseifdifi elseife elseifidn elseifidni elseifnb elseifndef end \
	endif endm endp ends eq equ even exitm extern externdef extrn for \
	forc ge goto group gt high highword if if1 if2 ifb ifdef ifdif \
	ifdifi ife  ifidn ifidni ifnb ifndef include includelib instr invoke \
	irp irpc label le length lengthof local low lowword lroffset \
	lt macro mask mod .msfloat name ne offset opattr option org %out \
	page popcontext proc proto ptr public purge pushcontext record \
	repeat rept seg segment short size sizeof sizestr struc struct \
	substr subtitle subttl textequ this title type typedef union while width\
    db dw dd dq dt resb resw resd resq rest incbin equ times \
	%define %idefine %xdefine %xidefine %undef %assign %iassign \
	%strlen %substr %macro %imacro %endmacro %rotate .nolist \
	%if %elif %else %endif %ifdef %ifndef %elifdef %elifndef \
	%ifmacro %ifnmacro %elifmacro %elifnmacro %ifctk %ifnctk %elifctk %elifnctk \
	%ifidn %ifnidn %elifidn %elifnidn %ifidni %ifnidni %elifidni %elifnidni \
	%ifid %ifnid %elifid %elifnid %ifstr %ifnstr %elifstr %elifnstr \
	%ifnum %ifnnum %elifnum %elifnnum %error %rep %endrep %exitrep \
	%include %push %pop %repl struct endstruc istruc at iend align alignb \
	%arg %stacksize %local %line \
	bits use16 use32 section absolute extern global common cpu org \
	section group import export"),
    _T("$ ? @b @f addr basic byte c carry? dword \
	far far16 fortran fword near near16 overflow? parity? pascal qword  \
	real4  real8 real10 sbyte sdword sign? stdcall sword syscall tbyte \
	vararg word zero? flat near32 far32 \
	abs all assumes at casemap common compact \
	cpu dotname emulator epilogue error export expr16 expr32 farstack flat \
	forceframe huge language large listing ljmp loadds m510 medium memory \
	nearstack nodotname noemulator nokeyword noljmp nom510 none nonunique \
	nooldmacros nooldstructs noreadonly noscoped nosignextend nothing \
	notpublic oldmacros oldstructs os_dos para private prologue radix \
	readonly req scoped setif2 smallstack tiny use16 use32 uses\
    a16 a32 o16 o32 byte word dword nosplit $ $$ seq wrt \
	flat large small .text .data .bss near far \
	%0 %1 %2 %3 %4 %5 %6 %7 %8 %9"),
    _T("addpd addps addsd addss andpd andps andnpd andnps \
	cmpeqpd cmpltpd cmplepd cmpunordpd cmpnepd cmpnltpd cmpnlepd cmpordpd \
	cmpeqps cmpltps cmpleps cmpunordps cmpneps cmpnltps cmpnleps cmpordps \
	cmpeqsd cmpltsd cmplesd cmpunordsd cmpnesd cmpnltsd cmpnlesd cmpordsd \
	cmpeqss cmpltss cmpless cmpunordss cmpness cmpnltss cmpnless cmpordss \
	comisd comiss cvtdq2pd cvtdq2ps cvtpd2dq cvtpd2pi cvtpd2ps \
	cvtpi2pd cvtpi2ps cvtps2dq cvtps2pd cvtps2pi cvtss2sd cvtss2si \
	cvtsd2si cvtsd2ss cvtsi2sd cvtsi2ss \
	cvttpd2dq cvttpd2pi cvttps2dq cvttps2pi cvttsd2si cvttss2si \
	divpd divps divsd divss fxrstor fxsave ldmxscr lfence mfence \
	maskmovdqu maskmovdq maxpd maxps paxsd maxss minpd minps minsd minss \
	movapd movaps movdq2q movdqa movdqu movhlps movhpd movhps movd movq \
	movlhps movlpd movlps movmskpd movmskps movntdq movnti movntpd movntps \
	movntq movq2dq movsd movss movupd movups mulpd mulps mulsd mulss \
	orpd orps packssdw packsswb packuswb paddb paddsb paddw paddsw \
	paddd paddsiw paddq paddusb paddusw pand pandn pause paveb pavgb pavgw \
	pavgusb pdistib pextrw pcmpeqb pcmpeqw pcmpeqd pcmpgtb pcmpgtw pcmpgtd \
	pf2id pf2iw pfacc pfadd pfcmpeq pfcmpge pfcmpgt pfmax pfmin pfmul \
	pmachriw pmaddwd pmagw pmaxsw pmaxub pminsw pminub pmovmskb \
	pmulhrwc pmulhriw pmulhrwa pmulhuw pmulhw pmullw pmuludq \
	pmvzb pmvnzb pmvlzb pmvgezb pfnacc pfpnacc por prefetch prefetchw \
	prefetchnta prefetcht0 prefetcht1 prefetcht2 pfrcp pfrcpit1 pfrcpit2 \
	pfrsqit1 pfrsqrt pfsub pfsubr pi2fd pf2iw pinsrw psadbw pshufd \
	pshufhw pshuflw pshufw psllw pslld psllq pslldq psraw psrad \
	psrlw psrld psrlq psrldq psubb psubw psubd psubq psubsb psubsw \
	psubusb psubusw psubsiw pswapd punpckhbw punpckhwd punpckhdq punpckhqdq \
	punpcklbw punpcklwd punpckldq punpcklqdq pxor rcpps rcpss \
	rsqrtps rsqrtss sfence shufpd shufps sqrtpd sqrtps sqrtsd sqrtss \
	stmxcsr subpd subps subsd subss ucomisd ucomiss \
	unpckhpd unpckhps unpcklpd unpcklps xorpd xorps"), 
    { true, false, false, true, true, false, true, true }, 15,  //! line, fold, wrap, indent, usetabs(good for makefiles), viewwhitespc
    2,
    { 
        wxSTC_ASM_DEFAULT, wxSTC_ASM_IDENTIFIER, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
    },
    {    //bold,italic,fore,back,face,size(pts)
        {  "Default Style", wxSTC_ASM_DEFAULT, "", "","000000","DADAC2","Tahoma","9"  },
        {  "Comment", wxSTC_ASM_COMMENT, "", "italic","6D6D6D","DADAC2","Tahoma","9"  },
        {  "Number", wxSTC_ASM_NUMBER, "bold", "","CE1C17","DADAC2","Tahoma","9"  },
        {  "String", wxSTC_ASM_STRING, "bold", "","109006","DADAC2","Tahoma","9"  },
        {   "Operator", wxSTC_ASM_OPERATOR, "bold", "","825002","DADAC2","Tahoma","9"  },
        {   "Identifier", wxSTC_ASM_IDENTIFIER, "bold", "","100030","DADAC2","Tahoma","9"  },
        {   "CPU Instruction", wxSTC_ASM_CPUINSTRUCTION, "", "","208060","DADAC2","Tahoma","9"  },
        {   "Math Instruction", wxSTC_ASM_MATHINSTRUCTION, "", "","508020","DADAC2","Tahoma","9"  },
        {   "Register", wxSTC_ASM_REGISTER, "", "","A05018","DADAC2","Tahoma","9"  },
        {   "Directive", wxSTC_ASM_DIRECTIVE, "", "","e1952d","DADAC2","Tahoma","9"  },
        {   "Directive Operand", wxSTC_ASM_DIRECTIVEOPERAND, "", "","9B007B","DADAC2","Tahoma","9"  },
        {   "CommentBlock", wxSTC_ASM_COMMENTBLOCK, "", "","5D5D5D","DADAC2","Tahoma","9"  },
        {   "Character", wxSTC_ASM_CHARACTER, "", "","4EC010","DADAC2","Tahoma","9"  },
        {   "String To EOL", wxSTC_ASM_STRINGEOL, "", "","000000","F0EA04","Verdana","9"  },
        {   "Ext. Instruction", wxSTC_ASM_EXTINSTRUCTION, "", "","9B009B","DADAC2","Tahoma","9" }
    }  //! End array of structures init
};

smallSyntax htmlSyntax = {
    "HTML",
    wxSTC_LEX_HTML,
    _T("html.api"), true, true,
    wxColour( 0xDA, 0xDA, 0xC2 ), ///Page
    wxColour( 0x20, 0x20, 0x20 ),  ///Line Fore
    wxColour( 0xC0, 0xC0, 0xA0 ),  ///Line Back
    wxColour( 0xEE, 0x78, 0x75 ),  ///Selection
    wxColour( 0x68, 0x76, 0x76 ),  ///Fold
    wxColour( 0x9f, 0x69, 0x34 ),  ///Indent
    4,
    _T("!doctype a abbr accept accept-charset accesskey acronym action address align alink alt applet \
archive asp:button asp:textbox asp:label asp:button axis b background base basefont bdo bgcolor bgsound big blink blockquote body border br \
button caption cellpadding cellspacing center char charoff charset checkbox checked cite class \
classid clear code codebase codetype col colgroup color cols colspan comment compact content \
coords data datafld dataformatas datapagesize datasrc datetime dd declare defer del dfn dir \
disabled div dl dt em embed enctype event face fieldset file font for form frame frameborder \
frameset h1 h2 h3 h4 h5 h6 head headers height hidden hr href hreflang hspace html http-equiv \
i id iframe image img input ins isindex ismap kbd label lang language layer leftmargin legend li \
link listing longdesc map marginheight marginwidth marquee maxlength media menu meta method \
multicol multiple name nobr noframes nohref noresize noscript noshade nowrap object ol onblur \
onchange onclick ondblclick onfocus onkeydown onkeypress onkeyup onload onmousedown \
onmousemove onmouseout onmouseover onmouseup onreset onselect onsubmit onunload \
optgroup option p param password plaintext pre profile prompt public q radio readonly rel \
reset rev rows rowspan rules runat samp scheme scope script select selected shape size small \
spacer span src standby start strike strong style sub submit summary sup tabindex table \
target tbody td text textarea tfoot th thead title topmargin tr tt type u ul usemap valign \
value valuetype var version vlink vspace wbr width "),
    ///JavaScript
    _T("abstract boolean break byte case catch char class \
const continue debugger default delete do double else enum export extends \
final finally float for function goto if implements import in instanceof \
int interface long native new package private protected public \
return short static super switch synchronized this throw throws \
transient try typeof var void volatile while with"),
    _T("addressof alias and as attribute base begin binary boolean byref byte byval call case compare \
const currency date decimal declare defbool defbyte defint deflng defcur defsng defdbl defdec \
defdate defstr defobj defvar dim do double each else elseif empty end enum eqv erase error \
event exit explicit false for friend function get gosub goto if imp implements in input integer \
is len let lib like load lock long loop lset me mid midb mod new next not nothing null object \
on option optional or paramarray preserve print private property public raiseevent randomize \
redim rem resume return rset seek select set single static step stop string sub then time to \
true type typeof unload until variant wend while with withevents xor"),
    ///Python
    _T("and assert break class continue def del elif \
else except exec finally for from global if import in is lambda None \
not or pass print raise return self try while yield"),
    //!PHP reserved words, core constants, types, OOP, functions
    _T("and or xor __file__ __line__ array as break case cfunction class const continue declare default die do echo else elseif empty enddeclare \
    endfor endforeach endif endswitch endwhile eval exit extends for foreach function global if include include_once isset list new old_function print \
    require require_once return static switch unset use var while __function__ __class__ php_version php_os default_include_path pear_install_dir \
    pear_extension_dir php_extension_dir php_bindir php_libdir php_datadir php_sysconfdir php_localstatedir php_config_file_path php_output_handler_start \
    php_output_handler_cont php_output_handler_end e_error e_warning e_parse e_notice e_core_error e_core_warning e_compile_error e_compile_warning \
    e_user_error e_user_warning e_user_notice e_all true false bool boolean int integer float double real string array object resource null class extends \
    parent stdclass directory __sleep __wakeup "),
    ///SGML
    _T("ELEMENT DOCTYPE ATTLIST ENTITY NOTATION"),
       { true, false, true, false, false, false, true, true },  110,   //! line, fold, wrap, indent, usetabs(good for makefiles), viewwhitespc
    16,  ///This is the number of call tip enabled styles below
    { 
        wxSTC_H_DEFAULT, wxSTC_HB_DEFAULT, wxSTC_HB_IDENTIFIER,  ///Client Side - don't allow Strings to do Call Tip
        wxSTC_HBA_DEFAULT, wxSTC_HBA_WORD, wxSTC_HBA_STRING, wxSTC_HBA_IDENTIFIER, wxSTC_HBA_STRINGEOL,
        wxSTC_HPHP_DEFAULT, wxSTC_HPHP_HSTRING, wxSTC_HPHP_SIMPLESTRING, wxSTC_HJ_WORD,
        wxSTC_H_TAG, wxSTC_H_TAGUNKNOWN, wxSTC_HJ_DEFAULT,  wxSTC_H_DOUBLESTRING
    },
   { //bold,italic,fore,back,face,size(pts)
      { "Default Style", wxSTC_H_DEFAULT,             "bold","","105072","DADAC2","Verdana","" },
      { "TAG", wxSTC_H_TAG,             "bold","","408000","DADAC2","Tahoma","" },
      { "TAGUNKNOWN", wxSTC_H_TAGUNKNOWN,             "bold","","704020","DADAC2","Tahoma","" },
      { "ATTRIBUTE", wxSTC_H_ATTRIBUTE,             "","","605010","DADAC2","Verdana","" },
      { "ATTRIBUTEUNKNOWN", wxSTC_H_ATTRIBUTEUNKNOWN,             "","","602010","DADAC2","Tahoma","" },
      { "NUMBER", wxSTC_H_NUMBER,             "","","B02010","DADAC2","Tahoma","" },
      { "DOUBLESTRING", wxSTC_H_DOUBLESTRING,             "","","507001","DADAC2","Tahoma","" },
      { "SINGLESTRING", wxSTC_H_SINGLESTRING,             "","","456060","DADAC2","Tahoma","" },
      { "OTHER", wxSTC_H_OTHER,             "","","2791AF","DADAC2","Tahoma","" },
      { "COMMENT", wxSTC_H_COMMENT,             "","","101410","CACAB6","Tahoma","" },
      { "ENTITY", wxSTC_H_ENTITY,             "","","502090","DADAC2","Tahoma","" },
      { "TAGEND", wxSTC_H_TAGEND,             "","","502090","DADAC2","Tahoma","" },
      { "XMLSTART", wxSTC_H_XMLSTART,             "","","502090","DADAC2","Tahoma","" },
      { "XMLEND", wxSTC_H_XMLEND,             "","","502090","DADAC2","Tahoma","" },
      { "SCRIPT", wxSTC_H_SCRIPT,             "bold","","709001","DADAC2","Tahoma","" },
      { "ASP", wxSTC_H_ASP,             "","","502090","DADAC2","Tahoma","" },
      { "ASPAT", wxSTC_H_ASPAT,             "","","502090","DADAC2","Tahoma","" },
      { "CDATA", wxSTC_H_CDATA,             "","","502090","DADAC2","Tahoma","" },
      { "QUESTION", wxSTC_H_QUESTION,             "bold","","A02080","DADAC2","Tahoma","" },
      { "VALUE", wxSTC_H_VALUE,             "bold","","E020D0","DADAC2","Tahoma","" },
      { "XCCOMMENT", wxSTC_H_XCCOMMENT,             "bold","","E020D0","DADAC2","Tahoma","" },
      { "SGML_DEFAULT", wxSTC_H_SGML_DEFAULT,             "bold","","E020D0","DADAC2","Tahoma","" },
      { "SGML_COMMAND", wxSTC_H_SGML_COMMAND,             "bold","","E020D0","DADAC2","Tahoma","" },
      { "SGML_1ST_PARAM", wxSTC_H_SGML_1ST_PARAM,             "bold","","E020D0","DADAC2","Tahoma","" },
      { "SGML_DOUBLESTRING", wxSTC_H_SGML_DOUBLESTRING,             "bold","","E020D0","DADAC2","Tahoma","" },
      { "SGML_SIMPLESTRING", wxSTC_H_SGML_SIMPLESTRING,             "bold","","E020D0","DADAC2","Tahoma","" },
      { "SGML_ERROR", wxSTC_H_SGML_ERROR,             "bold","","E020D0","DADAC2","Tahoma","" },
      { "SGML_SPECIAL", wxSTC_H_SGML_SPECIAL,             "bold","","E020D0","DADAC2","Tahoma","" },
      { "SGML_ENTITY", wxSTC_H_SGML_ENTITY,             "bold","","E020D0","DADAC2","Tahoma","" },
      { "SGML_COMMENT", wxSTC_H_SGML_COMMENT,             "bold","","E020D0","DADAC2","Tahoma","" },
      { "SGML_1ST_PARAM_COMMENT", wxSTC_H_SGML_1ST_PARAM_COMMENT,             "bold","","E020D0","DADAC2","Tahoma","" },
      { "SGML_BLOCK_DEFAULT", wxSTC_H_SGML_BLOCK_DEFAULT,             "bold","","E020D0","DADAC2","Tahoma","" },
      //!JavaScript
      { "JavaScript_START", wxSTC_HJ_START,             "","","502090","DADAC2","Tahoma","" },
      { "JavaScript_DEFAULT", wxSTC_HJ_DEFAULT,             "","","502090","DADAC2","Tahoma","" },
      { "JavaScript_COMMENT", wxSTC_HJ_COMMENT,            "", "","6D6D6D","DADAC2","Arial",""  },
      { "JavaScript_COMMENTLINE", wxSTC_HJ_COMMENTLINE,           "", "italic","5D5D5D","DADAC2","Times New Roman","10"  },
      { "JavaScript_COMMENTDOC", wxSTC_HJ_COMMENTDOC,            "", "","000000","CACAB2","Times New Roman",""  },
      { "JavaScript_NUMBER", wxSTC_HJ_NUMBER,           "bold", "","CE1C17","DADAC2","Tahoma",""  },
      { "JavaScript_WORD", wxSTC_HJ_WORD,            "bold", "","8f010c","DADAC2","Tahoma","" },
      { "JavaScript_KEYWORD", wxSTC_HJ_KEYWORD,             "","","5020D0","DADAC2","Tahoma","" },
      { "JavaScript_DOUBLESTRING", wxSTC_HJ_DOUBLESTRING,               "bold", "","109006","DADAC2","Verdana","8"  },
      { "JavaScript_SINGLESTRING", wxSTC_HJ_SINGLESTRING,             "","","508000","DADAC2","Tahoma","" },
      { "JavaScript_SYMBOLS", wxSTC_HJ_SYMBOLS,             "","","B46000","DADAC2","Tahoma","" },
      { "JavaScript_STRINGEOL", wxSTC_HJ_STRINGEOL,           "bold", "","000000","F0EA04","Verdana",""  },
      { "JavaScript_REGEX", wxSTC_HJ_REGEX,             "","","502020","DADAC2","Tahoma","" },
        //!ASP Javascript
      { "ASP Javascript Start", wxSTC_HJA_START,             "","","502090","DADAC2","Tahoma","" },
      { "ASP Javascript Default", wxSTC_HJA_DEFAULT,             "","","502090","DADAC2","Tahoma","" },
      { "ASP Javascript Comment", wxSTC_HJA_COMMENT,            "", "","6D6D6D","DADAC2","Arial",""  },
      { "ASP Javascript CommentLine", wxSTC_HJA_COMMENTLINE,           "", "italic","5D5D5D","DADAC2","Times New Roman","10"  },
      { "ASP Javascript CommentDoc", wxSTC_HJA_COMMENTDOC,            "", "","000000","CACAB2","Times New Roman",""  },
      { "ASP Javascript Number", wxSTC_HJA_NUMBER,           "bold", "","CE1C17","DADAC2","Tahoma",""  },
      { "ASP Javascript Word", wxSTC_HJA_WORD,            "bold", "","8f010c","DADAC2","Tahoma","" },
      { "ASP Javascript Keyword", wxSTC_HJA_KEYWORD,             "","","5020D0","DADAC2","Tahoma","" },
      { "ASP Javascript DoubleString", wxSTC_HJA_DOUBLESTRING,               "bold", "","109006","DADAC2","Verdana","8"  },
      { "ASP Javascript SingleString", wxSTC_HJA_SINGLESTRING,             "","","508000","DADAC2","Tahoma","" },
      { "ASP Javascript Symbols", wxSTC_HJA_SYMBOLS,             "","","B46000","DADAC2","Tahoma","" },
      { "ASP Javascript StringToEOL", wxSTC_HJA_STRINGEOL,           "bold", "","000000","F0EA04","Verdana",""  },
      { "ASP Javascript RegularEx", wxSTC_HJA_REGEX,             "","","502020","DADAC2","Tahoma","" },
        //!Embedded VBScript
      { "Embedded VB Start", wxSTC_HB_START,             "","","502090","DADAC2","Tahoma","" },
      { "Embedded VB Default", wxSTC_HB_DEFAULT,             "","","502090","DADAC2","Tahoma","" },
      { "Embedded VB CommentLine", wxSTC_HB_COMMENTLINE,             "","","502090","DADAC2","Tahoma","" },
      { "Embedded VB Number", wxSTC_HB_NUMBER,             "","","502090","DADAC2","Tahoma","" },
      { "Embedded VB Word", wxSTC_HB_WORD,             "","","502090","DADAC2","Tahoma","" },
      { "Embedded VB String", wxSTC_HB_STRING,             "","","502090","DADAC2","Tahoma","" },
      { "Embedded VB Identifier", wxSTC_HB_IDENTIFIER,             "","","502090","DADAC2","Tahoma","" },
      { "Embedded VB StringToEOL", wxSTC_HB_STRINGEOL,             "","","502090","DADAC2","Tahoma","" },
        //!ASP VBScript
      { "ASP VB Start", wxSTC_HBA_START,             "","","502090","DADAC2","Tahoma","" },
      { "ASP VB Default", wxSTC_HBA_DEFAULT,             "","","502090","DADAC2","Tahoma","" },
      { "ASP VB CommentLine", wxSTC_HBA_COMMENTLINE,             "","","502090","DADAC2","Tahoma","" },
      { "ASP VB Number", wxSTC_HBA_NUMBER,             "","","502090","DADAC2","Tahoma","" },
      { "ASP VB Word", wxSTC_HBA_WORD,             "","","502090","DADAC2","Tahoma","" },
      { "ASP VB String", wxSTC_HBA_STRING,             "","","502090","DADAC2","Tahoma","" },
      { "ASP VB Identifier", wxSTC_HBA_IDENTIFIER,             "","","502090","DADAC2","Tahoma","" },
      { "ASP VB StringToEOL", wxSTC_HBA_STRINGEOL,             "","","502090","DADAC2","Tahoma","" },
        //!Embedded Python
      { "Embedded Python Start", wxSTC_HP_START,             "","","502090","DADAC2","Tahoma","" },
      { "Embedded Python Default", wxSTC_HP_DEFAULT,             "","","502090","DADAC2","Tahoma","" },
      { "Embedded Python CommentLine", wxSTC_HP_COMMENTLINE,             "","","502090","DADAC2","Tahoma","" },
      { "Embedded Python Number", wxSTC_HP_NUMBER,             "","","502090","DADAC2","Tahoma","" },
      { "Embedded Python String", wxSTC_HP_STRING,             "","","502090","DADAC2","Tahoma","" },
      { "Embedded Python Character", wxSTC_HP_CHARACTER,             "","","502090","DADAC2","Tahoma","" },
      { "Embedded Python Word", wxSTC_HP_WORD,             "","","502090","DADAC2","Tahoma","" },
      { "Embedded Python Triple", wxSTC_HP_TRIPLE,             "","","502090","DADAC2","Tahoma","" },
      { "Embedded Python TripleDouble", wxSTC_HP_TRIPLEDOUBLE,             "","","502090","DADAC2","Tahoma","" },
      { "Embedded Python Classname", wxSTC_HP_CLASSNAME,             "","","502090","DADAC2","Tahoma","" },
      { "Embedded Python DefName", wxSTC_HP_DEFNAME,             "","","502090","DADAC2","Tahoma","" },
      { "Embedded Python Operator", wxSTC_HP_OPERATOR,             "","","502090","DADAC2","Tahoma","" },
      { "Embedded Python Identifier", wxSTC_HP_IDENTIFIER,             "","","502090","DADAC2","Tahoma","" },
        //!ASP Python
      { "ASP Python Start", wxSTC_HPA_START,             "","","502090","DADAC2","Tahoma","" },
      { "ASP Python Default", wxSTC_HPA_DEFAULT,             "","","502090","DADAC2","Tahoma","" },
      { "ASP Python CommentLine", wxSTC_HPA_COMMENTLINE,             "","","502090","DADAC2","Tahoma","" },
      { "ASP Python Number", wxSTC_HPA_NUMBER,             "","","502090","DADAC2","Tahoma","" },
      { "ASP Python String", wxSTC_HPA_STRING,             "","","502090","DADAC2","Tahoma","" },
      { "ASP Python Character", wxSTC_HPA_CHARACTER,             "","","502090","DADAC2","Tahoma","" },
      { "ASP Python Word", wxSTC_HPA_WORD,             "","","502090","DADAC2","Tahoma","" },
      { "ASP Python Triple", wxSTC_HPA_TRIPLE,             "","","502090","DADAC2","Tahoma","" },
      { "ASP Python TripleDouble", wxSTC_HPA_TRIPLEDOUBLE,             "","","502090","DADAC2","Tahoma","" },
      { "ASP Python Classname", wxSTC_HPA_CLASSNAME,             "","","502090","DADAC2","Tahoma","" },
      { "ASP Python DefName", wxSTC_HPA_DEFNAME,             "","","502090","DADAC2","Tahoma","" },
      { "ASP Python Operator", wxSTC_HPA_OPERATOR,             "","","502090","DADAC2","Tahoma","" },
      { "ASP Python Identifier", wxSTC_HPA_IDENTIFIER,             "","","502090","DADAC2","Tahoma","" },
        //!PHP
      { "PHP_DEFAULT", wxSTC_HPHP_DEFAULT,             "","","000040","DADAC2","Tahoma","" },
      { "PHP_HSTRING", wxSTC_HPHP_HSTRING,             "","","00A020","DADAC2","Tahoma","" },
      { "PHP_SIMPLESTRING", wxSTC_HPHP_SIMPLESTRING,   "bold", "","109006","DADAC2","Verdana","8"  },  
      { "PHP_WORD", wxSTC_HPHP_WORD,             "bold","","80080","DADAC2","Tahoma","" },
      { "PHP_NUMBER", wxSTC_HPHP_NUMBER,             "bold","","D06020","DADAC2","Tahoma","" },
      { "PHP_VARIABLE", wxSTC_HPHP_VARIABLE,             "bold","","304410","DADAC2","Tahoma","" },
      { "PHP_COMMENT", wxSTC_HPHP_COMMENT,             "","","000040","D0D0BE","Times New Roman","" },
      { "PHP_COMMENTLINE", wxSTC_HPHP_COMMENTLINE,             "","italic","606060","DADAC2","Times New Roman","10" },
      { "PHP_HSTRING_VARIABLE", wxSTC_HPHP_HSTRING_VARIABLE,             "","","000050","DADAC2","Tahoma","" },
      { "PHP_OPERATOR", wxSTC_HPHP_OPERATOR,             "bold","","A050AF","DADAC2","Tahoma","" }
    }  //! End array of structures init
};

wxString g_cppWords2;
wxString g_javaWords2;
wxString g_csWords2;
wxString g_sqlWords2;

///The ORDER here is critical
/// As listed in the ExtPanel modes listbox (SettingsDialog.cpp). Order must match smallSyntaxes[]
smallSyntax * smallSyntaxes[] = { &defaultSyntax, &pythonSyntax, &cppSyntax, &htmlSyntax,
        &perlSyntax, &javaSyntax, &csSyntax, &makeSyntax, &batchSyntax , &cssSyntax, &lispSyntax,
        &rubySyntax, &sqlSyntax, &fortranSyntax, &f77Syntax, &asmSyntax, &vbscriptSyntax };
        
//No need for this array since mode listbox (ExtPanel) indexes are as ordered in smallSyntaxes, so get this info directly from smallSyntaxes //int lexerNum[] = { wxSTC_LEX_NULL, wxSTC_LEX_PYTHON, wxSTC_LEX_CPP, wxSTC_LEX_HTML, wxSTC_LEX_PERL,
//        wxSTC_LEX_MAKEFILE, wxSTC_LEX_BATCH, wxSTC_LEX_LISP, wxSTC_LEX_RUBY};
        
stringToInt ext2lex;  ///define an extension to lexer hash map

SyntaxStructs syntaxStructs;  ///Define our syntax structures hashmap
//syntaxStructs[ wxSTC_LEX_NULL ] = &defaultSyntax;
//syntaxStructs[wxSTC_LEX_VBSCRIPT] = &vbscriptSyntax;


///Notebook page to lexer #. So we can pre-select a notebook page based on the current lexer.
//int lenPg2lex = NUMSECTIONS;
///Order of Notebook pages in SettingsDialog!
int pg2lex[ ] = { wxSTC_LEX_NULL, wxSTC_LEX_CPP, wxSTC_LEX_PERL, wxSTC_LEX_RUBY,
            wxSTC_LEX_HTML, wxSTC_LEX_PYTHON, wxSTC_LEX_JAVA, wxSTC_LEX_CS, wxSTC_LEX_SQL,
            wxSTC_LEX_CSS, wxSTC_LEX_LISP, wxSTC_LEX_F77, wxSTC_LEX_FORTRAN,
            wxSTC_LEX_MAKEFILE, wxSTC_LEX_BATCH, wxSTC_LEX_ASM, wxSTC_LEX_VBSCRIPT }; 

int lenLNFN =  38;  ///MUST BE UPDATED WITH ADDITION OF NEW LEXER. //Size of would be nice       
wxString lexerNameFromNum[] = {_T("Container"), _T("Plain Text"),_T("PYTHON"),_T("C++"),_T("HTML"),
        _T("XML"), _T("PERL"), _T("SQL"), _T("VB"), _T("Properties"), _T("ErrorList"), _T("MAKEFILE"),
        _T("BATCH"), _T("JAVA"), _T("Latex"), _T("C#"), _T("Diff"), _T("Conf"), _T("Pascal"), ///borrowed Lua for C#
        _T("Ave"), _T("Ada"), _T("LISP"), _T("RUBY"), _T("Eiffel"), _T("EIFFELKW"), _T("TCL"), _T("NNCRONTAB"), _T("BULLANT"),
        _T("VB"), _T("N/A"), _T("N/A"), _T("BAAN"), _T(""), _T(""), _T("ASM"), _T("CPPNOCASE"), _T("FORTRAN9X"), _T("FORTRAN77"), _T("CSS")  };
