/*! Our syntax file */
#ifndef SYNTAX_H
#define SYNTAX_H

#define NUMSECTIONS 17
#define wxSTC_LEX_JAVA 13  ///We borrow the lexer number for XCODE which we would never use
#define wxSTC_LEX_CS 15  ///We borrow the lexer number for LUA which we would never use

struct styleAttr{
    char * item;
    int styleNum;
    char bold[5]; //! accomodate "bold" = 4+ NUL
    char italic[7];
    char fore[7];
    char back[7];
    char face[32];
    char size[4]; // 2+NUL
};

struct options{
    bool line; //! line num visible?
    bool fold;  //! Show fold margin
    bool wrap;  //! Wrap lines
    bool indent;  //! Show indentation guides
    bool usetabs; //! Use actual tabs instead of spaces for tabs
    bool viewws; //! View whitespace
    bool horizSb; /// Show horiz scrollbar
    bool vertSb; /// Show vert scrollbar
};

struct smallSyntax {
    wxChar *name;
    int lexer;
    const wxChar * callTipFile;
    bool batchLike;  ///Can spaces activate a function
    bool cLike;  ///Can parens "( )" activate a function
    wxColour pageColor;
    wxColour lineForeColor;
    wxColour lineBackColor;
    wxColour selColor;
    wxColour foldColor;
    wxColour indentColor;
    long tabWidth;
    const wxChar *words1;  ///I think keywords must be lowercase
    const wxChar *words2;
    const wxChar *words3;
    const wxChar *words4;
    const wxChar *words5;
    const wxChar *words6;
    options option;
    int stylesUsed; //! Length of the styles array
    int ctEnStyCnt;
    int ctEnStyles[16];  ///Style nums that are call tip, objMem Comp, enabled. Max 16
   styleAttr styles[];  ///Because this is dynamic it must be at the very END. No other dynamic arrays allowed before it
};

//! These let the other modules know that the definition is in another module, wait for it at link time
extern smallSyntax pythonSyntax;
extern smallSyntax cppSyntax;
extern smallSyntax perlSyntax;
extern smallSyntax makeSyntax;
extern smallSyntax defaultSyntax;
extern smallSyntax htmlSyntax;
extern smallSyntax javaSyntax;
extern smallSyntax csSyntax;
extern smallSyntax lispSyntax;
extern smallSyntax rubySyntax;
extern smallSyntax cssSyntax;
extern smallSyntax sqlSyntax;
extern smallSyntax batchSyntax;
extern smallSyntax fortranSyntax;
extern smallSyntax f77Syntax;
extern smallSyntax asmSyntax;
extern smallSyntax vbscriptSyntax;

extern const wxChar * F1Words;
extern const wxChar * F2Words;
extern const wxChar * F3Words;

extern const wxChar * CSSWords;

extern smallSyntax *  smallSyntaxes [];

extern wxString lexerNameFromNum[];
extern int lenLNFN; ///length for lexerNameFromNum
///Map extensions to lexer int
WX_DECLARE_STRING_HASH_MAP( int, stringToInt );  ///string key, int values
extern stringToInt ext2lex;

///Lexer number to smallSyntaxStruct Hashmap
WX_DECLARE_HASH_MAP( int, smallSyntax*, wxIntegerHash, wxIntegerEqual, SyntaxStructs );
extern SyntaxStructs syntaxStructs;  ///Actual assignments/initial values, must be set inside of a running function
        ///Case here: ZionEdit::ZionEdit()

extern int pg2lex[];
//extern int lenPg2lex;

extern wxString g_cppWords2;
extern wxString g_javaWords2;
extern wxString g_csWords2;
extern wxString g_sqlWords2;
extern wxColour g_caretColor;
extern wxColour g_selColor;
extern wxColour g_foldColor;

#endif