#ifndef ZION_H
#define ZION_H
//! Event IDs located in globaldefs.h

class ZionApp : public wxApp
{  
    friend class ZionFrame; //needed?
    public:
        virtual bool OnInit();
        ZionFrame * m_zionFrame;
};

//! Setup the application class
DECLARE_APP (ZionApp); //!not previously used in first project

/*! Ascertain our available fonts */
class FontEnumr: public wxFontEnumerator    //! Store our system fonts
{
    public:
        bool OnFacename( const wxString & face ) 
        {   g_faceNames.Add( face ); return true;  }
};

/// Class for our traverser object. Object will be passed to wxDir::Traverse() 
class DirTravSimple : public wxDirTraverser
{
    public:
        DirTravSimple(wxArrayString& files) : m_files(files) { } ///Init our arrayString ref. with a wxArrayString
        virtual wxDirTraverseResult OnFile(const wxString& filename) {
            m_files.Add(filename);
            return wxDIR_CONTINUE;
        }
        virtual wxDirTraverseResult OnDir(const wxString& WXUNUSED(dirname)) {
            return wxDIR_CONTINUE;
        }
    private:
        wxArrayString& m_files;
};

extern wxColour StringToColor(const wxString & );
extern unsigned long localPid;

struct StartEnd {
    int start;  int end;
};

//! The Application MainWindow
class MyServer; class MyClient;
class ZionFrame : public wxFrame
{
    friend class ZionEdit;
    public:
        ZionFrame( const wxString & arg0, const wxString & arg1, const wxChar *title,
                 int xpos, /*int ypos, int width, int height, */ const wxString & pathToIni, long ourStyle); ///xpos = calling frames xposition
        ~ZionFrame();
        class ZionEdit * m_zedit; //! Link to the wxStyledTextCtrl Class
        
        wxSingleInstanceChecker * m_singleInstanceChecker;
        bool m_isServer;
        bool m_ipcOk;
        bool m_iniDiff;
        bool m_styleLock;
        wxString m_zExecStr; ///Holder for parsed execStr (server only)
        int m_startOpt;  /// 0 - half max, 1 - maximized, 2- custom
        wxPoint m_startPos; /// used with custom
        wxSize m_startSize;  ///used with custom and normal
        int m_lineHits;
 
        ///Some Edit Module Settings(TODO: check validity
        int m_startEol;
        int m_printColMode;
        int m_tabWidth;
        bool m_htmlAC; bool m_htmlBR;
        int m_startLex;
        
        wxString MiniStats();
        void DoExecSubProcess( const wxString & );
        
        void OnPrint(wxCommandEvent &);
        void OnPrintSetup(wxCommandEvent &);
        void OnPrintPreview(wxCommandEvent &);
        wxRect ZionFrame::DeterminePrintSize();
        
        void OnOpenFile (wxCommandEvent & event);
        void OnExit     (wxCommandEvent & event);
        void OnSave		(wxCommandEvent &);
        void OnSaveAs	(wxCommandEvent &);
        void OnClose	(wxCloseEvent & event);
        void OnExternalRaise (wxCommandEvent & );  
        void OnExternalClose (wxCommandEvent & );  
        void OnReadIni (wxCommandEvent & );  
        void OnWriteIni (wxCommandEvent & );  
        void OnChooseColour(wxCommandEvent & event);
        void OnAbout   (wxCommandEvent &);
        void OnZionEdit   (wxCommandEvent &);
        void OnZionEditPending( wxCommandEvent & );
        wxString GetIniDir() { return m_iniDir; }
        bool ReadIni( int readFlags, bool init = false );  ///Only on init will we allow creating "history.ini"
        bool WriteIni();
        bool PushDir( const wxString & currentDir, bool localPush = false);
        bool PushFile( const wxString & currentFile, bool localPush = false);
        bool PushExecStr( const wxString & execStr, bool localPush = false);
        void RemoveSubmenu( const wxString& );
        bool PushPrint(bool localPush = false);
        wxString PickupExecStr();
        wxString GetServerStatus();
        wxString GetStyleLock(const wxString & ztopic, bool local = false);
        void NotifyReadPrint(); ///Notify the server
        void NotifyFindUpdate(); ///Notify the server
        void NotifyReplUpdate(); ///Notify the server
        void NotifyRaise( const wxString& pid ); ///Notify server or initiate server action
        void AdviseRaise(wxCommandEvent &); ///Advise the clients
        void NotifyClose( long pid ); ///Notify server or initiate server action
        void AdviseClose(wxCommandEvent &); ///Advise the clients
        void AdviseFindUpdate(wxCommandEvent &); ///Advise the clients
        void AdviseReplUpdate(wxCommandEvent &); ///Advise the clients
        void AdviseReadPrint(wxCommandEvent &); ///Advise the clients
        void NotifyReadStyles(wxCommandEvent &);
        void AdviseReadStyles(wxCommandEvent &); //const wxString & ztopic, bool local = false
        bool RequestStyleLock();
        void OnBecomeServer(wxCommandEvent &);
        void OnReconn(wxCommandEvent &);
        bool DoReconn();
        void OnAssignPriorities(wxCommandEvent &);
        bool CheckFileInUse(wxString & str, bool remoteOnly = false);
        void OnIPCTimer(wxTimerEvent & event);
        wxMenuBar *m_menuBar; ///creates the application menu bar
        wxMenu      *fileMenu;
        wxMenu      *editMenu;
        wxMenu      *lineSubMenu;
        wxMenu      *editSpecial;
        wxMenu      *syntaxMenu;
        wxMenu      *optionsMenu;
        wxMenu      *helpMenu;
        wxMenu      *newSubMenu;
        wxMenu      *newZionSubMenu;
        wxMenu      *newHTMLSub;
        wxMenu      *ipcMenu;
        wxMenu      *gotoMenu;
        wxMenu      *togglesMenu;
        wxMenuItem * togglesHorizSb;  ///So we can disable when wrapping on
        ///MODE SPECIFIC MENUS
        wxMenu      *htmlMenu;
        wxMenu      *htmlTable;
        wxMenu      *htmlHeading;
        wxMenu      *htmlList;
        wxMenu      *htmlStyle;
        wxMenu      *cppMenu;
        ///Java and CSharp will actually borrow the cppMenu //wxMenu      *javaMenu;
        wxMenu      *perlMenu;
        wxMenu      *rubyMenu;
        wxMenu      *vbscriptMenu;
        wxMenu      *makeMenu;
        wxMenu      *sqlMenu;
        wxMenu      *pythonMenu;
        wxMenu      *asmMenu;
        wxMenu      *cssMenu;
        wxMenu      *batchMenu;
        wxMenu      *lispMenu;
        wxMenu      *fortranMenu;
        wxMenu      *fortran77Menu;
        
        wxMenu      *recentFilesMenu;
        wxMenu      *recentDirsMenu;
        wxMenu * dirSubs[NUMRECENTDIRS];
        wxMenuItem * dirSubItems[NUMRECENTDIRS];
        
        wxString  m_advisedData; ///Used by FindUpdate
        wxString  m_advisedReplData; ///Used by ReplUpdate
        wxString m_pidToClose; ///Window to be closed
        wxString m_pidToRaise; ///Window to be raised

	private:
        bool m_extClose;
        long m_zoomLevel; ///Numeric representation of initial zoom
        bool m_autoZoom; ///Per session autoZoom enable
        bool m_autoBook;  ///Per session autoBookmarks enable
        wxString    m_cppWord2;
        wxString    m_javaWord2;
        wxString    m_sqlWord2;
        wxString    m_csWord2;
        bool m_dontPrintNums;
        wxProcess * m_execProcess;
        long m_execLastPid;  ///Pid of last process executed
        wxString m_execStr;
        wxString m_lastExecStr;
        wxArrayString m_execHistory;
        bool m_execAppend;   ///Append to this window or exec in new window?
        bool m_captureOut;   ///Capture output or not
        bool m_runWithCmd;   ///Run with cmd or not
        wxString m_lastWdir;  ///Remember last working directory
        wxMessageDialog * m_execNotify;
        
        void GetOutErrStream( wxInputStream * );
        bool ReadIfHasInput();
        
        wxMenuItem * m_recentPopup;
        void PrependToPopup( int );  ///Save the current menu item into the popup menu
        
        void OnBookmarkTimer( wxTimerEvent & event );
        bool DoWinLocDlg( wxString& str );///bool was the dialog OK'd or Cancel
                ///wxString = Empty - This window chosen; wxString = NonEmpty - New window chosen
        bool FindSection( wxTextFile&, const wxString&, StartEnd&, bool createIt = false );
        void OnFullScrnToggle( wxCommandEvent & );
        void OnClearStatus( wxCommandEvent & );
        void OnGoogleIt( wxCommandEvent & );
        void OnShowCallTip( wxCommandEvent & );
        void OnExecSubProcess(wxCommandEvent &);
        void OnEndSubProcess(wxProcessEvent &);
        void OnDropFiles(wxDropFilesEvent &);
        void OnInfo(wxCommandEvent & event);
        void OnWindow(wxCommandEvent & event);
        void OnNewZion(wxCommandEvent & event);
        void OnHtmlACToggle( wxCommandEvent & event );
        void OnHtmlBRToggle( wxCommandEvent & event );
        void OnRecentDir(wxCommandEvent & event);
        void OnCmdPrompt(wxCommandEvent & event);
        void OnOnlineDocs(wxCommandEvent & event);
        void OnCheckLatest(wxCommandEvent & event);
        void OnBugReport(wxCommandEvent & event);
        void OnHtmlPreview(wxCommandEvent & event);
        void OnExplore(wxCommandEvent & event);
        void OnRecentFile(wxCommandEvent & event);
        void OnRecentFileItem(wxCommandEvent & event);
        void OnUIReconn( wxUpdateUIEvent & event );
        void OnUIBecomeServer( wxUpdateUIEvent & event );
        void OnUIPaste( wxUpdateUIEvent & event );
        void OnUICut( wxUpdateUIEvent & event );
        void OnUIUndo( wxUpdateUIEvent & event );
        void OnUIRedo( wxUpdateUIEvent & event );
        void OnUICopy( wxUpdateUIEvent & event );
        void OnUILineToggle( wxUpdateUIEvent & event );
        void OnUIHtmlACToggle( wxUpdateUIEvent & event );
        void OnUIHtmlBRToggle( wxUpdateUIEvent & event );
        void OnUIFoldToggle( wxUpdateUIEvent & event );
        void OnUIWhiteSpaceToggle( wxUpdateUIEvent & event );
        void OnUIHorizSbToggle( wxUpdateUIEvent & event );
        void OnUIVertSbToggle( wxUpdateUIEvent & event );
        void OnUIWrapToggle( wxUpdateUIEvent & event );
        void OnUIIndentToggle( wxUpdateUIEvent & event );
        void OnUIUseTabsToggle( wxUpdateUIEvent & event );
        void OnUICallTipsToggle( wxUpdateUIEvent & event );
        void OnUICTACToggle( wxUpdateUIEvent & event );
        void OnUIReadOnlyToggle( wxUpdateUIEvent & event );
        void OnUIAutoCompToggle( wxUpdateUIEvent & event );
        void OnUIAutoZoomToggle( wxUpdateUIEvent & event );
        void OnUIAutoBookToggle( wxUpdateUIEvent & event );
        void OnUIMode( wxUpdateUIEvent & event );
        void OnUIEOL( wxUpdateUIEvent & event );
        void OnCallTipsToggle( wxCommandEvent & event );
        void OnCTACToggle( wxCommandEvent & event );
        void OnAutoCompToggle( wxCommandEvent & event );
        void OnAutoZoomToggle( wxCommandEvent & event );
        void OnAutoBookToggle( wxCommandEvent & event );
        void CreateMenu();
        int ServerOrClient();
        bool CheckForIni();
        inline void LoadDefaultExtMap();
        void OnHelp(wxCommandEvent &);
        void OnEditIni(wxCommandEvent &);
        void OnShowPriority(wxCommandEvent &);
        void OnShowIPCStatus(wxCommandEvent &);
        void OnShowConns(wxCommandEvent &);
        void OnMenuReconn(wxCommandEvent &);
        void OnAdviseReadHistory(wxCommandEvent &);
        wxString m_iniDir;    //! For ini location
        wxString m_exeDir;      //! For exe location
        bool  m_FailedZionIni;
        wxTimer m_timer;
        wxTimer m_bookTimer;
    //! This class will process events
	DECLARE_EVENT_TABLE()
};

class ServerConn : public wxConnection
{
    public:
        ServerConn(const wxString& ztopic);
        ~ServerConn();
        bool OnExecute(const wxString& topic, wxChar *data, int size, wxIPCFormat format);
        wxChar *OnRequest(const wxString& topic, const wxString& item, int *size, wxIPCFormat format);
        bool OnPoke(const wxString& topic, const wxString& item, wxChar *data, int size, wxIPCFormat format);
        bool OnStartAdvise(const wxString& topic, const wxString& item);
        wxString currentFile;
        bool styleLock;
        bool m_srcRequestFile;  /// are we the source of a "File" request?
        bool m_srcExecStyle;  /// are we the source of a Style update?
        bool m_srcPokeFindUpdate;
        bool m_srcPokeReplUpdate;
        bool m_closing;
    private:
        wxString m_ztopic;
};

class MyServer: public wxServer
{
    public:
        wxConnectionBase *OnAcceptConnection(const wxString& topic);
};

//! The custom wxConnection class will intercept communications in a 'conversation' with a server
class ClientConn: public wxConnection
{
    public:
        bool OnAdvise(const wxString& topic, const wxString& item,
                wxChar *data, int size, wxIPCFormat format);
        bool OnDisconnect();
};
//! the custom wxClient is required so that a user-overridden wxClient::OnMakeConnection member can return a wxConnection of the required class, when a connection is made.
class MyClient: public wxClient
{
    public:
        wxConnectionBase *OnMakeConnection();
};

#endif        