// AUTHOR: Rohan Allison
// COPYRIGHT: 2006
//  License: Artistic

#ifndef _ZIONEDIT_H_
#define _ZIONEDIT_H_

class ClientConn;
extern ClientConn * zconnection;
class MyServer;
extern MyServer * zserver;

extern int g_numTopRecentFiles; ///Displayed in top level of menu
extern int g_numRecentDirs;  ///Displayed

extern wxPoint zion_contextMenuPos;

static const wxChar *FILETYPES =
	_T(	
        "All files|*.*|"
        "C/C++ source files (*.cpp,*.c,*.cxx,*.cc)|*.cpp;*.c;*.cxx;*.cc|"
        "C/C++ header files (*.h,*.hpp)|*.h;*.hpp|"
        "Python files (*.py)|*.py|"
        "Perl files (*.pl,*.perl)|*.pl;*perl|"
        "Ruby files (*.rb,*.ruby)|*.rb;*ruby|"
        "VB files (*.vb,*.vbs)|*.vb;*vbs|"
        "SQL files (*.sql)|*.sql|"
        "CSS files (*.css)|*.css|"
        "asm files (*.asm,*.masm,*.nasm,*.fasm,*.tasm)|*.asm;*.masm;*.nasm;*.fasm;*.tasm|"
        "Lisp files (*.lisp)|*.lisp|"
        "Java files (*.java)|*.java|"
        "C# files (*.cs)|*.cs|"
        "Make files (Mak*,mak*,*.mak)|Mak*;mak*;*.mak|"
        "Hypertext markup files (*.html,*.htm,*.sgml)|*.html;*.htm;*.sgml|"
        "PHP files (*.php,*.PHP)|*.php;*.PHP|"
        "Text files (*.txt,*.ini,*.cfg)|*.txt;*.ini;*.cfg"
	  );
      
wxColour StringToColor( const wxString & );  //! Global declaration

WX_DECLARE_STRING_HASH_MAP( wxString, Str2StrHash );  ///Implementation in zionedit.cpp

///CALL TIPS - AUTOCOMPLETE; Call Tip triggered by "(" or " " (spc) in VB
///CALL TIPS DATA STORAGE CLASS
class MyCallTip{
    public:
        wxArrayString m_fullDefs;  ///Synched arrays
        wxArrayString m_descripts;  ///Synched arrays
};  ///Note: dynamic WxWidgets data types  manage themselves when declared as automatic and don't need a destructor to release memory.
///The unique function will be the key into the CallTipsHash
WX_DECLARE_STRING_HASH_MAP( MyCallTip * , CallTipsHash );  ///Implementation in zionedit.cpp

class MemberList{
    public:
        wxArrayString m_array;  ///Accuml here
        wxString m_list;  ///m_array is sorted and joined to this list (will become local AC list)
        Str2StrHash m_descrip; ///Member descriptions
};
WX_DECLARE_STRING_HASH_MAP( MemberList * , MemberLists );  ///Implementation in zionedit.cpp

///Notes: *On Calltip start hilite 1st param, seek first "(" to first comma
/// *On Comma char and call tip already active, seek to next comma
/// *On Call tip cancel, delete the object of this class.
class ActiveCallTip{  ///For current unique function
    public:
        ActiveCallTip( MyCallTip * ct, int line ) {
            m_ct = ct;
            if( ct != NULL ) m_numOvrlds = ct->m_fullDefs.GetCount();
            else m_numOvrlds = 0;
            m_line = line;
            m_ovrldIdx = 0; m_paramIdx = 0;
            m_startPos = 0; m_endPos = 0;
        };
        /** Note: Absolute index of current def in Full Def = m_defIdx + m_ovrldIdx **/
        MyCallTip* m_ct;  ///NULL  will indicate a disabled call tip
        int m_numOvrlds; ///Save locally for easier access //Needed
        int m_ovrldIdx; ///Index to current overload i.e. which overload are we on ///Should init to 0
        int m_paramIdx;  ///What param within the curr def are we pointing to
        wxArrayInt m_paramLocs;  ///Param locations;
            ///current ovrld scanned at CallTip creation and Overload switched.
        int m_startPos;  ///This makes things simpler when displaying the call tip
        int m_endPos;  ///Pos just after last char of current param (end calltip hilite pos)///On comma entered, will seek ahead to next param
        int m_line;  ///Will tell us if our line Position changed, then cancel ALL active call tips
};

class HtmlAutoComp {
  public:
    HtmlAutoComp() { };
//        m_htmlAC.active = true; ///hardwired on for now
//        m_htmlAC.brActive = false;
//        m_htmlAC.inTag = false;  m_htmlAC.tagWaiting = false;
//    };
    void Init() { ///Where are we using Init?
        active = true;  brActive = false; tag = wxEmptyString;
        inTag = false;  tagWaiting = false; line = 0; idTrigger = false;
    };
    void ResetTag() {
        inTag = false;  tagWaiting = false;
        tag = wxEmptyString;
        idTrigger = false;
    }
    //void NewTag(int currLine);  ///Establish a new tag context - use only after a jump-in -ie We click within a tag and start typeing
    int line; ///Current line of tag - We will limit tag compl to singe line
    bool active; ///is Html autocomplete turned on
    bool brActive;  ///Auto <br> ?
    bool inTag; ///are we currently building a tag name
    bool tagWaiting; ///is there a tag bufferred and waiting for a >
    bool idTrigger;  ///possible autoassoc idval to tagname ex. <asp:label id="myLabel">
    wxString tag; ///The buffered tag
};

class ZionEdit;

class EditPrint: public wxPrintout
{
    public:
        EditPrint(ZionEdit *edit, wxChar *title = _T(""));
        ~EditPrint();

        //! event handlers
        bool OnPrintPage(int page);
        bool OnBeginDocument(int startPage, int endPage);
        void OnPreparePrinting();  ///Don't count pages here!
        void OnEndPrinting();
        //! print functions
        bool HasPage(int page);
        void GetPageInfo(int *minPage, int *maxPage, int *selPageFrom, int *selPageTo);

    private:
        ZionEdit *m_edit;
        wxArrayInt pages; ///Track page positions
        int m_selStart;  int m_selEnd;
        int m_pgCount;
        wxRect m_pageRect;
        wxRect m_printRect;
        bool m_linesOff;

        bool PrintScaling(wxDC *dc);
};

class ZionFrame; class FindResultsDialog;
class SettingsDialog;
class ZionEdit : public wxStyledTextCtrl
{
    friend class EditPrint; friend class GeneralOptsPanel; friend class SettingsDialog;
    friend class ZionFrame;

    public: //! I know, a lot of my public f( )s need to be made private
        ZionEdit(ZionFrame *ourFrame, wxWindow *parent, const wxString & str, bool min = false /* Minimal Startup */, wxWindowID id = -1,
        const wxPoint &pos = wxDefaultPosition,
        const wxSize &size = wxDefaultSize,
        long style = wxSUNKEN_BORDER|wxVSCROLL
        );
        ~ZionEdit();
        //! Public vars
        wxArrayString m_fileList;
        bool m_dirty;
        int m_startOpt;  /// 0 - half max, 1 - maximized, 2- custom
        wxPoint m_startPos; /// used with custom
        wxSize m_startSize;  ///used with custom and normal
        
        /// Public Utility F()s
        wxString GetIniDir();  ///Get iniDir from parent
        wxArrayString RemoteFindAll();
        void DoReplaceAll();  ///Used by External Replace All
        void ShowPosition();
        void ScrollFromEdge();
        //! Event Handlers
        void OnMarginClick (wxStyledTextEvent &event);
        void OnCharAdded (wxStyledTextEvent &event);
        void OnModified (wxStyledTextEvent &event);
        wxString GetContextWord( bool leftOnly=false );  ///Return the word (indentifier) near the cursor; f()s have priority
        bool GetContextFunct( wxString & fname, bool leftOnly = false );
        void OnCallTipClicked (wxStyledTextEvent &event);
        void OnAutoCompSelChange(wxStyledTextEvent &event);
        ActiveCallTip * CreateHiliteCT( CallTipsHash::iterator it );
        wxString BuildWord( int pos, bool asp = false );  ///Just Identifier chars, asp.net compliant
        wxString BuildWord2( int pos ); ///Object Compliant
        
        void OnDoneLoading( wxStyledTextEvent &event );
        
        void OnLineNumToggle (wxCommandEvent & event);
        void OnFoldToggle (wxCommandEvent & event);
        void OnWrapToggle (wxCommandEvent & event);
        void OnHorizSbToggle (wxCommandEvent & event);
        void OnVertSbToggle (wxCommandEvent & event);
        void OnIndentToggle (wxCommandEvent & event);
        void OnWhiteSpaceToggle (wxCommandEvent & event);
        void OnUseTabsToggle (wxCommandEvent & event);
        void OnReadOnlyToggle (wxCommandEvent & event);
        
        void OnUndo (wxCommandEvent & event);
        void OnRedo (wxCommandEvent & event);
        void OnFind (wxCommandEvent & event);
        void OnFindAll (wxCommandEvent & event);
        //! Handlers for  Find/Replace
        void OnFindReplace(wxCommandEvent &);
        void OnDlgFind(FindReplEvent & event);
        void OnDlgFindNext(FindReplEvent & event);
        void OnDlgReplace (FindReplEvent & event);
        void OnDlgReplaceAll (FindReplEvent & event);
        void OnDlgFindClose (FindReplEvent & event);
        void OnFindNext (wxCommandEvent & event);
        void OnFindPrev (wxCommandEvent & event);
        void OnClearFind (wxCommandEvent & event);
        void OnClearRepl (wxCommandEvent & event);
        void OnGotoLine (wxCommandEvent & event);
        void OnGotoStart (wxCommandEvent & event);
        void OnGotoEnd (wxCommandEvent & event);
        void OnNew(wxCommandEvent & event);
        void OnFileClose(wxCommandEvent & event);
        void DoNew( int lexer );
        void OnOpen(wxCommandEvent & event);
        void OnReload( wxCommandEvent & event);
        
		  void OnSave		(wxCommandEvent &);
		  void OnSaveAs	(wxCommandEvent &);
          void DoSaveAs();
        //void StyleAtCaret(wxCommandEvent & event);
        void OnChooseLexer(wxCommandEvent & event);
        void OnSelectWord(wxCommandEvent & event);
        
        /*! Utility functions */
        void DoSelectWord();
        void DoSettings( int stylenum =-1);
        bool DetermineReadonly();
        void DoFindAllFiles( const wxString & dirStr);
        void DoFindAll( bool noDlg = false, bool rtFind = false );
        void Reload( int lexer, int pos = 0, bool force = false ); ///Force lexer setup, even if same
        void ReloadFromBuffer( int lexer, int pos = 0, bool force = false );  ///Reload without using a file
		  wxString GetCurrentFilename();
		  void SetCurrentFilename(wxString afile);
		  bool SaveCurrentFile();
        bool CheckAllowSave( wxString msgStr = wxEmptyString );
        void Open( const wxString& dir = wxEmptyString ); 
        void DirectOpen( const wxString & arg, bool local = false );
         /// FindNext would override the search pattern  i.e. we could be changing our search pattern midstream, if there is some text selected so tell it we are using a non-modal dialog;
        ///Basically specifying non-modal says that we are in non-interactive mode - flags and search pattern have already been set and we are just moving merrily along. 
        bool FindNext( bool nonmodal = false, bool notify= true, bool rtFind = false ); ///Default to a modal dialog and do notify of new find string
        bool SetPrefs();
        void SetupLexer(const int lexer, bool force = false);
        //void SetupSyntaxDlgSmall(struct smallSyntax &);
        void ProcessSmallSyntax(struct smallSyntax &); //! Special Requirement!
        void SyntaxFromFile( const wxString & );
        int GetCurrentLexer();
        void ShowEOLMode();
        //! Public vars
        bool m_settingsDirty;
        bool m_LNVisible;
        bool m_FoldMarginVisible;
        int m_searchStartPos;
        FindReplData m_findReplData;  //! data class for use with all Find functions
        FindReplData m_findReplData2;  //! data class only for Real time Find All. see OnRTFindAll
        FilesResultsData m_filesResultsData; ///Storage class for Find in All Files results
        wxString m_currentDir;
        int m_startLex;
        int m_currentLexer;

    private:
        bool m_min;
        HtmlAutoComp m_htmlAC;
        int m_acAttribQuote;  ///Mode for Attribute completion - Single, Double or No Quotes
        //int m_acMemListShown;  ///Index into g_memberLists[] pointing to the current MemberLists hash for ac shown
        MemberLists *m_acCurrHash;  ///Autocomplete - Indentifies current AC Shown
        wxString m_acCurrObj;  ///Autocomplete - Indentifies current AC Shown
        //CallTipNoteDlg * m_ctacNote;
        DescriptDlg * m_descript; ///For AC - Yes we want these managed separately (esp. for deletes)
        DescriptDlg * m_ctDescript; ///For CT
        bool m_newLine;
        int m_line;
        bool FindWord_style( int a );
        void DoAutoComplete(); ///Global, non-obj AC
        void DoAmpComplete(); ///Html special chars
        void PreACShow(MemberLists * ptrHash, const wxString & obj, int count=0 );  ///Obj member AC - Just before call to Scintilla ///Stores pointer to hash and obj string //count = acChars
        void ShowCallTips( int activeCallTipIdx );
        void OnContextCallTip( wxCommandEvent & event );
        void OnAssocToType( wxCommandEvent & event );
        void OnDumpCallTips( wxCommandEvent & event );
        void LoadHardHash();
        bool LoadCallTips( smallSyntax & aSyntax );
        bool ParseCallTipFile( const wxString & str, smallSyntax & aSyntax );
        void ExtractObjMems( MemberLists & hash, const wxString & obj, wxString mem );
        void ExtractObjMemsWDescrip( MemberLists & hash, const wxString & obj, wxString mem );
        void ExtractFunc( MemberLists & hash, const wxString & obj, wxString mem );
        //void ExtractHtmlAttrs( MemberLists & hash, const wxString & obj, wxString mem, bool addToTagList=false );
        void ExtractHtmlAttrsWDescrip( MemberLists & hash, const wxString & obj, wxString mem, bool addToTagList=false, const wxString & tagDescript = wxEmptyString );
        void AddWebCtrl( MemberLists & hash, const wxString & obj, bool methods=false );
        void SortKWList( wxString & list );
        void DoHtmlColors( MemberLists & hash, const wxString & objLwr );
        void SortMemList( MemberLists & hash );
        void ShowObjACList( MemberLists & hash );
        void DoACHtmlTag( const wxString & tag ); ///Autocomplete tag
        void DoACHtmlAttr( const wxString & tag ); ///Autocomplete attributes within a tag
        bool m_callTipsLoaded;
        bool m_callTipsEn;
        void LocateCallTipParams( wxArrayInt & locs, int idx ); ///SCAN CURRENT DEFINITION to determine Parameter locs.
        void OnOpenSelected( wxCommandEvent & event );
        void RepositionIfSel();  /// helper f() so we are positioned correctly for line ops
        bool ProcessSel( int * a=NULL, int * b=NULL, bool lesser=false, bool select=true );  ///lesser = leave at lesser position i.e. backwards selection
        /* inline */ wxString CheckPreserveCase();
        void OnRemoveBlankLines( wxCommandEvent &event );
        void OnRemoveBlockComments( wxCommandEvent &event );
        void OnRemoveHtmlComment( wxCommandEvent &event );
        void OnAutoCompleteShow( wxCommandEvent &event );
        void OnLocalFindAllTweak( myFindResultsEvent/*FilesResultsEvent*/ &event);
        void OnResultsClicked( myFindResultsEvent &event );
        void OnResultsClosing( myFindResultsEvent &event );
        void OnFilesResultsClicked( FilesResultsEvent &event );
        void OnFilesResultsClosing( FilesResultsEvent &event );
        void OnFilesReplace( FilesResultsEvent &event );
        void OnFilesReplaceAll( FilesResultsEvent &event );
        void OnRemoteFindAll( FilesResultsEvent &event );  ///FindInAllFiles willl send an event to tiigger this f() ///This will load the FilesResultsData with the list for the current file
        void OnSelToMatch( wxCommandEvent & );
        void OnRTFindAll( wxCommandEvent & );
        void OnFindAllFiles( wxCommandEvent & );
        void OnChangeStyle(wxCommandEvent &);
        void OnFindUpdate(wxCommandEvent &);
        void OnReplUpdate(wxCommandEvent &);
        void OnKey(wxStyledTextEvent &event);
        void OnAutoCompSel(wxStyledTextEvent &event);
        void OnHtmlMatchTag(wxCommandEvent &);
        void OnHtmlSkipOverTag(wxCommandEvent &);
        void OnHtmlSkipBeforeTag(wxCommandEvent &);
        void OnNewHtmlTempl(wxCommandEvent &);
        void OnNewAsp(wxCommandEvent &);
        void OnZoomIn(wxCommandEvent &);
        void OnResetZoom(wxCommandEvent &);
        void OnZoomOut(wxCommandEvent &);
        void OnSelectionToggle( wxCommandEvent & );
        void Found( int result, bool rtFind = false );
        void OnGotoBraceMatch( wxCommandEvent & );
        void OnConvertEOL( wxCommandEvent & );
        void OnIncRedIndent( wxCommandEvent & );
        void OnUpperLowerCase( wxCommandEvent & );
        void OnHtmlInsertTag( wxCommandEvent & );
        void OnHtmlComment( wxCommandEvent & );
        void OnHtmlQuickTag( wxCommandEvent & );
        void OnHtmlAnchor( wxCommandEvent & );
        void OnHtmlNBSP( wxCommandEvent & );
        void OnHtmlNDASH( wxCommandEvent & );
        void OnHtmlQuote( wxCommandEvent & );
        void OnHtmlBR( wxCommandEvent & );
        void OnHtmlSupTH( wxCommandEvent & );
        void OnHtmlPhp( wxCommandEvent & );
        void OnHtmlAsp( wxCommandEvent & );
        void OnHtmlAspDrtv( wxCommandEvent & );
        void OnHtmlAspNet( wxCommandEvent & );
        void OnHtmlTemplAsp( wxCommandEvent & );
        void OnHtmlTemplBasic( wxCommandEvent & );
        void OnHtmlTemplBasicNoHead( wxCommandEvent & );
        void OnHtmlTemplStrict( wxCommandEvent & );
        void OnHtmlTemplTrans( wxCommandEvent & );
        void OnLineComment( wxCommandEvent & );
        void OnLineUnComment( wxCommandEvent & );
        void OnCppCurly( wxCommandEvent & );
        void OnCppForIf( wxCommandEvent & );
        void OnCppComment( wxCommandEvent & );
        void OnCppT( wxCommandEvent & );
        
        void OnPopup( wxContextMenuEvent & );
        void ContextMenuStuff();
        void CleanMenu();
        void CleanMenuBar(const wxString & menuString);
        void OnCut( wxCommandEvent &);
        void OnCopy( wxCommandEvent &);
        void OnPaste( wxCommandEvent &);
        void OnSelectLine( wxCommandEvent & );
        void OnSelectAll( wxCommandEvent & );
        void OnDuplicateLine (wxCommandEvent & event);
        void OnCutLine (wxCommandEvent & event);
        void OnTransposeLine (wxCommandEvent & event);
        void OnCopyLine (wxCommandEvent & event);
        void OnSettings(wxCommandEvent & event);
        void OnPosChanged(wxStyledTextEvent &event);
        void OnBookmark(wxCommandEvent & event);
        void OnAddBookmark(wxCommandEvent & event);
        void OnClearBookmarks(wxCommandEvent & event);
        ZionFrame * m_ourFrame; //track our parent frame

        wxString m_currentFilename;
        bool m_settled;
        bool m_dontPrintNums;
        wxMenu * m_popup;
        //Replaced by the full edit menu //wxMenu * findSubMenu;
        wxMenu * windowMenu;
        wxMenu * bookMenu;
        wxArrayString m_bookmarks;
        wxArrayInt m_bookIdx;

        //!Search vars
        wxString m_searchPattern;
        bool m_findWrap;
        int m_searchFlags;
        int m_incrementalStartPos;
        class SyntaxDialog * m_syntaxDlg;  //! Fwd Ref?
        FindReplDialog * m_dlgReplace;
        FindResultsDialog * m_fresDialog;
        FilesResultsDialog * m_filesDialog;
        // margin variables
        int m_LNMarginID;   //line number margin
        int m_LNMarginWidth;
        int m_FoldMarginID;
        int m_FoldMarginWidth;
    DECLARE_EVENT_TABLE()
};

#endif //_ZIONEDIT_H_
