// AUTHOR: Rohan Allison
// COPYRIGHT: 2006
//  License: Artistic

#include <wx/wx.h>
#include <wx/filefn.h>
#include <wx/filename.h>
#include <wx/gdicmn.h>
#include <wx/colordlg.h>
#include <wx/textfile.h>
#include <wx/ipc.h>
#include <wx/snglinst.h>
#include <wx/tokenzr.h>
#include <wx/statline.h>
#include <wx/stream.h>
#include <wx/timer.h>
#include <wx/print.h>
#include <wx/printdlg.h>
#include <wx/process.h>
#include <wx/dir.h>
#include <wx/clipbrd.h>
#include <wx/html/htmlwin.h>
// #include <wx/sound.h>
#include <stc.h>
#include <wx/fontenum.h>
#include <wx/dynarray.h>

#include "globaldefs.h"
#include "syntax.h"
#include "FindReplDialog.h"  ///Also good practice: smaller module header before larger ones
#include "ZionDialogs.h"
#include "zionedit.h"
#include "zion.h" //! GOOD PRACTICE: Local header file included last

#define VERSION "2.2.4"

/** Globals and Statics**/

//! global print data, to remember settings during the session
wxPrintData *g_printData = (wxPrintData*) NULL;
wxPageSetupData *g_pageSetupData = (wxPageSetupData*) NULL;
unsigned long localPid=wxGetProcessId(); //! Could check for a zero return

wxPoint zion_contextMenuPos(0,0);

///Candidates for a global options structure
int g_numTopRecentFiles; ///Displayed in top level of menu ///Candidate for a global Opts structure
int g_numRecentDirs;  ///Displayed
///Many permanent settings will be stored in globals. Per session settings will be stored in member variables of ZionFrame.
int g_extInitLexer = -1;  ///Selection forced from outside
bool g_autoBook;
bool g_autoZoom;
int g_numACChars = 4;
int g_lineNumWidth = 4;
bool g_acActive = true;
bool g_ctActive = true;
wxString g_pathToBrowser = _T("C:\\Program Files\\Internet Explorer\\iexplore.exe");
wxString g_pathToApi = _T("USEINI");
wxString g_initialZoom;

wxString hostName = wxGetHostName();
wxString service = IPC_SERVICE;  wxString priority("0");
static ZionFrame * s_ourFrame = NULL;
static int reconnCount = 0;

//! Client
ClientConn * zconnection = NULL;  //! As a client we will track only one conn. (to the server)
//! Server
class ServerConn;
//! Server will track several conns by Process ID 
WX_DECLARE_STRING_HASH_MAP( ServerConn*, Connections );  //!We are declaring Connections as a type of Hash Map using wxStrings as keys to ServerConn pointers
Connections zconns;

wxString workingTitle = wxString::Format( _T(" - ZionEdit  (%lu)"), localPid );  //! Concatenate title 
//int junk = workingTitle.Printf( _T(" - ZionEdit  (%lu)"), localPid );  //! Concatenate title //Compiler has a problem if return value isn't used. It thinks a declaration is being made
const wxChar *TITLE 	= workingTitle.c_str();
const wxChar *version = VERSION;

wxArrayString g_recentDirs;
wxArrayString g_faceNames;  //! We want to always define things in a cpp file. If we put them in header files that are
                            //! included in multiple files, we can get a mess of multiple definitions. When in doubt make clean, then make
MyServer *zserver;  MyClient * zclient;

IMPLEMENT_APP(ZionApp)

//! Init our App
bool ZionApp::OnInit()
{
    wxString fileToOpen, strLwr, str2, t("Untitled ");   //! wxStrings are initialized to ""
    t.append(TITLE);
    wxString pathToIni, findString, replString, execString, workingDir; ///String init to empty
    bool replMode = false; bool pickupExecStr = false; 
    int i;  
    wxString utilStr, posStr;
    long W=-1, H=-1, X=-1, Y=-1, N=-1, Q= -1, LEX=0;/// -1 will tell us the values has not been set by a command line option
    long flags = 0; long xpos =0;
    //! TODO? - Read in start up size and location from a [STARTUP] section in zion.ini
    for( i = 1; i < argc ; i++ ) {
        str2= argv[i];   /// Flags are NOT case-sensitive and we are trying to maintain this, but the contents of some flags ARE case sens.
        strLwr = str2.Lower();  ///We will therefore check twice for the flags of case sens. contents to ensure we get both cases of the flag
        if( str2.StartsWith( "-g", &utilStr)  ) utilStr.ToLong(&flags);
        else if( str2.StartsWith( "-G", &utilStr)  ) utilStr.ToLong(&flags);
        ///Non-casesensitive "contents"
        else if( strLwr.StartsWith( "-w", &utilStr )  ) utilStr.ToLong(&W);  //!Put the rest of the string in utilStr then convert
        else if( strLwr.StartsWith( "-h", &utilStr)  ) utilStr.ToLong(&H);  //! Caution we will be passing a long into an int in ZionFrame's constructor
        else if( strLwr.StartsWith( "-x", &utilStr)  ) utilStr.ToLong(&X);
        else if( strLwr.StartsWith( "-y", &utilStr)  ) utilStr.ToLong(&Y);
        else if( strLwr.StartsWith( "-n", &utilStr)  ) utilStr.ToLong(&N);  ///lineNum
        else if( strLwr.StartsWith( "-q", &utilStr)  ) utilStr.ToLong(&Q);  ///Column
        else if( strLwr.StartsWith( "-lex", &utilStr)  ) { utilStr.ToLong(&LEX); g_extInitLexer = static_cast<int>(LEX); }
        else if( strLwr.StartsWith( "-zposx", &utilStr)  ) utilStr.ToLong(&xpos); ///[Initially] give the new window the xposition of the calling window, so that when maximized, it will maximize to the screen of the caller
        ///Case sensitive "contents"
        else if( str2.StartsWith( "-i", &pathToIni)  ) { } //! Specified path may be relative so we cannot assume first char is a drive letter.
        else if( str2.StartsWith( "-I", &pathToIni)  ) { } //! Specified path may be relative so we cannot assume first char is a drive letter.
        else if( str2.StartsWith( "-f", &findString)  ) { }
        else if( str2.StartsWith( "-F", &findString)  ) { }
        else if( str2.StartsWith( "-p", &replString)  ) { replMode = true; }  ///Since replace string can be legitimately empty in replace mode, we needed another way to determine repl mode, hence the bool
        else if( str2.StartsWith( "-P", &replString)  ) { replMode = true; }
        else if( str2.StartsWith( "-e", &execString)  ) { }
        else if( str2.StartsWith( "-E", &execString)  ) { }
        else if( str2.StartsWith( "-d", &workingDir)  ) { }
        else if( str2.StartsWith( "-D", &workingDir)  ) { }
        else if( str2.StartsWith( "-zexec" ) ) pickupExecStr = true;
        else if( str2.StartsWith( "-Zexec" ) ) pickupExecStr = true;
         ///These are mutually exclusive. The last one specified will be used
         else if( strLwr == "-t" || strLwr == "-l" || strLwr == "-b" ||
                strLwr == "-r" || strLwr == "-tl" || strLwr == "-tr" ||
                strLwr == "-bl" || strLwr == "-br" || strLwr == "-c" || strLwr == "-m" )
            posStr = strLwr;  ///hold it for now
        else
            fileToOpen  = str2;
    }
    ///If pathToIni is surrouned by quotes remove them
    if( pathToIni.GetChar(0) == '"' ) {
        pathToIni.Remove( 0, 1 ); ///At pos 0 remove 1 char
        if( pathToIni.Last() == '"' ) pathToIni.RemoveLast(); ///remove last only if there was a first
    }
    
    /*! Create our MAINFRAME */
    s_ourFrame = m_zionFrame = new ZionFrame( argv[0], fileToOpen, t, xpos, pathToIni,
            wxDEFAULT_FRAME_STYLE | wxMAXIMIZE );
    //! We must create the window before we can use it's constituent functions */
    int x, y, w, h;
    m_zionFrame->GetSize( &w, &h );  //! Window size (maximized) in zionframe constructor
    m_zionFrame->GetPosition( &x, &y );  //! Window size (maximized) in zionframe constructor
    w -= 6;  h -= 6; ///Adj for maximize removing borders
    
    ///At this point we have the values for a maximized window in w and h
    ///X or Y positions may or may not have been specified

    ///Position and Width determining options
    if( posStr == "-t" )          {  X=x; Y=0;  H= h/2; W=w; }
    else if( posStr == "-b" )  {  X=x;  Y=h/2; H=h/2; W=w; }
    else if( posStr == "-l" )  {  X=x;  Y=0;  W=w/2;
        H = m_zionFrame->m_startOpt == 2 ? m_zionFrame->m_startSize.GetHeight() : h ;
    }
    else if( posStr == "-r" )  {  X= x + w/2; Y=0;  H=h; W=w/2;
        H = m_zionFrame->m_startOpt == 2 ? m_zionFrame->m_startSize.GetHeight() : h ;
    }
    else if( posStr == "-tl" )  {  X=x; Y=0; W=w/2;  H=h/2;  }
    else if( posStr == "-tr" )  {  X=x + w/2; Y=0; W=w/2;  H=h/2;  }
    else if( posStr == "-bl" )  {  X=x; Y=h/2; W=w/2;  H=h/2;  }
    else if( posStr == "-br" )  {  X=x+w/2; Y=h/2;  W=w/2;  H=h/2;  }
    else if( posStr == "-c" )  {  X=x+w/4; W=w/2;
        H = m_zionFrame->m_startOpt == 2 ? m_zionFrame->m_startSize.GetHeight() : h ;
    }
    else if( posStr == "-m" )  {  X=x; Y=0; W=w; H=h; }

    ///USE DEFAULTS if one or more parameters not specified
    if( W == -1 ) { ///width not specified, so use a default
        if( m_zionFrame->m_startOpt == 0 ) W = w/2;  ///normal /// case of 1 (maximized) do nothing (already at maximum width)
        if( m_zionFrame->m_startOpt == 1 ) W = w;  ///(maximized)
        if( m_zionFrame->m_startOpt == 2 ) W = m_zionFrame->m_startSize.GetWidth(); ///custom
    }
    if( H == -1) { 
        if( m_zionFrame->m_startOpt == 2)  H = m_zionFrame->m_startSize.GetHeight(); ///custom
        else H = h; /// case of startOpt 0,1 (normal,maximized)
    }
    if( X == -1 ) {
        X = m_zionFrame->m_startOpt == 2 ? m_zionFrame->m_startPos.x  : 0; ///custom pos or 0
    }
    if( Y == -1 ) {
        Y = m_zionFrame->m_startOpt == 2 ? m_zionFrame->m_startPos.y  : 0; ///custom pos or 0
    }
    
    m_zionFrame->SetWindowStyle( ( wxDEFAULT_FRAME_STYLE /*^  wxRESIZE_BORDER*/ ) ); ///Get out of the maximized style
    ///Set FINAL SIZE AND POSITION
    m_zionFrame->SetSize( X, Y, W, H ); //! Reset size and position (from maximized). Make sure of  x and y positions
    
    SetTopWindow(m_zionFrame);
    
    /** AUTOMATION Section **/
    ///External find all - We don't want to show the window yet
    if( replMode && findString != _T("")) { /// REPLACEALL
        if( m_zionFrame->m_zedit->GetReadOnly() ) return FALSE;
        ///Remove surrounding double quotes if any
        if( findString.GetChar(0) == '"' ) {
            findString.Remove( 0, 1 ); ///At pos 0 remove 1 char
            if( findString.Last() == '"' ) findString.RemoveLast();
        }
        if( replString.GetChar(0) == '"' ) {
            replString.Remove( 0, 1 );
            if( replString.Last() == '"' ) replString.RemoveLast();
        }
    
        m_zionFrame->m_zedit->m_findReplData.UpdateFindString( findString );
        m_zionFrame->m_zedit->m_findReplData.UpdateReplString( replString );
        m_zionFrame->m_zedit->m_findReplData.SetFlags( (int) flags );
        m_zionFrame->m_zedit->m_searchStartPos=0;
        m_zionFrame->m_zedit->DoReplaceAll();
        ///save
        wxCommandEvent anEvent( wxEVT_COMMAND_MENU_SELECTED, ZION_SAVE ); ///type, id
        m_zionFrame->ProcessEvent(anEvent);
        return FALSE; ///Close when finished
    }
    /*! SHOW MAIN FRAME */
    m_zionFrame->Show(TRUE);
    
    ///WORKING DIRECTORY. Normally working dir is set to our last used dir
    if( workingDir != wxEmptyString ) {
        ///Remove double quotes if any (We should use quotes in case path has spaces) 
        if( workingDir.GetChar(0) == '"' ) {
            workingDir.Remove( 0, 1 ); ///At pos 0 remove 1 char
            if( workingDir.Last() == '"' ) workingDir.RemoveLast();
        }
        wxSetWorkingDirectory( workingDir ); ///Make sure we are starting up in the requested directory
        m_zionFrame->m_zedit->m_currentDir = workingDir; ///Let the edit module know too
    }
    
    if( findString != _T("") ) { /// FINDALL ///todo change to wxEmptyString
        ///Remove double quotes if any
        if( findString.GetChar(0) == '"' ) {
            findString.Remove( 0, 1 ); ///At pos 0 remove 1 char
            if( findString.Last() == '"' )
            findString.RemoveLast();
        }
        m_zionFrame->m_zedit->m_findReplData.UpdateFindString( findString );
        m_zionFrame->m_zedit->m_findReplData.SetFlags( (int) flags );
        m_zionFrame->m_zedit->m_searchStartPos=0;
        m_zionFrame->m_zedit->DoFindAll();
    }
    /// LINENUMBER request
    else if( N != -1 ) { 
        if( N > 0 && N <= m_zionFrame->m_zedit->GetLineCount() )
            m_zionFrame->m_zedit->GotoLine(N -1);
        if( Q != -1 && Q > 0 ) {  ///Row requested (can only be requested together with a column)
            int pos = m_zionFrame->m_zedit->GetCurrentPos() + Q-1;
            if( pos < m_zionFrame->m_zedit->GetLength() ) {
                m_zionFrame->m_zedit->GotoPos(pos);
            }
        }
        m_zionFrame->m_zedit->ShowPosition();
    }
    ///EXECUTE SUBPROCESS (It is good idea to preestablish our working dir before this.
    else if( pickupExecStr ) {
        wxString str = m_zionFrame->PickupExecStr();
        if(  str != wxEmptyString ) {
           m_zionFrame->DoExecSubProcess( str );
        }
        else  wxMessageBox( _T("Failed acquiring Exec String from Server"), _T("IPC Error") );
    }
    else if( execString != wxEmptyString ) {
        ///Remove double quotes if any
        if( execString.GetChar(0) == '"' ) {
            execString.Remove( 0, 1 ); ///At pos 0 remove 1 char
            if( execString.Last() == '"' )
            execString.RemoveLast();
        }
        m_zionFrame->DoExecSubProcess( execString );
    }
    
    return TRUE; ///Yes let the window live on
}

BEGIN_EVENT_TABLE( ZionFrame, wxFrame )
  EVT_UPDATE_UI_RANGE( ZION_NULL, ZION_F77, ZionFrame::OnUIMode ) //! wx will send an update event for EACH menuitem in the range
  EVT_UPDATE_UI_RANGE( ZION_LINE_WINDOWS, ZION_LINE_UNIX, ZionFrame::OnUIEOL ) //! wx will send an update event for EACH menuitem in the range
  EVT_UPDATE_UI( ZION_MENURECONN, ZionFrame::OnUIReconn )
  EVT_UPDATE_UI( ZION_BECOMESERVER, ZionFrame::OnUIBecomeServer )
  EVT_UPDATE_UI( ZION_PASTE, ZionFrame::OnUIPaste )
  EVT_UPDATE_UI( ZION_COPY, ZionFrame::OnUICopy )
  EVT_UPDATE_UI( ZION_CUT, ZionFrame::OnUICut )
  EVT_UPDATE_UI( ZION_UNDO, ZionFrame::OnUIUndo )
  EVT_UPDATE_UI( ZION_REDO, ZionFrame::OnUIRedo )
  EVT_UPDATE_UI( ZION_LINE_TOGGLE, ZionFrame::OnUILineToggle)
  EVT_UPDATE_UI( ZION_HTML_AC_TOGGLE, ZionFrame::OnUIHtmlACToggle)
  EVT_UPDATE_UI( ZION_HTML_BR_TOGGLE, ZionFrame::OnUIHtmlBRToggle)
  EVT_UPDATE_UI( ZION_FOLD_TOGGLE, ZionFrame::OnUIFoldToggle)
  EVT_UPDATE_UI( ZION_WRAP_TOGGLE, ZionFrame::OnUIWrapToggle)
  EVT_UPDATE_UI( ZION_INDENT_TOGGLE, ZionFrame::OnUIIndentToggle)
  EVT_UPDATE_UI( ZION_USETABS_TOGGLE, ZionFrame::OnUIUseTabsToggle)
  EVT_UPDATE_UI( ZION_WHITESPACE_TOGGLE, ZionFrame::OnUIWhiteSpaceToggle)
  EVT_UPDATE_UI( ZION_HORIZSB_TOGGLE, ZionFrame::OnUIHorizSbToggle)
  EVT_UPDATE_UI( ZION_VERTSB_TOGGLE, ZionFrame::OnUIVertSbToggle)
  EVT_UPDATE_UI( ZION_READONLY_TOGGLE, ZionFrame::OnUIReadOnlyToggle)
  EVT_UPDATE_UI( ZION_AUTOZOOM_TOGGLE, ZionFrame::OnUIAutoZoomToggle)
  EVT_UPDATE_UI( ZION_AUTOCOMP_TOGGLE, ZionFrame::OnUIAutoCompToggle)
  EVT_UPDATE_UI( ZION_CALLTIPS_TOGGLE, ZionFrame::OnUICallTipsToggle)
  EVT_UPDATE_UI( ZION_CTAC_TOGGLE, ZionFrame::OnUICTACToggle)
  EVT_UPDATE_UI( ZION_AUTOBOOK_TOGGLE, ZionFrame::OnUIAutoBookToggle)
  
  EVT_TIMER(ZION_IPCTIMER, ZionFrame::OnIPCTimer)
  EVT_TIMER(ZION_BOOKMARKTIMER, ZionFrame::OnBookmarkTimer)
  EVT_MENU ( ZION_ADDBOOKMARK, 	ZionFrame::OnZionEdit )
  EVT_MENU ( ZION_CLEARBOOKMARKS, 	ZionFrame::OnZionEdit )
  
  EVT_COMMAND_RANGE(ZION_BOOK_1, ZION_BOOK_1 + NUMBOOKMARKS -1,
        wxEVT_COMMAND_MENU_SELECTED, ZionFrame::OnZionEdit)
  
  EVT_MENU ( ZION_BECOMESERVER, 	ZionFrame::OnBecomeServer)
  EVT_MENU ( ZION_MENURECONN, 	ZionFrame::OnMenuReconn)
  EVT_MENU ( ZION_ADVISE_READHISTORY, 	ZionFrame::OnAdviseReadHistory)
  EVT_MENU ( ZION_ADVISE_READPRINT, 	ZionFrame::AdviseReadPrint)
  EVT_MENU ( ZION_ADVISE_FINDUPDATE, 	ZionFrame::AdviseFindUpdate)
  EVT_MENU ( ZION_ADVISE_REPLUPDATE, 	ZionFrame::AdviseReplUpdate)
  EVT_MENU ( ZION_ADVISE_READSTYLES, 	ZionFrame::AdviseReadStyles)
  EVT_MENU ( ZION_ADVISE_CLOSE, 	ZionFrame::AdviseClose)
  EVT_MENU ( ZION_ADVISE_RAISE, 	ZionFrame::AdviseRaise)
  EVT_MENU ( ZION_FINDUPDATE, 	ZionFrame::OnZionEditPending)
  EVT_MENU ( ZION_REPLUPDATE, 	ZionFrame::OnZionEditPending)
  EVT_MENU ( ZION_NOTIFYSTYLES, 	ZionFrame::NotifyReadStyles)
  EVT_MENU ( ZION_RECONN, 	ZionFrame::OnReconn)
  EVT_MENU ( ZION_SHOWIPCSTATUS, 	ZionFrame::OnShowIPCStatus)
  EVT_MENU ( ZION_ASSIGNPRIORITIES, 	ZionFrame::OnAssignPriorities)
  EVT_MENU ( ZION_EXTERNALCLOSE, 	ZionFrame::OnExternalClose)
  EVT_MENU ( ZION_EXTERNALRAISE, 	ZionFrame::OnExternalRaise)
  
  EVT_MENU ( ZION_FILECLOSE, 	ZionFrame::OnZionEdit )
  EVT_COMMAND_RANGE( ZION_NEWZION, ZION_NEWZION_M,
        wxEVT_COMMAND_MENU_SELECTED, ZionFrame::OnNewZion )
  EVT_MENU ( ZION_PRINT, 	ZionFrame::OnPrint)
  EVT_MENU ( ZION_PRINTSETUP, 	ZionFrame::OnPrintSetup)
  EVT_MENU ( ZION_PRINTPREVIEW, 	ZionFrame::OnPrintPreview)
  EVT_MENU ( ZION_HELP, 	ZionFrame::OnHelp)
  EVT_MENU ( ZION_INFO, 	ZionFrame::OnInfo)
  EVT_MENU ( ZION_DUMPCALLTIPS, 	ZionFrame::OnZionEdit )
  EVT_MENU ( ZION_CONTEXTCALLTIP, 	ZionFrame::OnZionEdit )
  EVT_MENU ( ZION_ABOUT, 	ZionFrame::OnAbout)
  EVT_CLOSE (ZionFrame::OnClose)	//takes only one arg
  EVT_MENU ( ZION_SETTINGS, ZionFrame::OnZionEdit)
  
  EVT_COMMAND_RANGE( ZION_READINI, ZION_READINI_HISTORY,
        wxEVT_COMMAND_MENU_SELECTED, ZionFrame::OnReadIni )

  EVT_MENU ( ZION_WRITEINI, ZionFrame::OnWriteIni)
  EVT_MENU ( ZION_EDITINI, ZionFrame::OnEditIni)
  EVT_MENU ( ZION_EDITAPI, ZionFrame::OnEditIni)
  EVT_MENU ( ZION_EDIT_HISTORY_INI, ZionFrame::OnEditIni)
  EVT_MENU ( ZION_EXIT,  	ZionFrame::OnExit)
  //! Many events are patched thru to our STC  
  EVT_MENU ( ZION_LINE_TOGGLE, ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_FULLSCRN_TOGGLE, ZionFrame::OnFullScrnToggle )
  EVT_MENU ( ZION_FOLD_TOGGLE, ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_WRAP_TOGGLE, ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_HORIZSB_TOGGLE, ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_VERTSB_TOGGLE, ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_WHITESPACE_TOGGLE, ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_INDENT_TOGGLE, ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_USETABS_TOGGLE, ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_READONLY_TOGGLE, ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_AUTOBOOK_TOGGLE, ZionFrame::OnAutoBookToggle )
  EVT_MENU ( ZION_AUTOZOOM_TOGGLE, ZionFrame::OnAutoZoomToggle )
  EVT_MENU ( ZION_AUTOCOMP_TOGGLE, ZionFrame::OnAutoCompToggle )
  EVT_MENU ( ZION_CALLTIPS_TOGGLE, ZionFrame::OnCallTipsToggle )
  EVT_MENU ( ZION_CTAC_TOGGLE, ZionFrame::OnCTACToggle )
  EVT_MENU ( ZION_OPEN,  	ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_OPENSELECTED,  	ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_CUT,  	ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_COPY,  	ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_PASTE,  	ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_SELECTRECT,  	ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_SELECTALL,  	ZionFrame::OnZionEdit)
  
  EVT_COMMAND_RANGE(ZION_WIN_1, ZION_WIN_1 + NUMWINDOWS -1,
        wxEVT_COMMAND_MENU_SELECTED, ZionFrame::OnWindow)
  
  EVT_COMMAND_RANGE(ZION_DIR_1, ZION_DIR_1 + NUMRECENTDIRS -1,
        wxEVT_COMMAND_MENU_SELECTED, ZionFrame::OnRecentDir)
  EVT_COMMAND_RANGE(ZION_FILE_1, ZION_FILE_1 + NUMRECENTFILES -1,
        wxEVT_COMMAND_MENU_SELECTED, ZionFrame::OnRecentFile)
  EVT_COMMAND_RANGE(ZION_FILEITEM_1, ZION_FILEITEM_LAST,
        wxEVT_COMMAND_MENU_SELECTED, ZionFrame::OnRecentFileItem)
  EVT_COMMAND_RANGE(ZION_OPENITEM_1, ZION_OPENITEM_LAST,
        wxEVT_COMMAND_MENU_SELECTED, ZionFrame::OnRecentFileItem)
  EVT_COMMAND_RANGE(ZION_REMOVEITEM_1, ZION_REMOVEITEM_LAST,
        wxEVT_COMMAND_MENU_SELECTED, ZionFrame::OnRecentFileItem)

  EVT_MENU ( ZION_CMDPROMPT,  	ZionFrame::OnCmdPrompt)
  EVT_MENU ( ZION_RUNANDCAPTURE,  	ZionFrame::OnExecSubProcess)
  EVT_MENU ( ZION_EXPLORE,  	ZionFrame::OnExplore)
  EVT_MENU ( ZION_BUGREPORT,  	ZionFrame::OnBugReport)
  EVT_MENU ( ZION_CHECKLATEST,  	ZionFrame::OnCheckLatest)
  EVT_MENU ( ZION_ONLINEDOCS,  	ZionFrame::OnOnlineDocs)
  EVT_MENU ( ZION_GOOGLEIT,  	ZionFrame::OnGoogleIt)
  EVT_MENU ( ZION_RELOAD,  	ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_NEWNULL,  	ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_NEWCPP,  	ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_NEWPYTHON,  	ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_NEWHTML,  	ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_NEWHTML_TEMPLSTRICT,  	ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_NEWHTML_TEMPLTRANS,  	ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_NEWHTML_TEMPLBASIC,  	ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_NEWHTML_TEMPLBASICNH,  	ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_NEWHTML_TEMPLASP,  	ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_NEWPERL,  	ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_NEWJAVA,  	ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_NEWSQL,  	ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_NEWCS,  	ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_NEWMAKE,  	ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_NEWBATCH,  	ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_NEWLISP,  	ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_NEWCSS,  	ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_NEWRUBY,  	ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_NEWVBSCRIPT,  	ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_NEWASM,  	ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_NEWFORTRAN,  	ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_NEWF77,  	ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_SAVE,  	ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_SAVE_AS, ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_UNDO, ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_REDO, ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_FIND, ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_FINDALL, ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_FINDALLFILES, ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_FINDNEXT, ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_FINDPREV, ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_FINDREPLACE, ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_GOTOLINE, ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_GOTOBRACE, ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_GOTOSTART, ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_GOTOEND, ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_SELTOBRACE, ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_LINE_WINDOWS, ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_LINE_UNIX, ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_LINE_MAC, ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_DUPLICATELINE, ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_CUTLINE, ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_COPYLINE, ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_SELECTLINE, ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_SELECTWORD, ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_TRANSPOSELINE, ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_INCINDENT, ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_REDINDENT, ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_AUTOCOMPSHOW, ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_REMOVE1BLANK, ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_REMOVEALLBLANKS, ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_REMOVEGT2BLANKS, ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_UPPERCASE, ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_LOWERCASE, ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_ZOOMIN, ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_ZOOMOUT, ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_RESETZOOM, ZionFrame::OnZionEdit)
  
    EVT_MENU ( ZION_HTMLPREVIEW, ZionFrame::OnHtmlPreview)
    
    EVT_MENU ( ZION_HTMLMENU_BR, ZionFrame::OnZionEdit)
    EVT_MENU ( ZION_HTML_AC_TOGGLE, ZionFrame::OnHtmlACToggle)
    EVT_MENU ( ZION_HTML_BR_TOGGLE, ZionFrame::OnHtmlBRToggle)
    EVT_MENU ( ZION_HTMLMENU_BOLD, ZionFrame::OnZionEdit)
    EVT_MENU ( ZION_HTMLMENU_STRONG, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_HTMLMENU_ITALIC, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_HTMLMENU_UNDERLINE, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_HTMLMENU_EM, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_HTMLMENU_PARAGRAPH, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_HTMLMENU_COMMENT, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_HTMLMENU_UNCOMMENT, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_HTMLMENU_QUICKTAG, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_HTMLMENU_ANCHOR, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_HTMLMENU_NBSP, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_HTMLMENU_NDASH, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_HTMLMENU_BLOCKQUOTE, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_HTMLMENU_QUOTE, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_HTMLMENU_CENTER, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_HTMLMENU_SPAN, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_HTMLMENU_FONT, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_HTMLMENU_FORM, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_HTMLMENU_CODE, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_HTMLMENU_PRE, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_HTMLMENU_SUPTH, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_HTMLMENU_H3, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_HTMLMENU_H1, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_HTMLMENU_H2, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_HTMLMENU_H4, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_HTMLMENU_UL, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_HTMLMENU_OL, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_HTMLMENU_LI, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_HTMLMENU_TABLE, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_HTMLMENU_TD, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_HTMLMENU_TR, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_HTMLMENU_TH, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_HTMLMENU_SUP, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_HTMLMENU_SKIPTAG, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_HTMLMENU_SKIPBEFORETAG, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_HTML_MATCHTAG, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_HTML_PHP, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_HTML_ASP, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_HTML_SCRIPT, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_HTML_ASPDRTV, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_HTML_ASPNET, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_HTML_TEMPLSTRICT, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_HTML_TEMPLTRANS, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_HTML_TEMPLBASIC, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_HTML_TEMPLBASICNOHEAD, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_HTML_TEMPLASP, ZionFrame::OnZionEdit)
    
    EVT_MENU (ZION_CPPMENU_CURLY, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_CPPMENU_FOR, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_CPPMENU_WHILE, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_VBMENU_IF, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_VBMENU_ELSE, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_CPPMENU_IF, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_CPPMENU_ELSE, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_CPPMENU_ELSEIF, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_CPPMENU_COMMENT, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_CPPMENU_UNCOMMENT, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_CPPMENU_LINEUNCOMMENT, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_CPPMENU_LINECOMMENT, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_CPPMENU_T, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_CPPMENU_DOWHILE, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_PERLMENU_FOREACH, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_PERLMENU_ELSIF, ZionFrame::OnZionEdit)
    
    EVT_MENU (ZION_VBMENU_LINECOMMENT, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_VBMENU_LINEUNCOMMENT, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_PERLMENU_LINECOMMENT, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_PERLMENU_LINEUNCOMMENT, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_SQLMENU_LINECOMMENT, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_SQLMENU_LINEUNCOMMENT, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_ASMMENU_LINEUNCOMMENT, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_ASMMENU_LINECOMMENT, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_BATCHMENU_LINEUNCOMMENT, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_BATCHMENU_LINECOMMENT, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_BATCHMENU_LINEUNCOMMENT2, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_BATCHMENU_LINECOMMENT2, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_FORTRANMENU_LINEUNCOMMENT, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_FORTRANMENU_LINECOMMENT, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_FORTRAN77MENU_LINEUNCOMMENT, ZionFrame::OnZionEdit)
    EVT_MENU (ZION_FORTRAN77MENU_LINECOMMENT, ZionFrame::OnZionEdit)

 /// Force Syntax
  EVT_MENU (ZION_NULL, ZionFrame::OnZionEdit)
  EVT_MENU (ZION_PYTHON, ZionFrame::OnZionEdit)
  EVT_MENU (ZION_CPP, ZionFrame::OnZionEdit)
  EVT_MENU (ZION_HTML, ZionFrame::OnZionEdit)
  EVT_MENU (ZION_XML, ZionFrame::OnZionEdit)
  EVT_MENU (ZION_PERL, ZionFrame::OnZionEdit)
  EVT_MENU (ZION_JAVA, ZionFrame::OnZionEdit)
  EVT_MENU (ZION_CS, ZionFrame::OnZionEdit)
  EVT_MENU (ZION_SQL, ZionFrame::OnZionEdit)
  EVT_MENU (ZION_MAKE, ZionFrame::OnZionEdit)
  EVT_MENU (ZION_BATCH, ZionFrame::OnZionEdit)
  EVT_MENU (ZION_LISP, ZionFrame::OnZionEdit)
  EVT_MENU (ZION_RUBY, ZionFrame::OnZionEdit)
  EVT_MENU (ZION_VBSCRIPT, ZionFrame::OnZionEdit)
  EVT_MENU (ZION_CSS, ZionFrame::OnZionEdit)
  EVT_MENU (ZION_ASM, ZionFrame::OnZionEdit)
  EVT_MENU (ZION_FORTRAN, ZionFrame::OnZionEdit)
  EVT_MENU (ZION_F77, ZionFrame::OnZionEdit)

  EVT_MENU ( ZION_COLOURVALUE, ZionFrame::OnChooseColour)
  EVT_MENU ( ZION_CLEARSTATUS, ZionFrame::OnClearStatus )
  EVT_MENU ( ZION_CALLTIPSHOW, ZionFrame::OnShowCallTip )
  EVT_MENU ( ZION_ASSOCTYPE, ZionFrame::OnZionEdit)
  EVT_MENU ( ZION_STYLE, ZionFrame::OnZionEdit)
  
  EVT_END_PROCESS ( ZION_SUBPROCESS, ZionFrame::OnEndSubProcess )
  EVT_DROP_FILES( OnDropFiles )
END_EVENT_TABLE()

/*! Frame Constructor */
ZionFrame::ZionFrame (const wxString & arg0, const wxString & fileArg, const wxChar *title, int xpos,
        const wxString& iniPath, long ourStyle)
	: wxFrame( (wxFrame *) NULL, -1, title, wxPoint( xpos, 0 ), wxDefaultSize, ourStyle, "MainWindow" ) //call parent class
{
    /** Pointer Inits **/
    for( int i=0; i < NUMRECENTDIRS; i++ ) dirSubs[i] = new wxMenu;  /// Load dir submenu ptrs array
    for( int i=0; i < NUMRECENTDIRS; i++ ) dirSubItems[i] = NULL;  /// Load dir submenu menuitem ptrs array, i.e. The menuitem created when the submenu is added to recentDirsMenu
    m_zedit = NULL; /// edit control
    m_recentPopup = NULL;  ///track last recent item added to popup
    m_execProcess = (wxProcess *) NULL; ///for tracking an asynch'ly executed process
    m_execLastPid = 0;
    m_execAppend = false;  ///Default to open in new window
    m_captureOut = true;
    m_runWithCmd = true;
    //m_execNotify = (wxMessageDialog * ) NULL;
    //m_execNotify = new wxMessageDialog( this,  _T("Please WAIT until process completes... "), _T("Waiting for Process"), wxOK );
    
    /// Create print data and setup
    g_printData = new wxPrintData;
    g_pageSetupData = new wxPageSetupDialogData;

    m_FailedZionIni = false;  //! Remember our failed attempt to locate/Create Zion.ini
    m_extClose = false;
    m_startOpt = 0;  ///Normal half maximized startup
    m_startSize = wxSize(640, 480);
    m_startPos = wxPoint(0, 0);
    m_lineHits = 0;
    
    CreateStatusBar(5);
    int * tempArray = new int[5];
    tempArray[0]=-20; tempArray[1]=-5; tempArray[2] = -5; tempArray[3]= -2; tempArray[4]=-7;
    SetStatusWidths( 5, tempArray );
    delete [] tempArray;
    CreateMenu();
    wxIcon icon(_T("zion") ); ///see rsc.rc file
    SetIcon(icon);
    FontEnumr FEnumr;   FEnumr.EnumerateFacenames(); //! Fonts must be enumerated before we try to get them
    DragAcceptFiles(true);  ///Enable drop files
    
    /*! Resolve Paths */
    //! Save original startup path
    wxString str, str2;  m_exeDir = str2 = wxGetCwd();
    char c = str2.GetChar(0);  c = c & 0xDF;  str2.SetChar(0, c);   //! Correct for lower case drive letter // Windows only
    //! Determine Absolute Path of Executeable if a path is specified with the exe
    if( (str= arg0.BeforeLast('\\')) != wxEmptyString ) {
        wxSetWorkingDirectory(str);  //! str may contain a RELATIVE path
        m_exeDir = wxGetCwd();  //! Now get the absolute path of the executeable  = where am I?
        c = m_exeDir.GetChar(0);  c = c & 0xDF;  m_exeDir.SetChar(0, c);  //! correct case
    }
    wxSetWorkingDirectory( str2);  //! Reset to original working dir //We must reset here in case the -i param (following) is relative. We want to always use the original working dir as a reference for relative dirs.
    
    //! Determine Absolute Path  for supplied path to Zion.ini (-i xxx)
    if( iniPath != wxEmptyString ) { 
            wxSetWorkingDirectory( iniPath ); //! Set path to that specified
            m_iniDir = wxGetCwd();  //! Now get the absolute path of the executeable  = where am I?
            c = m_iniDir.GetChar(0);  c = c & 0xDF;  m_iniDir.SetChar(0, c); //! Correct our final ini dir for drive letter case
    }
    else m_iniDir = m_exeDir;  //! Ini Dir not specified so we will look for it in the exeDir
    wxSetWorkingDirectory( str2);  //! Reset to original working dir

    CheckForIni();
    
    ///Some Editor Defaults
    ///DESIGN CHANGE: SYNOPSIS BELOW IS No GOOD AS SETTINGS ARE BETTER COMMUNICATED WHEN THEY ARE GLOBAL.
    ///(The editor module had no way of knowing when a setting was updated via IPC)
    
    ///Settings Synopsis: Absolute defaults are set here. User defaults are read in from zion.ini with
    ///ReadIni(), updating these variables as necessary. (At this point the edit module does not even exist yet.)
    ///When the edit module fires up, it will reference these vars to set it's initial preferences (ZionEdit::SetPrefs()).
    ///Settings are from then on maintained in the edit modules own vars. For example, the
    ///Settings Dialog will init from and update to the edit modules own vars. ( Remember the edit module
    /// is a subclass of stc, so the "edit module vars" could refer to an stc var).
    
    ///So ReadIni will update these "startup" vars ///Write ini will pull settings from the edit module.
    m_startEol = wxSTC_EOL_CRLF; ///Win //will be updated in Readini
    m_printColMode = wxSTC_PRINT_BLACKONWHITE;
    m_tabWidth = 4;
    g_initialZoom = _T("0");
    m_zoomLevel = 0;  ///Just a numeric representation for our timer routine
    g_autoZoom = false;
    m_autoZoom = false;
    g_autoBook = true;
    m_autoBook = true;
//    g_numACChars = 4;
    g_acActive = true;
    g_ctActive = true;
    g_pathToBrowser = _T("C:\\Program Files\\Internet Explorer\\iexplore.exe");
    g_pathToApi = m_iniDir + _T("\\API_Files");
    g_numRecentDirs = 16;
    g_numTopRecentFiles = 10;
    LoadDefaultExtMap();  ///User extension mapping will be read with ReadIni()
    m_dontPrintNums = true;  ///By default don't print Line numbers
    m_htmlAC = true; m_htmlBR = false; /// Html AutoCompl, Auto BRs
    m_startLex = wxSTC_LEX_NULL;
    g_printData->SetPaperId(wxPAPER_LETTER); /// 8.5 x 11
    g_pageSetupData->SetMarginTopLeft( wxPoint(10, 10) ) ; ///left, top
    g_pageSetupData->SetMarginBottomRight( wxPoint(10, 15) ) ;  ///right, bottom
    g_pageSetupData->SetPrintData( * g_printData );  ///Associate the two
    
    /// READ IN USER VALUES and history
    ReadIni( READHISTORY | READSTYLES | READPRINT, true ); //! Check for preferences in zion.ini //update recentDirs menu // should be before edit instantiate//Loads startEol

    /*! Instantiate our edit control */
    m_zedit = new ZionEdit( this, this, str2, false, wxID_ANY, wxDefaultPosition, wxSize( 600, 740) );
    if( m_zedit->m_min ) {  ///For speed in FindInAllFiles
        m_zedit->SetUseHorizontalScrollBar(false);
        m_zedit->SetUseVerticalScrollBar(true);
    }
    m_zedit->SetFocus();
    
    ///Let the edit module load its own values when it's good and ready.
    //m_zedit->SetEolDefault(m_startEol);  m_zedit->ShowEOLMode();

    ///TESTING
    //mini_zedit = new wxStyledTextCtrl( this, wxID_ANY, wxDefaultPosition, wxSize( 600, 100)  );
    //! Layout
     wxBoxSizer *topsizer = new wxBoxSizer( wxVERTICAL );
     topsizer->Add(m_zedit, wxSizerFlags(1).Center().Border(wxALL, 0).Expand());
     //topsizer->Add(mini_zedit, wxSizerFlags(0).Center().Border(wxALL, 2).Expand());
     SetSizer(topsizer);  //!use the sizer for layout
    
    /** IPC **/
    m_advisedData = _T(""); ///We really shouldn't need this, but safety
    m_advisedReplData = _T(""); ///We really shouldn't need this, but safety
    m_pidToClose = _T("");
    m_pidToRaise = _T("");
    zserver = NULL; zclient = NULL; m_ipcOk = false; m_styleLock =false; //! m_styleLock for server use only
    ServerOrClient(); ///Determine if server or client. Will set the ipcOk etc
    
    m_iniDiff = true;
    if( !zserver && m_ipcOk && zconnection) {
            if( m_iniDir == zconnection->Request("Zionini") ) m_iniDiff = false;
            //! Diag// SetStatusText( m_iniDiff ? "Ini Different" : "Ini same" );
    } //! If m_iniDiff do local push of files/dirs, otherwise do remote push.
     
    //! Is there a file to open?
    if( fileArg != "" )  m_zedit->DirectOpen(fileArg, true);  //! fileArg's Directory will be pushed by DirectOpen //"true" to do just a local push
    //! Are we in our exe dir? //str2 = current working dir
    else if( str2 != m_exeDir ) {  //! We were probably called from an "Open With" or cmd line
        PushDir(str2, true);  //! So save the current dir //and don't broadcast readhistory
        /// Edit module will automatically initialize to the current dir //m_zedit->m_currentDir = str2;    //! Notify edit module of new dir 
    }
    else {      //! If we are not opening a file and we are in our exe dir, go to most recent dir- This is normal startup from an icon.
        str = recentDirsMenu->GetLabel(ZION_DIR_1);
        if(str != wxEmptyString) {
            wxSetWorkingDirectory(str);
            m_zedit->m_currentDir = str;    //! Notify edit module of new dir
        }
    }
    
    ///Get the last replace string
    if( !zserver && m_ipcOk && zconnection) {
        if( (str2 = zconnection->Request("ReplStr")) != wxEmptyString )
            m_zedit->m_findReplData.UpdateReplString( str2 );
    }
    
    m_bookTimer.SetOwner(this, ZION_BOOKMARKTIMER); /** This works! **/
    m_bookTimer.Start(1600);  //2000
    
}

ZionFrame::~ZionFrame() {} ///Assuming frames objects are automatically deleted on Destroy(); Remember delete more than once == Lockup!
//----------------------------------------------------------------------------------

///bool was the dialog OK'd or Cancel
///wxString = Empty - This window chosen
///wxString = NonEmpty - New window chosen
bool ZionFrame::DoWinLocDlg( wxString& str ) {
    if( m_zedit->m_currentFilename == "Untitled" && m_zedit->m_dirty == false ) { ///Force this window
        str = wxEmptyString;
        return true;
    }
    WinLocDlg winLoc(this);
    if( winLoc.ShowModal() == wxID_OK ) {
        if( winLoc.m_newWin->IsChecked() ) { ///NEW Window chosen
            switch( winLoc.m_radio->GetSelection() ) {
                case 0:  str = _T("-l"); break;
                case 1:  str = _T("-t"); break;
                case 2:  str = _T("-r"); break;
                case 3:  str = _T("-tl"); break;
                case 4:  str = _T("-c"); break;
                case 5:  str = _T("-tr"); break;
                case 6:  str = _T("-bl"); break;
                case 7:  str = _T("-b"); break;
                case 8:  str = _T("-br"); break;
                default: str = wxEmptyString;
            }
        }
        else str = wxEmptyString;  ///THIS WINDOW chosen
        return true;
    }
    return false;
}

wxString ZionFrame::PickupExecStr()
{
    wxString str;  ///wxStrings init to empty
    if( !zserver && m_ipcOk && zconnection) {
        str = zconnection->Request("ExecStr"); ///Need the conv to a string
    }
    return str;
}

void ZionFrame::OnAutoBookToggle( wxCommandEvent & WXUNUSED(event) )
{
    m_autoBook = ! m_autoBook;
}

void ZionFrame::OnAutoCompToggle( wxCommandEvent & WXUNUSED(event) )
{
    g_acActive = ! g_acActive;
}

void ZionFrame::OnCallTipsToggle( wxCommandEvent & WXUNUSED(event) )
{
    g_ctActive = ! g_ctActive;
}

void ZionFrame::OnCTACToggle( wxCommandEvent & WXUNUSED(event) )
{
    g_ctActive = g_acActive =  ! ( g_acActive || g_ctActive ); ///If any are on, turn both off
    if( ! g_ctActive ) { ///Only need to check one
        m_zedit->AutoCompCancel();  m_zedit->CallTipCancel();
    }
}

void ZionFrame::OnAutoZoomToggle( wxCommandEvent & WXUNUSED(event) )
{
    m_autoZoom = ! m_autoZoom;
}

void ZionFrame::OnShowCallTip( wxCommandEvent & WXUNUSED(event) )
{  
    wxString str(_T("\001 1 of 2 \002 MyFunction( int param1, int param2 )\nMyFunction2( int param1, long param2 )") ); //
    m_zedit->CallTipShow( m_zedit->GetCurrentPos(), str.c_str() );
    m_zedit->CallTipSetHighlight( 23, 33 );
}

void ZionFrame::OnClearStatus( wxCommandEvent & WXUNUSED(event) )
{
    SetStatusText( _T("") );  SetStatusText( _T(""), 4 );
}

void ZionFrame::OnBookmarkTimer( wxTimerEvent & WXUNUSED(event) )
{
    static int topLine; //lineHits made into a public member,  ///Line numbers will be unity based
    static bool markSet; ///Track whether bookmark set for this topLine;
    static bool zoomSet; ///Track whether zoom set for this topLine;
    static bool afterZoom; ///Track whether we have scrolled on zoom
    int fvLine = m_zedit->GetFirstVisibleLine();
    //Diag//SetStatusText( wxString::Format(_T("TLine: %d, FVLine: %d, LineH: %d, %s"), topLine, fvLine, m_lineHits, markSet? "Set" : "Not Set" ) );
    
    if( ! afterZoom  && fvLine != topLine ) {  ///On normal vertical scroll we have a new top line
        topLine = fvLine;
        m_lineHits = 0;
        markSet = false;  zoomSet = false; ///New location
        if( m_autoZoom ) m_zedit->SetZoom( (int) m_zoomLevel ); ///(ZOOM OUT) We are probably moving so zoom back out to our startup zoom level
    }
    else if( !markSet && m_autoBook && m_lineHits > 18 /*&& topLine !=0*/ ) { ///We use the session autobookmark enable instead of the global here
        wxCommandEvent anEvent( wxEVT_COMMAND_MENU_SELECTED, ZION_ADDBOOKMARK ); ///type, id
        AddPendingEvent(anEvent);
        markSet = true;
        m_lineHits = 0;  ///reset
    }
    else if( !zoomSet && m_autoZoom && m_lineHits > 12 && 
        m_zedit->GetSelectionStart() == m_zedit->GetSelectionEnd() ) { ///Nothing selected /*&& ! zoomSet && m_zedit->GetWrapMode() == wxSTC_WRAP_NONE*/   ///Our session enable will control immediate zoom . autoZoom  will synch with global on readini
         ///When wrapping on, Scintilla fails in figuring  how many lines are on screen
        int line = m_zedit->GetCurrentLine(); int lineCnt = m_zedit->GetLineCount();
        bool los = ( line >= fvLine && line < fvLine + m_zedit->LinesOnScreen() ); /// los is line on screen
        ///If last line not on screen okay to zoom (avoid loop at end of document)   ///Note fvLine + LinesOnScreen = LineCount   -->  is true when last line is on screen
        if( line < lineCnt - 3 ) { /// not 2 last lines
            ///Zoom in
            m_zedit->SetZoom( m_zedit->GetZoom() +1 ); /// Zoom in
            zoomSet = true;
            ///Make sure current line is comfortably on the screen
            if( los ) {  ///If current line wasn't previously on screen (before zoom) no sense scrolling
                m_zedit->ScrollFromEdge();  ///If current line is too close to the edge then attempt to get it off the edge
            }
        }
        ++m_lineHits;  ///for the sake of the other checks. Line hits should ALWAYS be allowed to increment
        //topLine = m_zedit->GetFirstVisibleLine();  ///After zoom first visible line might be changed, so lock it in at wherever it is.
        afterZoom = true;
    }
    else if( afterZoom ) { ///we should hit this one cycle after a zoom ///we had to do this to stabilize when line wrapping is on
        topLine = fvLine;
        afterZoom = false;
        ++m_lineHits;
        //Diag//SetStatusText( wxString::Format(_T("After Zoom - TLine: %d, FVLine: %d, LineH: %d, %s"), topLine, fvLine, m_lineHits, markSet? "Set" : "Not Set" ) );
    }
    
    else if( ! markSet && fvLine == topLine ) { ///If we hadn't scrolled in one timer pulse and hadn't already set a bookmark. This works because bookmarking occurs AFTER AutoZoom
        ++m_lineHits;
    }
    
    ///If we have a process running asynch
    if( m_execProcess != NULL ) {
        ReadIfHasInput();
    }
}

void ZionFrame::OnDropFiles( wxDropFilesEvent & event )
{
    wxString * strPtr = (wxString *) NULL;
    strPtr = event.GetFiles();
    if( strPtr == NULL || ! m_zedit->CheckAllowSave() ) return;
    m_zedit->DirectOpen( strPtr[0] );  ///DirectOpen will do a filename syntax check so no need to repeat that here
}

void ZionFrame::DoExecSubProcess( const wxString & execStr )
{
    ///Set Text Mode
    wxCommandEvent anEvent( wxEVT_COMMAND_MENU_SELECTED, ZION_NULL ); ///type, id
    AddPendingEvent(anEvent);
    
    m_execStr = execStr;
    wxCommandEvent anotherEvent( wxEVT_COMMAND_MENU_SELECTED, ZION_RUNANDCAPTURE ); ///type, id
    AddPendingEvent(anotherEvent);
}

void ZionFrame::OnExecSubProcess( wxCommandEvent & event )
{
    PrependToPopup( event.GetId() ); ///LAST CMD to popup    ///If we already have a subprocess running then we have to wait till it's complete
    if( m_execProcess == (wxProcess *) NULL ) {
        wxString wdir, origWdir;
        //------------------------------------------ Ascertain exec string ----------------------------------        
        
         ///SHOW DIALOG? ///If m_execStr is not empty this function was probably initiated from another window (with the -e option). m_execStr is used just for passing in a string to execute
        if( m_execStr == wxEmptyString ) { 
            ///Pick up Recent dirs from menu
            wxArrayString rdirs;
            for( int i=0; i< NUMRECENTDIRS; i++) {
                if( dirSubItems[i] == NULL ) break; ///We have hit the end of the used dirSubItems
                rdirs.Add( dirSubItems[i]->GetLabel() );
            }
            ExecDialog execDia( this, m_execAppend, m_execHistory, m_lastExecStr, m_captureOut, m_runWithCmd, &rdirs,
                    (m_lastWdir == wxEmptyString? wxGetCwd() : m_lastWdir ) );
            if( execDia.ShowModal() != wxID_OK )  return;
            // --------------- Get and Process execStr ------------------------
            m_execStr = execDia.m_execCombo->GetValue();
            ///Save execStr
            m_lastExecStr = m_execStr; ///So we can preload our combo
            PushExecStr( m_execStr );  ///IPC based update to m_execHistory
            ///Parse Exec String
            int n; wxString str3; int count=0; char ch; bool quotesReq = false;
            while( count++ < 10 && (n=m_execStr.Find('#') ) != -1 ) { ///Need count to prevent endless loop in case of unknown # param
            /// ...hence an exec str is limited to 10 params
                ch = m_execStr.GetChar(n+1);
                ///No need for else if in these since ch cannot match more than one of these at the same time
                if( ch == 'P' ) {   ///Full path
                    str3 = m_zedit->m_currentDir + "\\" + m_zedit->m_currentFilename;
                    m_execStr.Replace( "#P", str3 /* , true = default = replace all occurrences */ );
                }
                if( ch == 'p' ) {   ///Path sin filename
                    str3 = m_zedit->m_currentDir;
                    m_execStr.Replace( "#p", str3 );
                }
                if( ch == 'f' ) {   /// filename with extension
                    str3 = m_zedit->m_currentFilename;
                    m_execStr.Replace( "#f", str3 );
                }
                if( ch == 'b' ) {   /// filename without extension (basename)
                    if( m_zedit->m_currentFilename.Find('.') != -1 )
                        str3 = m_zedit->m_currentFilename.BeforeLast('.');
                    else str3 = m_zedit->m_currentFilename;
                    m_execStr.Replace( "#b", str3 );
                }
                if( ch == 'D' ) {   /// ISO Date
                    wxDateTime now = wxDateTime::Now();
                    str3.Printf( _T("%s"), now.FormatISODate() );
                    m_execStr.Replace( "#D", str3 );
                }
                if( ch == 'T' ) {   /// ISO Time
                    wxDateTime now = wxDateTime::Now();
                    str3.Printf( _T("%s"), now.FormatISOTime() );
                    m_execStr.Replace( "#T", str3 );
                }
                if( ch == 'Z' ) {   /// Full path to zion.exe
                    str3 = m_exeDir +_T("\\zion.exe");
                    m_execStr.Replace( "#Z", _T("\"") + str3 + _T("\"") );
                }
                if( ch == 'z' ) {   /// Just the path to zion.exe
                    str3 = m_exeDir;
                    m_execStr.Replace( "#z", str3 );
                }
                if( ch == '#' ) {   /// Allow the # char
                    m_execStr.Replace( "##", "#" );
                }
                ///Any parameter having spaces will need quotes
                if( str3.Find(' ') != -1 ) quotesReq = true;
            }///End While
            
            if( (m_runWithCmd= execDia.m_rbCmd->GetValue()) == true ) {  ///Run via cmd.exe?
                wxString str;
                if( ! wxGetEnv("ComSpec", &str) )  str = _T("%COMSPEC%");
                str.Append(_T(" /C ") );
                m_execStr.Prepend( str );
            }
            
            if( quotesReq && m_execStr.Find('"') == -1 ) {
                if( wxYES != wxMessageBox( wxString::Format(_T("Exec String:\n%s\ncontains spaces and so needs to be quoted with double quotes.\nContinue anyway?"), m_execStr),
                        _T("Double quotes needed!"), wxYES_NO ) ) {
                    //m_execAppend = false;
                    m_execStr = wxEmptyString;
                    return;
                }
            }
            ///Poke parsed execStr to server for pickup on external processing
            if( ! m_iniDiff && !zserver && m_ipcOk ) {
                if( ! zconnection->Poke("ParsedExecStr", (wxChar *) m_execStr.c_str()) ) {
                    wxMessageBox( _T("Failed storing Parsed Exec String to the server") );
                    m_execAppend = false;  m_execStr = wxEmptyString; return;
                }
            }
            else m_zExecStr = m_execStr;  ///server or standalone
            
            ///Pickup other parameters (other than exec string)
            m_captureOut = execDia.m_chkCaptureOut->GetValue(); ///Append to this window or exec in new window?
            if( m_captureOut ) m_execAppend = execDia.m_rbAppend->GetValue(); ///If we are not capturing, no need to pick up m_execAppend
            wdir = execDia.m_wdir->GetValue();  ///Working dir
            m_lastWdir = wdir;
        }
        ///EXTERNAL CALL
        else { 
            m_execAppend = true;  ///use this window and signal local processing
            m_captureOut = true;
            ///Turn wrapping on
            m_zedit->SetWrapMode(wxSTC_WRAP_WORD );
        }
        //------------------------------------------ Process ---------------------------------- 
        //wxMessageBox( m_execStr, _T("Exec String") );  //Diag
        ///Change Directory if dialog used
        origWdir = wxGetCwd();
        if( wdir != wxEmptyString ) {  ///wdir set in the dialog
            wxSetWorkingDirectory( wdir );
        }
        ///PROCESS LOCALLY
        if( m_captureOut && m_execAppend ) { ///then Process Locally
            m_execProcess = new wxProcess( this, ZION_SUBPROCESS );
            m_execProcess->Redirect();
            
            if( 0 == (m_execLastPid = wxExecute( m_execStr.c_str(), wxEXEC_ASYNC /* | wxEXEC_NOHIDE*/, m_execProcess )) ) { /// wxExecute( char **
                SetStatusText(_T("Subprocess failed to execute correctly") );
                delete m_execProcess;  m_execProcess = (wxProcess *) NULL; ///should we detach here? I figure since the process failed to execute, it didn't have a chance to delete itself, so we do the delete
            }
            else { ///Sub Process Launched
//                m_execNotify = new wxMessageDialog( this,  _T("Please WAIT until process completes... "),
//                        _T("Waiting for Process"), wxOK );
//                m_execNotify->Show(true);
//                    delete m_execProcess;  m_execProcess = (wxProcess *) NULL; ///Done with that process
                //m_execNotify = ( wxMessageDialog * ) NULL;
                wxString stat = wxString::Format(_T("PLEASE WAIT for Command Completion (%ld)"), m_execLastPid );
                SetStatusText( stat ); 
                if( m_zedit->GetCurrentFilename() == "Untitled" ) SetTitle( stat );
                //this->Disable();
            }
        }
        ///PROCESS EXTERNALLY
        else if( m_captureOut ) { ///Process externally
            wxString str2 = _T("\"") + m_exeDir + _T("\\zion.exe\" -br");  ///Quote the executeable path in case of spaces
            str2.Append( _T(" -d\"") + ( wdir == wxEmptyString ? wxGetCwd() : wdir ) + _T("\"") );  ///(We should use quotes in case path has spaces) 
            str2.Append( _T(" -Zexec") ); ///Notify the new window to pick up its exec string from the server.
            wxExecute( str2 );
            SetStatusText( str2 );
        }
        ///RUN WITHOUT CAPTURING
        else wxExecute( m_execStr );
 
  //------------------------------------ Resets ----------------------------------------- 
        m_execStr = wxEmptyString; ///reset
        wxSetWorkingDirectory( origWdir ); ///Reset working directory
    }
    else {
        wxKillError rc;
        if( ! wxProcess::Exists(m_execLastPid) ) {
            wxLogStatus( _T("Process %ld does not exist"), m_execLastPid );  return;
        }
        if( wxYES == wxMessageBox( _T("Subprocess function already in use\nAttempt to end the currently executing process?"),
                _T("End Previously Spawned Process?"), wxYES_NO ) ) {
            rc = wxProcess::Kill( m_execLastPid, wxSIGTERM, wxKILL_CHILDREN );
            if( rc == wxKILL_OK ) wxLogStatus(_T("Process %ld safely terminated"), m_execLastPid );
//            else if( wxYES == wxMessageBox( _T("Subprocess is not terminating gently\nDo you want to forcibly kill the process?"),
//            _T("Kill Spawned Process?"), wxYES_NO ) ) {
//                rc = wxProcess::Kill( m_execLastPid, wxSIGKILL, wxKILL_CHILDREN );
//                if( rc == wxKILL_OK ) wxLogStatus(_T("Process %ld killed"), m_execLastPid );
//                else {
//                    wxLogStatus( _T("Could not forcibly kill process.") );
//                    m_execProcess->Detach(); m_execProcess = NULL;
//                }
//            }
            else {
                switch(rc) {
                    case wxKILL_ACCESS_DENIED:
                        SetStatusText(_T("Access Denied for process termination") );  break;
                    case wxKILL_NO_PROCESS:
                        SetStatusText(_T("No Such Process") );  break;
                    case wxKILL_ERROR:
                        SetStatusText(_T("Unknown Error while terminating process") );  break;
                }
            }
        }
    }
}

void ZionFrame::GetOutErrStream( wxInputStream* outerr )
{
    if( outerr != NULL ) {
        ///Read the stream
        wxString data, byte;
        while( outerr->CanRead() ) {
            outerr->Read( wxStringBuffer(byte, 1), 1 );  ///wxStringBuffer will give access to a wxString's buffer as a pointer. Wow! ///read one byte at a time
            if( outerr->LastRead() <= 0 ) break;
            data += byte;
        }
        m_zedit->AppendText( data );
    }
}

bool ZionFrame::ReadIfHasInput()
{
    bool hasInput = false;  ///Todo wrap below in if( m_execProcess != NULL )
    if( m_execProcess->IsErrorAvailable() ) {
//        SetStatusText(wxString::Format(_T("Standard error of %d is available. Exit code: %d"), event.GetPid(), event.GetExitCode() ) );
        m_zedit->AppendText(_T("\n------------- STANDARD ERROR -------------\n") );
        wxInputStream* errstream = m_execProcess->GetErrorStream();
        GetOutErrStream( errstream );
        m_zedit->AppendText(_T("------------END STANDARD ERROR-----------\n") );
        hasInput = true;
    }
    if( m_execProcess->IsInputAvailable() ) {
//        SetStatusText(wxString::Format(_T("Standard output of %d is available. Exit code: %d"), event.GetPid(), event.GetExitCode() ) );
        wxInputStream* instream = m_execProcess->GetInputStream();
        GetOutErrStream( instream );
        hasInput = true;
    }
    if( hasInput )  {
        m_zedit->GotoLine( m_zedit->GetLineCount() - 1);
        m_zedit->EnsureVisible(m_zedit->GetCurrentPos());
    }
    return hasInput;
}

void ZionFrame::OnEndSubProcess( wxProcessEvent & event )
{
    while( ReadIfHasInput() ) ;
    m_zedit->GotoPos( m_zedit->GetLength() > 2 ? (m_zedit->GetLength() - 2) : 0 );
    wxString str = wxString::Format(_T("Exit code: %d, Process (%ld) ended. "), event.GetExitCode(), event.GetPid() );
    SetStatusText( str );  ///Clear status so we know subprocess ended
    if( m_zedit->GetCurrentFilename() == "Untitled" ) SetTitle( str );
    delete m_execProcess;  m_execProcess = (wxProcess *) NULL; ///Done with that process
    ///Try to play a sound
    printf( _T("\a") );
///It seems sound is not compiled into my wxWidgets
//    wxString str;
//    if( ! wxGetEnv(_T("SystemRoot"), &str) )  str = _T("c:\\windows");
//    
//    wxSound finito( str += _T("\\media\\notify.wav") );
//    if( finito.IsOk() )  finito.Play();
}

void ZionFrame::OnExternalRaise(wxCommandEvent & )
{
    //SetStatusText(_T("External Raise Requested") ); ///Diag- but leave in
    //if( ! IsActive() )  RequestUserAttention();
    if( ! IsIconized() ) Iconize(true);
    Iconize(false); Raise();  SetFocus();
}

void ZionFrame::OnExternalClose(wxCommandEvent & ) ///Used by FilesResults Dialog
{
    //SetStatusText(_T("External Close Requested") ); ///Diag- but leave in
    //if( ! IsActive() )  RequestUserAttention();
    m_extClose = true;
    Close(); ///generates a wxCloseEvent
}

void ZionFrame::OnExplore( wxCommandEvent & event ) {
    ///Show a RecentDirsDlg 
    wxString str(_T("explorer "));
    wxString startDir = m_zedit->m_currentDir; ///Save in case SaveAs dialog canceled
    wxArrayString rdirs;
    for( int i=0; i< NUMRECENTDIRS; i++) {
        if( dirSubItems[i] == NULL ) break; ///We have hit the end of the used dirSubItems
        rdirs.Add( dirSubItems[i]->GetLabel() );
    }
    if( rdirs.GetCount() > 0 ) {
        StartFolderDlg stDlg(this, GetPosition(), GetSize().GetWidth(), rdirs, _T("Explore From: "));
        if( stDlg.ShowModal() != wxID_OK ) return;
        wxString selDir = stDlg.m_list->GetStringSelection(); ///Pick up the dir we want to start browsing from
        if( stDlg.m_list->GetSelection() != 0 )  startDir = selDir; 
    }
    wxExecute( str + _T("\"") +startDir + _T("\"") );
    ///TESTING LAST CMD to popup
    PrependToPopup( event.GetId() );
}

void ZionFrame::OnBugReport(wxCommandEvent & WXUNUSED(event)) {
    wxExecute( wxString::Format(_T("%s \"http://www.zionedit.org/zionedit/feedback.php?version=%s\""),
            g_pathToBrowser, version ) );
}

void ZionFrame::OnInfo(wxCommandEvent & WXUNUSED(event)) {
    wxString str;
    int width =0, height=0, xpos=0, ypos=0;
    GetSize( &width, &height );   GetPosition( &xpos, &ypos );

    str.Printf( _T("\nMain Window Position and Size:\n [x,y   w x h]    %d,%d   %dx%d\n\n\
Total Lines: %d\nTotal Characters (including new lines): %d\n\nCurrent Line: %d\nCurrent Column: %d\nCurrent Position (zero-based): %d\n\n\
zion.exe location: %s\nzion.ini, history.ini:  %s\n\nThis window is %s"),
    xpos, ypos, width, height, m_zedit->GetLineCount(),  m_zedit->GetTextLength(), m_zedit->GetCurrentLine() +1,
    m_zedit->GetCurrentPos() - m_zedit->PositionFromLine(m_zedit->GetCurrentLine()) +1, ///Column 
    m_zedit->GetCurrentPos(), m_exeDir, m_iniDir,
     ( zserver ? _T("the Server") : _T("a Client") ) );
     if( !zserver) {
        str.Append( wxString::Format( _T("\nzion.ini file %s server's\n"), ( m_iniDiff ? _T("different than") : _T("same as") ) ) );
        if( priority == _T("1") )  str.Append( _T("This window is next in line to become server.\n") );
    }
    wxMessageBox( str, _T("Editor Statistics") );
}

void ZionFrame::OnHelp(wxCommandEvent & WXUNUSED(event)) {
    wxExecute( wxString::Format(_T("\"%s\" \"%s\\help.html\""), ///We will always quote paths
            g_pathToBrowser, m_exeDir ) );
}

void ZionFrame::OnOnlineDocs(wxCommandEvent & WXUNUSED(event)) {
    wxExecute(wxString::Format(_T("\"%s\" \"http://zionedit.org/index.php?option=com_content&task=view&id=21&Itemid=34\""),
            g_pathToBrowser /* , version */ ));
}

void ZionFrame::OnGoogleIt(wxCommandEvent & WXUNUSED(event)) {
    wxString str;
    if( m_zedit->GetSelectionStart() != m_zedit->GetSelectionEnd() )  str = m_zedit->GetSelectedText();
    else str = m_zedit->GetContextWord();

    if( str != wxEmptyString && str.Len() > 3 ) {
        ///Put the word on the clipboard
        if( wxTheClipboard->Open() ) {
            wxTheClipboard->SetData( new wxTextDataObject(str) ); /// This data objects is held by the clipboard// so do not delete it in the app.
            wxTheClipboard->Close();
        }
        wxMessageBox( _T("\"") + str + _T("\" has been placed on the Clipboard. Paste into the search box\n(Google restricts automatic searches)") );
        wxExecute(wxString::Format(
        //http://www.google.com/search?hl=en&q=response.write&btnG=Google+Search
//            _T("\"%s\" \"http://www.google.com/search?hl=en&q=%s&btnG=Google+Search\""),
            _T("\"%s\" \"http://www.google.com\""),
            g_pathToBrowser/*, m_zedit->GetSelectedText()*/ ) );
    }
}

void ZionFrame::OnCheckLatest(wxCommandEvent & WXUNUSED(event)) {
    wxExecute(wxString::Format(_T("\"%s\" \"http://zionedit.org/index.php?option=com_content&task=view&id=15&Itemid=29\""), /* ?version=%s */
            g_pathToBrowser /* , version */ ));
}

void ZionFrame::OnHtmlPreview(wxCommandEvent & WXUNUSED(event))
{
    if( ! m_zedit->CheckAllowSave() )  return;
    wxString str; str.Printf( _T("\"%s\" %s\\%s"), g_pathToBrowser, m_zedit->m_currentDir,
            /* wxFileName::GetPathSeparator(), */ m_zedit->GetCurrentFilename() );
    wxExecute(str);
}

//---------------------------------------------------------------------------------
/// print event handlers
void ZionFrame::OnPrintSetup (wxCommandEvent &WXUNUSED(event))
{
//    g_pageSetupData->SetPrintData( * g_printData ); 
    wxPageSetupDialog pageSetupDialog(this, g_pageSetupData);
    if( pageSetupDialog.ShowModal() == wxID_OK ) {
        ///Save Print data
        (*g_printData) = pageSetupDialog.GetPageSetupData().GetPrintData();
        (*g_pageSetupData) = pageSetupDialog.GetPageSetupData();
        PushPrint();
    }
}

void ZionFrame::OnPrintPreview (wxCommandEvent &WXUNUSED(event))
{
    wxPrintDialogData printDialogData( *g_printData );
    wxPrintPreview *preview =  new wxPrintPreview( new EditPrint (m_zedit), new EditPrint(m_zedit),
            &printDialogData );
    //preview->SetZoom(60);

    if (!preview->Ok()) {
        delete preview;
        wxMessageBox(_("There was a problem with previewing.\n\
                         Perhaps your current printer is not correctly installed?"),
                      _("Previewing"), wxOK);
        return;
    }
    wxRect rect = DeterminePrintSize();
    wxPreviewFrame *frame = new wxPreviewFrame (preview, this, _("Print Preview"));
    frame->SetSize (rect);
    frame->Centre(wxBOTH);
    frame->Initialize();
    frame->Show(true);
}

void ZionFrame::OnPrint (wxCommandEvent &WXUNUSED(event))
{
    wxPrintDialogData printDialogData( *g_printData );
    wxPrinter printer(&printDialogData);
    EditPrint printout(m_zedit);
    
//    int minpg=0; int maxpg=0; int pgFrom=0; int pgTo=0;
//    printout.GetPageInfo( &minpg, & maxpg, &pgFrom, & pgTo );
    //Diag//SetStatusText( wxString::Format(_T("minpg: %d, maxpg: %d, pgFrom: %d, pgTo: %d"),
    //        minpg, maxpg, pgFrom, pgTo ));

    if (!printer.Print (this, &printout, true)) {
        if (wxPrinter::GetLastError() == wxPRINTER_ERROR) {
        wxMessageBox (_("There was a problem with printing.\n\
                         Perhaps your current printer is not correctly?"),
                      _("Previewing"), wxOK);
            return;
        }
    }
    ///Careful I believe the printout object is deleted immediately after printing
    (*g_printData) = printer.GetPrintDialogData().GetPrintData();
}

wxRect ZionFrame::DeterminePrintSize()
{
    wxSize scr = wxGetDisplaySize();
    // determine position and size (shifting 16 left and down)
    wxRect rect = GetRect();  ///pos,size of our frame
    rect.x += 16;
    rect.y += 16;
    rect.width = wxMin (rect.width, (scr.x - rect.x));
    rect.height = wxMin (rect.height, (scr.y - rect.y));

    return rect;
}
//--------------------------------------

inline void ZionFrame::LoadDefaultExtMap()
{
    ext2lex[_T("txt")] = wxSTC_LEX_NULL;
    ext2lex[_T("text")] = wxSTC_LEX_NULL;
    ext2lex[_T("ini")] = wxSTC_LEX_NULL;
    ext2lex[_T("cfg")] = wxSTC_LEX_NULL;
    ext2lex[_T("log")] = wxSTC_LEX_NULL;
    ext2lex[_T("cpp")] = wxSTC_LEX_CPP;
    ext2lex[_T("c")] = wxSTC_LEX_CPP;
    ext2lex[_T("cxx")] = wxSTC_LEX_CPP;
    ext2lex[_T("h")] = wxSTC_LEX_CPP;
    ext2lex[_T("hpp")] = wxSTC_LEX_CPP;
    ext2lex[_T("hxx")] = wxSTC_LEX_CPP;
    ext2lex[_T("vb")] = wxSTC_LEX_VBSCRIPT;
    ext2lex[_T("vbs")] = wxSTC_LEX_VBSCRIPT;
    ext2lex[_T("bas")] = wxSTC_LEX_VBSCRIPT;
    ext2lex[_T("java")] = wxSTC_LEX_JAVA;
    ext2lex[_T("cs")] = wxSTC_LEX_CS;
    ext2lex[_T("sh")] = wxSTC_LEX_CPP;
    ext2lex[_T("f")] = wxSTC_LEX_F77;
    ext2lex[_T("for")] = wxSTC_LEX_F77;
    ext2lex[_T("f77")] = wxSTC_LEX_F77;
    ext2lex[_T("f90")] = wxSTC_LEX_FORTRAN;
    ext2lex[_T("f95")] = wxSTC_LEX_FORTRAN;
    ext2lex[_T("js")] = wxSTC_LEX_CPP;
    ext2lex[_T("hpp")] = wxSTC_LEX_CPP;
    ext2lex[_T("cc")] = wxSTC_LEX_CPP;
    ext2lex[_T("css")] = wxSTC_LEX_CSS;
    ext2lex[_T("api")] = wxSTC_LEX_HTML;
    ext2lex[_T("htm")] = wxSTC_LEX_HTML;
    ext2lex[_T("html")] = wxSTC_LEX_HTML;
    ext2lex[_T("php")] = wxSTC_LEX_HTML;
    ext2lex[_T("inc")] = wxSTC_LEX_HTML;
    ext2lex[_T("xml")] = wxSTC_LEX_HTML;
    ext2lex[_T("jhtml")] = wxSTC_LEX_HTML;
    ext2lex[_T("rhtml")] = wxSTC_LEX_HTML;
    ext2lex[_T("shtml")] = wxSTC_LEX_HTML;
    ext2lex[_T("asp")] = wxSTC_LEX_HTML;
    ext2lex[_T("aspx")] = wxSTC_LEX_HTML;
    ext2lex[_T("ascx")] = wxSTC_LEX_HTML;
    ext2lex[_T("htt")] = wxSTC_LEX_HTML;
    ext2lex[_T("cfm")] = wxSTC_LEX_HTML;
    ext2lex[_T("tpl")] = wxSTC_LEX_HTML;
    ext2lex[_T("dtd")] = wxSTC_LEX_HTML;
    ext2lex[_T("hta")] = wxSTC_LEX_HTML;
    ext2lex[_T("svg")] = wxSTC_LEX_HTML;
    ext2lex[_T("docbook")] = wxSTC_LEX_HTML;
    ext2lex[_T("sgml")] = wxSTC_LEX_HTML;
    ext2lex[_T("py")] = wxSTC_LEX_PYTHON;
    ext2lex[_T("pyw")] = wxSTC_LEX_PYTHON;
    ext2lex[_T("rb")] = wxSTC_LEX_RUBY;
    ext2lex[_T("rbw")] = wxSTC_LEX_RUBY;
    ext2lex[_T("ruby")] = wxSTC_LEX_RUBY;
    ext2lex[_T("sql")] = wxSTC_LEX_SQL;
    ext2lex[_T("asm")] = wxSTC_LEX_ASM;
    ext2lex[_T("masm")] = wxSTC_LEX_ASM;
    ext2lex[_T("fasm")] = wxSTC_LEX_ASM;
    ext2lex[_T("nasm")] = wxSTC_LEX_ASM;
    ext2lex[_T("bat")] = wxSTC_LEX_BATCH;
    ext2lex[_T("lisp")] = wxSTC_LEX_LISP;
    ext2lex[_T("cl")] = wxSTC_LEX_LISP;
    ext2lex[_T("l")] = wxSTC_LEX_LISP;
    ext2lex[_T("pm")] = wxSTC_LEX_PERL;
    ext2lex[_T("pod")] = wxSTC_LEX_PERL;
    ext2lex[_T("pl")] = wxSTC_LEX_PERL;
    ext2lex[_T("perl")] = wxSTC_LEX_PERL;
    ext2lex[_T("cgi")] = wxSTC_LEX_PERL;
    ext2lex[_T("no extension")] = wxSTC_LEX_NULL;
    ext2lex[_T("mak")] = wxSTC_LEX_MAKEFILE;
    ext2lex[_T("make")] = wxSTC_LEX_MAKEFILE;
}

void ZionFrame::OnHtmlACToggle( wxCommandEvent & WXUNUSED(event) )
{
    if( ! RequestStyleLock() ) return; //! Get StyleLock
    //No need to save since we are not reloading styles //if( ! m_zedit->CheckAllowSave() ) return; //! File must first be saved
    
    if( m_zedit->m_htmlAC.active ) {
        m_zedit->m_htmlAC.active = false;
        m_zedit->m_htmlAC.inTag =false;  /// Clear these also to indicate we are dumping any partial tags also
        m_zedit->m_htmlAC.tagWaiting = false;
        SetStatusText(_T("HTML Tag AutoComplete OFF - This option will persist if Settings are saved") );
    }
    else {
        m_zedit->m_htmlAC.active = true;
        SetStatusText(_T("HTML Tag AutoComplete ON - This option will persist if Settings are saved") );
    }
    m_zedit->m_settingsDirty = true;  /// Settings have been changed
}

void ZionFrame::OnHtmlBRToggle( wxCommandEvent & WXUNUSED(event) )
{
    if( m_zedit->m_htmlAC.brActive ) {
        m_zedit->m_htmlAC.brActive = false;
        SetStatusText( _T("Auto <br/> is OFF") );
    }
    else {
        m_zedit->m_htmlAC.brActive = true;
        SetStatusText( _T("Auto <br/> is ON") );
    }
}

void ZionFrame::OnNewZion( wxCommandEvent & event )
{
    wxString str; int a = event.GetId();
    switch(a) {
        case ZION_NEWZION_TL :
            str = _T("-tl"); break;
        case ZION_NEWZION_TR :
            str = _T("-tr"); break;
        case ZION_NEWZION_BL :
            str = _T("-bl"); break;
        case ZION_NEWZION_BR :
            str = _T("-br"); break;
        case ZION_NEWZION_L :
            str = _T("-l"); break;
        case ZION_NEWZION_R :
            str = _T("-r"); break;
        case ZION_NEWZION_T :
            str = _T("-t"); break;
        case ZION_NEWZION_B :
            str = _T("-b"); break;
        case ZION_NEWZION_M :
            str = _T("-m"); break;
        case ZION_NEWZION_C :
            str = _T("-c"); break;
    }
    wxExecute( wxString::Format(_T("\"%s\\zion.exe\" -zposx%d %s"), m_exeDir, GetPosition().x, str ) );
    //SetStatusText(_T("#z\\zion.exe ") + str );
}

wxString ZionFrame::MiniStats() ///Critical to window menu operation, do not change format
{
    wxString str;
    if(zserver)  str = GetServerStatus();
    else if(m_ipcOk && zconnection) {
        str = zconnection->Request(_T("ServerStatus") );
    }
    else str = _T("");
    return str;
}

//-----------------------------------------------------------------IPC
int ZionFrame::ServerOrClient()
{   //! Return values: 0--client, 1--server, -1--error
    ///Warn unfortunately ztopic is used as CLIENT Topic - format: zion_nnn where nnn is the process id of the client
    ///On the SERVER side the created ServerConn also uses a variable named ztopic which is of the form: nnn where nnn is the process id of the client. One day we'll change this.
    zclient = NULL; //! the custom wxClient is required so that a user-overridden wxClient::OnMakeConnection member can return a wxConnection of the required class, when a connection is made.
    wxString ztopic; ztopic.Printf( _T("zion_%lu"), localPid );  //! Concatenate topic
    
    m_singleInstanceChecker = new wxSingleInstanceChecker(wxT("ZionApp"));
    if (!m_singleInstanceChecker->IsAnotherRunning()) { //! If we are alone so become server
        zserver = new MyServer;
        if( ! zserver->Create(service) )  return -1;
        //wxLogStatus("Server window");
        m_timer.SetOwner(this, ZION_IPCTIMER); /// This works!
        m_timer.Start(8000); ///every 8s, I think
        m_ipcOk = true; return 1;
    }
    else {  //! OK, there IS another one running, so try to connect to it
        zclient = new MyClient;
        retry:
        int count = 0;
        do { //! MakeConnection() will contact the server . On connection, zclient::OnMakeConnection will be called which will return a pointer to a ClientConn object
            wxMilliSleep(50);
            zconnection = (ClientConn *) zclient->MakeConnection(hostName, service, ztopic);  /// CLIENT! ztopic is Topic /* Topic is  an identifier within the service */
        }
        while ( !zconnection && ++count < 8);
        
        if (!zconnection) { 
            if( wxMessageBox(_T("Failed to make connection to server.\nRetry?"),
                _T("Client Demo Error"), wxICON_ERROR | wxYES_NO | wxCANCEL ) == wxYES )
                goto retry;
            else m_ipcOk = false; return -1; //! Don't try again
        }
        else {
            bool ok1, ok2, ok3, ok4, ok5, ok6, ok7, ok8, ok9;
            ok1=ok2=ok3=ok4=ok5=ok6=ok7=ok8=ok9=false;
            count = 0;
            do { //! We MUST do a StartAdvise or the connection will not allow advise messages to client
                wxMilliSleep(10);
                ok1 = zconnection->StartAdvise("Priority");  //! 
                ok2 = zconnection->StartAdvise("ReadHistory");  
                ok3 = zconnection->StartAdvise("ReadStyles");
                ok4 = zconnection->StartAdvise("BecomeServer");
                ok5 = zconnection->StartAdvise("ReadPrint");
                ok6 = zconnection->StartAdvise("FindUpdate");
                ok7 = zconnection->StartAdvise("Close");
                ok8 = zconnection->StartAdvise("ReplUpdate");
                ok9 = zconnection->StartAdvise("Raise");
            }
            while ( ++count < 8 && ( !ok1 || !ok2 || !ok3 || !ok4 || !ok5 || !ok6 || !ok7 || !ok8 || !ok9 ) );
             if( ! ( ok1 && ok2 && ok3 && ok4 && ok5 && ok6 && ok7 && ok8 && ok9 ) )
                { m_ipcOk = false; SetStatusText("Failure in IPC -  Close then reopen the window"); }//DEBUG//(using the opposite condition) SetStatusText("Connection to server OK");
             m_ipcOk = true; return 0;  //! For Client
        }
    }
}

ServerConn::ServerConn(const wxString& ztopic) : wxConnection()
{
    m_ztopic = ztopic;  //! Each connection will retain the pid of the connected client
    zconns[m_ztopic] = this;  //! Store pointer to this connection in our Hash Map
    styleLock = false;
    m_srcRequestFile = false;
    m_srcExecStyle = false;
    m_srcPokeFindUpdate = false;
    m_srcPokeReplUpdate = false;
    m_closing = false;
}

ServerConn::~ServerConn()
{
    zconns.erase(m_ztopic);
    //DEBUG//wxLogStatus(_T("Connection broken! %d Established"), zconns.size() );
    ///THIS STEP IS VITAL!
    wxCommandEvent anEvent( wxEVT_COMMAND_MENU_SELECTED, ZION_ASSIGNPRIORITIES );
    s_ourFrame->AddPendingEvent(anEvent);
}

/** IPC Event Handlers **/
wxConnectionBase* MyServer::OnAcceptConnection(const wxString& topic)
{
    if ( topic.BeforeFirst('_') == "zion" ) return new ServerConn(topic.AfterFirst('_'));  ///The server conn is constructed with just the pid
    return NULL;    // unknown topic
}

bool ServerConn::OnStartAdvise( const wxString& WXUNUSED(topic), const wxString& WXUNUSED(item) )
{   //! This f() must return true for the Advise subscription to be valid
   return true; //! Return true for all items
}

bool ServerConn::OnExecute(const wxString& topic, wxChar *data, int WXUNUSED(size),
        wxIPCFormat WXUNUSED(format))
{
    if( topic.AfterFirst('_') == m_ztopic ) {   //! Topic can be a kind of security check
        //wxLogStatus(wxT("Execute command from %s: %s"), m_ztopic, data);
        wxString str = data;
        if( str == _T("StyleChange") ) { //s_ourFrame->SetStatusText("Executing StyleChange", 1);
            styleLock = false; //! We release style lock because this is usually done after a writeini
            //m_srcExecStyle = true;  /// This conn is the source of the style update
            wxCommandEvent anEvent(wxEVT_COMMAND_MENU_SELECTED, ZION_ADVISE_READSTYLES );
            s_ourFrame->AddPendingEvent(anEvent);
            return true;
        }
        if( str == _T("PrintChange") ) {
            wxCommandEvent anEvent(wxEVT_COMMAND_MENU_SELECTED, ZION_ADVISE_READPRINT );
            s_ourFrame->AddPendingEvent(anEvent);
            return true;
        }
        if(str == "StyleLock") {
                styleLock = true; return true;//! Give the lock unconditionally to this connection
        }
    }
    return false;
}

bool ServerConn::OnPoke(const wxString& topic, const wxString& item,
        wxChar *data, int WXUNUSED(size), wxIPCFormat WXUNUSED(format))
{
    if( topic.AfterFirst('_') == m_ztopic ) {   //! Topic (actually pid) can be a kind of security check
        wxString str; 
        if(item == "FileInUse") {  //! This will be called after reconvergence. Each window's Readonly status should already be established, so no reply is necessary
            currentFile = data;  return true;
        }
        if(item == "ParsedExecStr") {
            str = data;  s_ourFrame->m_zExecStr = str;  //! Server only will directly update m_zExecStr
            return true;
        }
        if(item == "PushExecStr") {
            str = data;  s_ourFrame->PushExecStr(str);
            return true;
        }
        if(item == "PushFile") {
            str = data;  s_ourFrame->PushFile(str);  //! Server only will write history to zion.ini //PushFile will broadcast a readhistory event
            return true;
        }
        if(item == "PushDir") {
            str = data;  s_ourFrame->PushDir(str);
            return true;
        }
        if(item == "PruneDir") {
            str = data;  s_ourFrame->RemoveSubmenu(str);
            return true;
        }
        if(item == "Close") {
            s_ourFrame->m_pidToClose = data;
            wxCommandEvent anEvent(wxEVT_COMMAND_MENU_SELECTED, ZION_ADVISE_CLOSE );
            s_ourFrame->AddPendingEvent(anEvent);
            return true;
        }
        if(item == "Raise") {
            s_ourFrame->m_pidToRaise = data;
            wxCommandEvent anEvent(wxEVT_COMMAND_MENU_SELECTED, ZION_ADVISE_RAISE );
            s_ourFrame->AddPendingEvent(anEvent);
            return true;
        }
        if( item == _T("FindUpdate") ) {
            s_ourFrame->m_advisedData = data;
            m_srcPokeFindUpdate = true;  /// Note that this conn is the source of the update
            wxCommandEvent anEvent(wxEVT_COMMAND_MENU_SELECTED, ZION_ADVISE_FINDUPDATE );
            s_ourFrame->AddPendingEvent(anEvent);
            //s_ourFrame->SetStatusText(_T("In Poke"), 2 );
            return true;
        }
        if( item == _T("ReplUpdate") ) {
            s_ourFrame->m_advisedReplData = data;
            m_srcPokeReplUpdate = true;  /// Note that this conn is the source of the update
            wxCommandEvent anEvent(wxEVT_COMMAND_MENU_SELECTED, ZION_ADVISE_REPLUPDATE );
            s_ourFrame->AddPendingEvent(anEvent);
            //s_ourFrame->SetStatusText(_T("In Poke"), 2 );
            return true;
        }
    }
    return false;
}

///The Request mechanism can send data back to the requestor without an Adivse
wxChar* ServerConn::OnRequest( const wxString& topic, const wxString& item,
        int * WXUNUSED(size), wxIPCFormat WXUNUSED(format) )
{
    if( topic.AfterFirst('_') == m_ztopic ) {   //! Topic (actually pid) can be a kind of security check (sent automatically with request)
        wxString str;
        if( item.BeforeFirst(':') == _T("File") ) {
            str = item.AfterFirst(':');
            //! See if the file is in use; "Untitled is never in use";
            if( str.AfterLast('\\') != _T("Untitled") ) {
                m_srcRequestFile = true;  /// Note that this connection is the one requesting the "File" check
                if( s_ourFrame->CheckFileInUse(str) ) {
                    currentFile = "[" + str + "]";  return "Readonly";
                }
                else { currentFile = str;  return ""; } /// File is not already in use, save it in our connection.
            }
        }
        else if(item == "ServerStatus")  return (wxChar*) s_ourFrame->GetServerStatus().c_str();
        else if(item == "Zionini")  return (wxChar*) s_ourFrame->GetIniDir().c_str();
        else if(item == "ExecStr") return (wxChar*) s_ourFrame->m_zExecStr.c_str();
        else if(item == "ReplStr")  return (wxChar*) s_ourFrame->m_zedit->m_findReplData.GetReplString().c_str();
        else if(item == "StyleLock") {
            str = s_ourFrame->GetStyleLock( m_ztopic );
            if( str == "LockGranted" ) {
                styleLock = true; return "LockGranted"; //! Since no one else has the lock, give it to this connection
            }
            else return (wxChar*) str.c_str();  //! pid of window with lock
        }
        else if( item.BeforeFirst(':') == _T("Close") ) { ///Close will request a push of the last file in use
            m_closing = true;  ///OnAdviseReadHistory will skip this connection
            str = item.AfterFirst(':');
            s_ourFrame->PushFile(str);  //! Server only will write history to zion.ini //PushFile will broadcast a readhistory event
            str = str.BeforeLast('\\');  ///Get path
            s_ourFrame->PushDir(str);  //! Server only will write history to zion.ini //PushDir will broadcast a readhistory event
            return _T("OK");
        }
    }
    return _T("");
}

wxString ZionFrame::GetStyleLock(const wxString & ztopic, bool local)  //! For use by Server //local check
{
    Connections::iterator it; wxString str;
    //! Check connections
    for( it = zconns.begin(); it != zconns.end(); ++it ) { 
        if( it->first == ztopic ) continue; //! Skip the requestor
        if( it->second->styleLock ) return it->first; //! Return the pid of window with lock
    }
    //! Check the server (if someone other than the server is doing the requesting)
    if( ! local ) {
        if(m_styleLock) { str.Printf(_T("%lu"), localPid ); return str; }  
    }
    return "LockGranted";
}
 
bool ZionFrame::RequestStyleLock()  //! For use by Client or in a client sense
{
    if(zserver && m_ipcOk && zconns.size() > 0) m_iniDiff = false; //! Force the server to check for a lock if  there are connections
    if(m_ipcOk && ! m_iniDiff) {   //! Server always starts with m_iniDiff = true
        wxString str, stlock;
        if( ! zserver && zconnection ) { //! Client
             stlock = zconnection->Request(_T("StyleLock"));
        }
        else if(zserver) {
            str.Printf(_T("%lu"), localPid );
            stlock = GetStyleLock( str, true ); //! Check only conns
        }
        if( stlock != "LockGranted" )  { //! then stlock will contain the pid of the window with the lock
            if( stlock != "" ) stlock = " (" + stlock + ")";
            str.Printf( _T("Another window%s may have the lock for style changes.\nSave Settings in that window first (Options | Save Settings)"), stlock );
            wxMessageBox(str, _T("No Style Lock"));  return false;
        }
        else {  m_styleLock = true;  //! DIAG -
            //SetStatusText("IPC - Style Lock Granted");return true;
        }
    }
    m_styleLock = true;
    //!DIAG
    //SetStatusText("Style Lock (non-IPC)"); 
    return true;  //! If we are operating independently, give the lock
}

void ZionFrame::OnAdviseReadHistory(wxCommandEvent & WXUNUSED(event)) //! Server use only
{
    Connections::iterator it;
    for( it = zconns.begin(); it != zconns.end(); ++it ) { //!Advise All Connections 
        if( it->second->m_closing ) continue;  ///Skip window that's closing
        it->second->Advise(_T("ReadHistory"), _T("") );
        wxMilliSleep(10);
    }
}

void ZionFrame::NotifyReadStyles(wxCommandEvent & WXUNUSED(event))
{
     //! We need to contact the server if we are client to let it know to start broadcasting "ReadStyles"
    if( m_ipcOk && !zserver && zconnection ) {
        m_styleLock = false;  //! Release Style Lock after WriteIni
        zconnection->Execute( _T("StyleChange"));
    }
    if(zserver) {
        wxCommandEvent anEvent( wxEVT_COMMAND_MENU_SELECTED, ZION_ADVISE_READSTYLES );  ///type, id
        s_ourFrame->AddPendingEvent(anEvent);
    }
 }

void ZionFrame::NotifyReadPrint()
{
     //! We need to contact the server if we are client to let it know to start broadcasting "ReadPrints"
    if( !m_iniDiff && m_ipcOk && !zserver && zconnection ) {
        //m_styleLock = false;  //! Release Style Lock after WriteIni
        zconnection->Execute( _T("PrintChange"));  ///Print Settings changed
    }
    if(zserver) {
        wxCommandEvent anEvent( wxEVT_COMMAND_MENU_SELECTED, ZION_ADVISE_READPRINT );  ///type, id
        s_ourFrame->AddPendingEvent(anEvent);
    }
 }

///FIND UPDATE starts here
void ZionFrame::NotifyFindUpdate() ///This will be called by the edit module AFTER loading our m_advisedData with the new find str
{
     //! We need to contact the server if we are client to let it know to start broadcasting "FindUpdate"
    if( !m_iniDiff && m_ipcOk && !zserver && zconnection ) {
        zconnection->Poke( _T("FindUpdate"), (wxChar *) m_advisedData.c_str() );  ///Find Settings changed
        //SetStatusText(_T("Client Poking") );
    }
    if(zserver) {
        wxCommandEvent anEvent( wxEVT_COMMAND_MENU_SELECTED, ZION_ADVISE_FINDUPDATE );  ///type, id
        s_ourFrame->AddPendingEvent(anEvent);
        //SetStatusText(_T("Server about to advise") );
    }
 }

///REPL UPDATE starts here
void ZionFrame::NotifyReplUpdate() ///This will be called by the edit module AFTER loading our m_advisedReplData with the new repl str
{
     //! We need to contact the server if we are client to let it know to start broadcasting "ReplUpdate"
    if( !m_iniDiff && m_ipcOk && !zserver && zconnection ) {
        zconnection->Poke( _T("ReplUpdate"), (wxChar *) m_advisedReplData.c_str() );  ///Repl Settings changed
        //SetStatusText(_T("Client Poking") );
    }
    if(zserver) {
        wxCommandEvent anEvent( wxEVT_COMMAND_MENU_SELECTED, ZION_ADVISE_REPLUPDATE );  ///type, id
        s_ourFrame->AddPendingEvent(anEvent);
        //SetStatusText(_T("Server about to advise") );
    }
 }

///REQUEST TO RAISE (Another window, given its process id)
void ZionFrame::NotifyRaise( const wxString& pid ) ///This will be called by the edit module (FindInAllFiles)
{
    long itemPid;  pid.ToLong( &itemPid );
    if( itemPid == localPid )  return;  ///This function can be called only from the active window so do nothing
    //! We need to contact the server if we are client to let it know we want to raise another window
    if( m_ipcOk && !zserver && zconnection ) { // !m_iniDiff ///we don't care if our ini is different
        zconnection->Poke( _T("Raise"), (wxChar *) pid.c_str() );  ///Find Settings changed
        //SetStatusText(_T("Client Poking") );
    }
    if(zserver) { ///Server can advise directly
        m_pidToRaise = pid;
        wxCommandEvent anEvent( wxEVT_COMMAND_MENU_SELECTED, ZION_ADVISE_RAISE );  ///type, id
        s_ourFrame->AddPendingEvent(anEvent);
        //SetStatusText(_T("Server about to advise") );
    }
 }

///REQUEST TO CLOSE (Another window, given its process id)
void ZionFrame::NotifyClose( long pid ) ///This will be called by the edit module (FindInAllFiles)
{
    wxString str; str.Printf(_T("%lu"), pid );
    //! We need to contact the server if we are client to let it know we want to close another window
    if( m_ipcOk && !zserver && zconnection ) { // !m_iniDiff ///we don't care if our ini is different
        zconnection->Poke( _T("Close"), (wxChar *) str.c_str() );  ///Find Settings changed
        //SetStatusText(_T("Client Poking") );
    }
    if(zserver) { ///Server can advise directly
        m_pidToClose = str;
        wxCommandEvent anEvent( wxEVT_COMMAND_MENU_SELECTED, ZION_ADVISE_CLOSE );  ///type, id
        s_ourFrame->AddPendingEvent(anEvent);
        //SetStatusText(_T("Server about to advise") );
    }
 }

void ZionFrame::AdviseReadPrint(wxCommandEvent & WXUNUSED(event))
{
    Connections::iterator it;
    for( it = zconns.begin(); it != zconns.end(); ++it ) //!Advise All Connections
        it->second->Advise( _T("ReadPrint"), _T("") ); //! Read print info from zion.ini
    //! Have the server update too
    wxCommandEvent anEvent(wxEVT_COMMAND_MENU_SELECTED, ZION_READINI_PRINT );
    s_ourFrame->AddPendingEvent(anEvent);
}
 
void ZionFrame::AdviseRaise(wxCommandEvent & WXUNUSED(event))  ///Called from client conn (after Poke "Raise" ) or from the server directly
{
/// Check if the server matches
    if( m_pidToRaise == wxString::Format( _T("%lu"), localPid) ) {
        if( ! IsIconized() ) Iconize(true);
        Iconize(false); Raise(); SetFocus();
//        wxCommandEvent anEvent( wxEVT_COMMAND_MENU_SELECTED, ZION_EXTERNALRAISE );
//        AddPendingEvent(anEvent); /// We use a pending event in case a modal dialog opened (But, that's just a theory)
        //SetStatusText( wxString::Format(_T("Advising server: %s to Raise..."), m_pidToRaise )); ///DIAG
        return;
    }
    else {
        //SetStatusText( wxString::Format(_T("Advising window %s to Raise..."), m_pidToRaise )); ///DIAG
        Connections::iterator it;
        for( it = zconns.begin(); it != zconns.end(); ++it ) { /// Check All Connections
            if( it->first == m_pidToRaise ) { /// Does the process have a conn here /// keys to the hash are like: nnn, where nnn = process id (wxString keys)
                it->second->Advise( _T("Raise"), (wxChar *) _T("") ); ///Advise it to raise
            }
        }
    }
}
 
void ZionFrame::AdviseClose(wxCommandEvent & WXUNUSED(event))  ///Called from client conn (after Poke "Close" ) or from the server directly
{
    //Diag//SetStatusText( wxString::Format(_T("Advising window %s to Close..."), m_pidToClose )); ///DIAG
    Connections::iterator it;
    for( it = zconns.begin(); it != zconns.end(); ++it ) { /// Check All Connections
        if( it->first == m_pidToClose ) { /// Does the process have a conn here /// keys to the hash are like: nnn, where nnn = process id (wxString keys)
            it->second->Advise( _T("Close"), (wxChar *) _T("") ); ///Advise it to close
        }
    }
//    We may not remotely close the server.
}
 
void ZionFrame::AdviseFindUpdate(wxCommandEvent & WXUNUSED(event))  ///Called from client conn (after Poke "FindUpdate" ) or from the server
{
    ///m_advisedData is already loaded by the poke
    //SetStatusText(_T("Advising FindUpdate..."),3 );
    Connections::iterator it;
    for( it = zconns.begin(); it != zconns.end(); ++it ) { //!Advise All Connections
        if( it->second->m_srcPokeFindUpdate ) { /// Is this the requestor?
            it->second->m_srcPokeFindUpdate = false; continue; /// Turn off the flag and Skip requestor
        }
        it->second->Advise( _T("FindUpdate"), (wxChar *) m_advisedData.c_str() );
    }
    //! Have the server update too
    wxCommandEvent anEvent( wxEVT_COMMAND_MENU_SELECTED, ZION_FINDUPDATE );
    ProcessEvent(anEvent); /// We will eventually use a pending event to the edit module in case a modal dialog opened in edit mod
}
 
void ZionFrame::AdviseReplUpdate(wxCommandEvent & WXUNUSED(event))  ///Called from client conn (after Poke "ReplUpdate" ) or from the server
{
    ///m_advisedReplData is already loaded by the poke
    //SetStatusText(_T("Advising ReplUpdate..."),3 );
    Connections::iterator it;
    for( it = zconns.begin(); it != zconns.end(); ++it ) { //!Advise All Connections
        if( it->second->m_srcPokeReplUpdate ) { /// Is this the requestor?
            it->second->m_srcPokeReplUpdate = false; continue; /// Turn off the flag and Skip requestor
        }
        it->second->Advise( _T("ReplUpdate"), (wxChar *) m_advisedReplData.c_str() );
    }
    //! Have the server update too
    wxCommandEvent anEvent( wxEVT_COMMAND_MENU_SELECTED, ZION_REPLUPDATE );
    ProcessEvent(anEvent); /// We will eventually use a pending event to the edit module in case a modal dialog opened in edit mod
}
 
void ZionFrame::AdviseReadStyles(wxCommandEvent & WXUNUSED(event))
{
    Connections::iterator it;
    for( it = zconns.begin(); it != zconns.end(); ++it ) { //!Advise All Connections
        if( it->second->m_srcExecStyle ) {
            it->second->m_srcExecStyle = false; continue; /// Turn off the flag and Skip requestor
        }
        it->second->Advise( _T("ReadStyles"), _T("") ); //! Pull styles from zion.ini
        wxMilliSleep(10);  ///Skew things a bit
    }    
    //! Have the server update too
        wxCommandEvent anEvent( wxEVT_COMMAND_MENU_SELECTED, ZION_READINI_STYLES );
        s_ourFrame->AddPendingEvent(anEvent);
}

/** Client Event Handlers **/ //------------------------------------------------------------------------------------------------------------------------
//! Could Request Zion.ini location. If not same as ours do not advise "ReloadIni", plus allow independent settings change
wxConnectionBase * MyClient::OnMakeConnection() //! by default this simply returns a new wxConnection object. We override this method to return a wxConnection descendant customised for the application.
{
    return new ClientConn; //! Return a custom connection object
}

/** We MUST do a StartAdvise or the connection WILL NOT ALLOW ADVISE messages to client **/
bool ClientConn::OnAdvise( const wxString& WXUNUSED(topic), const wxString& item,
        wxChar * data, int WXUNUSED(size), wxIPCFormat WXUNUSED(format) )
{
//    if( topic.BeforeFirst('_') == "zion" )  //! Topic can be a kind of security check
        if( item == "Priority") {
            priority = data;
        }
        if( item == "Raise") {
            wxCommandEvent anEvent( wxEVT_COMMAND_MENU_SELECTED, ZION_EXTERNALRAISE );
            s_ourFrame->AddPendingEvent(anEvent);
        }
        if( item == "Close") {
            wxCommandEvent anEvent( wxEVT_COMMAND_MENU_SELECTED, ZION_EXTERNALCLOSE );
            s_ourFrame->AddPendingEvent(anEvent);
        }
        if( item == "BecomeServer") {
            wxCommandEvent anEvent( wxEVT_COMMAND_MENU_SELECTED, ZION_BECOMESERVER );
            s_ourFrame->AddPendingEvent(anEvent);
        }
        if( item == "ReadStyles") {
            wxCommandEvent anEvent( wxEVT_COMMAND_MENU_SELECTED, ZION_READINI_STYLES );
            s_ourFrame->AddPendingEvent(anEvent);
        }
        if( item == "FindUpdate" ) {
            s_ourFrame->m_advisedData = data;
            wxCommandEvent anEvent( wxEVT_COMMAND_MENU_SELECTED, ZION_FINDUPDATE );
            s_ourFrame->AddPendingEvent(anEvent);
        }
        if( item == "ReplUpdate" ) {
            s_ourFrame->m_advisedReplData = data;
            wxCommandEvent anEvent( wxEVT_COMMAND_MENU_SELECTED, ZION_REPLUPDATE );
            s_ourFrame->AddPendingEvent(anEvent);
            //s_ourFrame->SetStatusText(_T("In OnAdvise ReplUdate"), 2);
        }
        if( item == "ReadPrint") {
        //Diag //s_ourFrame->SetStatusText(_T("In OnAdvise ReadPrint"), 2);
            wxCommandEvent anEvent( wxEVT_COMMAND_MENU_SELECTED, ZION_READINI_PRINT );
            s_ourFrame->AddPendingEvent(anEvent);
        }
        if( item == "ReadHistory") s_ourFrame->ReadIni( READHISTORY );
    return true;
}

bool ClientConn::OnDisconnect()
{
    zconnection = NULL;
    wxCommandEvent anEvent;  anEvent.SetId(ZION_RECONN);
    anEvent.SetEventType(wxEVT_COMMAND_MENU_SELECTED);
    s_ourFrame->AddPendingEvent(anEvent);
    wxConnection::OnDisconnect();  //! Call the parent disconnect f();
    return true;
}
//------------------------------------------------------------------------------------------------------
//! Event Handler
void ZionFrame::OnIPCTimer( wxTimerEvent & WXUNUSED(event) )
{
    wxCommandEvent anEvent; OnAssignPriorities(anEvent);
    //SetStatusText("Timer Running",1);
}

void ZionFrame::OnAssignPriorities(wxCommandEvent & WXUNUSED(event))
{
    Connections::iterator it; int count = 1; wxString value("0");
     for( it = zconns.begin(); it != zconns.end(); ++it, ++count ) {
        value.Printf("%d", count);
        if(! it->second->Advise("Priority", (wxChar*)value.c_str()) ) SetStatusText("IPC - Error advising priorities");  //! key is eq to it->first; value is eq to it->second
    }
}

bool ZionFrame::CheckFileInUse( wxString & str, bool remoteOnly )
{
    Connections::iterator it;
    //! "Remote" checks
    for( it = zconns.begin(); it != zconns.end(); ++it ) {  //! Check zconns hash
        if( it->second->m_srcRequestFile ) {
            it->second->m_srcRequestFile = false; continue; /// Turn off the flag and Skip requestor
        }
        if( it->second->currentFile == str ) return true;
    }
    //! Local check (if server has that file in use and it is not readonly)
    if(! remoteOnly && ! m_zedit->GetReadOnly() ) {
        if( m_zedit->m_currentDir + "\\" + m_zedit->GetCurrentFilename()  == str ) return true;  //! Check server's currentFile
    }
    return false;
}

void ZionFrame::OnMenuReconn( wxCommandEvent & WXUNUSED(event) )
{
    reconnCount = 0; priority = "99";
    DoReconn();
//    wxCommandEvent anEvent;  anEvent.SetId(ZION_RECONN);
//    OnReconn(anEvent);
}

void ZionFrame::OnShowIPCStatus(wxCommandEvent & WXUNUSED(event))
{  wxString str;
    if(zserver) {
        str = GetServerStatus();
    }
    else if(m_ipcOk && zconnection) {
        str.Printf( _T("Client Process ID: %lu\n"), localPid );
        str.Append( _T("Priority to become server: ") + priority );
        str.Append("\n_______________________");
        str.Append( zconnection->Request(_T("ServerStatus") ) );
    }
    wxMessageDialog statusDialog(this, str, _T("Inter-Process Communication Status"), wxOK );
    statusDialog.ShowModal();
}

wxString ZionFrame::GetServerStatus()
{
    wxString str, str2;
    str.Printf( _T("\nServer\n%lu "), localPid );
    str2 =  m_zedit->m_currentDir + _T("\\") + m_zedit->GetCurrentFilename();
    if(m_zedit->GetReadOnly()) str2 = _T("[") + str2 + _T("]");
    str.Append(str2 + (m_styleLock ?  " - StyleLock" : "") + _T("\n\nServer Connections: "));
    str2.Printf("%d", zconns.size()); str.Append(str2);
    Connections::iterator it;
    for( it = zconns.begin(); it != zconns.end(); ++it ) {  //! Check zconns hash
        str.Append("\n" + it->first + " " + it->second->currentFile );
        str.Append( (it->second->styleLock) ? " - StyleLock" : "");
    }
    return str;
}

void ZionFrame::OnReconn(wxCommandEvent &WXUNUSED(event) )
{
    DoReconn();
}

bool ZionFrame::DoReconn( )
{
    ///TODO: MAY not do since the appliction may already have a modal dialog open  -make into a dialog and offer to keep trying. Thanks to Debeaux!
    if( ++reconnCount >= 14 ) {
        SetStatusText("IPC Failed! Close this or all Zion windows, then retry");  return false;
    }
    if( zconnection ) {
        zconnection->Disconnect();
        delete zconnection;
        zconnection = NULL;
    }
    if(priority == "1" && ! zserver) {   //! If we are # 1, and not already a server, become server
        wxCommandEvent anEvent;  anEvent.SetId(ZION_BECOMESERVER);
        anEvent.SetEventType(wxEVT_COMMAND_MENU_SELECTED);
        AddPendingEvent(anEvent);
        m_ipcOk = true; return true;
    }
    long adelay; long delay;
    if(priority != "0") {
        priority.ToLong(&adelay);
        delay = adelay << 4;
    }
    else delay = localPid % 64 + 20;  //! at least 31ms
    wxMilliSleep(delay);  //! Delay based on process Id to provide some randomness
    wxString ztopic; ztopic.Printf( _T("zion_%lu"), localPid );  //! Concatenate topic
    zclient = NULL; //! the custom wxClient is required so that a user-overridden wxClient::OnMakeConnection member can return a wxConnection of the required class, when a connection is made.
    zclient = new MyClient;
    wxLogNull nolog; //! suppress the log messages from MakeConnection() (The assert-like messages)
    int count = 0;  zconnection = NULL;
    do { //! MakeConnection() will contact the server . On connection, zclient::OnMakeConnection will be called which will return a pointer to a ClientConn object
        wxMilliSleep(delay);
        zconnection = (ClientConn *) zclient->MakeConnection(hostName, service, ztopic);  //! ztopic is Topic /** Topic is  an identifier within the service */
    }
    while ( !zconnection && ++count < 10);
    
    if (!zconnection)  m_ipcOk = false; //! Could not make a connection as client
    else {  
        bool ok1, ok2, ok3, ok4, ok5, ok6, ok7,ok8, ok9; ok1=ok2=ok3=ok4=ok5=ok6=ok7=ok8=ok9=false;
        count = 0;
        do { /** We MUST do a StartAdvise or the connection WILL NOT ALLOW ADVISE messages to client **/
            ok1 = zconnection->StartAdvise("Priority");
            ok2 = zconnection->StartAdvise("ReadHistory");  
            ok3 = zconnection->StartAdvise("ReadStyles");
            ok4 = zconnection->StartAdvise("BecomeServer");
            ok5 = zconnection->StartAdvise("ReadPrint");
            ok6 = zconnection->StartAdvise("FindUpdate");
            ok7 = zconnection->StartAdvise("Close");
            ok8 = zconnection->StartAdvise("ReplUpdate");
            ok9 = zconnection->StartAdvise("Raise");
            wxMilliSleep(40);
        }
        while ( ++count < 10 && ( !ok1 || !ok2 || !ok3 || !ok4 || !ok5 || !ok6 || !ok7 || !ok8 || !ok9 ) ); ///End Do-While
        if( ok1 && ok2 && ok3 && ok4 && ok5 && ok6 && ok7 && ok8 && ok9 ) {
            SetStatusText(_T("Reconnection to server OK"));
            m_ipcOk = true; reconnCount = 0;
        }
        else { SetStatusText("Failure in IPC"); m_ipcOk= false; }

        if( m_ipcOk ) {
            //! Inform server of current file and status
            wxString str; str.Printf("%s\\%s", m_zedit->m_currentDir, m_zedit->GetCurrentFilename());
            if( m_zedit->GetReadOnly() ) str = "[" + str + "]";
            zconnection->Poke("FileInUse", (wxChar *) str.c_str());
            //! Re-establish styleLock
            if( m_styleLock && ! zserver ) zconnection->Execute("StyleLock");  //! Inform the server we have the style lock
            //! no need to request since we had to have previously requested to have it.

            m_iniDiff = true; ///reset to different
            if( m_iniDir == zconnection->Request("Zionini") ) m_iniDiff = false; //! If m_iniDiff do local push of files/dirs, otherwise do remote push.
        }
    }
    //! Are we good or bad?
    if( m_ipcOk ) { reconnCount =0; return true; } /** Good Exit **/
    else { //! Put a reconnection event into our queue i.e. keep trying
        wxCommandEvent anEvent(wxEVT_COMMAND_MENU_SELECTED, ZION_RECONN);
        AddPendingEvent(anEvent);
    }
    return false;
}

void ZionFrame::OnBecomeServer(wxCommandEvent & WXUNUSED(event))
{
    if(zconnection) { zconnection->Disconnect(); delete zconnection; }
    zconnection=NULL;
    if(zclient) delete zclient;  zclient = NULL;
    if(zserver) delete zserver; zserver = NULL;
    zserver = new MyServer;
    if( ! zserver->Create(service) )  { SetStatusText("Could not become server"); return; }
    priority = "0";
    m_iniDiff = true; //! Server can operate independently
    m_timer.SetOwner(this, ZION_IPCTIMER); /** This works! **/
    m_timer.Start(5000);
    SetStatusText("Server window");
}
//------------------------------------------------------------------------------------END IPC

/*! Utility Functions */

bool ZionFrame::PushExecStr(const wxString & execStr, bool localPush)
//! For the server, localPush works as an indicator not to broadcast a "ReadHistory" since the server always does a "local push" anyway.
/// That is the case when we are running in standalone mode.
{
    if( ! localPush && ! m_iniDiff && !zserver && m_ipcOk )
        if( zconnection->Poke("PushExecStr", (wxChar *) execStr.c_str()) )
            return true; //! If !m_iniDiff do remote push of files/dirs

    wxTextFile file(m_iniDir + "\\history.ini");   //! Windows specific! //simple text file object //Automatic var. - created on stack
    if( file.Exists() && file.Open() ) {
        size_t i, firstLine;
        StartEnd sectionPositions;  ///Struct to pass back section positions from FindSection()
//----------------------------------------------Update recent files        
        FindSection( file, _T("EXEC"), sectionPositions, true  ); ///true = createIt if not found

         /// Scan within Section and Update items
        firstLine = i = sectionPositions.start +1;  //! Header pos +1
        bool inList = false;
        /// while not end of section //eg "[END_CPP]"
        while( i < sectionPositions.end  ) { 
            if( file[i] == execStr ) { //! If that entry found, move it to top ///Normalize case
                inList = true;
                file.RemoveLine(i );  //!Doesn't work well after insert line for some reason
                file.InsertLine(execStr, firstLine);  //! Insert a copy into the top pos;/// Individual dir sections will not have to be changed here since were are not removing the file entry, just moving it
                break;
            } 
            i++;
        }
        if( ! inList )  {
           file.InsertLine( execStr, firstLine ); sectionPositions.end++; //! Insert Line at top
        }
        ///Clean up excess files
        for( ; sectionPositions.end - firstLine > NUMEXECSTRS;  sectionPositions.end-- ) { ///Remove excess
            file.RemoveLine(sectionPositions.end -1); 
        }

        if( ! file.Write() ) {
            SetStatusText("Failed saving exec string to \"history.ini\""); file.Close(); return false; ///The file is written here
        }
        file.Close();
        ///Here would be a good place to post a local pending event to ReadIni(READHISTORY) basically to Update the Recent menu
        if( zserver && !localPush ) { //! Advise all clients to re-read history in history.ini ///Applies to server only ///localPush means a private update of history.ini only, m_iniDiff = independent operation
          wxCommandEvent anEvent( wxEVT_COMMAND_MENU_SELECTED, ZION_ADVISE_READHISTORY );
          s_ourFrame->AddPendingEvent(anEvent);
        }
        /// Update recentFilesMenu */
        ReadIni(READHISTORY);  ///Server or independent op is not updated with AdviseReadHistory, so update it here
        return true;
      
    } //! file exists and opened
    else SetStatusText("Can't find \"history.ini\" in same folder as zion.ini"); //! Debug
    return false;
}


bool ZionFrame::PushPrint( bool WXUNUSED(localPush) )
//! For the server, localPush works as an indicator not to broadcast a "ReadHistory" since the server always does a "local push" anyway.
{
/// All changes to print will be updated immediately by the client to zion.ini, so there is no "try out" of options. Options are always applied to everyone.
//    if( ! localPush && ! m_iniDiff && !zserver && m_ipcOk ) ///Do remote push
//        if( zconnection->Poke("PushPrint", _T("")) )
//            return true;

    wxTextFile file(m_iniDir + "\\zion.ini");
    if( file.Exists() && file.Open() ) {
        size_t i, firstLine; wxString str;
        StartEnd sectionPositions;  ///Struct to pass back section positions from FindSection()

        FindSection( file, _T("PRINT"), sectionPositions, true  ); ///true = createIt if not found

         /// Remove all, if any, items
        firstLine = i = sectionPositions.start +1;  //! Header pos +1
        while( i < sectionPositions.end  ) { /// while not end of section //eg "[END_CPP]"
                file.RemoveLine(i );  //!Doesn't work well after insert line for some reason
            --sectionPositions.end; // i++;
        }
        
        wxPoint TL = g_pageSetupData->GetMarginTopLeft();
        str.Printf(_T("0|%d"), TL.y );
        file.InsertLine( str , i++ );  /// Top
        str.Printf(_T("1|%d"), TL.x );
        file.InsertLine( str , i++ );  /// Left
        
        wxPoint BR = g_pageSetupData->GetMarginBottomRight();
        str.Printf(_T("2|%d"), BR.y );
        file.InsertLine( str , i++ );  /// Bottom
        str.Printf(_T("3|%d"), BR.x );
        file.InsertLine( str , i++ );  /// Right
        
        str.Printf(_T("4|%d"), static_cast <int> (g_printData->GetPaperId()) );
        file.InsertLine( str , i++ );
        
        str.Printf(_T("5|%d"), static_cast <int> (g_printData->GetOrientation()) );
        file.InsertLine( str , i++ );

        if( ! file.Write() ) {
            SetStatusText("Failed saving print settings to zion.ini"); file.Close(); return false; ///The file is written here
        }
        file.Close();
        ///See about spreading the word that print settings have changed
        NotifyReadPrint();

        return true;
      
    } //! file exists and opened
    else SetStatusText("Can't find \"zion.ini\"", 1); //! Debug
    return false;
}


bool ZionFrame::PushFile(const wxString & addFile, bool localPush)
//! For the server, localPush works as an indicator not to broadcast a "ReadHistory" since the server always does a "local push" anyway.
{
    if( ! localPush && ! m_iniDiff && !zserver && m_ipcOk )
        if( zconnection->Poke("PushFile", (wxChar *) addFile.c_str()))
            return true; //! If !m_iniDiff do remote push of files/dirs

    wxString filename(addFile.AfterLast('\\'));
    
    if( filename == "Untitled" ) return false; //! We don't want to push "Untitled"
    wxTextFile file(m_iniDir + "\\history.ini");   //! Windows specific! //simple text file object //Automatic var. - created on stack
    if( file.Exists() && file.Open() ) {
        size_t i, firstLine;
        StartEnd sectionPositions;  ///Struct to pass back section positions from FindSection()
//----------------------------------------------Update recent files        
        FindSection( file, _T("RECENTFILES"), sectionPositions, true  ); ///true = createIt if not found

         /// Scan within Section and Update items
        firstLine = i = sectionPositions.start +1;  //! Header pos +1
        wxString addFileFix = addFile;   if( addFile.GetChar(3) == '\\' )  addFileFix.Remove(3, 1); ///Remove extra backslash from root entries
        bool inList = false;
        
        while( i < sectionPositions.end  ) { /// while not end of section //eg "[END_CPP]"
            if( file[i].Lower() == addFileFix.Lower() ) { //! If that entry found, move it to top ///Normalize case
                inList = true;
                file.RemoveLine(i );  //!Doesn't work well after insert line for some reason
                file.InsertLine(addFileFix, firstLine);  //! Insert a copy into the top pos;/// Individual dir sections will not have to be changed here since were are not removing the file entry, just moving it
                break;
            } 
            i++;
        }
        if( ! inList )  {
           file.InsertLine( addFileFix, firstLine ); sectionPositions.end++; //! Insert Line at top
        }
        
        ///Clean up excess files
        for( ; sectionPositions.end - firstLine > NUMRECENTFILES;  sectionPositions.end-- ) { ///Remove excess
            file.RemoveLine(sectionPositions.end -1); 
        }
//-----------------------------------------Update individual dir section
        if( FindSection( file, addFile.BeforeLast('\\'), sectionPositions/* , true */  ) ) { ///Search for corresponding dir section. If not found DON'T create it.

             /// Scan within Section and Update items
            firstLine = i = sectionPositions.start +1;  //! Header pos +1
            bool inList = false;
            while( i < sectionPositions.end  ) { /// while not end of section //eg "[END_CPP]"
                if( file[i].Lower() == filename.Lower()) { //! If that entry found, move it to top ///Normalize case
                    inList = true;
                    file.RemoveLine(i );  //!Doesn't work well after insert line for some reason
                    file.InsertLine(filename, firstLine);  //! Insert a copy into the top pos;/// Individual dir sections will not have to be changed here since were are not removing the file entry, just moving it
                    break;
                } 
                i++;
            }
            
            if( ! inList )  {
               file.InsertLine( filename, firstLine ); sectionPositions.end++; //! Insert Line at top
            }
            ///Clean up excess files
            for( ; sectionPositions.end - firstLine > NUMRECENTFILES;  sectionPositions.end-- ) { ///Remove excess
                file.RemoveLine(sectionPositions.end -1); 
            }
        }
        else SetStatusText(_T("Could not find individual dir section") );
//-----------------------------------------

        if( ! file.Write() ) {
            SetStatusText("Failed saving history to \"history.ini\""); file.Close(); return false; ///The file is written here
        }
        file.Close();
        ///The server is the only window that can actually update history.ini, clients Poke a Push request
        if( zserver && !localPush ) { //! Advise all clients to re-read history in history.ini ///Applies to server only ///localPush means a private update of history.ini only, m_iniDiff = independent operation
          wxCommandEvent anEvent( wxEVT_COMMAND_MENU_SELECTED, ZION_ADVISE_READHISTORY );
          s_ourFrame->AddPendingEvent(anEvent);
        }
        /// Update recentFilesMenu */
        ReadIni(READHISTORY);  ///Server or independent op is not updated with AdviseReadHistory, so update it here
        return true;
      
    } //! file exists and opened
    else SetStatusText("Can't find \"history.ini\" in same folder as zion.ini"); //! Debug
    return false;
}

bool ZionFrame::FindSection( wxTextFile& file, const wxString& section, StartEnd& locations, bool createIt ) ///WORKING
{
    bool headerFound = false, sectionFound = false; int i, m= file.GetLineCount();
    wxString str = _T("[") + section + _T("]");
    wxString str2 = _T("[END_") + section + _T("]");
    for( i=0; i < m; i++ ) {  /// Start from top scanning each line  for beginning of section
        if( file[i].Lower() == str.Lower()) { //! eg "[CPP]" ///If an orphaned opening header found, no problem. firstLine will readj to the LAST opening header found
            headerFound = true;
            locations.start = i;  continue; //! save position of next line ///no sense checking for str2 since it can't be in the same line
        }
        if( file[i].Lower() == str2.Lower() && headerFound == true ) { ///Look for End of Section  //! eg "[END_CPP]"
            sectionFound = true;
            locations.end = i; 
            break;
        }
    }
    if( !sectionFound && createIt ) {
        file.AddLine("");   /// blank line before section
        file.AddLine(str);   file.AddLine(str2);
        //SetStatusText( wxString::Format(_T("%s section was added"), str ) );
        m= file.GetLineCount();  ///update line count
        locations.start= m-2;  locations.end = m-1;  ///The last empty line is not counted
    }
    return sectionFound;
}

bool ZionFrame::PushDir(const wxString & dir, bool localPush)
{
///Synopsis: Normally only the server window will update history in history.ini - other windows will request a push, but if the window is operating standalone, as
/// in the case of  windows using different ini files (m_iniDiff = true), it will do its own push (i.e. go through with most of the PushDir() function.).
/// A local push indicates we want to write independently to our history.ini and we do not want to advise all connections to do a Readini() as in certain timing sensitive areas like OnClose().

    if( ! localPush && ! m_iniDiff && !zserver && m_ipcOk ) ///not isolated local push ///ini same /// not server /// ipc is ok
        if( zconnection->Poke("PushDir", (wxChar *) dir.c_str()) ) ///do remote push of files/dirs, otherwise do our own push
            return true; 
            
    wxTextFile file(GetIniDir() + "\\history.ini");   ///simple text file object - automatic var. - created on stack  //! Windows specific - backslash
    if( file.Exists() && file.Open() ) {
      size_t i, firstLine;
        
      /*! RECENTDIRS*/
       StartEnd sectionPositions;  ///Struct to pass back section positions from FindSection()

        FindSection( file, _T("RECENTDIRS"), sectionPositions, true  ); ///true = createIt if not found

         /// Scan within Section and Update items
        firstLine = i = sectionPositions.start +1;  //! Header pos +1
        bool inList = false; 

        while( i < sectionPositions.end  ) { /// while not end of section //eg "[END_CPP]"
            if( file[i].Lower() == dir.Lower()) { //! If that entry found, move it to top ///Normalize case
                inList = true;
                                                /// Individual dir sections will not have to be changed here since were are not removing the dir, just moving it
                file.RemoveLine(i );  //!Doesn't work well after insert line for some reason
                file.InsertLine(dir, firstLine); //! Insert a copy into the top pos;
                break;
            }   //! Dont break, let the loop run out so we know total # of entries in section
            i++;        ///Count = i - firstLine
        } //! At this point i is pointing at the End of section maker
        
          ///If Entry not in list, remove item(s) from bottom and insert entry at top 
        if( ! inList ) {  //! If the entry was in the list and moved up # entries doesn't change so no need to check #entries
            wxString removedDirs[20]; int count =0;
            //SetStatusText(wxString::Format("i = %d, file[i] = %s", i, file[i])); ///diag
            //while( (i - firstLine) - NUMRECENTDIRS  >= 0 )  //! If Max exceeded, remove from end  ///To be safe we will not assume only one item is to be removed
            file.InsertLine( dir, firstLine ); sectionPositions.end++; //! Insert Line at top
            ///Clean up excess dirs
            for( count=0; sectionPositions.end - firstLine > NUMRECENTDIRS; count++, sectionPositions.end-- ) { ///Remove excess
                if( count > 19 ) break; ///Vital: make sure array bounds not exceeded
                removedDirs[count] = file[sectionPositions.end -1];   ///Track removed entry
                file.RemoveLine(sectionPositions.end -1); 
            }
            //if( count > 0 ) SetStatusText(wxString::Format("Removed %d directory entry(s). Last: %s", count, removedDirs[count-1].AfterLast('\\' )) );  ///Diag
            ///Remove any excess individual dir sections
              for( i=0; i < count; i++)  { 
                    if( FindSection( file, removedDirs[i], sectionPositions ) ) { /// If Section Found remove it
                        while( sectionPositions.end >= sectionPositions.start ) {
                            file.RemoveLine(sectionPositions.start);
                            --sectionPositions.end;
                        }
                        if( file[sectionPositions.start - 1]  == _T("") )  file.RemoveLine(sectionPositions.start -1);  ///delete top line if empty as well ( we inserted newline then section header/footer)
                    } 
              }
        }
        
    ///If section not found, create it
    FindSection( file, dir, sectionPositions, true  );
        
      if( ! file.Write() ) {
        SetStatusText("Failed saving directory history to \"history.ini\""); file.Close(); return false; ///The file is written here
      }
      file.Close();
      ///Advise all clients to Update their Recent menu
      if( zserver && !localPush ) { //! Advise all clients to re-read history in history.ini ///Applies to server only ///localPush means a private update of zion.ini only, m_iniDiff = independent operation
          wxCommandEvent anEvent;  anEvent.SetId(ZION_ADVISE_READHISTORY);
          anEvent.SetEventType(wxEVT_COMMAND_MENU_SELECTED);
          s_ourFrame->AddPendingEvent(anEvent);
      }
      ReadIni(READHISTORY);  ///Server or independent op is not updated with AdviseReadHistory, so update it here
      return true;

    } //! file exists and opened
    else SetStatusText("Can't find \"history.ini\" in same folder as zion.ini"); //! Debug
    return false;
}

void ZionFrame::OnRecentFileItem(wxCommandEvent & event)
///We have tracked menuitems of recentDirsMenu upon the addition of submenus.
///dirSubs[] tracks the actual submenus, dirSubItems[] tracks the menuitems created as submenus are added.
{
    wxString str;
    wxMenuItem * itemPtr=NULL;
    wxMenu * itemPtr2=NULL;
    //SetStatusText( recentDirsMenu->GetLabel( event.GetId()) ); ///Test
    itemPtr = recentDirsMenu->FindItem( event.GetId() );
    itemPtr2 =  itemPtr->GetMenu();  ///Get the menu (in this case a submenu) that the item is attached to
    if( itemPtr == NULL || itemPtr2 == NULL ) { SetStatusText(_T("Error processsing Menu Item") ); return; } ///safety
    
    ///dirSub is the pointer to the submenu itself;
    ///dirSubItem is a pointer to the item generated when the submenu is attached to its parent (RecentDirs menu).
    for( int i=0; i < NUMRECENTDIRS;  i++) {
        if( dirSubs[i] == itemPtr2 ) {
            str = dirSubItems[i]->GetLabel();  break;  ///Get dir
        }
    }
    if( event.GetId() >= ZION_OPENITEM_1 && event.GetId() <= ZION_OPENITEM_LAST ) {
        m_zedit->Open( str );  return; /// Return asap - open will spur a lot of IPC activity ///Open will do CheckAllowSave()
    }
    if( event.GetId() >= ZION_REMOVEITEM_1 && event.GetId() <= ZION_REMOVEITEM_LAST ) {
        if( wxMessageBox( wxString::Format(_T("Are you sure you want to remove the folder\n%s\nfrom history?"), str ),
                _T("Remove Folder from History"), wxICON_EXCLAMATION | wxYES_NO ) == wxYES ) {
            RemoveSubmenu( str );  return;
        }
    }
    else { ///Submenu item
        if( m_zedit->CheckAllowSave() ) { //! Make sure saved first
            str.Append(_T("\\") + itemPtr->GetLabel() ); ///Get full filespec  //if(itemPtr) SetStatusText( str ); ///Diag
            wxString loc;
            if( DoWinLocDlg( loc ) ) {
                if( loc != wxEmptyString ) wxExecute( wxString::Format( _T("\"%s\\zion.exe\" -zposx%d %s \"%s\""), m_exeDir,
                GetPosition().x, loc, str ) );
                else  m_zedit->DirectOpen( str ); ///In this window
            } ///else Dialog was cancelled
        }
    }
}

void ZionFrame::RemoveSubmenu( const wxString & dir )
{
    ///Server only will do an actual write
    if( ! m_iniDiff && !zserver && m_ipcOk && zconnection->Poke("PruneDir", (wxChar *) dir.c_str()) ) ///do remote prune of files/dirs, otherwise do our own prune
            return; 
    //Diag //SetStatusText(_T("In Remove Submenu Proper") );        
    wxTextFile file(GetIniDir() + "\\history.ini");   ///simple text file object - automatic var. - created on stack  //! Windows specific - backslash
    if( file.Exists() && file.Open() ) {
        size_t i, firstLine;
          
        StartEnd sectionPositions;  ///Struct to pass back section positions from FindSection()
        
        ///Remove any RecentFile entries for that directory
        if( FindSection( file, _T("RECENTFILES"), sectionPositions ) ) {
            firstLine = i = sectionPositions.start +1;  //! Header pos +1
            while( i < sectionPositions.end  ) { /// while not end of section //eg "[END_CPP]"
                if( file[i].BeforeLast('\\').Lower() == dir.Lower() || file[i].BeforeLast('\\').Lower() + '\\' == dir.Lower() ) { ///Normalize case for cmp ///Root dir entries will come in with an extra backslash
                    file.RemoveLine(i);
                    --sectionPositions.end; ///Adjust end marker upward
                    continue;  ///We don't want to increment i as remove will move the next line up
                }
                i++;        ///Count = i - firstLine
            }
        }
        
        ///Remove line matching our dir str in "RecentDirs" Section
        if( FindSection( file, _T("RECENTDIRS"), sectionPositions ) ) {
            firstLine = i = sectionPositions.start +1;  //! Header pos +1
            while( i < sectionPositions.end  ) { /// while not end of section //eg "[END_CPP]"
                if( file[i].Lower() == dir.Lower() ) { ///Normalize case
                    file.RemoveLine(i );  break;
                }
                i++;
            }
        }
        
        ///Remove Individual dir section
        if( FindSection( file, dir, sectionPositions ) ) { /// If Section Found remove it
            while( sectionPositions.end >= sectionPositions.start ) {
                file.RemoveLine(sectionPositions.start);
                --sectionPositions.end;
            }
            if( file[sectionPositions.start - 1]  == _T("") )  file.RemoveLine(sectionPositions.start -1);  ///delete top line if empty as well ( we inserted newline then section header/footer)
        }
        
        if( ! file.Write() ) { ///Without a Write() changes are not saved!
            wxMessageBox("Failed saving directory history to \"history.ini\"", "Write to history.ini Failed" ); file.Close(); return; ///The file is written here
        }
        file.Close();
        
        ///Advise all clients to Update their Recent menu
        if( zserver ) { //! Advise all clients to re-read history in history.ini ///Applies to server only ///localPush means a private update of zion.ini only, m_iniDiff = independent operation
          wxCommandEvent anEvent;  anEvent.SetId(ZION_ADVISE_READHISTORY);
          anEvent.SetEventType(wxEVT_COMMAND_MENU_SELECTED);
          s_ourFrame->AddPendingEvent(anEvent);
        }
        ReadIni(READHISTORY);  ///Server or independent op is not updated with AdviseReadHistory, so update it here
    } //! file exists and opened
    else wxMessageBox("Can't find \"history.ini\" in same folder as zion.ini", _T("History.ini not Found!")); //! Debug

    return;
}

bool ZionFrame::CheckForIni()
{
    wxTextFile file(m_iniDir + "\\zion.ini");   //! Windows specific! //simple text file object //Automatic var. - created on stack
    if( ! file.Exists() ) { //! Not found in the standard loc
        ZionIniDialog iniDialog( this, m_iniDir );
        if( iniDialog.ShowModal() == wxID_OK ) {
            m_iniDir = iniDialog.m_text->GetValue();  //! TODO integrity checks here //! This establishes  a new start Dir
            if( m_iniDir.Last() == '\\' ) { //! Remove trailing backslash //GetChar( m_iniDir.Length() -1
                m_iniDir.RemoveLast(); // = m_iniDir.BeforeLast('\\');
            }
            wxTextFile file2(m_iniDir + "\\zion.ini");
            if( ! file2.Exists() ) {
                if( iniDialog.m_createIt->IsChecked() && ! file2.Create() ) {
                    wxMessageDialog failDialog(this, _T("Could not create zion.ini\nGiving up for this session\n\nRestart the Program to try again"),
                            _T("Failed Creating zion.ini"), wxOK );
                    failDialog.ShowModal();
                    return ! (m_FailedZionIni = true);  //! We could use this as a way to short circuit any f(0)s that write to zion.ini //return false;
                }
                else if(iniDialog.m_createIt->IsChecked()) {  //! Then the Create did return true
                    wxMessageDialog createdDialog(this, "Created zion.ini successfully", "Created zion.ini", wxOK );
                    createdDialog.ShowModal();  //! This one falls thru
                }
                else {  //! It didn't exist and we didn't want to create it
                    wxString str9;  str9.Printf("Could not Locate zion.ini in\n%s\nGiving up for this session\n\nRestart the Program to try again", m_iniDir);
                    wxMessageDialog locateDialog(this, str9, "Failed Locating zion.ini", wxOK );
                    locateDialog.ShowModal();
                    return ! (m_FailedZionIni = true);  //! We could use this as a way to short circuit any f(0)s that write to zion.ini //return false;
                }
            }
            if( ! file2.Open() )  { m_FailedZionIni = true; //! Platform asserts will flag an error
                wxMessageDialog failDialog(this, "Could not open zion.ini. Is it Readonly?\nGiving up for this session\n\nRestart the Program to try again",
                        "Failed Opening zion.ini", wxOK );
                failDialog.ShowModal();
            }
            else file2.Close();    //! m_FailedZionIni should still be false here
        }
    }
    else if( ! file.Open() ) { //! Further check on std loc
        m_FailedZionIni = true; //! Platform asserts will flag an error
        wxMessageDialog failDialog(this, "Could not open zion.ini. Is it Readonly?\nGiving up for this session\n\nRestart the Program to try again",
                "Failed Opening zion.ini", wxOK );
        failDialog.ShowModal();
    }
    else file.Close();  //! m_FailedZionIni should still be false here

    return ! m_FailedZionIni;
}

bool ZionFrame::WriteIni()
{
    wxString str, str2, str3, mySpec;
    wxTextFile file(GetIniDir() + "\\zion.ini");   //! Windows specific! //simple text file object //Automatic var. - created on stack
    if( file.Exists() && file.Open() ) {
        int numSections = NUMSECTIONS; //!  # pointers in smallSyntaxes
        int options = 0; size_t i;
        StartEnd sectionPositions;  ///Struct to pass back section positions from FindSection()

        ///SmallSyntaxes
        for( int j = 0; j < numSections;  j++ ) {  //! Loop through small syntaxes
            str.Printf("%s", smallSyntaxes[j]->name ); //! Set up our comparison strings outside the loop
            struct smallSyntax * aSyntax = NULL; aSyntax = smallSyntaxes[j];  //! Setup pointer to syntax structure
            ///Find Section and create it if not found
            FindSection( file, str, sectionPositions, true ); ///true = createIt if not found
            ///Empty Section
            i = sectionPositions.start +1;  /// i should be at line after section header
            while( i != sectionPositions.end ) { //! while not end of section //eg "[END_CPP]"
                file.RemoveLine(i); //! Remove till end of section // i++// No need to increment, the next line will move up 
                sectionPositions.end--;
            } //! At this point i is pointing at the End of section maker
            ///Write syntax mode options
            options = 0;
            if( aSyntax->option.line ) options |= ZION_OPT_LINE;
            if( aSyntax->option.fold ) options |= ZION_OPT_FOLD;
            if( aSyntax->option.wrap ) options |= ZION_OPT_WRAP;
            if( aSyntax->option.indent ) options |= ZION_OPT_INDENT;
            if( aSyntax->option.usetabs ) options |= ZION_OPT_USETABS;
            if( aSyntax->option.viewws ) options |= ZION_OPT_VIEWWS;
            if( aSyntax->option.horizSb ) options |= ZION_OPT_HORIZSB;
            if( aSyntax->option.vertSb ) options |= ZION_OPT_VERTSB;
            str3.Printf(_T("&%d"), options );
            file.InsertLine(str3, i); /// Insert into file
            i++; ///readj to end of section
            ///Per Mode Colors
            str3.Printf( "$%02x%02x%02x", aSyntax->pageColor.Red(),
                    aSyntax->pageColor.Green(),  aSyntax->pageColor.Blue() );
            file.InsertLine(str3, i); /// Insert into file */ //! Double check order here
            i++; ///readj to end of section for next insert
            str3.Printf( "!F%02x%02x%02x", aSyntax->lineForeColor.Red(),
                    aSyntax->lineForeColor.Green(),  aSyntax->lineForeColor.Blue() );
            file.InsertLine(str3, i); /// Insert into file */ //! Double check order here
            i++; ///readj to end of section for next insert
            str3.Printf( "!B%02x%02x%02x", aSyntax->lineBackColor.Red(),
                    aSyntax->lineBackColor.Green(),  aSyntax->lineBackColor.Blue() );
            file.InsertLine(str3, i); /// Insert into file */ //! Double check order here
            i++; ///readj to end of section for next insert
            str3.Printf( "!L%02x%02x%02x", aSyntax->foldColor.Red(),
                    aSyntax->foldColor.Green(),  aSyntax->foldColor.Blue() );
            file.InsertLine(str3, i); /// Insert into file */ //! Double check order here
            i++; ///readj to end of section for next insert
            str3.Printf( "!S%02x%02x%02x", aSyntax->selColor.Red(),
                    aSyntax->selColor.Green(),  aSyntax->selColor.Blue() );
            file.InsertLine(str3, i); /// Insert into file */ //! Double check order here
            i++; ///readj to end of section for next insert
            str3.Printf( "!I%02x%02x%02x", aSyntax->indentColor.Red(),
                    aSyntax->indentColor.Green(),  aSyntax->indentColor.Blue() );
            file.InsertLine(str3, i); /// Insert into file */ //! Double check order here
            i++; ///readj to end of section for next insert
            
            ///Write Out Styles
            for( int j = 0; j < aSyntax->stylesUsed; j++ ) {   //! For all styles used in that struct
                str3.Printf( "%d|%s,%s,%s,fore:%s,back:%s,face:%s,size:%s", aSyntax->styles[j].styleNum,
                aSyntax->styles[j].item, aSyntax->styles[j].bold, aSyntax->styles[j].italic, aSyntax->styles[j].fore,
                aSyntax->styles[j].back, aSyntax->styles[j].face, aSyntax->styles[j].size );
                file.InsertLine(str3, i);
                i++; ///readj to end of section
            }
        }  //! smallSyntax for loop
        
        /*! EXTRA */ //! CARET COLOR + CUSTOM COLORS
        str = "EXTRA"; //! Set up our comparison strings outside the loop
        FindSection( file, str, sectionPositions, true ); ///true = createIt if not found
        i = sectionPositions.start +1;  /// i should be at line after section header

        /// In Section */  
        while(  i != sectionPositions.end ) { //! while not end of section //eg "[END_CPP]"
            file.RemoveLine(i); //! Remove till end of section // i++// No need to increment, the next line will move up 
            sectionPositions.end--;
        }
        //! INSERT CARET COLOR  // At this point i is pointing at the End of section maker
        str3.Printf( "0|%02x%02x%02x", g_caretColor.Red(), g_caretColor.Green(),  g_caretColor.Blue() );
        file.InsertLine(str3, i); /// Insert into file */ //! Double check order here
        i++; ///readj to end of section for next insert
       
        for( int j=0; j < 16 ; j++ ) { //! CUSTOM COLORS
            str3.Printf( "%d|%02x%02x%02x", j+1, customColors[j].Red(), customColors[j].Green(), customColors[j].Blue() );
            file.InsertLine(str3, i); /// Insert into file */ //! Double check order here
            i++; ///readj to end of section for next insert
        }
        
        str3.Printf( "30|%d", m_zedit->GetEolDefault() ); /// EOL
        file.InsertLine(str3, i); /// Insert into file */ //! Double check order here
        i++; ///readj to end of section for next insert

        str3.Printf( "31|%d", m_zedit->GetPrintColourMode() ); /// PCOLMode
        file.InsertLine(str3, i); /// Insert into file */ //! Double check order here
        i++; ///readj to end of section for next insert

        str3.Printf( "33|%d", (m_zedit->m_dontPrintNums ? 1 : 0) );
        file.InsertLine(str3, i);
        i++; ///readj to end of section for next insert
        
        str3.Printf( "40|%d", g_numTopRecentFiles );
        file.InsertLine(str3, i);
        i++; ///readj to end of section for next insert
        
        str3.Printf( "41|%d", g_numRecentDirs );
        file.InsertLine(str3, i);
        i++; ///readj to end of section for next insert

        str3.Printf( "45|%d", (m_zedit->m_htmlAC.active ? 1 : 0) ); ///Remember we write (save)  from the editor's vars
        file.InsertLine(str3, i);
        i++; ///readj to end of section for next insert
        
        str3.Printf( "50|%d", m_zedit->m_startLex );
        file.InsertLine(str3, i);
        i++; ///readj to end of section for next insert
        
        str3.Printf( "52|%s", g_initialZoom );
        file.InsertLine(str3, i);
        i++; ///readj to end of section for next insert
        
        str3.Printf( "53|%s", g_pathToApi );
        file.InsertLine(str3, i);
        i++; ///readj to end of section for next insert
        
        str3.Printf( "54|%s", g_pathToBrowser );
        file.InsertLine(str3, i);
        i++; ///readj to end of section for next insert
        
        str3.Printf( "55|%d", g_lineNumWidth );
        file.InsertLine(str3, i);
        i++; ///readj to end of section for next insert
        
        str3.Printf( "56|%d", g_numACChars );
        file.InsertLine(str3, i);
        i++; ///readj to end of section for next insert
        
        str3.Printf( "57|%d", g_acActive ? 1 : 0 );
        file.InsertLine(str3, i);
        i++; ///readj to end of section for next insert
        
        str3.Printf( "58|%d", g_ctActive ? 1 : 0 );
        file.InsertLine(str3, i);
        i++; ///readj to end of section for next insert
        
        str3.Printf( "60|%d", m_zedit->m_startOpt );
        file.InsertLine(str3, i);
        i++; ///readj to end of section for next insert
        str3.Printf( "61|%d", m_zedit->m_startPos.x );
        file.InsertLine(str3, i);
        i++; ///readj to end of section for next insert
        str3.Printf( "62|%d", m_zedit->m_startPos.y );
        file.InsertLine(str3, i);
        i++; ///readj to end of section for next insert
        str3.Printf( "63|%d", m_zedit->m_startSize.GetWidth() );
        file.InsertLine(str3, i);
        i++; ///readj to end of section for next insert
        str3.Printf( "64|%d", m_zedit->m_startSize.GetHeight() );
        file.InsertLine(str3, i);
        i++; ///readj to end of section for next insert
        str3.Printf( "70|%d", g_autoBook ? 1 : 0 ); ///save from global
        file.InsertLine(str3, i);
        i++; ///readj to end of section for next insert
        str3.Printf( "72|%d", g_autoZoom ? 1 : 0 ); ///save from global
        file.InsertLine(str3, i);
        i++; ///readj to end of section for next insert
        
        /*! EXTENSIONS */ 
        str = "EXTENSIONS"; //! Set up our comparison strings outside the loop
        FindSection( file, str, sectionPositions, true ); ///true = createIt if not found

        /// In Section  ///First clear the section
        i = sectionPositions.start +1;  /// i should be at line after section header
        while( i != sectionPositions.end ) { //! while not end of section //eg "[END_CPP]"
            file.RemoveLine(i); //! Remove till end of section // i++// No need to increment, the next line will move up 
            sectionPositions.end--;
        }
        /// At this point i is pointing at the End of section maker
        stringToInt::iterator it;
        for( it = ext2lex.begin(); it != ext2lex.end(); ++it ) { ///Dump the whole hashmap
            str3 = wxString::Format(_T("%s,%d"), it->first, it->second  );
            file.InsertLine(str3, i); /// Insert into file
            i++; ///readj to end of section for next insert
        }
        
        /** C++_WORDS2 **/
        str = "C++_WORDS2";
        FindSection( file, str, sectionPositions, true ); ///true = createIt if not found
        /// In Section  ///First clear the section
        i = sectionPositions.start +1;  /// i should be at line after section header
        while( i != sectionPositions.end ) { //! while not end of section //eg "[END_CPP]"
            file.RemoveLine(i); //! Remove till end of section // i++// No need to increment, the next line will move up 
            sectionPositions.end--;
        }
        /// At this point i is pointing at the End of section maker
        wxStringTokenizer tknzr( g_cppWords2); ///Default to whitespace delims and empty tokens never returned
        str3 = _T("");
        while( tknzr.HasMoreTokens() ) {
            str3 += tknzr.GetNextToken();
            str3 += _T(" ");
            if( str3.Len() > 100 ) {
                file.InsertLine(str3, i); /// Insert into file
                i++; ///readj to end of section for next insert
                str3 = _T("");
            }
        }
        if( str3 != _T("") ) {  /// if remainder, write out the remainder
            file.InsertLine(str3, i);
            i++; ///readj to end of section for next insert
        }
        
        /** C#_WORDS2 **/
        str = "C#_WORDS2";
        FindSection( file, str, sectionPositions, true ); ///true = createIt if not found
        /// In Section  ///First clear the section
        i = sectionPositions.start +1;  /// i should be at line after section header
        while( i != sectionPositions.end ) { //! while not end of section //eg "[END_CPP]"
            file.RemoveLine(i); //! Remove till end of section // i++// No need to increment, the next line will move up 
            sectionPositions.end--;
        }
        /// At this point i is pointing at the End of section maker
        wxStringTokenizer tknzr2( g_csWords2); ///Default to whitespace delims and empty tokens never returned
        str3 = _T("");
        while( tknzr2.HasMoreTokens() ) {
            str3 += tknzr2.GetNextToken();
            str3 += _T(" ");
            if( str3.Len() > 100 ) {
                file.InsertLine(str3, i); /// Insert into file
                i++; ///readj to end of section for next insert
                str3 = _T("");
            }
        }
        if( str3 != _T("") ) {  /// if remainder, write out the remainder
            file.InsertLine(str3, i);
            i++; ///readj to end of section for next insert
        }
        
        /** JAVA_WORDS2 **/
        str = "JAVA_WORDS2";
        FindSection( file, str, sectionPositions, true ); ///true = createIt if not found
        /// In Section  ///First clear the section
        i = sectionPositions.start +1;  /// i should be at line after section header
        while( i != sectionPositions.end ) { //! while not end of section //eg "[END_CPP]"
            file.RemoveLine(i); //! Remove till end of section // i++// No need to increment, the next line will move up 
            sectionPositions.end--;
        }
        /// At this point i is pointing at the End of section maker
        wxStringTokenizer tknzr3( g_javaWords2); ///Default to whitespace delims and empty tokens never returned
        str3 = _T("");
        while( tknzr3.HasMoreTokens() ) {
            str3 += tknzr3.GetNextToken();
            str3 += _T(" ");
            if( str3.Len() > 100 ) {
                file.InsertLine(str3, i); /// Insert into file
                i++; ///readj to end of section for next insert
                str3 = _T("");
            }
        }
        if( str3 != _T("") ) {  /// if remainder, write out the remainder
            file.InsertLine(str3, i);
            i++; ///readj to end of section for next insert
        }
        
        /** SQL_WORDS2 **/
        str = "SQL_WORDS2";
        FindSection( file, str, sectionPositions, true ); ///true = createIt if not found
        /// In Section  ///First clear the section
        i = sectionPositions.start +1;  /// i should be at line after section header
        while( i != sectionPositions.end ) { //! while not end of section //eg "[END_CPP]"
            file.RemoveLine(i); //! Remove till end of section // i++// No need to increment, the next line will move up 
            sectionPositions.end--;
        }
        /// At this point i is pointing at the End of section maker
        wxStringTokenizer tknzr4( g_sqlWords2); ///Default to whitespace delims and empty tokens never returned
        str3 = _T("");
        while( tknzr4.HasMoreTokens() ) {
            str3 += tknzr4.GetNextToken();
            str3 += _T(" ");
            if( str3.Len() > 100 ) {
                file.InsertLine(str3, i); /// Insert into file
                i++; ///readj to end of section for next insert
                str3 = _T("");
            }
        }
        if( str3 != _T("") ) {  /// if remainder, write out the remainder
            file.InsertLine(str3, i);
            i++; ///readj to end of section for next insert
        }

//--------------------------------------------------------------------------------------        
      if( file.Write() ) {   SetStatusText("Settings saved to \"zion.ini\"");
          //We want to broadcast all the time to release stylelock //if(m_zedit->m_settingsDirty) { //! We are catching settings on the transition point
              wxCommandEvent anEvent;  anEvent.SetId(ZION_NOTIFYSTYLES);
              NotifyReadStyles(anEvent);
          //}
         m_zedit->m_settingsDirty = false;
         if( zserver ) m_styleLock = false;

         return true;
      }
      else { SetStatusText("Failed saving settings to \"zion.ini\". Retry later"); return false; }
    } //! file exists and opened
    
    else {
        SetStatusText("Can't find \"zion.ini\". Retry saving settings."); return false;
    }
}

bool ZionFrame::ReadIni( int readFlags, bool init )  ///Init will allow creation of history.ini on startup
{
    wxString str, str2, str3, mySpec;
    wxTextFile file(GetIniDir() + "\\zion.ini");   //! Windows specific! //simple text file object //Automatic var. - created on stack
    if( file.Exists() && file.Open() ) {
        long stylenum = 0; size_t i, m = file.GetLineCount();  bool inSection = false;
        StartEnd sectionPositions;  ///Struct to pass back section positions from FindSection()

        int numSections = NUMSECTIONS; //!  #  of pointers in smallSyntaxes
        if( readFlags & READSTYLES ) {
            for( int j = 0; j < numSections;  j++ )  {  /*! Sections for loop */
                str.Printf("[%s]", smallSyntaxes[j]->name ); //! Set up our comparison strings outside the loop
                str2.Printf("[END_%s]", smallSyntaxes[j]->name );
                struct smallSyntax * aSyntax = NULL; aSyntax = smallSyntaxes[j]; //! Setup pointer to syntax structure
                
                /*! Lines for loop */  
                for( i=0; i < m; i++ )  { //! Start from top scanning each line  for beginning of section
                    if( file[i] == str2 )   {   //!we are done //eg "[END_CPP]"
                        inSection = false;  break;      //! look for next section//!Optimize: should not need inSection=false;
                    }
                    if( file[i] == str )    { //! eg "[CPP]"
                        inSection = true;   continue; //! continue at next line
                    }

                    if(inSection)  {    //!This check must be after the check for section headers
                        if( file[i].StartsWith("#") || file[i] == "" ) continue; //! skip comment or blank line
                         ///Check for syntax mode options
                        if( file[i].StartsWith("&") && file[i].AfterFirst('&').ToLong(&stylenum) ) { ///stylenum is a convenient long to use
                            aSyntax->option.line = stylenum & ZION_OPT_LINE;
                            aSyntax->option.fold = stylenum & ZION_OPT_FOLD;
                            aSyntax->option.wrap = stylenum & ZION_OPT_WRAP;
                            aSyntax->option.indent = stylenum & ZION_OPT_INDENT;
                            aSyntax->option.usetabs = stylenum & ZION_OPT_USETABS;
                            aSyntax->option.viewws = stylenum & ZION_OPT_VIEWWS;
                            aSyntax->option.horizSb = stylenum & ZION_OPT_HORIZSB;
                            aSyntax->option.vertSb = stylenum & ZION_OPT_VERTSB;
                        }

                        if( file[i].StartsWith("$") )
                            aSyntax->pageColor = StringToColor( file[i].AfterFirst('$') );

                        if( file[i].StartsWith("!F") )
                            aSyntax->lineForeColor = StringToColor( file[i].AfterFirst('F') );
                            
                        if( file[i].StartsWith("!B") )
                            aSyntax->lineBackColor = StringToColor( file[i].AfterFirst('B') );
                            
                        if( file[i].StartsWith("!L") )
                            aSyntax->foldColor = StringToColor( file[i].AfterFirst('L') );
                            
                        if( file[i].StartsWith("!S") )
                            aSyntax->selColor = StringToColor( file[i].AfterFirst('S') );
                        
                        if( file[i].StartsWith("!I") )
                            aSyntax->indentColor = StringToColor( file[i].AfterFirst('I') );
                        
                        /// Extract style number
                        file[i].BeforeFirst('|').ToLong(&stylenum); //! stylenum is used from here on only as an index into the styles array so,
                        //! we readjust/ reset stylenum to the index of  the styles array containing that style. (In the case of HTML index != stylenum)
                        bool found = false;
                        for( long k=0; k< aSyntax->stylesUsed; k++ )  {  //! Scan for the index containing stylenum
                            if( stylenum == aSyntax->styles[k].styleNum) { stylenum = k ;  found = true; break;  } //! So we reuse stylenum as an index into sytes[]
                        }
                        if( ! found ) continue; //! Next line
                        mySpec =  file[i].AfterFirst('|');  //Index position is != styleNum
                        wxString colorNum, token;
                        wxStringTokenizer tknzr( mySpec, _T(","), wxTOKEN_RET_EMPTY );  //! Make sure return empty fields
                        if ( tknzr.HasMoreTokens() )    token = tknzr.GetNextToken(); //! Skip description
                        if ( tknzr.HasMoreTokens() )    token = tknzr.GetNextToken(); //! should be bold
                        if( token == "bold") strncpy( aSyntax->styles[stylenum].bold, "bold", 4 ); //!No danger of overflow - using literal
                        else  aSyntax->styles[stylenum].bold[0] = '\0';
                        if ( tknzr.HasMoreTokens() )    token = tknzr.GetNextToken();
                        if( token == "italic")  strncpy( aSyntax->styles[stylenum].italic, "italic", 6 );
                        else  aSyntax->styles[stylenum].italic[0] = '\0';
                        if ( tknzr.HasMoreTokens() )    token = tknzr.GetNextToken();
                        if( token.Find("fore:") != -1 ) {
                            colorNum = token.AfterFirst(':'); //! no need to check or trim since we are controlling size in our for loop
                            for( size_t n=0; n<6; n++ )  aSyntax->styles[stylenum].fore[n] = colorNum.GetChar(n);
                            aSyntax->styles[stylenum].fore[6] = '\0'; //!terminate string
                        }
                        if ( tknzr.HasMoreTokens() )    token = tknzr.GetNextToken();
                        if( token.Find("back:") != -1 ) {
                            colorNum = token.AfterFirst(':'); 
                            for( size_t n=0; n<6; n++ ) aSyntax->styles[stylenum].back[n] = colorNum.GetChar(n);
                            aSyntax->styles[stylenum].back[6] = '\0'; //!terminate string
                        }
                        if ( tknzr.HasMoreTokens() )     token = tknzr.GetNextToken();
                        if( token.Find("face:") != -1 ) {
                            colorNum = token.AfterFirst(':'); int len=colorNum.Length();//! no need to trim since we are controlling size
                            if(len > 31 ) //! face[] size is 32
                                { str3.Printf("Font: \"%s\" - name exceeds size limit", colorNum); SetStatusText(str3); continue; }
                            for( size_t n=0; n< len; n++ ) aSyntax->styles[stylenum].face[n] = colorNum.GetChar(n);
                            aSyntax->styles[stylenum].face[len] = '\0'; //!terminate string
                        }
                        if ( tknzr.HasMoreTokens() )    token = tknzr.GetNextToken();
                        if( token.Find("size:") != -1 ) {
                            colorNum = token.AfterFirst(':');    int len=colorNum.Length();//! no need to trim since we are controlling size
                            if(len > 2 ) { SetStatusText(wxString::Format("Field \"size:\" length (2) exceeded in %s", aSyntax->name)); continue; }
                            for( size_t n=0; n< len; n++ ) aSyntax->styles[stylenum].size[n] = colorNum.GetChar(n);
                            aSyntax->styles[stylenum].size[len] = '\0'; //!terminate string
                        }
                    } //! "inSection" for loop
                } //! Lines
            }  //! syntax section

             /*! EXTRA */
             /** Synopsis - The file is scanned for the Section Header. Once found, the inSection flag is set.
                 Every line thereafter is operated on in the "inSection" if-statement body until we hit the Section Footer 
                 which will immediately terminate the scan*/
             inSection = false;
             for( i=0; i < m; i++ )  { //! Start from top scanning each line  for beginning of section
                if( file[i] == "[END_EXTRA]")   {
                    inSection = false;  break;      //! look for next section//!Optimize? should not need inSection=false;
                }
                if( file[i] == "[EXTRA]" )    { 
                    inSection = true;   continue; //! continue at next line
                }
                if(inSection) {
                    //! For Each Line in Section do these
                    if( file[i].StartsWith("#") || file[i] == "" ) continue; //! skip comment or blank line
    
                    file[i].BeforeFirst('|').ToLong(&stylenum);  //! Put in stylenum
                    if(stylenum == 0 ) {  //! Bogus stylenum for Caret color
                       mySpec =  file[i].AfterFirst('|'); //! Pick up Caret Color
                       if (mySpec.Length() != 6) continue;  //! next line //6 hex digits
                       g_caretColor = StringToColor(mySpec);
                    }
                    if( stylenum > 0 && stylenum <= 16 ) { //! Pick up CustomColors
                       mySpec =  file[i].AfterFirst('|');
                       if (mySpec.Length() != 6) continue;  //! next line //6 hex digits
                       customColors[stylenum -1] = StringToColor(mySpec);
                    }
                    if(stylenum == 20 ) {  //! Bogus stylenum for Selection color
                       mySpec =  file[i].AfterFirst('|'); //! Pick up Selection Color
                       if (mySpec.Length() != 6) continue;  //! next line //6 hex digits
                       g_selColor = StringToColor(mySpec);
                    }
                    if(stylenum == 21 ) {  //! Bogus stylenum for Fold color
                       mySpec =  file[i].AfterFirst('|'); //! Pick up Fold Color
                       if (mySpec.Length() != 6) continue;  //! next line //6 hex digits
                       g_foldColor = StringToColor(mySpec);
                    }
                    if(stylenum == 30 ) {  //! Bogus stylenum for EOL Default
                       mySpec =  file[i].AfterFirst('|');
                       if (mySpec.Length() != 1) continue;  //! should only need one digit to represent EOL
                       long eol;  mySpec.ToLong(&eol); ///convert and put in eol
                       m_startEol = (int)eol;  ///editor may not exist yet so store in local var
                    }
                    if(stylenum == 31 ) {  //! Bogus stylenum for Print Color Mode
                       mySpec =  file[i].AfterFirst('|');
                       if(mySpec.Length() != 1) continue;  //! should only need one digit to represent PCOL
                       long pcol;  mySpec.ToLong(&pcol);
                       m_printColMode = (int)pcol;  ///editor may not exist yet so store in local var
                    }
                    if(stylenum == 32 ) {  //! Tab Width
                       mySpec =  file[i].AfterFirst('|');
                       if(mySpec.Length() != 1) continue;
                       long tab;  mySpec.ToLong(&tab);
                       m_tabWidth = (int)tab;  ///editor may not exist yet so store in local var
                    }
                    if(stylenum == 33 ) {  //! Don't Print Nums
                       mySpec =  file[i].AfterFirst('|');
                       if(mySpec.Length() != 1) continue;  //! should only need one digit to represent a bool
                       m_dontPrintNums = (mySpec == "1");
                    }
                    if(stylenum == 40 ) { 
                       mySpec =  file[i].AfterFirst('|');
                       if(mySpec.Length() > 2) continue;  //! should only need 2 digits
                       long num; mySpec.ToLong( &num );
                       g_numTopRecentFiles = (int) num;
                    }
                    if(stylenum == 41 ) {
                       mySpec =  file[i].AfterFirst('|');
                       if(mySpec.Length() > 2) continue;
                       long num; mySpec.ToLong( &num );
                       g_numRecentDirs = (int) num;
                    }
                    if(stylenum == 45 ) { 
                       mySpec =  file[i].AfterFirst('|');
                       if(mySpec.Length() != 1) continue;  //! should only need one digit to represent a bool
                       m_htmlAC = (mySpec == "1");
                    }
                    if(stylenum == 50 ) {  /// Startup Lex
                       mySpec =  file[i].AfterFirst('|');
                       if(mySpec.Length() > 2) continue;
                       long num; mySpec.ToLong( &num );
                       m_startLex = (int) num;
                    }
                    if(stylenum == 52 ) {  /// Startup Zoom level
                       mySpec =  file[i].AfterFirst('|');
                       if(mySpec.Length() > 2) continue;  /// 2 digit limit
                       g_initialZoom = mySpec; ///String value //value read into zion's var to be picked up by the edit module on construction
                       g_initialZoom.ToLong( &m_zoomLevel );  ///Used in timer routine
                    }
                    if(stylenum == 53 ) { 
                       mySpec =  file[i].AfterFirst('|');
                       if( ! wxFileName::DirExists( mySpec ) ) {
                         wxMessageBox( _T("Path to API Files is invalid\nSet a valid path in Options | Settings | General Options"),
                            _T("Invalid API Folder") ); }
                       else g_pathToApi = mySpec; ///String value //value read into zion's var to be picked up by the edit module on construction
                    }
                    if(stylenum == 54 ) { 
                       mySpec =  file[i].AfterFirst('|');
                       if( ! wxFileName::FileExists( mySpec ) )
                         wxMessageBox( _T("Path to Web Browser in zion.ini is invalid\nSet a valid path in Options | Settings | General Options"),
                            _T("Invalid Web Browser") );
                       else g_pathToBrowser = mySpec; ///String value //value read into zion's var to be picked up by the edit module on construction
                    }
                    if(stylenum == 55 ) { ///Line Num width
                       mySpec =  file[i].AfterFirst('|');  if(mySpec.Length() != 1) continue;
                       long num; mySpec.ToLong( &num );
                       g_lineNumWidth = (int) num;
                    }
                    if(stylenum == 56 ) { ///Num A/C chars
                       mySpec =  file[i].AfterFirst('|');  if(mySpec.Length() != 1) continue;
                       long num; mySpec.ToLong( &num );
                       g_numACChars = (int) num;
                    }
                    if(stylenum == 57 ) { ///AC Active
                       mySpec =  file[i].AfterFirst('|');  /*if(mySpec.Length() != 1) continue; ///This check not critical*/
                       long num; mySpec.ToLong( &num );
                       g_acActive =  num ? true: false;
                    }
                    if(stylenum == 58 ) { ///CT
                       mySpec =  file[i].AfterFirst('|');  /*if(mySpec.Length() != 1) continue; ///This check not critical*/
                       long num; mySpec.ToLong( &num );
                       g_ctActive =  num ? true: false;
                    }
                    if(stylenum == 60 ) {
                       mySpec =  file[i].AfterFirst('|'); //if(mySpec.Length() > 2) continue;
                       long num; mySpec.ToLong( &num );
                       m_startOpt = (int) num;
                    }
                    if(stylenum == 61 ) {
                       mySpec =  file[i].AfterFirst('|');
                       long num; mySpec.ToLong( &num );
                       m_startPos.x = (int) num;
                    }
                    if(stylenum == 62 ) {
                       mySpec =  file[i].AfterFirst('|');
                       long num; mySpec.ToLong( &num );
                       m_startPos.y = (int) num;
                    }
                    if(stylenum == 63 ) {
                       mySpec =  file[i].AfterFirst('|');
                       long num; mySpec.ToLong( &num );
                       m_startSize.SetWidth( (int) num );
                    }
                    if(stylenum == 64 ) {
                       mySpec =  file[i].AfterFirst('|');
                       long num; mySpec.ToLong( &num );
                       m_startSize.SetHeight((int) num );
                    }
                    if(stylenum == 70 ) {  /// Autobookmark enable
                       mySpec =  file[i].AfterFirst('|'); /* if(mySpec.Length() != 1) continue; ///we wont check since not critical*/ /// 1 digit limit
                       long num; mySpec.ToLong( &num );
                       g_autoBook = num ? true: false;
                       m_autoBook = g_autoBook;  /// Synch per session and permanent values.
                    }
                    if(stylenum == 72 ) {  /// Autobookmark enable
                       mySpec =  file[i].AfterFirst('|'); /* if(mySpec.Length() != 1) continue; ///we wont check since not critical*/ /// 1 digit limit
                       long num; mySpec.ToLong( &num );
                       g_autoZoom = num ? true: false;
                       m_autoZoom = g_autoZoom;  /// Synch per session and permanent values.
                    }
                }//! end "in section"
            } //! extra section for loop
            
            /*! EXTENSIONS */
             inSection = false;
             for( i=0; i < m; i++ )  { //! Start from top scanning each line  for beginning of section
                if( file[i] == "[END_EXTENSIONS]")   {
                    inSection = false;  break;      //! look for next section//!Optimize? should not need inSection=false;
                }
                if( file[i] == "[EXTENSIONS]" )    { 
                    inSection = true; continue; /// prepare to load ext. to lexer hashmap /// continue at next line
                }
                if(inSection) {
                    //! For Each Line in Section do these
                    if( file[i].StartsWith("#") || file[i] == "" ) continue; //! skip comment or blank line
                    
                    file[i].AfterFirst(',').ToLong(&stylenum);  //! Reuse stylenum (long) ///pick up lexer #
                    if( stylenum > 0 && stylenum < 76 )  ///sensible lexer number range
                        ext2lex[file[i].BeforeFirst(',')] = (int) stylenum; /// set the hash (string key, int value)
                }//! end in section
            } //! Extensions section for loop
            
            /** C++_WORD2 */
             g_cppWords2 = _T(""); inSection = false; 
             for( i=0; i < m; i++ )  { //! Start from top scanning each line  for beginning of section
                if( file[i] == "[END_C++_WORDS2]")   {
                    inSection = false;  break;      //! look for next section//!Optimize? should not need inSection=false;
                }
                if( file[i] == "[C++_WORDS2]" )    { 
                    inSection = true; continue; /// prepare to load ext. to lexer hashmap /// continue at next line
                }
                if(inSection) {
                    str = file[i];
                    ///MY OWN CHOMP!
                    if( str.Last() == '\n' )  str.RemoveLast();  ///Win - CR left, Unix - good
                    if( str.Last() == '\r' )  str.RemoveLast();  ///Win - leftover CR removed -good, Mac - good
                    str.Append(' '); ///Replace the removed newline with a space
                    g_cppWords2 += str; //m_cppWord2 (May not need this member)
                }
            }
            //Locks up! //delete cppSyntax.words2; ///Attempt to release memory occupied by the literal ///Can't release since it was created on the stack
            cppSyntax.words2 = g_cppWords2.c_str();  ///Pointer to the string data in the global returned.
            
            /** C#_WORD2 */
            g_csWords2 = _T(""); inSection = false;
            for( i=0; i < m; i++ )  { //! Start from top scanning each line  for beginning of section
                if( file[i] == "[END_C#_WORDS2]")   {
                    inSection = false;  break;      //! look for next section//!Optimize? should not need inSection=false;
                }
                if( file[i] == "[C#_WORDS2]" )    { 
                    inSection = true; continue; /// prepare to load ext. to lexer hashmap /// continue at next line
                }
                if(inSection) {
                    str = file[i];
                    ///MY OWN CHOMP!
                    if( str.Last() == '\n' )  str.RemoveLast();  ///Win - CR left, Unix - good
                    if( str.Last() == '\r' )  str.RemoveLast();  ///Win - leftover CR removed -good, Mac - good
                    str.Append(' '); ///Replace the removed newline with a space
                    g_csWords2 += str; //m_cppWord2 (May not need this member)
                }
            }
            csSyntax.words2 = g_csWords2.c_str();  ///Pointer to the string data in the global returned.
            
            /** JAVA_WORD2 */
            g_javaWords2 = _T(""); inSection = false;
             for( i=0; i < m; i++ )  { //! Start from top scanning each line  for beginning of section
                if( file[i] == "[END_JAVA_WORDS2]")   {
                    inSection = false;  break;      //! look for next section//!Optimize? should not need inSection=false;
                }
                if( file[i] == "[JAVA_WORDS2]" )    { 
                    inSection = true; continue; /// prepare to load ext. to lexer hashmap /// continue at next line
                }
                if(inSection) {
                    str = file[i];
                    ///MY OWN CHOMP!
                    if( str.Last() == '\n' )  str.RemoveLast();  ///Win - CR left, Unix - good
                    if( str.Last() == '\r' )  str.RemoveLast();  ///Win - leftover CR removed -good, Mac - good
                    str.Append(' '); ///Replace the removed newline with a space
                    g_javaWords2 += str; //m_cppWord2 (May not need this member)
                }
            }
            javaSyntax.words2 = g_javaWords2.c_str();  ///Pointer to the string data in the global returned.
            
            /** SQL_WORD2 */
            g_sqlWords2 = _T(""); inSection = false;
             for( i=0; i < m; i++ )  { //! Start from top scanning each line  for beginning of section
                if( file[i] == "[END_SQL_WORDS2]")   {
                    inSection = false;  break;      //! look for next section//!Optimize? should not need inSection=false;
                }
                if( file[i] == "[SQL_WORDS2]" )    { 
                    inSection = true; continue; /// prepare to load ext. to lexer hashmap /// continue at next line
                }
                if(inSection) {
                    str = file[i];
                    ///MY OWN CHOMP!
                    if( str.Last() == '\n' )  str.RemoveLast();  ///Win - CR left, Unix - good
                    if( str.Last() == '\r' )  str.RemoveLast();  ///Win - leftover CR removed -good, Mac - good
                    str.Append(' '); ///Replace the removed newline with a space
                    g_sqlWords2 += str;
                }
            }
            sqlSyntax.words6 = g_sqlWords2.c_str();  ///SQL words are in a different var
        } //! End READSTYLES if

        ///PRINT
        if( readFlags & READPRINT ) {
            if( FindSection( file, _T("PRINT"), sectionPositions ) ) {
                long T, L, B, R, id, orient; T=L=B=R=id=orient=0;
                for( i = sectionPositions.start +1; i < sectionPositions.end; i++ ) {
                    file[i].BeforeFirst('|').ToLong(&stylenum);  //! Put in stylenum
                    if(stylenum == 0 ) {  ///Top
                       mySpec =  file[i].AfterFirst('|');
                       if(mySpec.Length() > 2) continue;  /// Shouldn't need > 2 digits
                       mySpec.ToLong(&T);
                    }
                    if(stylenum == 1 ) {  ///Left
                       mySpec =  file[i].AfterFirst('|');
                       if(mySpec.Length() > 2) continue;  /// Shouldn't need > 2 digits
                       mySpec.ToLong(&L);
                    }
                    if(stylenum == 2 ) {  ///Bottom
                       mySpec =  file[i].AfterFirst('|');
                       if(mySpec.Length() > 2) continue;  /// Shouldn't need > 2 digits
                       mySpec.ToLong(&B);
                    }
                    if(stylenum == 3 ) {  ///Right
                       mySpec =  file[i].AfterFirst('|');
                       if(mySpec.Length() > 2) continue;  /// Shouldn't need > 2 digits
                       mySpec.ToLong(&R);
                    }
                    if(stylenum == 4 ) {  ///Paper id
                       mySpec =  file[i].AfterFirst('|');
                       if(mySpec.Length() > 2) continue;  /// Shouldn't need > 2 digits
                       mySpec.ToLong(&id);
                       g_printData->SetPaperId( static_cast</*enum*/ wxPaperSize> (id) );
                    }
                    if(stylenum == 5 ) {  ///Orientation
                       mySpec =  file[i].AfterFirst('|');
                       if(mySpec.Length() > 1) continue;
                       mySpec.ToLong(&orient);
                       g_printData->SetOrientation( (int) orient);
                    }
                }
                wxPoint LT(L,T);
                g_pageSetupData->SetMarginTopLeft( LT ); ///x,y
                wxPoint RB(R,B);
                g_pageSetupData->SetMarginBottomRight( RB );
            }
        }
        //return true;
    } //! file exists and opened
    else { SetStatusText("Can't find zion.ini"); /* return false; */  }
    file.Close();

    ///HISTORY
    wxTextFile file2(GetIniDir() + "\\history.ini");   //! Windows specific! //simple text file object //Automatic var. - created on stack
    if( ! file2.Exists() ) {
        if( ! init ) {  ///ONLY on startup can write to history.ini
            SetStatusText(_T("Could not find \"history.ini\"") );  return false;
        }
        if( wxMessageBox( _T("History.ini not found in same location as zion.ini!\nCreate it?"), _T("Create History.ini?"), wxYES_NO ) == wxYES ) {
            if( ! file2.Create() ) { SetStatusText(_T("Failed creating history.ini - Is media Readonly?") ); return false; }
            else {
                SetStatusText( _T("History.ini created successfully") );
                ///For performance reasons to Recent Files first
                if( file2.Open() ) {
                    file2.InsertLine(_T(""), 0); ///Start with a blank line
                    file2.InsertLine(_T("[RECENTFILES]"), 1 ); ///Start with a blank line
                    file2.InsertLine(_T("[END_RECENTFILES]"), 2);
                    file2.InsertLine(_T(""), 3); ///Start with a blank line
                    file2.InsertLine(_T("[EXEC]"), 4 ); ///Start with a blank line
                    file2.InsertLine(_T("[END_EXEC]"), 5 );
                    file2.Write();
                    file2.Close();
                }
            }
        }
        else { SetStatusText( _T("History.ini creation canceled") ); return false; }
    }
    if( file2.Open() ) {
      size_t i;
      StartEnd sectionPositions;  ///Struct to pass back section positions from FindSection()
        /*! HISTORY */
        if( readFlags & READHISTORY ) {
                size_t firstLine, endSection, n, r;
            
            /*! EXEC LOAD */ 
            m_execHistory.Empty();  ///Clear the history
            if( FindSection( file2, _T("EXEC"), sectionPositions ) ) { ///We cannot create a section here as file2 is never written back ///True will create it if not found
                firstLine = sectionPositions.start +1;  endSection = sectionPositions.end;
                //No limit on wxArrays! //if( g_numTopRecentFiles > NUMRECENTFILES ) g_numTopRecentFiles = NUMRECENTFILES;  /// Make sure we don't exceed the internal limit
                for( i= firstLine; i < endSection; i++ )  { m_execHistory.Add( file2[i] ); }
            }
                
            ///CLEAN RECENTFILE MENU
            r = recentFilesMenu->GetMenuItemCount();
            for( n = 0 ; n< r ; n++ ) recentFilesMenu->Delete( ZION_FILE_1 + n ); //! Remove all recentFile items

            /// CLEAN RECENTDIR
            r = recentDirsMenu->GetMenuItemCount();
            r = r - 1;  ///Adjust for Separator which is now counted as a menu item
            for( n = 0; n< r; n++ ) { /// Detach all known (sub)menu items
                if( recentDirsMenu->FindItem( ZION_DIR_1 + n )  != NULL) { 
                    ///CRITICAL CODE: Destroy obliterates the menu so the pointer dirSubs[n] is no longer valid so we must recreate a wxMenu for it to point to
                    recentDirsMenu->Destroy( ZION_DIR_1 + n );  ///Remove the entire submenu ///will delete all submenu items also
                    dirSubs[n] = new wxMenu; ///Recreate what was destroyed
                    dirSubItems[n] = NULL;  ///Reinit the menu item of the submenu(as an item is added to a parent menu, a menuitem is returned, we are saving these) 
                }
            }
            r = recentDirsMenu->GetMenuItemCount(); ///At this point all subdir items are removed from the menu. Now wipe it entirely clean of any recent files
            r = r - 1;  ///Adjust for Separator which is now counted as a menu item
            for( n = 0 ; n< r ; n++ ) recentDirsMenu->Delete( ZION_FILE_1 + n );
            //We will no longer delete the separator //recentDirsMenu->Delete( recentDirsMenu->FindItemByPosition(0) ); /// lastly, remove the separator also.
     
            /*! RECENTFILES */
            if( FindSection( file2, _T("RECENTFILES"), sectionPositions ) ) {
               firstLine = sectionPositions.start +1;  endSection = sectionPositions.end;
                //! Add
                if( g_numTopRecentFiles > NUMRECENTFILES ) g_numTopRecentFiles = NUMRECENTFILES;  /// Make sure we don't exceed the internal limit
                for( i= firstLine; i < endSection; i++ ) {
                    if( i - firstLine >  NUMRECENTFILES -1 ) break;   //! Safety
                    recentFilesMenu->Append( ZION_FILE_1 + i - firstLine, file2[i] ); ///Though we don't display recentfiles menu, we still use it internally to help locate file items
                    if( i - firstLine < g_numTopRecentFiles )  ///Add it to recentDirs also
                        recentDirsMenu->Insert(  i - firstLine /*position */, ZION_FILE_1 + i - firstLine, file2[i] ); ///cheat: add it the recentdirMenu too, but only take 10 items. We will also borrow the event handler for RecentFiles as our items will have the same ID as the corresponding item in recentfiles
                }
            }
     
                /*! RECENTDIRS */
            ///At this point we have located the first position after the section header and the section footer
            if( FindSection( file2, _T("RECENTDIRS"), sectionPositions)) { ///Better called sectionFound   // Comments not allowed in this section!  if( file[i].StartsWith("#") || file[i] == "" ) continue; //! skip comment or blank line
              firstLine = sectionPositions.start +1;  endSection = sectionPositions.end;
            
                /// ADD NEW SUBMENUS
                if( g_numRecentDirs > NUMRECENTDIRS ) g_numRecentDirs = NUMRECENTDIRS;  /// Make sure we don't exceed the internal limit
                int icount = 0; /// Track num of menu items (Total = Num of dirs x Num of items)
                ///Scan all dirs
                for( i= firstLine; i < endSection; i++ ) { /// (i  - firstLine) gives us a zero-based index
                   if( i - firstLine >  g_numRecentDirs -1 ) break;   //! Vital
                   ///Append submenu
                    dirSubItems[i - firstLine] = recentDirsMenu->Append( ZION_DIR_1 + i - firstLine,
                            file2[i], dirSubs[i - firstLine] ); ///A menuItem ptr is returned here, which we have no way of accessing later, so save it. A wxMenuItem * cannot be cast to a wxMenu

                    ///APPEND ITEMS TO SUBMENU
                    ///At this point we can scan for the individual dir section. ///Be careful not to override any existing variables and that the file is not changed during this time
                    size_t firstLine2, endSection2, i2;
                    ///First Append the "Open" item
                    dirSubs[i-firstLine]->Append( ZION_OPENITEM_1 + i-firstLine, _T("Open") );  /// i-firstLine is our zero-based index
                    dirSubs[i-firstLine]->AppendSeparator();
                    ///Scan the file again for the individual dir section /// Comments not allowed in this section!
                    if( FindSection( file2, file2[i], sectionPositions  ) ) {  ///Just scan, don't create section
                        firstLine2 = sectionPositions.start +1;  endSection2 = sectionPositions.end; /// At this point we have located the first position after the section header and the section footer
                       ///Scan all file items
                       for( i2= firstLine2; i2 < endSection2; i2++ ) { /// i  - firstLine gives us a zero-based index
                           if( i2 - firstLine2 >  NUMRECENTFILES -1 ) break;   //! Safety
                            ///Combine info from the outer and inner loops. Append all file items to the dir submenu
                           dirSubs[i-firstLine]->Append( ZION_FILEITEM_1 + icount++, file2[i2] ); /// icount: base + total items used - inc. after use*/
                       }
                    }
                    dirSubs[i-firstLine]->AppendSeparator();
                    dirSubs[i-firstLine]->Append( ZION_REMOVEITEM_1 + i-firstLine, _T("Prune") );  /// i-firstLine is our zero-based index
                }
             }
        } //! End READHISTORY if
    } ///End file2 opened
    else {
        SetStatusText(_T("Failed Reading \"history.ini\"") );
        return false;
    }
    return true;
}

void ZionFrame::CreateMenu() {
   m_menuBar = new wxMenuBar;
   
   newZionSubMenu = new wxMenu;
   newZionSubMenu->Append( ZION_NEWZION_L, _T("Left\tCtrl+F9") );
   newZionSubMenu->Append( ZION_NEWZION_C, _T("Center\tCtrl+F10") );
   newZionSubMenu->Append( ZION_NEWZION_R, _T("Right\tCtrl+F11") );
   newZionSubMenu->Append( ZION_NEWZION_T, _T("Top") );
   newZionSubMenu->Append( ZION_NEWZION_B, _T("Bottom") );
   newZionSubMenu->AppendSeparator();
   newZionSubMenu->Append( ZION_NEWZION_M, _T("Maximized") );
   newZionSubMenu->AppendSeparator();
   newZionSubMenu->Append( ZION_NEWZION_TR, _T("Top Right") );
   newZionSubMenu->Append( ZION_NEWZION_BR, _T("Bottom Right") );
   newZionSubMenu->Append( ZION_NEWZION_TL, _T("Top Left") );
   newZionSubMenu->Append( ZION_NEWZION_BL, _T("Bottom Left") );
   
   newHTMLSub = new wxMenu;
   newHTMLSub->Append(ZION_NEWHTML, _T("&Blank HTML\tShift+F11"));
   newHTMLSub->AppendSeparator();
   newHTMLSub->Append(ZION_NEWHTML_TEMPLSTRICT, _T("Use &Strict. Template\tCtrl+Shift+F11"), _T("Will ALWAYS open in THIS WINDOW"));
   newHTMLSub->Append(ZION_NEWHTML_TEMPLTRANS, _T("Use Trans. &Template\tCtrl+Shift+F12"), _T("Will ALWAYS open in THIS WINDOW"));
   newHTMLSub->Append(ZION_NEWHTML_TEMPLBASIC, _T("&Basic HTML"), _T("Will ALWAYS open in THIS WINDOW"));
   newHTMLSub->Append(ZION_NEWHTML_TEMPLBASICNH, _T("&Basic HTML no <head>"), _T("Will ALWAYS open in THIS WINDOW"));
   newHTMLSub->Append(ZION_NEWHTML_TEMPLASP, _T("&ASP"), _T("Will ALWAYS open in THIS WINDOW"));
   newSubMenu = new wxMenu;  //! Submenu should be populated before adding to parent menu!
   newSubMenu->Append( ZION_NEWZION, _T("&ZionEdit Window"), newZionSubMenu );
   newSubMenu->AppendSeparator();
   newSubMenu->Append(ZION_NEWHTML_SUB, _T("&HTML"), newHTMLSub);
   newSubMenu->Append(ZION_NEWCS, _T("&C# File\tShift+F5"));
   newSubMenu->Append(ZION_NEWCPP, _T("&C++ File\tShift+F6"));
   newSubMenu->Append(ZION_NEWJAVA, _T("&Java File\tShift+F7"));
   newSubMenu->Append(ZION_NEWVBSCRIPT, _T("&VB\tShift+F8"));
   newSubMenu->Append(ZION_NEWPYTHON, _T("P&ython Script\tShift+F9"));
   newSubMenu->Append(ZION_NEWRUBY, _T("&Ruby Script\tShift+F10"));
   newSubMenu->Append(ZION_NEWPERL, _T("&Perl Script"));
   newSubMenu->Append(ZION_NEWCSS, _T("&CSS File"));
   newSubMenu->Append(ZION_NEWSQL, _T("&SQL Script"));
   newSubMenu->Append(ZION_NEWASM, _T("&Asm File"));
   newSubMenu->Append(ZION_NEWFORTRAN, _T("&Fortran9X File"));
   newSubMenu->Append(ZION_NEWF77, _T("&Fortran77 File"));
   newSubMenu->Append(ZION_NEWLISP, _T("&Lisp File"));
   newSubMenu->Append(ZION_NEWMAKE, _T("&Make File"));
   newSubMenu->Append(ZION_NEWBATCH, _T("&Batch File"));
   newSubMenu->Append(ZION_NEWNULL, _T("&Text File"));
   
   recentDirsMenu = new wxMenu;     //! Items will be added later
   recentDirsMenu->AppendSeparator();
   recentFilesMenu = new wxMenu;     //! Ditto
   
	fileMenu = new wxMenu;
	fileMenu->Append(ZION_NEW, _T("&New"), newSubMenu);
	fileMenu->AppendSeparator();
	fileMenu->Append(ZION_OPEN, _T("&Open\tCtrl+O") );
	fileMenu->Append(ZION_OPENSELECTED, _T("Open Selected\tCtrl+Alt+O"), _T("Open the selected filename") );
	fileMenu->Append(ZION_RECENTDIRS, _T("Recen&t"), recentDirsMenu);
	fileMenu->Append(ZION_RELOAD, _T("&Reload\tF5"), _T("Reload current file into the editor") );
	fileMenu->Append(ZION_FILECLOSE, _T("C&lose\tShift+F4"), _T("Close the current file") );
	fileMenu->AppendSeparator();
	fileMenu->Append(ZION_SAVE, _T("&Save \tCtrl+S"));
	fileMenu->Append(ZION_SAVE_AS, "Save &As...\tCtrl+Alt+S");
	fileMenu->AppendSeparator();
	fileMenu->Append(ZION_PRINTSETUP, "Page Set&up");
	fileMenu->Append(ZION_PRINT, "&Print", _T("Preselect some text to print a portion of the file"));
	fileMenu->Append(ZION_PRINTPREVIEW, "Print Pre&view", _T("Preselect some text to print a portion of the file"));
	fileMenu->AppendSeparator();
	fileMenu->Append(ZION_CMDPROMPT, "&Cmd Prompt...\tCtrl+Alt+C", "Open a DOS window in our current directory");
	fileMenu->Append(ZION_EXPLORE, "&Explore...\tCtrl+E", "Open Windows Explorer in selected directory");
	fileMenu->Append(ZION_RUNANDCAPTURE, "&Run Command...\tCtrl+Alt+R", "Run Command with Option to Capture Output");
	fileMenu->AppendSeparator();
	fileMenu->Append(ZION_EXIT, "E&xit", "Bye!");

    editMenu = new wxMenu;
    editMenu->Append(ZION_UNDO, "&Undo\tCtrl+Z");
    editMenu->Append(ZION_REDO, "&Redo\tCtrl+Alt+Z");
    editMenu->AppendSeparator();
    editMenu->Append(ZION_CUT, "Cu&t\tCtrl+X");
    editMenu->Append(ZION_COPY, "&Copy\tCtrl+C");
    editMenu->Append(ZION_PASTE, "&Paste\tCtrl+V");
    editMenu->AppendSeparator();
    editMenu->Append(ZION_FIND, _T("&Find...\tCtrl+F"), _T("The standard Find Dialog"));
    editMenu->Append(ZION_FINDALL, "Find &All...\tF2", _T("[F2] Find and List all ocurrences (Preselect for immediate listing)"));
    editMenu->Append(ZION_FINDNEXT, "Find &Next \tF3", _T("[F3] Preselect a string for immediate search"));
    editMenu->Append(ZION_FINDPREV, _T("Find Pre&vious \tF4"), _T("Search backwards until top of file"));
    editMenu->Append(ZION_FINDALLFILES, _T("Find in All Files...\tF6"), _T("[F6] Find + List ALL Files containing a pattern"));
    editMenu->Append(ZION_FINDREPLACE, _T("Find/&Replace...\tCtrl+R"), _T("Classic Find/Replace Dialog") );
    editMenu->AppendSeparator();
    editMenu->Append(ZION_SELECTWORD, _T("Select &Word\tCtrl+W"), _T("Selects the word at the Caret") );
    editMenu->Append(ZION_SELECTLINE, _T("Select &Line\tCtrl+Alt+N") );
    editMenu->Append(ZION_SELTOBRACE, _T("Select to &Brace\tCtrl+Alt+M"), _T("Select to Matching Brace (Includes the braces)") );
    editMenu->Append(ZION_SELECTALL, _T("Select &All\tCtrl+A"), _T("Select the entire file") );
    editMenu->Append(ZION_SELECTRECT, _T("Vertical Selection\t(ALT-Drag)") ,_T("ALT before Drag or Shift+ALT+ArrowKey Rectangular Selection") );
    
    lineSubMenu = new wxMenu;
    lineSubMenu->AppendRadioItem(ZION_LINE_WINDOWS, _T("Windows CRLF"), "Set End of Line Chars to \\r\\n" ); //! Radio items must be in order ?
    lineSubMenu->AppendRadioItem(ZION_LINE_MAC, _T("Mac CR"), "Set End of Line Chars to \\r" );
    lineSubMenu->AppendRadioItem(ZION_LINE_UNIX, _T("Unix  LF"), "Set End of Line Chars to \\n" );
    
    ///WORKING//
    wxMenu * blankLinesMenu = new wxMenu;
    blankLinesMenu->Append(ZION_REMOVE1BLANK, _T("Del 1 in Group\tCtrl+Alt+B"), _T("Remove 1 blank line in group of consecutive blank lines - Selection Aware") );
    blankLinesMenu->Append(ZION_REMOVEGT2BLANKS, _T("Del > 2 Blanks"), _T("Remove any more than 2 consecutive blank lines - Selection Aware") );
    blankLinesMenu->Append(ZION_REMOVEALLBLANKS, _T("Del All Blank Lines"), _T("Removes ALL Blank lines - Selection Aware") );
    
    editSpecial = new wxMenu;
    editSpecial->Append(ZION_ASSOCTYPE, _T("&Assoc Idfr to Type\tCtrl+Alt+A"), _T("Associate Identifier to object type") );
    editSpecial->Append(ZION_AUTOCOMPSHOW, _T("Auto Complete\tCtrl+J"), _T("Shows the autocompletion list") );
    editSpecial->AppendSeparator();
    editSpecial->Append(ZION_DUPLICATELINE, _T("&Duplicate Line/Block\tCtrl+D"), _T("Duplicate a Line or Selected Block") );
    editSpecial->Append(ZION_COPYLINE, _T("Cop&y Line\tCtrl+Y") );
    editSpecial->Append(ZION_CUTLINE, _T("&Cut Line\tCtrl+L") );
    editSpecial->Append(ZION_TRANSPOSELINE, _T("&Transpose Line\tCtrl+T"), _T("Move current line up") );
    editSpecial->AppendSeparator();
    editSpecial->Append(ZION_UPPERCASE,_T("To &Uppercase\tCtrl+Alt+U"), _T("First select some text")  );
    editSpecial->Append(ZION_LOWERCASE,_T("To &Lowercase\tCtrl+Alt+L"), _T("First select some text")  );
    editSpecial->AppendSeparator();
    editSpecial->Append(ZION_INCINDENT, _T("++ Indent\t(Tab)"), _T("Select a block first") );
    editSpecial->Append(ZION_REDINDENT, _T("- - Indent\tShift+Tab"), _T("Select a block first") );
    editSpecial->AppendSeparator();
    editSpecial->Append(ZION_ZOOMIN, _T("Zoom In (Ctrl+Numpd+)"), _T("Make text larger - ( Ctrl + Numpad+ )")); //\tF11
    editSpecial->Append(ZION_ZOOMOUT, _T("Zoom Out(Ctrl+Numpd - )"), _T("Make text smaller - ( Ctrl+Numpad - )"));
    editSpecial->Append(ZION_RESETZOOM, _T("Reset Zoom level"), _T("Put text back to normal size"));
    editSpecial->AppendSeparator();
    editSpecial->Append( ZION_REMOVEBLANKS, _T("Delete Blank Lines"), blankLinesMenu );
    editSpecial->Append(ZION_LINE, _T("&Set EOL char(s)"), lineSubMenu );
    
	togglesMenu = new wxMenu; 
	togglesMenu->AppendCheckItem(ZION_FULLSCRN_TOGGLE, "&Full Screen\tF11");
	togglesMenu->AppendSeparator();
    togglesMenu->AppendCheckItem(ZION_CALLTIPS_TOGGLE, _T("Call Tips"));
    togglesMenu->AppendCheckItem( ZION_CTAC_TOGGLE, _T("CT + AC\tF12") );
    togglesMenu->AppendCheckItem(ZION_AUTOCOMP_TOGGLE, _T("Auto Complete"));
	togglesMenu->AppendSeparator();
    togglesMenu->AppendCheckItem(ZION_AUTOZOOM_TOGGLE, _T("&Auto Zoom\tCtrl+F8"));
    togglesMenu->AppendCheckItem(ZION_AUTOBOOK_TOGGLE, _T("&Auto Bookmarks"));

	togglesMenu->AppendSeparator();
	togglesHorizSb = togglesMenu->AppendCheckItem(ZION_HORIZSB_TOGGLE, "&Horizontal Scrollbar\tCtrl+F6", _T("Show Horizontal Scrollbar")); ///Here we are saving the menuitem so we can disable it in conjuction with WRAP.
    togglesMenu->AppendCheckItem(ZION_VERTSB_TOGGLE, _T("&Vertical Scrollbar\tCtrl+F7"), _T("Show Vertical Scrollbar"));
	togglesMenu->AppendCheckItem(ZION_LINE_TOGGLE, "&Line Numbers\tCtrl+F2");
	togglesMenu->AppendCheckItem(ZION_FOLD_TOGGLE, "&Fold Margin\tCtrl+F3", _T("Allows you to compact blocks of code you want out of the way"));
	togglesMenu->AppendCheckItem(ZION_INDENT_TOGGLE, "&Indentation Guides", _T("Vertical lines that help you keep indents in line"));
	togglesMenu->AppendSeparator();
    togglesMenu->AppendCheckItem(ZION_READONLY_TOGGLE, _T("&Read Only\tCtrl+F5"));
	togglesMenu->AppendCheckItem(ZION_USETABS_TOGGLE, "&Use Real Tabs", _T("Use real tabs instead of expanding tabs to spaces"));
	togglesMenu->AppendCheckItem(ZION_WHITESPACE_TOGGLE, "View &White Space", _T("Show tabs and spaces"));
	togglesMenu->AppendCheckItem(ZION_WRAP_TOGGLE, "&Wrap Lines\tCtrl+F4");
    gotoMenu = new wxMenu;
    gotoMenu->Append(ZION_GOTOBRACE, "&Matching Brace\tCtrl+M");
    gotoMenu->Append(ZION_GOTOSTART, "&Top of Document\tCtrl+Home");
    gotoMenu->Append(ZION_GOTOLINE, "Go to &Line...\tCtrl+G");
    gotoMenu->Append(ZION_GOTOEND, "&End of Document\tCtrl+End");
    
    syntaxMenu = new wxMenu;
    syntaxMenu->AppendCheckItem(ZION_NULL, "Plain &Text");
    syntaxMenu->AppendCheckItem(ZION_CPP, "&C++");
    syntaxMenu->AppendCheckItem(ZION_HTML, "&HTML etc.");
    syntaxMenu->AppendCheckItem(ZION_PERL, "&Perl");
    syntaxMenu->AppendCheckItem(ZION_RUBY, "&Ruby");
    syntaxMenu->AppendCheckItem(ZION_VBSCRIPT, "&VB");
    syntaxMenu->AppendCheckItem(ZION_CSS, "&CSS");
    syntaxMenu->AppendCheckItem(ZION_ASM, "&Asm");
    syntaxMenu->AppendCheckItem(ZION_JAVA, "&Java");
    syntaxMenu->AppendCheckItem(ZION_SQL, "&SQL");
    syntaxMenu->AppendCheckItem(ZION_CS, "&C#");
    syntaxMenu->AppendCheckItem(ZION_PYTHON, "P&ython");
    syntaxMenu->AppendCheckItem( ZION_FORTRAN, "&Fortran9X");
    syntaxMenu->AppendCheckItem( ZION_F77, "&Fortran77");
    syntaxMenu->AppendCheckItem(ZION_LISP, "&Lisp");
    syntaxMenu->AppendCheckItem(ZION_MAKE, "&Make");
    syntaxMenu->AppendCheckItem(ZION_BATCH, "&Batch");
   
    optionsMenu = new wxMenu;
    optionsMenu->Append(ZION_FORCESYNTAX, "&Force Mode", syntaxMenu);
    optionsMenu->AppendSeparator();
    optionsMenu->Append(ZION_SETTINGS, "&Settings...\tF9", "Change Styles and other Settings");
    optionsMenu->Append(ZION_WRITEINI, "S&ave Settings", "Make changes to \"Editor Settings\" permanent");
    optionsMenu->AppendSeparator();
    optionsMenu->Append(ZION_EDITINI, _T("&Edit zion.ini"));
    optionsMenu->Append(ZION_EDITAPI, _T("Edit A&PI file"), _T("Edit the Call Tip /AutoComp file for the current mode"));
    optionsMenu->Append(ZION_EDIT_HISTORY_INI, _T("Edit &history.ini"));
    optionsMenu->Append(ZION_READINI, _T("&Reload *.ini, *.api"), _T("Reload editor settings from .ini files"));
    optionsMenu->AppendSeparator();
    optionsMenu->Append(ZION_COLOURVALUE, "&Get Color Value...");
    optionsMenu->Append(ZION_CLEARSTATUS, "&Clear Status bar");
    //optionsMenu->Append(ZION_CALLTIPSHOW, "Test Call Tips");
    
    ipcMenu = new wxMenu;
    ipcMenu->Append(ZION_SHOWIPCSTATUS, _T("&IPC Status"));
    ipcMenu->Append(ZION_MENURECONN, "Reconn. to Server");
    ipcMenu->Append(ZION_BECOMESERVER, "Become Server");

    helpMenu = new wxMenu;
    helpMenu->Append(ZION_ABOUT, "A&bout");
    helpMenu->AppendSeparator();
    helpMenu->Append(ZION_CONTEXTCALLTIP, _T("Context help\tF1"), _T("Display any CallTip Info for the local word/function") );
    helpMenu->Append(ZION_GOOGLEIT, "&Google It\tCtrl+Alt+G", _T("SELECT some text to Look up on Google!"));
    helpMenu->Append(ZION_ONLINEDOCS, "&Online Docs");
    helpMenu->Append(ZION_HELP, "&Help File\tShift+F1");
    helpMenu->AppendSeparator();
    helpMenu->Append(ZION_BUGREPORT, "Bug &Report/Comment");
    helpMenu->Append(ZION_CHECKLATEST, "&Latest Version");
    helpMenu->AppendSeparator();
    helpMenu->Append(ZION_INFO, _T("Editor Statistics\tCtrl+Alt+E") );
    helpMenu->Append(ZION_DUMPCALLTIPS, _T("Dump CallTip Structs"), _T("This will WRITE to the current file buffer!!!") );
    helpMenu->Append(ZION_IPCSUBMENU, "&IPC", ipcMenu);
	
    m_menuBar ->Append(fileMenu, "&File");
    m_menuBar ->Append(editMenu, "&Edit");
    m_menuBar ->Append(editSpecial, "&SpecialOps");
    m_menuBar ->Append(togglesMenu, "&Toggles");
    m_menuBar ->Append(gotoMenu, "&GoTo");
    m_menuBar ->Append(optionsMenu, "&Options");
    m_menuBar ->Append(helpMenu, "&Help");
    SetMenuBar(m_menuBar);	//set the window's menubar
    
    wxMenu * html_embed = new wxMenu;
    html_embed->Append(ZION_HTML_SCRIPT, "<Script>", _T("Generic Script Tag"));
    html_embed->Append(ZION_HTML_ASPDRTV, "<%@  %>", _T("ASP Directive Tag"));
    html_embed->Append(ZION_HTML_ASP, "<%  %>\tCtrl+Shift+N", _T("ASP Tags"));
    html_embed->Append(ZION_HTML_ASPNET, "<asp: />\tCtrl+Shift+X", _T("ASP.Net Tags"));
    html_embed->Append(ZION_HTML_PHP, "<?php  ?>\tCtrl+Shift+P", _T("PHP Tags"));
    html_embed->AppendSeparator();
    html_embed->Append(ZION_CPPMENU_COMMENT, "Insert /* */", "Will comment a selected block also"); ///Short cut keys already used by regular html comment
    html_embed->Append(ZION_CPPMENU_UNCOMMENT, "Remove /* */\tCtrl+Shift+U", "Select a block to remove comments from");
    html_embed->Append(ZION_CPPMENU_LINECOMMENT, "Insert //", "Insert Line Comments - Block capable. Just select first");
    html_embed->Append(ZION_CPPMENU_LINEUNCOMMENT, "Remove //", "Remove Line Comments. Select block first");
    html_embed->Append(ZION_VBMENU_LINECOMMENT, "Insert '", "Insert VB Comments - Block capable! Just select first");
    html_embed->Append(ZION_VBMENU_LINEUNCOMMENT, "Remove '", "Remove VB Comments. Select block first");
    html_embed->Append(ZION_PERLMENU_LINECOMMENT, "Insert #", "Insert Comments - Block capable! Just select first");
    html_embed->Append(ZION_PERLMENU_LINEUNCOMMENT, "Remove #", "Remove Line Comments. Select block first");
    html_embed->AppendSeparator();
    html_embed->Append(ZION_CPPMENU_CURLY, "{  }", "Curly braces");
    html_embed->Append(ZION_CPPMENU_IF, "&if(  ) { }\tCtrl+Shift+I", _T("Select a Block to place it between braces"));
    html_embed->Append(ZION_VBMENU_IF, _T("If Then [VB]"), _T("Select a Block to place it in the body"));
    html_embed->Append(ZION_CPPMENU_ELSEIF, "&else if( ){ }", _T("A Selected block will be placed between braces"));
    html_embed->Append(ZION_CPPMENU_ELSE, "&else { }\tCtrl+Shift+E", _T("A Selected block will be placed between braces"));
    html_embed->Append(ZION_VBMENU_ELSE, _T("Else [VB]"), _T("A Selected block will be placed in the body"));
    html_embed->Append(ZION_CPPMENU_FOR, "&for( ; ; ) { }\tCtrl+Shift+F", _T("Select a Block to place it between braces"));
    html_embed->Append(ZION_PERLMENU_FOREACH, "foreach  { }", _T("Select a block to place it between braces"));
    html_embed->Append(ZION_CPPMENU_WHILE, "&while( ){ }\tCtrl+Shift+W", _T("A Selected block will be placed between braces"));
    html_embed->Append(ZION_CPPMENU_DOWHILE, "&do { }while( );\tCtrl+Alt+W", _T("A Selected block will be placed between braces"));
    
    wxMenu * html_templ = new wxMenu;
    html_templ->Append(ZION_HTML_TEMPLSTRICT, "Strict Headers", _T("Insert W3C strict headers into page"));
    html_templ->Append(ZION_HTML_TEMPLTRANS, "Transitional Headers", _T("Insert W3C transitional headers into page"));
    html_templ->Append(ZION_HTML_TEMPLBASIC, "Basic HTML", _T("Insert basic HTML header tags into page"));
    html_templ->Append(ZION_HTML_TEMPLBASICNOHEAD, "Basic HTML no <head>", _T("Insert basic HTML header tags into page, no <head> tag"));
    html_templ->Append(ZION_HTML_TEMPLASP, "ASP", _T("Insert basic ASP header tags into page"));
    
    htmlMenu = new wxMenu;
    htmlMenu->Append(ZION_HTML_TEMPL, "HTML Templates", html_templ);
    htmlMenu->Append(ZION_HTMLMENU_QUICKTAG, _T("Quick Tag...\tCtrl+Q"), _T("Preselect some text to surround with tag") );
    htmlMenu->Append(ZION_HTMLMENU_BR, _T("<br/>\tCtrl+Enter"), _T("Select a block to add <br/>  to multiple lines"));
    htmlMenu->Append(ZION_HTMLMENU_BOLD, "&Bold\tCtrl+B", _T("Most tags will encompass a selection"));
    htmlMenu->Append(ZION_HTMLMENU_ITALIC, "&Italic\tCtrl+I", _T("Will encompass a selection"));
    htmlMenu->Append(ZION_HTMLMENU_UNDERLINE, "&Underline\tCtrl+U");
    htmlMenu->Append(ZION_HTMLMENU_PARAGRAPH, "&Paragraph\tCtrl+P", _T("Will encompass a selection"));
    htmlMenu->Append(ZION_HTMLMENU_ANCHOR, "Anchor\tCtrl+Shift+A", _T("Will encompass a selection"));
    htmlMenu->Append(ZION_HTMLMENU_CENTER, _T("Center\tCtrl+Shift+C") );
    htmlMenu->Append(ZION_HTMLMENU_COMMENT, _T("Co&mment <!-- -->\tCtrl+Shift+M") );
    htmlMenu->Append(ZION_HTMLMENU_UNCOMMENT, _T("Remove <!-- -->"), _T("Remove HTML comments") );
    htmlMenu->AppendSeparator();
    htmlMenu->Append(ZION_HTML_EMBED, "SCRIPTING", html_embed);
    htmlTable = new wxMenu;
    htmlTable->Append(ZION_HTMLMENU_TABLE, "<table>");
    htmlTable->Append(ZION_HTMLMENU_TD, "<td>", "table data");
    htmlTable->Append(ZION_HTMLMENU_TR, "<tr>", "table row");
    htmlTable->Append(ZION_HTMLMENU_TH, "<th>", "table heading");
    htmlMenu->Append(ZION_HTMLTABLEMENU, "Table", htmlTable);
    
    htmlHeading = new wxMenu;
    htmlHeading->Append(ZION_HTMLMENU_H3, "<h3>\tCtrl+Shift+H");
    htmlHeading->Append(ZION_HTMLMENU_H1, "<h1>");
    htmlHeading->Append(ZION_HTMLMENU_H2, "<h2>");
    htmlHeading->Append(ZION_HTMLMENU_H4, "<h4>");
    htmlMenu->Append(ZION_HTMLHEADINGMENU, "Heading", htmlHeading);
    
    htmlList = new wxMenu;
    htmlList->Append(ZION_HTMLMENU_LI, "<li>");
    htmlList->Append(ZION_HTMLMENU_UL, "<ul>");
    htmlList->Append(ZION_HTMLMENU_OL, "<ol>");
    htmlMenu->Append(ZION_HTMLLISTMENU, "List", htmlList);
    wxMenu * htmlMore = new wxMenu;
    htmlMore->Append(ZION_HTMLMENU_BLOCKQUOTE, "<blockquote>", _T("Will encompass a selection") );
    htmlMore->Append(ZION_HTMLMENU_CODE, "<code>", _T("Will encompass a selection"));
    htmlMore->Append(ZION_HTMLMENU_EM, "<em>", _T("The <em> tag will encompass a selection"));
    htmlMore->Append(ZION_HTMLMENU_FONT, "<font>", _T("Will encompass a selection"));
    htmlMore->Append(ZION_HTMLMENU_FORM, "<form>", _T("Will encompass a selection"));
    htmlMore->Append(ZION_HTMLMENU_QUOTE, "&quot;\tCtrl+Shift+Q", _T("Will encompass a selection") );
    htmlMore->Append(ZION_HTMLMENU_NBSP, "&nbsp;\tCtrl+Space");
    htmlMore->Append(ZION_HTMLMENU_NDASH, "&ndash;\tCtrl+Shift+D");
    htmlMore->Append(ZION_HTMLMENU_PRE, "<pre>");
    htmlMore->Append(ZION_HTMLMENU_SPAN, "<span>", _T("Will encompass a selection"));
    htmlMore->Append(ZION_HTMLMENU_STRONG, "<strong>", _T("Will encompass a selection"));
    htmlMore->Append(ZION_HTMLMENU_SUP, "<sup>\tCtrl+Shift+S", "superscript");
    htmlMore->Append(ZION_HTMLMENU_SUPTH, "<sup>th</sup>");
    htmlMenu->Append(ZION_HTMLMOREMENU, "More Tags", htmlMore);
    htmlMenu->AppendSeparator();
    htmlMenu->Append(ZION_HTMLPREVIEW, "Preview in Browser\tF10", _T("Browser can be set in Settings | Global Options"));
    htmlMenu->AppendSeparator();
    htmlMenu->Append(ZION_HTML_MATCHTAG, "Match Tag\tCtrl+Shift+G");
    htmlMenu->Append(ZION_HTMLMENU_SKIPTAG, "S&kip Past Tag\tCtrl+K");
    htmlMenu->Append(ZION_HTMLMENU_SKIPBEFORETAG, "S&kip Before Tag\tCtrl+Shift+K");
    
    htmlMenu->AppendSeparator();
	htmlMenu->AppendCheckItem(ZION_HTML_AC_TOGGLE, _T("Tag Auto Compl\tCtrl+Shift+T"));
	htmlMenu->AppendCheckItem(ZION_HTML_BR_TOGGLE, _T("Auto <BR>\tCtrl+Shift+B"));
    
    cppMenu = new wxMenu;
    cppMenu->Append(ZION_CPPMENU_CURLY, "{  }\tCtrl+Shift+C", _T("Braces will surround a selection - will not indent"));
    //cppMenu->AppendSeparator();
    cppMenu->Append(ZION_CPPMENU_IF, "if(  ) { }\tCtrl+Shift+I", _T("Select a Block to place it between braces"));
    cppMenu->Append(ZION_CPPMENU_ELSEIF, "else if( ){ }", _T("A Selected block will be placed between braces"));
    cppMenu->Append(ZION_CPPMENU_ELSE, "else { }\tCtrl+Shift+E", _T("A Selected block will be placed between braces"));
    cppMenu->Append(ZION_CPPMENU_FOR, "for( ; ; ) { }\tCtrl+Shift+F", _T("Select a Block to place it between braces"));
    cppMenu->Append(ZION_CPPMENU_WHILE, "while( ){ }\tCtrl+Shift+W", _T("A Selected block will be placed between braces"));
    cppMenu->Append(ZION_CPPMENU_DOWHILE, "do { }while( );\tCtrl+Alt+W", _T("A Selected block will be placed between braces"));
    //cppMenu->AppendSeparator();
    cppMenu->Append(ZION_CPPMENU_LINECOMMENT, "Insert //", "Insert Line Comments - Block capable. Just select first");
    cppMenu->Append(ZION_CPPMENU_LINEUNCOMMENT, "Remove //", "Remove Line Comments. Select block first");
    cppMenu->Append(ZION_CPPMENU_COMMENT, "Insert /* */\tCtrl+Shift+M", "Will comment a selected block also");
    cppMenu->Append(ZION_CPPMENU_UNCOMMENT, "Remove /* */\tCtrl+Shift+U", "Select a block to remove comments from");
    cppMenu->Append(ZION_CPPMENU_T, _T("_T(\" \")\tCtrl+Shift+T"), _T("Can Enclose a Selected String"));

    cssMenu = new wxMenu;
    cssMenu->Append(ZION_CPPMENU_CURLY, "{  }\tCtrl+Shift+C", _T("Braces will surround a selection - will not indent"));
    cssMenu->Append(ZION_CPPMENU_COMMENT, "Insert /* */\tCtrl+Shift+M", "Will comment a selected block also");
    cssMenu->Append(ZION_CPPMENU_UNCOMMENT, "Remove /* */\tCtrl+Shift+U", "Select a block to remove comments from");
    
    sqlMenu = new wxMenu;
    sqlMenu->Append(ZION_SQLMENU_LINECOMMENT, "Insert --\tCtrl+Shift+M", "Insert Comments - Block capable! Just select first");
    sqlMenu->Append(ZION_SQLMENU_LINEUNCOMMENT, "Remove --", "Remove Line Comments. Select block first");

    pythonMenu = new wxMenu;
    pythonMenu->Append(ZION_PERLMENU_LINECOMMENT, "Insert #s\tCtrl+Shift+M", "Insert Comments - Block capable! Just select first");
    pythonMenu->Append(ZION_PERLMENU_LINEUNCOMMENT, "Remove #s", "Remove Line Comments. Select block first");

    batchMenu = new wxMenu;
    batchMenu->Append(ZION_BATCHMENU_LINECOMMENT, "Insert REM \tCtrl+Shift+M", "Insert Comments - Block capable! Just select first");
    batchMenu->Append(ZION_BATCHMENU_LINEUNCOMMENT, "Remove REM", "Remove Line Comments. Select block first");
    batchMenu->Append(ZION_BATCHMENU_LINECOMMENT2, "Insert ::\tCtrl+Shift+M", "Insert Comments - Block capable! Just select first");
    batchMenu->Append(ZION_BATCHMENU_LINEUNCOMMENT2, "Remove ::", "Remove Line Comments. Select block first");

    lispMenu = new wxMenu;  ///same comment style as asm
    lispMenu->Append(ZION_ASMMENU_LINECOMMENT, "Insert ; \tCtrl+Shift+M", "Insert Comments - Block capable! Just select first");
    lispMenu->Append(ZION_ASMMENU_LINEUNCOMMENT, "Remove ;", "Remove Line Comments. Select block first");

    fortranMenu = new wxMenu;
    fortranMenu->Append(ZION_FORTRANMENU_LINECOMMENT, "Insert ! \tCtrl+Shift+M", "Insert Comments - Block capable! Just select first");
    fortranMenu->Append(ZION_FORTRANMENU_LINEUNCOMMENT, "Remove !", "Remove Line Comments. Select block first");

    fortran77Menu = new wxMenu;
    fortran77Menu->Append(ZION_FORTRAN77MENU_LINECOMMENT, "Insert c \tCtrl+Shift+M", "Insert Comments - Block capable! Just select first");
    fortran77Menu->Append(ZION_FORTRAN77MENU_LINEUNCOMMENT, "Remove c", "Remove Line Comments. Select block first");

    asmMenu = new wxMenu;
    asmMenu->Append(ZION_ASMMENU_LINECOMMENT, "Insert ; \tCtrl+Shift+M", "Insert Comments - Block capable! Just select first");
    asmMenu->Append(ZION_ASMMENU_LINEUNCOMMENT, "Remove ;", "Remove Line Comments. Select block first");

    makeMenu = new wxMenu;
    makeMenu->Append(ZION_PERLMENU_LINECOMMENT, "Insert #s\tCtrl+Shift+M", "Insert Comments - Block capable! Just select first");
    makeMenu->Append(ZION_PERLMENU_LINEUNCOMMENT, "Remove #s", "Remove Line Comments. Select block first");
    makeMenu->Append(ZION_FORTRANMENU_LINECOMMENT, "Insert ! \tCtrl+Shift+M", "Insert Preprocessor symbols - Block capable! Just select first");
    makeMenu->Append(ZION_FORTRANMENU_LINEUNCOMMENT, "Remove !", "Remove Preprocessor symbols. Select block first");

    rubyMenu = new wxMenu;
    rubyMenu->Append(ZION_CPPMENU_CURLY, "{  }\tCtrl+Shift+C", _T("Braces will surround a selection - will not indent") );
    rubyMenu->Append(ZION_PERLMENU_LINECOMMENT, "Insert #s\tCtrl+Shift+M", "Insert Comments - Block capable! Just select first");
    rubyMenu->Append(ZION_PERLMENU_LINEUNCOMMENT, "Remove #s", "Remove Line Comments. Select block first");

    vbscriptMenu = new wxMenu;
    vbscriptMenu->Append(ZION_VBMENU_LINECOMMENT, "Insert 's\tCtrl+Shift+M", "Insert Comments - Block capable! Just select first");
    vbscriptMenu->Append(ZION_VBMENU_LINEUNCOMMENT, "Remove 's", "Remove Line Comments. Select block first");
    vbscriptMenu->AppendSeparator();
    vbscriptMenu->Append(ZION_VBMENU_IF, "&If  Then\tCtrl+Shift+I", _T("Select a Block to place it in the body"));
    vbscriptMenu->Append(ZION_VBMENU_ELSE, "&Else\tCtrl+Shift+E", _T("A Selected block will be placed in the body"));

    perlMenu = new wxMenu;
    perlMenu->Append(ZION_CPPMENU_CURLY, "{  }\tCtrl+Shift+C", _T("Braces will surround a selection - will not indent"));
    perlMenu->AppendSeparator();
    perlMenu->Append(ZION_CPPMENU_IF, "&if(  ) { }\tCtrl+Shift+I", _T("Select a block to place it between braces"));
    perlMenu->Append(ZION_PERLMENU_ELSIF, "&elsif( ) { }", _T("A Selected block will be placed between braces"));
    perlMenu->Append(ZION_CPPMENU_ELSE, "&else { }\tCtrl+Shift+E", _T("A Selected block will be placed between braces"));
    perlMenu->Append(ZION_CPPMENU_FOR, "&for( ; ; ) { }\tCtrl+Shift+F", _T("Select a block to place it between braces"));
    perlMenu->Append(ZION_PERLMENU_FOREACH, "foreach  { }", _T("Select a block to place it between braces"));
    perlMenu->Append(ZION_CPPMENU_WHILE, "&while( ){ }\tCtrl+Shift+W", _T("A Selected block will be placed between braces"));
    perlMenu->Append(ZION_CPPMENU_DOWHILE, "&do { }while( );\tCtrl+Alt+W", _T("A Selected block will be placed between braces"));
    perlMenu->AppendSeparator();
    perlMenu->Append(ZION_PERLMENU_LINECOMMENT, "Insert #s\tCtrl+Shift+M", "Insert Comments - Block capable! Just select first");
    perlMenu->Append(ZION_PERLMENU_LINEUNCOMMENT, "Remove #s", "Remove Line Comments. Select block first");
    perlMenu->Append(ZION_CPPMENU_T, "_T(\" \")\tCtrl+Shift+T");
}

/*! Event Handlers */

void ZionFrame::OnUIEOL( wxUpdateUIEvent & event )
{
    event.Check( m_zedit->GetEOLMode() == event.GetId() - ZION_LINE_WINDOWS );
}

void ZionFrame::OnUIMode( wxUpdateUIEvent & event )
{
    event.Check( m_zedit->m_currentLexer == event.GetId() - ZION_CONTAINER );
}

void ZionFrame::OnUIReconn( wxUpdateUIEvent & event )
{
    event.Enable( ! zserver );
}

void ZionFrame::OnUIBecomeServer( wxUpdateUIEvent & event )
{
    event.Enable( ! m_ipcOk && ! zserver );
}

void ZionFrame::OnUIUndo( wxUpdateUIEvent & event )
{
    event.Enable( m_zedit->CanUndo() );
}

void ZionFrame::OnUIRedo( wxUpdateUIEvent & event )
{
    event.Enable( m_zedit->CanRedo() );
}

void ZionFrame::OnUICut( wxUpdateUIEvent & event )
{
    event.Enable( ! ( m_zedit->GetReadOnly() || m_zedit->GetSelectedText() == wxEmptyString ) );
}

void ZionFrame::OnUICopy( wxUpdateUIEvent & event )
{
    event.Enable( ! (m_zedit->GetSelectedText() == wxEmptyString) );
}

void ZionFrame::OnUIPaste( wxUpdateUIEvent & event )
{
    event.Enable( m_zedit->CanPaste() );
}

void ZionFrame::OnUIHtmlACToggle( wxUpdateUIEvent & event )
{
    event.Check( m_zedit->m_htmlAC.active );
}

void ZionFrame::OnUIHtmlBRToggle( wxUpdateUIEvent & event )
{
    event.Check( m_zedit->m_htmlAC.brActive );
}

void ZionFrame::OnUILineToggle( wxUpdateUIEvent & event )
{
    event.Check( m_zedit->m_LNVisible);
}

void ZionFrame::OnUIFoldToggle( wxUpdateUIEvent & event )
{
    event.Check( m_zedit->m_FoldMarginVisible);
}

void ZionFrame::OnUIIndentToggle( wxUpdateUIEvent & event )
{
    event.Check( m_zedit->GetIndentationGuides() );
}

void ZionFrame::OnUIAutoCompToggle( wxUpdateUIEvent & event )
{
    event.Check( g_acActive );
}

void ZionFrame::OnUICallTipsToggle( wxUpdateUIEvent & event )
{
    event.Check( g_ctActive );
}

void ZionFrame::OnUICTACToggle( wxUpdateUIEvent & event )
{
    event.Check( g_ctActive && g_acActive );
}

void ZionFrame::OnUIAutoZoomToggle( wxUpdateUIEvent & event )
{
    event.Check( m_autoZoom );
}

void ZionFrame::OnUIAutoBookToggle( wxUpdateUIEvent & event )
{
    event.Check( m_autoBook );
}

void ZionFrame::OnUIReadOnlyToggle( wxUpdateUIEvent & event )
{
    event.Check( m_zedit->GetReadOnly() );
}

void ZionFrame::OnUIWhiteSpaceToggle( wxUpdateUIEvent & event )
{
    event.Check( m_zedit->GetViewWhiteSpace() );
}

void ZionFrame::OnUIWrapToggle( wxUpdateUIEvent & event )
{
    bool wrap = m_zedit->GetWrapMode();
    event.Check( wrap );
    togglesHorizSb->Enable( ! wrap );  ///Horiz ScrollBars will not be on in wrap mode
}

void ZionFrame::OnUIHorizSbToggle( wxUpdateUIEvent & event )
{
    event.Check( m_zedit->GetUseHorizontalScrollBar() );
}

void ZionFrame::OnUIVertSbToggle( wxUpdateUIEvent & event )
{
    event.Check( m_zedit->GetUseVerticalScrollBar() );
}

void ZionFrame::OnUIUseTabsToggle( wxUpdateUIEvent & event )
{
    event.Check( m_zedit->GetUseTabs() );
}

void ZionFrame::OnShowPriority(wxCommandEvent & WXUNUSED(event))
{
    wxMessageBox( priority, "Server Priority" );
}

void ZionFrame::OnCmdPrompt(wxCommandEvent & event )
{  
    wxString str;
    if( ! wxGetEnv("SystemRoot", &str) )  str = "c:\\windows";
    
    ///Show a RecentDirsDlg "Initial Directory"; Choice will be passed on to wxFileDialog as its "Default Dir"
    wxString startDir, prevDir = wxGetCwd(); ///Save in case SaveAs dialog canceled
    wxArrayString rdirs;
    for( int i=0; i< NUMRECENTDIRS; i++ ) {
        if( dirSubItems[i] == NULL ) break; ///We have hit the end of the used dirSubItems
        rdirs.Add( dirSubItems[i]->GetLabel() );
    }
    if( rdirs.GetCount() > 0 ) {
        StartFolderDlg stDlg( this, GetPosition(), GetSize().GetWidth(), rdirs, _T("Start Cmd Prompt at Folder") );
        if( stDlg.ShowModal() != wxID_OK ) return;
        startDir = stDlg.m_list->GetStringSelection(); ///Pick up the dir we want to start browsing from
        if( stDlg.m_list->GetSelection() != 0 )  wxSetWorkingDirectory( startDir );
    }
    ///Do it
    wxExecute(str + "\\system32\\cmd.exe");
    wxSetWorkingDirectory( prevDir );  ///Set back to our current file's dir
    ///TESTING LAST CMD to popup
    PrependToPopup( event.GetId() );
}

void ZionFrame::OnRecentFile(wxCommandEvent & event)
{
    if( m_zedit->CheckAllowSave() ) { //! Make sure saved first
        wxString str = recentFilesMenu->GetLabel( event.GetId() );

        wxString loc;
        if( DoWinLocDlg( loc ) ) {
            if( loc != wxEmptyString ) wxExecute( wxString::Format( _T("\"%s\\zion.exe\" -zposx%d %s \"%s\""), m_exeDir, GetPosition().x, loc, str ) );
           else  m_zedit->DirectOpen( str ); ///In this window
        } ///else Dialog was cancelled
    }
}

void ZionFrame::OnWindow(wxCommandEvent & event)
{
    wxString str = m_zedit->windowMenu->GetLabel( event.GetId() ); //! Get the string from the menu
    NotifyRaise( str.AfterLast(' ') );
}

void ZionFrame::OnRecentDir(wxCommandEvent & event)
{
    m_zedit->Open( recentDirsMenu->GetLabel( event.GetId() ) );
}

///Readini is called before the edit module is created, but since OnReadIni is an event handler, the edit module has done been created already
void ZionFrame::OnReadIni(wxCommandEvent & event)
{
    int savePos = m_zedit->GetCurrentPos(); ///workaround wrap bug
    bool wrapMode = false;
    if( m_zedit->GetWrapMode() != wxSTC_WRAP_NONE ) {
        m_zedit->GotoPos(0);  ///workaround wrap bug
        wrapMode = true;
    }

    switch( event.GetId() ) {
        case ZION_READINI :
            ReadIni(READHISTORY | READSTYLES | READPRINT);
            //m_zedit->LoadCallTips( *(syntaxStructs[m_zedit->m_currentLexer]) );
            if( ! wrapMode ) m_zedit->SetupLexer(m_zedit->GetCurrentLexer(), true); //! Force editor to load styles
            else /*if( m_zedit->CheckAllowSave(_T("Reloading to incorporate style changes...\n")) )*/ m_zedit->ReloadFromBuffer(m_zedit->GetCurrentLexer(), savePos, true);  /// Force editor to  reload styles
            break; 
        case ZION_READINI_PRINT :
            ReadIni( READPRINT );
            break;
        case ZION_READINI_STYLES :
            ReadIni( READSTYLES );
            if( ! wrapMode ) m_zedit->SetupLexer(m_zedit->GetCurrentLexer(), true);  //! Force editor to reload styles
            else /*if( m_zedit->CheckAllowSave(_T("Reloading to incorporate style changes...\n")) )*/ m_zedit->ReloadFromBuffer(m_zedit->GetCurrentLexer(), savePos, true);  /// Force editor to  reload styles
            break;
        case ZION_READINI_HISTORY :
            ReadIni( READHISTORY );
            //break;
    }
 //! We will  do temporary position changing in the edit module's SetupLexer f()    //int savePos = m_zedit->GetCurrentPos(); m_zedit->GotoPos(0);//m_zedit->GotoPos(savePos);
}

void ZionFrame::OnWriteIni( wxCommandEvent & WXUNUSED(event) )
{   WriteIni();
}

void ZionFrame::OnChooseColour(wxCommandEvent & WXUNUSED(event) )
{
  wxColourData data; int i;
  data.SetChooseFull(true);
  for (i = 0; i < 16; i++) {    //wxColour colour(i*16, i*16, i*16);
    data.SetCustomColour(i, customColors[i]);
  }
  wxColourDialog dialog(this, &data);
  if (dialog.ShowModal() == wxID_OK) {
        wxColourData retData = dialog.GetColourData();
        wxColour col = retData.GetColour();
        for( i=0; i < 16; i++) {
            if( retData.GetCustomColour(i) != customColors[i] ) {
                m_zedit->m_settingsDirty = true;
                customColors[i] = retData.GetCustomColour(i);
            }
        }
        wxString msg, msg2;
        msg.Printf("%02x%02x%02x", col.Red(), col.Green(), col.Blue() ); ///leading zeros two places in hex
        msg2.Printf("%d, %d, %d", col.Red(), col.Green(), col.Blue() );
        ColorResultDialog crd( this, msg, msg2 );
        crd.ShowModal();
    }
}

/*! Gateway to our Stc */
void ZionFrame::OnZionEdit(wxCommandEvent & event)
{
    if (m_zedit) m_zedit->ProcessEvent(event); //! pass the event to our Stc
}

void ZionFrame::OnZionEditPending(wxCommandEvent & event)
{
    if (m_zedit) m_zedit->AddPendingEvent(event); //! pass the event to our Stc
}

void ZionFrame::OnExit( wxCommandEvent & WXUNUSED(event) )  //! My event handler
{
	wxCloseEvent anEvent;
	anEvent.SetCanVeto(TRUE);
	OnClose(anEvent);
}

void ZionFrame::OnClose(wxCloseEvent & event)
{
    wxString msg2 = "Settings have changed!\nSave settings now?";
    wxString msg3 = "Error saving zion.ini";
    //! Save Settings
    bool settingsSaved = false; ///This will warn us to slow down on IPC activity
   if( m_zedit->m_settingsDirty )
	{	wxMessageDialog settingsWarn
		(this, msg2, "Settings Changed", wxICON_QUESTION|wxYES_NO);
		if( settingsWarn.ShowModal() == wxID_YES) {
            settingsSaved = true;
            if( ! WriteIni() )  ///This will spur a LOT of IPC activity
                wxMessageDialog errDlg( this, msg3, "Failed saving zion.ini", wxICON_HAND | wxOK );
       } 
	}
    if( ! settingsSaved ) { ///Try to cut down on IPC
        msg3 = m_zedit->m_currentDir;   //! Reuse!
        msg2 = msg3 + "\\" + m_zedit->GetCurrentFilename(); //! Windows Specific
        if(zserver) { PushDir(msg3, true);     //! Make sure our last location and file are on top ///Danger of lockup due to timing issues on broadcast
            PushFile( msg2, true ); ///The server will advise a READ_HISTORY eventually
        }
    }
    //! Save document; destroy
	 if( !m_zedit->GetReadOnly() &&
            ( m_zedit->GetModify() || m_zedit->m_dirty || m_zedit->GetCurrentFilename() == _T("Untitled") ) ///No readonly does turn off the modified flag!!
            && event.CanVeto() ) {
        if( ! m_zedit->CheckAllowSave(_T("Shutting this window down...\n")) )  return;
	 } //! If we can't veto, we just have to destroy even if modified
    
    ///Client requesting last push of last file opened
//    if( !m_extClose && !zserver && m_ipcOk && !m_iniDiff ) {
//        zconnection->Request( _T("Close:") + msg2 ); ///No sense checking for a return value.
//        wxMilliSleep(10);
//    }
    /** Assign priorities **/ //!(again to be sure)
    if(zserver) {
        wxCommandEvent anEvent;  anEvent.SetId(ZION_ASSIGNPRIORITIES);
        anEvent.SetEventType(wxEVT_COMMAND_MENU_SELECTED);
        ProcessEvent(anEvent);  //! We want to wait till the event is processed
        wxMilliSleep(10);
    }
    if(zserver) delete zserver;
    zserver = NULL;
    if(zclient) delete zclient;
    zclient = NULL;

    /// delete global print data and setup
    if (g_printData) delete g_printData;
    if (g_pageSetupData) delete g_pageSetupData;
    
    Destroy();
    event.Skip();  ///This lets Something higher up do the final closeout and properly notify windows on termination
}
 
void ZionFrame::OnEditIni(wxCommandEvent & event)
{
    if( m_zedit->CheckAllowSave() ) { //! Make sure saved first
        wxString str;
        switch( event.GetId() ) {
            case ZION_EDITAPI:  str = g_pathToApi + _T("\\") + syntaxStructs[m_zedit->m_currentLexer]->callTipFile; break;
            case ZION_EDITINI:  str =  m_iniDir + _T("\\zion.ini"); break;
            case ZION_EDIT_HISTORY_INI:  str =  m_iniDir + _T("\\history.ini"); break;
        }
        m_zedit->DirectOpen( str );
    }
}

void ZionFrame::OnFullScrnToggle( wxCommandEvent & event ) {
    if( !IsFullScreen() ) {
        SetWindowStyleFlag( GetWindowStyleFlag() | wxMAXIMIZE );
        ShowFullScreen( true, wxFULLSCREEN_NOMENUBAR | wxFULLSCREEN_NOSTATUSBAR );
        SetWindowStyleFlag( ( GetWindowStyleFlag() | wxBORDER_NONE ) & ~( wxMINIMIZE_BOX | wxMAXIMIZE_BOX |
                wxRESIZE_BORDER | wxSYSTEM_MENU | wxCAPTION | wxCLOSE_BOX ) );  ///Turn off these bits
    }
    else {
        ShowFullScreen( false, wxFULLSCREEN_NOMENUBAR | wxFULLSCREEN_NOSTATUSBAR );
        SetWindowStyleFlag( (GetWindowStyleFlag() | wxDEFAULT_FRAME_STYLE) & ~wxMAXIMIZE );
    }
}

///Should not call in event handlers that can be called from other event handlers.
///Do not call for CheckItem (Toggle) event handlers- will lock up!
void ZionFrame::PrependToPopup( int id ) {  ///TODO - Should we move all this to the edit module?
    if( m_recentPopup != NULL ) { ///Clean first
        m_zedit->m_popup->Remove( m_recentPopup );
        m_recentPopup = NULL;  ///Make absolutely sure. Menus will cause a crash in a heartbeat!
    }
    ///Add it while saving the new item created - well I hope it's new.
    wxMenuItem * mItem = NULL;
    mItem = m_zedit->m_popup->FindItem(id);
    if( mItem != NULL ) {
        m_recentPopup = m_zedit->m_popup->Prepend( mItem );
    }
    ///Diag
    else SetStatusText(_T("Last Item not in Popup!") );
}

void ZionFrame::OnAbout( wxCommandEvent & event )
{
    wxString str, compilerTimeStamp = __TIMESTAMP__ /*, compilerVersion = __DMC_VERSION_STRING__*/ ;
    str.Printf(_T("Rohan Allison's Zion Editor\nVersion: %s\nCompiled: %s\n\
http://www.zionedit.org\nCoded with, you guessed it: ZionEdit!\n\nCreated with the world class\n\
WxWidgets Toolkit (2.8.7) - Thanks Julian!\nwww.wxwidgets.org\n\nBased on the awesome \
Scintilla Edit Component.\nThanks to Total Control Software\n\
for providing the wonderful wxSTC class\n\n\
Compiled with the fast and tight, Industry Standard \nMicroSoft Visual Studio 2008\n\nSpecial thanks to Otto Wyss for his\n\
outstanding wxScintilla test application\nwithout which this application would\n\
not have gotten off the ground\n\n\"I am the Alpha and the Omega,\"\nsays the Lord God, \"who is, and who was,\n\
and who is to come, the Almighty.\"\n\
    - Rev 1:8\n"),
            version, compilerTimeStamp /* compilerVersion */);
    wxMessageDialog aboutDialog(this, str, _T("About ZionEdit"), wxOK );
	aboutDialog.ShowModal();
}
