// AUTHOR: Rohan Allison
// COPYRIGHT: 2007
//  License: Artistic 
/*! Implementation of our scintilla based edit class */
#include <wx/wx.h>
#include <wx/filename.h>
#include <wx/filefn.h>
#include <wx/ffile.h>
#include <wx/tokenzr.h>
#include <wx/gdicmn.h>
#include <wx/msgdlg.h>
#include <wx/spinctrl.h>
#include <wx/statline.h>
#include <wx/notebook.h>
#include <wx/filepicker.h>
// #include <wx/fdrepdlg.h>
#include <wx/ipc.h>
#include <wx/snglinst.h>
#include <wx/tokenzr.h>
#include <wx/fontenum.h>
#include <wx/textfile.h>
#include <wx/print.h>
#include <wx/process.h>
#include <wx/dynarray.h>
#include <wx/dir.h>
#include <wx/progdlg.h> ///progress dialog
#include <wx/html/htmlwin.h>

#include <stc.h>
//! Local Includes
#include "globaldefs.h"
#include "syntax.h"
#include "FindReplDialog.h"
#include "ZionDialogs.h"
#include "ModeOptsPanel.h"
#include "SettingsDialog.h"
#include "zion.h" //! for knowledge of class ZionFrame
#include "zionedit.h"  //! This module's header last

///Track Current Call Tips
WX_DECLARE_OBJARRAY( ActiveCallTip, ArrayOfCallTips );
#include <wx/arrimpl.cpp> // this is a magic incantation which must be done to get WxObject Arrays working!
WX_DEFINE_OBJARRAY(ArrayOfCallTips);
ArrayOfCallTips activeCallTips;

/*! Globals and Statics*/
///CallTip Structures
CallTipsHash g_callTips;
Str2StrHash g_subs;  ///Shortcuts, substitutions
//Str2StrHash g_memDescrip;  ///Obj.mem=>Description
MemberLists g_objMems;
MemberLists g_cssAttrs;
MemberLists g_scopeMems;
MemberLists g_htmlTags;  /// tags -> Attribs
MemberLists g_tagsList;  ///tags with case as entered - for AC, not lookup
MemberLists g_hardHash;  ///hard wired Memberlist - not read from a file
MemberLists g_htmlAttrs; /// Attribs -> Values
MemberLists g_jsStyleVals;  ///JavaScript style values
//MemberLists * g_memberLists[] = { &g_objMems, &g_cssAttrs, &g_scopeMems, &g_htmlTags, &g_htmlAttrs, &g_jsStyleVals };
//CallAC g_callAC; ///TODO change to s_callAC since it is not globally accessible

static const wxChar* htmlColors[] = {
"aliceblue","antiquewhite","aqua","aquamarine","azure","beige","bisque","black",
"blanchedalmond","blue","blueviolet","brown","burlywood","cadetblue","chartreuse",
"chocolate","coral","cornflowerblue","cornsilk","crimson","cyan","darkblue","darkcyan",
"darkgoldenrod","darkgray","darkgrey","darkgreen","darkkhaki","darkmagenta","darkolivegreen",
"darkorange","darkorchid","darkred","darksalmon","darkseagreen","darkslateblue",
"darkslategray","darkslategrey","darkturquoise","darkviolet","deeppink","deepskyblue",
"dimgray","dimgrey","dodgerblue","firebrick","floralwhite","forestgreen","fuchsia","gainsboro",
"ghostwhite","gold","goldenrod","gray","grey","green","greenyellow","honeydew","hotpink",
"indianred","indigo","ivory","khaki","lavender","lavenderblush","lawngreen","lemonchiffon",
"lightblue","lightcoral","lightcyan","lightgoldenrodyellow","lightgray","lightgrey","lightgreen",
"lightpink","lightsalmon","lightseagreen","lightskyblue","lightslategray","lightslategrey",
"lightsteelblue","lightyellow","lime","limegreen","linen","magenta","maroon","mediumaquamarine",
"mediumblue","mediumorchid","mediumpurple","mediumseagreen","mediumslateblue",
"mediumspringgreen","mediumturquoise","mediumvioletred","midnightblue","mintcream",
"mistyrose","moccasin","navajowhite","navy","oldlace","olive","olivedrab","orange","orangered",
"orchid","palegoldenrod","palegreen","paleturquoise","palevioletred","papayawhip","peachpuff",
"peru","pink","plum","powderblue","purple","red","rosybrown","royalblue","saddlebrown",
"salmon","sandybrown","seagreen","seashell","sienna","silver","skyblue","slateblue","slategray",
"slategrey","snow","springgreen","steelblue","tan","teal","thistle","tomato","turquoise","violet",
"wheat","white","whitesmoke","yellow","yellowgreen"
};
static const wxChar* bools[] = {"true", "false"};

static wxString apiACList; ///AutoCompl list from API files
static wxString apiKWList; ///Keyword list from API files - must be lowercase - ///TODO verfiy
static wxString apiTagList; ///Tags list from API files
static wxString acList;
static wxArrayString acArrayList;

wxColour StringToColor( const wxString & str ) /*! Global */
{
    long color, tempLong; unsigned char red, green, blue; wxColour retColor;
    str.ToLong(&color, 16);
    tempLong = color & 0xFF0000L; tempLong =  tempLong >> 16; red = (unsigned char) tempLong;
    tempLong = color & 0x00FF00L; tempLong =  tempLong >> 8; green = (unsigned char) tempLong;
    tempLong = color & 0x0000FFL; blue = (unsigned char) tempLong;
    retColor.Set( red, green, blue);
    return retColor;
}

///Good Sort Algorithm, wxArrayString's sort works good, so used it.
//static  int MySort( const wxString & first, const wxString & second )
//{
//    char c, d;
//    int l = first.Length();  int l2 = second.Length();
//    for( int i=0; i < l; i++ ) {
//        if( i == l2 ) return 1;  /// The second string has run out so the first is greater
//        if( (c=first.GetChar(i)) == (d=second.GetChar(i)) ) continue;  ///letter match
//        if( c > 0x60 && c < 0x7B )  c = c & 0xDF; ///Textual Comp by normalizing to lowercase
//        if( d > 0x60 && d < 0x7B )  d = d & 0xDF;
//        if( c > d )  return 1; 
//        else return -1;
//    }
//    ///All letters have matched so far
//    if( l == l2 )  return 0;  ///Strings exactly equal
//    else return -1; ///Second string is longer since first ran out
//    //return first.CmpNoCase( second );
//}

BEGIN_EVENT_TABLE (ZionEdit, wxStyledTextCtrl)
    EVT_CONTEXT_MENU ( ZionEdit::OnPopup )
    EVT_MENU ( ZION_FINDUPDATE,  	ZionEdit::OnFindUpdate)
    EVT_MENU ( ZION_REPLUPDATE,  	ZionEdit::OnReplUpdate)
    EVT_MENU ( ZION_OPEN,  	ZionEdit::OnOpen)
    EVT_MENU ( ZION_OPENSELECTED,  	ZionEdit::OnOpenSelected )
    EVT_MENU ( ZION_RELOAD,  	ZionEdit::OnReload)
    EVT_MENU ( ZION_SAVE,  	ZionEdit::OnSave)
    EVT_MENU ( ZION_SAVE_AS, ZionEdit::OnSaveAs)
    EVT_MENU ( ZION_CUT,  	ZionEdit::OnCut)
    EVT_MENU ( ZION_COPY,  	ZionEdit::OnCopy)
    EVT_MENU ( ZION_PASTE,  	ZionEdit::OnPaste)
    EVT_MENU ( ZION_SELECTRECT,  	ZionEdit::OnSelectionToggle)
    EVT_MENU ( ZION_SELECTALL,  	ZionEdit::OnSelectAll)
    EVT_MENU ( ZION_AUTOCOMPSHOW,  ZionEdit::OnAutoCompleteShow)
    EVT_MENU ( ZION_DUMPCALLTIPS,  ZionEdit::OnDumpCallTips )
    EVT_MENU ( ZION_CONTEXTCALLTIP,  ZionEdit::OnContextCallTip )
    EVT_MENU ( ZION_ASSOCTYPE,  ZionEdit::OnAssocToType )
    EVT_MENU ( ZION_REMOVE1BLANK,  ZionEdit::OnRemoveBlankLines)
    EVT_MENU ( ZION_REMOVEALLBLANKS,  ZionEdit::OnRemoveBlankLines)
    EVT_MENU ( ZION_REMOVEGT2BLANKS,  ZionEdit::OnRemoveBlankLines)
    
    EVT_MENU (ZION_LINE_TOGGLE, ZionEdit::OnLineNumToggle)
    EVT_MENU (ZION_FOLD_TOGGLE, ZionEdit::OnFoldToggle)
    EVT_MENU (ZION_WRAP_TOGGLE, ZionEdit::OnWrapToggle)
    EVT_MENU (ZION_INDENT_TOGGLE, ZionEdit::OnIndentToggle)
    EVT_MENU (ZION_USETABS_TOGGLE, ZionEdit::OnUseTabsToggle)
    EVT_MENU (ZION_WHITESPACE_TOGGLE, ZionEdit::OnWhiteSpaceToggle)
    EVT_MENU (ZION_HORIZSB_TOGGLE, ZionEdit::OnHorizSbToggle)
    EVT_MENU (ZION_VERTSB_TOGGLE, ZionEdit::OnVertSbToggle)
    EVT_MENU (ZION_READONLY_TOGGLE, ZionEdit::OnReadOnlyToggle)
    EVT_MENU (ZION_UNDO, ZionEdit::OnUndo)
    EVT_MENU (ZION_REDO, ZionEdit::OnRedo)
    EVT_MENU (ZION_LINE_WINDOWS, ZionEdit::OnConvertEOL)
    EVT_MENU (ZION_LINE_UNIX, ZionEdit::OnConvertEOL)
    EVT_MENU (ZION_LINE_MAC, ZionEdit::OnConvertEOL)
    EVT_MENU (ZION_DUPLICATELINE, ZionEdit::OnDuplicateLine)
    EVT_MENU (ZION_COPYLINE, ZionEdit::OnCopyLine)
    EVT_MENU (ZION_CUTLINE, ZionEdit::OnCutLine)
    EVT_MENU (ZION_SELECTLINE, ZionEdit::OnSelectLine)
    EVT_MENU (ZION_SELECTWORD, ZionEdit::OnSelectWord)
    EVT_MENU (ZION_TRANSPOSELINE, ZionEdit::OnTransposeLine)
    EVT_MENU (ZION_INCINDENT, ZionEdit::OnIncRedIndent)
    EVT_MENU (ZION_REDINDENT, ZionEdit::OnIncRedIndent)
    EVT_MENU (ZION_UPPERCASE, ZionEdit::OnUpperLowerCase)
    EVT_MENU (ZION_LOWERCASE, ZionEdit::OnUpperLowerCase)
    EVT_MENU (ZION_ZOOMIN, ZionEdit::OnZoomIn)
    EVT_MENU (ZION_ZOOMOUT, ZionEdit::OnZoomOut)
    EVT_MENU (ZION_RESETZOOM, ZionEdit::OnResetZoom)
    EVT_MENU (ZION_GOTOSTART, ZionEdit::OnGotoStart)
    EVT_MENU (ZION_GOTOLINE, ZionEdit::OnGotoLine)
    EVT_MENU (ZION_GOTOBRACE, ZionEdit::OnGotoBraceMatch)
    EVT_MENU (ZION_GOTOEND, ZionEdit::OnGotoEnd)
    EVT_MENU (ZION_SELTOBRACE, ZionEdit::OnSelToMatch)
    EVT_MENU (ZION_SETTINGS, ZionEdit::OnSettings)
   
    EVT_MENU (ZION_NEWHTML_TEMPLSTRICT, ZionEdit::OnNewHtmlTempl)
    EVT_MENU (ZION_NEWHTML_TEMPLTRANS, ZionEdit::OnNewHtmlTempl)
    EVT_MENU (ZION_NEWHTML_TEMPLBASIC, ZionEdit::OnNewHtmlTempl)
    EVT_MENU (ZION_NEWHTML_TEMPLBASICNH, ZionEdit::OnNewHtmlTempl)
    EVT_MENU (ZION_NEWHTML_TEMPLASP, ZionEdit::OnNewHtmlTempl)
    EVT_MENU (ZION_FILECLOSE, ZionEdit::OnFileClose)
    EVT_MENU (ZION_NEWNULL, ZionEdit::OnNew)
    EVT_MENU (ZION_NEWPYTHON, ZionEdit::OnNew)
    EVT_MENU (ZION_NEWCPP, ZionEdit::OnNew)
    EVT_MENU (ZION_NEWHTML, ZionEdit::OnNew)
    EVT_MENU (ZION_NEWPERL, ZionEdit::OnNew)
    EVT_MENU (ZION_NEWJAVA, ZionEdit::OnNew)
    EVT_MENU (ZION_NEWCS, ZionEdit::OnNew)
    EVT_MENU (ZION_NEWMAKE, ZionEdit::OnNew)
    EVT_MENU (ZION_NEWBATCH, ZionEdit::OnNew)
    EVT_MENU (ZION_NEWLISP, ZionEdit::OnNew)
    EVT_MENU (ZION_NEWRUBY, ZionEdit::OnNew)
    EVT_MENU (ZION_NEWVBSCRIPT, ZionEdit::OnNew)
    EVT_MENU (ZION_NEWCSS, ZionEdit::OnNew)
    EVT_MENU (ZION_NEWSQL, ZionEdit::OnNew)
    EVT_MENU (ZION_NEWASM, ZionEdit::OnNew)
    EVT_MENU (ZION_NEWFORTRAN, ZionEdit::OnNew)
    EVT_MENU (ZION_NEWF77, ZionEdit::OnNew)

    EVT_MENU (ZION_NULL, ZionEdit::OnChooseLexer)
    EVT_MENU (ZION_PYTHON, ZionEdit::OnChooseLexer)
    EVT_MENU (ZION_CPP, ZionEdit::OnChooseLexer)
    EVT_MENU (ZION_HTML, ZionEdit::OnChooseLexer)
    EVT_MENU (ZION_XML, ZionEdit::OnChooseLexer)
    EVT_MENU (ZION_PERL, ZionEdit::OnChooseLexer)
    EVT_MENU (ZION_MAKE, ZionEdit::OnChooseLexer)
    EVT_MENU (ZION_BATCH, ZionEdit::OnChooseLexer)
    EVT_MENU (ZION_JAVA, ZionEdit::OnChooseLexer)
    EVT_MENU (ZION_CS, ZionEdit::OnChooseLexer)
    EVT_MENU (ZION_LISP, ZionEdit::OnChooseLexer)
    EVT_MENU (ZION_RUBY, ZionEdit::OnChooseLexer)
    EVT_MENU (ZION_VBSCRIPT, ZionEdit::OnChooseLexer)
    EVT_MENU (ZION_CSS, ZionEdit::OnChooseLexer)
    EVT_MENU (ZION_SQL, ZionEdit::OnChooseLexer)
    EVT_MENU (ZION_ASM, ZionEdit::OnChooseLexer)
    EVT_MENU (ZION_FORTRAN, ZionEdit::OnChooseLexer)
    EVT_MENU (ZION_F77, ZionEdit::OnChooseLexer)
    EVT_MENU (ZION_STYLE, ZionEdit::OnChangeStyle)   //StyleAtCaret//Change to Sytle at cursor
    EVT_MENU (ZION_HTMLMENU_BOLD, ZionEdit::OnHtmlInsertTag)
    EVT_MENU (ZION_HTMLMENU_STRONG, ZionEdit::OnHtmlInsertTag)
    EVT_MENU (ZION_HTMLMENU_ITALIC, ZionEdit::OnHtmlInsertTag)
    EVT_MENU (ZION_HTMLMENU_UNDERLINE, ZionEdit::OnHtmlInsertTag)
    EVT_MENU (ZION_HTMLMENU_PARAGRAPH, ZionEdit::OnHtmlInsertTag)
    EVT_MENU (ZION_HTMLMENU_EM, ZionEdit::OnHtmlInsertTag)
    EVT_MENU (ZION_HTMLMENU_ANCHOR, ZionEdit::OnHtmlAnchor)
    EVT_MENU (ZION_HTMLMENU_COMMENT, ZionEdit::OnHtmlComment)
    EVT_MENU (ZION_HTMLMENU_UNCOMMENT, ZionEdit::OnRemoveHtmlComment)
    EVT_MENU (ZION_HTMLMENU_QUICKTAG, ZionEdit::OnHtmlQuickTag)
    EVT_MENU (ZION_HTMLMENU_NBSP, ZionEdit::OnHtmlNBSP)
    EVT_MENU (ZION_HTMLMENU_NDASH, ZionEdit::OnHtmlNDASH)
    EVT_MENU (ZION_HTMLMENU_QUOTE, ZionEdit::OnHtmlQuote)
    EVT_MENU (ZION_HTMLMENU_BR, ZionEdit::OnHtmlBR)
    EVT_MENU (ZION_HTMLMENU_SUPTH, ZionEdit::OnHtmlSupTH)
    EVT_MENU (ZION_HTML_PHP, ZionEdit::OnHtmlPhp)
    EVT_MENU (ZION_HTML_ASP, ZionEdit::OnHtmlAsp)
    EVT_MENU (ZION_HTML_ASPDRTV, ZionEdit::OnHtmlAspDrtv)
    EVT_MENU (ZION_HTML_SCRIPT, ZionEdit::OnHtmlInsertTag)
    EVT_MENU (ZION_HTML_ASPNET, ZionEdit::OnHtmlAspNet)
    EVT_MENU (ZION_HTML_TEMPLASP, ZionEdit::OnHtmlTemplAsp)
    EVT_MENU (ZION_HTML_TEMPLBASIC, ZionEdit::OnHtmlTemplBasic)
    EVT_MENU (ZION_HTML_TEMPLBASICNOHEAD, ZionEdit::OnHtmlTemplBasicNoHead)
    EVT_MENU (ZION_HTML_TEMPLSTRICT, ZionEdit::OnHtmlTemplStrict)
    EVT_MENU (ZION_HTML_TEMPLTRANS, ZionEdit::OnHtmlTemplTrans)
    EVT_MENU (ZION_HTMLMENU_SUP, ZionEdit::OnHtmlInsertTag)
    EVT_MENU (ZION_HTMLMENU_CENTER, ZionEdit::OnHtmlInsertTag)
    EVT_MENU (ZION_HTMLMENU_SPAN, ZionEdit::OnHtmlInsertTag)
    EVT_MENU (ZION_HTMLMENU_FONT, ZionEdit::OnHtmlInsertTag)
    EVT_MENU (ZION_HTMLMENU_CODE, ZionEdit::OnHtmlInsertTag)
    EVT_MENU (ZION_HTMLMENU_BLOCKQUOTE, ZionEdit::OnHtmlInsertTag)
    EVT_MENU (ZION_HTMLMENU_FORM, ZionEdit::OnHtmlInsertTag)
    EVT_MENU (ZION_HTMLMENU_PRE, ZionEdit::OnHtmlInsertTag)
    EVT_MENU (ZION_HTML_MATCHTAG, ZionEdit::OnHtmlMatchTag)
    EVT_MENU (ZION_HTMLMENU_SKIPTAG, ZionEdit::OnHtmlSkipOverTag)
    EVT_MENU (ZION_HTMLMENU_SKIPBEFORETAG, ZionEdit::OnHtmlSkipBeforeTag)
    EVT_MENU (ZION_HTMLMENU_H3, ZionEdit::OnHtmlInsertTag)
    EVT_MENU (ZION_HTMLMENU_H1, ZionEdit::OnHtmlInsertTag)
    EVT_MENU (ZION_HTMLMENU_H2, ZionEdit::OnHtmlInsertTag)
    EVT_MENU (ZION_HTMLMENU_H4, ZionEdit::OnHtmlInsertTag)
    EVT_MENU (ZION_HTMLMENU_UL, ZionEdit::OnHtmlInsertTag)
    EVT_MENU (ZION_HTMLMENU_OL, ZionEdit::OnHtmlInsertTag)
    EVT_MENU (ZION_HTMLMENU_LI, ZionEdit::OnHtmlInsertTag)
    EVT_MENU (ZION_HTMLMENU_TABLE, ZionEdit::OnHtmlInsertTag)
    EVT_MENU (ZION_HTMLMENU_TD, ZionEdit::OnHtmlInsertTag)
    EVT_MENU (ZION_HTMLMENU_TR, ZionEdit::OnHtmlInsertTag)
    EVT_MENU (ZION_HTMLMENU_TH, ZionEdit::OnHtmlInsertTag)
    EVT_MENU (ZION_CPPMENU_CURLY, ZionEdit::OnCppCurly)
    EVT_MENU (ZION_CPPMENU_FOR, ZionEdit::OnCppForIf)
    EVT_MENU (ZION_CPPMENU_WHILE, ZionEdit::OnCppForIf)
    EVT_MENU (ZION_CPPMENU_IF, ZionEdit::OnCppForIf)
    EVT_MENU (ZION_CPPMENU_ELSE, ZionEdit::OnCppForIf)
    EVT_MENU (ZION_VBMENU_IF, ZionEdit::OnCppForIf)
    EVT_MENU (ZION_VBMENU_ELSE, ZionEdit::OnCppForIf)
    EVT_MENU (ZION_CPPMENU_ELSEIF, ZionEdit::OnCppForIf)
    EVT_MENU (ZION_CPPMENU_DOWHILE, ZionEdit::OnCppForIf)
    EVT_MENU (ZION_PERLMENU_FOREACH, ZionEdit::OnCppForIf)
    EVT_MENU (ZION_PERLMENU_ELSIF, ZionEdit::OnCppForIf)
    EVT_MENU (ZION_CPPMENU_COMMENT, ZionEdit::OnCppComment)
    EVT_MENU (ZION_CPPMENU_UNCOMMENT, ZionEdit::OnRemoveBlockComments)
    EVT_MENU (ZION_CPPMENU_LINEUNCOMMENT, ZionEdit::OnLineUnComment)
    EVT_MENU (ZION_CPPMENU_LINECOMMENT, ZionEdit::OnLineComment)
    EVT_MENU (ZION_CPPMENU_T, ZionEdit::OnCppT)
    EVT_MENU (ZION_VBMENU_LINEUNCOMMENT, ZionEdit::OnLineUnComment)
    EVT_MENU (ZION_VBMENU_LINECOMMENT, ZionEdit::OnLineComment)
    EVT_MENU (ZION_PERLMENU_LINEUNCOMMENT, ZionEdit::OnLineUnComment)
    EVT_MENU (ZION_PERLMENU_LINECOMMENT, ZionEdit::OnLineComment)
    EVT_MENU (ZION_SQLMENU_LINEUNCOMMENT, ZionEdit::OnLineUnComment)
    EVT_MENU (ZION_SQLMENU_LINECOMMENT, ZionEdit::OnLineComment)
    EVT_MENU (ZION_ASMMENU_LINEUNCOMMENT, ZionEdit::OnLineUnComment)
    EVT_MENU (ZION_ASMMENU_LINECOMMENT, ZionEdit::OnLineComment)
    EVT_MENU (ZION_BATCHMENU_LINEUNCOMMENT, ZionEdit::OnLineUnComment)
    EVT_MENU (ZION_BATCHMENU_LINECOMMENT, ZionEdit::OnLineComment)
    EVT_MENU (ZION_BATCHMENU_LINEUNCOMMENT2, ZionEdit::OnLineUnComment)
    EVT_MENU (ZION_BATCHMENU_LINECOMMENT2, ZionEdit::OnLineComment)
    EVT_MENU (ZION_FORTRANMENU_LINEUNCOMMENT, ZionEdit::OnLineUnComment)
    EVT_MENU (ZION_FORTRANMENU_LINECOMMENT, ZionEdit::OnLineComment)
    EVT_MENU (ZION_FORTRAN77MENU_LINEUNCOMMENT, ZionEdit::OnLineUnComment)
    EVT_MENU (ZION_FORTRAN77MENU_LINECOMMENT, ZionEdit::OnLineComment)
    
    EVT_MENU (ZION_FIND, ZionEdit::OnFind)
    EVT_MENU (ZION_FINDALL, ZionEdit::OnFindAll)
    EVT_TEXT( ZION_RTFIND, ZionEdit::OnRTFindAll )
    EVT_MENU (ZION_FINDALLFILES, ZionEdit::OnFindAllFiles)
    EVT_MENU (ZION_FINDNEXT, ZionEdit::OnFindNext)
    EVT_MENU (ZION_FINDPREV, ZionEdit::OnFindPrev)
    EVT_MENU (ZION_FINDREPLACE, ZionEdit::OnFindReplace)  //!Show the Find/Replace dialog
    EVT_MENU (ZION_FR_CLEAR_FIND, ZionEdit::OnClearFind)
    EVT_MENU (ZION_FR_CLEAR_REPL, ZionEdit::OnClearRepl)  
    
    EVT_FIND (wxID_ANY, ZionEdit::OnDlgFind)  //! Sent by Find/Replace dialog
    EVT_FIND_NEXT (wxID_ANY, ZionEdit::OnDlgFindNext)
    EVT_FIND_REPLACE(wxID_ANY, ZionEdit::OnDlgReplace)  //! Sent by Find/Replace dialog
    EVT_FIND_REPLACE_ALL (wxID_ANY, ZionEdit::OnDlgReplaceAll) //! Sent by Find/Replace dialog
    EVT_FIND_CLOSE(wxID_ANY, ZionEdit::OnDlgFindClose)  //! Sent by Find/Replace dialog
    
    EVT_MENU ( ZION_ADDBOOKMARK, 	ZionEdit::OnAddBookmark )
    EVT_MENU ( ZION_CLEARBOOKMARKS, 	ZionEdit::OnClearBookmarks )
      
    EVT_COMMAND_RANGE( ZION_BOOK_1, ZION_BOOK_1 + NUMBOOKMARKS -1,
            wxEVT_COMMAND_MENU_SELECTED, ZionEdit::OnBookmark )
    
    EVT_STC_MARGINCLICK (-1,  ZionEdit::OnMarginClick)  //why -1 ? = wxID_ANY = there is only one event of this type so we dont care what the id is
    EVT_STC_CHARADDED (-1,  ZionEdit::OnCharAdded)
    EVT_STC_CALLTIP_CLICK(-1, ZionEdit::OnCallTipClicked)
    EVT_STC_USERLISTSELECTION(-1, ZionEdit::OnAutoCompSelChange)
    //EVT_STC_DWELLSTART ( -1,  ZionEdit::OnDumpCallTips )
    EVT_STC_DONELOADING (-1, ZionEdit::OnDoneLoading)
    EVT_STC_SAVEPOINTLEFT (-1,  ZionEdit::OnModified)
    EVT_STC_POSCHANGED (-1, ZionEdit::OnPosChanged)
    EVT_STC_KEY (-1, ZionEdit::OnKey)
    EVT_STC_AUTOCOMP_SELECTION(-1, ZionEdit::OnAutoCompSel)
    EVT_RESULTS_CLICKED(-1, ZionEdit::OnResultsClicked)  ///(Local) Find All results clicked
    EVT_RESULTS_CLOSING(-1, ZionEdit::OnResultsClosing)
    EVT_LOCALFINDALLTWEAK(-1, ZionEdit::OnLocalFindAllTweak)
    EVT_FILESRESULTS_CLICKED(-1, ZionEdit::OnFilesResultsClicked)
    EVT_FILESRESULTS_CLOSING(-1, ZionEdit::OnFilesResultsClosing)
    EVT_FILESREPLACE(-1, ZionEdit::OnFilesReplace)
    EVT_FILESREPLACEALL(-1, ZionEdit::OnFilesReplaceAll)
    EVT_REMOTEFINDALL(-1,  ZionEdit::OnRemoteFindAll)
    
END_EVENT_TABLE()

/*! ZionEdit Constructor */
ZionEdit::ZionEdit( ZionFrame * ourFrame, wxWindow *parent, const wxString & str, bool min, wxWindowID id,
            const wxPoint &pos,
            const wxSize &size,
            long style )
    : wxStyledTextCtrl (parent, id, pos, size, style)
{   
    /*! Init our Vars */
    m_min = min;
    m_ourFrame = ourFrame; //! pointer to our parent frame
    m_currentDir = str;  //! For some reason the drive letter can be lwr case here - ( If cmd box shows lower case drive letters)
    m_dirty = false;
    m_settingsDirty = false;
    m_settled = true;
    m_LNMarginID = 0; ///IDs determine order of margins
    m_FoldMarginID = 1;
    m_FoldMarginWidth = 10;
    m_FoldMarginVisible = true;
    m_LNVisible = false;
    m_searchStartPos = 0;  /// So find wrapping will know when to stop
    m_incrementalStartPos = 0;  /// Track our last find pos
    m_findWrap = false;
    m_callTipsLoaded = false;
    m_acAttribQuote = NOQUOTE;
    m_acCurrHash = NULL;
    m_descript = NULL;  ///For display of AC descriptions
    m_ctDescript = NULL;  ///For display of CT descriptions
    //had to be put in initializer list //m_acCurrHash = g_objMems;  ///curr AC hash shown
    m_htmlAC.Init();
//    m_line = 0;
//    m_newLine = false;
    m_dlgReplace = NULL;  //! VITAL!
    m_fresDialog = NULL;  ///Find Results
    m_filesDialog = NULL; ///All Files Results
    
//    findSubMenu = new wxMenu;
    windowMenu = new wxMenu;
    bookMenu = new wxMenu;
    m_popup = new wxMenu;
    
    ///Init some vars just in case. SetPrefs() should override these. These will also prevail if we are in "min" mode
    m_dontPrintNums = true;
    
    SetCurrentFilename(_T("Untitled"));
    SetUseVerticalScrollBar( false );  ///These now handled in Lexer Setup, but we init here for the case of m_min
    SetUseHorizontalScrollBar( false );
   
    if( ! m_min ) { ///Non-vital functions bypassed for a non-displayed edit object
    
        syntaxStructs.clear();
        syntaxStructs[ wxSTC_LEX_NULL ] = &defaultSyntax;
        syntaxStructs[wxSTC_LEX_VBSCRIPT] = &vbscriptSyntax;
        syntaxStructs[ wxSTC_LEX_CPP ] = &cppSyntax;
        syntaxStructs[ wxSTC_LEX_PERL ] = &perlSyntax;
        syntaxStructs[ wxSTC_LEX_RUBY ] = &rubySyntax;
        syntaxStructs[ wxSTC_LEX_HTML ] = &htmlSyntax;
        syntaxStructs[ wxSTC_LEX_PYTHON ] = &pythonSyntax;
        syntaxStructs[ wxSTC_LEX_JAVA ] = &javaSyntax;
        syntaxStructs[ wxSTC_LEX_CS ] = &csSyntax;
        syntaxStructs[ wxSTC_LEX_SQL ] = &sqlSyntax;
        syntaxStructs[ wxSTC_LEX_CSS ] = &cssSyntax;
        syntaxStructs[ wxSTC_LEX_LISP ] = &lispSyntax;
        syntaxStructs[ wxSTC_LEX_F77 ] = &f77Syntax;
        syntaxStructs[ wxSTC_LEX_FORTRAN ] = &fortranSyntax;
        syntaxStructs[ wxSTC_LEX_MAKEFILE ] = &makeSyntax;
        syntaxStructs[ wxSTC_LEX_BATCH ] = &batchSyntax;
        syntaxStructs[ wxSTC_LEX_ASM ] = &asmSyntax;

        ContextMenuStuff();
        LoadHardHash();
        ///Load some additional syntax words
        fortranSyntax.words1 = F1Words;
        fortranSyntax.words2 = F2Words;
        fortranSyntax.words3 = F3Words;
        f77Syntax.words1 = F1Words;
        f77Syntax.words2 = F2Words;
        f77Syntax.words3 = F3Words;
        cssSyntax.words3 = CSSWords;
        m_currentLexer = wxSTC_LEX_CPP;
        m_startLex = m_ourFrame->m_startLex; /// Pick up from frame.
        SetupLexer( (g_extInitLexer != -1 ? g_extInitLexer : m_startLex), true); /// Force the setup for startup. The frame has the user values ready for editor (Readini)
        SetPrefs(); ///Many settings are picked up from the frame here
        ShowEOLMode();
        ///Do Initial Zoom
        long zoom;  g_initialZoom.ToLong( &zoom );
        SetZoom( (int) zoom ); ///was reinventing the wheel below:
//        if( zoom < 0 ) {
//            for( int i= zoom; i < 0; i++)
//                CmdKeyExecute(wxSTC_CMD_ZOOMOUT);
//        }
//        else { ///zoom >= 0
//            for( int i= zoom; i > 0; i-- )
//                CmdKeyExecute(wxSTC_CMD_ZOOMIN);
//        }
        //m_ourFrame->SetStatusText( wxString::Format( _T("%d"), zoom ), 3 ); ///%l does display
//        SetVisiblePolicy( wxSTC_VISIBLE_STRICT /* visiblePolicy */, 0 /* visibleSlop */ );
//        SetYCaretPolicy( wxSTC_CARET_STRICT /* caretPolicy */, 0 /* caretSlop */ );
//        SetXCaretPolicy( wxSTC_CARET_STRICT /* caretPolicy */, 0 /* caretSlop */ );
    }
}

ZionEdit::~ZionEdit() {
    activeCallTips.Clear();
}
//------------------------------------------------------------------------------------------------------------------------

wxString ZionEdit::GetIniDir() { return m_ourFrame->GetIniDir(); }

void ZionEdit::OnAssocToType( wxCommandEvent & event )
{
    m_ourFrame->PrependToPopup( event.GetId() ); ///LAST CMD to popup
    wxArrayString objs; wxString str, idfr;  const int dlgWidth = 100;
    ///Current word selection
    if( GetSelectionStart() == GetSelectionEnd() )  DoSelectWord();
    if( GetSelectionStart() == GetSelectionEnd() )  { 
        wxMessageBox(_T("Please Select a Word to associate with Object Type") );  return;
    }
    idfr = GetSelectedText();  idfr = idfr.Lower();
    if( idfr.Trim().Trim(false) == _T("") ) {  ///Trim right, left
        wxMessageBox(_T("Please Select a Word to associate with Object Type") );  return;
    }

    MemberLists::iterator it;
    for( it = g_objMems.begin(); it != g_objMems.end(); ++it ) {
        objs.Add( it->first /*+ _T(" (Type)") + it->second->m_list.Mid(0, 45)*/ );  ///keys and values
    }
    CallTipsHash::iterator it3;
    for( it3 = g_callTips.begin(); it3 != g_callTips.end(); ++it3 )  {
        if( it3->first.Contains(_T(".") ) ) {  ///Only object functions will be considered
            str = it3->first.BeforeFirst('.');
            if( objs.Index( str ) == wxNOT_FOUND )  objs.Add( str /* + _T(" (Func)")*/ );
        }
    }
    
    int x = GetPosition().x + GetSize().GetWidth() - dlgWidth - 60;  ///Right edge - dlgWidth - n
    
    wxString result = wxGetSingleChoice( wxString::Format(_T("Associate identifier  \"%s\" to known class.\n(Use \"Options | Edit API file\" for permanent associations)"),
            idfr ), _T("Associate Identifier To Type"), objs, this, x, -1, false, dlgWidth, 180 );
    if( result != wxEmptyString )  g_subs[ idfr ] = result; ///Map aliases to base function or obj
}

void ZionEdit::OnSelectWord( wxCommandEvent & WXUNUSED(event) ) {
    DoSelectWord();
}

void ZionEdit::DoSelectWord() {
    int start, end;  char c;
    CmdKeyExecute(wxSTC_CMD_WORDLEFT);
    start = GetCurrentPos();
    for( end=start +1; end < GetLineEnd(); end++ ) {
        c =  GetCharAt(end);
        if(  ! isalnum( c ) && c != '_' /* && c != '-' */ )  break; 
    } ///Either exit (GetlineEnd or non-word char, will put us in the right place
    SetSelection( start, end );
}

void ZionEdit::OnAddBookmark( wxCommandEvent & WXUNUSED(event) ) ///Click
{
    int line = GetCurrentLine();  bool found = false;

    ///If current(caret) line not visible, don't add the bookmark
    int firstLine = GetFirstVisibleLine();
    //m_ourFrame->SetStatusText( wxString::Format( _T("line: %d, firstLine: %d"), line, firstLine ), 4 );
    if( line >= firstLine && line < firstLine + LinesOnScreen() ) {
        ///If the line is already bookmarked remove it from the list
        size_t count = m_bookIdx.GetCount();
        for( size_t n=0; n < count; n++ ) {
            if( m_bookIdx[n] == line ) {
                found = true;
                m_bookIdx.RemoveAt(n);
                m_bookmarks.RemoveAt(n);
            }
        }
        
        ///If operating at maximum
        if( m_bookIdx.GetCount() == NUMBOOKMARKS ) { ///Remove oldest entry (I hope)
            m_bookIdx.RemoveAt(0);
            m_bookmarks.RemoveAt(0);
        }
        ///Do the addition
        if( m_bookIdx.GetCount() < NUMBOOKMARKS ) { ///We call the GetCount() again to be absolutely sure of the count at this point
            m_bookIdx.Add( line );
            wxString str = wxString::Format(_T("%d: %s"), line +1, GetLine(line).Mid(0, 70) );
            count = str.Len(); char c;
            for( size_t n=0; n < count; n++ ) {  ///Xlat whitespc chars to spaces
                c = str.GetChar(n);
                if( c == '\t' || c== '\r' || c == '\n' ) str.SetChar( n, ' ' );
            }
            m_bookmarks.Add( str ); ///All our line functions are zero based
            //m_ourFrame->SetStatusText(_T("Bookmark added") );
        }
        else m_ourFrame->SetStatusText(_T("Number of bookmarks exceeded\nClose this window then reopen") ); ///This should never happen
    }
}

void ZionEdit::OnClearBookmarks( wxCommandEvent & WXUNUSED(event) ) 
{
    m_bookIdx.Empty();
    m_bookmarks.Empty();
}

void ZionEdit::OnBookmark( wxCommandEvent & event )
{
    wxString str = bookMenu->GetLabel( event.GetId() ); //! Get the string from the menu
    long line = 0;
    ///Parse line of m_bookmarks
    str.BeforeFirst( ':' ).ToLong( &line );
    GotoLine( line -1 ); ///Representation: unity based, internally: zero based
    ScrollFromEdge(); ///Move position away from edge
}

void ZionEdit::OnRemoveBlockComments( wxCommandEvent & event )
{
    m_ourFrame->PrependToPopup( event.GetId() ); ///LAST CMD to popup
    ///Determine selected range - selection required
    int a=0, b=0, z= GetLength();  bool startFound = false;
    if( ProcessSel( &a, &b, false, false ) ) { ///don't reverse, don't select
        while( a < b -1 ) { ///Make room for second char
            if( !startFound && z != a + 2 /* not at end */ && GetCharAt(a) == '/' && GetCharAt(a+1) == '*' ) {
                startFound = true;
                GotoPos(a + 2);
                DeleteBackNotLine(); DeleteBackNotLine();
                b -= 2; ///adj end
            }
            if( startFound && z != a + 2 /* not at end */ && GetCharAt(a) == '*' && GetCharAt(a+1) == '/' ) {
                GotoPos(a + 2);
                DeleteBackNotLine(); DeleteBackNotLine();
                break;
            }
            ++a;
        }
    }
}

void ZionEdit::OnRemoveHtmlComment( wxCommandEvent & event )
{
    m_ourFrame->PrependToPopup( event.GetId() ); ///LAST CMD to popup
    ///Determine selected range - selection required
    int a=0, b=0, z= GetLength();  bool startFound = false;
    if( ProcessSel( &a, &b, false, false ) ) { ///don't reverse, don't select
        while( a < b -2 ) { ///Make room for 3 ending chars 
            if( !startFound && z > a + 6 /* not at end */ && GetCharAt(a) == '<' && GetCharAt(a+1) == '!' &&
                    GetCharAt(a+2) == '-' && GetCharAt(a+3) == '-' ) {
                startFound = true;
                GotoPos(a + 4);
                DeleteBackNotLine(); DeleteBackNotLine(); DeleteBackNotLine(); DeleteBackNotLine();
                b -= 4; ///adj end
            }
            if( startFound && z > a + 2 /* not at end */ && GetCharAt(a) == '-' && GetCharAt(a+1) == '-' && GetCharAt(a+2) == '>' ) {
                GotoPos(a + 3);
                DeleteBackNotLine(); DeleteBackNotLine(); DeleteBackNotLine();
                break;
            }
            ++a;
        }
    }
}

void ZionEdit::OnRemoveBlankLines( wxCommandEvent & event )
{
    m_ourFrame->PrependToPopup( event.GetId() ); ///LAST CMD to popup
//    wxMessageBox( _T("After removing blank lines, be sure the End of Line mode is what you expect.\n\
//If in doubt set your desired EOL mode in SpecialOps | Set EOL Chars"), _T("Check EOL mode") );
    bool delAllLines = false, remgt2 = false;
    ///Default is to remove 1 blank line
    if( event.GetId() == ZION_REMOVEALLBLANKS )
        delAllLines = true;  /// OPTION to remove ALL blank lines
    if( event.GetId() == ZION_REMOVEGT2BLANKS )
        remgt2 = true;
        
    char c;  int currentLine = 0;
    int lastLine=0;  int blankCount = 0;
    
    ///If there is a selection set our range based on it
    int a=0, b=0;
    if( ProcessSel( &a, &b, false, false ) ) { ///don't reverse, don't select
        currentLine = LineFromPosition( a );  ///currentLine will hold our starting position
        lastLine = LineFromPosition( b );
    }
    else { ///Range is the whole document
        if( (lastLine = GetLineCount() -1) < 1 )  return; ///At minimum there is at least 1 line
        currentLine = 0;  //GotoPos(0);
    }
    ///At this points limits are established: starting line == currentLine,  ending line == lastLine

    ///Delete One Line Algo
    bool previousBlank = false;  ///okay to delete at start
    
    bool empty; int posEnd;
    while( currentLine < lastLine /*+ 1*/ ) {
        GotoLine( currentLine );  ///sets the caret at the start of the line
        c = GetCharAt( GetCurrentPos() );  //PositionFromLine(currentLine)
        
        ///Synopsis: We are only deleting one line in a group of consecutive lines. A non-blank line is like a reset,
        ///allowing us to be able to delete the next blank line we find.
        if( c != '\n' && c != '\r' ) {  ///if Non-blank line           (fyi: logic inverted using DeMorgan's Theorem)
            ///If the line contains only whitespace, it is effectively blank
            empty = true;  posEnd = GetLineEnd();
            for( int j=GetCurrentPos(); j < posEnd; j++ ) {
                if( (c=GetCharAt(j)) != ' ' && c != '\t' ) {
                    empty = false; break;
                }
            }
            if( ! empty ) {
                previousBlank = false; blankCount=0; currentLine++; continue; ///We have a reset. We can delete the next line if it's blank- next iteration
            }
        }
        ///At this point current line is blank
        
        if( remgt2 ) {  ///Remove gt 2 lines algorithm
            if( blankCount < 2 )  {  ++blankCount; ++currentLine;  }
            else {
                LineDelete();  --lastLine;  ///blankCount will remain at its present value, so we delete till we hit the reset (a non-blank line)
            }
        }
        else if( delAllLines || ! previousBlank ) {  ///If delAllLines ignore previousBlank
            LineDelete();  --lastLine;  previousBlank = true; ///for next time let us know the previous line was blank
        }
        ///Consecutive blank lines
        else currentLine++;  ///Skip past consecutive blank lines since first one would have already been deleted
    }
}


void ZionEdit::OnKey(wxStyledTextEvent &event)
{
   //Testing // static int count; ///Should be init to zero
   if( event.GetKey() == WXK_BACK ) {  ///Backspace// '0x8' - raw code no workie
      if(m_htmlAC.inTag) {
         m_htmlAC.tag.RemoveLast();  ///Hopefully there are some checks built in here.
         //m_ourFrame->SetStatusText(wxString::Format(_T("Backspc in tag")) );
      }
   }
   if( event.GetKey() == 0x99FE ) { // '0x8' - raw code no workie
       if( m_htmlAC.inTag || m_htmlAC.tagWaiting ) { /// If there is a tag in the making or already buffered
            ///Clean up
            m_htmlAC.inTag = false;  m_htmlAC.tagWaiting = false;
            m_htmlAC.tag = wxEmptyString; ///Clean up
       }
      m_ourFrame->SetStatusText(wxString::Format(_T("Bypassing HTML AC")) );
   }
}

void ZionEdit::OnFindUpdate(wxCommandEvent & WXUNUSED(event))
{
    m_findReplData.UpdateFindString( m_ourFrame->m_advisedData ); 
     //m_ourFrame->SetStatusText(_T("Find Updated"), 2 );
}

void ZionEdit::OnReplUpdate(wxCommandEvent & WXUNUSED(event))
{
    m_findReplData.UpdateReplString( m_ourFrame->m_advisedReplData ); 
     //m_ourFrame->SetStatusText(_T("Repl Updated"), 2 );
}

void ZionEdit::OnHtmlSkipOverTag(wxCommandEvent & event) ///CTRL+K
{
    m_ourFrame->PrependToPopup( event.GetId() ); ///LAST CMD to popup
    int a = GetCurrentPos();  int length = GetLength(); //! Length of Document

    while( ++a < length ) {
        if( GetCharAt(a) == '>' ) {
            GotoPos(a+1);  break;
        }
    }
}

void ZionEdit::OnHtmlMatchTag(wxCommandEvent & event)
{
    m_ourFrame->PrependToPopup( event.GetId() ); ///LAST CMD to popup
    wxString tagToMatch, currentTag;
    char c;  bool rvsMatch = false;
    int startTagPos=0, endTagPos=0, a = GetCurrentPos(), docLength = GetLength();

    ///LOOK BACKWARDS FOR FIRST '<'
    while( GetCharAt(a--) != '<'  ) {  ///decrement after check
        if( a < 0 ) { m_ourFrame->SetStatusText(_T("Scanning backwards - could not find an opening tag ('<')")); return; }
    }
    startTagPos = ++a; 
    
    ///Start fwd skipping any spaces or tabs
    while( true ) {
        c=GetCharAt(++a);
        if( a >= docLength ) { m_ourFrame->SetStatusText(_T("Error ascertaining tag to match - EOF reached")); return; }
        if( c == ' ' || c == '\t' || c == '\n' || c=='\r' )  continue;
        break;
    }

    ///Pickup the TAG TO MATCH
    ///An ending tag will trigger a rvs search
    if( GetCharAt(a) == '/' ) {  ///we are assuming that '/' is immediately after the opening '<'
        rvsMatch = true;
        a++; ///adj to next char for tag name extraction
        ///Need to determine end of this tag for selection
        int i = a;
        while( GetCharAt(i) != '>' ) {
            if( ++i >= docLength ) { m_ourFrame->SetStatusText(_T("Error finding end of tag to match - EOF reached")); return; }
        }
        endTagPos = ++i;
    }
    ///Scan and further checks on the tag
    while( (c=GetCharAt(a)) != '>' && c != ' ' && c != '\t' ) {
        if( c == '/'  )  { m_ourFrame->SetStatusText(_T("Not matching XHTML single tag")); return; }
        if( c == '?' || c == '%' || c=='!' )  { m_ourFrame->SetStatusText(_T("Not matching script or special tags")); return; }
        if( a >= docLength ) { m_ourFrame->SetStatusText(_T("Error constructing tag to match - EOF reached")); return; }
        tagToMatch += c;
        a++;
    }
    ///Don't attempt to match single tags
    if( tagToMatch.Len() < 1 || tagToMatch == _T("br") || tagToMatch == _T("meta") || tagToMatch == _T("col") || tagToMatch == _T("hr")
         || tagToMatch == _T("frame") || tagToMatch == _T("img") || tagToMatch == _T("input") || tagToMatch == _T("link")
          || tagToMatch == _T("param") || tagToMatch == _T("wbr") )
            { m_ourFrame->SetStatusText(_T("Not matching single tag")); return; }
    
    m_ourFrame->SetStatusText( wxString::Format(_T("Mode: %s  Tag to match: \"%s\""), ( rvsMatch? "Rvs": "Fwd" ), tagToMatch ) );  ///DIAG
    
    int track = 1;  int lastTagStart=0; bool endTag = false;
    
    if( ! rvsMatch ) {
        ///SCAN FWD FOR THE MATCH ///At this point a is at a whitespace char after tag name proper or at a '>'
        while( track > 0 ) { ///BTW, a negative number is true
            ///SEARCH FWD FOR NEXT "<"
            while( GetCharAt(++a) != '<' ) {
                if( a >= docLength ) { m_ourFrame->SetStatusText(_T("No match found! EOF Reached")); return; }
            }
            ///At this point 'a' has to be pointing at a '<'
            ///SKIP ANY SPACES OR TABS
            while( true ) {
                c=GetCharAt(++a);
                if( a >= docLength ) { m_ourFrame->SetStatusText(_T("EOF Reached while scanning possible match")); return; }
                if( c == ' ' || c == '\t' || c == '\n' || c=='\r' )  continue;
                break;
            }
            /// At this point we should be at the first meaningful char in the tag
            if( GetCharAt(a) == '/' )  { endTag = true; ++a; }///skip past it 
            ///Pick up the possible match
            currentTag = wxEmptyString;
            while( (c=GetCharAt(a)) != '>' && c != ' ' && c != '\t'  ) {
                if( a >= docLength ) { m_ourFrame->SetStatusText(_T("EOF Reached while contructing possible match")); return; }
                currentTag += c;
                a++;
            }
            //m_ourFrame->SetStatusText( wxString::Format(_T("Current Tag: %c%s"), ( endTag? '/': ' ' ), currentTag ) );
            //wxMilliSleep(40);
            ///Compare
            if( currentTag == tagToMatch ) {
                if( endTag ) --track;
                else ++track;
            }
            endTag = false;
        }
    }
    else {
        ///SCAN RVS FOR THE MATCH
        lastTagStart = startTagPos;  ///init search pos
        
        while( track > 0 ) {
            if( lastTagStart <= 0 )  { m_ourFrame->SetStatusText(_T("No match found! SOF Reached")); return; }
            a = lastTagStart;  ///Continue rvs search before start position or last tag checked
            //m_ourFrame->SetStatusText(_T(" "));
            
            ///SEARCH BCKWD FOR NEXT "<"
            do {  ///decrement after check
                if( --a < 0 ) { m_ourFrame->SetStatusText(_T("No match found! SOF Reached")); return; }
            } while( GetCharAt(a) != '<'  ); 
            ///At this point 'a' has to be pointing at a '<'
            lastTagStart = a;  ///save for selection in rvs match
            ///SKIP ANY SPACES OR TABS
            while( true ) {
                c=GetCharAt(++a);
                if( a >= docLength ) { m_ourFrame->SetStatusText(_T("EOF Reached while scanning possible match")); return; }
                if( c == ' ' || c == '\t' || c == '\n' || c=='\r' )  continue;
                break;
            }
            /// At this point we should be at the first meaningful char in the tag
            if( GetCharAt(a) == '/' )  { endTag = true; ++a; } ///skip past it
            else endTag = false;
            ///Pick up the possible match
            currentTag = wxEmptyString;
            while( (c=GetCharAt(a)) != '>' && c != ' ' && c != '\t'  ) {
                if( a >= docLength ) { m_ourFrame->SetStatusText(_T("EOF Reached while contructing possible match")); return; }
                currentTag += c;
                a++;
            }
            ///Compare
            //m_ourFrame->SetStatusText( wxString::Format(_T("Current Tag: %c%s"), ( endTag? '/': ' ' ), currentTag ) );
            //wxMilliSleep(40);
            if( currentTag == tagToMatch ) {
                endTag ? ++track : --track;
            }
        }
    }
    ///MATCH FOUND AT THIS POINT
    if( ! rvsMatch ) SetSelection(startTagPos, a+1);
    else SetSelection( lastTagStart, endTagPos );
 }

void ZionEdit::OnHtmlSkipBeforeTag(wxCommandEvent & event) ///Shift+CTRL+K
{
    m_ourFrame->PrependToPopup( event.GetId() ); ///LAST CMD to popup
    int a = GetCurrentPos();

    while( --a >= 0 ) {
        if( GetCharAt(a) == '<' ) {
            GotoPos(a);  break;
        }
    }
}

wxString ZionEdit::BuildWord( int a, bool asp ) /// 'a' must be placed one position past the last alphanum ( actually identifier) char
{
    char c = GetCharAt( a-1 ); ///2 before current pos
    wxString partStr; int count=0;
    if( isalnum( c ) || c == '_' || ( asp && (c== ':' /*|| c == '%'*/ || c == '@') ) ) {  ///if not word char. Are '_' and '-' considered alphanumeric? - No!
        ///Search backwards ///Start looking for the beginning of the word
            ///Continue searching while  wordchars or '_' or '.'  ///Note: Full object name will be used as the function name
        while( a > 0 ) { ///for safety.
            c = GetCharAt( a-1 );
            if( ! ( isalnum( c ) || c == '_' || ( asp && (c== ':' || c == '%' || c == '@') ) ) )  break;  ///if not word char
            count++;  a--;
        }
        for( int j=0; j< count; j++ )  partStr += GetCharAt( a + j ); ///Build partStr in fwd direction
        return partStr;
    }
    else return wxString(_T("") );
}

bool ZionEdit::FindWord_style( int a ) /// 'a' must be placed one position past the last alphanum ( actually identifier) char
{
    //m_ourFrame->SetStatusText(_T("In FindWord_style()"), 2 );
    bool found; char c;  ///2 before current pos
    ///Scan backwrds for '='
    for( ; a > 0; --a ) {
        if( GetCharAt(a-1) == '=' ) {  found = true;  break;  }
    }
    if( found ) {  ///if not word char. Are '_' and '-' considered alphanumeric? - No!
        wxString partStr; int count=0;
        ///Search backwards ///Start looking for the beginning of the word
            ///Continue searching while  wordchars or '_' or '.'  ///Note: Full object name will be used as the function name
        --a;  ///Get off of '='
        if( GetCharAt(a-1) == ' ' ) --a;
        while( a > 0 ) { ///for safety.
            c = GetCharAt( a-1 );
            if( ! isalnum( c ) /*&& c != '_'*/  )  break;  ///if not word char (word "style" has only 
            count++;  a--;
        }
        for( int j=0; j< count; j++ )  partStr += GetCharAt( a + j ); ///Build partStr in fwd direction
        
        if( partStr.Lower() == _T("style") )
        {   return true; }
    }
    return false;
}

wxString ZionEdit::BuildWord2( int a ) /// Object Compliant
{
    char c = GetCharAt( a-1 );
    wxString partStr; int count=0;
    if( isalnum( c ) || c == '_' /*|| c == '.' */ )  {  ///if not word char. Are '_' and '-' considered alphanumeric? - No!
        ///Search backwards ///Start looking for the beginning of the word
            ///Continue searching while  wordchars or '_' or '.'  ///Note: Full object name will be used as the function name
        while( a > 1 ) { ///for safety.
            c = GetCharAt( a-1 );
            if( ! isalnum( c ) && c != '_' && c != '.' && ! ( c=='>' && GetCharAt(a -2) == '-' ) 
                    && ! ( c=='-' && GetCharAt(a) == '>' ) )  break;  ///if not word char. /// We are also looking for pointer condition "->"
            count++;  a--;
        }
        for( int j=0; j< count; j++ )  partStr += GetCharAt( a + j ); ///Build partStr in fwd direction
        return partStr;
    }
    else return wxString(_T("") );
}

ActiveCallTip * ZionEdit::CreateHiliteCT( CallTipsHash::iterator it ) {
    ActiveCallTip * act = new ActiveCallTip( it->second, GetCurrentLine() ); ///it->second = CallTip obj
    //goodCTCreated = true;
    ///On initial call tip creation/ display, we always want to start with the FIRST overload
    ///Set CallTip Hilite pos
    int pos;
    if( (pos= it->second->m_fullDefs[0].Find( '(' )) != -1 ) { ///Find first '(' ///if found
        act->m_startPos = ++pos;  ///One past '('
        ///Find end of parameter
        act->m_endPos = act->m_startPos;  ///Init hilite to start
        for( ; it->second->m_fullDefs[0].Len(); pos++) {
            if( it->second->m_fullDefs[0].GetChar(pos) == ',' ||
                    it->second->m_fullDefs[0].GetChar(pos) == ')' ) {
                act->m_endPos = pos;  ///One past the last char.
                break;
            }
        }
    }
    return act;
}

///Notification that a char WAS added to the edit control
void ZionEdit::OnCharAdded (wxStyledTextEvent &event)
{ 
    if(! m_settled ) return;
    char chr = event.GetKey(); //Stc will only add printable chars to the control
    m_dirty = true;
    
    ///SET UP FOR LEXER ENABLE
    int preCurrPos = GetCurrentPos() - 1;
    int style2 = 0;  ///TODO ReOptimize after testing
    bool ctEnForStyle = false; ///CallTips En for this style
    if( preCurrPos >= 1 ) {
        //This always returns 0 //style = GetStyleAt( preCurrPos );
        style2 = GetStyleAt( preCurrPos - 1 );  ///Get style for char 2 places behind current position (before '(')
        for( int i= 0; i < syntaxStructs[ m_currentLexer ]->ctEnStyCnt; i++ ) {
            if( syntaxStructs[m_currentLexer]->ctEnStyles[i] == style2 ) {
                ctEnForStyle = true;  ///ONE OF THE ENABLES FOR CALLTIPS and OBJ MEM COMPLETION 
                    /// May have to use a whole different set for css style completions (ex. css allowed in strings, functions not)
                break;
            }
        }
        //Diag //m_ourFrame->SetStatusText(wxString::Format(_T("Pos = %d, Style0 = %d Style = %d,  Style2 = %d"),
        //preCurrPos+1, GetStyleAt(GetCurrentPos()), GetStyleAt(GetCurrentPos() -1), style2 ) );
    }
    
    //! On Newline Set Indentation to match previous line // Change this if support for mac files with \r is needed
    if( chr == '\n' ) {
        int currentLine = GetCurrentLine();
        if( m_currentLexer == wxSTC_LEX_HTML && m_htmlAC.brActive &&  ///Auto BR
                !m_htmlAC.inTag && !m_htmlAC.tagWaiting ) {
            GotoLine( currentLine - 1);
            InsertText( GetLineEnd(), _T("<br/>") );
            GotoLine( currentLine);
        }
        int lineIndent = GetLineIndentation(currentLine - 1);
        if( currentLine > 0 && lineIndent != 0 ) {  ///at start there is no line to compare to
            SetLineIndentation (currentLine, lineIndent); ///match previous line
            int a= PositionFromLine(currentLine); int b = GetLineEnd();  int i;
            for( i=a; i < b; i++) {
                if( GetCharAt( i ) != ' ' && GetCharAt( i ) != '\t' ) break;
            }
            GotoPos(i);  /// Limit to the end of the current line
        }
         //Diag //m_ourFrame->SetStatusText( wxString::Format(_T("lineIndent: %d"), lineIndent ), 2 );
    }
    
    ///CALL TIP AND A/C CHECKS
    if( (chr == '\n' || chr == ';')  && m_callTipsLoaded) {  ///TODO - Fine tune this per LEXER
        ///Remove ALL  ActiveCallTips
        activeCallTips.Empty();  ///Will delete all ActiveCallTip objects
        CallTipCancel();
        if( chr != ';' ) return;  ///We will at least use ';' for styles
    }

    if( chr == '}' || chr == '{' || chr == '(' || chr == ')' )  {  ///We want brace hilighting regardless of Call tips
        int a, currPos = GetCurrentPos();
        if( (a = BraceMatch( currPos -1))  == -1 )  BraceBadLight( currPos -1 ); 
        else { BraceHighlight( a, currPos -1 );
        //wxString str; str.Printf("BracePos = %d, Match Pos = %d, End Styled = %d", currPos -1, a, GetEndStyled() ); m_ourFrame->SetStatusText(str);
        }
    }
        
    ///INIT CALL TIPS    Note: Once there is a previous Call Tip, each "(" will open up a Call tip even if fullDef is empty (no match)
    if( chr == '(' && g_ctActive && m_callTipsLoaded && syntaxStructs[ m_currentLexer ]->cLike && ctEnForStyle ) {
        int count=0, a = GetCurrentPos(); 
        wxString partStr; char c=' '; 
            a--;  ///position at '('
        ///Search backwards
        if( GetCharAt(a-1) == ' ' ) --a;  ///Skip  one initial space if any.  ///TODO the space algo below will take over here - TEST!!!
        ///Start looking for the beginning of the word
            ///Continue searching while  wordchars or '_' or '.'  ///Note: Full object name will be used as the function name
            ///We will stop after the first object parent - looking for only one '.'
        bool parentFound = false;
        while( a > 0 ) { ///for safety.
            c = GetCharAt(a-1);
            if( ! isalnum( c ) && c != '_' && c !='.'  && c != ':' )  break;  ///if not word char. Are '_' and '-' considered alphanumeric? - No!
            if( c == '.' ) {
                if( ! parentFound ) parentFound = true; ///We will only allow one level of obj parent
                else  break;
            }
            count++;  a--;
        }
        for( int j=0; j< count; j++ )  partStr += GetCharAt(a + j); ///Build partStr in fwd direction /// a is at start of partStr
        partStr = partStr.Lower(); ///All keys to hashes are lower
        ///Look up partStr in g_callTips
        unsigned int i;
        ActiveCallTip * act;  bool goodCTCreated = false;
        CallTipsHash::iterator it;
        it = g_callTips.find( partStr );
        if( it == g_callTips.end() ) {
            ///NOT FOUND INITIALLY
            ///See if succeeding an object constructor
            if( partStr.GetChar(0) == '.' ) { ///Example partStr= ".Len()"
                if( (c= GetCharAt(a -1)) == ')' ) {  ///Example: Sheets(1).mmm()
                    ///We need to find the identifier before the matching '('
                    int b;
                    if( (b= BraceMatch(a -1)) != -1) a = b;  ///Move back to the matching brace
                    char c =  GetCharAt(a-1);
                    if( isalnum( c ) || c == '_' )  {  ///if is word char (identifier) ///Strict [ only looking for '-' ]
                        wxString partStr2 = BuildWord( a );  ///Scan backwards for identifier
                        ///then use our hashmap's search feature to find the key, then access
                        partStr = partStr2.Lower() + partStr;  ///Reassemble
                        ///Do CallTip
                        it = g_callTips.find( partStr ); /// eg. wxString.Len
                        if( it != g_callTips.end() ) {
                            goodCTCreated = true;
                            act = CreateHiliteCT( it );
                        }
                    }
                }
            }
            else {
                ///Search for SUBSTITUTIONS then
                int pos;
                if( (pos= partStr.Find('.')) != -1 ) {
                    wxString before = partStr.Mid(0, pos);
                    wxString after = partStr.Mid(pos);  ///The rest, including '.'
                    if( g_subs.find(before) != g_subs.end() ) {
                        before = g_subs[before];  ///If in the subst hash, do the substitution
                        partStr = before + after;  ///Reassemble
                        ///Do g_callTips find again
                        if( (it= g_callTips.find( partStr )) != g_callTips.end() ) { 
                            ///Create Good CallTip
                            goodCTCreated = true;
                            act = CreateHiliteCT(it);
                        }
                    }
                }
                ///Pointer Operator
                else if( GetCharAt( a - 1 ) == '>' && GetCharAt( a -2 ) == '-' ) {
                    a -= 2;
                    char c =  GetCharAt(a-1);
                    if( isalnum( c ) || c == '_' )  {  ///if is word char (identifier) ///Strict [ only looking for '-' ]
                        wxString partStr2 = BuildWord( a );
                        ///then use our hashmap's search feature to find the key, then access
                        wxString partStrSave = partStr;
                        partStr = partStr2.Lower() + _T(".") + partStr;  ///Reassemble
                        bool found = false;
                        if( (it= g_callTips.find(partStr)) != g_callTips.end() ) found = true;
                        else {
                            partStr = g_subs[partStr2.Lower()] + _T(".") + partStrSave;  ///Reassemble
                            //m_ourFrame->SetStatusText( partStr, 4 );  ///DIAG
                            if( (it= g_callTips.find(partStr)) != g_callTips.end() ) found = true;
                        }
                        if( found ) {
                        ///Do CallTip
                            ///Candidate for function named CreateHiliteCallTip(CallTipsHash::iterator it)
                            goodCTCreated = true;
                            act = CreateHiliteCT(it);
                        }
                    }
                }
            }
        }
        else {
            goodCTCreated = true;
            act = CreateHiliteCT( it );
        }
        
        if( ! goodCTCreated  ) act = new ActiveCallTip( NULL, GetCurrentLine() );  ///Create a new disabled Call Tip regardless

        activeCallTips.Add( act ); ///assign to activeCallTips

        int idx = activeCallTips.GetCount() - 1;
        ShowCallTips(idx); ///Will take care of setting param hilite, arrows,etc.
        
        return;
    }
        
    ///INIT CALL TIPS  on SPACE
    if( chr == ' ' && g_ctActive  && m_callTipsLoaded && ctEnForStyle  && 
        ( ( m_currentLexer != wxSTC_LEX_HTML &&  syntaxStructs[ m_currentLexer ]->batchLike ) ||
          ( m_currentLexer == wxSTC_LEX_HTML && GetStyleAt(preCurrPos - 1) == wxSTC_HBA_IDENTIFIER /* style before is VB Identifier */ ) ) ) {

        int a = GetCurrentPos() -1;  char c;
        c = GetCharAt(a-1);
        if( isalnum( c ) || c == '_' ) {  ///if not word char. Are '_' and '-' considered alphanumeric? - No!
            wxString partStr; int count=0;
            ///Search backwards
            ///Start looking for the beginning of the word
                ///Continue searching while  wordchars or '_' or '.'  ///Note: Full object name will be used as the function name
            while( a > 0 ) { ///for safety.
                c = GetCharAt(a-1);
                if( ! isalnum( c ) && c != '_' && c !='.'  )  break;  ///if not word char. Are '_' and '-' considered alphanumeric? - No!
                count++;  a--;
            }
            for( int j=0; j< count; j++ )  partStr += GetCharAt(a + j); ///Build partStr in fwd direction
            
            ActiveCallTip * act;
            CallTipsHash::iterator it;
            it = g_callTips.find( partStr.Lower() );
            if( it == g_callTips.end() ) act = new ActiveCallTip( NULL, GetCurrentLine() ); ///Create a new Disabled Call Tip regardless
            else {
                act = CreateHiliteCT( it );  ///Create and Hilite call tip
                activeCallTips.Add( act ); ///assign to activeCallTips
                int idx = activeCallTips.GetCount() - 1;
                ShowCallTips(idx); ///Will take care of setting param hilite, arrows,etc.
            }
        }
        ///We want a space to go on the HTML AC  //return;  
    }
    
    ///JAVASCRIPT STYLE VALS ///TODO - Generalize this to other attributes
    if( chr == '=' && g_acActive && m_callTipsLoaded && syntaxStructs[ m_currentLexer ]->cLike && ctEnForStyle ) {
        int count=0, a = GetCurrentPos(); 
        wxString pobj, partStr; char c=' '; 
            a--;  ///position at '='
        ///Search backwards
        if( GetCharAt(a-1) == ' ' ) --a;  ///Skip  one initial space if any.  ///TODO the space algo below will take over here - TEST!!!
        ///Start looking for the beginning of the word
            ///Continue searching while  wordchars or '_' or '.'  ///Note: Full object name will be used as the function name
            ///We will stop after the first object parent - looking for only one '.'
        bool parentFound = false;
        while( a > 0 ) { ///for safety.
            c = GetCharAt(a-1);
            if( ! isalnum( c ) && c != '_' && c !='.' )  break;  ///if not word char. Are '_' and '-' considered alphanumeric? - No!
            if( c == '.' ) {
                if( ! parentFound ) parentFound = true; ///We will only allow one level of obj parent
                else  break;
            }
            count++;  a--;
        }
        for( int j=0; j< count; j++ ) {  ///Build partStr in fwd direction // 'a' is at start of partStr
            if( (c= GetCharAt(a + j)) == '.' ) {
                pobj = partStr;  ///Save parent obj
                partStr.Empty();  ///Reset partStr - attribute
                continue;
            }
            partStr += c;
        }
        if( pobj == _T("style") || pobj == _T("document") ) {
            partStr = partStr.Lower();
            //m_ourFrame->SetStatusText( partStr ); ///DIAG
            if( g_jsStyleVals.find(partStr) != g_jsStyleVals.end() ) {
                InsertText( GetCurrentPos(), _T(" '") );  GotoPos( GetCurrentPos() + 2 ); ///nicety ///Let us insert only if we have a list to show
                m_acAttribQuote = SINGLEQUOTE;
                PreACShow( &g_jsStyleVals, partStr );  ///CSS or Regular Obj Mem
                //AutoCompShow( 0, g_jsStyleVals[partStr]->m_list );  ///CSS or Regular Obj Mem
            }
            else if( g_subs.find(partStr) != g_subs.end() && g_jsStyleVals.find(g_subs[partStr]) != g_jsStyleVals.end() ) {
                InsertText( GetCurrentPos(), _T(" '") );  GotoPos( GetCurrentPos() + 2 ); ///nicety ///Let us insert only if we have a list to show
                m_acAttribQuote = SINGLEQUOTE;
                PreACShow( &g_jsStyleVals, g_subs[partStr] );  ///CSS or Regular Obj Mem
                //AutoCompShow( 0, g_jsStyleVals[g_subs[partStr]]->m_list );
            }
        }
    }

    ///END FUNCTION ENDS CALL TIP
    if( chr == ')'  && m_callTipsLoaded ) {
        CallTipCancel();
        ///Delete Current/Last ActiveCallTip
        int cnt=0;  ///doesn't hurt to init
        if( (cnt=activeCallTips.GetCount()) > 0 )   activeCallTips.RemoveAt(cnt - 1);  ///If there are any existing call tips, remove last ///SAFE
        return;
    }
    
    ///COMMA
    ///Note: Once there is a previous Call Tip, each "(" will open up a Call tip even if fullDef is empty (no match)
    if( chr == ',' && g_ctActive  && m_callTipsLoaded ) { ///will advance to the next param in the current call tip, or do nothing if fullDef empty
        ///TODO If lexer is not parameter capable then return
        int cnt=0;
        if( (cnt=activeCallTips.GetCount()) > 0 ) {
            --cnt; ///set to last index, which is always the current call tip
            if( activeCallTips[cnt].m_ct == NULL )  return;  ///Skip disabled call tip
            ///SWITCH TO NEXT PARAM
            ++activeCallTips[cnt].m_paramIdx;
            ///Scan Def for param positions. For performance reasons we scan 
            wxArrayInt locs;
            LocateCallTipParams( locs, cnt );
            activeCallTips[cnt].m_startPos = 0;  activeCallTips[cnt].m_endPos = 0;
            if( 2 + 2 * activeCallTips[cnt].m_paramIdx <= locs.GetCount() ) { ///If the paramIdx is not outside the bounds of current Definition
                activeCallTips[cnt].m_startPos = locs[ activeCallTips[cnt].m_paramIdx * 2 ];
                activeCallTips[cnt].m_endPos = locs[ activeCallTips[cnt].m_paramIdx * 2 + 1 ];
            }
            ShowCallTips(cnt); ///Will take care of setting param hilite, arrows,etc.
        }
        else {  ///Try to Do new CallTip
            wxString fname;
            if( GetContextFunct( fname, true ) ) {
                CallTipsHash::iterator it = g_callTips.find( fname );
                if( it != g_callTips.end() ) {
                    int lineStart = PositionFromLine( GetCurrentLine() );
                    int commaCnt = 0;  char c;  int b;
                    ///COUNT COMMAS TO THE LEFT
                    for( int i= GetCurrentPos() -1; i >= lineStart; i-- ) { ///A reasonable search, limit to the start of line
                        c = GetCharAt(i);
                        if( c== ',' ) { ///count the 1st one also
                            ++commaCnt;  continue;
                        }
                        if( c == ')' ) {
                            if( (b= BraceMatch( i )) != -1)  i = b;  ///Move back to the matching brace
                            continue;  ///This allows us to skip functions in the middle, but choose the one, if we started on one.
                        }
                        if( c == '(' ) { break; }
                    }
                    ActiveCallTip * act = CreateHiliteCT( it );
                    activeCallTips.Add( act );  ///assign to activeCallTips
                    int cnt = activeCallTips.GetCount() -1;
                    activeCallTips[cnt].m_paramIdx = commaCnt;  ///Set up correct param
                    ///Scan Def for param positions. For performance reasons we scan param positions every time we need them, rather than scanning initially at the cost of slower startup
                    wxArrayInt locs;
                    LocateCallTipParams( locs, cnt );
                    activeCallTips[cnt].m_startPos = 0;  activeCallTips[cnt].m_endPos = 0;
                    if( 2 + 2 * activeCallTips[cnt].m_paramIdx <= locs.GetCount() ) { ///If the paramIdx is not outside the bounds of current Definition
                        activeCallTips[cnt].m_startPos = locs[ activeCallTips[cnt].m_paramIdx * 2 ];
                        activeCallTips[cnt].m_endPos = locs[ activeCallTips[cnt].m_paramIdx * 2 + 1 ];
                    }
                    ShowCallTips( cnt ); ///Will take care of setting param hilite, arrows,etc.
                }
            }
        }
        return;
    }

    ///OBJ COMPLETION

    if( (chr == '.' || ( chr == '>' &&  GetCharAt(preCurrPos - 1) == '-' ) ) && g_acActive && m_callTipsLoaded   /*&& ctEnForStyle*/ )  ShowObjACList( g_objMems );

    ///CSS/SCOPE COMPLETION - scope would need operator style to work
    if( chr == ':' && g_ctActive  && m_callTipsLoaded  && BuildWord( preCurrPos ).Lower() != "asp" /* && ctEnForStyle*/ )  ShowObjACList( g_cssAttrs );

    ///DIAG
    //m_ourFrame->SetStatusText(wxString::Format(_T("Char %c added"), chr ) );
    
    /** HTML TAGS AUTOCOMP  **/  ///HTML AC is Line based.
    if( m_currentLexer == wxSTC_LEX_HTML ) { // && m_htmlAC.active 
        if(  chr == '&' && g_acActive  && ! AutoCompActive() ) {
           DoAmpComplete(); ///AUTOCOMP for Special Chars 
        }
        wxString tagLwr = m_htmlAC.tag.Lower(); ///Optimzd
        
        if( chr == '<' ) {
            m_htmlAC.inTag = true; m_htmlAC.tagWaiting = false;  ///We are starting to build a tag and we don't have a complete one yet
            m_htmlAC.tag= wxEmptyString;
        }
        else if( chr == ' ' ) {  ///SPACE
            if( m_htmlAC.inTag ) {
                m_htmlAC.tagWaiting = true;  m_htmlAC.inTag = false;
                m_htmlAC.tag = BuildWord( preCurrPos, true );  ///Refresh our tag (needed if after autocomp), true for asp compliant
                tagLwr = m_htmlAC.tag.Lower(); 
            }
            else { ///See if we JUMPED IN THE MIDDLE of a tag, then try to build a tag
                char c; int i; bool found = false;   ///On "jump in", the tag may have already been completed.
                int lineStart = PositionFromLine( GetCurrentLine() );
                for( i= preCurrPos - 1; i >= lineStart; i-- ) { ///A reasonable search, limit to the start of line
                    if( GetCharAt(i) == '>' )  return; ///we are completely outside a tag, abort
                    if( GetCharAt(i) == '<' ) { found = true; break; }
                }
                if( ! found )  return;  ///Space will not try to do any further AC or CT beyond this point
                ///Build tag in fwd
                wxString tag;
                for( ++i ; i < GetLineEnd(); i++) {  ///GetLineEnd returns the last insert pos at eol, that is pos of the 1st newline char (a Roh custom f())
                    if( (c= GetCharAt(i)) != ' ' ) tag += c; ///Build tag
                    else break;  ///break on space
                }
                if( tag.Len() > 0 ) { m_htmlAC.tagWaiting = true; m_htmlAC.inTag = false;
                    m_htmlAC.tag = tag; tagLwr = tag.Lower(); 
                }
            }
            ///Try to do Attribute AutoComp
            //m_ourFrame->SetStatusText( tagLwr ); ///DIAG ///Referring to g_subs like this will cause a single key to be added to g_subs:  g_subs[tagLwr]  so we must always do a find first
            if( g_acActive ) {
                ///Auto Associate?
                if( m_htmlAC.idTrigger ) { ///If last word after '=' trigger was 'id'  do a special BuildWord to ascertain id value
                    int i = preCurrPos;
                    if( GetCharAt(i-1) == '"' ) i--;
                    wxString idVal = BuildWord( i );
                    //m_ourFrame->SetStatusText( wxString::Format(_T("idVal =  %s"), idVal ) );  ///DIAG
                    if( idVal != wxEmptyString && tagLwr != wxEmptyString ) {
                        wxString jsOrAsp = tagLwr.StartsWith("asp") ? tagLwr : "getelementbyid"; 
                        g_subs[ idVal.Lower() ] = jsOrAsp;  /// DotNet associates to the tagname, JavaScript associates to the generic element object
                        m_ourFrame->SetStatusText(wxString::Format(_T("Assocd %s to %s"), idVal, jsOrAsp ) );  ///DIAG
                    }
                    m_htmlAC.idTrigger = false;
                }
                ///Regular AutoComp
                if( ( style2 !=6 || GetCharAt(preCurrPos -1) == '"' ) && g_htmlTags.find(tagLwr) != g_htmlTags.end() ) { ///style 6 is Html double string
                    PreACShow( &g_htmlTags, tagLwr );
                    //AutoCompShow( 0, g_htmlTags[tagLwr]->m_list );
                }
                else if( ( style2 !=6 || GetCharAt(preCurrPos -1) == '"' ) && g_subs.find(tagLwr) != g_subs.end() && g_htmlTags.find(g_subs[tagLwr]) != g_htmlTags.end() ) {
                    PreACShow( &g_htmlTags, g_subs[tagLwr] );
                    //AutoCompShow( 0, g_htmlTags[g_subs[tagLwr]]->m_list );
                }
            }
            //m_ourFrame->SetStatusText( wxString(_T("Current Tag: ")) + m_htmlACtag );
        }
        else if( chr == '>' ) {
            AutoCompCancel();
            if( m_htmlAC.inTag || m_htmlAC.tagWaiting ) {
                ///Auto Associate?
                if( m_htmlAC.idTrigger ) { ///If last word after '=' trigger was 'id'  do a special BuildWord to ascertain id value
                    int i = preCurrPos;
                    if( GetCharAt(i-1) == '"' ) i--;
                    wxString idVal = BuildWord( i );
                    //m_ourFrame->SetStatusText( wxString::Format(_T("idVal =  %s"), idVal ) );  ///DIAG
                    if( idVal != wxEmptyString && tagLwr != wxEmptyString ) {
                        wxString jsOrAsp = tagLwr.StartsWith("asp") ? tagLwr : "getelementbyid"; 
                        g_subs[ idVal.Lower() ] = jsOrAsp;  /// DotNet associates to the tagname, JavaScript associates to the generic element object
                        m_ourFrame->SetStatusText(wxString::Format(_T("Assocd %s to %s"), idVal, jsOrAsp ) );  ///DIAG
                    }
                    m_htmlAC.idTrigger = false;
                }
                if( m_htmlAC.active && tagLwr != _T("br") && tagLwr != _T("col") && tagLwr != _T("hr") && tagLwr != _T("") // case of vb <>
                        && tagLwr != _T("frame") && tagLwr != _T("img") && tagLwr != _T("input")  && tagLwr != _T("link")  && tagLwr != _T("meta")
                          && tagLwr != _T("param")  && tagLwr != _T("wbr")  
                        && ! tagLwr.StartsWith("!") &&  ! tagLwr.StartsWith("?") && ! tagLwr.StartsWith("%")  &&  GetCharAt(preCurrPos - 1) != '/' ) {  ///Don't ac script or single tags
                    if( !m_htmlAC.tagWaiting ) m_htmlAC.tag = BuildWord( preCurrPos, true );  ///Refresh our tag (needed if after autocomp, but not after attribs added) ///true for asp compliant
                    if( tagLwr == "body" || tagLwr == "div" || tagLwr == "form" || tagLwr == "head" || tagLwr == "html" || tagLwr == "ol" || tagLwr == "pre"
                            || tagLwr == "script" || tagLwr =="select" || tagLwr =="style" || tagLwr =="table" || tagLwr=="ul" ) {
                        int brStart = BraceMatch( preCurrPos );  ///Find starting '<'  - it is a fair assumption that one should be there if Intag or tagwaiting
                        int line = GetCurrentLine();  int lnStart = PositionFromLine( line );  bool aloneOnLine = true;
                        if( brStart > lnStart ) { ///Check if alone on the line, otherwise we will not do multiline completion
                            for( int i=brStart -1; i >= lnStart; i-- ) { ///Start one less than '<'
                                if( GetCharAt(i) != ' ' && GetCharAt(i) != '\t' ) { aloneOnLine = false;  break; }
                            }
                        }
                        if( aloneOnLine ) { ///Do Completion across 3 lines
                            int ind = GetLineIndentation(line);
                            //DIAG //m_ourFrame->SetStatusText(wxString::Format("Indentation %d", ind) );
                            int pos = GetCurrentPos();  //wxString currTag = m_htmlAC.tag;  ///m_htmlAC will get wiped out by GotoLine()
                            InsertText( pos, wxString::Format(_T("\r\n\r\n</%s>"), m_htmlAC.tag ) );  ///TODO - Optimize by adding the ending tag here at once.
                            SetLineIndentation( line+1, ind );
                            SetLineIndentation( line+2, ind );
                            GotoLine(line+1); ///Line change will reset the current tag in m_htmlAC
                            GotoPos( GetLineEnd() );  ///Effectively go to the end of the indentation
                        }
                        else InsertText( GetCurrentPos(), wxString::Format(_T("</%s>"), m_htmlAC.tag ) );
                    }
                    else InsertText( GetCurrentPos(), wxString::Format(_T("</%s>"), m_htmlAC.tag ) );
                }
                m_htmlAC.ResetTag(); //inTag = false;  m_htmlAC.tagWaiting = false; ///Clean up //m_htmlAC.tag = wxEmptyString; ///Clean up
            }
        }
        else if( chr == '/' ) {
            if( m_htmlAC.inTag || m_htmlAC.tagWaiting ) { /// If there is a tag in the making or already buffered
                m_htmlAC.ResetTag();
            }
        }
        else if( m_htmlAC.inTag ) { /// If a REGULAR CHAR just added to the tag name itself 
            ///On AutoComp Selection event, if in HTML mode, refresh m_htmlAC.tag by scanning backwards for '<' or ' ', then scan forward. Use BuildWord(GetCurrentPos())
            m_htmlAC.tag.Append(chr); ///add it, then try to complete it - Normal
            if( chr == '?' || chr == '!' ) { ///Abort on invalid tag char // chr == '%' ||
                m_htmlAC.ResetTag();
            }
            else if( g_acActive ) DoACHtmlTag( m_htmlAC.tag.Lower() ); ///Do [word] AC for tagname
        }
        else if( m_htmlAC.tagWaiting ) { ///We have a tag to work with.  ///Attributes and values here
            if( g_acActive ) {
            ///Do Attribute VALUE triggers
                if( chr == ';' && FindWord_style(preCurrPos) ) { ///End of an style property, so we scan back looking for an '='
                    if( g_htmlAttrs.find(_T("style")) != g_htmlAttrs.end() ) {
                        PreACShow( &g_htmlAttrs, wxString("style") );
                        //AutoCompShow( 0, g_htmlAttrs[_T("style")]->m_list );  ///CSS or Regular Obj Mem
                    }
                }
                else if( chr == '=' ) { /// chr == '='  ///Do Value Completion
                    wxString attribute = BuildWord( preCurrPos ); ///Just before '='
                    if( g_htmlAttrs.find(attribute) != g_htmlAttrs.end() ) {  ///Attrib FOUND
                        InsertText( GetCurrentPos(), _T("\"") );  GotoPos( GetCurrentPos() + 1 ); ///nicety ///Let us insert only if we have a list to show
                        if( attribute.Lower() != _T("style") ) { ///Auto-add double quote
                            m_acAttribQuote = DOUBLEQUOTE;
                        }
                        PreACShow( &g_htmlAttrs, attribute );
                        //AutoCompShow( 0, g_htmlAttrs[attribute]->m_list );  ///CSS or Regular Obj Mem
                    }
                    else if( g_htmlAttrs.find(attribute.Lower()) != g_htmlAttrs.end() ) {  ///Attrib lowercase FOUND
                        attribute = attribute.Lower();
                        InsertText( GetCurrentPos(), _T("\"") );  GotoPos( GetCurrentPos() + 1 ); ///nicety ///Let us insert only if we have a list to show
                        if( attribute != _T("style") ) { ///Auto-add double quote
                            m_acAttribQuote = DOUBLEQUOTE;
                        }
                        PreACShow( &g_htmlAttrs, attribute );
                        //AutoCompShow( 0, g_htmlAttrs[attribute]->m_list );  ///CSS or Regular Obj Mem
                    }
                    else if( g_subs.find(attribute) != g_subs.end() && g_htmlAttrs.find(g_subs[attribute]) != g_htmlAttrs.end() ) {
                        InsertText( GetCurrentPos(), _T("\"") );  GotoPos( GetCurrentPos() + 1 ); ///nicety ///Let us insert only if we have a list to show
                        if( attribute.Lower() != _T("style") ) { ///Auto-add double quote
                            m_acAttribQuote = DOUBLEQUOTE;
                        }
                        PreACShow( &g_htmlAttrs, g_subs[attribute] );
                        //AutoCompShow( 0, g_htmlAttrs[g_subs[attribute]]->m_list );
                    }
                    else if( attribute.Lower() == "id" ) { m_htmlAC.idTrigger = true; }
                }
                else DoACHtmlAttr( m_htmlAC.tag.Lower() ); ///Since we already know the tag, all other completion [we are assuming] must be an attribute
            }
        }
        else { ///WE ARE EFFECTIVELY OUTSIDE OF A SINGLE TAG
            ///See if we JUMPED IN THE MIDDLE of a tag, then try to build a tag
            char c; int i; bool found = false;
            int lineStart = PositionFromLine( GetCurrentLine() );
            for( i= preCurrPos; i >= lineStart; i-- ) { ///A reasonable search, limit to the start of line
                if( GetCharAt(i) == '>' )  break; ///we are completely outside a tag, abort
                if( GetCharAt(i) == '<' ) { found = true; break; }
            }
            if( found ) {  ///Build tag in fwd
                wxString tag; bool abort = false;
                for( ++i; i < GetLineEnd(); i++ ) {  ///GetLineEnd returns the last insert pos at eol, that is pos of the 1st newline char
                    c = GetCharAt(i);
                    if( c == ' ' ) break;
                    else if( c == '/' || c == '%' || c == '?' || c == '!' ) {  abort = true;  break;  }
                    else tag += c; ///Build tag
                }
                if( tag.Len() > 0 && ! abort ) {
                    m_htmlAC.tagWaiting = true; m_htmlAC.inTag = false;  m_htmlAC.tag = tag;
                    if( g_acActive ) { ///Yes we want this
                    ///Do Attribute Triggers
                        if( chr == ';' && FindWord_style(preCurrPos) ) { ///End of an style property, so we scan back looking for an '='
                            if( g_htmlAttrs.find(_T("style")) != g_htmlAttrs.end() ) {
                                PreACShow( &g_htmlAttrs, wxString("style") );
                                //AutoCompShow( 0, g_htmlAttrs[_T("style")]->m_list );  ///CSS or Regular Obj Mem
                            }
                        }
                        else if( chr == '=' ) { /// chr == '='  ///Do Value Completion
                            wxString attribute = BuildWord( preCurrPos ); ///Just before '='
                            if( g_htmlAttrs.find(attribute) != g_htmlAttrs.end() ) {  ///TODO - could add a lowercase attribute block - but this case is unlikely on jump in with ('=')
                                InsertText( GetCurrentPos(), _T("\"") );  GotoPos( GetCurrentPos() + 1 ); ///nicety ///Let us insert only if we have a list to show
                                if( attribute.Lower() != _T("style") ) { ///Auto-add double quote
                                    m_acAttribQuote = DOUBLEQUOTE;
                                }
                                PreACShow( &g_htmlAttrs, attribute );
                                //AutoCompShow( 0, g_htmlAttrs[attribute]->m_list );  ///CSS or Regular Obj Mem
                            }
                            else if( g_htmlAttrs.find(attribute.Lower()) != g_htmlAttrs.end() ) {  ///Attrib lowercase FOUND
                                attribute = attribute.Lower();
                                InsertText( GetCurrentPos(), _T("\"") );  GotoPos( GetCurrentPos() + 1 ); ///nicety ///Let us insert only if we have a list to show
                                if( attribute != _T("style") ) { ///Auto-add double quote
                                    m_acAttribQuote = DOUBLEQUOTE;
                                }
                                PreACShow( &g_htmlAttrs, attribute );
                                //AutoCompShow( 0, g_htmlAttrs[attribute]->m_list );  ///CSS or Regular Obj Mem
                            }
                            else if( g_subs.find(attribute) != g_subs.end() && g_htmlAttrs.find(g_subs[attribute]) != g_htmlAttrs.end() ) {
                                InsertText( GetCurrentPos(), _T("\"") );  GotoPos( GetCurrentPos() + 1 ); ///nicety ///Let us insert only if we have a list to show
                                if( attribute.Lower() != _T("style") ) { ///Auto-add double quote
                                    m_acAttribQuote = DOUBLEQUOTE;
                                }
                                PreACShow( &g_htmlAttrs, g_subs[attribute] );
                                //AutoCompShow( 0, g_htmlAttrs[g_subs[attribute]]->m_list );
                            }
                        }
                        else DoACHtmlAttr( m_htmlAC.tag.Lower() ); ///Since we already know the tag, all other completion [we are assuming] must be an attribute
                    }
                }
            }
            ///HTML GENERAL (Outside of Tag AC)
            else if( g_acActive  && ! AutoCompActive() ) {
                /*chr == '&' ?  DoAmpComplete() :*/  DoAutoComplete(); ///AUTOCOMP for Special Chars or General Words
            }
        }
        //m_ourFrame->SetStatusText( wxString(_T("Current Tag: ")) + m_htmlAC.tag );
    }
    
    ///STANDARD (Non-Tag) AUTO COMPLETE
    else if( g_acActive && m_currentLexer != wxSTC_LEX_NULL ) { ///AutoComplete enabled and not text mode
        if( ! AutoCompActive() ) DoAutoComplete(); ///Do autocomplete if list not already shown ///Once the list is shown it takes care of itself i.e. we call it once per word
        //Diag// else m_ourFrame->SetStatusText( _T("Auto Complete already active") );
    }
}

void ZionEdit::DoACHtmlTag( const wxString & tagLwr )
{
    //if( g_htmlTags.find(tagLwr) == g_htmlTags.end() )  return;  ///Critical. We don't want to reference a key that doesn't exist
    //if( g_htmlAttrs.find( g_htmlTags[tagLwr] ) == g_htmlTags.end() )  return;  ///Critical. We don't want to reference a key that doesn't exist
    int j, count=0, a = GetCurrentPos();
    wxString partStr; char c=' '; bool found;
    ///Start looking for the beginning of the word
    while( a > 0 ) { ///for safety.
        c = GetCharAt(a-1);
        if( ! ( isalnum( c ) || c == '_'  || c == '-' || c == ':' ) )  break;  ///if not word char or '_' and '-' 
        count++;  a--;
    }
    if( count > 0 ) { ///Do AC on the 1st char
        for( j=0; j< count; j++ )  partStr += GetCharAt(a + j); ///Build partStr in fwd direction
        partStr = partStr.Lower();
        found = false;
        for( j=0; j< g_tagsList["Tags"]->m_array.GetCount(); j++ ) {
            if( g_tagsList["Tags"]->m_array[j].Lower().StartsWith(partStr) )
                {  found = true; } ///Don't break - build full list
        }
        if( found ) {
            //this is already done by STC if( CallTipActive() )  CallTipCancel();
            //m_acCurrObj = "Tags";   //wxEmptyString =>Signify we don't have valid descriptions for AC Shown
            PreACShow( &g_tagsList, "Tags", partStr.Len() );
            //AutoCompShow( count, list ); ///Don't show till count alphanums or _ typed
        }
    }
}

void ZionEdit::DoACHtmlAttr( const wxString & tagLwr )
{
    if( g_htmlTags.find(tagLwr) == g_htmlTags.end() )  return;  ///Critical. We don't want to reference a key that doesn't exist
    //if( g_htmlAttrs.find( g_htmlTags[tagLwr] ) == g_htmlTags.end() )  return;  ///Critical. We don't want to reference a key that doesn't exist
    int count=0, a = GetCurrentPos();
    wxString partStr; char c=' '; bool found = false;
    ///Start looking for the beginning of the word
    while( a > 0 ) { ///for safety.
        c = GetCharAt(a-1);
        if( ! ( isalnum( c ) || c == '_'  || c == '-' ) )  break;  ///if not word char or '_' and '-' 
        count++;  a--;
    }
    if( count > g_numACChars ) {
        for( int j=0; j< count; j++ )  partStr += GetCharAt(a + j); ///Build partStr in fwd direction
        //m_ourFrame->SetStatusText(partStr);
        ///Look up in attribs list
        for( int i=0; i< g_htmlTags[tagLwr]->m_array.GetCount(); i++ ) {
            if( g_htmlTags[tagLwr]->m_array[i].StartsWith( partStr.c_str() ) )  { found = true; break; }
        }
        if( found ) {
            if( CallTipActive() )  CallTipCancel();
            m_acCurrObj = wxEmptyString;  ///Signify we don't have valid descriptions for AC Shown
            AutoCompShow( count, g_htmlTags[tagLwr]->m_list ); ///Don't show till count alphanums or _ typed
        }
        else { ///Look up in global list
            for( int i=0; i< acArrayList.GetCount(); i++ ) {
                if( acArrayList[i].StartsWith( partStr.c_str() ) )  { found = true; break; }
            }
            if( found ) {
                if( CallTipActive() )  CallTipCancel();
                m_acCurrObj = wxEmptyString;  ///Signify we don't have valid descriptions for AC Shown
                AutoCompShow( count, acList ); ///Don't show till count alphanums or _ typed
            }
            
        }
    }
}

///AutoComp Started or Selection Changed (we hijack SCN_USERLISTSELECTION)
void ZionEdit::OnAutoCompSelChange( wxStyledTextEvent &event ) {
    wxString val = event.GetText();
    int acUL = event.GetListType();  // if acUL > 1 then a new acLB was created so position our descript dlg accordingly
    if( acUL == -1 && m_descript ) { ///Destroy description dialog
        m_descript->Destroy(); m_descript = NULL;  m_acAttribQuote = NOQUOTE; return;
    }
    int acX = event.GetWParam();
    int acY = PointFromPosition(GetCurrentPos()).y;  ///TODO - rename acY //event.GetLParam();
    if( val != wxEmptyString /* no sel is a change */ && m_acCurrObj != wxEmptyString && m_descript ) { ///We have an obj completion and the Note dlg is already displayed
        wxString descript = (*m_acCurrHash)[m_acCurrObj]->m_descrip[val]; ///A pointer to a hash to a hash!!!
        wxString str, obj;
        ///Fix up
        if( m_acCurrObj == "Amp" ) {
            obj = "HTML Char Entity ";
            val = val.Prepend("&amp;");
        }
        else obj = m_acCurrObj;

        if( descript != wxEmptyString ) {
        ///Test show it in HTML win
            str = _T("<html><body BGCOLOR=\"#F2E494\"><font color=\"#784520\"><b>") +
                obj + _T("=> ") + val + "</b></font>&nbsp;&nbsp;" + descript + _T("</body></html>");
            m_descript->m_htmlWin->SetPage(str);
        }
        else {
            str = _T("<html><body BGCOLOR=\"#F2E494\"><font color=\"#784520\"><b>") +
                obj + _T("=>") + val + _T("</b></font></body></html>");
            m_descript->m_htmlWin->SetPage(str);
        }
        ///Adjust Position after initial creation
        if( acUL >= 99888 ) {
//            str.Printf(_T("acY: %d, FrameClientOrigin: %d, parY(Screen): %d, parY(Client): %d\nCurrent point.y: %d"), ///Diag
//                    acY, GetClientAreaOrigin().y, m_ourFrame->GetPosition().y, ScreenToClient( m_ourFrame->GetPosition() ).y,
//                    PointFromPosition(GetCurrentPos()).y );
//            wxMessageBox(str);
            int y = m_ourFrame->GetPosition().y; ///Default to top pos
            int clientOffset = -( ScreenToClient(m_ourFrame->GetPosition()).y ); ///ScreenToClient returns a fixed -ve offset to the Client Area -weird!
            acY = y + clientOffset + acY; ///Xlat to Screen coord
            int parH = m_ourFrame->GetSize().GetHeight();
            ///By default the DescriptDlg is at the top
            if( m_ourFrame->IsFullScreen() ) y += 4;  ///In Vista fullscreen is off the edge
            if( GetCurrentLine() == 0 )  y += parH - 48 - 13;  ///put it to the btm 
            else if( acY > y + parH/2 ) { /// > half way down
                if( y + parH - acY > 220 )  /// and not within nnn of the btm
                    y += parH - 48 - 13;  ///put it to the btm  ///height of descriptDlg is 48
            }
            ///Upper
            else if( m_ourFrame->IsFullScreen() ) y += parH -48 -13;
            m_descript->SetSize( wxDefaultCoord, y, wxDefaultCoord, wxDefaultCoord );
        }
    }
}

 ///AutoComp Ended - CANCELED or word COMPLETED
void ZionEdit::OnAutoCompSel( wxStyledTextEvent & event )
{
    if( m_descript ) { m_descript->Destroy(); m_descript = NULL; } ///Destroy description dialog
    if( event.GetLParam() == -1 ) m_acAttribQuote = NOQUOTE;  ///AC canceled
    if( m_acAttribQuote == NOQUOTE )  return;  ///covers both local and ext setting of m_acAttribQuote

    int a = GetCurrentPos();
    if( m_acAttribQuote == SINGLEQUOTE )  InsertText( a, "'" );
    else InsertText( a, "\"" );
    GotoPos( a+ 1 );
    //wxMessageBox(_T("AUTO_COMP_SEL Event") );
    m_acAttribQuote = NOQUOTE;
}

///This is where obj member completion starts
void ZionEdit::PreACShow( MemberLists * ptrHash, const wxString & obj, int count ) {
    m_acCurrHash = ptrHash;  ///Save for use in the OnAutoCompSelChange handler
    m_acCurrObj = obj;  ///Ditto
    if( m_descript ) { m_descript->Destroy(); m_descript = NULL; }
    m_descript = new DescriptDlg( this, m_ourFrame->GetPosition(), m_ourFrame->GetSize().GetWidth(), _T("heading"), _T("Detailed Description") );
    m_descript->Show(true);

    AutoCompSetIgnoreCase(false); ///Use strict casing - otherwize buggy
    AutoCompShow( count, (*ptrHash)[obj]->m_list );
}

void ZionEdit::ShowObjACList( MemberLists & hash )
{ 
    ///Show only that object's members
    if( CallTipActive() )  CallTipCancel();
    if( AutoCompActive() )  AutoCompCancel();  ///Whole new AC
    ///Backwards search
    int a = GetCurrentPos() -1;  char c = GetCharAt(a-1);
    if( isalnum( c ) || c == '_' )  {  ///if is word char (identifier)
        wxString partStr;  bool found = false;  int count=0;
        ///Search backwards for the beginning of the word ///Continue searching while  wordchars or '_'  ///Note: We only want the leftmost obj name
        while( a > 0 ) { ///for safety.
            c = GetCharAt(a-1);
            if( ! isalnum( c ) && c != '_' && c != '-' && c != ':' )  break;  ///TESTING - see if c !=':' breaks anything - Good so far
            count++;  a--;
        }
        for( int j=0; j< count; j++ )  partStr += GetCharAt(a + j); ///Build partStr in fwd direction
        ///then use our hashmap's search feature to find the key, then access
        partStr = partStr.Lower();
        //m_ourFrame->SetStatusText( partStr ); ///DIAG
        if( hash.find(partStr) != hash.end() ) {
            PreACShow( &hash, partStr );  ///CSS or Regular Obj Mem
            //AutoCompShow( 0, hash[partStr]->m_list );  ///CSS or Regular Obj Mem
        }
        else if( g_subs.find(partStr) != g_subs.end() && hash.find(g_subs[partStr]) != hash.end() ) {
            PreACShow( &hash, g_subs[partStr] );  ///CSS or Regular Obj Mem
            //AutoCompShow( 0, hash[g_subs[partStr]]->m_list );
        }
        //else  m_ourFrame->SetStatusText(_T("Use CTRL-ALT-A to associate selected identifier to known type => Autocomplete") );
    }
    ///Deal with Scope resolution operator
    else if( c == ':' && GetCharAt(a) == ':' ) {  ///If we are picking up another colon instead of an (identifier type char)
        --a;  ///Move everything back one
        char c =  GetCharAt(a-1);
        if( isalnum( c ) || c == '_' )  {  ///if is word char (identifier)
            wxString partStr = BuildWord(a);
            ///then use our hashmap's search feature to find the key, then access
            partStr = partStr.Lower();
            if( g_scopeMems.find(partStr) != g_scopeMems.end() ) {
                PreACShow( &g_scopeMems, partStr );  ///CSS or Regular Obj Mem
                //AutoCompShow( 0, g_scopeMems[partStr]->m_list );
            }
        }
    }
    ///Deal with pointer operator
    else if( c == '-' && GetCharAt(a) == '>' ) {  ///If we are picking up another colon instead of an (identifier type char)
        --a;  ///Move everything back one
        char c =  GetCharAt(a-1);
        if( isalnum( c ) || c == '_' )  {  ///if is word char (identifier)
            wxString partStr = BuildWord(a);
            partStr = partStr.Lower();
            if( hash.find(partStr) != hash.end() ) {
                PreACShow( &hash, partStr );  ///CSS or Regular Obj Mem
                //AutoCompShow( 0, hash[partStr]->m_list );  ///CSS or Regular Obj Mem
            }
            ///If not found, then see it is a substitution, then look with its base
            else if( g_subs.find(partStr) != g_subs.end() && hash.find(g_subs[partStr]) != hash.end() ) {
                PreACShow( &hash, g_subs[partStr] );  ///CSS or Regular Obj Mem
                //AutoCompShow( 0, hash[g_subs[partStr]]->m_list );
            }
        }
    }
    ///Deal with ( ).member
    else if( c == ')' && GetCharAt(a) == '.' ) {  ///Example: Sheets(1).mmm
        ///We need to find the identifier before the matching '('
        int b;
        if( (b= BraceMatch(a -1)) != -1) a = b;  ///Move back to the matching brace
        char c =  GetCharAt(a-1);
        if( isalnum( c ) || c == '_' )  {  ///if is word char (identifier)
            wxString partStr = BuildWord(a);
            ///then use our hashmap's search feature to find the key, then access
            partStr = partStr.Lower();
            if( g_objMems.find(partStr) != g_objMems.end() ) {
                //AutoCompShow( 0, g_objMems[partStr]->m_list );
                PreACShow( &hash, partStr );  ///CSS or Regular Obj Mem
            }
        }
    }
    return; ///Interaction with HTML AC -good
}

void ZionEdit::LocateCallTipParams( wxArrayInt & locs, int idx ) ///SCAN CURRENT DEFINITION to determine Parameter locs.
{  /// locs array must be empty///For saving successive start and end positions of parms (Level3)
    wxString str  = activeCallTips[idx].m_ct->m_fullDefs[ activeCallTips[idx].m_ovrldIdx ]; ///Get definition of current Call Tip
    int pos; 
    ///LOCATE PARAMETERS
    if( (pos= str.Find( '(' )) != -1 ) { ///Find first '('
        ++pos; ///skip past '('
        locs.Add(pos);  ///1st param start
        ///Scan the rest of the definition
        for( ; pos< str.Len(); pos++ ) {
            if( str.GetChar(pos) == ',' ) {
                locs.Add(pos); /// Ending of the prev param
                locs.Add(pos + 1); ///Start of next param
            }
            if( str.GetChar(pos) == ')' ) {
                locs.Add(pos);  ///Last param ending pos
                break;
            } 
        }
    }
}

wxString ZionEdit::GetContextWord( bool leftOnly )
{
    char c; int i; bool found = false; wxString fname;
    int posStartIdfr = -1; bool inIdfr= false; bool wordSet = false;
    int lineStart = PositionFromLine( GetCurrentLine() );
    ///SCAN LEFT
    for( i= GetCurrentPos() -1; i >= lineStart; i-- ) { ///A reasonable search, limit to the start of line
        c = GetCharAt(i);
        if( c == ')' ) { 
            int b;
            if( (b= BraceMatch( i )) != -1) i = b;  ///Move back to the matching brace
            if( i != GetCurrentPos() )  {
                inIdfr = false;  ///if we were building a last-resort word, end it
                continue;  ///This allows us to skip functions in the middle, but choose the one, if we started on one.
            }
            else { found = true;  break; } ///We match to the starting brace of  the function we started on.
        }
        if( c == '(' ) { found = true; break; }
        ///mark identifier point while we are scanning
        if( isalnum( c ) || c == '_' ) inIdfr = true; ///Else it is already set so we are not looking for new word
        else if( inIdfr ) { ///We have hit a transition pt from identifier to non-identifier
            if( !wordSet ) { posStartIdfr = i + 1; wordSet = true; }///Save this point
            inIdfr = false;  ///Enabling will give first word on line; else nearest word to the left
        }
    }
    if( found ) { ///Build the function name
        if( GetCharAt( i - 1) == ' ' )  i--;
        fname = BuildWord2( i );
        //m_ourFrame->SetStatusText( fname );
    }
    else if( ! leftOnly ) {
        ///SCAN RIGHT
        for( i = GetCurrentPos(); i < GetLineEnd(); i++ ) {
            if( GetCharAt( i ) == '(' ) { found = true; break; }
        }
        if( found ) { ///Build the function name
            if( GetCharAt( i - 1) == ' ' )  i--;
            fname = BuildWord2( i );
            //m_ourFrame->SetStatusText( fname );
        }
        ///Last Resort - just lookup the nearest word to the left
        else {
            if( posStartIdfr == -1 ) posStartIdfr = lineStart;  ///If we didn't find a nearest word loc, then start from SOL 
            for( int j= posStartIdfr; j< GetLineEnd(); j++ ) {
                c = GetCharAt( j );
                if( isalnum( c ) || c == '_' ) fname += c;
                else break;
            }
            //m_ourFrame->SetStatusText( fname );
        }
    }
    return fname;
}

bool ZionEdit::GetContextFunct( wxString & fname, bool leftOnly )  ///fname must not be const. Allow subs. leftOnly allows this to be used for calltip lookup on jump in
{
    fname = GetContextWord( leftOnly );
    if( fname.Len() > 0 ) {  ///Look call tip
        fname = fname.Lower();  int pos= -1;
        if( g_callTips.find( fname ) != g_callTips.end() ) {
            return true;
        }
        else if( (pos= fname.Find('.')) != -1 ) { ///WE WILL NEED TO SPLIT OFF THE OBJ, subst, then reconcat
            wxString obj = fname.Mid(0, pos);
            if( g_subs.find(obj) != g_subs.end() ) { ///Any Substitutions
                    fname = g_subs[obj] + fname.Mid( pos );  ///Re-concat
                    //m_ourFrame->SetStatusText(fname);
                if( g_callTips.find( fname ) != g_callTips.end() ) {
                    return true;
                }
            }
        }
        else if( (pos= fname.Find('>')) != -1 && fname.GetChar(pos - 1) == '-' && m_currentLexer != wxSTC_LEX_HTML ) { ///Pointer
            wxString obj = fname.Mid(0, pos -1);
            if( g_subs.find(obj) != g_subs.end() ) { ///Any Substitutions
                    fname = g_subs[obj] + _T(".") + fname.Mid( pos + 1 );  ///Re-concat
                    //m_ourFrame->SetStatusText(fname);
                if( g_callTips.find( fname ) != g_callTips.end() ) {
                    return true;
                }
            }
        }
    }
    return false;
}

void ZionEdit::OnContextCallTip( wxCommandEvent & WXUNUSED(event) )  ///A Form of Context Help
{
    wxString fname;
    if( GetContextFunct( fname ) ) { ///Display Description ///function can change fname ///will scan for function Left or Right
        wxString nofn; int limit = g_callTips[fname]->m_fullDefs.GetCount();
        for( int i=0; i < limit; i++ ) {  ///Rem - Array are synched
            nofn = wxString::Format(_T("%d of %d<br>"), i + 1, limit ); 
            CallTipNoteDlg noteDlg( this, m_ourFrame->GetPosition(), m_ourFrame->GetSize().GetWidth(),
                    nofn + g_callTips[ fname ]->m_fullDefs[i], g_callTips[ fname ]->m_descripts[i] );
            noteDlg.ShowModal();
        }
    }
    else { ///Do help file
        if( wxFileName::FileExists( wxString::Format(_T("%s\\help.html"), m_ourFrame->m_exeDir ) ) ) {
            wxExecute( wxString::Format(_T("\"%s\" \"%s\\help.html\""), ///We will always quote paths
                    g_pathToBrowser, m_ourFrame->m_exeDir ) );
        }
        else m_ourFrame->SetStatusText(_T("Help file: [help.html] not found in Zion's directory") );
    }
}

///This will be called from OnCharAdded and OnCallTipClicked()
void ZionEdit::ShowCallTips( int activeCallTipIdx )
{
    //if( AutoCompActive() )  AutoCompCancel();
    ///Will access the activeCallTips to pick up params to display
    int i = activeCallTipIdx;
    if( activeCallTips[i].m_ct == NULL )  return;  ///Skip blank call tip
    
    int start = activeCallTips[i].m_startPos; int end = activeCallTips[i].m_endPos;
    ///Build CallTip
    wxString callTip;  wxString str, str2;
    if( activeCallTips[i].m_numOvrlds > 1 ) { ///Deal with multiple ovrloads
        callTip = wxString::Format(_T("\001 %d of %d \002 "), activeCallTips[i].m_ovrldIdx + 1, activeCallTips[i].m_numOvrlds ); ///Prepend Arrows
        int lenArrows = callTip.Len();
        start += lenArrows; end += lenArrows;
        //m_ourFrame->SetStatusText(wxString::Format(_T("CTLenofArrows = %d, start = %d, end = %d "), callTip.Len(), start, end ) );  ///Diag
        ///Add the rest of the string.
        str = activeCallTips[i].m_ct->m_fullDefs[ activeCallTips[i].m_ovrldIdx ];  ///Use overload index to determine which str goes to the top
        //str = wxString::Format("%s", activeCallTips[i].m_ovrldIdx +1, activeCallTips[i].m_ct->m_fullDefs[ activeCallTips[i].m_ovrldIdx ] );  ///Use overload index to determine which str goes to the top
        if( str.Len() > 110 ) {
            int a;
            if( (a=str.Mid(104).Find(" ")) != -1 ) {  ///Attempt to break long lines on space
                callTip +=  str.Left( 104+a ) + "\n                      " + str.Mid( 105+a );
                if( start > 104 + a )  start += 23;
                if( end > 104 + a )  end += 22; ///not sure why this needs less.
            }
            else callTip += str;
        }
        else callTip += str;
        
        ///Add the other line
        callTip += "\n      ";  ///Prepare for 2nd CT (usually 2nd line)
        if( activeCallTips[i].m_ovrldIdx + 1 < activeCallTips[i].m_numOvrlds ) {
            str = wxString::Format("%d.  %s", activeCallTips[i].m_ovrldIdx+2, activeCallTips[i].m_ct->m_fullDefs[ activeCallTips[i].m_ovrldIdx + 1 ]);
            if( str.Len() > 110 ) {
                int a;
                if( (a=str.Mid(104).Find(" ")) != -1 ) {  ///Attempt to break long lines on space
                    callTip +=  str.Left( 104+a ) + "\n                      " + str.Mid( 105+a );
                }
                else callTip += str;
            }
            else callTip += str;
        }
        else {
            str = wxString::Format("1. %s", activeCallTips[i].m_ct->m_fullDefs[0]);
            if( str.Len() > 110 ) {
                int a;
                if( (a=str.Mid(104).Find(" ")) != -1 ) {  ///Attempt to break long lines on space
                    callTip +=  str.Left( 104+a ) + "\n                      " + str.Mid( 105+a );
                }
                else callTip += str;
            }
            else callTip += str;
        }
    }
    else {
        str = activeCallTips[i].m_ct->m_fullDefs[0];
        if( str.Len() > 110 ) {
            int a;
            if( (a=str.Mid(104).Find(" ")) != -1 ) {  ///Attempt to break long lines on space
                callTip +=  str.Left( 104+a ) + "\n                      " + str.Mid( 105+a );
                if( start > 104 + a )  start += 23;
                if( end > 104 + a )  end += 22;
            }
            else callTip += str;
        }
        else callTip += str;
    }
    
    ///SHOW a DescriptDlg
    if( m_ctDescript ) { m_ctDescript->Destroy(); m_ctDescript = NULL; } ///Clean up any previous
    wxString body = activeCallTips[i].m_ct->m_descripts[ activeCallTips[i].m_ovrldIdx ]; ///Body
    wxString heading = activeCallTips[i].m_ct->m_fullDefs[ activeCallTips[i].m_ovrldIdx ]; ///Heading
    ///Positioning
    int acY = PointFromPosition(GetCurrentPos()).y;  ///TODO - rename acY
    int y = m_ourFrame->GetPosition().y; ///Default to top pos
    int clientOffset = -( ScreenToClient(m_ourFrame->GetPosition()).y ); ///ScreenToClient returns a fixed -ve offset to the Client Area -weird!
    acY = y + clientOffset + acY; ///Xlat to Screen coord
    int parH = m_ourFrame->GetSize().GetHeight();
    ///By default the DescriptDlg is at the top
    if( m_ourFrame->IsFullScreen() ) y += 4;  ///In Vista fullscreen is off the edge
    if( GetCurrentLine() == 0 )  y += parH - 48 - 13;  ///put it to the btm 
    else if( acY > y + parH/2 ) { ///less > half way down
        if( y + parH - acY > 220 )  /// and not within nnn of the btm
            y += parH - 48 - 13;  ///put it to the btm  ///height of descriptDlg is 48
    }
    ///Upper
    else if( m_ourFrame->IsFullScreen() ) y += parH -48 -13;

    ///Do it 2
    m_ctDescript = new DescriptDlg( this, m_ourFrame->GetPosition(), m_ourFrame->GetSize().GetWidth(), heading, body );
    m_ctDescript->SetSize( wxDefaultCoord, y, wxDefaultCoord, wxDefaultCoord );
    m_ctDescript->Show(true);
    SetFocus(); ///Put focus back to the Edit window
    
    ///DO IT
    int lineStart = PositionFromLine( GetCurrentLine() );
    int ctStart =  GetCurrentPos();
    if( ctStart - lineStart + callTip.Len() /*activeCallTips[i].m_ct->m_fullDefs[0].Len()*/ > 130 )
            ctStart = lineStart + 7;  ///TODO way of automating to screen width
    CallTipShow( ctStart, callTip );
    CallTipSetForegroundHighlight( wxColour( 0xB6, 0x00, 0x20 ) );
    CallTipSetBackground( wxColour( 0xFF, 0xFF, 0xD2 ) );
    CallTipSetForeground( wxColour( 0x60, 0x60, 0x60 ) );
    CallTipSetHighlight( start, end );
}

void ZionEdit::OnCallTipClicked( wxStyledTextEvent &event ) ///[Arrow] clicked
{
    if( event.GetPosition() == 999 ) { ///Cancel Sig
        if( m_ctDescript ) { m_ctDescript->Destroy(); m_ctDescript = NULL; } ///TODO - TESTING ///Clean up any previous
    }
    ///Determine the index of the active call tip
    int actIdx=0;
    if( (actIdx=activeCallTips.GetCount() ) <= 0 )  return; ///Safety
    --actIdx; 
    
    if( event.GetPosition() == 0 ) { ///Body of the CT clicked
        ///Display Description
        CallTipNoteDlg noteDlg( this, wxPoint(m_ourFrame->GetPosition().x, m_ourFrame->GetPosition().y ),
                m_ourFrame->GetSize().GetWidth(), activeCallTips[actIdx].m_ct->m_fullDefs[ activeCallTips[actIdx].m_ovrldIdx ],
                activeCallTips[actIdx].m_ct->m_descripts[ activeCallTips[actIdx].m_ovrldIdx ] );
        noteDlg.ShowModal();
    }
    else {
        if( event.GetPosition() == 1 ) { ///Up Arrow
            if( activeCallTips[actIdx].m_ovrldIdx > 0 ) {
                --activeCallTips[actIdx].m_ovrldIdx;
            }
            else activeCallTips[actIdx].m_ovrldIdx = activeCallTips[actIdx].m_numOvrlds - 1;  ///Last idx
        }
        else if( event.GetPosition() == 2 ) { ///Down Arrow
            if( activeCallTips[actIdx].m_ovrldIdx < activeCallTips[actIdx].m_numOvrlds - 1 ) { ///If we are not at the last index
                ++activeCallTips[actIdx].m_ovrldIdx;
            }
            else activeCallTips[actIdx].m_ovrldIdx = 0;
        }
        else return;  ///unknown code'
        
        ///Reset Param Hilite positions for the new overload
        wxArrayInt locs;  ///Stores start and end pos as pairs
        LocateCallTipParams( locs, actIdx );
        activeCallTips[actIdx].m_startPos = 0;  activeCallTips[actIdx].m_endPos = 0;
        if( 2 + 2 * activeCallTips[actIdx].m_paramIdx <= locs.GetCount() ) { ///If the paramIdx is not outside the bounds of current Definition
            activeCallTips[actIdx].m_startPos = locs[ activeCallTips[actIdx].m_paramIdx * 2 ];
            activeCallTips[actIdx].m_endPos = locs[ activeCallTips[actIdx].m_paramIdx * 2 + 1 ];
        }
        ShowCallTips(actIdx); ///Will take care of setting param hilite, arrows,etc.
    }
}

void ZionEdit::OnDumpCallTips( wxCommandEvent & event )
{
    wxMessageDialog warnDialog( this, _T("This function will write to the current Edit buffer.\nAre you sure you want to continue?"), "Write to Edit buffer?",
            wxICON_EXCLAMATION | wxYES_NO );
    if( warnDialog.ShowModal() != wxID_YES )  return;
    AddText( wxString::Format(_T("Top level CallTip File:\n\t%s\n"), syntaxStructs[ m_currentLexer ]->callTipFile ) );
    MemberLists::iterator it;
    Str2StrHash::iterator it2;
/*    AddText( wxString::Format(_T("\r\nCall Tips Enabled for the following %s styles:\n"), syntaxStructs[ m_currentLexer ]->name ) );  ///DIAG
    for( int i= 0; i < syntaxStructs[ m_currentLexer ]->ctEnStyCnt; i++ ) {
        AddText( wxString::Format(_T("%d - %s; "), syntaxStructs[ m_currentLexer ]->ctEnStyles[i] ,
                wxString( syntaxStructs[ m_currentLexer ]->styles[ (syntaxStructs[m_currentLexer]->ctEnStyles[i] ) ].item ) ) );  ///DIAG
    }
*/
//    AddText( wxString::Format(_T("\nwxID_HIGHEST= %d, wxSTC_CMD_WORDRIGHTENDEXTEND= %d, ZION_ABOUT= %d \n"),
//            wxID_HIGHEST, wxSTC_CMD_WORDRIGHTENDEXTEND, ZION_ABOUT ) );
    AddText(_T("________________________________________________________________________\r\nJS STYLE VALUES") );
    AddText("\r\n");
    for( it = g_jsStyleVals.begin(); it != g_jsStyleVals.end(); ++it )  {///iterate
        AddText( it->first + _T("-->") + it->second->m_list + _T("\n"));  ///copy keys and values
    }
    AddText(_T("________________________________________________________________________\r\nHTML TAGS") );
    AddText("\r\n");
    for( it = g_htmlTags.begin(); it != g_htmlTags.end(); ++it )  {///iterate
        AddText( it->first + _T("-->") + it->second->m_list + _T("\n"));  ///copy keys and values
    }
    AddText(_T("________________________________________________________________________\r\nHTML ATTRIBS") );
    AddText("\r\n");
    for( it = g_htmlAttrs.begin(); it != g_htmlAttrs.end(); ++it )  {///iterate
        AddText( it->first + _T("-->") + it->second->m_list + _T("\n"));  ///copy keys and values
    }
    AddText(_T("________________________________________________________________________\r\nSUBSTITUTES/SHORTCUTS") );
    AddText("\r\n");
    for( it2 = g_subs.begin(); it2 != g_subs.end(); ++it2 )  {
        AddText( it2->first + _T("-->") + it2->second + _T("\n"));  ///copy keys and values
    }
    AddText(_T("________________________________________________________________________\r\nCSS ATTRIBS") );
    AddText("\r\n");
    for( it = g_cssAttrs.begin(); it != g_cssAttrs.end(); ++it )  {///iterate
        AddText( it->first + _T("-->") + it->second->m_list + _T("\n"));  ///copy keys and values
    }
    AddText(_T("________________________________________________________________________\r\nSCOPE MEMBERS") );
    AddText("\r\n");
    for( it = g_scopeMems.begin(); it != g_scopeMems.end(); ++it )  {///iterate
        AddText( it->first + _T("-->") + it->second->m_list + _T("\n"));  ///copy keys and values
    }
    AddText(_T("________________________________________________________________________\r\nOBJ MEMBERS") );
    AddText("\r\n");
    for( it = g_objMems.begin(); it != g_objMems.end(); ++it )  {///iterate 
        AddText( it->first + _T("-->") + it->second->m_list + _T("\n"));  ///copy keys and values
    }
    AddText(_T("________________________________________________________________________\r\nUNIQUE FUNCTION NAMES") );
    AddText("\r\n");
    CallTipsHash::iterator it3;
    for( it3 = g_callTips.begin(); it3 != g_callTips.end(); ++it3 )  {
        AddText( it3->first + _T("-->") + it3->second->m_fullDefs[0] + _T("\n"));
    }
    AddText(_T("________________________________________________________________________\r\nAPI Keyword List") );
    AddText("\r\n");
    AddText( apiKWList + _T("\r\n") );
    AddText(_T("________________________________________________________________________\r\nAPI Tag List") );
    AddText("\r\n");
    AddText( apiTagList + _T("\r\n") );
}

void ZionEdit::DoAutoComplete() ///Global, non-object completion
{
    int count=0, a = GetCurrentPos();
    wxString partStr; char c=' '; bool found = false;
    ///Start looking for the beginning of the word
    while( a > 0 ) { ///for safety.
        c = GetCharAt(a-1);
        if( ! ( isalnum( c ) || c == '_'  || c == '-' ) )  break;  ///if not word char. Are '_' and '-' considered alphanumeric?
        count++;  a--;
    }
    if( count > g_numACChars ) {
        for( int j=0; j< count; j++ )  partStr += GetCharAt(a + j); ///Build partStr in fwd direction
        for( unsigned int i=0; i< acArrayList.GetCount(); i++ ) {
            if( acArrayList[i].StartsWith( partStr.c_str() ) )  { found = true; break; }
        }
        if( found ) {
            //Not necessary - done by stc //if( CallTipActive() )  CallTipCancel();
            m_acCurrObj = wxEmptyString;  ///Signify we don't have valid descriptions for AC Shown
            AutoCompShow( count, acList ); ///Don't show till count alphanums or _ typed
        }
    }
    //DIAG// m_ourFrame->SetStatusText( wxString::Format( _T("partStr = \"%s\""), partStr ) );
}

void ZionEdit::DoAmpComplete() {
    PreACShow( &g_hardHash, "Amp", 0 );
}

void ZionEdit::LoadHardHash() { ///Load the hash that is not built from a file ///Called once from the Ctor
    ///HTML SPECIAL CHARS - these must be strictly binary sorted for AC to work
    wxString spCh[] = { "#126;","#35;","#44;","#8355;","#8471;","#8480;","#9774;","#9775;",
"Aacute;","Acirc;","AElig;","Agrave;","Alpha;","Aring;","Atilde;","Auml;","Beta;","Ccedil;","Chi;",
"Dagger;","Delta;","ETH;","Eacute;","Ecirc;","Egrave;","Epsilon;","Eta;","Euml;","Gamma;","Iacute;","Icirc;","Igrave;",
"Iota;","Iuml;","Kappa;","Lambda;","Mu;","Ntilde;","Nu;","Oacute;","Ocirc;","OElig;","Ograve;","Omega;","Omicron;",
"Oslash;","Otilde;","Ouml;","Phi;","Pi;","Psi;","Prime;","Rho;","Scaron;","Sigma;","THORN;","Tau;","Theta;",
"Uacute;","Ugrave;","Ucirc;","Upsilon;","Uuml;","Xi;","Yacute;","Yuml;","Zeta;",
"aacute;","acirc;","acute;","aelig;","agrave;","alefsym;","alpha;","amp;","and;","ang;","aring;",
"asymp;","atilde;","auml;","bdquo;","beta;","breve;","brvbar;","bull;","cap;","ccedil;","cedil;",
"cent;","chi;","circ;","clubs;","cong;","copy;","crarr;","cup;","curren;","dArr;","dagger;","darr;",
"deg;","delta;","diams;","divide;","eacute;","ecirc;","egrave;","empty;","emsp;", "ensp;",
"epsilon;","equiv;","eta;","eth;","euml;","euro;","exist;","fnof;","forall;","frac12;","frac14;",
"frac34;","frasl;","gamma;", "ge;", "gt;","hArr;","harr;",  "hearts;","hellip;","iacute;",
"icirc;","iexcl;","igrave;","image;","infin;","inodot;","int;","iota;","iquest;","isin;","iuml;",
"kappa;","lArr;","lambda;","lang;","laquo;","larr;","lceil;","ldquo;","le;","lfloor;",
"lowast;","loz;","lrm;","lsaquo;","lsquo;","lt;","macr;","mdash;","mirco;","middot;","minus;",
"mu;","nabla;","nbsp;","ndash;","ne;","ni;","not;","notin;","nsub;","ntilde;","nu;","oacute;",
"ocirc;","oelig;","ogon","ograve;","oline;","omega;","omicron;","oplus;","or;","ordf;","ordm;",
"oslash;","otilde;","otimes;","ouml;","para;","part;","permil;","perp;","phi;",
"pi;","piv;","plusmn;","pound;","prime;","prod;","prop;","psi;","quot;","rArr;","radic;","rang;",
"raquo;","rarr;","rceil;","rdquo;","real;","reg;","rfloor;","rho;","rlm;","rsaquo;","rsquo;","sbquo;",
"scaron;","sdot;","sect;","shy;","sigma;","sigmaf;","sim;","spades;","sub;","sube;","sum;","sup;",
"sup1;","sup2;","sup3;","supe;","szlig;","tau;","there4;","thetasym;","thinsp;","thorn;","tilde;",
"times;","trade;","uArr;","uacute;","uarr;","ucirc;","ugrave;","uml;","upsih;","upsilon;","uuml;",
"weierp;","xi;","yacute;","yen;","yuml;","zeta;","zwj;","zwnj;"
        };
    wxString dsc[] = { /// MOST DESCRIPTIONS THANKS TO O'Reilly's "Dynamic HTML - The Definitive Reference" ISBN-13: 978-0-596-52740-2
        ///SOME DESCRIPTIONS THANKS TO http://www.natural-innovations.com/wa/doc-charset.html
        "( ~ ) Ascii Tilde", "( # ) Ascii Number sign","( , ) Ascii Comma", "French dollar - franc","Sound recording copyright (P)",
        "Service mark, (SM)", "Peace Sign", "yin yang",
        "( &Aacute; )  Capital A with acute",
        "( &Acirc; ) Capital letter A with circumflex",
        "( &AElig; ) Capital ligature AE",
        "( &Agrave; ) Capital letter A with grave",
        "Capital letter alpha",
        "Capital letter <b>A</b> with ring",
        "( &Atilde; ) Capital letter A with tilde",
        "( &Auml; ) Capital letter <b>a</b> with diaeresis (umlaut)",
        "Capital letter beta",
        "( &Ccedil; ) Capital C with cedilla",
        "Capital letter chi",
        "Double Dagger",
        "Capital letter Delta","( &ETH; ) Capital ETH", "( &Eacute; ) Capital <b>E</b> with acute",
        "( &Ecirc; ) Capital E with circumflex","( &Egrave; ) Capital <b>E</b> with grave",
        "Capital epsilon","Capital eta","( &Euml; ) Capital E with diaeresis (umlaut)","Capital Gamma",
        "( &Iacute; ) Capital <b>I</b> with accute","( &Icirc; ) Capital I with circumflex","( &Igrave; ) Capital <b>I</b> with grave",
        "Capital iota","( &Iuml; ) Capital I with diaeresis (umlaut)","Capital kappa",
        "Capital letter Lambda;","Capital letter mu","( &Ntilde; ) Capital letter N with tilde","Capital letter nu","( &Oacute; ) Capital letter O with acute",
        "( &Ocirc; ) Capital letter O with circumflex","Capital ligature OE","( &Ograve; ) Capital letter O with grave","Capital letter omega","Capital letter omicron",
        "( &Oslash; ) Capital letter O with slash","( &Otilde; ) Capital letter O with tilde","( &Ouml; ) Capital O with diaeresis (umlaut)","Capital letter phi","Capital letter pi",
        "Capital letter psi","Double prime/seconds/Inches sign","Capital letter rho","Capital S with caron","Capital letter sigma",
        "( &THORN; ) Capital letter thorn","Capital letter tau","Capital letter theta","( &Uacute; ) Capital letter U with acute","( &Ucirc; ) Capital U with circumflex",
        "( &Ugrave; ) Capital U with grave","Capital letter upsilon","( &Uuml; ) Capital U with diaeresis (umlaut)","Capital letter xi","( &Yacute; ) Capital letter Y with acute",
        "Capital letter Y with diaeresis (umlaut)","Capital letter zeta",
        "( &aacute; ) Small letter <b>a</b> with acute",
        "( &acirc; ) Small letter A with circumflex",
        "( &acute; ) Acute accent",
        "( &aelig; ) Small ligature ae",
        "( &agrave; ) Small letter a with grave",
        "Alef symbol",
        "Small letter alpha","( &amp; ) <b>Ampersand</b>","Logical and","Angle",
        "( &aring; ) Small letter <b>a</b> with ring","Almost equal to",
        "( &atilde; ) Small letter <b>a</b> with tilde",
        "( &auml; ) Small letter <b>a</b> with diaeresis (umlaut)","Double low-9 quotation mark",
        "Small letter beta","Breve","( &brvbar; ) Broken vertical bar","Bullet","Intersection",
        "( &ccedil; ) Small c with cedilla",
        "( &cedil; ) Cedilla","( &cent; ) Cent sign",
        "Small letter chi","Modifier letter circumflex accent","Black club suit (shamrock)","Approximately equal to",
        "( &copy; )  the copyright symbol","Carriage return or Enter", "Union","( &curren; ) Currency","Double down arrow","Dagger",
        "Down arrow","( &deg; ) Degree",
        "Small letter Delta","Black diamond","( &divide; ) Division sign",
        "( &eacute; ) Small <b>E</b> with accute","( &ecirc; ) Small <b>e</b> with circumflex",
        "( &egrave; ) Small <b>e</b> with grave", "Zero with slash - Empty set / diameter",
        "Extra wide space", "Narrow space", "Small epsilon","Identical to","Small eta",
        "( &eth; ) Small eth", "( &euml; ) Small e with diaeresis (umlaut)", "European dollar", "There exists",
        "Small f with serif (florin)","For all","( &frac12; ) Fraction - half","( &frac14; ) Fraction - quarter","( &frac34; ) Fraction - three quarters",
        "Fraction slash","Small gamma","Greater-than or equal to","( &gt; ) Greater-than",
        "Horizontal double head double shank arrow","Horizontal double head arrow",
         "Black heart", "Horizontal ellipsis","( &iacute; ) Small <b>i</b> with acute",
        "( &icirc; ) Small i with circumflex", "( &iexcl; ) Inverted exclamation mark","( &igrave; ) Small i with grave","Blackletter uppercase I",
        "Infinity symbol","Letter <b>i</b> with no dot","Integral sign", "Small iota","( &iquest; ) Inverted question mark","Element of",
        "( &iuml; ) Small I with diaeresis (umlaut)","Small letter kappa","left pointing double arrow","Small letter Lambda","Left pointing angle bracket",
        "( &laquo; ) Left pointing double angle quotation mark (guillemet)","left pointing arrow","Left upper corner",
        "Left double quotation mark","Less-than or equal to","Left lower corner","Asterisk","Diamond outline (Lozenge)", "Left-to-right mark",
        "Single left-pointing angle quotation","Forward leaning single quotation mark","Less-than","Macron (overline)",
        "Em dash - a wide dash","Micro sign","Georgian comma (dot)","Minus sign","Small letter mu","Nabla",
        "Non-breaking space","En dash - a narrow dash","Not equal to","Contains as member", "Not sign (discretionary hyphen)",
        "Not an element of","Not a subset of","( &ntilde; ) Small letter n with tilde","Small letter nu",
        "( &oacute; ) Small letter o with acute","( &circumflex; ) Small letter o with circumflex",
        "Small ligature oe","ogonek","( &ograve; ) Small letter o with grave","overline",
        "Small letter omega","Small letter omicron","Circled plus","Logical Or","Feminine ordinal indicator",
        "Masculine ordinal indicator","( &slash; ) Small letter o with slash",
        "( &otilde; ) Small letter o with tildle", "times symbol in circle","( &ouml; ) Small O with diaeresis (umlaut)","( &para; ) Paragraph (pilcrow) sign",
        "Partial differential","Per thousand (mille) sign", "Inverted T (orthagonal/perpendicular) sign","Small letter phi",
        "Small letter pi","symbol","( &plusmn; ) Plus-or-minus sign","( &pound; ) Pound Sterling","Prime/Minutes/Feet sign",
        "N-ary product (product sign)","Proportional to","Small letter psi","( &quot; ) Double Quotation mark","Right-pointing double arrow","Square Root",
        "Right-pointing angle bracket (ket)","( &raquo; ) Right-pointing double angle quotation mark (guillemet)","Right pointing arrow",
        "Upper right corner","Right double quotation mark","Blackletter uppercase R","( &reg; ) Registered trademarksign",
        "Lower right corner","Small letter rho","Right-to-left mark","Single right-pointing angle quotation mark",
        "Right single quotation mark","Single low-9 quotation mark","Small s with caron","Dot","( &sect; ) Section sign",
        "( &shy; ) Soft hypen (discretionary hyphen)","Small letter sigma","Small letter <i>final</i> sigma","Tilde operator",
        "Black spade suit","Subset of","Subset of or equal to","N-ary Sumation","Superset of","( &sup1; ) Superscript 1","( &sup2; ) Superscript 2 (squared)",
        "( &sup3; ) Superscript 3 (cubed)", "Superset of or equal to","( &szlig; ) Small letter sharp s (ess-zed)","Small letter tau",
        "Small letter theta","Small letter theta - the symbol", "Thin space","( &thorn; ) Small letter thorn",
        "Small tilde","( &times; ) Multiplication sign","Trademark sign","Double Up Arrow","( &uacute; ) Small letter u with acute","Up Arrow",
        "( &ucirc; ) Small letter u with circumflex","( &ugrave; ) Small u with grave","( &uml; ) diaeresis (umlaut)","upsilon with hook symbol",
        "Small letter upsilon","( &uuml; ) Small u with diaeresis (umlaut)","Script capital P",
        "Small letter xi","( &yacute; ) Small letter Y with acute","( &yen; ) Yen/yuan sign",
        "( &yuml; ) Small letter y with diaeresis (umlaut)","Small letter zeta","Zero width joiner","Zero width nonjoiner"
    }; ///wxString[] arrays cannot be automatically counted so adj the number below if new strings are added
    g_hardHash["Amp"] = new MemberList;
    ///Build List - must be binary sorted
    for( int j=0; j< 262; j++ ) { //251
        g_hardHash["Amp"]->m_list += spCh[j] + " ";
        g_hardHash["Amp"]->m_descrip[ spCh[j] ] = dsc[j];
    }
}

void ZionEdit::OnAutoCompleteShow( wxCommandEvent & event ) ///Ctrl-J
{
    m_ourFrame->PrependToPopup( event.GetId() ); ///LAST CMD to popup
    int count=0, a = GetCurrentPos(); char c=' ';
    while( a > 0 ) {
        c = GetCharAt(a-1);
        if( ! ( isalnum( c ) || c == '_'  || c == '-' ) )  break;
        count++;  a--;
    }
    if( count == 0 ) return; ///nothing to look up
    m_acCurrObj = wxEmptyString;  ///Signify we don't have valid descriptions for AC Shown
    AutoCompShow( count, acList );
}

void ZionEdit::OnSelectionToggle( wxCommandEvent & )
{
    SetSelectionMode(wxSTC_SEL_RECTANGLE);
}

void ZionEdit::OnResultsClicked( myFindResultsEvent &event )
{
    //GotoPos( GetCurrentLine() );  ///Force Scroll screen to horiz start
    LineScroll( -100, 0 );  ///Reset Horizontal position
    int pos = PositionFromLine(event.GetY() -1) + event.GetX() -1; ///start pos // -1 for zero base from unity base
    if(pos >= 0 && pos < GetLength() ) { //! Safety
        EnsureVisible(pos); /*GotoLine( event.GetY() - 1 );  GotoPos(pos); */  ///Wrapmode is having a problem with positions, hence GotoLine()
        SetSelection( pos, pos - (event.GetX() -1) + event.GetZ() - 1 ); /// Move back to start of line then add the end position /// selection will move the current pos
        ScrollFromEdge(); ///Position away from vertical edges of screen
    }
//DIAG // m_ourFrame->SetStatusText( wxString::Format(_T("L: %d, St: %d, End: %d, StartPos: %d, EndPos: %d"),
//            event.GetY(),event.GetX(), event.GetZ(), pos, pos - (event.GetX() -1) +event.GetZ() - 1) );
//    /// Send out an update.
//    m_ourFrame->m_advisedData = m_findReplData.GetFindString();
//    m_ourFrame->NotifyFindUpdate();
}

void ZionEdit::OnResultsClosing( myFindResultsEvent &WXUNUSED(event) )
{
    if(m_fresDialog) m_fresDialog->Destroy();
    m_fresDialog = NULL;
    m_findReplData.m_results.Clear();
    /// Send out an update.
    m_ourFrame->m_advisedData = m_findReplData.GetFindString();
    m_ourFrame->NotifyFindUpdate();
    ShowPosition();
}

void ZionEdit::OnGotoBraceMatch( wxCommandEvent & event)
{
    m_ourFrame->PrependToPopup( event.GetId() ); ///LAST CMD to popup
    int a, currPos = GetCurrentPos();
    if( (a = BraceMatch(currPos))  != -1 )  GotoPos(a);  //! If there is a match, go to it
    else if( (a = BraceMatch(currPos -1))  != -1 )  GotoPos(a);  ///As in the case of an "as you type" bracehilite
}
//------------------------------------------------------------------
void ZionEdit::OnSettings(wxCommandEvent & WXUNUSED(event)) { DoSettings(); }

void ZionEdit::OnChangeStyle(wxCommandEvent & WXUNUSED(event))
{
    int styleByte = 0;
    styleByte =  GetStyleAt(PositionFromPoint(zion_contextMenuPos)); //! Get Sytle number
    DoSettings(styleByte);
}

void ZionEdit::DoSettings( int stylenum ) /// -1 = unset, i.e. no stylenum specified
{
    if( ! m_ourFrame->RequestStyleLock() ) return; //! Get StyleLock
    //CheckAllowSave(); //! No longer mandatory -> //File doesn't need to be saved
//    ///Make sure '*' removed if any because of possible "clean" reload after Settings
//    wxString str = m_ourFrame->GetTitle();
//    if( str.GetChar(0) == '*' ) {  str.Remove( 0, 1 ); m_ourFrame->SetTitle(str);  }
    AutoCompCancel(); ///Make sure autocomp is off
    wxString str;

    SettingsDialog * sd = new SettingsDialog( this, stylenum, m_currentLexer,
    wxPoint(m_ourFrame->GetPosition().x, m_ourFrame->GetPosition().y ), m_ourFrame->GetSize().GetWidth() );
    if( sd->ShowModal() == wxID_OK ) {
        int a = wxSTC_PRINT_INVERTLIGHT;
        m_settingsDirty = true;  //! Assume settings have been changed
        g_caretColor = sd->m_generalOptsPanel->m_caretColor;
//        g_selColor = sd->m_generalOptsPanel->m_selColor;

        ///Print Color Mode
        str = sd->m_generalOptsPanel->m_pcolMode->GetValue();
        if(str==_T("As on screen")) a = wxSTC_PRINT_NORMAL;
        //default if(str==_T("Invert Colors")) a = wxSTC_PRINT_INVERTLIGHT;
        if(str==_T("Black on white")) a = wxSTC_PRINT_BLACKONWHITE;
        if(str==_T("All Backgrounds to white")) a = wxSTC_PRINT_COLOURONWHITE;
        if(str==_T("Default Background to white")) a = wxSTC_PRINT_COLOURONWHITEDEFAULTBG;
        SetPrintColourMode(a);
        ///Recent Files
        long b;
        str = sd->m_generalOptsPanel->m_recentFiles->GetValue();
        str.ToLong( &b );  ///put it in b
        g_numTopRecentFiles = (int) b;
        ///Recent Dirs
        str = sd->m_generalOptsPanel->m_recentDirs->GetValue();
        str.ToLong( &b );  ///put it in b
        g_numRecentDirs = (int) b;
        
        ///DontPrintLineNums
        m_dontPrintNums = sd->m_generalOptsPanel->m_dontPrintNums->GetValue();
        
        ///Line Num Width
        sd->m_generalOptsPanel->m_lineNumWidth->GetValue().ToLong(&b);
        g_lineNumWidth = (int) b;
        
        ///Startup Lexer
        str = sd->m_generalOptsPanel->m_startLex->GetValue();
        for( int i=0; i < lenLNFN; i++) {
            if( lexerNameFromNum[i] == str ) {  m_startLex = i; break; }
        }
        ///Initial Zoom Level
        g_initialZoom = sd->m_generalOptsPanel->m_zoom->GetValue();
        ///PathToBrowser
        g_pathToBrowser = sd->m_generalOptsPanel->m_pathToBrowser->GetValue();
        ///PathToApi
        g_pathToApi = sd->m_generalOptsPanel->m_pathToApi->GetValue();
        
        ///Call Tips
        g_ctActive = sd->m_generalOptsPanel->m_ctCheckBox->GetValue();
        
        ///Auto Complete
        g_acActive = sd->m_generalOptsPanel->m_acCheckBox->GetValue();
        sd->m_generalOptsPanel->m_numACChars->GetValue().ToLong(&b);
        g_numACChars = (int) b;
        
        ///Auto Bookmarks
        g_autoBook = sd->m_generalOptsPanel->m_autoBook->GetValue();
        ///Auto Zoom
        g_autoZoom = sd->m_generalOptsPanel->m_autoZoom->GetValue();
        
        ///Default Startup Size and Positioning
        if( sd->m_generalOptsPanel->m_rbNorm->GetValue() )   m_startOpt = 0;
        if( sd->m_generalOptsPanel->m_rbMax->GetValue() )   m_startOpt = 1;
        if( sd->m_generalOptsPanel->m_rbCustom->GetValue() )   m_startOpt = 2;
        long stTop, stLeft, stWidth, stHeight;
        sd->m_generalOptsPanel->m_top->GetValue().ToLong(&stTop);
        sd->m_generalOptsPanel->m_left->GetValue().ToLong(&stLeft);
        sd->m_generalOptsPanel->m_width->GetValue().ToLong(&stWidth);
        sd->m_generalOptsPanel->m_height->GetValue().ToLong(&stHeight);
        m_startPos = wxPoint( stLeft, stTop );
        m_startSize = wxSize( stWidth, stHeight );
        
        ///Extensions
        ext2lex.clear();
        stringToInt::iterator it;
        for( it = sd->m_extPanel->m_e2lcopy.begin(); it != sd->m_extPanel->m_e2lcopy.end(); ++it ) ///iterate over settings dialog's copy of the hashmap
            ext2lex[it->first] = it->second;  ///copy keys and values
        
        str = sd->m_generalOptsPanel->m_eolMode->GetValue();
        if(str=="Windows") m_eolDefault= wxSTC_EOL_CRLF; /// (full access to parent's protected members)
        if(str=="Mac") m_eolDefault= wxSTC_EOL_CR;
        if(str=="Unix") m_eolDefault= wxSTC_EOL_LF;
        if( sd->m_generalOptsPanel->m_eolComboChanged && m_eolDefault != GetEOLMode() ) { ///CombChanged and current mode different from default?
            if( wxMessageBox(_T("Convert Line Endings of the current file?"),
                    _T("Convert Current File?"), wxICON_QUESTION | wxYES_NO ) == wxYES )
                SetEOLMode( m_eolDefault );  ShowEOLMode();
        }
        ///CppWord List
        if( sd->m_cppWordsChanged ) {
            g_cppWords2 = sd->m_cppWords2;
            cppSyntax.words2 = g_cppWords2.c_str();  ///Pointer to the string data in the global returned.
        }
        ///CS Word List
        if( sd->m_csWordsChanged ) {
            g_csWords2 = sd->m_csWords2;
            csSyntax.words2 = g_csWords2.c_str();  ///Pointer to the string data in the global returned.
        }
        ///SQL Word List
        if( sd->m_sqlWordsChanged ) {
            g_sqlWords2 = sd->m_sqlWords2;
            sqlSyntax.words6 = g_sqlWords2.c_str();  ///Pointer to the string data in the global returned.
        }
        ///JavaWord List
        if( sd->m_javaWordsChanged ) {
            g_javaWords2 = sd->m_javaWords2;
            javaSyntax.words2 = g_javaWords2.c_str();  ///Pointer to the string data in the global returned.
        }
        
        defaultSyntax.pageColor = StringToColor(defaultSyntax.styles[0].back);  ///Synch background color for text mode
        ///RELOAD
        if( GetWrapMode() != wxSTC_WRAP_NONE ) {
            ReloadFromBuffer( m_currentLexer, GetCurrentPos(), true ); ///Eventually sets up lexer too ///force lexer setup since we want our changes to take place immediately
        }
        else SetupLexer(m_currentLexer, true); ///force lexer setup
    } ///End OK
    
    //! We maintain StyleLock until our changes are saved unless, of course, we cancel
    else { //! Release Style Lock 
        if( m_ourFrame->m_ipcOk && ! zserver && zconnection )  { //! Overkill but safety
            zconnection->Poke( _T("ReleaseStyleLock"), _T("") ); m_ourFrame->m_styleLock = false;}
        else if( zserver ) m_ourFrame->m_styleLock = false;
        m_ourFrame->SetStatusText(_T("")); /// Clean up "StyleLock" messages
    }
    sd->Destroy(); //! No further need to reference the dialog
    m_ourFrame->SetStatusText(_T(""));
}

void ZionEdit::ShowEOLMode()
{
    wxString eolstr;
    int eolMode = GetEOLMode();
    switch( eolMode ) { 
        case wxSTC_EOL_CRLF:    eolstr = _T("Win");  break;
        case wxSTC_EOL_CR:       eolstr = _T("Mac");  break;
        case wxSTC_EOL_LF:        eolstr = "Unix";  break;
        default:    eolstr = "?";
    }
    m_ourFrame->SetStatusText(eolstr, 3);
    ShowPosition();
//    wxString str; int line = GetCurrentLine();
//    str.Printf(_T("%d: %d"), line+1, GetCurrentPos() - PositionFromLine(line) +1 );
//    m_ourFrame->SetStatusText( str , 4 );
}

void ZionEdit::ContextMenuStuff()
{   //! Events are piped to our main frame first!
    ///Append manual add
    bookMenu->Append( ZION_ADDBOOKMARK, _T("Add Bookmark") );
    bookMenu->AppendSeparator(); ///New bookmarks will be added below
    bookMenu->AppendSeparator();
    bookMenu->Append( ZION_CLEARBOOKMARKS, _T("Clear Bookmarks") );

    m_popup->Append(ZION_CUT, _T("Cut"));
    m_popup->Append(ZION_COPY, _T("Copy"));
    m_popup->Append(ZION_PASTE, _T("Paste"));
	m_popup->Append(ZION_BOOKMARKS, _T("Bookmarks"), bookMenu);
	m_popup->Append(ZION_RECENTDIRS, _T("Recent"), m_ourFrame->recentDirsMenu);
	m_popup->Append(ZION_WINDOWS, _T("Window"), windowMenu );
    m_popup->AppendSeparator();
    m_popup->Append(ZION_TOGGLE, _T("Toggles"), m_ourFrame->togglesMenu );
//    findSubMenu->Append(ZION_FIND, _T("Quick Find...\tCtrl+F"), _T("Ctrl+F, The standard Find Dialog"));
//    findSubMenu->Append(ZION_FINDALL, _T("Find All...\tF2"), _T("F2 - Find and List all ocurrences - Preselect for immediate listing"));
//    findSubMenu->Append(ZION_FINDNEXT, _T("Find Next\tF3"), _T("Search Backwards for the hilighted or previously found selection"));
//    findSubMenu->Append(ZION_FINDPREV, _T("Find Previous\tF4"), _T("Search Backwards for the hilighted or previously found selection"));
//    findSubMenu->Append(ZION_FINDREPLACE, _T("Find/Replace...\tCtrl+R"), _T("Open Find/Replace Dialog"));
//    findSubMenu->Append(ZION_FINDALLFILES, "Find in All Files...\tF6", _T("Find + List all Files containing a pattern (Preselect for immediate listing)"));
    m_popup->Append(ZION_EDITMENU, _T("Edit"), m_ourFrame->editMenu);
    m_popup->Append(ZION_GOTO, _T("Go to"), m_ourFrame->gotoMenu );
    m_popup->Append( ZION_SPECIALOPS, _T("SpecialOps"), m_ourFrame->editSpecial );
    m_popup->AppendSeparator();
    m_popup->Append(ZION_STYLE, _T("Change Style"));
    m_popup->AppendSeparator();
    m_popup->Append(ZION_FILEMENU, _T("File"), m_ourFrame->fileMenu);
    m_popup->Append(ZION_SAVE, _T("Save"));
}

bool ZionEdit::ProcessSel( int * a, int * b, bool lesser, bool select ) ///return true if there is a selection, a = holder for lower selected value, b= holder for higher,
///lesser if we want to 
{
    int e=0, f=0;
    if( (e=GetSelectionStart()) == (f=GetSelectionEnd()) )  return false;  ///Nothing selected
    ///Beyond this point we always return true because we have a selection
    if( a == NULL || b == NULL ) return true;
    ///We always want a to be the lower value
    if( e < f ) { *a = e; *b = f;  }
    else { *a = f; *b = e;  } 

    if( !select) { GotoPos( lesser? e : f ); return true; } ///Don't bother selecting, just go to the lower position
    ///We want to be selected beyond this point
    if(! lesser) SetSelection( e, f ); ///forward select to leave cursor at higher value.
    else SetSelection( f, e ); ///backwards select when we want to leave cursor at lower value
    return true;
}

void ZionEdit::OnZoomIn (wxCommandEvent &event) {
    m_ourFrame->PrependToPopup( event.GetId() ); ///LAST CMD to popup
    CmdKeyExecute(wxSTC_CMD_ZOOMIN);
}

void ZionEdit::OnZoomOut (wxCommandEvent &event)
{
    m_ourFrame->PrependToPopup( event.GetId() ); ///LAST CMD to popup
    CmdKeyExecute(wxSTC_CMD_ZOOMOUT);
}

void ZionEdit::OnResetZoom(wxCommandEvent &WXUNUSED(event))
{
    SetZoom(0);
}

void ZionEdit::OnCut (wxCommandEvent &WXUNUSED(event)) {
    if (GetReadOnly() || GetSelectedText() == wxEmptyString) return;
    CmdKeyExecute(wxSTC_CMD_CUT);
}

void ZionEdit::OnCopy (wxCommandEvent &WXUNUSED(event)) {
    if (GetSelectedText() == wxEmptyString) return;
    CmdKeyExecute(wxSTC_CMD_COPY);
}

void ZionEdit::OnPaste(wxCommandEvent &WXUNUSED(event)) {
    if (!CanPaste()) return;
    CmdKeyExecute(wxSTC_CMD_PASTE);
}

void ZionEdit::OnPopup( wxContextMenuEvent & event )
{
    //! Show context menu at event point if it's within the window, or at caret location if not
    wxPoint pt = event.GetPosition();
    ScreenToClient(&pt.x, &pt.y);
    wxHitTest ht = this->HitTest(pt);
    if (ht != wxHT_WINDOW_INSIDE) {
        pt = this->PointFromPosition(this->GetCurrentPos());
    }
    zion_contextMenuPos = pt; //! Save it for "Style at Cursor"
    //! UIupdate
    if( CanPaste() )    m_popup->Enable(ZION_PASTE, true);
    else m_popup->Enable(ZION_PASTE, false);
    
    if(GetSelectedText() == wxEmptyString) {
        m_popup->Enable(ZION_COPY, false);
        m_popup->Enable(ZION_CUT, false);
    }
    else {
        m_popup->Enable(ZION_COPY, true);
        m_popup->Enable(ZION_CUT, true);
    }
    ///PREPARE WINDOWS
//    ///Remove Window menu
//    int menuID = m_popup->FindItem(_T("Window")); //! menu id returned
//    if( menuID != wxNOT_FOUND )  m_popup->Remove( menuID );
                
    size_t n=0, r=0;  wxString str;
    
    ///CLEAN Bookmark Menu
    r = bookMenu->GetMenuItemCount() - 4; /// 2x2 for "Manual Append" item and Separator and Clear bookmarks
    for( n = 0 ; n< r ; n++ ) bookMenu->Delete( ZION_BOOK_1 + n ); //! Remove all window items
    
    r = m_bookIdx.GetCount();
    for( n=0; n<r; n++ ) {
        bookMenu->Insert( 2, ZION_BOOK_1 +n, m_bookmarks[n] );
    }
    
    ///CLEAN Window Menu
    r = windowMenu->GetMenuItemCount();
    for( n = 0 ; n< r ; n++ ) windowMenu->Delete( ZION_WIN_1 + n ); //! Remove all window items
               
    ///Get the window List
    str = m_ourFrame->MiniStats();
    //Diag //AppendText(_T("\n----------------------------------------------------------------------\n") + str );
    if( str != _T("") ) {
        ///Process List
        wxStringTokenizer tknzr( str, _T("\n"), wxTOKEN_STRTOK ); ///Specify new line delim and empty tokens never returned
        wxString str3, str4, str5;
        n = 0; ///We deliberately use a while here instead of a for so our continue will not increment n
        while( n < NUMWINDOWS && tknzr.HasMoreTokens() ) {  ///SAFETY check on n or will lock up
            str3 = tknzr.GetNextToken();
            if( str3.StartsWith(_T("S") ) ) continue;  /// "Server..." ///We don't want to increment n here
            if( str3.Find( _T(" - StyleLock") ) != -1 )  str3 = str3.Mid( 0, str3.Len() - 12 );  ///Chop 12 off the end
            ///Put the filename first
            if( str3.Find( '\\' ) != -1 )  {
                str4 = str3.AfterLast('\\');  ///If not "Untitled" pick up Filename
                if( str4.Last() == ']' ) str4.Prepend(_T("[")); ///Readonly indicator
            }
            else  str4 = _T("Untitled");
            str4 += _T(" - ");
            if( str != _T("Untitled - ") ) {
                str5 = str3.AfterFirst(' ').BeforeLast('\\'); ///Path
                if( str5.StartsWith(_T("[")) )  str5.Remove( 0, 1 ); ///At pos 0 remove 1 char
                str4 += str5;
            }
            str4 += _T("  "); ///This spc is critical
            str4 += str3.BeforeFirst(' '); ///Pickup process id
            ///Append to Window Menu
           wxMenuItem * mi = windowMenu->AppendCheckItem( ZION_WIN_1 +n, str4 );
            long itemPid; str3.BeforeFirst(' ').ToLong(&itemPid);
            if( itemPid == localPid )  mi->Check() ;  ///Done after append //localPid is a global defined in zion.cpp
            n++;
        }
    }

    /// Do it 
    m_ourFrame->PopupMenu(m_popup, pt); //! wxWindow->PopupMenu
}

void ZionEdit::OnLineUnComment(wxCommandEvent & event)
{
    m_ourFrame->PrependToPopup( event.GetId() ); ///LAST CMD to popup
    wxString eolstr, comment;    wxString str = GetSelectedText();
    switch( GetEOLMode() ) {
        case wxSTC_EOL_CRLF:    eolstr = "\r\n";  break;
        case wxSTC_EOL_CR:       eolstr = "\r";  break;
        case wxSTC_EOL_LF:        eolstr = "\n";  break;
        default:    return;
    }
    switch( event.GetId() ) {
        case ZION_CPPMENU_LINEUNCOMMENT:  comment = "//"; break;
        case ZION_PERLMENU_LINEUNCOMMENT:  comment = "#"; break;
        case ZION_VBMENU_LINEUNCOMMENT:  comment = "'"; break;
        case ZION_SQLMENU_LINEUNCOMMENT:  comment = "--"; break;
        case ZION_ASMMENU_LINEUNCOMMENT:  comment = ";"; break;
        case ZION_FORTRANMENU_LINEUNCOMMENT:  comment = "!"; break;
        case ZION_FORTRAN77MENU_LINEUNCOMMENT:  comment = "c"; break;
        case ZION_BATCHMENU_LINEUNCOMMENT:  comment = "REM "; break;
        case ZION_BATCHMENU_LINEUNCOMMENT2:  comment = "::"; break;
        default: return;
    }
    
    int a, b; ///Hold initial Selection range. As we delete chars we will decrease b
    if( ProcessSel( &a, &b, false, false ) ) {
        int currentLine = LineFromPosition( a );
        do {
            if( GetCharAt( PositionFromLine(currentLine) ) == comment.GetChar(0) ) {
                GotoPos( PositionFromLine(currentLine) + comment.Length() );  //! It seems we can only delete backwards
                for( int i=0; i< comment.Length(); i++) {
                   CmdKeyExecute( wxSTC_CMD_DELETEBACKNOTLINE ); 
                }
                b -= comment.Length();
            }
            currentLine++;
        } while( PositionFromLine(currentLine) < b - comment.Length() /* -1 */ ); /// Removed "-1" in case selection is just past comments
        
   }
    else {
        wxMessageDialog resultDialog(this, "Please select a line commented block first", "Select First", wxOK );  //! Debug
        resultDialog.ShowModal();
    }
}

void ZionEdit::OnLineComment(wxCommandEvent & event)
{
    m_ourFrame->PrependToPopup( event.GetId() ); ///LAST CMD to popup
    
    wxString comment; int start;
    switch( event.GetId() ) {
        case ZION_CPPMENU_LINECOMMENT:  comment = "//"; break;
        case ZION_PERLMENU_LINECOMMENT:  comment = "#"; break;
        case ZION_VBMENU_LINECOMMENT:  comment = "'"; break;
        case ZION_SQLMENU_LINECOMMENT:  comment = "--"; break;
        case ZION_ASMMENU_LINECOMMENT:  comment = ";"; break;
        case ZION_FORTRANMENU_LINECOMMENT:  comment = "!"; break;
        case ZION_FORTRAN77MENU_LINECOMMENT:  comment = "c"; break;
        case ZION_BATCHMENU_LINECOMMENT:  comment = "REM "; break;
        case ZION_BATCHMENU_LINECOMMENT2:  comment = "::"; break;
       default: return;
    }
    
    ///TODO: Optimize with ProcessSel. Make sure currentPos restored for "else"
    if( GetSelectionStart() != GetSelectionEnd() ) {
        if( (start = GetAnchor() ) > GetCurrentPos() ) { //! If backwards selection, swap //Think: we drop the anchor then start selecting
            SetSelectionStart(GetCurrentPos());
            SetSelectionEnd(start);
        }
        
        ///New Concept: use selection just as a range. As long as the start of the next line is < the Selection End, add comments to start of line.
        /// !!!Insert will automatically extend the selection, so we will use that property
         int currentLine = LineFromPosition( GetSelectionStart() );
        do {
            InsertText( PositionFromLine(currentLine), comment );
            ++currentLine;
        } while ( PositionFromLine(currentLine) < GetSelectionEnd());
        
    }
    else InsertText( GetCurrentPos(), comment );
}

void ZionEdit::OnIncRedIndent( wxCommandEvent & event)
{
    m_ourFrame->PrependToPopup( event.GetId() ); ///LAST CMD to popup
    wxString str = GetSelectedText();  wxCommandEvent anEvent;
    switch( GetEOLMode() ) {
        case wxSTC_EOL_CRLF:
            if( str.Find("\r\n") == -1 ) OnSelectLine(anEvent); //! Couldn't find EOL char(s) in selected text so select the WHOLE line;
            break;
        case wxSTC_EOL_CR:
            if( str.Find("\r") == -1 ) OnSelectLine(anEvent);
            break;
        case wxSTC_EOL_LF:
            if( str.Find("\n") == -1 ) OnSelectLine(anEvent);
            break;
        default:    return;
    }
    if( event.GetId() == ZION_INCINDENT )   CmdKeyExecute(wxSTC_CMD_TAB);
    else if( event.GetId() == ZION_REDINDENT )   CmdKeyExecute(wxSTC_CMD_BACKTAB);
}

void ZionEdit::OnUpperLowerCase( wxCommandEvent & event)
{
    m_ourFrame->PrependToPopup( event.GetId() ); ///LAST CMD to popup
    if( ! ProcessSel() )
        m_ourFrame->SetStatusText("SELECT SOME TEXT FIRST!");
    else if( event.GetId() == ZION_UPPERCASE ) CmdKeyExecute(wxSTC_CMD_UPPERCASE);
    else if( event.GetId() == ZION_LOWERCASE ) CmdKeyExecute(wxSTC_CMD_LOWERCASE);
}

void ZionEdit::OnCppForIf(wxCommandEvent & event)
{
    m_ourFrame->PrependToPopup( event.GetId() ); ///LAST CMD to popup
    
    int id = event.GetId();
    bool vb = ( id == ZION_VBMENU_IF || id == ZION_VBMENU_ELSE );
    int savePos = GetCurrentPos(); /// We should already be at the insertion point
    wxString topString;  int finalOffset=0, d=0;
    switch(id) { /// _T("") is killing my space here. Don't use
        case ZION_CPPMENU_ELSE: finalOffset = 5; topString = "else "; break;
        case ZION_VBMENU_ELSE: finalOffset = 5; topString = "Else "; break;
        case ZION_CPPMENU_FOR: finalOffset = 5; topString = "for( ; ; ) "; break; 
        case ZION_CPPMENU_IF: finalOffset = 4; topString = "if(  ) "; break;
        case ZION_VBMENU_IF: finalOffset = 3; topString = "If  Then"; break;
        case ZION_CPPMENU_ELSEIF: finalOffset = 9; topString = "else if(  ) "; break;
        case ZION_PERLMENU_ELSIF: finalOffset = 7; topString = "elsif(  ) "; break;
        case ZION_PERLMENU_FOREACH: finalOffset = 8; topString = "foreach  "; break;
        case ZION_CPPMENU_WHILE: finalOffset = 7; topString = "while(  ) "; break;
        case ZION_CPPMENU_DOWHILE: topString = "do "; break;
    }
    int a, b;
    if( ProcessSel( &a, &b, false, false ) ) { ///don't reverse, don't select
        GotoPos(a);
        a = PositionFromLine( GetCurrentLine() );
        int c, e= GetLineEnd(); bool found=false;  /// c will hold position at end of indent
        ///We could optimize to use GetLineIndentation(), but we will keep this because of the return capability
        for( c=a; c < e; c++) {
            if( GetCharAt(c) != ' ' && GetCharAt(c) != '\t' )  /// if non-whitespc
                { found =true; break; }
        }
        if( ! found )  return; ///There should be a non-whitespc char on the first line!
        /// c now holds the absolute column start location
        SetSelection( a, b );
        CmdKeyExecute(wxSTC_CMD_TAB);
        ///Prepare top string
        wxString str;
        str.Append( ' ', c - a ); ///Create the indent -pad with spaces
        if( ! vb ) str.Append( topString + "{\n" );
        else str.Append( topString + "\n" );
        InsertText( a, str );  ///InsertText should extend our selection
        ///Prepare lower text
        b = GetSelectionEnd();
        str.Empty();
        str.Append( ' ', c - a ); ///pad with spaces (this is our indent)
        switch(id) {
            case ZION_CPPMENU_DOWHILE: str.Append("}while(  );\n"); break;
            case ZION_VBMENU_IF:
            case ZION_VBMENU_ELSE:    str.Append("End If\n"); break;
            default:    str.Append( "}\n" );
        }
        InsertText( b, str );
        GotoPos( id == ZION_CPPMENU_DOWHILE ? ( b +   c - a   + 8 ) : (c + finalOffset) );  /// c-a is our indent
    }
    else { /// No Selection
        InsertText( savePos, topString );   //! Insert Text does not update positions! But, it does (should) extend selections
        switch(id) { /// Position for insertion of first curly brace
            case ZION_CPPMENU_ELSE: d = 5; break;
            case ZION_CPPMENU_FOR: d = 11; break; 
            case ZION_CPPMENU_IF: d = 7; break;
            case ZION_CPPMENU_ELSEIF: d = 12; break;
            case ZION_CPPMENU_WHILE: d = 10; break;
            case ZION_CPPMENU_DOWHILE: d = 3; break;
            case ZION_PERLMENU_FOREACH: d = 9; break;
            case ZION_PERLMENU_ELSIF: d = 10; break;
        }
        GotoPos( savePos + d ); ///ProcessSel may have shifted things
        if( ! vb ) {
            wxCommandEvent anEvent;
            OnCppCurly( anEvent );  ///Will leave position at one line greater than start if no selection
        }
        else { ///vb
            int lineIndent = GetLineIndentation( GetCurrentLine() );
            InsertText( GetLineEnd(), "\nEnd If" );
            SetLineIndentation( GetCurrentLine() + 1, lineIndent );
        }
        ///Do While will return to a special pos
        if( id == ZION_CPPMENU_DOWHILE ) {
            int lineIndent = GetLineIndentation( GetCurrentLine() + 1 );
            int pos = PositionFromLine( GetCurrentLine() + 1 ) + lineIndent +1; ///Line of last curly
            InsertText( pos, "while(  );" );
            GotoPos( pos + 7 );
        }
        else GotoPos( savePos + finalOffset ); 
    }
}

void ZionEdit::OnCppCurly(wxCommandEvent & event)
{
    //This event called from other events. m_ourFrame->PrependToPopup( event.GetId() ); ///LAST CMD to popup
    int savePos = GetCurrentPos();
    int a, b;
    if( ProcessSel( &a, &b, true, false ) ) {
        InsertText( a, _T("{") ); InsertText( b+1, _T("}") );
        return;
    }
    GotoPos( savePos );
    int currentLine = GetCurrentLine();
    int lineIndent = GetLineIndentation(currentLine);  //! 
    InsertText( GetCurrentPos(), "{\n" );
        
    SetLineIndentation (currentLine +1, lineIndent);
    int p = PositionFromLine (currentLine +1) + lineIndent;
    InsertText( p, "    \n");
    SetLineIndentation (currentLine +2, lineIndent);
    InsertText( PositionFromLine(currentLine+2) + lineIndent , "}" );
    GotoPos( p + 4);
}

void ZionEdit::OnCppT(wxCommandEvent & event)
{
    m_ourFrame->PrependToPopup( event.GetId() ); ///LAST CMD to popup
    if( GetSelectionStart() != GetSelectionEnd()  ) {
        int temp;
        if( (temp = GetAnchor() ) > GetCurrentPos() ) { //! If backwards selection, swap
            SetSelectionStart(GetCurrentPos());
            SetSelectionEnd(temp);
        }
        if(GetCharAt(GetSelectionStart()) == '\"') { //! If selection starts with double quote don't include quotes
            InsertText( GetSelectionStart() , "_T(" );
            InsertText( GetSelectionEnd() , ")" );
        }
        else { //! Include quotes
            InsertText( GetSelectionStart() , "_T(\"" );
            InsertText( GetSelectionEnd() , "\")" );
        }
    }
    else { //! Nothing selected 
        int p = GetCurrentPos();
        InsertText( p, "_T(\"\")" );
        GotoPos( p + 4);
    }
}

void ZionEdit::OnCppComment(wxCommandEvent & event)
{
    m_ourFrame->PrependToPopup( event.GetId() ); ///LAST CMD to popup
    int a=0, b=0;
    if( ProcessSel( &a, &b, false, false ) ) { ///leave at forward(greater) position, no need to select
        InsertText( a , "/*" );
        InsertText( b + 2 , "*/" );
    }
    else { InsertText( GetCurrentPos(), "/*  */" ); GotoPos( GetCurrentPos() + 3 ); }
}

void ZionEdit::OnHtmlInsertTag(wxCommandEvent & event)
{
    m_ourFrame->PrependToPopup( event.GetId() ); ///LAST CMD to popup
    int temp, temp2; wxString str, str2, str3;
    switch( event.GetId() ) {
        case ZION_HTMLMENU_BOLD: { str = "b";  break; }
        case ZION_HTMLMENU_STRONG: { str = "strong";  break; }
        case ZION_HTMLMENU_ITALIC: { str = "i";  break; }
        case ZION_HTMLMENU_UNDERLINE: { str = "u";  break; }
        case ZION_HTMLMENU_PARAGRAPH: { str = "p";  break; }
        case ZION_HTMLMENU_EM: { str = "em";  break; }
        case ZION_HTMLMENU_CENTER: { str = "center";  break; }
        case ZION_HTML_SCRIPT: { str = "script";  break; }
        case ZION_HTMLMENU_SPAN: { str = "span";  break; }
        case ZION_HTMLMENU_FONT: { str = "font";  break; }
        case ZION_HTMLMENU_FORM: { str = "form";  break; }
        case ZION_HTMLMENU_BLOCKQUOTE: { str = "blockquote";  break; }
        case ZION_HTMLMENU_SUP: { str = "sup";  break; }
        case ZION_HTMLMENU_CODE: { str = "code";  break; }
        case ZION_HTMLMENU_PRE: { str = "pre";  break; }
        case ZION_HTMLMENU_H3: { str = "h3";  break; }
        case ZION_HTMLMENU_H1: { str = "h1";  break; }
        case ZION_HTMLMENU_H2: { str = "h2";  break; }
        case ZION_HTMLMENU_H4: { str = "h4";  break; }
        case ZION_HTMLMENU_LI: { str = "li";  break; }
        case ZION_HTMLMENU_UL: { str = "ul";  break; }
        case ZION_HTMLMENU_OL: { str = "ol";  break; }
        case ZION_HTMLMENU_TABLE: { str = "table";  break; }
        case ZION_HTMLMENU_TD: { str = "td";  break; }
        case ZION_HTMLMENU_TR: { str = "tr";  break; }
        case ZION_HTMLMENU_TH: { str = "th";  break; }
        //case ZION_HTMLMENU_: { str = "";  break; }
    }
    str2.Printf("<%s>",str);  str3.Printf("</%s>",str);
    if( GetSelectionStart() != GetSelectionEnd()  ) {
        if( (temp = GetAnchor() ) > GetCurrentPos() ) { //! If backwards selection, swap //Think: we drop the anchor then start selecting
            SetSelectionStart(GetCurrentPos());
            SetSelectionEnd(temp);
        }
        temp = GetSelectionStart();
        InsertText( temp, str2 );
        temp2 = GetSelectionEnd();  //! SelectionEnd seems to get adjusted after an insert (automatically increased)
        InsertText( temp2 , str3 );

        SetSelectionStart(temp + str.Length() + 2); //! eg. adj for  <b>
        SetSelectionEnd(temp2);
    }
    else { InsertText( GetCurrentPos(), str2+str3 ); GotoPos( GetCurrentPos() + str.Length() + 2 ); }
}

void ZionEdit::OnHtmlQuickTag(wxCommandEvent &event)
{
    m_ourFrame->PrependToPopup( event.GetId() ); ///LAST CMD to popup
    ///Reverse selection if backwards
    AddTagsDialog tag( this, m_ourFrame->GetPosition(), m_ourFrame->GetSize().GetWidth() );
    if( tag.ShowModal() != wxID_OK ) return;
    wxString str = tag.m_text->GetValue();
    int len = str.Len();
    if( str != wxEmptyString ) {
        int a=0, b=0;
        if( ProcessSel( &a, &b ) ) { ///fwd, do reselect
            InsertText( a, _T("<") + str + _T(">") );
            InsertText( b + len + 2, _T("</") + str + _T(">") );
            SetSelectionStart( a + len + 2 );  ///Maintain selection
            SetSelectionEnd( b + len + 2 );
        }
        else {
            int p=GetCurrentPos();
            InsertText( p, _T("<") + str + _T("></") + str + _T(">") );
            GotoPos( p + str.Len() + 2 );
        }
    }
}

void ZionEdit::OnHtmlComment(wxCommandEvent & event)
{
    m_ourFrame->PrependToPopup( event.GetId() ); ///LAST CMD to popup
    int temp;
    if( GetSelectionStart() != GetSelectionEnd() ) {
        if( (temp = GetAnchor() ) > GetCurrentPos() ) { //! If backwards selection, swap
            SetSelectionStart(GetCurrentPos());
            SetSelectionEnd(temp);
        }
        InsertText( GetSelectionStart() , "<!--" );
        InsertText( GetSelectionEnd() , "-->" );
    }
    else { InsertText( GetCurrentPos(), "<!-- -->" ); GotoPos( GetCurrentPos() + 4 ); }
}

void ZionEdit::OnHtmlAnchor(wxCommandEvent & event)
{
    m_ourFrame->PrependToPopup( event.GetId() ); ///LAST CMD to popup
    int a=0, b=0;
    if( ProcessSel(&a, &b, false, false ) ) { ///regular forward selection, don't reselect
        InsertText( a, "<a href=\"\">" );
        InsertText( b+11, "</a>" ); GotoPos( a + 9 );
    }
    else {
        InsertText( (a=GetCurrentPos()), "<a href=\"\"></a>" ); GotoPos( a + 9 );
    }
}

void ZionEdit::OnHtmlBR(wxCommandEvent & event)
{
    m_ourFrame->PrependToPopup( event.GetId() ); ///LAST CMD to popup
    wxString str(_T("<br/>") );
    int a=0, b=0;
    if( ProcessSel(&a, &b, false, false ) ) { ///regular forward selection, don't reselect
        ///Concept: use selection just as a range. As long as the start of the next line is < the Selection End, add <br>s to the end of line.
        /// !!!Insert will automatically extend the selection, so we will use that property
        int currentLine = LineFromPosition( a );
        GotoLine( currentLine ); 
        do {
            InsertText( GetLineEnd(), str ); b += 5; ///GetLineEnd() operates on currentLine //adjust upper limit
            GotoLine( ++currentLine );
        } while (GetLineEnd() < b); ///Insert will extend the selection, so no
        
    }
    else {
        //InsertText( GetCurrentPos(), str );
        int c = GetCurrentPos();
        InsertText( c, "<br/>" );  GotoPos( c +5 );
    }
}

void ZionEdit::OnHtmlNBSP(wxCommandEvent & event)
{
    m_ourFrame->PrependToPopup( event.GetId() ); ///LAST CMD to popup
    int a = GetCurrentPos();
    InsertText( a, "&nbsp;" );  GotoPos( a +6 );
}

void ZionEdit::OnHtmlQuote(wxCommandEvent & event)
{
    m_ourFrame->PrependToPopup( event.GetId() ); ///LAST CMD to popup
    int a, b;
    if( ProcessSel( &a, &b, true, false ) ) { ///Leave at the lesser position, don't select
        InsertText( a, "&quot;" );  InsertText( b +6, "&quot;" );
        SetSelection( a, b+12 );
    }
    else {
        a = GetCurrentPos();
        InsertText( a, "&quot;" );  GotoPos( a +6 );
    }
}

void ZionEdit::OnHtmlNDASH(wxCommandEvent & event)
{
    m_ourFrame->PrependToPopup( event.GetId() ); ///LAST CMD to popup
    int a = GetCurrentPos();
    InsertText( a, "&ndash;" );  GotoPos( a +7 );
}

void ZionEdit::OnHtmlSupTH(wxCommandEvent & event)
{
    m_ourFrame->PrependToPopup( event.GetId() ); ///LAST CMD to popup
    int a = GetCurrentPos();
    InsertText( a, "<sup>th</sup>" );  GotoPos( a +13 );
}

void ZionEdit::OnHtmlPhp(wxCommandEvent & event)
{
    m_ourFrame->PrependToPopup( event.GetId() ); ///LAST CMD to popup
    int a, b;
    if( ProcessSel( &a, &b, true, false ) ) { ///Leave at the lesser position, don't select
        InsertText( a, "<?php  " );  InsertText( b +7, " ?>" );
        SetSelection( a+7, b+7 );
    }
    else {
        a = GetCurrentPos();
        InsertText( a, "<?php  ?>" );  GotoPos( a +6 );
    }
}

void ZionEdit::OnHtmlAspDrtv(wxCommandEvent & event)
{
    m_ourFrame->PrependToPopup( event.GetId() ); ///LAST CMD to popup
    int a = GetCurrentPos();
    InsertText( a, "<%@  %>" );  GotoPos( a +4 );
}

void ZionEdit::OnHtmlAsp(wxCommandEvent & event)
{
    m_ourFrame->PrependToPopup( event.GetId() ); ///LAST CMD to popup
    int a, b;
    if( ProcessSel( &a, &b, true, false ) ) { ///Leave at the lesser position, don't reselect
        InsertText( a, "<% " );  InsertText( b +3, " %>" );
        SetSelection( a+3, b+3 );
    }
    else {
        a = GetCurrentPos();
        InsertText( a, "<%  %>" );  GotoPos( a +3 );
    }
}

void ZionEdit::OnHtmlAspNet(wxCommandEvent & event)
{
    m_ourFrame->PrependToPopup( event.GetId() ); ///LAST CMD to popup
    int a, b;
    if( ProcessSel( &a, &b, true, false ) ) { ///Leave at the lesser position, don't reselect
        InsertText( a, "<asp:" );  InsertText( b +5, " />" );
        SetSelection( a+5, b+5 );
    }
    else {
        a = GetCurrentPos();
        InsertText( a, "<asp:  />" );  GotoPos( a +5 );
    }
}

void ZionEdit::OnHtmlTemplAsp(wxCommandEvent & WXUNUSED(event))
{
    int a, b;
    ///Block Mode?
    if( ProcessSel( &a, &b, true, false ) ) { ///Leave at the lesser position, don't select  //default => regular fwd selection and leave selected
        InsertText( a, _T("<%@Language = VBScript %>\n<% Option Explicit %>\n\
<html>\n<head>\n<title>New ASP</title>\n</head>\n<body>\n<%\n") );
        InsertText( b + 103, _T("\n%>\n</body>\n</html>\n") );
        SetSelection( a+103, b+103 );
    }
    else {
        a = GetCurrentPos();
        InsertText( a, _T("<%@Language = VBScript %>\n<% Option Explicit %>\n\
<html>\n<head>\n<title>New ASP</title>\n</head>\n<body>\n<%\n") );
        InsertText( GetLength(), _T("\n%>\n</body>\n</html>\n") );  ///We want to insert at the very end position == length of doc
        GotoPos( a + 103 );
    }
}

void ZionEdit::OnHtmlTemplBasic(wxCommandEvent & WXUNUSED(event))
{
    int a, b;
    ///Block Mode?
    if( ProcessSel( &a, &b, true, false ) ) { ///Leave at the lesser position, don't select  //default => regular fwd selection and leave selected
        InsertText( a, _T("<html>\n<head>\n</head>\n<body>\n") );
        InsertText( b + 29, _T("\n</body>\n</html>\n") );
        SetSelection( a+29, b+29 );
    }
    else {
        a = GetCurrentPos();
        InsertText( a, _T("<html>\n<head>\n</head>\n<body>\n") );
        InsertText( GetLength(), _T("\n</body>\n</html>\n") );  ///We want to insert at the very end position == length of doc
        GotoPos( a + 29 );
    }
}

void ZionEdit::OnHtmlTemplBasicNoHead(wxCommandEvent & WXUNUSED(event))
{
    int a, b;
    ///Block Mode?
    if( ProcessSel( &a, &b, true, false ) ) { ///Leave at the lesser position, don't select  //default => regular fwd selection and leave selected
        InsertText( a, _T("<html>\n<body>\n") );
        InsertText( b + 14, _T("\n</body>\n</html>\n") );
        SetSelection( a+14, b+14 );
    }
    else {
        a = GetCurrentPos();
        InsertText( a, _T("<html>\n<body>\n") );
        InsertText( GetLength(), _T("\n</body>\n</html>\n") );  ///We want to insert at the very end position == length of doc
        GotoPos( a + 14 );
    }
}

void ZionEdit::OnHtmlTemplTrans(wxCommandEvent & WXUNUSED(event))
{
    int a, b;
    wxString header = _T("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">\n\
<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"en\" lang=\"en\">\n\
<head><title>New HTML</title>\n</head>\n<body>\n");
    ///Block Mode?
    if( ProcessSel( &a, &b, true, false ) ) { ///Leave at the lesser position, don't select  //default => regular fwd selection and leave selected
        InsertText( a, header );
        InsertText( b + 235, _T("\n</body>\n</html>\n") );
        SetSelection( a+235, b+235 );
    }
    else {
        a = GetCurrentPos();
        InsertText( a, header );
        InsertText( GetLength(), _T("\n</body>\n</html>\n") );  ///We want to insert at the very end position == length of doc
        GotoPos( a + 235 );
    }
}

void ZionEdit::OnHtmlTemplStrict(wxCommandEvent & WXUNUSED(event))
{
    int a, b;
    wxString header = _T("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\"\
\"http://www.w3.org/TR/xhtml1/DTD/xhtml-strict.dtd\">\n\
<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"en\" lang=\"en\">\n\
<head><title>New HTML</title>\n</head>\n<body>\n");
    ///Block Mode?
    if( ProcessSel( &a, &b, true, false ) ) { ///Leave at the lesser position, don't select  //default => regular fwd selection and leave selected
        InsertText( a, header );
        InsertText( b + 221, _T("\n</body>\n</html>\n") );
        SetSelection( a+221, b+221 );
    }
    else {
        a = GetCurrentPos();
        InsertText( a, header );
        InsertText( GetLength(), _T("\n</body>\n</html>\n") );  ///We want to insert at the very end position == length of doc
        GotoPos( a + 221 );
    }
}

void ZionEdit::OnSelectAll (wxCommandEvent & WXUNUSED(event))
{    CmdKeyExecute (wxSTC_CMD_SELECTALL);
}
///--------------------------------------------------------------
void ZionEdit::RepositionIfSel()
{
    int a, b;
    ProcessSel( &a, &b, true, false ); ///Leave at the lesser position, don't select
}

void ZionEdit::OnDuplicateLine (wxCommandEvent & event)
{    
    SelectionDuplicate(); ///Oopsie - I was reinventing the wheel here, plus clipboard not used!
//    int a, b;/*  = GetSelectionStart();
//    int b = GetSelectionEnd(); */
//    ///Block Mode?
//    if( ProcessSel( &a, &b ) ) { /// regular fwd selection and leave selected
//        //if( a > b ) { int c = a; a=b; b=c;  } ///swap
//        CmdKeyExecute (wxSTC_CMD_COPY);  ///The block should still be selected
//        GotoPos(b); ///a this point b should be the forward position
//        CmdKeyExecute (wxSTC_CMD_PASTE);
//        SetSelection( a, b );
//    }
//    else CmdKeyExecute (wxSTC_CMD_LINEDUPLICATE);
}

void ZionEdit::OnCutLine(wxCommandEvent & event) //! FIX NOMENCLATURE //Cut Line
{    RepositionIfSel(); CmdKeyExecute (wxSTC_CMD_LINECUT);
}

void ZionEdit::OnCopyLine(wxCommandEvent & WXUNUSED(event)) 
{    
    RepositionIfSel(); ///If there is a selection, place the caret at the lower position 
    CmdKeyExecute (wxSTC_CMD_LINECOPY);
}

void ZionEdit::OnTransposeLine(wxCommandEvent & event)
{    RepositionIfSel(); CmdKeyExecute (wxSTC_CMD_LINETRANSPOSE);
}

void ZionEdit::OnGotoLine(wxCommandEvent & event)
{
    LineDialog * aLineDialog = new LineDialog(this, wxPoint(m_ourFrame->GetPosition().x, m_ourFrame->GetPosition().y ), m_ourFrame->GetSize().GetWidth(),
            GetCurrentLine() +1, GetLineCount() ); //! Spin ctrl will show unity based values
    if( aLineDialog->ShowModal() == wxID_OK)  {
        GotoLine( aLineDialog->m_spin->GetValue() -1 ); 
        ScrollFromEdge();
    }
}

void ZionEdit::ScrollFromEdge()
{
    ///Try to position the current line away from the edges of the screen so we can see more of its context
    if( GetWrapMode() == wxSTC_WRAP_NONE ) { /// Positioning with wrapping gets tricky
        int a= GetFirstVisibleLine();  int b = LinesOnScreen();
        int c = GetCurrentLine();
        if( b > 8 && c -a < b>>3 ) ///If we have enough lines to work with and  we are less than a 1/4 screen  from top
            LineScroll( 0, - b>>3); ///Scroll back a 1/8 screen
        else if( b > 8 && a+b -c < b>>3) LineScroll( 0, b>>3); /// lf last line - current line is < a 1/4 screen, scroll down a 1/4 screen
    }
    else { ///Trying a new algo for wrapped mode
        ///Positioning
        int acY = PointFromPosition(GetCurrentPos()).y;  ///TODO - rename acY
        int y = m_ourFrame->GetPosition().y; ///Default to top pos
        int clientOffset = -( ScreenToClient(m_ourFrame->GetPosition()).y ); ///ScreenToClient returns a fixed -ve offset to the Client Area -weird!
        acY = y + clientOffset + acY; ///Xlat to Screen coord
        int parH = m_ourFrame->GetSize().GetHeight();
        if( acY > y + parH/2 ) { /// > half way down
            if( y + parH - acY < 80 ) { /// and  within nnn of the btm
                LineScroll(0, 3); ///scroll down a few lines
            }
        }
        else if( acY - y < 110 )  LineScroll(0, -3);
    }
}

void ZionEdit::OnGotoStart(wxCommandEvent & event)
{  GotoLine(0);
}

void ZionEdit::OnGotoEnd(wxCommandEvent & event)
{  GotoLine( GetLineCount() - 1);
}

void ZionEdit::OnConvertEOL( wxCommandEvent & event )
{
    int eolMode;
    switch (event.GetId()) {
        case ZION_LINE_UNIX: { eolMode = wxSTC_EOL_LF; break;}
        case ZION_LINE_MAC: { eolMode = wxSTC_EOL_CR; break;}
        default:
        case ZION_LINE_WINDOWS: { eolMode = wxSTC_EOL_CRLF; break;}
    }
    ConvertEOLs (eolMode);
    SetEOLMode (eolMode);
    ShowEOLMode();
}

/** FIND/REPLACE **/

///Find/Replace Dialog
void ZionEdit::OnFindReplace(wxCommandEvent & event) //! Fire up the dialog or remove it if already exist
{
    m_ourFrame->PrependToPopup( event.GetId() ); ///LAST CMD to popup
    if(m_dlgReplace) { //! This check should only be necessary for NonModal Dialogs
        delete m_dlgReplace; m_dlgReplace = NULL;   //! If the dialog already exists delete it and return
    }
    else {
        if( GetSelectionStart() != GetSelectionEnd() ) {
            m_findReplData.SetFindString( GetSelectedText() ); //! seed the F/R dlg  We may not want to store it in findReplData yet, but
                    ///there is a quirk in wxWidgets that automatically "looks up" the string fully or partially matching our string in the Find combo
            m_findReplData.SetFlags(0);
        }
        m_dlgReplace = new FindReplDialog(this, m_ourFrame->GetPosition(), m_ourFrame->GetSize().GetWidth(), &m_findReplData, &m_findReplData2);
        m_dlgReplace->Show(true);   //! Show non-modal
    }
}

void ZionEdit::OnDlgFind( FindReplEvent & WXUNUSED(event) )
///Find/Replace Dialogs "Find" event handler called only once for a new pattern
{
    m_findWrap = false;  //! Start a fresh search
    m_searchFlags = m_findReplData.GetFlags();   //! It seems wxSTC's search flags matchs wxWidget's
    //int temp;  //! forward selection will leave currentPos ahead of anchor
    int a=GetAnchor(), b=GetCurrentPos(); ///Using GetSelectionStart-End to ascertain positions is futile //Think: we drop the anchor then start selecting
    if( a > b ) { //! If backwards selection, swap
        SetSelection(b, a); ///move currentPos to higher value (force forwards sel)
        m_searchStartPos = a; ///track starting pos
    }
    else m_searchStartPos = b;

    FindNext(true);  //! FindNext will override the search pattern if there is some text selected so tell it we are using a non-modal dialog; i.e. we could be changing our search pattern midstream.
    //m_ourFrame->SetStatusText(wxString::Format(_T("EVT_FIND - FindStr: \"%s\" - Flags: %d"), m_findReplData.GetFindString(), m_findReplData.GetFlags()) );  //! DIAG
//    ///FindNext will automatically do an update Send out an update.
//    m_ourFrame->m_advisedData = m_findReplData.GetFindString();
//    m_ourFrame->NotifyFindUpdate();
}

void ZionEdit::OnDlgFindNext( FindReplEvent & WXUNUSED(event) )    //! Called AFTER the first find
{
    FindNext(true, false); //! true - called from a nonmodal dialog, no need for an update now
    //m_ourFrame->SetStatusText(wxString::Format(_T("EVT_FIND_NEXT - FindStr: \"%s\" - Flags: %d"), m_findReplData.GetFindString(), m_findReplData.GetFlags()) );  //! DIAG
}

void ZionEdit::OnDlgReplace(FindReplEvent & WXUNUSED(event) ) 
{
    //wxString str(GetSelectedText()); //Because of regular exp.s we cannot do simple == comparison between selected text and searchpattern, so just skip it
    if( GetSelectionStart() != GetSelectionEnd()  ) { // str != "" && str.Lower() == m_findReplData.GetFindString().Lower() )  //! If the previous selection is not our search pattern skip
        int temp;
        if( (temp = GetAnchor() ) < GetCurrentPos() )  m_searchStartPos = temp; ///On replace we want to start at lower position //! forward selection will leave currentPos ahead, so record achor position
        else m_searchStartPos = GetCurrentPos();
        
        ReplaceSelection( CheckPreserveCase() /* m_findReplData.GetReplString() */ ); //! If we already found it, replace it.
    }
    else { 
        if( FindNext(true, false) )  ///No IPC update as this handler (OnDlgReplace) can be used in a loop  //! FindNext() should return true and hilight the selection //! NonModalMode
            if( GetSelectedText() != _T("") )   ReplaceSelection( CheckPreserveCase() ); //! Lot's of safeties
    }
    /// Send out an update.
    m_ourFrame->m_advisedReplData = m_findReplData.GetReplString();
    m_ourFrame->NotifyReplUpdate();
}

wxString ZionEdit::CheckPreserveCase()
{        ///Roughly: If(preserve case) ///Ascertain case of selection ///Set replacement str to case of selection ///Replace

    wxString str2(m_findReplData.GetReplString());
    
    if( m_findReplData.GetFlags() & ZION_FR_FLAG_PRESERVECASE ) {
        wxString str(GetSelectedText() );
        char c, c0 =str.GetChar(0);  char c1= ' ';
        
        if( c0 > 'a' -1 && c0 < 'z'+1 ) { /// First Letter lowercase // cannot use >= because of &&
            for( unsigned int i=1; i < str.Length(); i++ ) {
                c = str.GetChar(i);
                if( c > 'A'-1 && c < 'Z'+1 ) return str2;  ///Mixed case so just use the given replacement string
            }
            str2 = str2.Lower();
        }
        else if( c0 > 'A'-1 && c0 < 'Z'+1 ) {
            if( str.Length() > 1 &&  (c1 = str.GetChar(1)) ///we can't have 0 here so the && continues
            && (c1  > 'a'-1) && (c1 < 'z'+1) ) { /// If  there is a 2nd char and it is lower, just capitalize first letter
                c = str2.GetChar(0);  c = c & 0xDF;  str2.SetChar(0, c);  /// Just first letter uppercase
            }
            else str2= str2.Upper(); ///All uppercase
        }
    }
    return str2;
}

void ZionEdit::OnDlgReplaceAll( FindReplEvent & WXUNUSED(event) )
{
    /// Send out an update.
    m_ourFrame->m_advisedReplData = m_findReplData.GetReplString();
    m_ourFrame->NotifyReplUpdate();
    
    if( GetSelectedText() != _T("") ) { // str != "" && str.Lower() == m_findReplData.GetFindString().Lower() )  //! If the previous selection is not our search pattern skip
        ///Correct for backwards selection
        ///ReplaceAll() replaces the first selection regardless of start and end positions so set our search start point to the lower position.
        int a=GetAnchor(), b=GetCurrentPos(); ///Using GetSelectionStart-End to ascertain positions is futile //Think: we drop the anchor then start selecting
        if( a < b ) { //! If normal selection, swap
            SetSelection(b, a); ///move currentPos to lower value (force backwards sel)
            m_searchStartPos = a; ///track starting pos
        }
        else m_searchStartPos = b;
    }
    else m_searchStartPos = GetCurrentPos(); ///so we'll know where to stop
    
    if( GetSelectionStart() != GetSelectionEnd() )   ReplaceSelection(CheckPreserveCase() /* m_findReplData.GetReplString() */ ); //! If we already found it, replace it.
    while( FindNext(true, false) ) ///No update-Loop, we don't want to overload IPC    //! Find and Replace the rest ///not super quick find. We really should reverse the logic and have super quick find pass true to FindNext()
        if( GetSelectionStart() != GetSelectionEnd() )   ReplaceSelection( CheckPreserveCase() /* m_findReplData.GetReplString() */ ); //! Lot's of safeties
}

void ZionEdit::OnDlgFindClose( FindReplEvent & WXUNUSED(event) )
{
    m_dlgReplace->Destroy(); m_dlgReplace = NULL;
    ShowPosition();
    /// Send out an update.
//    m_ourFrame->m_advisedData = m_findReplData.GetFindString();
//    m_ourFrame->NotifyFindUpdate();
}

///EXTERNAL REPLACE
void ZionEdit::DoReplaceAll()
{
    while( FindNext(true, false) )     //! Find and Replace the rest ///true == not super quick find. We really should reverse the logic and have super quick find pass true to FindNext()
        if( GetSelectionStart() != GetSelectionEnd() )   ReplaceSelection( CheckPreserveCase() /* m_findReplData.GetReplString() */ ); //! Lot's of safeties
}

void ZionEdit::OnFilesResultsClicked( FilesResultsEvent & event )
{
    wxString str;
    str.Printf( _T("%s\\zion.exe -f\"%s\" -g%d -r \"%s\" "),
            m_ourFrame->m_exeDir, m_filesResultsData.m_patt,
            m_filesResultsData.m_flags, m_filesResultsData.m_files[event.GetIdx()] );
    m_ourFrame->SetStatusText(str);
    ///Close Previous Window?
    if( m_filesResultsData.m_closeOnSwitch && m_filesResultsData.m_lastPid != -1 ) /// first click doesn't have a previous window
        m_ourFrame->NotifyClose( m_filesResultsData.m_lastPid ); ///Let Server know we would like to close window of last pid
//        wxKill( m_filesResultsData.m_lastPid, wxSIGQUIT );
    //wxMilliSleep(800); ///Wait for the closed window to send a history update to be written by the server //oops we might be the server
    m_filesResultsData.m_lastPid = wxExecute(str);
}

void ZionEdit::OnFilesResultsClosing( FilesResultsEvent & WXUNUSED(event) )
{
    if(m_filesDialog) m_filesDialog->Destroy();
    m_filesDialog = NULL;
    m_filesResultsData.m_files.Clear();
    m_filesResultsData.m_lastPid = -1;
    m_fileList.Clear(); ///wxArrayString of files string found in
}

void ZionEdit::OnFind( wxCommandEvent & event )   /// Standard Find
{
    m_ourFrame->PrependToPopup( event.GetId() ); ///LAST CMD to popup
    m_findWrap = false;  //! Start a fresh search
    ///Get the search pattern
    if( GetSelectionStart() != GetSelectionEnd() ) {
        m_findReplData.UpdateFindString( GetSelectedText() );
        m_findReplData.SetFlags(0); /// Starting a new search
    }

    wxArrayString rdirs;
    for( int i=0; i< NUMRECENTDIRS; i++ ) {
        if( m_ourFrame->dirSubItems[i] == NULL ) break; ///We have hit the end of the used dirSubItems
        rdirs.Add( m_ourFrame->dirSubItems[i]->GetLabel() );
    }
    FindDialog searchDialog( this, wxPoint(m_ourFrame->GetPosition().x, m_ourFrame->GetPosition().y),
            m_ourFrame->GetSize().GetWidth(), &m_findReplData, &m_findReplData2, &rdirs );   ///created on the stack just like an automatic variable
    if(searchDialog.ShowModal() == wxID_OK) /// Flags and FindString are set internally in the Dialog onOK. i.e. Dialog updates findReplData
    {
        if( searchDialog.m_allFiles->IsChecked() )  DoFindAllFiles( searchDialog.m_dirName->GetLabel() ); ///Search externally
        
        else { ///Internal Search
            m_searchStartPos = GetCurrentPos(); /// So find wrapping will know when to stop
            m_ourFrame->SetStatusText(_T("")); //! Clear
            int result = -1;
            
            if( searchDialog.m_all->IsChecked() )  /// FIND ALL and show the find results dialog
                DoFindAll();
            else { /// Do incremental search
                result = FindText( GetCurrentPos(), GetLength(), m_findReplData.GetFindString(),  /// Not GetLength() - 1 or last pattern may not be found
                m_findReplData.GetFlags() );
                if ( result != -1 )  {
                    Found( result);
                    /// Send out an update.
                    m_ourFrame->m_advisedData = m_findReplData.GetFindString();
                    m_ourFrame->NotifyFindUpdate();
                }
                else { //! Give it one more chance from top
                    m_findWrap = true;
                    GotoPos(0);
                    FindNext(true); //! FindNext will report when it is finished
                }
            }
        }
    }
}

void ZionEdit::OnFindAllFiles( wxCommandEvent & WXUNUSED(event) ) /// Will ForceDialog
{
    wxString dirStr(wxGetCwd()); ///Default to current Dir
    
    if( GetSelectionStart() != GetSelectionEnd() ) { ///Selection
        m_findReplData.UpdateFindString(GetSelectedText()); ///We always want to update findstring before an actual search operation. SetFindString sets only the displayed text in the textcontrol of the combo
        m_findReplData.SetFlags(0); //! Reset
    }
    wxArrayString rdirs;
    for( int i=0; i< NUMRECENTDIRS; i++) {
        if( m_ourFrame->dirSubItems[i] == NULL ) break; ///We have hit the end of the used dirSubItems
        rdirs.Add( m_ourFrame->dirSubItems[i]->GetLabel() );
    }
    
    FindDialog searchDialog( this, wxPoint(m_ourFrame->GetPosition().x, m_ourFrame->GetPosition().y),
            m_ourFrame->GetSize().GetWidth(), &m_findReplData, &m_findReplData2, &rdirs, false, true ); /// Dialog updates findReplData onOK
    if( searchDialog.ShowModal() != wxID_OK) return;  //! Allow cancel before changing search parameters
    dirStr = searchDialog.m_dirName->GetValue(); /// Get path info from dialog

    DoFindAllFiles( dirStr );
}

void ZionEdit::DoFindAllFiles( const wxString & path )
{
    if(m_filesDialog) { m_filesDialog->Destroy(); m_filesDialog = NULL; }  //! Must set to NULL so our if() will work. Need a close event from our NonModal dialog so we'll know when to set to NULL
    /// Send out an update.
    m_ourFrame->m_advisedData = m_findReplData.GetFindString();
    m_ourFrame->NotifyFindUpdate();
    ///Open a new edit module
    wxString str2 =wxGetCwd();
    ZionEdit * stc = new ZionEdit( m_ourFrame, (wxWindow *)this, str2, true, wxID_ANY, wxDefaultPosition, wxSize( 600, 740) );
    /// Setup Enumeration of files in Dir and subdir. 
    wxArrayString files; ///Container for the names of all files in the directory tree
    DirTravSimple traverser(files);
    ///Files Enumeration
    wxString dirStr = path; ///Since we are passing this value further on to the result Dialog we are probably safer working from a copy from here on //searchDialog.m_dirName->GetLabel();
    wxDir dir(dirStr);
    dir.Traverse(traverser);
    
    ///Setup up search patt and flags
    wxString str, patt = m_findReplData.GetFindString();
    m_filesResultsData.m_patt = m_findReplData.GetFindString();  ///We want to copy patt, flags and file list to a separate structure so we can do another in-file search while the results dialog is still active
    m_fileList.Empty(); ///Results to be passed to the FilesResults Dialog ///Fixed up list
    m_filesResultsData.m_files.Empty();  ///Doesn't release allocated memory. Will do a Clear() in the Close handler ///Absolute list
    m_filesResultsData.m_flags =  m_findReplData.GetFlags();
    
    size_t c = files.GetCount();
    int pos, selStart, result;
    ///Preload Extensions for faster lookup in for loop
    wxArrayString exts;
    stringToInt::iterator it;
    for( it = ext2lex.begin(); it != ext2lex.end(); ++it ) { 
        exts.Add(it->first); ///load our wxArrayString
    }
    size_t extsLen = exts.GetCount();
    
    ///SCAN ALL FILES returned from Dir Traverse
    m_ourFrame->SetStatusText(_T("PLEASE WAIT - Scanning Files..."));
    wxProgressDialog progDia( _T("Scanning ALL Files"), wxString::Format(_T("SCANNING  ALL Files in and below  %s\nCurrent File:  %s"), dirStr, files[0] ),
            c, this, wxPD_AUTO_HIDE|wxPD_APP_MODAL|wxPD_CAN_ABORT );
    bool loopCanceled = false;
    for( size_t i=0; i<c; i++) {
        ///Ext in List?
        bool inList = false;
        wxString fext = files[i].AfterLast('.');
        for( size_t j=0; j< extsLen; j++ ) {
            if( exts[j].Lower() == fext.Lower() ) { inList = true; break; }
        }
        ///Filename (like makefile) in list
        if( files[i].AfterLast('\\').Lower() == _T("makefile") )  inList = true;
        
        if( ! inList )  { m_ourFrame->SetStatusText( wxString::Format(_T("%s - unknown"), files[i].AfterLast('\\') ),  4 ); continue; }
       
        ///Size Exceeded? ///TODO: use wxFileName::Size()
        wxFile tf( files[i].c_str() );  wxFileOffset ofs = 800000; ///Limit to about 1M
        if( tf.Length() > ofs )
            { m_ourFrame->SetStatusText( wxString::Format(_T("%s - size exceeded"), files[i].AfterLast('\\') ),  4 ); continue; }
        
        ///Load and search
        //Diag//m_ourFrame->SetStatusText( files[i].AfterLast('\\'), 3 );
        stc->LoadFile( files[i] );
        stc->m_findReplData.UpdateFindString( patt );
        stc->m_findReplData.SetFlags( m_findReplData.GetFlags() );
        stc->m_searchStartPos = 0;
        stc->FindNext(true, false);
        if( ( result= stc->GetSelectionStart()) != stc->GetSelectionEnd() ) { ///FOUND!
            pos = selStart = result;
            str = stc->GetSelectedText();  ///ie get the result
            ///Prepend
            int selectionLine = stc->LineFromPosition(result);
            int lineStart = stc->PositionFromLine( selectionLine );
            while( --pos >=  lineStart && str.Length() < 50 ) /// Prepend up to 50 chars before the found str
                str.Prepend( wxString::Format( _T("%c"), stc->GetCharAt(pos) ) );
            str.Prepend( wxString::Format( _T("[line %d]   "), selectionLine +1 ) );   /// +1 for unity base //,selStart - lineStart+1, stc->m_lastFind_max - lineStart+1 )
                   
            //! Append
            pos = stc->m_lastFind_max; /// we haven't done any Gotos so our selection should still be good
            int length = stc->GetLength(); //! Length of Document
            int lineEnd = stc->GetLineEnd(); ///Optimized //PositionFromLine(GetCurrentLine() + 1) - ( GetEOLMode() ? 1 : 2 );
            while( pos < lineEnd && str.Length() < 62 && pos < length ) {
                str.Append( wxString::Format( _T("%c"), stc->GetCharAt(pos) ) );   ++pos;
            }
            str.Prepend( wxString::Format( _T("%s "), files[i].Mid(dirStr.Length() +1) ) ); ///Mid: remainder of the string after length of search top dir
            
            m_fileList.Add( str ); ///This is displayed
            m_ourFrame->SetStatusText( _T("Found in ") + files[i] ); 
            m_filesResultsData.m_files.Add(files[i]); /// store for later ref by the FilesResultsDialog click event
        }
        stc->ResetDoc(); stc->ClearAll();
        ///Update Progress
        if( ! progDia.Update( i+1, wxString::Format(_T("SCANNING  ALL Files in and below  %s\nCurrent File:  %s\n\"Cancel\" stops the scan and shows files found so far"),
                dirStr, files[(i+1<c)? i+1 : i] ) ) ) {
            m_ourFrame->SetStatusText(_T("Scan STOPPED") );
            loopCanceled = true; break;
        }
    }
    stc->Destroy();
    // We are choosing to stop instead of actually cancelling //if( loopCanceled ) return; ///May want to CLean up Data Structs also
    //m_ourFrame->SetStatusText(_T("PLEASE WAIT - Creating Results Dialog..."));
    m_filesDialog = new FilesResultsDialog( this, m_ourFrame->GetPosition(), m_ourFrame->GetSize().GetWidth(),
            &m_fileList, patt, dirStr, &m_filesResultsData, &m_findReplData ); ///TODO: optimize this
    m_filesDialog->Show(true);  //! Non-Modal
    ShowEOLMode();
    //m_ourFrame->SetStatusText(_T(""));
    //No Longer using 2 //m_ourFrame->SetStatusText(_T(""), 2);
}

void ZionEdit::OnRemoteFindAll(FilesResultsEvent & WXUNUSED(event) ) ///Will be triggered from the  FindInAllFiles dialog
{
    //Diag //m_ourFrame->SetStatusText(_T("In Remote FindAll") );
    ZionEdit * stc = new ZionEdit( m_ourFrame, (wxWindow *)this, wxGetCwd(), true, wxID_ANY, wxDefaultPosition, wxDefaultSize );
    ///All params here are from the last FindInAllFiles search, not findReplData
    stc->LoadFile( m_filesResultsData.m_files[m_filesResultsData.m_idx] );
    stc->m_findReplData.UpdateFindString( m_filesResultsData.m_patt );
    stc->m_findReplData.SetFlags( m_filesResultsData.m_flags );
    /// Start pos set in RemoteFindAll stc->m_searchStartPos = 0;
    m_filesResultsData.m_results = stc->RemoteFindAll();  ///result return in param
    stc->Destroy();
}

void ZionEdit::OnLocalFindAllTweak( myFindResultsEvent/*FilesResultsEvent*/ & WXUNUSED(event) ) ///Will be triggered from FindResults dialog
{
    ///m_findReplData should already be prepared
    m_findReplData.m_results = RemoteFindAll();  ///Borrow this function
}

wxArrayString ZionEdit::RemoteFindAll() ///Used externally (FindInAllFiles) and internally by OnLocalFindAllTweak
{
    wxString str;  wxArrayString results;
    //m_ourFrame->SetStatusText( wxString::Format( _T("lsp: %s"), m_findReplData.m_lastSearchPos), 3 );
    GotoPos( m_searchStartPos = PositionFromLine(m_findReplData.m_lastSearchPos -1) ); // 0  /// Always start at the top /// So find wrapping will know when to stop
    while( FindNext(true, false)  && GetSelectionStart() != GetSelectionEnd() ) {     /// The found string is selected
        int pos, selStart; pos = selStart = GetSelectionStart(); /// selStart - Optimized use variable instead of f()
        int selectionLine = LineFromPosition(pos);  ///Save it //! Let's not assume the line of the start of  selection is the current line
        str = GetSelectedText();  ///ie get the result
        
        ///CONSTRUCT RESULT line ///Prepend
        int lineStart = PositionFromLine(selectionLine);
        while( --pos >=  lineStart && str.Length() < 50 ) /// Prepend up to 50 chars before the found str
            str.Prepend( wxString::Format( _T("%c"), GetCharAt(pos) ) );
        str.Prepend( wxString::Format( _T("[%d,%d-%d]   "), selectionLine +1,
                selStart - lineStart+1, m_lastFind_max - lineStart+1 ) ); /// +1 for unity base
        /// Append
        pos = m_lastFind_max; /// we haven't done any Gotos so our selection should still be good
        int length = GetLength(); //! Length of Document
        int lineEnd = GetLineEnd(); ///Optimized //PositionFromLine(GetCurrentLine() + 1) - ( GetEOLMode() ? 1 : 2 );
        while( pos < lineEnd && str.Length() < 68 && pos < length ) {
            str.Append( wxString::Format( _T("%c"), GetCharAt(pos) ) );   ++pos;
        }
        results.Add(str);
    }
    return results;
}

void ZionEdit::OnFilesReplace( FilesResultsEvent & WXUNUSED(event) )
{
    //Diag //m_ourFrame->SetStatusText(_T("In FilesReplace Event...") );
    wxString str;
    str.Printf( _T("%s\\zion.exe -f\"%s\" -p\"%s\" -g%d -r \"%s\" "),
            m_ourFrame->m_exeDir, m_filesResultsData.m_patt,  m_filesResultsData.m_rPatt,
            m_filesResultsData.m_flags, m_filesResultsData.m_files[m_filesResultsData.m_idx] );
    m_ourFrame->SetStatusText(str);
    wxExecute(str,  wxEXEC_SYNC);  ///Wait till the window closes itself
}

void ZionEdit::OnFilesReplaceAll( FilesResultsEvent & WXUNUSED(event) )
{
    m_ourFrame->SetStatusText(_T("PLEASE WAIT!  Performing Replace in ALL Files...") );
    wxProgressDialog progDia( _T("REPLACING in ALL Files"), _T(""), m_filesResultsData.m_files.GetCount(),
        this, wxPD_AUTO_HIDE|wxPD_APP_MODAL|wxPD_CAN_ABORT );  ///Leave the message text empty as it will be updated  immediately

    wxString str; long pid;
    for( ; m_filesResultsData.m_idx < m_filesResultsData.m_files.GetCount(); m_filesResultsData.m_idx++ ) {
        ///Update the Progress bar
        if( ! progDia.Update( m_filesResultsData.m_idx, wxString::Format(_T("REPLACING in ALL (Remaining) Files. Current File:\n %s"),
                m_filesResultsData.m_files[m_filesResultsData.m_idx] ) ) ) {
            m_ourFrame->SetStatusText(_T("Replace CANCELED!") );
            return;
        }
        str.Printf( _T("%s\\zion.exe -f\"%s\" -p\"%s\" -g%d -r \"%s\" "),
                m_ourFrame->m_exeDir, m_filesResultsData.m_patt,  m_filesResultsData.m_rPatt,
                m_filesResultsData.m_flags, m_filesResultsData.m_files[m_filesResultsData.m_idx] );
        //m_ourFrame->SetStatusText(str); ///Diag 
        pid = wxExecute(str, wxEXEC_SYNC);  /// The process should self-terminate //SYNC - wait for the process to terminate
    }
    m_ourFrame->SetStatusText(_T("Replace in All Files COMPLETE") );
}

void ZionEdit::DoFindAll( bool noDlg, bool rtFind ) ///Shared findall code
{
    if(m_fresDialog) {
        m_fresDialog->Destroy(); m_fresDialog = NULL; //! Must set to NULL so our if() will work. A close event from our NonModal dialog also sets this to NULL
    }
    rtFind ? m_findReplData2.m_results.Empty() : m_findReplData.m_results.Empty();  ///We will be using m_results again pretty soon, so no need to Clear - deallocate memory
    wxString str, str2; char ch;  int firstFound = -1, firstEnd = -1;
    
    while( FindNext(true, false, rtFind)  && GetSelectionStart() != GetSelectionEnd() ) {     //! Find All ///The found string is selected
        int pos, selStart;
        pos = selStart = GetSelectionStart(); /// selStart - Optimized use variable instead of f()
        if( firstFound == -1 ) { firstFound = pos; firstEnd = m_lastFind_max; } //! Save the first range only the first time
        int selectionLine = LineFromPosition(pos);  ///Save it //! Let's not assume the line of the start of  selection is the current line
        str = GetSelectedText();  ///ie get the result
        ///CONSTRUCT RESULT line
        ///Prepend
        int lineStart = PositionFromLine(selectionLine);
        while( --pos >=  lineStart && str.Length() < 50 ) {/// Prepend up to 50 chars before the found str
            str2 = wxEmptyString;
            ch = GetCharAt(pos);
            if( ch == '\t' ) str2.Append( ' ', GetTabWidth() );
            else  str2 = wxString::Format( _T("%c"), GetCharAt(pos) );
            str.Prepend( str2 );
        }
        str.Prepend( wxString::Format( _T("[%d,%d-%d]   "), selectionLine +1,
                selStart - lineStart+1, m_lastFind_max - lineStart+1 ) ); /// +1 for unity base
        //! Append
        pos = m_lastFind_max; /// we haven't done any Gotos so our selection should still be good
        int length = GetLength(); //! Length of Document
        int lineEnd = GetLineEnd(); ///Optimized //PositionFromLine(GetCurrentLine() + 1) - ( GetEOLMode() ? 1 : 2 );
        while( pos < lineEnd && str.Length() < 68 && pos < length ) {
            ch = GetCharAt(pos);
            if( ch == '\t' || ch== '\r' || ch == '\n' ) ch = ' ';
            str.Append( ch /*wxString::Format( _T("%c"),*/  /*)*/ );   ++pos;
        }
        rtFind ? m_findReplData2.m_results.Add(str) : m_findReplData.m_results.Add(str); ///Save the result in our FindReplData
    }
    ///Do RESULTS Dialog
    if( ! noDlg ) {
        m_fresDialog = new FindResultsDialog( this, m_ourFrame->GetPosition(), m_ourFrame->GetSize().GetWidth(), &m_findReplData );
        m_fresDialog->Show(true);  //! Non-Modal
    }
    
    if( firstFound != -1 ) { /// i.e. > 0 occurences
        SetSelection( firstFound, firstEnd );   //! Hilight it ///Apparently moves our currentPos to the endOfSelection also!
        m_ourFrame->SetStatusText(_T("") );
        ///Try to position the selection away from the edges of the screen so we can see more of its context
        ScrollFromEdge();
        /// Send out an update.
        if( ! rtFind ) {
            m_ourFrame->m_advisedData = m_findReplData.GetFindString();
            m_ourFrame->NotifyFindUpdate();
        }
    }
    else GotoPos( m_searchStartPos );  ///Don't just "fall" to the end of  the document
}

void ZionEdit::OnRTFindAll( wxCommandEvent & WXUNUSED(event) ) {  /// DIALOG updates findReplData and flags REALTIME
    int a = GetFirstVisibleLine(); ///Maintain caret position

    m_searchStartPos = GetCurrentPos();  ///CRITICAL to locating All occurences!
    int savPos = m_searchStartPos;
    m_findWrap = false;  //! Start a fresh search
    DoFindAll( true, true );  ///no dlg, real time find
    GotoPos( savPos );

    int b = GetFirstVisibleLine();
    if( a != b ) LineScroll(0, a - b ); ///cols, lines // -ve will scroll backwards.
}

void ZionEdit::OnFindAll( wxCommandEvent & WXUNUSED(event) )  ///This is called from various places
{
    if( GetSelectionStart() == GetSelectionEnd() ) { ///No selection
        m_findReplData.SetFlags(0); ///Starting a fresh search, so clear flags
        FindDialog searchDialog( this, wxPoint(m_ourFrame->GetPosition().x, m_ourFrame->GetPosition().y),
                m_ourFrame->GetSize().GetWidth(), &m_findReplData, &m_findReplData2, NULL, true ); //! Get our search pattern. true == findAllMode
        if( searchDialog.ShowModal() != wxID_OK)
            return;  //! Allow cancel before changing search parameters
         /// Dialog updates findReplData onOK //m_findReplData.SetFlags( 0 | (searchDialog.m_regex->GetValue() ? ZION_FR_REGEX: 0) );  //! Pull search string from dialog
        m_searchStartPos = GetCurrentPos();  ///CRITICAL to locating All occurences!
    }
    else { /// Super Quick FindAll
//        int a=GetAnchor(), b=GetCurrentPos(); ///Using GetSelectionStart-End to ascertain positions is futile
//        if( a > b ) { //! If backwards selection, swap //Think: we drop the anchor then start selecting
//            SetSelection(b, a); ///choose higher value
//            m_searchStartPos = a; ///track starting pos
//        }
//        else m_searchStartPos = b;
        int a, b;
        ProcessSel( &a, &b ); ///Correct for backwards selection
        m_searchStartPos = b; ///start at the higher value
        m_findReplData.SetFlags(0); //! Dialog could change this
        
        if( b - a > 100 ) { /// Selection (search patt) length exceeded, use the dialog anyway //This is to alleviate a lockup when  selection is too large
            FindDialog searchDialog( this, wxPoint(m_ourFrame->GetPosition().x, m_ourFrame->GetPosition().y),
                    m_ourFrame->GetSize().GetWidth(), &m_findReplData, &m_findReplData2, NULL, true ); //! Get our search pattern. true == findAllMode
            if( searchDialog.ShowModal() != wxID_OK ) return;  //! Allow cancel before changing search parameters
            /// Dialog updates findReplData onOK //m_findReplData.SetFlags( 0 | (searchDialog.m_regex->GetValue() ? ZION_FR_REGEX: 0) );  //! Pull search string from dialog
            m_searchStartPos = a;  ///start at the beginning of the "failed" selection
        }
        else m_findReplData.UpdateFindString( GetSelectedText() ); ///We always want to update findstring before an actual search operation. SetFindString sets only the displayed text in the textcontrol of the combo
    }
    
    m_findWrap = false;  //! Start a fresh search
    //SearchAnchor(); //! If dialog requests current position, not start of document (For now we start from curr. position)
    DoFindAll();
}

bool ZionEdit::FindNext( bool nonmodal, bool notify, bool rtFind )
{
    if( !nonmodal && GetSelectionStart() != GetSelectionEnd() ) {  /// If not a non-modal dialog like FindAll ie. if modal ///Dialogless searches will use the existing flags
        ///Make sure selection length not exceeded
        int a, b; ProcessSel( &a, &b );
        if( b - a > 100 ) {
            m_ourFrame->SetStatusText(_T("SELECTION LENGTH EXCEEDED") ); return false;  /// Todo: add a more decent exit
        }
        wxString str9 = GetSelectedText();
        m_findReplData.UpdateFindString(str9); //! Use this as the new search pattern /// This enables us to do a super quick search
    }
    
   wxString str; int result = -1;
   if( ! rtFind ) {
       result = FindText( GetCurrentPos() /*  +1 */, GetLength(), m_findReplData.GetFindString(),  /// Not GetLength() - 1 or last pattern may not be found
                m_findReplData.GetFlags() );
   }
   else {
       result = FindText( GetCurrentPos() /*  +1 */, GetLength(), m_findReplData2.GetFindString(),  /// Not GetLength() - 1 or last pattern may not be found
                m_findReplData2.GetFlags() );
   }
            
   //if( ! m_min ) m_ourFrame->SetStatusText(_T("")); 

    if( result !=  -1 ) { //! FOUND
        if( m_findWrap && result >= m_searchStartPos ) { //! We wrapped but are at or passing our original position
            if( m_incrementalStartPos ) GotoPos( m_incrementalStartPos );  ///Maintain last position at the end of a regular search. We want this for the F/R dialog. We don't really care for FindAll since it will set its own position /// Do we want to stop at the last found or our original start position or last found position?
            if( ! m_min ) { str.Printf(_T("Finished searching for: \"%s\" (Wrapped once)"), rtFind?
                    m_findReplData2.GetFindString() : m_findReplData.GetFindString() );
                m_ourFrame->SetStatusText(str); }
            m_findWrap = false; //! Clear for next search
            return false; //! We already wrapped once so  cleanup, print msg and return
        }
        if(notify && ! rtFind) { ///IPC update ///rtFind should never notify
            m_ourFrame->m_advisedData = m_findReplData.GetFindString();
            m_ourFrame->NotifyFindUpdate();
        }
        Found( result, rtFind ); ///This is how the calling f() knows the location of the found string
        if( ! m_min && !nonmodal && !rtFind ) { //non-modal - don't show statustext
            str.Printf("Found at line %d%s", LineFromPosition(result) +1, (m_findWrap ? " (Wrapped once)" : "") );
            m_ourFrame->SetStatusText(str);
        }
        return true;
    }
    else {
        if(! m_findWrap) {
            m_findWrap = true; //! Give it one more chance from top
            GotoPos(0);
            return FindNext( true, notify, rtFind ); ///true- findstring is already established so bypass to step; Notify if found
        }
        else { /// Cannot be found anywhere in the file! (Wrap already set) so cleanup, print msg and quit
            if( ! m_min ) {
                str.Printf(_T("Finished searching for: \"%s\" (Wrapped once)"), rtFind?
                    m_findReplData2.GetFindString() : m_findReplData.GetFindString() );
                m_ourFrame->SetStatusText(str);
            }
            m_findWrap = false; //! Clear for next search
            GotoPos( m_searchStartPos ); 
            return false; //! We already wrapped once so return
        }
    }
    //! This point is never reached
    //return false;
}

void ZionEdit::OnFindPrev( wxCommandEvent & event )
{
    m_ourFrame->PrependToPopup( event.GetId() ); ///LAST CMD to popup
    int a, b;
    if( ProcessSel( &a, &b ) && b - a > 100 ) {
        m_ourFrame->SetStatusText(_T("Selection length exceeded") ); return;  /// Todo: add a more decent exit
    }
    wxString str9 = GetSelectedText();
    if( str9 != "" ) {
        m_findReplData.SetFlags(0); //! For now non-modal searches are case-insensitive and not whole-word
        m_findReplData.UpdateFindString(str9); //! Use this as the new search pattern /// This enables us to do a super quick search
        m_ourFrame->m_advisedData = str9;
        m_ourFrame->NotifyFindUpdate();
    }
    
   int moveBack = GetCurrentPos() - m_findReplData.GetFindString().Length() -1; 
   GotoPos( moveBack < 0 ? 0 : moveBack  );   //! Start searching after the last found pattern
   SearchAnchor();  //!Start searching from caret position
   wxString str;   int result = -1;

   result = SearchPrev( m_findReplData.GetFlags() /* searchFlags */, m_findReplData.GetFindString() );
   m_ourFrame->SetStatusText(""); //! Clear
   if( result != -1 )  Found(result);
   else {
        int moveFwd =   GetCurrentPos() + m_findReplData.GetFindString().Length() + 1;
        if (moveFwd < GetLength()) GotoPos( moveFwd );  //! Put caret back
        str.Printf("Finished searching for: \"%s\"", m_findReplData.GetFindString());  m_ourFrame->SetStatusText(str);
   }
}

void ZionEdit::Found( int result, bool rtFind )
{   wxString str; int max;
    EnsureVisible( LineFromPosition(result) );  //! Make sure it's unfolded
    if( !rtFind ) {
        max = ( m_findReplData.GetFlags() & ZION_FR_FLAG_REGEX ) ? m_lastFind_max //GetLineEnd()  /*  from stc.h */
                : result + m_findReplData.GetFindString().Length();
    }
    else {
        max = ( m_findReplData2.GetFlags() & ZION_FR_FLAG_REGEX ) ? m_lastFind_max //GetLineEnd()  /*  from stc.h */
                : result + m_findReplData2.GetFindString().Length();
    }
    SetSelection( result, max );   //! Hilight it ///Apparently moves our currentPos to the endOfSelection also.!!!!
    m_incrementalStartPos = result;  ///track last found position
    result = LineFromPosition(result); result += 1;  ///reuse
}

void ZionEdit::OnFindNext( wxCommandEvent & WXUNUSED(event) ) ///THIS IS CALLED FROM VARIOUS PLACES
{
    if( GetSelectionStart() == GetSelectionEnd() && m_findReplData.GetFindString().Length() == 0 ) {
        m_ourFrame->SetStatusText( _T("EMPTY SEARCH STRING!") );  return;
    }
    m_searchStartPos=GetCurrentPos() + 1; FindNext(); ///"Interactive Mode" - work with selection if there's one
}

void ZionEdit::OnClearFind( wxCommandEvent & WXUNUSED(event) )
{
    //Diag//m_ourFrame->SetStatusText(_T("In ClearFind"));
    m_findReplData.m_findStrs.Clear();
    if(m_dlgReplace) m_dlgReplace->RedrawFRCombo(true); ///true for find
}

void ZionEdit::OnClearRepl( wxCommandEvent & WXUNUSED(event) )
{
    //Diag//m_ourFrame->SetStatusText(_T("In ClearRepl"));
    m_findReplData.m_replStrs.Clear();
    m_dlgReplace->RedrawFRCombo(false); ///false for repl
}


/*! LEXER Stuff */ //--------------------------------------------------------------------------------------------------------------

void ZionEdit::OnChooseLexer(wxCommandEvent & event)
{
    int savePos = GetCurrentPos();
    if( GetWrapMode() != wxSTC_WRAP_NONE ) GotoPos(0);  ///workaround wrap bug
    /*if( CheckAllowSave(_T("Reloading Styles...\n")) )*/
    ReloadFromBuffer( event.GetId() - ZION_CONTAINER, savePos, true );//m_currentLexer = event.GetId() - ZION_CONTAINER;  //!Work out lexer constant
}

void ZionEdit::OnReload( wxCommandEvent & event )
{
    m_ourFrame->PrependToPopup( event.GetId() ); ///LAST CMD to popup
    if( CheckAllowSave() ) {
        int a = GetFirstVisibleLine();
        Reload(m_currentLexer, GetCurrentPos());
        ///Maintain caret position
        int b = GetFirstVisibleLine();
        if( a != b ) LineScroll(0, a - b ); ///cols, lines // -ve will scroll backwards.
        m_bookmarks.Empty();  ///clear our bookmarks
        m_bookIdx.Empty();  ///clear our indices
    }
}

int ZionEdit::GetCurrentLexer() { return m_currentLexer; }

void ZionEdit::CleanMenuBar( const wxString & menuString )
{
    int position = -1;
    position = m_ourFrame->m_menuBar->FindMenu(menuString);
    if( position != wxNOT_FOUND )  m_ourFrame->m_menuBar->Remove( position );
}

void ZionEdit::CleanMenu( )
{
    wxMenuItem * item = NULL;
    
    ///REMOVE FROM MENUBAR
    CleanMenuBar("CSS"); 
    CleanMenuBar("HTML");
    CleanMenuBar("C++");    /// '&' Doesn't matter in search string
    CleanMenuBar("Java"); 
    CleanMenuBar("C#"); 
    CleanMenuBar("PERL");
    CleanMenuBar("PYTHON");
    CleanMenuBar("RUBY");
    CleanMenuBar("VB");
    CleanMenuBar("SQL");
    CleanMenuBar("ASM");
    CleanMenuBar("FORTRAN");
    CleanMenuBar("FORTRAN77");
    CleanMenuBar("LISP");
    CleanMenuBar("BATCH");
    CleanMenuBar("MAKEFILE");
    
     ///REMOVE FROM POPUP
//    menuID = m_popup->FindItem(menuString); //! menu id returned ///There is a bug in menu::FindItem(const & wxString). It keeps finding "HTML": when it isn't there
    
    if( (item = m_popup->FindItem( ZION_HTMLMENU )) != NULL )  m_popup->Remove(item);
    if( (item = m_popup->FindItem( ZION_CPPMENU )) != NULL )  m_popup->Remove(item);
    if( (item = m_popup->FindItem( ZION_MAKEMENU )) != NULL )  m_popup->Remove(item);
    if( (item = m_popup->FindItem( ZION_RUBYMENU )) != NULL )  m_popup->Remove(item);
    if( (item = m_popup->FindItem( ZION_VBSCRIPTMENU )) != NULL )  m_popup->Remove(item);
    if( (item = m_popup->FindItem( ZION_PERLMENU )) != NULL )  m_popup->Remove(item);
    if( (item = m_popup->FindItem( ZION_CSSMENU )) != NULL )  m_popup->Remove(item);
    if( (item = m_popup->FindItem( ZION_PYTHONMENU )) != NULL )  m_popup->Remove(item);
    if( (item = m_popup->FindItem( ZION_SQLMENU )) != NULL )  m_popup->Remove(item);
    if( (item = m_popup->FindItem( ZION_LISPMENU )) != NULL )  m_popup->Remove(item);
    if( (item = m_popup->FindItem( ZION_FORTRANMENU )) != NULL )  m_popup->Remove(item);
    if( (item = m_popup->FindItem( ZION_FORTRAN77MENU )) != NULL )  m_popup->Remove(item);
    if( (item = m_popup->FindItem( ZION_BATCHMENU )) != NULL )  m_popup->Remove(item);
    if( (item = m_popup->FindItem( ZION_ASMMENU )) != NULL )  m_popup->Remove(item);
    
    ///Remove recent item from Popup
    if( m_ourFrame->m_recentPopup != NULL ) { ///Clean first -- TODO - put this in lexer setup also
        m_popup->Remove( m_ourFrame->m_recentPopup );
        m_ourFrame->m_recentPopup = NULL;  ///Make absolutely sure. Menus will cause a crash in a heartbeat!
    }
}

bool ZionEdit::ParseCallTipFile( const wxString & fileName, smallSyntax & aSyntax )
{
    ///START OF FILE READING FUNCTION
    if( ! wxFileName::FileExists( fileName ) ) {
        //m_ourFrame->SetStatusText(wxString::Format(_T("Could not find API file: %s"), fileName ) ); ///DIAG
        return false;
    }
    wxString buffer;
    wxFFile * file = new wxFFile( fileName, "r" );
    if( ! file->ReadAll( &buffer ) )  return false; ///UTF8 is fine for now
    file->Close();
    delete file;  ///release the object
    
    int pos = 0;
    wxString lastUnqFunc = _T("$!*'!$~$-^$k$@"); ///What are the chances of a f() found first with this name?
    wxString str, descript, fullDef;
    CallTipsHash::iterator it;  ///This will help us search if a given key (unique function name) exists in the call tips hash

    wxStringTokenizer tknzr( buffer, _T("\n"), wxTOKEN_STRTOK ); ///empty tokens never returned
    char ch;  ///for storing 1st char on line
    while( tknzr.HasMoreTokens() ) {
        str = tknzr.GetNextToken();  if( str.Last() == '\r' )  str.RemoveLast();
        
        if( str == _T("") ) continue; ///skip blank lines
        
        ch = str.GetChar(0);
        ///INVALID LINE
        if( ch == ' ' ) continue;    ///Lines starting with a space are skipped
        
        ///COMMENT
        if( ch == '#' ) continue;  ///COMMENT LINE - ONLY IF FIRST CHAR ON LINE
        
        ///INCLUDE
        if( ch == '<' ) {  ///Do include function that will call the File Reading Function recursively
            int p; wxString fname; char c;
            for( p= 1; p < str.Len(); p++ ) {
                if( (c= str.GetChar(p)) == '>' )  break;
                fname += c;
            }
            ParseCallTipFile( g_pathToApi  + "\\" + fname, aSyntax );  ///Recursive call
            continue;  ///Go to the next line
        }
        //----------------------------------------------------------Main Checks --------------------------------------------------------------------------------------------------------------
        ///SHORTCUTS ///Shortcuts are global aliases to a known attribute or object type
        ///Since attributes represent keys, both keys and values will be stored as lowercase
        if( ch == '!' && (pos= str.Find('.', true)) != -1 ) { ///true == from end
            wxString obj = str.Mid(1, pos-1).Lower(); ///All keys are stored as lower, so when referring to a key as a value, it must be lower too.
            wxString mem = str.Mid(pos+1).Lower(); /// ditto
            Str2StrHash::iterator it;
            ///Deal with list
            if( mem.Find(',') != -1 ) {  /// if list
                wxString token;
                wxStringTokenizer tknzr( mem, _T(","), wxTOKEN_STRTOK ); ///empty tokens nor delimiter never returned
                while( tknzr.HasMoreTokens() ) {
                    token = tknzr.GetNextToken().Lower();
                    if( token.GetChar(0) == ' ' ) token.Remove(0, 1); ///Remove 1 extraneous space at start of token
                    //if( token.Last() == ' ' ) token.RemoveLast();  //We won't do this for performance gain.
                    g_subs[ token.Lower() ] = obj;  ///Overwrite - no problem
                }
            }
            else g_subs[mem] = obj; ///Map aliases to base function or obj
        }
        ///JavaScript style values
        else if( ch == '=' && (pos= str.Find(':' /*, true*/ )) != -1 ) {
            wxString obj = str.Mid(1, pos-1).Lower();
            wxString mem = str.Mid(pos+1);
            ExtractHtmlAttrsWDescrip( g_jsStyleVals, obj, mem );  ///TODO - may need to separate from HTML Tag Extract
        }
        ///[Early] OBJECT MEMBER listing - This is the new method
        else if( ch == '.' ) { ///Only lines starting with a '.' can have a description.
            str.Remove(0,1);  ///Remove the '.'
            int pComma, pTilde, dPos = -1;
            wxString upperStr; bool tildeLwr = false;
            pComma= str.Find(',');
            pTilde= str.Find('~');
            if( pTilde != -1 && pComma != -1 ) { ///Both found, use lesser
                if( pComma < pTilde )  dPos = pComma;
                else {
                    dPos =  pTilde;
                    tildeLwr = true; ///There is a possible tag description
                }
            }
            else if( pTilde != -1 ) dPos = pTilde;
            else if( pComma != -1 ) dPos = pComma;
            ///else dPos will be -1
            if( dPos != -1 ) upperStr = str.Mid( 0, dPos );  ///separate off description or list
            else upperStr = str;  ///no description or list
            ///FROM HERE ON WE WILL SEARCH upperStr only for triggers instead of all of 'str'; Offsets from left will be equal
            ///STANDARD OBJ MEM ///TODO - HANDLE MULTILEVEL OBJ!!!!
            int splitPt;
            if( (dPos =upperStr.Find('.')) != -1 ) { ///Find from left
                splitPt = upperStr.Find('.', true);  ///from right - This will be our true split point
                upperStr = str.Mid( 0, splitPt ); ///Separate obj from member listing
                wxString mem = str.Mid( splitPt +1 );
                if( dPos != splitPt ) { ///We have > 1  '.' and need to tokenize
                    wxArrayString objs;
                    wxStringTokenizer tknzr( upperStr, wxString('.'), wxTOKEN_STRTOK ); ///empty tokens nor delimiter never returned
                    while( tknzr.HasMoreTokens() )  objs.Add( tknzr.GetNextToken() );
                    int cnt =objs.GetCount();
                    for( int i=0; i < cnt -1; i++)  ExtractObjMems( g_objMems, objs[i], objs[i+1] ); ///Establish obj relations - these cannot have descriptions
                    ExtractObjMemsWDescrip( g_objMems, objs[cnt-1], mem ); ///Last generation - Std extraction
                }
                else ExtractObjMemsWDescrip( g_objMems, upperStr, mem ); ///Std extraction
            }
            
            ///HTML Tags and Attrib NAMES
            else if( m_currentLexer == wxSTC_LEX_HTML && (pos= upperStr.Find('<')) != -1 ) {
                if( tildeLwr && upperStr.GetChar(pos +1) == ':') { ///There is a  tag description - upperStr will not include the '~'
                    wxString descript = upperStr.Mid( pos+2 );
                    ExtractHtmlAttrsWDescrip( g_htmlTags, str.Mid(0, pos), str.Mid( upperStr.Len()+1 ), true, descript ); ///Exclude tag descript ///true  - add to keyword list  
                    //m_ourFrame->SetStatusText( descript );  ///DIAG
                }
                else ExtractHtmlAttrsWDescrip( g_htmlTags, str.Mid(0, pos), str.Mid(pos+1), true );  ///true  - add to keyword list  
            }
            ///CSS and Scope resolution
            else if( (pos= upperStr.Find(':')) != -1 ) {  ///Find will find the first occurence
                if( upperStr.GetChar( pos+1 ) == ':' ) {  ///Scope resln operator
                    ExtractObjMemsWDescrip( g_scopeMems, str.Mid(0, pos), str.Mid(pos+2) );  
                }
                else ExtractObjMemsWDescrip( g_cssAttrs, str.Mid(0, pos), str.Mid(pos+1) );  
            }
            ///HTML /CSS ATTRIB VALUES
            else if( m_currentLexer == wxSTC_LEX_HTML || m_currentLexer == wxSTC_LEX_CSS ) { ///This is kind of a broad assumption
                if( (pos= upperStr.Find('=')) != -1 )  ExtractObjMemsWDescrip( g_htmlAttrs, str.Mid(0, pos), str.Mid(pos+1) );  
            }
        }

        ///BATCH LIKE FUNCTIONS
        else if( aSyntax.batchLike &&  (pos= str.Find('~')) != -1 && str.GetChar(pos - 1) != ')' && str.GetChar(pos - 2) != ')' ) {  /// '~' is the new universal  description separator
                    ///if ~ found and there is not a ')' up to 2 places before it
            descript = str.Mid(pos + 1); 
            str = str.Mid( 0, pos ); ///Done with description
            fullDef = str;
            
            if( (pos= str.Find(' ')) != -1 ) { ///If we can split off just Function name to be used as key
                str = str.Mid( 0, pos );
            }//else rest of the line is function name
            
            it = g_callTips.find( str.Lower() );  ///We normalize keys to lowercase
            if( it == g_callTips.end() ) { ///Not found
                g_callTips[ str.Lower() ] = new MyCallTip;
            }
            g_callTips[ str.Lower() ]->m_fullDefs.Add( fullDef );  ///Store this overload...
            g_callTips[ str.Lower() ]->m_descripts.Add( descript );   ///and its description
            
            apiKWList += str + _T(" "); /*.Lower()*/
            ///If Worthy, Add to Global AC list
            if( str.Len() > 3 ) {
                apiACList += str + _T(" ");
            }
            ///We don't need this for batch since we are not finding subparts of the unique funciton    //if( lastUnqFunc != str );
        }
        
        ///C-LIKE
        else if( aSyntax.cLike && (pos=str.Find(')')) != -1  /*&& aSyntax.name != wxString(_T("BATCH"))*/ ) {  ///BATCH is an older syntax format and doesn't comply well with C-like parens.
            descript = str.Mid(pos + 1); ///12132008 - change from "pos +2"
            str = str.Mid( 0, pos +1 );  ///Chop off description, but include last ')'
            fullDef = str;
            ///Pick up FUNCTION NAME ///Function Names can include an object and member
            if( (pos=str.Find('(')) != -1 ) {
                str = str.Mid( 0, pos );  ///Down to the bare function name
                ///Determine OBJ Members - Extract only the last two members
                wxArrayString objs;
                wxStringTokenizer tknzr( str, wxString('.'), wxTOKEN_STRTOK ); ///empty tokens nor delimiter never returned
                while( tknzr.HasMoreTokens() )  objs.Add( tknzr.GetNextToken() );
                int cnt= objs.GetCount(); ///Remember tokenizer will return at least ONE token
                if( (cnt= objs.GetCount()) > 1 ) {
                    str = objs[cnt-2] + '.' + objs[cnt-1];  ///2nd to last and last obj levels
                }
                ///Determine if function name UNIQUE
                it = g_callTips.find( str.Lower() );  ///We normalize keys to lowercase
                if( it == g_callTips.end() ) { g_callTips[ str.Lower() ] = new MyCallTip; }  ///Not found
                g_callTips[ str.Lower() ]->m_fullDefs.Add( fullDef );  ///Store this overload...
                ///If there is a description for all overloads (need only list with one overload) eg. Sum(a, b) ~Adds a and b ~ Returns int. Adds the values of A and B. Use with caution.
                wxString oDscrp;  bool oDscrpPresent = false;
                descript.Trim(false);  ///from left
                if( descript.GetChar(0) == '~' ) { //descript.Remove(0, 1); ///we want to keep the '~'
                    oDscrpPresent = true;
                    if( (pos= descript.Mid(1).Find('~')) != -1 ) {  ///if second one found
                        oDscrp = descript.Mid(0, pos+1); ///we want the '~' included
                        descript = descript.Mid(pos+2); ///rest of the str
                        //m_ourFrame->SetStatusText(oDscrp);  ///DIAG
                    }
                    else {
                        oDscrp = descript; ///oDscrp will have a '~' in front ->This is needed by ExtractObjMemsWDescrip
                        descript.Remove(0, 1);
                    }
                }
                g_callTips[ str.Lower() ]->m_descripts.Add( descript );   ///and its description
                
                if( str != lastUnqFunc ) { ///New Function Name. Overload Descripts must be placed in the first list overload.
                    lastUnqFunc = str;
                    
                    ///Create automatic obj associations - NOTE this will not take place if the preceding single obj level function is the same.
                    if( cnt > 1 ) { ///Remember tokenizer will return at least ONE token
                        for( int i=1; i < cnt; i++ ) {
                            if( i == cnt -1 && oDscrpPresent ) { ///LAST PAIR and there is an overload description, no commas allowed
                                /*if( oDscrp.Find(',') != -1 ) {
                                    m_ourFrame->SetStatusText( oDscrp.Prepend(_T("Commas not allowed in overload description: ") ) );  ///DIAG
                                    ExtractFunc( g_objMems, objs[i -1], objs[i] ); ///standard extract
                                }
                                else*/ ExtractFunc( g_objMems, objs[i -1], objs[i] + oDscrp ); ///Create final assocn
                            }
                            else ExtractObjMems( g_objMems, objs[i -1], objs[i] ); ///Create assocns for each obj relation ///standard extract
                        }
                    }
                    ///Scope resolution
                    else if( (pos= str.Find(':')) != -1 && str.GetChar( pos+1 ) == ':' ) {  ///Find will find the first occurence ///hopefully the compiler follows short circuit rules
                        wxString mem = str.Mid(pos+2);
                        ExtractFunc( g_scopeMems, str.Mid(0, pos), mem );  
                    }
                     ///Add Unique Function to Global AC list 
                    else {
                        apiACList += str + _T(" ");
                        apiKWList += str /*.Lower()*/ + _T(" ");
                    }
                }
            } 
        }
        ///BATCH LIKE FUNCTIONS without '~'
        else if( aSyntax.batchLike &&  (pos= str.Find(' ')) != -1 ) { ///Here the whole string is the full definition, we cannot isolate description
            fullDef = str;
            descript = _T("");
            str = str.Mid( 0, pos ); ///Just Function name
            it = g_callTips.find( str.Lower() );  ///We normalize keys to lowercase
            if( it == g_callTips.end() ) { ///Not found
                g_callTips[ str.Lower() ] = new MyCallTip;
            }
            g_callTips[ str.Lower() ]->m_fullDefs.Add( fullDef );  ///Store this overload...
            g_callTips[ str.Lower() ]->m_descripts.Add( descript );   ///and its description
            
            apiKWList += str + _T(" "); /*.Lower()*/
            ///If Worthy, Add to Global AC list
            if( str.Len() > 3 ) {
                apiACList += str + _T(" ");
            }
        } /// At this point we COULD NOT FIND A FUNCTION DEFINITION

        ///Simple AC
        else {
            apiACList += str + _T(" ");  ///Treat the whole string as an autocomplete string
            apiKWList += str + _T(" "); /*.Lower()*/
        }
    }
    
    return true;
}

bool ZionEdit::LoadCallTips(smallSyntax & aSyntax)
{
    g_callTips.clear();   ///Clear the big hash /// Rem hash uses lower case f(); empty is only a bool check
    g_objMems.clear();
    g_scopeMems.clear();
    g_cssAttrs.clear();
    g_subs.clear();
    g_htmlTags.clear();
    g_tagsList.clear();
    g_tagsList["Tags"] = new MemberList; ///There will only be one key in this hash
    g_htmlAttrs.clear();
    g_jsStyleVals.clear();
    apiACList = _T("");  ///Init our global [extra] AC list
    apiKWList = _T("");  ///Init our global [extra] Keyword list
    apiTagList = _T("");

    ParseCallTipFile( g_pathToApi + "\\" + aSyntax.callTipFile, aSyntax );
    
    ///SORT ALL m_arrays  of MemberList hashes, and PRODUCE m_list
    SortMemList( g_objMems );
    SortMemList( g_cssAttrs );
    SortMemList( g_htmlTags );
    SortMemList( g_tagsList );
    SortMemList( g_htmlAttrs );
    SortMemList( g_jsStyleVals );
    SortMemList( g_scopeMems );
    
    return true;  //status;
}

void ZionEdit::SortMemList( MemberLists & hash ) 
{
    MemberLists::iterator itm;
    for( itm=hash.begin(); itm != hash.end(); itm++ ) {
        itm->second->m_array.Sort(/*MySort*/);  ///List (below) must be sorted for AC to work
        itm->second->m_list = wxEmptyString;
        for( int j=0; j < itm->second->m_array.GetCount(); j++ ) {
            itm->second->m_list += itm->second->m_array[j] + _T(" ");
        }
        //we use the m_array later - don't clear // itm->second->m_array.Clear(); ///We won't release memory since the hash objs are cleared in LoadCallTips();
    }
}

void ZionEdit::DoHtmlColors( MemberLists & hash, const wxString & objLwr)
{
    hash[objLwr]->m_array = wxArrayString( 147, htmlColors );
}

void ZionEdit::ExtractHtmlAttrsWDescrip( MemberLists & hash, const wxString & obj, wxString mem, bool addToTagList, const wxString & tagDescript )
{
    wxString objLwr = obj.Lower();
    ///Ready the object ///Determine if obj name unique
    MemberLists::iterator it;
    it = hash.find( objLwr );  ///We normalize keys to lowercase
    if( it == hash.end() ) { ///Not found
        hash[ objLwr ] = new MemberList;
        objLwr.StartsWith("asp:") ? g_tagsList["Tags"]->m_array.Add(objLwr) : g_tagsList["Tags"]->m_array.Add(obj);  ///Save the tag with case
        g_tagsList["Tags"]->m_descrip[obj] = tagDescript;
        if( addToTagList ) apiTagList += obj + _T(" "); /// Build keyword list for HTML lexer
    }
    ///Deal with list
    if( mem.Find(',') != -1 ) {  /// if list
        wxStringTokenizer tknzr( mem, _T(","), wxTOKEN_STRTOK ); ///empty tokens nor delimiter never returned
        wxString str;
        while( tknzr.HasMoreTokens() ) {
            str = tknzr.GetNextToken();
            if( str.GetChar(0) == ' ' ) str.Remove(0, 1); ///Remove 1 extraneous space at start of token
            //if( token.Last() == ' ' ) token.RemoveLast();  ///performance?
            
            if( /*m_currentLexer == wxSTC_LEX_HTML &&*/ str == _T("dhtml") ) {
                hash[objLwr]->m_array.Add(_T("onclick") );
                hash[objLwr]->m_array.Add(_T("ondblclick") );
                hash[objLwr]->m_array.Add(_T("onkeydown") );
                hash[objLwr]->m_array.Add(_T("onkeypress") );
                hash[objLwr]->m_array.Add(_T("onkeyup") );
                hash[objLwr]->m_array.Add(_T("onmousedown") );
                hash[objLwr]->m_array.Add(_T("onmousemove") );
                hash[objLwr]->m_array.Add(_T("onmouseout") );
                hash[objLwr]->m_array.Add(_T("onmouseover") );
                hash[objLwr]->m_array.Add(_T("onmouseup") );
            }
            else if( str.Lower() == _T("webctrl") ) {
                AddWebCtrl( g_htmlTags, objLwr );
                //hash[objLwr]->m_descrip[""] = _T("");
            }
            else if( str.Lower() == _T("style") ) {
                hash[objLwr]->m_array.Add( str );
                hash[objLwr]->m_descrip[str] = _T("Change colors, fonts, size, positioning, and various other formatting attributes");
            }
            else if( str.Lower() == _T("htmlcolors") )  hash[objLwr]->m_array = wxArrayString( 147, htmlColors ); ///This will overwrite the list so htmlcolors must be the first item in an attribute list
            ///Here we can do further checks for description
            else {
                ///Extract Description
                int a=0; wxString descrip;
                if( (a=str.Find('~')) != -1 ) {
                    descrip = str.Mid(a+1); ///we can do empty checks on use
                    str = str.Mid(0, a);
                    hash[objLwr]->m_descrip[str] = descrip;
                }
                hash[objLwr]->m_array.Add( str );
            }
        }
    }
    else if( mem.Lower() == _T("htmlcolors") )  hash[objLwr]->m_array = wxArrayString( 147, htmlColors ); ///This will overwrite the list
    else if( mem.Lower() == _T("webctrl") )  AddWebCtrl( g_htmlTags, objLwr );
    else {
        ///Extract Description
        int a=0; wxString descrip;
        if( (a=mem.Find('~')) != -1 ) {
            descrip = mem.Mid(a+1); ///we can do empty checks on use
            mem = mem.Mid(0, a);
            hash[objLwr]->m_descrip[mem] = descrip;
        }
        hash[ objLwr ]->m_array.Add( mem );  ///Store the member as a whole
    }
}

void ZionEdit::AddWebCtrl( MemberLists & hash, const wxString & objLwr, bool methods ) {
    hash[objLwr]->m_array.Add("accesskey" );
    hash[objLwr]->m_descrip["accesskey"] = _T("Gets or sets the access key for the Web server control");
    hash[objLwr]->m_array.Add("backcolor" );
    hash[objLwr]->m_descrip["backcolor"] = _T("Gets or sets the background color");
    hash[objLwr]->m_array.Add("bindingContainer" );
    hash[objLwr]->m_descrip["bindingContainer"] = _T("Gets the control that contains this control's data binding");
    hash[objLwr]->m_array.Add("bordercolor" );
    //hash[objLwr]->m_descrip[""] = _T("");
    hash[objLwr]->m_array.Add("borderstyle" );
    //hash[objLwr]->m_descrip[""] = _T("");
    hash[objLwr]->m_array.Add("borderwidth" );
    //hash[objLwr]->m_descrip[""] = _T("");
    hash[objLwr]->m_array.Add("clientID" );
    hash[objLwr]->m_descrip["clientID"] = _T("Gets the server control identifier generated by ASP.NET");
    hash[objLwr]->m_array.Add("Controls" );
    hash[objLwr]->m_descrip["Controls"] = _T("Gets a ControlCollection object that represents the child controls for a specified server control in the UI hierarchy. (Inherited from Control.)");
    hash[objLwr]->m_array.Add("CssClass" );
    hash[objLwr]->m_descrip["CssClass"] = _T("Gets or sets the Cascading Style Sheet (CSS) class rendered by the Web server control on the client");
    hash[objLwr]->m_array.Add("enabled" );
    hash[objLwr]->m_descrip["enabled"] = _T("Gets or sets a value indicating whether the Web server control is enabled");
    hash[objLwr]->m_array.Add("font" );
    hash[objLwr]->m_descrip["font"] = _T("Gets the font properties associated with the Web server control");
    hash[objLwr]->m_array.Add("forecolor" );
    hash[objLwr]->m_descrip["forecolor"] = _T("Gets or sets the foreground color (typically the color of the text) of the Web server control");
    hash[objLwr]->m_array.Add("HasAttributes" );
    hash[objLwr]->m_descrip["HasAttributes"] = _T("Gets a value indicating whether the control has attributes set");
    hash[objLwr]->m_array.Add("height" );
    hash[objLwr]->m_descrip["height"] = _T("Gets or sets the height of the Web server control");
    hash[objLwr]->m_array.Add("id" );
    hash[objLwr]->m_array.Add("ondatabinding" );
    hash[objLwr]->m_descrip["ondatabinding"] = _T("Occurs when the server control binds to a data source");
    hash[objLwr]->m_array.Add("oninit" );
    hash[objLwr]->m_descrip["oninit"] = _T("Occurs when the server control is initialized");
    hash[objLwr]->m_array.Add("onload" );
    hash[objLwr]->m_descrip["onload"] = _T("Occurs when the server control is loaded into the Page object");
    hash[objLwr]->m_array.Add("onprerender" );
    hash[objLwr]->m_descrip["onprerender"] = _T("Occurs after the Control object is loaded but prior to rendering");
    hash[objLwr]->m_array.Add("onunload" );
    hash[objLwr]->m_descrip["onunload"] = _T("Occurs when the server control is unloaded from memory");
    hash[objLwr]->m_array.Add("Page" );
    hash[objLwr]->m_descrip["Page"] = _T("Gets a reference to the Page instance that contains the server control");
    hash[objLwr]->m_array.Add("Parent" );
    hash[objLwr]->m_descrip["Parent"] = _T("Gets a reference to the server control's parent control in the page control hierarchy");
    hash[objLwr]->m_array.Add("skinid" );
    hash[objLwr]->m_descrip["skinid"] = _T("Gets or sets the skin to apply to the control");
    hash[objLwr]->m_array.Add("style" );
    hash[objLwr]->m_descrip["style"] = _T("Gets a collection of text attributes that will be rendered as a style attribute on the outer tag of the Web server control");
    hash[objLwr]->m_array.Add("tabIndex" );
    hash[objLwr]->m_descrip["tabIndex"] = _T("Gets or sets the tab index of the Web server control");
    hash[objLwr]->m_array.Add("templateControl" );
    hash[objLwr]->m_descrip["templateControl"] = _T("Gets or sets a reference to the template that contains this control");
    hash[objLwr]->m_array.Add("tooltip" );
    hash[objLwr]->m_descrip["tooltip"] = _T("Gets or sets the hint text");
    hash[objLwr]->m_array.Add("UniqueID" );
    hash[objLwr]->m_descrip["UniqueID"] = _T("Gets the unique, hierarchically qualified identifier for the server control");
    hash[objLwr]->m_array.Add("visible" );
    hash[objLwr]->m_descrip["visible"] = _T("Gets or sets whether the server control is visible");
    hash[objLwr]->m_array.Add("width" );
    hash[objLwr]->m_array.Add("runat" );
    
    if( methods ) {  ///TODO - Add WebCtrl Methods - for std obj mem - code access to webctrl
        hash[objLwr]->m_array.Add("Focus" );
        hash[objLwr]->m_descrip["Focus"] = _T("Sets input focus to the control");
        hash[objLwr]->m_array.Add("HasControls" );
        hash[objLwr]->m_descrip["HasControls"] = _T("Determines if the server control contains any child controls");
        hash[objLwr]->m_array.Add("IsEnabled" );
        hash[objLwr]->m_array.Add("ToString" );
        hash[objLwr]->m_descrip["ToString"] = _T("Returns a String that represents the current Object");
    }
    
}

void ZionEdit::ExtractObjMems( MemberLists & hash, const wxString & obj, wxString mem )
{
    ///Determine if obj name unique
    MemberLists::iterator it;
    it = hash.find( obj.Lower() );  ///We normalize keys to lowercase
    if( it == hash.end() ) { ///Not found
        hash[ obj.Lower() ] = new MemberList;
        apiACList += obj + _T(" ");  ///Add Unique Function to Global AC list  ///These can be used as keyword2s
        apiKWList += obj + _T(" "); /*.Lower()*/
    }
    ///Deal with list
    wxString objLwr = obj.Lower();
    wxString memLwr = mem.Lower();
    if( mem.Find(',') != -1 ) {  /// if list
        wxString token;
        wxStringTokenizer tknzr( mem, _T(","), wxTOKEN_STRTOK ); ///empty tokens nor delimiter never returned
        while( tknzr.HasMoreTokens() ) {
            token = tknzr.GetNextToken();
            //////Remove 1 extraneous space at start of token
            if( token.GetChar(0) == ' ' ) token.Remove(0, 1); 
            //if( token.Lower() == _T("htmlcolors") ) DoHtmlColors( hash, objLwr );  ///TODO - can htmlcolors exist in a list? - No
            if( token.Lower() == _T("webctrl") )  AddWebCtrl( hash, objLwr );
            else hash[objLwr]->m_array.Add( token );
        }
    }
    ///No list
    else if( memLwr == _T("bool") )  hash[objLwr]->m_array = wxArrayString( 2, bools ); ///Deal with pseudo member value
    else if( memLwr == _T("htmlcolors") ) hash[objLwr]->m_array = wxArrayString( 147, htmlColors );  ///ditto
    else if( memLwr == _T("webctrl") ) AddWebCtrl( hash, objLwr );
    else if( hash[ objLwr ]->m_array.Index(mem.c_str(), false) == -1 ) { ///false = ignore case ///Check for duplicates in single strings. See what kind of a performance impact this makes
        hash[ objLwr ]->m_array.Add( mem );  ///Store the member as a whole
    }
}

void ZionEdit::ExtractObjMemsWDescrip( MemberLists & hash, const wxString & obj, wxString mem )
{
    ///Determine if obj name unique
    MemberLists::iterator it;
    it = hash.find( obj.Lower() );  ///We normalize keys to lowercase
    if( it == hash.end() ) { ///Not found
        hash[ obj.Lower() ] = new MemberList;
        apiACList += obj + _T(" ");  ///Add Unique Function to Global AC list  ///These can be used as keyword2s
        apiKWList += obj + _T(" "); /*.Lower()*/
    }
    ///Deal with list
    wxString objLwr = obj.Lower();
    if( mem.Find(',') != -1 ) {  /// if list - NO DUPLICATE CHECKS - for performance reasons
        wxString token;
        wxStringTokenizer tknzr( mem, _T(","), wxTOKEN_STRTOK ); ///empty tokens nor delimiter never returned
        while( tknzr.HasMoreTokens() ) {
            token = tknzr.GetNextToken();
            if( token.GetChar(0) == ' ' ) token.Remove(0, 1); ///Remove 1 extraneous space at start of token
            if( token.Last() == ' ' ) token.RemoveLast();  ///performance?
            //if( token.Lower() == _T("htmlcolors") ) DoHtmlColors( hash, objLwr );  ///TODO - can htmlcolors exist in a list?
            ///Extract Description
            int a=0; wxString descrip;
            if( (a=token.Find('~')) != -1 ) {
                descrip = token.Mid(a+1); ///we can do empty checks on use
                token = token.Mid(0, a);
                hash[objLwr]->m_descrip[token] = descrip;
            }
            ///Add the new child
            if( token.Lower() == _T("webctrl") ) {
                 AddWebCtrl( hash, objLwr );
            }
            ///Add the new child if not already in the list
            else if( hash[ objLwr ]->m_array.Index(token, false) == -1 ) { ///false = ignore case ///Check for DUPLICATES in single strings. See what kind of a performance impact this makes
                hash[ objLwr ]->m_array.Add( token ); ///Do it
            }
        }
    }
    ///No list
    else if( mem.Lower() == _T("bool") )  hash[objLwr]->m_array = wxArrayString( 2, bools ); ///Deal with pseudo member value
    else if( mem.Lower() == _T("htmlcolors") ) hash[objLwr]->m_array = wxArrayString( 147, htmlColors );  ///ditto
    else {
        ///Extract Description
        int a=0; wxString descrip;
        if( (a=mem.Find('~')) != -1 ) {
            descrip = mem.Mid(a+1); ///we can do empty checks on use
            mem = mem.Mid(0, a);
            hash[objLwr]->m_descrip[mem] = descrip;
        }
        ///Add the new child
        if( mem.Lower() == _T("webctrl") ) {
             AddWebCtrl( hash, objLwr );
        }
        ///Add the new child if not already in the list
        if( hash[ objLwr ]->m_array.Index(mem, false) == -1 ) { ///false = ignore case ///Check for DUPLICATES in single strings. See what kind of a performance impact this makes
            hash[ objLwr ]->m_array.Add( mem );  ///Store the member as a whole
        }
    }
}

void ZionEdit::ExtractFunc( MemberLists & hash, const wxString & obj, wxString mem )
{
    ///Determine if obj name unique
    MemberLists::iterator it;
    it = hash.find( obj.Lower() );  ///We normalize keys to lowercase
    if( it == hash.end() ) { ///Not found
        hash[ obj.Lower() ] = new MemberList;
        apiACList += obj + _T(" ");  ///Add Unique Function to Global AC list  ///These can be used as keyword2s
        apiKWList += obj + _T(" "); /*.Lower()*/
    }
    ///Extract Description - ///No list allowed
    wxString objLwr = obj.Lower();
    wxString descrip; int a;
    if( (a=mem.Find('~')) != -1 ) {
        descrip = mem.Mid(a+1); ///we can do empty checks on use
        mem = mem.Mid(0, a);
        hash[objLwr]->m_descrip[mem] = descrip;
    }
    ///Add the new child if not already in the list
    if( hash[ objLwr ]->m_array.Index(mem, false) == -1 ) { ///false = ignore case ///Check for DUPLICATES in single strings. See what kind of a performance impact this makes
        hash[ objLwr ]->m_array.Add( mem );  ///Store the member as a whole
    }
}

/*! LEXER Selection Engine */
void ZionEdit::SetupLexer(const int lexer, bool force)
{  
    if( lexer == m_currentLexer && ! force ) return;  /// Skip if lexer setup not necessary
    m_currentLexer = lexer;  /// Then update it!
    
    int savePos = -1;   //! Because of wrap bug we will go to pos 0 while changing styles
    if( GetWrapMode() ) { savePos = GetCurrentPos(); GotoPos(0); }
    wxString str;
    ClearDocumentStyle();   //! This clears out the effect of changing the number of StyleBits
    CleanMenu();  ///Menubar and popup menus
    
    int position = 0;  m_callTipsLoaded = false;
    
    switch (lexer)
    {
        case wxSTC_LEX_PYTHON:
            SetStyleBits(5);  SetLexer(lexer); //! Tell the edit module we are changing lexers
            SetKeyWords(0, pythonSyntax.words1); ///KEYWORDS ARE CASE SENSITIVE (SEEM TO WORK BEST AS ALL UPPER CASE OR ALL LOWER CASE
            SetKeyWords(1, "");
            SetKeyWords(2, "");
            SetKeyWords(3, "");
            SetKeyWords(4, "");
            SetKeyWords(5, "");
            SetProperty("fold","1");
            if( (g_ctActive || g_acActive) && LoadCallTips( pythonSyntax) )  m_callTipsLoaded = true;  ///Must load before ProcessSyntax
            ProcessSmallSyntax( pythonSyntax); 
            if( m_ourFrame->m_menuBar->FindMenu("&PYTHON") == wxNOT_FOUND ) {
                position = m_ourFrame->m_menuBar->GetMenuCount();
                m_ourFrame->m_menuBar->Insert( position -1, m_ourFrame->pythonMenu, "&PYTHON" ); //! Insert into main menu bar
                m_popup->Prepend( ZION_PYTHONMENU, "PYTHON", m_ourFrame->pythonMenu ); //! Append to our context menu
            }
            break;
        case wxSTC_LEX_RUBY:
            SetStyleBits(5);  SetLexer(lexer); //! Tell the edit module we are changing lexers
            SetKeyWords(0, rubySyntax.words1);
            SetKeyWords(1, "");
            SetKeyWords(2, "");
            SetKeyWords(3, "");
            SetKeyWords(4, "");
            SetKeyWords(5, "");
            SetProperty("fold","1");
            SetProperty("fold.comment", "1");
            SetProperty("comment.block", "#~" );
            if( (g_ctActive || g_acActive) && LoadCallTips(rubySyntax) )  m_callTipsLoaded = true;  ///Must load before ProcessSyntax
            ProcessSmallSyntax( rubySyntax); 
            if( m_ourFrame->m_menuBar->FindMenu("&RUBY") == wxNOT_FOUND ) {
                position = m_ourFrame->m_menuBar->GetMenuCount();
                m_ourFrame->m_menuBar->Insert( position -1, m_ourFrame->rubyMenu, "&RUBY" ); //! Insert into main menu bar
                m_popup->Prepend( ZION_RUBYMENU, "RUBY", m_ourFrame->rubyMenu ); //! Append to our context menu
            }
            break;
        case wxSTC_LEX_VBSCRIPT:
            SetStyleBits(5);  SetLexer(lexer); //! Tell the edit module we are changing lexers
            if( (g_ctActive || g_acActive) && LoadCallTips(vbscriptSyntax) )  m_callTipsLoaded = true;  ///Must load before Keywords set
            SetKeyWords(0, vbscriptSyntax.words1);
            SetKeyWords(1, apiKWList);
            SetKeyWords(2, _T(""));
            SetKeyWords(3, _T(""));
            SetKeyWords(4, _T(""));
            SetKeyWords(5, _T(""));
            SetProperty("fold","1");
            SetProperty("fold.comment", "1");
            SetProperty("comment.block", "#~" );
            ProcessSmallSyntax( vbscriptSyntax ); 
            if( m_ourFrame->m_menuBar->FindMenu("&VB") == wxNOT_FOUND ) {
                position = m_ourFrame->m_menuBar->GetMenuCount();
                m_ourFrame->m_menuBar->Insert( position -1, m_ourFrame->vbscriptMenu, "&VB" ); //! Insert into main menu bar
                m_popup->Prepend( ZION_VBSCRIPTMENU, "VB", m_ourFrame->vbscriptMenu ); //! Append to our context menu
            }
            break;
        case wxSTC_LEX_CSS: 
            SetStyleBits(5);  SetLexer(lexer); //! Tell the edit module we are changing lexers
            SetKeyWords(0, cssSyntax.words1); ///CSS1 Keywords
            SetKeyWords(1, cssSyntax.words2); ///Pseudo Classes
            SetKeyWords(2, cssSyntax.words3); ///CSS2 Keywords
            SetKeyWords(3, "");
            SetKeyWords(4, "");
            SetKeyWords(5, "");
            SetProperty("fold","1");
            if( (g_ctActive || g_acActive) && LoadCallTips(cssSyntax) )  m_callTipsLoaded = true;  ///Must load before ProcessSyntax
            ProcessSmallSyntax( cssSyntax);
            if( m_ourFrame->m_menuBar->FindMenu("&CSS") == wxNOT_FOUND ) {
                position = m_ourFrame->m_menuBar->GetMenuCount();
                m_ourFrame->m_menuBar->Insert( position -1, m_ourFrame->cssMenu, "&CSS" ); //! Insert into main menu bar
                m_popup->Prepend( ZION_CSSMENU, "CSS", m_ourFrame->cssMenu ); //! Append to our context menu
            }
            break;
        case wxSTC_LEX_SQL: 
            SetStyleBits(5);  SetLexer(lexer); //! Tell the edit module we are changing lexers
            SetKeyWords(0, sqlSyntax.words1);
            SetKeyWords(1, _T(""));
            SetKeyWords(5, sqlSyntax.words6); ///Userwords2
            SetProperty("fold","1");
            if( (g_ctActive || g_acActive) && LoadCallTips(sqlSyntax) )  m_callTipsLoaded = true;  ///Must load before ProcessSyntax
            ProcessSmallSyntax( sqlSyntax); 
            if( m_ourFrame->m_menuBar->FindMenu("&SQL") == wxNOT_FOUND ) {
                position = m_ourFrame->m_menuBar->GetMenuCount();
                m_ourFrame->m_menuBar->Insert( position -1, m_ourFrame->sqlMenu, "&SQL" ); //! Insert into main menu bar
                m_popup->Prepend(ZION_SQLMENU, "SQL", m_ourFrame->sqlMenu  ); //! Append to our context menu
            }
            break;
        case wxSTC_LEX_ASM: 
            SetStyleBits(5);  SetLexer(lexer); //! Tell the edit module we are changing lexers
            SetKeyWords(0, asmSyntax.words1);
            SetKeyWords(1, asmSyntax.words2);
            SetKeyWords(2, asmSyntax.words3);
            SetKeyWords(3, "");
            SetKeyWords(4, "");
            SetKeyWords(5, "");
            SetProperty("fold","1");
            SetProperty("fold.comment", "1");
            if( (g_ctActive || g_acActive) && LoadCallTips(asmSyntax) )  m_callTipsLoaded = true;  ///Must load before ProcessSyntax
            ProcessSmallSyntax( asmSyntax); 
            if( m_ourFrame->m_menuBar->FindMenu("&ASM") == wxNOT_FOUND ) {
                position = m_ourFrame->m_menuBar->GetMenuCount();
                m_ourFrame->m_menuBar->Insert( position -1, m_ourFrame->asmMenu, "&ASM" ); //! Insert into main menu bar
                m_popup->Prepend(ZION_ASMMENU, "ASM", m_ourFrame->asmMenu  ); //! Append to our context menu
            }
            break;
        case wxSTC_LEX_FORTRAN: 
            SetStyleBits(5);  SetLexer(lexer);
            SetKeyWords(0, fortranSyntax.words1);
            SetKeyWords(1, fortranSyntax.words2);
            SetKeyWords(2, fortranSyntax.words3);
            SetKeyWords(3, "");
            SetKeyWords(4, "");
            SetKeyWords(5, "");
            SetProperty("fold","1");
            SetProperty("fold.comment", "1");
            if( (g_ctActive || g_acActive) && LoadCallTips(fortranSyntax) )  m_callTipsLoaded = true;  ///Must load before ProcessSyntax
            ProcessSmallSyntax( fortranSyntax); 
            if( m_ourFrame->m_menuBar->FindMenu("&FORTRAN") == wxNOT_FOUND ) {
                position = m_ourFrame->m_menuBar->GetMenuCount();
                m_ourFrame->m_menuBar->Insert( position -1, m_ourFrame->fortranMenu, "&FORTRAN" ); //! Insert into main menu bar
                m_popup->Prepend(ZION_FORTRANMENU, "FORTRAN", m_ourFrame->fortranMenu  ); //! Append to our context menu
            }
            break;
        case wxSTC_LEX_F77:
            SetStyleBits(5);  SetLexer(lexer);
            SetKeyWords(0, fortranSyntax.words1);
            SetKeyWords(1, fortranSyntax.words2);
            SetKeyWords(2, fortranSyntax.words3);
            SetKeyWords(3, "");
            SetKeyWords(4, "");
            SetKeyWords(5, "");
            SetProperty("fold","1");
            SetProperty("fold.comment", "1");
            if( (g_ctActive || g_acActive) && LoadCallTips(f77Syntax) )  m_callTipsLoaded = true;  ///Must load before ProcessSyntax
            ProcessSmallSyntax( f77Syntax); 
            if( m_ourFrame->m_menuBar->FindMenu("&FORTRAN77") == wxNOT_FOUND ) {
                position = m_ourFrame->m_menuBar->GetMenuCount();
                m_ourFrame->m_menuBar->Insert( position -1, m_ourFrame->fortran77Menu, "&FORTRAN77" ); //! Insert into main menu bar
                m_popup->Prepend(ZION_FORTRAN77MENU, "FORTRAN77", m_ourFrame->fortran77Menu  ); //! Append to our context menu
            }
            break;
        case wxSTC_LEX_BATCH:
            SetStyleBits(5);  SetLexer(lexer);
            SetKeyWords(0, batchSyntax.words1);
            SetKeyWords(1, "");
            SetKeyWords(2, "");
            SetKeyWords(3, "");
            SetKeyWords(4, "");
            SetKeyWords(5, "");
            SetProperty("fold","1");
            if( (g_ctActive || g_acActive) && LoadCallTips(batchSyntax) )  m_callTipsLoaded = true;  ///Must load before ProcessSyntax
            ProcessSmallSyntax( batchSyntax); 
            if( m_ourFrame->m_menuBar->FindMenu("&BATCH") == wxNOT_FOUND ) {
                position = m_ourFrame->m_menuBar->GetMenuCount();
                m_ourFrame->m_menuBar->Insert( position -1, m_ourFrame->batchMenu, "&BATCH" ); //! Insert into main menu bar
                m_popup->Prepend(ZION_BATCHMENU, "BATCH", m_ourFrame->batchMenu  ); //! Append to our context menu
            }
            break;
        case wxSTC_LEX_LISP:
            SetStyleBits(5);  SetLexer(lexer);
            SetKeyWords(0, lispSyntax.words1);
            SetKeyWords(1, "");
            SetKeyWords(2, "");
            SetKeyWords(3, "");
            SetKeyWords(4, "");
            SetKeyWords(5, "");
            SetProperty("fold","1");
            if( (g_ctActive || g_acActive) && LoadCallTips(lispSyntax) )  m_callTipsLoaded = true;  ///Must load before ProcessSyntax
            ProcessSmallSyntax( lispSyntax); 
            if( m_ourFrame->m_menuBar->FindMenu("&LISP") == wxNOT_FOUND ) {
                position = m_ourFrame->m_menuBar->GetMenuCount();
                m_ourFrame->m_menuBar->Insert( position -1, m_ourFrame->lispMenu, "&LISP" ); //! Insert into main menu bar
                m_popup->Prepend(ZION_LISPMENU, "LISP", m_ourFrame->lispMenu  ); //! Append to our context menu
            }
            break;
        case wxSTC_LEX_MAKEFILE:
            SetStyleBits(5); SetLexer(lexer); 
            SetKeyWords(0, makeSyntax.words1);
            SetKeyWords(1, "");
            SetKeyWords(2, "");
            SetKeyWords(3, "");
            SetKeyWords(4, "");
            SetKeyWords(5, "");
            SetProperty("fold","0");
            if( (g_ctActive || g_acActive) && LoadCallTips(makeSyntax) )  m_callTipsLoaded = true;  ///Must load before ProcessSyntax
            ProcessSmallSyntax(makeSyntax);
            if( m_ourFrame->m_menuBar->FindMenu("&MAKEFILE") == wxNOT_FOUND ) {
                position = m_ourFrame->m_menuBar->GetMenuCount();
                m_ourFrame->m_menuBar->Insert( position -1, m_ourFrame->makeMenu, "&MAKEFILE" ); //! Insert into main menu bar
                m_popup->Prepend( ZION_MAKEMENU, "MAKEFILE", m_ourFrame->makeMenu ); //! Append to our context menu
            }
            break;
        case wxSTC_LEX_PERL:
            SetStyleBits(5);
            SetLexer(lexer);
            SetKeyWords(0, perlSyntax.words1);
            SetKeyWords(1, "");
            SetKeyWords(2, "");
            SetKeyWords(3, "");
            SetKeyWords(4, "");
            SetKeyWords(5, "");
            SetProperty("fold","1");
            if( (g_ctActive || g_acActive) && LoadCallTips(perlSyntax) )  m_callTipsLoaded = true;  ///Must load before ProcessSyntax
            ProcessSmallSyntax(perlSyntax); 
            if( m_ourFrame->m_menuBar->FindMenu("&PERL") == wxNOT_FOUND ) {
                position = m_ourFrame->m_menuBar->GetMenuCount();
                m_ourFrame->m_menuBar->Insert( position -1, m_ourFrame->perlMenu, "&PERL" ); //! Insert into main menu bar
                m_popup->Prepend(ZION_PERLMENU, "PERL", m_ourFrame->perlMenu  ); //! Append to our context menu
            }
            break;
        case wxSTC_LEX_HTML:
            SetStyleBits(7);
            SetLexer(lexer);
            if( (g_ctActive || g_acActive) && LoadCallTips(htmlSyntax) )  m_callTipsLoaded = true;  ///Must load before keywords set
        //    ///Sort APIKWList and Remove Duplicates
            SortKWList( apiKWList );
            apiTagList += htmlSyntax.words1;
            SortKWList( apiTagList );
            apiTagList = apiTagList.Lower();
            SetKeyWords(0, apiTagList ); // + htmlSyntax.words1);
            SetKeyWords(1, htmlSyntax.words2);
            SetKeyWords(2, htmlSyntax.words3);
            SetKeyWords(3, htmlSyntax.words4);
            SetKeyWords(4, htmlSyntax.words5);
            SetKeyWords(5, htmlSyntax.words6);
            SetProperty("fold","1");
            SetProperty("fold.comment", "1");
            SetProperty("fold.html", "1");
            ProcessSmallSyntax( htmlSyntax);
            if( m_ourFrame->m_menuBar->FindMenu("HT&ML") == wxNOT_FOUND ) {
                position = m_ourFrame->m_menuBar->GetMenuCount();
                m_ourFrame->m_menuBar->Insert( position -1, m_ourFrame->htmlMenu, "HT&ML" ); //! Insert into main menu bar
                m_popup->Prepend(ZION_HTMLMENU, "HT&ML", m_ourFrame->htmlMenu  ); //! Append to our context menu
            }
            break;
        case wxSTC_LEX_CPP:
            SetStyleBits(5);
            SetLexer(lexer);
            if( (g_ctActive || g_acActive) && LoadCallTips(cppSyntax) )  m_callTipsLoaded = true;
            SetKeyWords(0, cppSyntax.words1);
            SetKeyWords(1, apiKWList + cppSyntax.words2);  //);
            SetKeyWords(2, "");
            SetKeyWords(3, "");
            SetKeyWords(4, "");
            SetKeyWords(5, "");
            SetProperty("fold","1");
            SetProperty("fold.comment", "1");
            SetProperty("fold.preprocessor", "1");
            ProcessSmallSyntax( cppSyntax); 
            if( m_ourFrame->m_menuBar->FindMenu("C++") == wxNOT_FOUND ) {
                position = m_ourFrame->m_menuBar->GetMenuCount();
                m_ourFrame->m_menuBar->Insert( position -1, m_ourFrame->cppMenu, "&C++" );
                m_popup->Prepend(ZION_CPPMENU, "C++", m_ourFrame->cppMenu  );
            }
            break;
        case wxSTC_LEX_CS:
            SetStyleBits(5);  SetLexer(wxSTC_LEX_CPP); ///Set the real lexer. Lex_CS is a pseudo-lexer
            SetKeyWords(0, csSyntax.words1);
            SetKeyWords(1, csSyntax.words2);
            SetKeyWords(2, "");
            SetKeyWords(3, "");
            SetKeyWords(4, "");
            SetKeyWords(5, "");
            SetProperty("fold","1");
            SetProperty("fold.comment", "1");
            if( (g_ctActive || g_acActive) && LoadCallTips(csSyntax) )  m_callTipsLoaded = true;  ///Must load before ProcessSyntax
            ProcessSmallSyntax( csSyntax); 
            if( m_ourFrame->m_menuBar->FindMenu("C#") == wxNOT_FOUND ) {
                position = m_ourFrame->m_menuBar->GetMenuCount();
                m_ourFrame->m_menuBar->Insert( position -1, m_ourFrame->cppMenu, "&C#" ); ///We borrow the CPP menu
                m_popup->Prepend(ZION_CPPMENU, "C#", m_ourFrame->cppMenu  );
            }
            break;
        case wxSTC_LEX_JAVA:
            SetStyleBits(5);  SetLexer(wxSTC_LEX_CPP); ///Set the real lexer. Lex_Java is a pseudo-lexer
            SetKeyWords(0, javaSyntax.words1);
            SetKeyWords(1, javaSyntax.words2);
            SetKeyWords(2, "");
            SetKeyWords(3, "");
            SetKeyWords(4, "");
            SetKeyWords(5, "");
            SetProperty("fold","1");
            SetProperty("fold.comment", "1");
            if( (g_ctActive || g_acActive) && LoadCallTips(javaSyntax) )  m_callTipsLoaded = true;  ///Must load before ProcessSyntax
            ProcessSmallSyntax( javaSyntax); 
            if( m_ourFrame->m_menuBar->FindMenu("Java") == wxNOT_FOUND ) {
                position = m_ourFrame->m_menuBar->GetMenuCount();
                m_ourFrame->m_menuBar->Insert( position -1, m_ourFrame->cppMenu, "&Java" ); ///We borrow the CPP menu
                m_popup->Prepend(ZION_CPPMENU, "Java", m_ourFrame->cppMenu  );
            }
            break;
        case wxSTC_LEX_NULL:  ///TEXT
        default:
            SetStyleBits(5);
            SetLexer(lexer);
            if( (g_ctActive || g_acActive) && LoadCallTips(defaultSyntax) )  m_callTipsLoaded = true;  ///Must load callTips for text mode also so the structures can at least be cleared
            ProcessSmallSyntax( defaultSyntax );
            break;
    }
    
    if( savePos != -1 ) GotoPos(savePos); //! Restore pos, because of wrap bug
}

void ZionEdit::ProcessSmallSyntax(smallSyntax & aSyntax) //! Pull data out of the syntax struct
{
    wxString str, str2;
    /// Page Color must be set before StyleClearAll() so that the page can be cleared (filled) with the style 32's background
    ///Set Mode's Page Color
    StyleSetBackground( wxSTC_STYLE_DEFAULT, aSyntax.pageColor );
    /// Set the Remainder of the Default Style (32)
    str2 = _T("");  //!Check against empty wxString so we can use wxString's compare f()
    ( str2 == defaultSyntax.styles[0].bold ) ? StyleSetBold( defaultSyntax.styles[0].styleNum, false ) : //! This is the only way to turn of default bold (and italic)
            StyleSetBold( defaultSyntax.styles[0].styleNum, true );
    ( str2 == defaultSyntax.styles[0].italic ) ? StyleSetItalic( defaultSyntax.styles[0].styleNum, false ) :
            StyleSetItalic( defaultSyntax.styles[0].styleNum, true );
    str.Printf( "fore:#%s,face:%s,size:%s", defaultSyntax.styles[0].fore,
            defaultSyntax.styles[0].face, defaultSyntax.styles[0].size );
    StyleSetSpec( defaultSyntax.styles[0].styleNum, str );    //! [0] = STYLE_DEFAULT which is actually 32
    
    StyleClearAll(); //! set all styles to STYLE_DEFAULT, i.e. Fill the (page) background

    bool textMode = aSyntax.name == wxString(_T("DEFAULT"));
    if( !textMode ) {
        for( int i=0; i < aSyntax.stylesUsed; i++) {
            ( str2 == aSyntax.styles[i].bold ) ? StyleSetBold( aSyntax.styles[i].styleNum, false ) : //! This is the only way to turn of default bold (and italic)
                    StyleSetBold( aSyntax.styles[i].styleNum, true );
            (str2 == aSyntax.styles[i].italic ) ? StyleSetItalic( aSyntax.styles[i].styleNum, false ) :
                    StyleSetItalic( aSyntax.styles[i].styleNum, true );
            str.Printf( "fore:#%s,back:#%s,face:%s,size:%s", aSyntax.styles[i].fore, aSyntax.styles[i].back,
                    aSyntax.styles[i].face, aSyntax.styles[i].size );
            StyleSetSpec( aSyntax.styles[i].styleNum, str );
        }
    }

    ///Line Number margin
    SetMarginType (m_LNMarginID, wxSTC_MARGIN_NUMBER);  //! set as a number margin
    ///width should be set only after all font params are set//accomodate 999 lines
    switch( g_lineNumWidth) {
        case 1 : str = _T("19");  break;
        case 2 : str = _T("199");  break;
        case 3 : str = _T("1999");  break;
        case 4 : str = _T("19999");  break;
        case 5 : str = _T("199999");  break;
        case 6 : str = _T("1999999");  break;
    }
    m_LNMarginWidth = TextWidth( wxSTC_STYLE_LINENUMBER, str ); //!width should be set only after all font params are set//accomodate 999 lines
    
    SetUseTabs (aSyntax.option.usetabs); //! Usetabs
    SetTabWidth ( aSyntax.tabWidth );
    SetIndent( aSyntax.tabWidth );
    
    SetUseHorizontalScrollBar( aSyntax.option.horizSb );   
    SetUseVerticalScrollBar( aSyntax.option.vertSb );   
    int a= GetCurrentPos();  ///Workaround for nasty lock-up bug when wrapping is on (stc gives a length exceeded assertion)
//    if( GetWrapMode() == wxSTC_WRAP_NONE && (a= GetCurrentPos()) != 0 ) { ///Might be safer to use stc's GetWrapMode()
        GotoPos(0);
        SetWrapMode(wxSTC_WRAP_NONE);
        Colourise( 0, GetLength() - 1 );
        GotoPos(a);
//    }
//    else Colourise( 0, GetLength() - 1 );

    aSyntax.option.wrap ? SetWrapMode(wxSTC_WRAP_WORD ) :    //! Wrap or not?
            SetWrapMode(wxSTC_WRAP_NONE);
    m_LNVisible =  aSyntax.option.line; //! Line Numbers
    SetMarginWidth (m_LNMarginID, m_LNVisible? m_LNMarginWidth : 0);
    m_FoldMarginVisible =  aSyntax.option.fold; //! Fold margin
    SetMarginWidth (m_FoldMarginID, m_FoldMarginVisible? m_FoldMarginWidth : 0); ///Setting the width to zero makes it invisible
    
	SetIndentationGuides(aSyntax.option.indent); //! Indent

    SetViewWhiteSpace (aSyntax.option.viewws? wxSTC_WS_VISIBLEALWAYS: wxSTC_WS_INVISIBLE);
    SetFoldMarginColour(true, wxColour (_T("WHEAT")));
    SetFoldMarginHiColour( true, aSyntax.foldColor );  //g_foldColor
    
    ///Line Number Colors (each mode will set it's own)
    StyleSetForeground( defaultSyntax.styles[1].styleNum, aSyntax.lineForeColor );
    StyleSetBackground( defaultSyntax.styles[1].styleNum, aSyntax.lineBackColor );
    ///Selection Color
    SetSelBackground(true, aSyntax.selColor);   //! g_selColorSelection hilight color // 0xB0F0FF
    //! IndentGuides
    StyleSetForeground( defaultSyntax.styles[5].styleNum, aSyntax.indentColor);
    
    ///AC ///acList is static so lifetime extends beyond this f();
    acList = aSyntax.words1;
    if( aSyntax.lexer == wxSTC_LEX_SQL )  acList = acList.Upper();
    acList += " ";
    acList += aSyntax.words2;
    acList += " ";
    acList += aSyntax.words3;
    acList += " ";
    acList += aSyntax.words4;
    acList += " ";
    acList += aSyntax.words5;
    acList += " ";
    acList += aSyntax.words6;
    acList += " ";
    acList += apiACList;  ///Add words from API file
    if( aSyntax.lexer == wxSTC_LEX_HTML ) {
        acList += " ";
        acList += CSSWords;
    }
    
    acArrayList.Empty();
    wxStringTokenizer strings( acList );
    while( strings.HasMoreTokens() ) {
        str = strings.GetNextToken();
        if( str.Len() < 4 ) continue;  ///No sense including short strings in the list ///Lowest Option is AFTER 2 chars -- However short strings allowed for ref. benefits
        acArrayList.Add( str );
    }
    acArrayList.Sort(/* MySort */); ///The list must be sorted case sensitively for Auto Complete to work
    acList = "";
    for( int i=0; i < acArrayList.GetCount(); i++ ) {
        ///AVOID DUPLICATES
        if( i > 0 && acArrayList[i] == acArrayList[i-1] ) {
            acArrayList.RemoveAt(i);  i--; ///Indexes should move down
            continue; 
        }
        acList.Append( acArrayList[i] + " " );
        //acList.Append( ( aSyntax.lexer == wxSTC_LEX_SQL ? (acArrayList[i] =acArrayList[i].Upper()) : acArrayList[i] ) + " " );
    }
//    AutoCompSetIgnoreCase(true); //    AutoCompSetAutoHide( true );
    
    /// OTHER GLOBAL DEFAULT STYLES
    ///Caret Color
    SetCaretForeground(g_caretColor); SetCaretWidth(2);
    ///Line Numbers-- just the font -- colors are per mode.
    ( str2 == defaultSyntax.styles[1].bold ) ? StyleSetBold( defaultSyntax.styles[1].styleNum, false ) :   //! This is the only way to turn of default bold (and italic)
            StyleSetBold( defaultSyntax.styles[1].styleNum, true );
    ( str2 == defaultSyntax.styles[1].italic ) ? StyleSetItalic( defaultSyntax.styles[1].styleNum, false ) :
            StyleSetItalic( defaultSyntax.styles[1].styleNum, true );
    str.Printf( "face:%s,size:%s", defaultSyntax.styles[1].face, defaultSyntax.styles[1].size ); ///Color set per mode
    StyleSetSpec( defaultSyntax.styles[1].styleNum, str );
    //! Brace Highlight
    ( str2 == defaultSyntax.styles[2].bold ) ? StyleSetBold( defaultSyntax.styles[2].styleNum, false ) :   //! This is the only way to turn of default bold (and italic)
            StyleSetBold( defaultSyntax.styles[2].styleNum, true );
    ( str2 == defaultSyntax.styles[2].italic ) ? StyleSetItalic( defaultSyntax.styles[2].styleNum, false ) :
            StyleSetItalic( defaultSyntax.styles[2].styleNum, true );
    str.Printf( "fore:#%s,back:#%s,face:%s,size:%s",
            defaultSyntax.styles[2].fore, defaultSyntax.styles[2].back,
            defaultSyntax.styles[2].face, defaultSyntax.styles[2].size );
    StyleSetSpec( defaultSyntax.styles[2].styleNum, str );
        //! Brace BadLight
    ( str2 == defaultSyntax.styles[3].bold ) ? StyleSetBold( defaultSyntax.styles[3].styleNum, false ) :   //! This is the only way to turn of default bold (and italic)
            StyleSetBold( defaultSyntax.styles[3].styleNum, true );
    ( str2 == defaultSyntax.styles[3].italic ) ? StyleSetItalic( defaultSyntax.styles[3].styleNum, false ) :
            StyleSetItalic( defaultSyntax.styles[3].styleNum, true );
    str.Printf( "fore:#%s,back:#%s,face:%s,size:%s",
            defaultSyntax.styles[3].fore, defaultSyntax.styles[3].back,
            defaultSyntax.styles[3].face, defaultSyntax.styles[3].size );
    StyleSetSpec( defaultSyntax.styles[3].styleNum, str );

    ///Set Mode in StatusBar
    str.Printf("%s", ( textMode ? _T("Plain Text") : aSyntax.name ) ); //Mode:
    m_ourFrame->SetStatusText(str, 2);
}

void ZionEdit::SortKWList( wxString & list ) {
    ///Sort APIKWList and Remove Duplicates
    wxArrayString acArrayList2;  wxString str;
    wxStringTokenizer strings2( list );
    while( strings2.HasMoreTokens() ) {
        str = strings2.GetNextToken();
        //if( str.Len() < 2 ) continue;  ///No sense including short strings in the list ///Lowest Option is AFTER 2 chars -- However short strings allowed for ref. benefits
        acArrayList2.Add( str );
    }
    acArrayList2.Sort(/* MySort */); ///Sorting is vital to the step below
    list.Empty();
    for( int i=0; i < acArrayList2.GetCount(); i++ ) {
        ///AVOID DUPLICATES
        if( i > 0 && acArrayList2[i] == acArrayList2[i-1] ) {
            acArrayList2.RemoveAt(i);  i--; ///Indexes should move down
            continue; 
        }
        list.Append( acArrayList2[i] + " " );
    }
}

bool ZionEdit::SetPrefs()
{
    SetMargins(1, 2);    //left,right
    SetTabIndents (true);
    
    ///Init From Frames values.
    /// The frame has the user values ready for the edit module (after Readini)
    SetEOLMode(m_ourFrame->m_startEol);
    SetPrintColourMode( m_ourFrame->m_printColMode );
    m_dontPrintNums = m_ourFrame->m_dontPrintNums;
    m_htmlAC.active = m_ourFrame->m_htmlAC;
    
    m_startOpt = m_ourFrame->m_startOpt;
    m_startPos.x = m_ourFrame->m_startPos.x;
    m_startPos.y = m_ourFrame->m_startPos.y;
    m_startSize = m_ourFrame->m_startSize;
    
    SetBackSpaceUnIndents(true);
    ///Clear some hotkeys
    CmdKeyClear ('D', wxSTC_SCMOD_CTRL);
    CmdKeyClear ('L', wxSTC_SCMOD_CTRL);
    CmdKeyClear ('R', wxSTC_SCMOD_CTRL);
    CmdKeyClear ('R', wxSTC_SCMOD_SHIFT | wxSTC_SCMOD_CTRL);
    CmdKeyClear ('T', wxSTC_SCMOD_CTRL);
    CmdKeyClear ('U', wxSTC_SCMOD_CTRL);
    CmdKeyClear ('U', wxSTC_SCMOD_SHIFT | wxSTC_SCMOD_CTRL);
    CmdKeyClear ('W', wxSTC_SCMOD_CTRL);
    //! folding
    SetMarginType (m_FoldMarginID, wxSTC_MARGIN_SYMBOL);
    SetMarginMask (m_FoldMarginID, wxSTC_MASK_FOLDERS);
    SetMarginSensitive (m_FoldMarginID, true);

    SetFoldFlags (wxSTC_FOLDFLAG_LINEAFTER_CONTRACTED );
    //!Square headers
    MarkerDefine(wxSTC_MARKNUM_FOLDER, wxSTC_MARK_BOXPLUS);
    MarkerSetBackground (wxSTC_MARKNUM_FOLDER, wxColour (_T("WHEAT")));

    MarkerDefine(wxSTC_MARKNUM_FOLDEROPEN, wxSTC_MARK_BOXMINUS);
    MarkerSetForeground (wxSTC_MARKNUM_FOLDERSUB, wxColour (_T("DARK GREEN")));

    MarkerDefine(wxSTC_MARKNUM_FOLDERSUB, wxSTC_MARK_VLINE);
    MarkerSetBackground (wxSTC_MARKNUM_FOLDERSUB, wxColour (_T("VIOLET")));

    MarkerDefine(wxSTC_MARKNUM_FOLDERTAIL, wxSTC_MARK_LCORNER);
    MarkerSetBackground (wxSTC_MARKNUM_FOLDERTAIL, wxColour (_T("MAROON")));

    MarkerDefine(wxSTC_MARKNUM_FOLDEREND, wxSTC_MARK_BOXPLUSCONNECTED);
    MarkerSetBackground (wxSTC_MARKNUM_FOLDEREND, wxColour (_T("VIOLET")));

    MarkerDefine(wxSTC_MARKNUM_FOLDEROPENMID, wxSTC_MARK_BOXMINUSCONNECTED);
    MarkerSetBackground (wxSTC_MARKNUM_FOLDEROPENMID, wxColour (_T("WHEAT")));

    MarkerDefine(wxSTC_MARKNUM_FOLDERMIDTAIL, wxSTC_MARK_TCORNER);
    MarkerSetBackground (wxSTC_MARKNUM_FOLDERMIDTAIL, wxColour (_T("VIOLET")));
    MarkerSetForeground (wxSTC_MARKNUM_FOLDERMIDTAIL, wxColour (_T("MAROON")));
    return true;
}

//! Process events from our Frame
void ZionEdit::OnLineNumToggle( wxCommandEvent & WXUNUSED(event) )
{
    m_LNVisible = !m_LNVisible;
    SetMarginWidth (m_LNMarginID, m_LNVisible? m_LNMarginWidth : 0);
}

void ZionEdit::OnIndentToggle( wxCommandEvent & WXUNUSED(event) )
{
    GetIndentationGuides() ? SetIndentationGuides(false) :
        SetIndentationGuides(true);
}

void ZionEdit::OnHorizSbToggle( wxCommandEvent & WXUNUSED(event) )
{
    SetUseHorizontalScrollBar( ! GetUseHorizontalScrollBar() );
}

void ZionEdit::OnVertSbToggle( wxCommandEvent & WXUNUSED(event) )
{
    SetUseVerticalScrollBar( ! GetUseVerticalScrollBar() );
}

void ZionEdit::OnWrapToggle( wxCommandEvent & WXUNUSED(event) )
{
    GetWrapMode() ? SetWrapMode(wxSTC_WRAP_NONE) : SetWrapMode(wxSTC_WRAP_WORD );
}

void ZionEdit::OnUseTabsToggle( wxCommandEvent & WXUNUSED(event) )
{
    GetUseTabs() ? SetUseTabs(false) : SetUseTabs(true);
    wxString str;  str.Printf("Use Real Tabs is %s", GetUseTabs() ? "ON" : "OFF" );
    m_ourFrame->SetStatusText(str);
}

void ZionEdit::OnReadOnlyToggle( wxCommandEvent & WXUNUSED(event) )
{
    wxString str;
    bool isReadOnly = GetReadOnly(); bool inUse = false;
    if(isReadOnly) { //! GOING TO WRITEABLE
        //! We need to check if the file is in use
        if( zserver ) { //! If we are the server, do a direct lookup for file match
            str.Printf("%s\\%s", m_currentDir, m_currentFilename);
            if( m_ourFrame->CheckFileInUse(str, true) )  inUse = true; //! The server needs only check its remote conns. So, remoteOnly =true
        }
        else if(m_ourFrame->m_ipcOk && !zserver && zconnection) { //! Do the look up thru a connection->Request()
            str.Printf("File:%s\\%s", m_currentDir, m_currentFilename);  //! We append "File" to our request string to facilitate multiple Types of requests
            str = zconnection->Request(str);  /// Careful! Request returns a char*  //! wxString must be used for proper comparison!
            if(str == "Readonly") inUse = true;
        }
        if(inUse) {
            if( wxMessageBox ( m_currentDir + "\\" + m_currentFilename +
                    "\nis already in opened as writeable in another window!\nMake this window writeable anyway?",
                    _T("Already Writeable"), wxICON_EXCLAMATION|wxYES_NO ) != wxYES ) return; //! Yes to continue, anything else aborts
        }
        ///Make Writeable
        Reload( m_currentLexer, GetCurrentPos()); ///We reload just in case there were changes in another file.
        SetReadOnly(false); //! Toggle occurs here!
    }
    else { //! GOING TO READONLY
        if( ! CheckAllowSave(_T("Transitioning to Readonly...\n")) ) return;
        m_dirty = false;  ///If there were any changes not saved above, forever discard them
        SetReadOnly(true);  //! Toggle occurs here!
    }
    if( m_ourFrame->m_ipcOk && !zserver && zconnection ) { //! Inform the server of our final choice
        str.Printf("%s\\%s", m_currentDir, m_currentFilename);
        if( GetReadOnly() ) str = "[" + str + "]";
        zconnection->Poke("FileInUse", (wxChar *) str.c_str());
    }
    if(! isReadOnly ) { //! Care! - isReadonly is set before the toggle
        m_ourFrame->SetStatusText("[Readonly]" , 1);
        m_ourFrame->SetTitle("[" +m_currentFilename + "] - " + m_currentDir + TITLE); //!File already in use?
    }
    else {
        m_ourFrame->SetTitle(m_currentFilename + " - " + m_currentDir + TITLE);
        m_ourFrame->SetStatusText( "", 1);
    }
}

void ZionEdit::OnFoldToggle( wxCommandEvent & WXUNUSED(event) )
{
    m_FoldMarginVisible = !m_FoldMarginVisible;
    SetMarginWidth(m_FoldMarginID, m_FoldMarginVisible? m_FoldMarginWidth : 0);
}

void ZionEdit::OnWhiteSpaceToggle (wxCommandEvent & WXUNUSED(event) )
{
    SetViewWhiteSpace (GetViewWhiteSpace() == wxSTC_WS_INVISIBLE?
                wxSTC_WS_VISIBLEALWAYS: wxSTC_WS_INVISIBLE);
}

/*! NEW */
void ZionEdit::OnNew(wxCommandEvent & event)
{
    wxString str;
    if( m_ourFrame->DoWinLocDlg( str ) ) {
        int lexer = event.GetId() - ZION_NEW;   ///Work out lexer constant
        if( str != wxEmptyString ) {
            wxExecute( wxString::Format(_T("\"%s\\zion.exe\" -zposx%d -lex%d \"%s\""),
                    m_ourFrame->m_exeDir, m_ourFrame->GetPosition().x, lexer, str ) );
        }
        else DoNew( lexer );  ///In this window
    }
}

void ZionEdit::OnFileClose( wxCommandEvent & WXUNUSED(event) )
{
    wxString str = m_currentDir + "\\" + m_currentFilename;
    if(zserver) { m_ourFrame->PushDir(m_currentDir);     //! Make sure our last location and file are on top ///Here we can have the server broadcast READ_HISTORY
        m_ourFrame->PushFile(str);  //! Windows Specific
    }
    DoNew( m_currentLexer );  ///Reload with same lexer
}

void ZionEdit::DoNew( int lexer )
{
    m_ourFrame->SetStatusText(_T(""));
    if( ! CheckAllowSave() )  return;
    /*! Do it */
    ResetDoc();  ClearAll();
    m_htmlAC.Init();
    m_bookIdx.Empty();
    m_bookmarks.Empty();

    SetEOLMode(GetEolDefault()); //m_eolDefault /// Set the default eolmode ///OPtimize!

    SetCurrentFilename("Untitled");
    m_ourFrame->SetTitle("Untitled - " + wxGetCwd() + TITLE);
    SetupLexer( lexer ); ///Change lexer on New, but not on file close
    m_ourFrame->SetStatusText(" ", 1);
    ShowEOLMode();
    SetSavePoint(); //! It seems if we don't set the savepoint soon enough, modification status doesn't work right
    m_dirty = false;

    if( m_ourFrame->m_ipcOk && !zserver && zconnection ) {
        //! Inform server of current file and status ///No need for the server to report since it already knows it's own file in use (m_zedit->m_currentFile)
        wxString str; str.Printf("%s\\Untitled", m_currentDir);
        zconnection->Poke( "FileInUse", (wxChar *) str.c_str() );
        //DetermineReadonly(); /// Won't update for "Untitled" //Trying to use this just to update the servers zconn hash
    }
}

void ZionEdit::OnNewHtmlTempl(wxCommandEvent& event )
{
    if( wxYES != wxMessageBox( _T("This will open a new file in the CURRENT window.\nContinue?") , _T("Open here?"), wxYES_NO ) ) {
        return;
    }
    DoNew(wxSTC_LEX_HTML);
    wxString str;
    if( event.GetId() ==  ZION_NEWHTML_TEMPLTRANS )
        str = _T("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">\n\
<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"en\" lang=\"en\">\n\
<head><title>New HTML</title>\n</head>\n<body>\n\n</body>\n</html>");
    else if( event.GetId() ==  ZION_NEWHTML_TEMPLSTRICT )
        str = _T("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\"\n\
\"http://www.w3.org/TR/xhtml1/DTD/xhtml-strict.dtd\">\n\
<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"en\" lang=\"en\">\n\
<head><title>New HTML</title>\n</head>\n<body>\n\n</body>\n</html>");
    else if( event.GetId() ==  ZION_NEWHTML_TEMPLBASIC )
        str = _T("<html>\n<head>\n</head>\n<body>\n\n</body>\n</html>");
    else if( event.GetId() ==  ZION_NEWHTML_TEMPLBASICNH )
        str = _T("<html>\n<body>\n\n</body>\n</html>");
    else if( event.GetId() ==  ZION_NEWHTML_TEMPLASP )
        str = _T("<%@Language = VBScript %>\n<% Option Explicit %>\n<html>\n<head>\n<title>New ASP</title>\n</head>\n<body>\n<%\n\n%>\n</body>\n</html>");

    InsertText(0, str);
}

/*void ZionEdit::OnNewAsp( wxCommandEvent & WXUNUSED(event) )
{
    if( wxYES != wxMessageBox( _T("This will open a new file in the CURRENT window.\nContinue?"), _T("Open here?"), wxYES_NO ) ) {
        return;
    }
    DoNew(wxSTC_LEX_HTML);
    wxString str = "<%@Language = VBScript %>\n<% Option Explicit %>\n<html>\n<head>\n<title>New ASP</title>\n</head>\n<body>\n<%\n\n%>\n</body>\n</html>";
    InsertText( 0, str );
}
*/
void ZionEdit::SyntaxFromFile( const wxString & fn)
{
    int aLexer = wxSTC_LEX_NULL;  ///Default to text mode
    wxString str = fn;  //! can't modify const
    if(str.IsEmpty()) return;
    str = str.Lower();
    wxString noext = str.BeforeLast('.');  ///filename without extension
    wxString ext = str.AfterLast('.');

    if( str == "makefile" || noext == "makefile" )
        aLexer = wxSTC_LEX_MAKEFILE;

    else if( ext2lex[ext] > 0 ) ///Pick up our mapping from the hash // -1 or 0 is an error
        aLexer = ext2lex[ext];
    
    SetupLexer( aLexer );
    ///Diag //m_ourFrame->SetStatusText(wxString::Format(_T("lexer# %d"), m_currentLexer) );
}

bool ZionEdit::CheckAllowSave( wxString msgStr )
{
    if( ! m_dirty || GetReadOnly() ) return true;    //! If file already saved, no sense going on
    wxString msg;   int ID;
    if( msgStr != wxEmptyString ) {
        msg = msgStr;
    }
    msg.Append( wxString::Format(_T("You have made changes to: \n%s\nSAVE Changes?"), GetCurrentFilename() ) );
    wxMessageDialog warnDialog( this, msg, "SAVE Changes?",
                wxICON_EXCLAMATION | wxYES_NO | wxCANCEL| wxYES_DEFAULT );
    ID = warnDialog.ShowModal();
    if(  ID == wxID_CANCEL)  return false;
    if(  ID == wxID_YES )  { //! Save the file
        wxCommandEvent anEvent;
        OnSave(anEvent);
        if (m_dirty) return false;
    }
    return true;
}

void ZionEdit::Open( const wxString & dir )
{
    if( ! CheckAllowSave() ) return;
    wxFileDialog * ofDialog = new wxFileDialog( this, "Open File",
            dir != wxEmptyString ? dir : wxGetCwd(),
            "", FILETYPES, wxOPEN, wxDefaultPosition);	//no default file
    
    if( ofDialog->ShowModal() == wxID_OK) {
        wxString fname = ofDialog->GetPath(); ///get full filespec from dialog
        ///WinLocDlg
        wxString loc;
        if( m_ourFrame->DoWinLocDlg( loc ) ) {
            if( loc != wxEmptyString ) {
                wxExecute( wxString::Format( _T("\"%s\\zion.exe\" -zposx%d %s \"%s\""), m_ourFrame->m_exeDir, m_ourFrame->GetPosition().x, loc, fname ) );
            }
            else DirectOpen( fname ); ///In this window
        }
    }
}

/*! OPEN */
void ZionEdit::OnOpen(wxCommandEvent & WXUNUSED(event) )
{
    Open();
}

void ZionEdit::OnOpenSelected(wxCommandEvent & event )
{
    m_ourFrame->PrependToPopup( event.GetId() ); ///LAST CMD to popup
    wxString str, str2;
    if( (str = GetSelectedText()) != wxEmptyString ) {
        ///Attempt to extract line number
        int pos1; long num=0;
        ///Case of e.g. ZionDialogs (23)
        if( (pos1=str.Find('(')) != -1 && str.Find(')') != -1 ) {  
            str2 = str.BeforeLast(')').AfterFirst('(') ; ///Extract line number
            str2.ToLong( &num );  str2.Printf(_T("%d"), (int) num );  ///Ensure it's just a number (long won't print with %l) (no space allowed after option letter eg n40 = goto line 40
            str = str.Mid( 0, pos1 );  ///Truncate ///Note: pos1 serves here as a count
            str.Prepend( _T("-n") + str2 + _T(" ") );
        }
        ///Case of e.g. ZionDialogs [23]
        else if( (pos1=str.Find('[')) != -1 && str.Find(']') != -1 ) {
            str2 = str.BeforeLast(']').AfterFirst('[') ; ///Extract line number
            str2.ToLong( &num );  str2.Printf(_T("%d"), (int) num );  ///Ensure it's just a number (long won't print with %l) (no space allowed after option letter eg n40 = goto line 40
            str = str.Mid( 0, pos1 );  ///Truncate ///Note: pos1 serves here as a COUNT
            str.Prepend( _T("-n") + str2 + _T(" ") );
        }
        ///Case of e.g. ZionDialogs:23
        else if( (pos1=str.Find(':')) != -1 ) {
            str2 = str.AfterFirst(':');  ///It is up the user to make a decent selection here!
            str2.ToLong( &num );  str2.Printf(_T("%d"), (int) num );
            str = str.Mid( 0, pos1 );  ///Truncate ///Note: pos1 serves here as a COUNT
            str.Prepend( _T("-n") + str2 + _T(" ") );
        }
        wxExecute( m_ourFrame->m_exeDir +_T("\\zion.exe -c ") + str );
        m_ourFrame->SetStatusText( _T("zion.exe -c ") + str );
    }
}

void ZionEdit::ReloadFromBuffer( int lexer, int pos, bool force )  ///This is used for simple reloading styles without having to save the current file
{   //! File should be saved before this point
    bool isReadonly = GetReadOnly(); ///save readonly status for later
    wxString editorText = GetText(); ///save edit buffer
    if( GetCurrentFilename() != _T("Untitled") ) m_settled = false; ///We are about to modify the edit buffer
    
    ResetDoc();  ClearAll();  //! This will clear our readonly status so we have to reset it
    SetupLexer( lexer, force );  ///Pass on the force arg
    
    SetText(editorText); /// Basically clears the buffer and inserts the text
    
    if( GetCurrentFilename() != _T("Untitled") ) { EmptyUndoBuffer(); SetSavePoint(); }

    ShowEOLMode();
//Here we want to leave the modified indicator on    ///Remove modified indicator from titlebar - Should be taken care of in OnDoneLoading handler
    if( pos ) GotoPos(pos);
    if(isReadonly) { SetReadOnly(true); m_ourFrame->SetStatusText("[Readonly]", 1); }
}

void ZionEdit::Reload( int lexer, int pos, bool force )
{   //! File should be saved before this point
    bool isReadonly = GetReadOnly(); ///save readonly status for later
    m_settled = false; ///We are about to modify the edit buffer
    ResetDoc();  ClearAll();  //! This will clear our readonly status so we have to reset it
    SetupLexer( lexer, force );  ///Pass on the force arg
    if( GetCurrentFilename() == _T("Untitled") ) {  m_settled = true; return; }
    
    m_dirty = false;  ///We are going to reload the buffer so we are starting from scratch. This is important to do before LoadFile().
    if( ! LoadFile( GetCurrentFilename() ) ) { 
        wxMessageBox(_T("Could not reload current file!\nThe current buffer is in an unknown state. Please CLOSE this window."),
        _T("Error on Reload"), wxICON_ERROR );
    }
    ShowEOLMode();

    if(isReadonly) { SetReadOnly(true); m_ourFrame->SetStatusText("[Readonly]", 1); }
    m_ourFrame->SetStatusText(_T("Reloaded"), 0);
//    ///Remove modified indicator from titlebar - Should be taken care of in OnDoneLoading handler
    if( pos )  GotoPos(pos);
}

void ZionEdit::DirectOpen( const wxString & arg, bool local )
{
    wxString path, fn;
    wxFileName wfn(arg); ///wxFileName allows us to perform certain tests on a full filespec
    if( ! wfn.IsOk() ) {
        wxMessageBox( wxString::Format(_T("File name to open:\n%s is not syntactically correct"), arg ),
                _T("Error in Filename") );
        return;
    }
    fn = wfn.GetFullName(); /// Name and ext
    SetCurrentFilename(fn);
    path = wfn.GetPath();
//    if( /*path != "" &&*/ wfn.DirExists() ) /// DirExists() may be unreliable. The check below should be enough anyway. If the path is invalid we should get an error setting to it.
    if( path != "" && path != "." ) {
        if ( ! wxSetWorkingDirectory(path) ) {
            wxMessageBox( wxString::Format(_T("Could not change dir to:  \"%s\".\nMake sure the path still accessible"), path ),
                    _T("Could not change directory") );
            return;
        }
        m_currentDir = wxGetCwd();
        char c = m_currentDir.GetChar(0);  c = c & 0xDF;  m_currentDir.SetChar(0, c);   //! Correct for lower case drive letter // Windows only
    }
    m_settled = false;  ///After loadfile is complete an event will be sent out whose handler will set m_settled to true.
    ResetDoc();  ClearAll(); /*! CRITICAL */   //! Seems to clear bugs if we clear before load
    ///Normalize
    m_dirty = false;
    if( ! m_min ) {
        m_ourFrame->SetStatusText(_T("") );
        m_ourFrame->SetStatusText(" ", 1);  //! Clear our modified flag
        ///Remove modified indicator from titlebar
        wxString str = m_ourFrame->GetTitle();
        if( str.GetChar(0) == '*' )  { str.Remove( 0, 1 );  m_ourFrame->SetTitle(str); }
    }
    SyntaxFromFile(fn); ///Also sets up lexer
    
    if( wfn.FileExists() ) {
        if( ! LoadFile( wfn.GetFullPath() /* fn */) ) { ///We are loading with the full filespec regardless of our working dir.
            wxMessageBox( wxString::Format(_T("Error loading file \"%s\""), fn ), _T("Error loading file") );
            return;
        }
    }
    else { ///If file doesn't exists we will "create" a new one, at least in the edit buffer
        m_settled = true; ///We didn't load so we are settled immediately
        m_dirty = false;
        m_ourFrame->SetStatusText(_T("New File"), 1 );  
    }
    ///GENERAL INITS
    m_ourFrame->m_lineHits = 0; ///Reset count to zoom and autobookmark
    SetZoom( (int) m_ourFrame->m_zoomLevel );  ///Reset to our startup zoom level
    m_bookmarks.Empty(); ///Clear bookmarks
    m_bookIdx.Empty(); ///Clear indices
    m_htmlAC.Init();
    
    m_ourFrame->PushDir( m_currentDir, local ); ///Directories must be pushed first
    wxMilliSleep(20);
    m_ourFrame->PushFile( m_currentDir + _T("\\")+ fn, local ); //! If we are pushing a file we should push its dir also. wxWidgets specifies root with a trailing blackslash so don't duplicate backslash
    wxMilliSleep(20);
    if( DetermineReadonly() ) //! Also registers the file name with the server (if we are a client)
        m_ourFrame->SetTitle("[" +fn + "] - " + m_currentDir + TITLE); //!File already in use?
    else m_ourFrame->SetTitle(fn + " - " + m_currentDir + TITLE);
    ShowEOLMode(); ///File must be loaded first
}

bool ZionEdit::DetermineReadonly()  //! Also registers the file name with the server (if we are a client)
{
     wxString str;
     if( zserver && m_currentFilename != "Untitled" ) { //! If we are the server, directly check the zconns hash
        str.Printf("%s\\%s", m_currentDir, m_currentFilename);
        if( m_ourFrame->CheckFileInUse(str, true) ) {  //! The server needs only check its remote conns. So, remoteOnly =true
            wxMessageBox( str + "\nis already in use!\nThis window will be set to Read Only", _T("Server: File Open Request"), wxOK);
            m_ourFrame->SetStatusText("[Readonly]", 1);
            SetReadOnly(true);  return true;
        } 
    }
    else if(m_ourFrame->m_ipcOk && !zserver && zconnection) { //! Do the look up thru a connection->Request()
        str.Printf("File:%s\\%s", m_currentDir, m_currentFilename);  //! We append "File" to our request to facilitate multiple Types of requests
        wxString str2 = zconnection->Request(str);
        if(str2 == "Readonly") {
            wxMessageBox(m_currentDir + "\\" + m_currentFilename + "\nis already in use!\nThe window will be set to Read Only",
            _T("Client: File Open Request"), wxOK);
            m_ourFrame->SetStatusText("[Readonly]", 1);
            SetReadOnly(true); return true;
        }
    }
    return false;
}

void ZionEdit::OnModified (wxStyledTextEvent & WXUNUSED(event) )
{ 
    if( ! m_settled )  return;
    if( ! m_dirty ) { //! if document not already modified
        m_dirty = true;
        if( ! m_min ) {
            m_ourFrame ->SetStatusText(  "*Modified*", 1 );
            m_ourFrame ->SetStatusText(" ");
            m_ourFrame->SetTitle( "*" + m_ourFrame->GetTitle() );
        }
    }
}
/*! SAVE */
void ZionEdit::OnSave(wxCommandEvent & event)
{  
	if( GetCurrentFilename() == "Untitled" )  OnSaveAs(event); //! borrow the OnSaveAs event handler
	else SaveCurrentFile();
}

void ZionEdit::OnSaveAs(wxCommandEvent & event ) {
    //with the event passed in from save, things get weird //m_ourFrame->PrependToPopup( event.GetId() ); ///LAST CMD to popup
    ///Show a RecentDirsDlg "Initial Directory"; Choice will be passed on to wxFileDialog as its "Default Dir"
    wxString startDir, prevDir = wxGetCwd(); ///Save in case SaveAs dialog canceled
    wxArrayString rdirs;
    for( int i=0; i< NUMRECENTDIRS; i++) {
        if( m_ourFrame->dirSubItems[i] == NULL ) break; ///We have hit the end of the used dirSubItems
        rdirs.Add( m_ourFrame->dirSubItems[i]->GetLabel() );
    }
    if( rdirs.GetCount() > 0 ) {
        StartFolderDlg stDlg(this, m_ourFrame->GetPosition(), m_ourFrame->GetSize().GetWidth(), rdirs);
        if( stDlg.ShowModal() != wxID_OK ) return;
        startDir = stDlg.m_list->GetStringSelection(); ///Pick up the dir we want to start browsing from
        if( stDlg.m_list->GetSelection() != 0 )  wxSetWorkingDirectory( startDir );
        //if( startDir != _T("Current Directory") ) 
            
    }
   wxFileDialog * ofDialog = new wxFileDialog( this, _T("Save File As"), wxGetCwd(), ( m_currentFilename == "Untitled" ? _T("") : m_currentFilename ), ///This dialog may not be handling spaces in dir names properly
	FILETYPES, wxSAVE|wxOVERWRITE_PROMPT|wxFD_CHANGE_DIR );  ///CRITICAL TAKE COMMENTS OUT FOR NORMAL
  
    if( ofDialog->ShowModal() == wxID_OK) {
        wxString f = ofDialog->GetFilename();
        SetCurrentFilename(f);
        ///DONE BY wxFD_CHANGE_DIR //wxSetWorkingDirectory(ofDialog->GetDirectory()); //! set the wx global working dir sourced from the dialog
        if( ! SaveCurrentFile() ) return;  ///TODO: Dbl chk error exit

        wxString wdir = wxGetCwd();
        char c = wdir.GetChar(0);  c = c & 0xDF;  wdir.SetChar(0, c);   //! Correct for lower case drive letter // Windows only
        if( m_currentDir != wdir ) {
            m_currentDir = wdir;
            m_ourFrame->PushDir(m_currentDir);  //! Save in [RECENT_DIR]
        }
        wxString str = m_currentDir + "\\" + f;
        m_ourFrame->PushFile(str); //! Remember Save as is similar to open
        if(DetermineReadonly())
            m_ourFrame->SetTitle("[" + f + "] - " + m_currentDir + TITLE);
        else m_ourFrame->SetTitle(f + " - " + m_currentDir + TITLE);
        
        ///Redo syntax (if necessary - SetupLexer() checks) ///m_currentFilename is always validated by DirectOpen()
        ///In scintilla, when changing styles we must clear the buffer, chg styles, then reload the file into that buffer
        int savePos = GetCurrentPos();
        if( GetWrapMode() != wxSTC_WRAP_NONE ) GotoPos(0);  ///workaround wrap bug
        ///SYNTAX FROM EXTENSION
        int aLexer = wxSTC_LEX_NULL;  ///Default to text mode
        str = f.Lower();
        wxString noext = str.BeforeLast('.');  ///filename without extension
        wxString ext = str.AfterLast('.');
        if( str == "makefile" || noext == "makefile" )
            aLexer = wxSTC_LEX_MAKEFILE;
        else if( ext2lex[ext] > 0 ) ///Pick up our mapping from the hash // -1 or 0 is an error
            aLexer = ext2lex[ext];
        Reload( aLexer, savePos, false );  ///False = setup lexer only if going into new mode - TODO move this description to header
    }
    else wxSetWorkingDirectory( prevDir );  ///Revert
}

bool ZionEdit::SaveCurrentFile()
{	
	wxString s = m_currentFilename;
	if( s.Length() > 0) ///This is the NAME here. We DO want to be able to save a zero length file
	{
        ConvertEOLs( GetEOLMode() );  ///Normalize all EOLs ///This should make VisStudio happy
        if( SaveFile(s) ) {
            m_ourFrame->SetStatusText(_T("Save completed") );
            m_dirty = false; m_settled = true;  ///we are ready for the next modify
            ///Clear modified indicators
            m_ourFrame->SetStatusText(" ", 1);
            wxString str = m_ourFrame->GetTitle();
            if( str.GetChar(0) == '*' )  { str.Remove( 0, 1 );  m_ourFrame->SetTitle(str); }
            return true;
        }
        else{ wxMessageBox(_T("Error while saving file!"), _T("Save Error!"));
            return false;
        }
	}
    else {
        wxMessageBox(_T("Error:  Filename length not > 0"), _T("Save Error - Filename") );
        return false;
    }
}

void ZionEdit::OnDoneLoading (wxStyledTextEvent & WXUNUSED(event) )
{
    wxMilliSleep(50);
    if( m_dirty == true || m_currentFilename == _T("Untitled") )  return;
    
    m_settled = true;  ///If we are not settled then allow a bypass of setting "modified" until settled.
    ///Normalize
    //m_dirty = false;
    if( ! m_min ) {
        m_ourFrame->SetStatusText(" ", 1);  //! Clear our modified flag
        ///Remove modified indicator from titlebar
        wxString str = m_ourFrame->GetTitle();
        if( str.GetChar(0) == '*' )  { str.Remove( 0, 1 );  m_ourFrame->SetTitle(str); }
    }
}

wxString ZionEdit::GetCurrentFilename()
{	return m_currentFilename; }

void ZionEdit::SetCurrentFilename(wxString n)
{	m_currentFilename = n;}

void ZionEdit::OnRedo(wxCommandEvent &event) {
    //These have UI events - risk of lockup. m_ourFrame->PrependToPopup( event.GetId() ); ///LAST CMD to popup
    if (!CanRedo()) return;
    Redo ();
}

void ZionEdit::OnUndo(wxCommandEvent &event) {
    //These have UI events - risk of lockup. m_ourFrame->PrependToPopup( event.GetId() ); ///LAST CMD to popup
    if (!CanUndo()) return;
    Undo ();
}

void ZionEdit::OnMarginClick (wxStyledTextEvent &event) //! Stc Margin Click
{   wxString str = "OnMarginClick()";
    if (event.GetMargin() == m_FoldMarginID) {
        int lineClick = LineFromPosition (event.GetPosition());
        int levelClick = GetFoldLevel (lineClick);
        if ((levelClick & wxSTC_FOLDLEVELHEADERFLAG) > 0)
            ToggleFold (lineClick);
    }
}

void ZionEdit::OnSelectLine( wxCommandEvent & event )
{
    //Called from other event handlers //m_ourFrame->PrependToPopup( event.GetId() ); ///LAST CMD to popup
    int end, currentLine = GetCurrentLine();
    //! We want to select the line including line endings
    if( PositionFromLine(currentLine +1) )   //! Check for doc end
        end = PositionFromLine( currentLine +1 );
    else end = PositionFromLine( currentLine ) + LineLength(currentLine);
    
    SetSelection( PositionFromLine(currentLine), end );
}

void ZionEdit::OnSelToMatch( wxCommandEvent & event )
{
    m_ourFrame->PrependToPopup( event.GetId() ); ///LAST CMD to popup
    int pos = GetCurrentPos();
    char chr = GetCharAt(pos);
    if( chr == '}' || chr == '{' || chr == '(' || chr == ')' )  {
        int a;
        if( (a = BraceMatch(pos))  != -1 ) SetSelection(pos, a+1);
    }
}

void ZionEdit::ShowPosition()
{
    wxString str; int line = GetCurrentLine();
    str.Printf(_T("%d: %d"), line+1, GetCurrentPos() - PositionFromLine(line) +1 );
    m_ourFrame->SetStatusText( str , 4 );
    ///Dynamic Line number width - wishful thinking
//    int newWidth = 0;
//    if( line < 999 ) /* really 1000 [zero based] */ newWidth = TextWidth (wxSTC_STYLE_LINENUMBER, _T("_999")); ///5 digits
//    else if( line < 9999 ) /* 10000 */ newWidth = TextWidth (wxSTC_STYLE_LINENUMBER, _T("_9999")); ///5 digits
//    else newWidth = TextWidth (wxSTC_STYLE_LINENUMBER, _T("_99999")); ///5 digits
//    ///Set it if different
//    if( newWidth != m_LNMarginWidth ) {
//        SetMarginWidth (m_LNMarginID, m_LNVisible? m_LNMarginWidth : 0);
//        Colourise( 0, GetLength() - 1 ); 
//        //StyleSetUnderline( defaultSyntax.styles[1].styleNum, false );  ///This is just a redraw trigger
//    }
}

void ZionEdit::OnPosChanged( wxStyledTextEvent &event ) ///This does the highlighting when at the brace pos (by kybd or click)
{
    if( m_min )  return;
    int pos = event.GetPosition();
    char chr = GetCharAt(pos);
    if( chr == '}' || chr == '{' || chr == '(' || chr == ')' ) {
        int a;
        if( (a = BraceMatch(pos) )  == -1 ) {
            BraceBadLight( pos); 
        }
        else BraceHighlight( a, pos);
    }
    else ResetBraces();
    
    int line = GetCurrentLine();
    
    ///Remove ALL  ActiveCallTips
    int actCnt = activeCallTips.GetCount();
    if( actCnt > 0 ) {
        if( activeCallTips[actCnt - 1].m_line != line ) activeCallTips.Empty();  ///Will delete all ActiveCallTip objects
    }
    
    ///Track new line for all who use it
    if( m_line != line ) { ///NEW LINE OCCURRED
        m_line = line;
        m_newLine = true;
        m_acAttribQuote = NOQUOTE;  ///Turn off attrib quote mode
        m_htmlAC.ResetTag();
        if( m_ctDescript ) { m_ctDescript->Destroy(); m_ctDescript = NULL; }  ///Cancel Calltip descript
        //m_ourFrame->SetStatusText(_T(""));
    }
    else m_newLine = false;

    ShowPosition(); ///On status bar
}

//void HtmlAutoComp::NewTag( bool newLine )  ///Establish a new tag context - use only after a jump-in -ie We click within a tag and start typeing
//{
//    tagWaiting = true;
//    tag= wxEmptyString;
//    line = currLine;
//}

//----------------------------------------PRINTING-------------------------------------
/// Many, many thanks to Otto Wyss for the basis of this printing code
///extracted from the  wxScintilla test application
///and I cannot give enough thanks to Julian Smart and the wxWidgets team
///for such a tremendous application framework! (www.wxwidgets.org)

// Used wxArrayInt instead // WX_DEFINE_ARRAY(int, PagePos);

/** PRINTOUT CLASS **/
EditPrint::EditPrint (ZionEdit *edit, wxChar *title)
              : wxPrintout(title) {
    m_edit = edit;
    m_pgCount = 0;
    pages.Empty();  ///mark as empty, no memory dealloc
    m_linesOff = false;  /// will track if we forced line nums off
    m_selStart = edit->GetSelectionStart();
    m_selEnd = edit->GetSelectionEnd();
    m_edit->SetPrintWrapMode(wxSTC_WRAP_WORD);  ///TODO: make this an option
}

void EditPrint::OnPreparePrinting()
{
    ///Temporarily turn off line nums if desired
    m_linesOff = false; /// Track - force lineNums off?
    if( m_edit->m_dontPrintNums && m_edit->m_LNVisible ) {  ///If we dont want to print Line Nums and they are visible
        m_linesOff = true;
        m_edit->SetMarginWidth( m_edit->m_LNMarginID, 0 );
    }
}

bool EditPrint::OnPrintPage (int page)
{
    int c = page;
    if( c < 2 ) c = 0; else c--;   ///First page starts at zero  ///The start of page 2 is last position of the first page printed i.e. index 0
    //wxMessageBox( wxString::Format( _T("Page: %d, pages[%d]: %d, Total Pgs%d"), page, c, pages[c], m_pgCount  ) );
    ///Turn off line nums if desired
    bool m_linesOff = false; ///save original line number state
    if( m_edit->m_dontPrintNums && m_edit->m_LNVisible ) {  ///If we dont want to print Line Nums and they are visible
        m_linesOff = true;
        m_edit->SetMarginWidth( m_edit->m_LNMarginID, 0 );
    }
    
    wxDC *dc = GetDC();  if( !dc ) return false;
    
    PrintScaling (dc); /// scale DC
    
    /// Print Page
    m_edit->FormatRange( 1, pages[c],
            (m_selStart==m_selEnd)? m_edit->GetLength() : m_selEnd,  ///If selection end at sel end
           //m_edit->GetLength(),  ///If selection end at sel end
            dc, dc, m_printRect, m_pageRect );

    ///Restore line num state
    if( m_linesOff ) { ///If line numbers were originally on then toggled off
        m_edit->SetMarginWidth( m_edit->m_LNMarginID, m_edit->m_LNMarginWidth );
    }
    return true;
}

EditPrint::~EditPrint()
{
    //pages.Clear(); /// not sure if this is causing instability
}

void EditPrint::OnEndPrinting()
{
    //This causes lock ups here -move to destructor //pages.Clear();  ///mark as empty and release allocated memory
    ///Restore line num state
    if( m_linesOff ) { ///If line numbers were originally on then toggled off
        m_edit->SetMarginWidth( m_edit->m_LNMarginID, m_edit->m_LNMarginWidth );
    }
}

bool EditPrint::OnBeginDocument (int startPage, int endPage)
{
    if( !wxPrintout::OnBeginDocument (startPage, endPage) ) {  ///call the parent f()
        return false;
    }
    return true;
}

void EditPrint::GetPageInfo( int *minPage, int *maxPage, int *selPageFrom, int *selPageTo )
/// Pages must be counted here, not in OnPreparePrinting - no workie
{
    *minPage = 0;
    *maxPage = 0;
    *selPageFrom = 0;
    *selPageTo = 0;
    
    pages.Empty();  ///mark as empty, but don't bother releasing memory
    pages.Add( (m_selStart==m_selEnd ? 0 : m_selStart) );  ///set first index to start of  range  //pages[0] = start of range;
    m_pgCount = 0;
    
    /// scale DC if possible
    wxDC *dc = GetDC();
    if( !dc ) { return; }
    PrintScaling (dc);

    /// get print page informations and convert to printer pixels
    wxSize ppiScr;  GetPPIScreen (&ppiScr.x, &ppiScr.y);
    if (ppiScr.x == 0) { // most possible guess:
        ppiScr.x = 96; //96
        ppiScr.y = 96;
    }
    
    wxSize page = g_pageSetupData->GetPaperSize();
    page.x = static_cast<int> ( page.x * ppiScr.x / 25.4 );  ///mm to pixels
    page.y = static_cast<int> ( page.y * ppiScr.y / 25.4 );
    m_pageRect = wxRect(0, 0, page.x, page.y);

    /// get margins informations and convert to printer pixels
    int  top, bottom, left, right;
    wxPoint TL = g_pageSetupData->GetMarginTopLeft(); /// wxPoint( top, left ) does NOT work
    top = TL.y;  left = TL.x;
    wxPoint BR = g_pageSetupData->GetMarginBottomRight();
    bottom = BR.y;  right = BR.x;
    
    top = static_cast<int> (top * ppiScr.y / 25.4);  // mm /25.5 [=inch] * ppi  [ = point (or pixels) ]
    bottom = static_cast<int> (bottom * ppiScr.y / 25.4);
    left = static_cast<int> (left * ppiScr.x / 25.4);
    right = static_cast<int> (right * ppiScr.x / 25.4);
    m_printRect = wxRect( left, top, page.x - left - right /* width */,  page.y - top - bottom /* height */ );  ///less the margins

    /// count pages
    int docLength =  ( m_selStart==m_selEnd? m_edit->GetLength() : m_selEnd );  ///If selection, end at sel end
    ///already added 0;
    while( pages.Item(m_pgCount) < docLength  ) {
         pages.Insert( m_edit->FormatRange( 0,  /// Track last position printed
                pages.Item(m_pgCount), docLength, dc /* draw */, dc /* target */,
                m_printRect /*Render Rect  */, m_pageRect ), m_pgCount + 1 /* Hopefully, before the end */ );
         m_pgCount++;
    }

    if( m_pgCount > 0 ) *minPage = 1;
    *maxPage = m_pgCount;
    
    *selPageFrom = *minPage;
    *selPageTo = *maxPage;
}

bool EditPrint::HasPage( int page )
{
    return ( page > 0 && page <= m_pgCount );
}

bool EditPrint::PrintScaling (wxDC *dc)
{
    if (!dc) return false; /// check for dc, return if none

    /// get printer and screen sizing values
    wxSize ppiScr;  GetPPIScreen( &ppiScr.x, &ppiScr.y );
    if (ppiScr.x == 0) { ppiScr.x = 96; ppiScr.y = 96; } // guess: 96
    //m_edit->m_ourFrame->SetStatusText( wxString::Format(_T("ppiScr x=%d, y=%d"), ppiScr.x, ppiScr.y ), 1);
    
    wxSize ppiPrt;  GetPPIPrinter (&ppiPrt.x, &ppiPrt.y);
    if (ppiPrt.x == 0) { // scaling factor to 1
        ppiPrt.x = ppiScr.x;
        ppiPrt.y = ppiScr.y;
    }
    //m_edit->m_ourFrame->SetStatusText( wxString::Format(_T("ppiPrt x=%d, y=%d"), ppiPrt.x, ppiPrt.y ), 2);
    wxSize dcSize = dc->GetSize();
    wxSize pageSize;
    GetPageSizePixels( &pageSize.x, &pageSize.y );

    //m_edit->m_ourFrame->SetStatusText( wxString::Format(
    //        _T("dcSize x=%d, y=%d; pageSize x=%d, y=%d"), dcSize.x, dcSize.y, pageSize.x, pageSize.y ), 0);
    /// set user scale
    float scale_x = (float)(ppiPrt.x * dcSize.x) /  /// Pixels per Inch
                    (float)(ppiScr.x * pageSize.x);
    float scale_y = (float)(ppiPrt.y * dcSize.y) /
                    (float)(ppiScr.y * pageSize.y);
    dc->SetUserScale (scale_x, scale_y);
    //m_edit->m_ourFrame->SetStatusText( wxString::Format(_T("scale x=%f, y=%f"), scale_x, scale_y ), 3);
    //dc->SetUserScale( wxMin(scale_x, scale_y), wxMin(scale_x, scale_y) );

    return true;
}
//----------------------------------------END EDITPRINT
